/** vim: et:ts=4:sw=4:sts=4
 * @license RequireJS 2.1.11 Copyright (c) 2010-2014, The Dojo Foundation All Rights Reserved.
 * Available via the MIT or new BSD license.
 * see: http://github.com/jrburke/requirejs for details
 */
//Not using strict: uneven strict support in browsers, #392, and causes
//problems with requirejs.exec()/transpiler plugins that may not be strict.
/*jslint regexp: true, nomen: true, sloppy: true */
/*global window, navigator, document, importScripts, setTimeout, opera */

var requirejs, require, define;
(function (global) {
    var req, s, head, baseElement, dataMain, src,
        interactiveScript, currentlyAddingScript, mainScript, subPath,
        version = '2.1.11',
        commentRegExp = /(\/\*([\s\S]*?)\*\/|([^:]|^)\/\/(.*)$)/mg,
        cjsRequireRegExp = /[^.]\s*require\s*\(\s*["']([^'"\s]+)["']\s*\)/g,
        jsSuffixRegExp = /\.js$/,
        currDirRegExp = /^\.\//,
        op = Object.prototype,
        ostring = op.toString,
        hasOwn = op.hasOwnProperty,
        ap = Array.prototype,
        apsp = ap.splice,
        isBrowser = !!(typeof window !== 'undefined' && typeof navigator !== 'undefined' && window.document),
        isWebWorker = !isBrowser && typeof importScripts !== 'undefined',
        //PS3 indicates loaded and complete, but need to wait for complete
        //specifically. Sequence is 'loading', 'loaded', execution,
        // then 'complete'. The UA check is unfortunate, but not sure how
        //to feature test w/o causing perf issues.
        readyRegExp = isBrowser && navigator.platform === 'PLAYSTATION 3' ?
                      /^complete$/ : /^(complete|loaded)$/,
        defContextName = '_',
        //Oh the tragedy, detecting opera. See the usage of isOpera for reason.
        isOpera = typeof opera !== 'undefined' && opera.toString() === '[object Opera]',
        contexts = {},
        cfg = {},
        globalDefQueue = [],
        useInteractive = false;

    function isFunction(it) {
        return ostring.call(it) === '[object Function]';
    }

    function isArray(it) {
        return ostring.call(it) === '[object Array]';
    }

    /**
     * Helper function for iterating over an array. If the func returns
     * a true value, it will break out of the loop.
     */
    function each(ary, func) {
        if (ary) {
            var i;
            for (i = 0; i < ary.length; i += 1) {
                if (ary[i] && func(ary[i], i, ary)) {
                    break;
                }
            }
        }
    }

    /**
     * Helper function for iterating over an array backwards. If the func
     * returns a true value, it will break out of the loop.
     */
    function eachReverse(ary, func) {
        if (ary) {
            var i;
            for (i = ary.length - 1; i > -1; i -= 1) {
                if (ary[i] && func(ary[i], i, ary)) {
                    break;
                }
            }
        }
    }

    function hasProp(obj, prop) {
        return hasOwn.call(obj, prop);
    }

    function getOwn(obj, prop) {
        return hasProp(obj, prop) && obj[prop];
    }

    /**
     * Cycles over properties in an object and calls a function for each
     * property value. If the function returns a truthy value, then the
     * iteration is stopped.
     */
    function eachProp(obj, func) {
        var prop;
        for (prop in obj) {
            if (hasProp(obj, prop)) {
                if (func(obj[prop], prop)) {
                    break;
                }
            }
        }
    }

    /**
     * Simple function to mix in properties from source into target,
     * but only if target does not already have a property of the same name.
     */
    function mixin(target, source, force, deepStringMixin) {
        if (source) {
            eachProp(source, function (value, prop) {
                if (force || !hasProp(target, prop)) {
                    if (deepStringMixin && typeof value === 'object' && value &&
                        !isArray(value) && !isFunction(value) &&
                        !(value instanceof RegExp)) {

                        if (!target[prop]) {
                            target[prop] = {};
                        }
                        mixin(target[prop], value, force, deepStringMixin);
                    } else {
                        target[prop] = value;
                    }
                }
            });
        }
        return target;
    }

    //Similar to Function.prototype.bind, but the 'this' object is specified
    //first, since it is easier to read/figure out what 'this' will be.
    function bind(obj, fn) {
        return function () {
            return fn.apply(obj, arguments);
        };
    }

    function scripts() {
        return document.getElementsByTagName('script');
    }

    function defaultOnError(err) {
        throw err;
    }

    //Allow getting a global that is expressed in
    //dot notation, like 'a.b.c'.
    function getGlobal(value) {
        if (!value) {
            return value;
        }
        var g = global;
        each(value.split('.'), function (part) {
            g = g[part];
        });
        return g;
    }

    /**
     * Constructs an error with a pointer to an URL with more information.
     * @param {String} id the error ID that maps to an ID on a web page.
     * @param {String} message human readable error.
     * @param {Error} [err] the original error, if there is one.
     *
     * @returns {Error}
     */
    function makeError(id, msg, err, requireModules) {
        var e = new Error(msg + '\nhttp://requirejs.org/docs/errors.html#' + id);
        e.requireType = id;
        e.requireModules = requireModules;
        if (err) {
            e.originalError = err;
        }
        return e;
    }

    if (typeof define !== 'undefined') {
        //If a define is already in play via another AMD loader,
        //do not overwrite.
        return;
    }

    if (typeof requirejs !== 'undefined') {
        if (isFunction(requirejs)) {
            //Do not overwrite and existing requirejs instance.
            return;
        }
        cfg = requirejs;
        requirejs = undefined;
    }

    //Allow for a require config object
    if (typeof require !== 'undefined' && !isFunction(require)) {
        //assume it is a config object.
        cfg = require;
        require = undefined;
    }

    function newContext(contextName) {
        var inCheckLoaded, Module, context, handlers,
            checkLoadedTimeoutId,
            config = {
                //Defaults. Do not set a default for map
                //config to speed up normalize(), which
                //will run faster if there is no default.
                waitSeconds: 7,
                baseUrl: './',
                paths: {},
                bundles: {},
                pkgs: {},
                shim: {},
                config: {}
            },
            registry = {},
            //registry of just enabled modules, to speed
            //cycle breaking code when lots of modules
            //are registered, but not activated.
            enabledRegistry = {},
            undefEvents = {},
            defQueue = [],
            defined = {},
            urlFetched = {},
            bundlesMap = {},
            requireCounter = 1,
            unnormalizedCounter = 1;

        /**
         * Trims the . and .. from an array of path segments.
         * It will keep a leading path segment if a .. will become
         * the first path segment, to help with module name lookups,
         * which act like paths, but can be remapped. But the end result,
         * all paths that use this function should look normalized.
         * NOTE: this method MODIFIES the input array.
         * @param {Array} ary the array of path segments.
         */
        function trimDots(ary) {
            var i, part, length = ary.length;
            for (i = 0; i < length; i++) {
                part = ary[i];
                if (part === '.') {
                    ary.splice(i, 1);
                    i -= 1;
                } else if (part === '..') {
                    if (i === 1 && (ary[2] === '..' || ary[0] === '..')) {
                        //End of the line. Keep at least one non-dot
                        //path segment at the front so it can be mapped
                        //correctly to disk. Otherwise, there is likely
                        //no path mapping for a path starting with '..'.
                        //This can still fail, but catches the most reasonable
                        //uses of ..
                        break;
                    } else if (i > 0) {
                        ary.splice(i - 1, 2);
                        i -= 2;
                    }
                }
            }
        }

        /**
         * Given a relative module name, like ./something, normalize it to
         * a real name that can be mapped to a path.
         * @param {String} name the relative name
         * @param {String} baseName a real name that the name arg is relative
         * to.
         * @param {Boolean} applyMap apply the map config to the value. Should
         * only be done if this normalization is for a dependency ID.
         * @returns {String} normalized name
         */
        function normalize(name, baseName, applyMap) {
            var pkgMain, mapValue, nameParts, i, j, nameSegment, lastIndex,
                foundMap, foundI, foundStarMap, starI,
                baseParts = baseName && baseName.split('/'),
                normalizedBaseParts = baseParts,
                map = config.map,
                starMap = map && map['*'];

            //Adjust any relative paths.
            if (name && name.charAt(0) === '.') {
                //If have a base name, try to normalize against it,
                //otherwise, assume it is a top-level require that will
                //be relative to baseUrl in the end.
                if (baseName) {
                    //Convert baseName to array, and lop off the last part,
                    //so that . matches that 'directory' and not name of the baseName's
                    //module. For instance, baseName of 'one/two/three', maps to
                    //'one/two/three.js', but we want the directory, 'one/two' for
                    //this normalization.
                    normalizedBaseParts = baseParts.slice(0, baseParts.length - 1);
                    name = name.split('/');
                    lastIndex = name.length - 1;

                    // If wanting node ID compatibility, strip .js from end
                    // of IDs. Have to do this here, and not in nameToUrl
                    // because node allows either .js or non .js to map
                    // to same file.
                    if (config.nodeIdCompat && jsSuffixRegExp.test(name[lastIndex])) {
                        name[lastIndex] = name[lastIndex].replace(jsSuffixRegExp, '');
                    }

                    name = normalizedBaseParts.concat(name);
                    trimDots(name);
                    name = name.join('/');
                } else if (name.indexOf('./') === 0) {
                    // No baseName, so this is ID is resolved relative
                    // to baseUrl, pull off the leading dot.
                    name = name.substring(2);
                }
            }

            //Apply map config if available.
            if (applyMap && map && (baseParts || starMap)) {
                nameParts = name.split('/');

                outerLoop: for (i = nameParts.length; i > 0; i -= 1) {
                    nameSegment = nameParts.slice(0, i).join('/');

                    if (baseParts) {
                        //Find the longest baseName segment match in the config.
                        //So, do joins on the biggest to smallest lengths of baseParts.
                        for (j = baseParts.length; j > 0; j -= 1) {
                            mapValue = getOwn(map, baseParts.slice(0, j).join('/'));

                            //baseName segment has config, find if it has one for
                            //this name.
                            if (mapValue) {
                                mapValue = getOwn(mapValue, nameSegment);
                                if (mapValue) {
                                    //Match, update name to the new value.
                                    foundMap = mapValue;
                                    foundI = i;
                                    break outerLoop;
                                }
                            }
                        }
                    }

                    //Check for a star map match, but just hold on to it,
                    //if there is a shorter segment match later in a matching
                    //config, then favor over this star map.
                    if (!foundStarMap && starMap && getOwn(starMap, nameSegment)) {
                        foundStarMap = getOwn(starMap, nameSegment);
                        starI = i;
                    }
                }

                if (!foundMap && foundStarMap) {
                    foundMap = foundStarMap;
                    foundI = starI;
                }

                if (foundMap) {
                    nameParts.splice(0, foundI, foundMap);
                    name = nameParts.join('/');
                }
            }

            // If the name points to a package's name, use
            // the package main instead.
            pkgMain = getOwn(config.pkgs, name);

            return pkgMain ? pkgMain : name;
        }

        function removeScript(name) {
            if (isBrowser) {
                each(scripts(), function (scriptNode) {
                    if (scriptNode.getAttribute('data-requiremodule') === name &&
                            scriptNode.getAttribute('data-requirecontext') === context.contextName) {
                        scriptNode.parentNode.removeChild(scriptNode);
                        return true;
                    }
                });
            }
        }

        function hasPathFallback(id) {
            var pathConfig = getOwn(config.paths, id);
            if (pathConfig && isArray(pathConfig) && pathConfig.length > 1) {
                //Pop off the first array value, since it failed, and
                //retry
                pathConfig.shift();
                context.require.undef(id);
                context.require([id]);
                return true;
            }
        }

        //Turns a plugin!resource to [plugin, resource]
        //with the plugin being undefined if the name
        //did not have a plugin prefix.
        function splitPrefix(name) {
            var prefix,
                index = name ? name.indexOf('!') : -1;
            if (index > -1) {
                prefix = name.substring(0, index);
                name = name.substring(index + 1, name.length);
            }
            return [prefix, name];
        }

        /**
         * Creates a module mapping that includes plugin prefix, module
         * name, and path. If parentModuleMap is provided it will
         * also normalize the name via require.normalize()
         *
         * @param {String} name the module name
         * @param {String} [parentModuleMap] parent module map
         * for the module name, used to resolve relative names.
         * @param {Boolean} isNormalized: is the ID already normalized.
         * This is true if this call is done for a define() module ID.
         * @param {Boolean} applyMap: apply the map config to the ID.
         * Should only be true if this map is for a dependency.
         *
         * @returns {Object}
         */
        function makeModuleMap(name, parentModuleMap, isNormalized, applyMap) {
            var url, pluginModule, suffix, nameParts,
                prefix = null,
                parentName = parentModuleMap ? parentModuleMap.name : null,
                originalName = name,
                isDefine = true,
                normalizedName = '';

            //If no name, then it means it is a require call, generate an
            //internal name.
            if (!name) {
                isDefine = false;
                name = '_@r' + (requireCounter += 1);
            }

            nameParts = splitPrefix(name);
            prefix = nameParts[0];
            name = nameParts[1];

            if (prefix) {
                prefix = normalize(prefix, parentName, applyMap);
                pluginModule = getOwn(defined, prefix);
            }

            //Account for relative paths if there is a base name.
            if (name) {
                if (prefix) {
                    if (pluginModule && pluginModule.normalize) {
                        //Plugin is loaded, use its normalize method.
                        normalizedName = pluginModule.normalize(name, function (name) {
                            return normalize(name, parentName, applyMap);
                        });
                    } else {
                        normalizedName = normalize(name, parentName, applyMap);
                    }
                } else {
                    //A regular module.
                    normalizedName = normalize(name, parentName, applyMap);

                    //Normalized name may be a plugin ID due to map config
                    //application in normalize. The map config values must
                    //already be normalized, so do not need to redo that part.
                    nameParts = splitPrefix(normalizedName);
                    prefix = nameParts[0];
                    normalizedName = nameParts[1];
                    isNormalized = true;

                    url = context.nameToUrl(normalizedName);
                }
            }

            //If the id is a plugin id that cannot be determined if it needs
            //normalization, stamp it with a unique ID so two matching relative
            //ids that may conflict can be separate.
            suffix = prefix && !pluginModule && !isNormalized ?
                     '_unnormalized' + (unnormalizedCounter += 1) :
                     '';

            return {
                prefix: prefix,
                name: normalizedName,
                parentMap: parentModuleMap,
                unnormalized: !!suffix,
                url: url,
                originalName: originalName,
                isDefine: isDefine,
                id: (prefix ?
                        prefix + '!' + normalizedName :
                        normalizedName) + suffix
            };
        }

        function getModule(depMap) {
            var id = depMap.id,
                mod = getOwn(registry, id);

            if (!mod) {
                mod = registry[id] = new context.Module(depMap);
            }

            return mod;
        }

        function on(depMap, name, fn) {
            var id = depMap.id,
                mod = getOwn(registry, id);

            if (hasProp(defined, id) &&
                    (!mod || mod.defineEmitComplete)) {
                if (name === 'defined') {
                    fn(defined[id]);
                }
            } else {
                mod = getModule(depMap);
                if (mod.error && name === 'error') {
                    fn(mod.error);
                } else {
                    mod.on(name, fn);
                }
            }
        }

        function onError(err, errback) {
            var ids = err.requireModules,
                notified = false;

            if (errback) {
                errback(err);
            } else {
                each(ids, function (id) {
                    var mod = getOwn(registry, id);
                    if (mod) {
                        //Set error on module, so it skips timeout checks.
                        mod.error = err;
                        if (mod.events.error) {
                            notified = true;
                            mod.emit('error', err);
                        }
                    }
                });

                if (!notified) {
                    req.onError(err);
                }
            }
        }

        /**
         * Internal method to transfer globalQueue items to this context's
         * defQueue.
         */
        function takeGlobalQueue() {
            //Push all the globalDefQueue items into the context's defQueue
            if (globalDefQueue.length) {
                //Array splice in the values since the context code has a
                //local var ref to defQueue, so cannot just reassign the one
                //on context.
                apsp.apply(defQueue,
                           [defQueue.length, 0].concat(globalDefQueue));
                globalDefQueue = [];
            }
        }

        handlers = {
            'require': function (mod) {
                if (mod.require) {
                    return mod.require;
                } else {
                    return (mod.require = context.makeRequire(mod.map));
                }
            },
            'exports': function (mod) {
                mod.usingExports = true;
                if (mod.map.isDefine) {
                    if (mod.exports) {
                        return (defined[mod.map.id] = mod.exports);
                    } else {
                        return (mod.exports = defined[mod.map.id] = {});
                    }
                }
            },
            'module': function (mod) {
                if (mod.module) {
                    return mod.module;
                } else {
                    return (mod.module = {
                        id: mod.map.id,
                        uri: mod.map.url,
                        config: function () {
                            return  getOwn(config.config, mod.map.id) || {};
                        },
                        exports: mod.exports || (mod.exports = {})
                    });
                }
            }
        };

        function cleanRegistry(id) {
            //Clean up machinery used for waiting modules.
            delete registry[id];
            delete enabledRegistry[id];
        }

        function breakCycle(mod, traced, processed) {
            var id = mod.map.id;

            if (mod.error) {
                mod.emit('error', mod.error);
            } else {
                traced[id] = true;
                each(mod.depMaps, function (depMap, i) {
                    var depId = depMap.id,
                        dep = getOwn(registry, depId);

                    //Only force things that have not completed
                    //being defined, so still in the registry,
                    //and only if it has not been matched up
                    //in the module already.
                    if (dep && !mod.depMatched[i] && !processed[depId]) {
                        if (getOwn(traced, depId)) {
                            mod.defineDep(i, defined[depId]);
                            mod.check(); //pass false?
                        } else {
                            breakCycle(dep, traced, processed);
                        }
                    }
                });
                processed[id] = true;
            }
        }

        function checkLoaded() {
            var err, usingPathFallback,
                waitInterval = config.waitSeconds * 1000,
                //It is possible to disable the wait interval by using waitSeconds of 0.
                expired = waitInterval && (context.startTime + waitInterval) < new Date().getTime(),
                noLoads = [],
                reqCalls = [],
                stillLoading = false,
                needCycleCheck = true;

            //Do not bother if this call was a result of a cycle break.
            if (inCheckLoaded) {
                return;
            }

            inCheckLoaded = true;

            //Figure out the state of all the modules.
            eachProp(enabledRegistry, function (mod) {
                var map = mod.map,
                    modId = map.id;

                //Skip things that are not enabled or in error state.
                if (!mod.enabled) {
                    return;
                }

                if (!map.isDefine) {
                    reqCalls.push(mod);
                }

                if (!mod.error) {
                    //If the module should be executed, and it has not
                    //been inited and time is up, remember it.
                    if (!mod.inited && expired) {
                        if (hasPathFallback(modId)) {
                            usingPathFallback = true;
                            stillLoading = true;
                        } else {
                            noLoads.push(modId);
                            removeScript(modId);
                        }
                    } else if (!mod.inited && mod.fetched && map.isDefine) {
                        stillLoading = true;
                        if (!map.prefix) {
                            //No reason to keep looking for unfinished
                            //loading. If the only stillLoading is a
                            //plugin resource though, keep going,
                            //because it may be that a plugin resource
                            //is waiting on a non-plugin cycle.
                            return (needCycleCheck = false);
                        }
                    }
                }
            });

            if (expired && noLoads.length) {
                //If wait time expired, throw error of unloaded modules.
                err = makeError('timeout', 'Load timeout for modules: ' + noLoads, null, noLoads);
                err.contextName = context.contextName;
                return onError(err);
            }

            //Not expired, check for a cycle.
            if (needCycleCheck) {
                each(reqCalls, function (mod) {
                    breakCycle(mod, {}, {});
                });
            }

            //If still waiting on loads, and the waiting load is something
            //other than a plugin resource, or there are still outstanding
            //scripts, then just try back later.
            if ((!expired || usingPathFallback) && stillLoading) {
                //Something is still waiting to load. Wait for it, but only
                //if a timeout is not already in effect.
                if ((isBrowser || isWebWorker) && !checkLoadedTimeoutId) {
                    checkLoadedTimeoutId = setTimeout(function () {
                        checkLoadedTimeoutId = 0;
                        checkLoaded();
                    }, 50);
                }
            }

            inCheckLoaded = false;
        }

        Module = function (map) {
            this.events = getOwn(undefEvents, map.id) || {};
            this.map = map;
            this.shim = getOwn(config.shim, map.id);
            this.depExports = [];
            this.depMaps = [];
            this.depMatched = [];
            this.pluginMaps = {};
            this.depCount = 0;

            /* this.exports this.factory
               this.depMaps = [],
               this.enabled, this.fetched
            */
        };

        Module.prototype = {
            init: function (depMaps, factory, errback, options) {
                options = options || {};

                //Do not do more inits if already done. Can happen if there
                //are multiple define calls for the same module. That is not
                //a normal, common case, but it is also not unexpected.
                if (this.inited) {
                    return;
                }

                this.factory = factory;

                if (errback) {
                    //Register for errors on this module.
                    this.on('error', errback);
                } else if (this.events.error) {
                    //If no errback already, but there are error listeners
                    //on this module, set up an errback to pass to the deps.
                    errback = bind(this, function (err) {
                        this.emit('error', err);
                    });
                }

                //Do a copy of the dependency array, so that
                //source inputs are not modified. For example
                //"shim" deps are passed in here directly, and
                //doing a direct modification of the depMaps array
                //would affect that config.
                this.depMaps = depMaps && depMaps.slice(0);

                this.errback = errback;

                //Indicate this module has be initialized
                this.inited = true;

                this.ignore = options.ignore;

                //Could have option to init this module in enabled mode,
                //or could have been previously marked as enabled. However,
                //the dependencies are not known until init is called. So
                //if enabled previously, now trigger dependencies as enabled.
                if (options.enabled || this.enabled) {
                    //Enable this module and dependencies.
                    //Will call this.check()
                    this.enable();
                } else {
                    this.check();
                }
            },

            defineDep: function (i, depExports) {
                //Because of cycles, defined callback for a given
                //export can be called more than once.
                if (!this.depMatched[i]) {
                    this.depMatched[i] = true;
                    this.depCount -= 1;
                    this.depExports[i] = depExports;
                }
            },

            fetch: function () {
                if (this.fetched) {
                    return;
                }
                this.fetched = true;

                context.startTime = (new Date()).getTime();

                var map = this.map;

                //If the manager is for a plugin managed resource,
                //ask the plugin to load it now.
                if (this.shim) {
                    context.makeRequire(this.map, {
                        enableBuildCallback: true
                    })(this.shim.deps || [], bind(this, function () {
                        return map.prefix ? this.callPlugin() : this.load();
                    }));
                } else {
                    //Regular dependency.
                    return map.prefix ? this.callPlugin() : this.load();
                }
            },

            load: function () {
                var url = this.map.url;

                //Regular dependency.
                if (!urlFetched[url]) {
                    urlFetched[url] = true;
                    context.load(this.map.id, url);
                }
            },

            /**
             * Checks if the module is ready to define itself, and if so,
             * define it.
             */
            check: function () {
                if (!this.enabled || this.enabling) {
                    return;
                }

                var err, cjsModule,
                    id = this.map.id,
                    depExports = this.depExports,
                    exports = this.exports,
                    factory = this.factory;

                if (!this.inited) {
                    this.fetch();
                } else if (this.error) {
                    this.emit('error', this.error);
                } else if (!this.defining) {
                    //The factory could trigger another require call
                    //that would result in checking this module to
                    //define itself again. If already in the process
                    //of doing that, skip this work.
                    this.defining = true;

                    if (this.depCount < 1 && !this.defined) {
                        if (isFunction(factory)) {
                            //If there is an error listener, favor passing
                            //to that instead of throwing an error. However,
                            //only do it for define()'d  modules. require
                            //errbacks should not be called for failures in
                            //their callbacks (#699). However if a global
                            //onError is set, use that.
                            if ((this.events.error && this.map.isDefine) ||
                                req.onError !== defaultOnError) {
                                try {
                                    exports = context.execCb(id, factory, depExports, exports);
                                } catch (e) {
                                    err = e;
                                }
                            } else {
                                exports = context.execCb(id, factory, depExports, exports);
                            }

                            // Favor return value over exports. If node/cjs in play,
                            // then will not have a return value anyway. Favor
                            // module.exports assignment over exports object.
                            if (this.map.isDefine && exports === undefined) {
                                cjsModule = this.module;
                                if (cjsModule) {
                                    exports = cjsModule.exports;
                                } else if (this.usingExports) {
                                    //exports already set the defined value.
                                    exports = this.exports;
                                }
                            }

                            if (err) {
                                err.requireMap = this.map;
                                err.requireModules = this.map.isDefine ? [this.map.id] : null;
                                err.requireType = this.map.isDefine ? 'define' : 'require';
                                return onError((this.error = err));
                            }

                        } else {
                            //Just a literal value
                            exports = factory;
                        }

                        this.exports = exports;

                        if (this.map.isDefine && !this.ignore) {
                            defined[id] = exports;

                            if (req.onResourceLoad) {
                                req.onResourceLoad(context, this.map, this.depMaps);
                            }
                        }

                        //Clean up
                        cleanRegistry(id);

                        this.defined = true;
                    }

                    //Finished the define stage. Allow calling check again
                    //to allow define notifications below in the case of a
                    //cycle.
                    this.defining = false;

                    if (this.defined && !this.defineEmitted) {
                        this.defineEmitted = true;
                        this.emit('defined', this.exports);
                        this.defineEmitComplete = true;
                    }

                }
            },

            callPlugin: function () {
                var map = this.map,
                    id = map.id,
                    //Map already normalized the prefix.
                    pluginMap = makeModuleMap(map.prefix);

                //Mark this as a dependency for this plugin, so it
                //can be traced for cycles.
                this.depMaps.push(pluginMap);

                on(pluginMap, 'defined', bind(this, function (plugin) {
                    var load, normalizedMap, normalizedMod,
                        bundleId = getOwn(bundlesMap, this.map.id),
                        name = this.map.name,
                        parentName = this.map.parentMap ? this.map.parentMap.name : null,
                        localRequire = context.makeRequire(map.parentMap, {
                            enableBuildCallback: true
                        });

                    //If current map is not normalized, wait for that
                    //normalized name to load instead of continuing.
                    if (this.map.unnormalized) {
                        //Normalize the ID if the plugin allows it.
                        if (plugin.normalize) {
                            name = plugin.normalize(name, function (name) {
                                return normalize(name, parentName, true);
                            }) || '';
                        }

                        //prefix and name should already be normalized, no need
                        //for applying map config again either.
                        normalizedMap = makeModuleMap(map.prefix + '!' + name,
                                                      this.map.parentMap);
                        on(normalizedMap,
                            'defined', bind(this, function (value) {
                                this.init([], function () { return value; }, null, {
                                    enabled: true,
                                    ignore: true
                                });
                            }));

                        normalizedMod = getOwn(registry, normalizedMap.id);
                        if (normalizedMod) {
                            //Mark this as a dependency for this plugin, so it
                            //can be traced for cycles.
                            this.depMaps.push(normalizedMap);

                            if (this.events.error) {
                                normalizedMod.on('error', bind(this, function (err) {
                                    this.emit('error', err);
                                }));
                            }
                            normalizedMod.enable();
                        }

                        return;
                    }

                    //If a paths config, then just load that file instead to
                    //resolve the plugin, as it is built into that paths layer.
                    if (bundleId) {
                        this.map.url = context.nameToUrl(bundleId);
                        this.load();
                        return;
                    }

                    load = bind(this, function (value) {
                        this.init([], function () { return value; }, null, {
                            enabled: true
                        });
                    });

                    load.error = bind(this, function (err) {
                        this.inited = true;
                        this.error = err;
                        err.requireModules = [id];

                        //Remove temp unnormalized modules for this module,
                        //since they will never be resolved otherwise now.
                        eachProp(registry, function (mod) {
                            if (mod.map.id.indexOf(id + '_unnormalized') === 0) {
                                cleanRegistry(mod.map.id);
                            }
                        });

                        onError(err);
                    });

                    //Allow plugins to load other code without having to know the
                    //context or how to 'complete' the load.
                    load.fromText = bind(this, function (text, textAlt) {
                        /*jslint evil: true */
                        var moduleName = map.name,
                            moduleMap = makeModuleMap(moduleName),
                            hasInteractive = useInteractive;

                        //As of 2.1.0, support just passing the text, to reinforce
                        //fromText only being called once per resource. Still
                        //support old style of passing moduleName but discard
                        //that moduleName in favor of the internal ref.
                        if (textAlt) {
                            text = textAlt;
                        }

                        //Turn off interactive script matching for IE for any define
                        //calls in the text, then turn it back on at the end.
                        if (hasInteractive) {
                            useInteractive = false;
                        }

                        //Prime the system by creating a module instance for
                        //it.
                        getModule(moduleMap);

                        //Transfer any config to this other module.
                        if (hasProp(config.config, id)) {
                            config.config[moduleName] = config.config[id];
                        }

                        try {
                            req.exec(text);
                        } catch (e) {
                            return onError(makeError('fromtexteval',
                                             'fromText eval for ' + id +
                                            ' failed: ' + e,
                                             e,
                                             [id]));
                        }

                        if (hasInteractive) {
                            useInteractive = true;
                        }

                        //Mark this as a dependency for the plugin
                        //resource
                        this.depMaps.push(moduleMap);

                        //Support anonymous modules.
                        context.completeLoad(moduleName);

                        //Bind the value of that module to the value for this
                        //resource ID.
                        localRequire([moduleName], load);
                    });

                    //Use parentName here since the plugin's name is not reliable,
                    //could be some weird string with no path that actually wants to
                    //reference the parentName's path.
                    plugin.load(map.name, localRequire, load, config);
                }));

                context.enable(pluginMap, this);
                this.pluginMaps[pluginMap.id] = pluginMap;
            },

            enable: function () {
                enabledRegistry[this.map.id] = this;
                this.enabled = true;

                //Set flag mentioning that the module is enabling,
                //so that immediate calls to the defined callbacks
                //for dependencies do not trigger inadvertent load
                //with the depCount still being zero.
                this.enabling = true;

                //Enable each dependency
                each(this.depMaps, bind(this, function (depMap, i) {
                    var id, mod, handler;

                    if (typeof depMap === 'string') {
                        //Dependency needs to be converted to a depMap
                        //and wired up to this module.
                        depMap = makeModuleMap(depMap,
                                               (this.map.isDefine ? this.map : this.map.parentMap),
                                               false,
                                               !this.skipMap);
                        this.depMaps[i] = depMap;

                        handler = getOwn(handlers, depMap.id);

                        if (handler) {
                            this.depExports[i] = handler(this);
                            return;
                        }

                        this.depCount += 1;

                        on(depMap, 'defined', bind(this, function (depExports) {
                            this.defineDep(i, depExports);
                            this.check();
                        }));

                        if (this.errback) {
                            on(depMap, 'error', bind(this, this.errback));
                        }
                    }

                    id = depMap.id;
                    mod = registry[id];

                    //Skip special modules like 'require', 'exports', 'module'
                    //Also, don't call enable if it is already enabled,
                    //important in circular dependency cases.
                    if (!hasProp(handlers, id) && mod && !mod.enabled) {
                        context.enable(depMap, this);
                    }
                }));

                //Enable each plugin that is used in
                //a dependency
                eachProp(this.pluginMaps, bind(this, function (pluginMap) {
                    var mod = getOwn(registry, pluginMap.id);
                    if (mod && !mod.enabled) {
                        context.enable(pluginMap, this);
                    }
                }));

                this.enabling = false;

                this.check();
            },

            on: function (name, cb) {
                var cbs = this.events[name];
                if (!cbs) {
                    cbs = this.events[name] = [];
                }
                cbs.push(cb);
            },

            emit: function (name, evt) {
                each(this.events[name], function (cb) {
                    cb(evt);
                });
                if (name === 'error') {
                    //Now that the error handler was triggered, remove
                    //the listeners, since this broken Module instance
                    //can stay around for a while in the registry.
                    delete this.events[name];
                }
            }
        };

        function callGetModule(args) {
            //Skip modules already defined.
            if (!hasProp(defined, args[0])) {
                getModule(makeModuleMap(args[0], null, true)).init(args[1], args[2]);
            }
        }

        function removeListener(node, func, name, ieName) {
            //Favor detachEvent because of IE9
            //issue, see attachEvent/addEventListener comment elsewhere
            //in this file.
            if (node.detachEvent && !isOpera) {
                //Probably IE. If not it will throw an error, which will be
                //useful to know.
                if (ieName) {
                    node.detachEvent(ieName, func);
                }
            } else {
                node.removeEventListener(name, func, false);
            }
        }

        /**
         * Given an event from a script node, get the requirejs info from it,
         * and then removes the event listeners on the node.
         * @param {Event} evt
         * @returns {Object}
         */
        function getScriptData(evt) {
            //Using currentTarget instead of target for Firefox 2.0's sake. Not
            //all old browsers will be supported, but this one was easy enough
            //to support and still makes sense.
            var node = evt.currentTarget || evt.srcElement;

            //Remove the listeners once here.
            removeListener(node, context.onScriptLoad, 'load', 'onreadystatechange');
            removeListener(node, context.onScriptError, 'error');

            return {
                node: node,
                id: node && node.getAttribute('data-requiremodule')
            };
        }

        function intakeDefines() {
            var args;

            //Any defined modules in the global queue, intake them now.
            takeGlobalQueue();

            //Make sure any remaining defQueue items get properly processed.
            while (defQueue.length) {
                args = defQueue.shift();
                if (args[0] === null) {
                    return onError(makeError('mismatch', 'Mismatched anonymous define() module: ' + args[args.length - 1]));
                } else {
                    //args are id, deps, factory. Should be normalized by the
                    //define() function.
                    callGetModule(args);
                }
            }
        }

        context = {
            config: config,
            contextName: contextName,
            registry: registry,
            defined: defined,
            urlFetched: urlFetched,
            defQueue: defQueue,
            Module: Module,
            makeModuleMap: makeModuleMap,
            nextTick: req.nextTick,
            onError: onError,

            /**
             * Set a configuration for the context.
             * @param {Object} cfg config object to integrate.
             */
            configure: function (cfg) {
                //Make sure the baseUrl ends in a slash.
                if (cfg.baseUrl) {
                    if (cfg.baseUrl.charAt(cfg.baseUrl.length - 1) !== '/') {
                        cfg.baseUrl += '/';
                    }
                }

                //Save off the paths since they require special processing,
                //they are additive.
                var shim = config.shim,
                    objs = {
                        paths: true,
                        bundles: true,
                        config: true,
                        map: true
                    };

                eachProp(cfg, function (value, prop) {
                    if (objs[prop]) {
                        if (!config[prop]) {
                            config[prop] = {};
                        }
                        mixin(config[prop], value, true, true);
                    } else {
                        config[prop] = value;
                    }
                });

                //Reverse map the bundles
                if (cfg.bundles) {
                    eachProp(cfg.bundles, function (value, prop) {
                        each(value, function (v) {
                            if (v !== prop) {
                                bundlesMap[v] = prop;
                            }
                        });
                    });
                }

                //Merge shim
                if (cfg.shim) {
                    eachProp(cfg.shim, function (value, id) {
                        //Normalize the structure
                        if (isArray(value)) {
                            value = {
                                deps: value
                            };
                        }
                        if ((value.exports || value.init) && !value.exportsFn) {
                            value.exportsFn = context.makeShimExports(value);
                        }
                        shim[id] = value;
                    });
                    config.shim = shim;
                }

                //Adjust packages if necessary.
                if (cfg.packages) {
                    each(cfg.packages, function (pkgObj) {
                        var location, name;

                        pkgObj = typeof pkgObj === 'string' ? { name: pkgObj } : pkgObj;

                        name = pkgObj.name;
                        location = pkgObj.location;
                        if (location) {
                            config.paths[name] = pkgObj.location;
                        }

                        //Save pointer to main module ID for pkg name.
                        //Remove leading dot in main, so main paths are normalized,
                        //and remove any trailing .js, since different package
                        //envs have different conventions: some use a module name,
                        //some use a file name.
                        config.pkgs[name] = pkgObj.name + '/' + (pkgObj.main || 'main')
                                     .replace(currDirRegExp, '')
                                     .replace(jsSuffixRegExp, '');
                    });
                }

                //If there are any "waiting to execute" modules in the registry,
                //update the maps for them, since their info, like URLs to load,
                //may have changed.
                eachProp(registry, function (mod, id) {
                    //If module already has init called, since it is too
                    //late to modify them, and ignore unnormalized ones
                    //since they are transient.
                    if (!mod.inited && !mod.map.unnormalized) {
                        mod.map = makeModuleMap(id);
                    }
                });

                //If a deps array or a config callback is specified, then call
                //require with those args. This is useful when require is defined as a
                //config object before require.js is loaded.
                if (cfg.deps || cfg.callback) {
                    context.require(cfg.deps || [], cfg.callback);
                }
            },

            makeShimExports: function (value) {
                function fn() {
                    var ret;
                    if (value.init) {
                        ret = value.init.apply(global, arguments);
                    }
                    return ret || (value.exports && getGlobal(value.exports));
                }
                return fn;
            },

            makeRequire: function (relMap, options) {
                options = options || {};

                function localRequire(deps, callback, errback) {
                    var id, map, requireMod;

                    if (options.enableBuildCallback && callback && isFunction(callback)) {
                        callback.__requireJsBuild = true;
                    }

                    if (typeof deps === 'string') {
                        if (isFunction(callback)) {
                            //Invalid call
                            return onError(makeError('requireargs', 'Invalid require call'), errback);
                        }

                        //If require|exports|module are requested, get the
                        //value for them from the special handlers. Caveat:
                        //this only works while module is being defined.
                        if (relMap && hasProp(handlers, deps)) {
                            return handlers[deps](registry[relMap.id]);
                        }

                        //Synchronous access to one module. If require.get is
                        //available (as in the Node adapter), prefer that.
                        if (req.get) {
                            return req.get(context, deps, relMap, localRequire);
                        }

                        //Normalize module name, if it contains . or ..
                        map = makeModuleMap(deps, relMap, false, true);
                        id = map.id;

                        if (!hasProp(defined, id)) {
                            return onError(makeError('notloaded', 'Module name "' +
                                        id +
                                        '" has not been loaded yet for context: ' +
                                        contextName +
                                        (relMap ? '' : '. Use require([])')));
                        }
                        return defined[id];
                    }

                    //Grab defines waiting in the global queue.
                    intakeDefines();

                    //Mark all the dependencies as needing to be loaded.
                    context.nextTick(function () {
                        //Some defines could have been added since the
                        //require call, collect them.
                        intakeDefines();

                        requireMod = getModule(makeModuleMap(null, relMap));

                        //Store if map config should be applied to this require
                        //call for dependencies.
                        requireMod.skipMap = options.skipMap;

                        requireMod.init(deps, callback, errback, {
                            enabled: true
                        });

                        checkLoaded();
                    });

                    return localRequire;
                }

                mixin(localRequire, {
                    isBrowser: isBrowser,

                    /**
                     * Converts a module name + .extension into an URL path.
                     * *Requires* the use of a module name. It does not support using
                     * plain URLs like nameToUrl.
                     */
                    toUrl: function (moduleNamePlusExt) {
                        var ext,
                            index = moduleNamePlusExt.lastIndexOf('.'),
                            segment = moduleNamePlusExt.split('/')[0],
                            isRelative = segment === '.' || segment === '..';

                        //Have a file extension alias, and it is not the
                        //dots from a relative path.
                        if (index !== -1 && (!isRelative || index > 1)) {
                            ext = moduleNamePlusExt.substring(index, moduleNamePlusExt.length);
                            moduleNamePlusExt = moduleNamePlusExt.substring(0, index);
                        }

                        return context.nameToUrl(normalize(moduleNamePlusExt,
                                                relMap && relMap.id, true), ext,  true);
                    },

                    defined: function (id) {
                        return hasProp(defined, makeModuleMap(id, relMap, false, true).id);
                    },

                    specified: function (id) {
                        id = makeModuleMap(id, relMap, false, true).id;
                        return hasProp(defined, id) || hasProp(registry, id);
                    }
                });

                //Only allow undef on top level require calls
                if (!relMap) {
                    localRequire.undef = function (id) {
                        //Bind any waiting define() calls to this context,
                        //fix for #408
                        takeGlobalQueue();

                        var map = makeModuleMap(id, relMap, true),
                            mod = getOwn(registry, id);

                        removeScript(id);

                        delete defined[id];
                        delete urlFetched[map.url];
                        delete undefEvents[id];

                        //Clean queued defines too. Go backwards
                        //in array so that the splices do not
                        //mess up the iteration.
                        eachReverse(defQueue, function(args, i) {
                            if(args[0] === id) {
                                defQueue.splice(i, 1);
                            }
                        });

                        if (mod) {
                            //Hold on to listeners in case the
                            //module will be attempted to be reloaded
                            //using a different config.
                            if (mod.events.defined) {
                                undefEvents[id] = mod.events;
                            }

                            cleanRegistry(id);
                        }
                    };
                }

                return localRequire;
            },

            /**
             * Called to enable a module if it is still in the registry
             * awaiting enablement. A second arg, parent, the parent module,
             * is passed in for context, when this method is overridden by
             * the optimizer. Not shown here to keep code compact.
             */
            enable: function (depMap) {
                var mod = getOwn(registry, depMap.id);
                if (mod) {
                    getModule(depMap).enable();
                }
            },

            /**
             * Internal method used by environment adapters to complete a load event.
             * A load event could be a script load or just a load pass from a synchronous
             * load call.
             * @param {String} moduleName the name of the module to potentially complete.
             */
            completeLoad: function (moduleName) {
                var found, args, mod,
                    shim = getOwn(config.shim, moduleName) || {},
                    shExports = shim.exports;

                takeGlobalQueue();

                while (defQueue.length) {
                    args = defQueue.shift();
                    if (args[0] === null) {
                        args[0] = moduleName;
                        //If already found an anonymous module and bound it
                        //to this name, then this is some other anon module
                        //waiting for its completeLoad to fire.
                        if (found) {
                            break;
                        }
                        found = true;
                    } else if (args[0] === moduleName) {
                        //Found matching define call for this script!
                        found = true;
                    }

                    callGetModule(args);
                }

                //Do this after the cycle of callGetModule in case the result
                //of those calls/init calls changes the registry.
                mod = getOwn(registry, moduleName);

                if (!found && !hasProp(defined, moduleName) && mod && !mod.inited) {
                    if (config.enforceDefine && (!shExports || !getGlobal(shExports))) {
                        if (hasPathFallback(moduleName)) {
                            return;
                        } else {
                            return onError(makeError('nodefine',
                                             'No define call for ' + moduleName,
                                             null,
                                             [moduleName]));
                        }
                    } else {
                        //A script that does not call define(), so just simulate
                        //the call for it.
                        callGetModule([moduleName, (shim.deps || []), shim.exportsFn]);
                    }
                }

                checkLoaded();
            },

            /**
             * Converts a module name to a file path. Supports cases where
             * moduleName may actually be just an URL.
             * Note that it **does not** call normalize on the moduleName,
             * it is assumed to have already been normalized. This is an
             * internal API, not a public one. Use toUrl for the public API.
             */
            nameToUrl: function (moduleName, ext, skipExt) {
                var paths, syms, i, parentModule, url,
                    parentPath, bundleId,
                    pkgMain = getOwn(config.pkgs, moduleName);

                if (pkgMain) {
                    moduleName = pkgMain;
                }

                bundleId = getOwn(bundlesMap, moduleName);

                if (bundleId) {
                    return context.nameToUrl(bundleId, ext, skipExt);
                }

                //If a colon is in the URL, it indicates a protocol is used and it is just
                //an URL to a file, or if it starts with a slash, contains a query arg (i.e. ?)
                //or ends with .js, then assume the user meant to use an url and not a module id.
                //The slash is important for protocol-less URLs as well as full paths.
                if (req.jsExtRegExp.test(moduleName)) {
                    //Just a plain path, not module name lookup, so just return it.
                    //Add extension if it is included. This is a bit wonky, only non-.js things pass
                    //an extension, this method probably needs to be reworked.
                    url = moduleName + (ext || '');
                } else {
                    //A module that needs to be converted to a path.
                    paths = config.paths;

                    syms = moduleName.split('/');
                    //For each module name segment, see if there is a path
                    //registered for it. Start with most specific name
                    //and work up from it.
                    for (i = syms.length; i > 0; i -= 1) {
                        parentModule = syms.slice(0, i).join('/');

                        parentPath = getOwn(paths, parentModule);
                        if (parentPath) {
                            //If an array, it means there are a few choices,
                            //Choose the one that is desired
                            if (isArray(parentPath)) {
                                parentPath = parentPath[0];
                            }
                            syms.splice(0, i, parentPath);
                            break;
                        }
                    }

                    //Join the path parts together, then figure out if baseUrl is needed.
                    url = syms.join('/');
                    url += (ext || (/^data\:|\?/.test(url) || skipExt ? '' : '.js'));
                    url = (url.charAt(0) === '/' || url.match(/^[\w\+\.\-]+:/) ? '' : config.baseUrl) + url;
                }

                return config.urlArgs ? url +
                                        ((url.indexOf('?') === -1 ? '?' : '&') +
                                         config.urlArgs) : url;
            },

            //Delegates to req.load. Broken out as a separate function to
            //allow overriding in the optimizer.
            load: function (id, url) {
                req.load(context, id, url);
            },

            /**
             * Executes a module callback function. Broken out as a separate function
             * solely to allow the build system to sequence the files in the built
             * layer in the right sequence.
             *
             * @private
             */
            execCb: function (name, callback, args, exports) {
                return callback.apply(exports, args);
            },

            /**
             * callback for script loads, used to check status of loading.
             *
             * @param {Event} evt the event from the browser for the script
             * that was loaded.
             */
            onScriptLoad: function (evt) {
                //Using currentTarget instead of target for Firefox 2.0's sake. Not
                //all old browsers will be supported, but this one was easy enough
                //to support and still makes sense.
                if (evt.type === 'load' ||
                        (readyRegExp.test((evt.currentTarget || evt.srcElement).readyState))) {
                    //Reset interactive script so a script node is not held onto for
                    //to long.
                    interactiveScript = null;

                    //Pull out the name of the module and the context.
                    var data = getScriptData(evt);
                    context.completeLoad(data.id);
                }
            },

            /**
             * Callback for script errors.
             */
            onScriptError: function (evt) {
                var data = getScriptData(evt);
                if (!hasPathFallback(data.id)) {
                    return onError(makeError('scripterror', 'Script error for: ' + data.id, evt, [data.id]));
                }
            }
        };

        context.require = context.makeRequire();
        return context;
    }

    /**
     * Main entry point.
     *
     * If the only argument to require is a string, then the module that
     * is represented by that string is fetched for the appropriate context.
     *
     * If the first argument is an array, then it will be treated as an array
     * of dependency string names to fetch. An optional function callback can
     * be specified to execute when all of those dependencies are available.
     *
     * Make a local req variable to help Caja compliance (it assumes things
     * on a require that are not standardized), and to give a short
     * name for minification/local scope use.
     */
    req = requirejs = function (deps, callback, errback, optional) {

        //Find the right context, use default
        var context, config,
            contextName = defContextName;

        // Determine if have config object in the call.
        if (!isArray(deps) && typeof deps !== 'string') {
            // deps is a config object
            config = deps;
            if (isArray(callback)) {
                // Adjust args if there are dependencies
                deps = callback;
                callback = errback;
                errback = optional;
            } else {
                deps = [];
            }
        }

        if (config && config.context) {
            contextName = config.context;
        }

        context = getOwn(contexts, contextName);
        if (!context) {
            context = contexts[contextName] = req.s.newContext(contextName);
        }

        if (config) {
            context.configure(config);
        }

        return context.require(deps, callback, errback);
    };

    /**
     * Support require.config() to make it easier to cooperate with other
     * AMD loaders on globally agreed names.
     */
    req.config = function (config) {
        return req(config);
    };

    /**
     * Execute something after the current tick
     * of the event loop. Override for other envs
     * that have a better solution than setTimeout.
     * @param  {Function} fn function to execute later.
     */
    req.nextTick = typeof setTimeout !== 'undefined' ? function (fn) {
        setTimeout(fn, 4);
    } : function (fn) { fn(); };

    /**
     * Export require as a global, but only if it does not already exist.
     */
    if (!require) {
        require = req;
    }

    req.version = version;

    //Used to filter out dependencies that are already paths.
    req.jsExtRegExp = /^\/|:|\?|\.js$/;
    req.isBrowser = isBrowser;
    s = req.s = {
        contexts: contexts,
        newContext: newContext
    };

    //Create default context.
    req({});

    //Exports some context-sensitive methods on global require.
    each([
        'toUrl',
        'undef',
        'defined',
        'specified'
    ], function (prop) {
        //Reference from contexts instead of early binding to default context,
        //so that during builds, the latest instance of the default context
        //with its config gets used.
        req[prop] = function () {
            var ctx = contexts[defContextName];
            return ctx.require[prop].apply(ctx, arguments);
        };
    });

    if (isBrowser) {
        head = s.head = document.getElementsByTagName('head')[0];
        //If BASE tag is in play, using appendChild is a problem for IE6.
        //When that browser dies, this can be removed. Details in this jQuery bug:
        //http://dev.jquery.com/ticket/2709
        baseElement = document.getElementsByTagName('base')[0];
        if (baseElement) {
            head = s.head = baseElement.parentNode;
        }
    }

    /**
     * Any errors that require explicitly generates will be passed to this
     * function. Intercept/override it if you want custom error handling.
     * @param {Error} err the error object.
     */
    req.onError = defaultOnError;

    /**
     * Creates the node for the load command. Only used in browser envs.
     */
    req.createNode = function (config, moduleName, url) {
        var node = config.xhtml ?
                document.createElementNS('http://www.w3.org/1999/xhtml', 'html:script') :
                document.createElement('script');
        node.type = config.scriptType || 'text/javascript';
        node.charset = 'utf-8';
        node.async = true;
        return node;
    };

    /**
     * Does the request to load a module for the browser case.
     * Make this a separate function to allow other environments
     * to override it.
     *
     * @param {Object} context the require context to find state.
     * @param {String} moduleName the name of the module.
     * @param {Object} url the URL to the module.
     */
    req.load = function (context, moduleName, url) {
        var config = (context && context.config) || {},
            node;
        if (isBrowser) {
            //In the browser so use a script tag
            node = req.createNode(config, moduleName, url);

            node.setAttribute('data-requirecontext', context.contextName);
            node.setAttribute('data-requiremodule', moduleName);

            //Set up load listener. Test attachEvent first because IE9 has
            //a subtle issue in its addEventListener and script onload firings
            //that do not match the behavior of all other browsers with
            //addEventListener support, which fire the onload event for a
            //script right after the script execution. See:
            //https://connect.microsoft.com/IE/feedback/details/648057/script-onload-event-is-not-fired-immediately-after-script-execution
            //UNFORTUNATELY Opera implements attachEvent but does not follow the script
            //script execution mode.
            if (node.attachEvent &&
                    //Check if node.attachEvent is artificially added by custom script or
                    //natively supported by browser
                    //read https://github.com/jrburke/requirejs/issues/187
                    //if we can NOT find [native code] then it must NOT natively supported.
                    //in IE8, node.attachEvent does not have toString()
                    //Note the test for "[native code" with no closing brace, see:
                    //https://github.com/jrburke/requirejs/issues/273
                    !(node.attachEvent.toString && node.attachEvent.toString().indexOf('[native code') < 0) &&
                    !isOpera) {
                //Probably IE. IE (at least 6-8) do not fire
                //script onload right after executing the script, so
                //we cannot tie the anonymous define call to a name.
                //However, IE reports the script as being in 'interactive'
                //readyState at the time of the define call.
                useInteractive = true;

                node.attachEvent('onreadystatechange', context.onScriptLoad);
                //It would be great to add an error handler here to catch
                //404s in IE9+. However, onreadystatechange will fire before
                //the error handler, so that does not help. If addEventListener
                //is used, then IE will fire error before load, but we cannot
                //use that pathway given the connect.microsoft.com issue
                //mentioned above about not doing the 'script execute,
                //then fire the script load event listener before execute
                //next script' that other browsers do.
                //Best hope: IE10 fixes the issues,
                //and then destroys all installs of IE 6-9.
                //node.attachEvent('onerror', context.onScriptError);
            } else {
                node.addEventListener('load', context.onScriptLoad, false);
                node.addEventListener('error', context.onScriptError, false);
            }
            node.src = url;

            //For some cache cases in IE 6-8, the script executes before the end
            //of the appendChild execution, so to tie an anonymous define
            //call to the module name (which is stored on the node), hold on
            //to a reference to this node, but clear after the DOM insertion.
            currentlyAddingScript = node;
            if (baseElement) {
                head.insertBefore(node, baseElement);
            } else {
                head.appendChild(node);
            }
            currentlyAddingScript = null;

            return node;
        } else if (isWebWorker) {
            try {
                //In a web worker, use importScripts. This is not a very
                //efficient use of importScripts, importScripts will block until
                //its script is downloaded and evaluated. However, if web workers
                //are in play, the expectation that a build has been done so that
                //only one script needs to be loaded anyway. This may need to be
                //reevaluated if other use cases become common.
                importScripts(url);

                //Account for anonymous modules
                context.completeLoad(moduleName);
            } catch (e) {
                context.onError(makeError('importscripts',
                                'importScripts failed for ' +
                                    moduleName + ' at ' + url,
                                e,
                                [moduleName]));
            }
        }
    };

    function getInteractiveScript() {
        if (interactiveScript && interactiveScript.readyState === 'interactive') {
            return interactiveScript;
        }

        eachReverse(scripts(), function (script) {
            if (script.readyState === 'interactive') {
                return (interactiveScript = script);
            }
        });
        return interactiveScript;
    }

    //Look for a data-main script attribute, which could also adjust the baseUrl.
    if (isBrowser && !cfg.skipDataMain) {
        //Figure out baseUrl. Get it from the script tag with require.js in it.
        eachReverse(scripts(), function (script) {
            //Set the 'head' where we can append children by
            //using the script's parent.
            if (!head) {
                head = script.parentNode;
            }

            //Look for a data-main attribute to set main script for the page
            //to load. If it is there, the path to data main becomes the
            //baseUrl, if it is not already set.
            dataMain = script.getAttribute('data-main');
            if (dataMain) {
                //Preserve dataMain in case it is a path (i.e. contains '?')
                mainScript = dataMain;

                //Set final baseUrl if there is not already an explicit one.
                if (!cfg.baseUrl) {
                    //Pull off the directory of data-main for use as the
                    //baseUrl.
                    src = mainScript.split('/');
                    mainScript = src.pop();
                    subPath = src.length ? src.join('/')  + '/' : './';

                    cfg.baseUrl = subPath;
                }

                //Strip off any trailing .js since mainScript is now
                //like a module name.
                mainScript = mainScript.replace(jsSuffixRegExp, '');

                 //If mainScript is still a path, fall back to dataMain
                if (req.jsExtRegExp.test(mainScript)) {
                    mainScript = dataMain;
                }

                //Put the data-main script in the files to load.
                cfg.deps = cfg.deps ? cfg.deps.concat(mainScript) : [mainScript];

                return true;
            }
        });
    }

    /**
     * The function that handles definitions of modules. Differs from
     * require() in that a string for the module should be the first argument,
     * and the function to execute after dependencies are loaded should
     * return a value to define the module corresponding to the first argument's
     * name.
     */
    define = function (name, deps, callback) {
        var node, context;

        //Allow for anonymous modules
        if (typeof name !== 'string') {
            //Adjust args appropriately
            callback = deps;
            deps = name;
            name = null;
        }

        //This module may not have dependencies
        if (!isArray(deps)) {
            callback = deps;
            deps = null;
        }

        //If no name, and callback is a function, then figure out if it a
        //CommonJS thing with dependencies.
        if (!deps && isFunction(callback)) {
            deps = [];
            //Remove comments from the callback string,
            //look for require calls, and pull them into the dependencies,
            //but only if there are function args.
            if (callback.length) {
                callback
                    .toString()
                    .replace(commentRegExp, '')
                    .replace(cjsRequireRegExp, function (match, dep) {
                        deps.push(dep);
                    });

                //May be a CommonJS thing even without require calls, but still
                //could use exports, and module. Avoid doing exports and module
                //work though if it just needs require.
                //REQUIRES the function to expect the CommonJS variables in the
                //order listed below.
                deps = (callback.length === 1 ? ['require'] : ['require', 'exports', 'module']).concat(deps);
            }
        }

        //If in IE 6-8 and hit an anonymous define() call, do the interactive
        //work.
        if (useInteractive) {
            node = currentlyAddingScript || getInteractiveScript();
            if (node) {
                if (!name) {
                    name = node.getAttribute('data-requiremodule');
                }
                context = contexts[node.getAttribute('data-requirecontext')];
            }
        }

        //Always save off evaluating the def call until the script onload handler.
        //This allows multiple modules to be in a file without prematurely
        //tracing dependencies, and allows for anonymous module support,
        //where the module name is not known until the script onload event
        //occurs. If no context, use the global queue, and get it processed
        //in the onscript load callback.
        (context ? context.defQueue : globalDefQueue).push([name, deps, callback]);
    };

    define.amd = {
        jQuery: true
    };


    /**
     * Executes the text. Normally just uses eval, but can be modified
     * to use a better, environment-specific call. Only used for transpiling
     * loader plugins, not for plain JS modules.
     * @param {String} text the text to execute/evaluate.
     */
    req.exec = function (text) {
        /*jslint evil: true */
        return eval(text);
    };

    //Set up with config info.
    req(cfg);
}(this));

define("../bower_components/requirejs/require", function(){});

/*!
 * jQuery JavaScript Library v2.1.0
 * http://jquery.com/
 *
 * Includes Sizzle.js
 * http://sizzlejs.com/
 *
 * Copyright 2005, 2014 jQuery Foundation, Inc. and other contributors
 * Released under the MIT license
 * http://jquery.org/license
 *
 * Date: 2014-01-23T21:10Z
 */

(function( global, factory ) {

	if ( typeof module === "object" && typeof module.exports === "object" ) {
		// For CommonJS and CommonJS-like environments where a proper window is present,
		// execute the factory and get jQuery
		// For environments that do not inherently posses a window with a document
		// (such as Node.js), expose a jQuery-making factory as module.exports
		// This accentuates the need for the creation of a real window
		// e.g. var jQuery = require("jquery")(window);
		// See ticket #14549 for more info
		module.exports = global.document ?
			factory( global, true ) :
			function( w ) {
				if ( !w.document ) {
					throw new Error( "jQuery requires a window with a document" );
				}
				return factory( w );
			};
	} else {
		factory( global );
	}

// Pass this if window is not defined yet
}(typeof window !== "undefined" ? window : this, function( window, noGlobal ) {

// Can't do this because several apps including ASP.NET trace
// the stack via arguments.caller.callee and Firefox dies if
// you try to trace through "use strict" call chains. (#13335)
// Support: Firefox 18+
//

var arr = [];

var slice = arr.slice;

var concat = arr.concat;

var push = arr.push;

var indexOf = arr.indexOf;

var class2type = {};

var toString = class2type.toString;

var hasOwn = class2type.hasOwnProperty;

var trim = "".trim;

var support = {};



var
	// Use the correct document accordingly with window argument (sandbox)
	document = window.document,

	version = "2.1.0",

	// Define a local copy of jQuery
	jQuery = function( selector, context ) {
		// The jQuery object is actually just the init constructor 'enhanced'
		// Need init if jQuery is called (just allow error to be thrown if not included)
		return new jQuery.fn.init( selector, context );
	},

	// Matches dashed string for camelizing
	rmsPrefix = /^-ms-/,
	rdashAlpha = /-([\da-z])/gi,

	// Used by jQuery.camelCase as callback to replace()
	fcamelCase = function( all, letter ) {
		return letter.toUpperCase();
	};

jQuery.fn = jQuery.prototype = {
	// The current version of jQuery being used
	jquery: version,

	constructor: jQuery,

	// Start with an empty selector
	selector: "",

	// The default length of a jQuery object is 0
	length: 0,

	toArray: function() {
		return slice.call( this );
	},

	// Get the Nth element in the matched element set OR
	// Get the whole matched element set as a clean array
	get: function( num ) {
		return num != null ?

			// Return a 'clean' array
			( num < 0 ? this[ num + this.length ] : this[ num ] ) :

			// Return just the object
			slice.call( this );
	},

	// Take an array of elements and push it onto the stack
	// (returning the new matched element set)
	pushStack: function( elems ) {

		// Build a new jQuery matched element set
		var ret = jQuery.merge( this.constructor(), elems );

		// Add the old object onto the stack (as a reference)
		ret.prevObject = this;
		ret.context = this.context;

		// Return the newly-formed element set
		return ret;
	},

	// Execute a callback for every element in the matched set.
	// (You can seed the arguments with an array of args, but this is
	// only used internally.)
	each: function( callback, args ) {
		return jQuery.each( this, callback, args );
	},

	map: function( callback ) {
		return this.pushStack( jQuery.map(this, function( elem, i ) {
			return callback.call( elem, i, elem );
		}));
	},

	slice: function() {
		return this.pushStack( slice.apply( this, arguments ) );
	},

	first: function() {
		return this.eq( 0 );
	},

	last: function() {
		return this.eq( -1 );
	},

	eq: function( i ) {
		var len = this.length,
			j = +i + ( i < 0 ? len : 0 );
		return this.pushStack( j >= 0 && j < len ? [ this[j] ] : [] );
	},

	end: function() {
		return this.prevObject || this.constructor(null);
	},

	// For internal use only.
	// Behaves like an Array's method, not like a jQuery method.
	push: push,
	sort: arr.sort,
	splice: arr.splice
};

jQuery.extend = jQuery.fn.extend = function() {
	var options, name, src, copy, copyIsArray, clone,
		target = arguments[0] || {},
		i = 1,
		length = arguments.length,
		deep = false;

	// Handle a deep copy situation
	if ( typeof target === "boolean" ) {
		deep = target;

		// skip the boolean and the target
		target = arguments[ i ] || {};
		i++;
	}

	// Handle case when target is a string or something (possible in deep copy)
	if ( typeof target !== "object" && !jQuery.isFunction(target) ) {
		target = {};
	}

	// extend jQuery itself if only one argument is passed
	if ( i === length ) {
		target = this;
		i--;
	}

	for ( ; i < length; i++ ) {
		// Only deal with non-null/undefined values
		if ( (options = arguments[ i ]) != null ) {
			// Extend the base object
			for ( name in options ) {
				src = target[ name ];
				copy = options[ name ];

				// Prevent never-ending loop
				if ( target === copy ) {
					continue;
				}

				// Recurse if we're merging plain objects or arrays
				if ( deep && copy && ( jQuery.isPlainObject(copy) || (copyIsArray = jQuery.isArray(copy)) ) ) {
					if ( copyIsArray ) {
						copyIsArray = false;
						clone = src && jQuery.isArray(src) ? src : [];

					} else {
						clone = src && jQuery.isPlainObject(src) ? src : {};
					}

					// Never move original objects, clone them
					target[ name ] = jQuery.extend( deep, clone, copy );

				// Don't bring in undefined values
				} else if ( copy !== undefined ) {
					target[ name ] = copy;
				}
			}
		}
	}

	// Return the modified object
	return target;
};

jQuery.extend({
	// Unique for each copy of jQuery on the page
	expando: "jQuery" + ( version + Math.random() ).replace( /\D/g, "" ),

	// Assume jQuery is ready without the ready module
	isReady: true,

	error: function( msg ) {
		throw new Error( msg );
	},

	noop: function() {},

	// See test/unit/core.js for details concerning isFunction.
	// Since version 1.3, DOM methods and functions like alert
	// aren't supported. They return false on IE (#2968).
	isFunction: function( obj ) {
		return jQuery.type(obj) === "function";
	},

	isArray: Array.isArray,

	isWindow: function( obj ) {
		return obj != null && obj === obj.window;
	},

	isNumeric: function( obj ) {
		// parseFloat NaNs numeric-cast false positives (null|true|false|"")
		// ...but misinterprets leading-number strings, particularly hex literals ("0x...")
		// subtraction forces infinities to NaN
		return obj - parseFloat( obj ) >= 0;
	},

	isPlainObject: function( obj ) {
		// Not plain objects:
		// - Any object or value whose internal [[Class]] property is not "[object Object]"
		// - DOM nodes
		// - window
		if ( jQuery.type( obj ) !== "object" || obj.nodeType || jQuery.isWindow( obj ) ) {
			return false;
		}

		// Support: Firefox <20
		// The try/catch suppresses exceptions thrown when attempting to access
		// the "constructor" property of certain host objects, ie. |window.location|
		// https://bugzilla.mozilla.org/show_bug.cgi?id=814622
		try {
			if ( obj.constructor &&
					!hasOwn.call( obj.constructor.prototype, "isPrototypeOf" ) ) {
				return false;
			}
		} catch ( e ) {
			return false;
		}

		// If the function hasn't returned already, we're confident that
		// |obj| is a plain object, created by {} or constructed with new Object
		return true;
	},

	isEmptyObject: function( obj ) {
		var name;
		for ( name in obj ) {
			return false;
		}
		return true;
	},

	type: function( obj ) {
		if ( obj == null ) {
			return obj + "";
		}
		// Support: Android < 4.0, iOS < 6 (functionish RegExp)
		return typeof obj === "object" || typeof obj === "function" ?
			class2type[ toString.call(obj) ] || "object" :
			typeof obj;
	},

	// Evaluates a script in a global context
	globalEval: function( code ) {
		var script,
			indirect = eval;

		code = jQuery.trim( code );

		if ( code ) {
			// If the code includes a valid, prologue position
			// strict mode pragma, execute code by injecting a
			// script tag into the document.
			if ( code.indexOf("use strict") === 1 ) {
				script = document.createElement("script");
				script.text = code;
				document.head.appendChild( script ).parentNode.removeChild( script );
			} else {
			// Otherwise, avoid the DOM node creation, insertion
			// and removal by using an indirect global eval
				indirect( code );
			}
		}
	},

	// Convert dashed to camelCase; used by the css and data modules
	// Microsoft forgot to hump their vendor prefix (#9572)
	camelCase: function( string ) {
		return string.replace( rmsPrefix, "ms-" ).replace( rdashAlpha, fcamelCase );
	},

	nodeName: function( elem, name ) {
		return elem.nodeName && elem.nodeName.toLowerCase() === name.toLowerCase();
	},

	// args is for internal usage only
	each: function( obj, callback, args ) {
		var value,
			i = 0,
			length = obj.length,
			isArray = isArraylike( obj );

		if ( args ) {
			if ( isArray ) {
				for ( ; i < length; i++ ) {
					value = callback.apply( obj[ i ], args );

					if ( value === false ) {
						break;
					}
				}
			} else {
				for ( i in obj ) {
					value = callback.apply( obj[ i ], args );

					if ( value === false ) {
						break;
					}
				}
			}

		// A special, fast, case for the most common use of each
		} else {
			if ( isArray ) {
				for ( ; i < length; i++ ) {
					value = callback.call( obj[ i ], i, obj[ i ] );

					if ( value === false ) {
						break;
					}
				}
			} else {
				for ( i in obj ) {
					value = callback.call( obj[ i ], i, obj[ i ] );

					if ( value === false ) {
						break;
					}
				}
			}
		}

		return obj;
	},

	trim: function( text ) {
		return text == null ? "" : trim.call( text );
	},

	// results is for internal usage only
	makeArray: function( arr, results ) {
		var ret = results || [];

		if ( arr != null ) {
			if ( isArraylike( Object(arr) ) ) {
				jQuery.merge( ret,
					typeof arr === "string" ?
					[ arr ] : arr
				);
			} else {
				push.call( ret, arr );
			}
		}

		return ret;
	},

	inArray: function( elem, arr, i ) {
		return arr == null ? -1 : indexOf.call( arr, elem, i );
	},

	merge: function( first, second ) {
		var len = +second.length,
			j = 0,
			i = first.length;

		for ( ; j < len; j++ ) {
			first[ i++ ] = second[ j ];
		}

		first.length = i;

		return first;
	},

	grep: function( elems, callback, invert ) {
		var callbackInverse,
			matches = [],
			i = 0,
			length = elems.length,
			callbackExpect = !invert;

		// Go through the array, only saving the items
		// that pass the validator function
		for ( ; i < length; i++ ) {
			callbackInverse = !callback( elems[ i ], i );
			if ( callbackInverse !== callbackExpect ) {
				matches.push( elems[ i ] );
			}
		}

		return matches;
	},

	// arg is for internal usage only
	map: function( elems, callback, arg ) {
		var value,
			i = 0,
			length = elems.length,
			isArray = isArraylike( elems ),
			ret = [];

		// Go through the array, translating each of the items to their new values
		if ( isArray ) {
			for ( ; i < length; i++ ) {
				value = callback( elems[ i ], i, arg );

				if ( value != null ) {
					ret.push( value );
				}
			}

		// Go through every key on the object,
		} else {
			for ( i in elems ) {
				value = callback( elems[ i ], i, arg );

				if ( value != null ) {
					ret.push( value );
				}
			}
		}

		// Flatten any nested arrays
		return concat.apply( [], ret );
	},

	// A global GUID counter for objects
	guid: 1,

	// Bind a function to a context, optionally partially applying any
	// arguments.
	proxy: function( fn, context ) {
		var tmp, args, proxy;

		if ( typeof context === "string" ) {
			tmp = fn[ context ];
			context = fn;
			fn = tmp;
		}

		// Quick check to determine if target is callable, in the spec
		// this throws a TypeError, but we will just return undefined.
		if ( !jQuery.isFunction( fn ) ) {
			return undefined;
		}

		// Simulated bind
		args = slice.call( arguments, 2 );
		proxy = function() {
			return fn.apply( context || this, args.concat( slice.call( arguments ) ) );
		};

		// Set the guid of unique handler to the same of original handler, so it can be removed
		proxy.guid = fn.guid = fn.guid || jQuery.guid++;

		return proxy;
	},

	now: Date.now,

	// jQuery.support is not used in Core but other projects attach their
	// properties to it so it needs to exist.
	support: support
});

// Populate the class2type map
jQuery.each("Boolean Number String Function Array Date RegExp Object Error".split(" "), function(i, name) {
	class2type[ "[object " + name + "]" ] = name.toLowerCase();
});

function isArraylike( obj ) {
	var length = obj.length,
		type = jQuery.type( obj );

	if ( type === "function" || jQuery.isWindow( obj ) ) {
		return false;
	}

	if ( obj.nodeType === 1 && length ) {
		return true;
	}

	return type === "array" || length === 0 ||
		typeof length === "number" && length > 0 && ( length - 1 ) in obj;
}
var Sizzle =
/*!
 * Sizzle CSS Selector Engine v1.10.16
 * http://sizzlejs.com/
 *
 * Copyright 2013 jQuery Foundation, Inc. and other contributors
 * Released under the MIT license
 * http://jquery.org/license
 *
 * Date: 2014-01-13
 */
(function( window ) {

var i,
	support,
	Expr,
	getText,
	isXML,
	compile,
	outermostContext,
	sortInput,
	hasDuplicate,

	// Local document vars
	setDocument,
	document,
	docElem,
	documentIsHTML,
	rbuggyQSA,
	rbuggyMatches,
	matches,
	contains,

	// Instance-specific data
	expando = "sizzle" + -(new Date()),
	preferredDoc = window.document,
	dirruns = 0,
	done = 0,
	classCache = createCache(),
	tokenCache = createCache(),
	compilerCache = createCache(),
	sortOrder = function( a, b ) {
		if ( a === b ) {
			hasDuplicate = true;
		}
		return 0;
	},

	// General-purpose constants
	strundefined = typeof undefined,
	MAX_NEGATIVE = 1 << 31,

	// Instance methods
	hasOwn = ({}).hasOwnProperty,
	arr = [],
	pop = arr.pop,
	push_native = arr.push,
	push = arr.push,
	slice = arr.slice,
	// Use a stripped-down indexOf if we can't use a native one
	indexOf = arr.indexOf || function( elem ) {
		var i = 0,
			len = this.length;
		for ( ; i < len; i++ ) {
			if ( this[i] === elem ) {
				return i;
			}
		}
		return -1;
	},

	booleans = "checked|selected|async|autofocus|autoplay|controls|defer|disabled|hidden|ismap|loop|multiple|open|readonly|required|scoped",

	// Regular expressions

	// Whitespace characters http://www.w3.org/TR/css3-selectors/#whitespace
	whitespace = "[\\x20\\t\\r\\n\\f]",
	// http://www.w3.org/TR/css3-syntax/#characters
	characterEncoding = "(?:\\\\.|[\\w-]|[^\\x00-\\xa0])+",

	// Loosely modeled on CSS identifier characters
	// An unquoted value should be a CSS identifier http://www.w3.org/TR/css3-selectors/#attribute-selectors
	// Proper syntax: http://www.w3.org/TR/CSS21/syndata.html#value-def-identifier
	identifier = characterEncoding.replace( "w", "w#" ),

	// Acceptable operators http://www.w3.org/TR/selectors/#attribute-selectors
	attributes = "\\[" + whitespace + "*(" + characterEncoding + ")" + whitespace +
		"*(?:([*^$|!~]?=)" + whitespace + "*(?:(['\"])((?:\\\\.|[^\\\\])*?)\\3|(" + identifier + ")|)|)" + whitespace + "*\\]",

	// Prefer arguments quoted,
	//   then not containing pseudos/brackets,
	//   then attribute selectors/non-parenthetical expressions,
	//   then anything else
	// These preferences are here to reduce the number of selectors
	//   needing tokenize in the PSEUDO preFilter
	pseudos = ":(" + characterEncoding + ")(?:\\(((['\"])((?:\\\\.|[^\\\\])*?)\\3|((?:\\\\.|[^\\\\()[\\]]|" + attributes.replace( 3, 8 ) + ")*)|.*)\\)|)",

	// Leading and non-escaped trailing whitespace, capturing some non-whitespace characters preceding the latter
	rtrim = new RegExp( "^" + whitespace + "+|((?:^|[^\\\\])(?:\\\\.)*)" + whitespace + "+$", "g" ),

	rcomma = new RegExp( "^" + whitespace + "*," + whitespace + "*" ),
	rcombinators = new RegExp( "^" + whitespace + "*([>+~]|" + whitespace + ")" + whitespace + "*" ),

	rattributeQuotes = new RegExp( "=" + whitespace + "*([^\\]'\"]*?)" + whitespace + "*\\]", "g" ),

	rpseudo = new RegExp( pseudos ),
	ridentifier = new RegExp( "^" + identifier + "$" ),

	matchExpr = {
		"ID": new RegExp( "^#(" + characterEncoding + ")" ),
		"CLASS": new RegExp( "^\\.(" + characterEncoding + ")" ),
		"TAG": new RegExp( "^(" + characterEncoding.replace( "w", "w*" ) + ")" ),
		"ATTR": new RegExp( "^" + attributes ),
		"PSEUDO": new RegExp( "^" + pseudos ),
		"CHILD": new RegExp( "^:(only|first|last|nth|nth-last)-(child|of-type)(?:\\(" + whitespace +
			"*(even|odd|(([+-]|)(\\d*)n|)" + whitespace + "*(?:([+-]|)" + whitespace +
			"*(\\d+)|))" + whitespace + "*\\)|)", "i" ),
		"bool": new RegExp( "^(?:" + booleans + ")$", "i" ),
		// For use in libraries implementing .is()
		// We use this for POS matching in `select`
		"needsContext": new RegExp( "^" + whitespace + "*[>+~]|:(even|odd|eq|gt|lt|nth|first|last)(?:\\(" +
			whitespace + "*((?:-\\d)?\\d*)" + whitespace + "*\\)|)(?=[^-]|$)", "i" )
	},

	rinputs = /^(?:input|select|textarea|button)$/i,
	rheader = /^h\d$/i,

	rnative = /^[^{]+\{\s*\[native \w/,

	// Easily-parseable/retrievable ID or TAG or CLASS selectors
	rquickExpr = /^(?:#([\w-]+)|(\w+)|\.([\w-]+))$/,

	rsibling = /[+~]/,
	rescape = /'|\\/g,

	// CSS escapes http://www.w3.org/TR/CSS21/syndata.html#escaped-characters
	runescape = new RegExp( "\\\\([\\da-f]{1,6}" + whitespace + "?|(" + whitespace + ")|.)", "ig" ),
	funescape = function( _, escaped, escapedWhitespace ) {
		var high = "0x" + escaped - 0x10000;
		// NaN means non-codepoint
		// Support: Firefox
		// Workaround erroneous numeric interpretation of +"0x"
		return high !== high || escapedWhitespace ?
			escaped :
			high < 0 ?
				// BMP codepoint
				String.fromCharCode( high + 0x10000 ) :
				// Supplemental Plane codepoint (surrogate pair)
				String.fromCharCode( high >> 10 | 0xD800, high & 0x3FF | 0xDC00 );
	};

// Optimize for push.apply( _, NodeList )
try {
	push.apply(
		(arr = slice.call( preferredDoc.childNodes )),
		preferredDoc.childNodes
	);
	// Support: Android<4.0
	// Detect silently failing push.apply
	arr[ preferredDoc.childNodes.length ].nodeType;
} catch ( e ) {
	push = { apply: arr.length ?

		// Leverage slice if possible
		function( target, els ) {
			push_native.apply( target, slice.call(els) );
		} :

		// Support: IE<9
		// Otherwise append directly
		function( target, els ) {
			var j = target.length,
				i = 0;
			// Can't trust NodeList.length
			while ( (target[j++] = els[i++]) ) {}
			target.length = j - 1;
		}
	};
}

function Sizzle( selector, context, results, seed ) {
	var match, elem, m, nodeType,
		// QSA vars
		i, groups, old, nid, newContext, newSelector;

	if ( ( context ? context.ownerDocument || context : preferredDoc ) !== document ) {
		setDocument( context );
	}

	context = context || document;
	results = results || [];

	if ( !selector || typeof selector !== "string" ) {
		return results;
	}

	if ( (nodeType = context.nodeType) !== 1 && nodeType !== 9 ) {
		return [];
	}

	if ( documentIsHTML && !seed ) {

		// Shortcuts
		if ( (match = rquickExpr.exec( selector )) ) {
			// Speed-up: Sizzle("#ID")
			if ( (m = match[1]) ) {
				if ( nodeType === 9 ) {
					elem = context.getElementById( m );
					// Check parentNode to catch when Blackberry 4.6 returns
					// nodes that are no longer in the document (jQuery #6963)
					if ( elem && elem.parentNode ) {
						// Handle the case where IE, Opera, and Webkit return items
						// by name instead of ID
						if ( elem.id === m ) {
							results.push( elem );
							return results;
						}
					} else {
						return results;
					}
				} else {
					// Context is not a document
					if ( context.ownerDocument && (elem = context.ownerDocument.getElementById( m )) &&
						contains( context, elem ) && elem.id === m ) {
						results.push( elem );
						return results;
					}
				}

			// Speed-up: Sizzle("TAG")
			} else if ( match[2] ) {
				push.apply( results, context.getElementsByTagName( selector ) );
				return results;

			// Speed-up: Sizzle(".CLASS")
			} else if ( (m = match[3]) && support.getElementsByClassName && context.getElementsByClassName ) {
				push.apply( results, context.getElementsByClassName( m ) );
				return results;
			}
		}

		// QSA path
		if ( support.qsa && (!rbuggyQSA || !rbuggyQSA.test( selector )) ) {
			nid = old = expando;
			newContext = context;
			newSelector = nodeType === 9 && selector;

			// qSA works strangely on Element-rooted queries
			// We can work around this by specifying an extra ID on the root
			// and working up from there (Thanks to Andrew Dupont for the technique)
			// IE 8 doesn't work on object elements
			if ( nodeType === 1 && context.nodeName.toLowerCase() !== "object" ) {
				groups = tokenize( selector );

				if ( (old = context.getAttribute("id")) ) {
					nid = old.replace( rescape, "\\$&" );
				} else {
					context.setAttribute( "id", nid );
				}
				nid = "[id='" + nid + "'] ";

				i = groups.length;
				while ( i-- ) {
					groups[i] = nid + toSelector( groups[i] );
				}
				newContext = rsibling.test( selector ) && testContext( context.parentNode ) || context;
				newSelector = groups.join(",");
			}

			if ( newSelector ) {
				try {
					push.apply( results,
						newContext.querySelectorAll( newSelector )
					);
					return results;
				} catch(qsaError) {
				} finally {
					if ( !old ) {
						context.removeAttribute("id");
					}
				}
			}
		}
	}

	// All others
	return select( selector.replace( rtrim, "$1" ), context, results, seed );
}

/**
 * Create key-value caches of limited size
 * @returns {Function(string, Object)} Returns the Object data after storing it on itself with
 *	property name the (space-suffixed) string and (if the cache is larger than Expr.cacheLength)
 *	deleting the oldest entry
 */
function createCache() {
	var keys = [];

	function cache( key, value ) {
		// Use (key + " ") to avoid collision with native prototype properties (see Issue #157)
		if ( keys.push( key + " " ) > Expr.cacheLength ) {
			// Only keep the most recent entries
			delete cache[ keys.shift() ];
		}
		return (cache[ key + " " ] = value);
	}
	return cache;
}

/**
 * Mark a function for special use by Sizzle
 * @param {Function} fn The function to mark
 */
function markFunction( fn ) {
	fn[ expando ] = true;
	return fn;
}

/**
 * Support testing using an element
 * @param {Function} fn Passed the created div and expects a boolean result
 */
function assert( fn ) {
	var div = document.createElement("div");

	try {
		return !!fn( div );
	} catch (e) {
		return false;
	} finally {
		// Remove from its parent by default
		if ( div.parentNode ) {
			div.parentNode.removeChild( div );
		}
		// release memory in IE
		div = null;
	}
}

/**
 * Adds the same handler for all of the specified attrs
 * @param {String} attrs Pipe-separated list of attributes
 * @param {Function} handler The method that will be applied
 */
function addHandle( attrs, handler ) {
	var arr = attrs.split("|"),
		i = attrs.length;

	while ( i-- ) {
		Expr.attrHandle[ arr[i] ] = handler;
	}
}

/**
 * Checks document order of two siblings
 * @param {Element} a
 * @param {Element} b
 * @returns {Number} Returns less than 0 if a precedes b, greater than 0 if a follows b
 */
function siblingCheck( a, b ) {
	var cur = b && a,
		diff = cur && a.nodeType === 1 && b.nodeType === 1 &&
			( ~b.sourceIndex || MAX_NEGATIVE ) -
			( ~a.sourceIndex || MAX_NEGATIVE );

	// Use IE sourceIndex if available on both nodes
	if ( diff ) {
		return diff;
	}

	// Check if b follows a
	if ( cur ) {
		while ( (cur = cur.nextSibling) ) {
			if ( cur === b ) {
				return -1;
			}
		}
	}

	return a ? 1 : -1;
}

/**
 * Returns a function to use in pseudos for input types
 * @param {String} type
 */
function createInputPseudo( type ) {
	return function( elem ) {
		var name = elem.nodeName.toLowerCase();
		return name === "input" && elem.type === type;
	};
}

/**
 * Returns a function to use in pseudos for buttons
 * @param {String} type
 */
function createButtonPseudo( type ) {
	return function( elem ) {
		var name = elem.nodeName.toLowerCase();
		return (name === "input" || name === "button") && elem.type === type;
	};
}

/**
 * Returns a function to use in pseudos for positionals
 * @param {Function} fn
 */
function createPositionalPseudo( fn ) {
	return markFunction(function( argument ) {
		argument = +argument;
		return markFunction(function( seed, matches ) {
			var j,
				matchIndexes = fn( [], seed.length, argument ),
				i = matchIndexes.length;

			// Match elements found at the specified indexes
			while ( i-- ) {
				if ( seed[ (j = matchIndexes[i]) ] ) {
					seed[j] = !(matches[j] = seed[j]);
				}
			}
		});
	});
}

/**
 * Checks a node for validity as a Sizzle context
 * @param {Element|Object=} context
 * @returns {Element|Object|Boolean} The input node if acceptable, otherwise a falsy value
 */
function testContext( context ) {
	return context && typeof context.getElementsByTagName !== strundefined && context;
}

// Expose support vars for convenience
support = Sizzle.support = {};

/**
 * Detects XML nodes
 * @param {Element|Object} elem An element or a document
 * @returns {Boolean} True iff elem is a non-HTML XML node
 */
isXML = Sizzle.isXML = function( elem ) {
	// documentElement is verified for cases where it doesn't yet exist
	// (such as loading iframes in IE - #4833)
	var documentElement = elem && (elem.ownerDocument || elem).documentElement;
	return documentElement ? documentElement.nodeName !== "HTML" : false;
};

/**
 * Sets document-related variables once based on the current document
 * @param {Element|Object} [doc] An element or document object to use to set the document
 * @returns {Object} Returns the current document
 */
setDocument = Sizzle.setDocument = function( node ) {
	var hasCompare,
		doc = node ? node.ownerDocument || node : preferredDoc,
		parent = doc.defaultView;

	// If no document and documentElement is available, return
	if ( doc === document || doc.nodeType !== 9 || !doc.documentElement ) {
		return document;
	}

	// Set our document
	document = doc;
	docElem = doc.documentElement;

	// Support tests
	documentIsHTML = !isXML( doc );

	// Support: IE>8
	// If iframe document is assigned to "document" variable and if iframe has been reloaded,
	// IE will throw "permission denied" error when accessing "document" variable, see jQuery #13936
	// IE6-8 do not support the defaultView property so parent will be undefined
	if ( parent && parent !== parent.top ) {
		// IE11 does not have attachEvent, so all must suffer
		if ( parent.addEventListener ) {
			parent.addEventListener( "unload", function() {
				setDocument();
			}, false );
		} else if ( parent.attachEvent ) {
			parent.attachEvent( "onunload", function() {
				setDocument();
			});
		}
	}

	/* Attributes
	---------------------------------------------------------------------- */

	// Support: IE<8
	// Verify that getAttribute really returns attributes and not properties (excepting IE8 booleans)
	support.attributes = assert(function( div ) {
		div.className = "i";
		return !div.getAttribute("className");
	});

	/* getElement(s)By*
	---------------------------------------------------------------------- */

	// Check if getElementsByTagName("*") returns only elements
	support.getElementsByTagName = assert(function( div ) {
		div.appendChild( doc.createComment("") );
		return !div.getElementsByTagName("*").length;
	});

	// Check if getElementsByClassName can be trusted
	support.getElementsByClassName = rnative.test( doc.getElementsByClassName ) && assert(function( div ) {
		div.innerHTML = "<div class='a'></div><div class='a i'></div>";

		// Support: Safari<4
		// Catch class over-caching
		div.firstChild.className = "i";
		// Support: Opera<10
		// Catch gEBCN failure to find non-leading classes
		return div.getElementsByClassName("i").length === 2;
	});

	// Support: IE<10
	// Check if getElementById returns elements by name
	// The broken getElementById methods don't pick up programatically-set names,
	// so use a roundabout getElementsByName test
	support.getById = assert(function( div ) {
		docElem.appendChild( div ).id = expando;
		return !doc.getElementsByName || !doc.getElementsByName( expando ).length;
	});

	// ID find and filter
	if ( support.getById ) {
		Expr.find["ID"] = function( id, context ) {
			if ( typeof context.getElementById !== strundefined && documentIsHTML ) {
				var m = context.getElementById( id );
				// Check parentNode to catch when Blackberry 4.6 returns
				// nodes that are no longer in the document #6963
				return m && m.parentNode ? [m] : [];
			}
		};
		Expr.filter["ID"] = function( id ) {
			var attrId = id.replace( runescape, funescape );
			return function( elem ) {
				return elem.getAttribute("id") === attrId;
			};
		};
	} else {
		// Support: IE6/7
		// getElementById is not reliable as a find shortcut
		delete Expr.find["ID"];

		Expr.filter["ID"] =  function( id ) {
			var attrId = id.replace( runescape, funescape );
			return function( elem ) {
				var node = typeof elem.getAttributeNode !== strundefined && elem.getAttributeNode("id");
				return node && node.value === attrId;
			};
		};
	}

	// Tag
	Expr.find["TAG"] = support.getElementsByTagName ?
		function( tag, context ) {
			if ( typeof context.getElementsByTagName !== strundefined ) {
				return context.getElementsByTagName( tag );
			}
		} :
		function( tag, context ) {
			var elem,
				tmp = [],
				i = 0,
				results = context.getElementsByTagName( tag );

			// Filter out possible comments
			if ( tag === "*" ) {
				while ( (elem = results[i++]) ) {
					if ( elem.nodeType === 1 ) {
						tmp.push( elem );
					}
				}

				return tmp;
			}
			return results;
		};

	// Class
	Expr.find["CLASS"] = support.getElementsByClassName && function( className, context ) {
		if ( typeof context.getElementsByClassName !== strundefined && documentIsHTML ) {
			return context.getElementsByClassName( className );
		}
	};

	/* QSA/matchesSelector
	---------------------------------------------------------------------- */

	// QSA and matchesSelector support

	// matchesSelector(:active) reports false when true (IE9/Opera 11.5)
	rbuggyMatches = [];

	// qSa(:focus) reports false when true (Chrome 21)
	// We allow this because of a bug in IE8/9 that throws an error
	// whenever `document.activeElement` is accessed on an iframe
	// So, we allow :focus to pass through QSA all the time to avoid the IE error
	// See http://bugs.jquery.com/ticket/13378
	rbuggyQSA = [];

	if ( (support.qsa = rnative.test( doc.querySelectorAll )) ) {
		// Build QSA regex
		// Regex strategy adopted from Diego Perini
		assert(function( div ) {
			// Select is set to empty string on purpose
			// This is to test IE's treatment of not explicitly
			// setting a boolean content attribute,
			// since its presence should be enough
			// http://bugs.jquery.com/ticket/12359
			div.innerHTML = "<select t=''><option selected=''></option></select>";

			// Support: IE8, Opera 10-12
			// Nothing should be selected when empty strings follow ^= or $= or *=
			if ( div.querySelectorAll("[t^='']").length ) {
				rbuggyQSA.push( "[*^$]=" + whitespace + "*(?:''|\"\")" );
			}

			// Support: IE8
			// Boolean attributes and "value" are not treated correctly
			if ( !div.querySelectorAll("[selected]").length ) {
				rbuggyQSA.push( "\\[" + whitespace + "*(?:value|" + booleans + ")" );
			}

			// Webkit/Opera - :checked should return selected option elements
			// http://www.w3.org/TR/2011/REC-css3-selectors-20110929/#checked
			// IE8 throws error here and will not see later tests
			if ( !div.querySelectorAll(":checked").length ) {
				rbuggyQSA.push(":checked");
			}
		});

		assert(function( div ) {
			// Support: Windows 8 Native Apps
			// The type and name attributes are restricted during .innerHTML assignment
			var input = doc.createElement("input");
			input.setAttribute( "type", "hidden" );
			div.appendChild( input ).setAttribute( "name", "D" );

			// Support: IE8
			// Enforce case-sensitivity of name attribute
			if ( div.querySelectorAll("[name=d]").length ) {
				rbuggyQSA.push( "name" + whitespace + "*[*^$|!~]?=" );
			}

			// FF 3.5 - :enabled/:disabled and hidden elements (hidden elements are still enabled)
			// IE8 throws error here and will not see later tests
			if ( !div.querySelectorAll(":enabled").length ) {
				rbuggyQSA.push( ":enabled", ":disabled" );
			}

			// Opera 10-11 does not throw on post-comma invalid pseudos
			div.querySelectorAll("*,:x");
			rbuggyQSA.push(",.*:");
		});
	}

	if ( (support.matchesSelector = rnative.test( (matches = docElem.webkitMatchesSelector ||
		docElem.mozMatchesSelector ||
		docElem.oMatchesSelector ||
		docElem.msMatchesSelector) )) ) {

		assert(function( div ) {
			// Check to see if it's possible to do matchesSelector
			// on a disconnected node (IE 9)
			support.disconnectedMatch = matches.call( div, "div" );

			// This should fail with an exception
			// Gecko does not error, returns false instead
			matches.call( div, "[s!='']:x" );
			rbuggyMatches.push( "!=", pseudos );
		});
	}

	rbuggyQSA = rbuggyQSA.length && new RegExp( rbuggyQSA.join("|") );
	rbuggyMatches = rbuggyMatches.length && new RegExp( rbuggyMatches.join("|") );

	/* Contains
	---------------------------------------------------------------------- */
	hasCompare = rnative.test( docElem.compareDocumentPosition );

	// Element contains another
	// Purposefully does not implement inclusive descendent
	// As in, an element does not contain itself
	contains = hasCompare || rnative.test( docElem.contains ) ?
		function( a, b ) {
			var adown = a.nodeType === 9 ? a.documentElement : a,
				bup = b && b.parentNode;
			return a === bup || !!( bup && bup.nodeType === 1 && (
				adown.contains ?
					adown.contains( bup ) :
					a.compareDocumentPosition && a.compareDocumentPosition( bup ) & 16
			));
		} :
		function( a, b ) {
			if ( b ) {
				while ( (b = b.parentNode) ) {
					if ( b === a ) {
						return true;
					}
				}
			}
			return false;
		};

	/* Sorting
	---------------------------------------------------------------------- */

	// Document order sorting
	sortOrder = hasCompare ?
	function( a, b ) {

		// Flag for duplicate removal
		if ( a === b ) {
			hasDuplicate = true;
			return 0;
		}

		// Sort on method existence if only one input has compareDocumentPosition
		var compare = !a.compareDocumentPosition - !b.compareDocumentPosition;
		if ( compare ) {
			return compare;
		}

		// Calculate position if both inputs belong to the same document
		compare = ( a.ownerDocument || a ) === ( b.ownerDocument || b ) ?
			a.compareDocumentPosition( b ) :

			// Otherwise we know they are disconnected
			1;

		// Disconnected nodes
		if ( compare & 1 ||
			(!support.sortDetached && b.compareDocumentPosition( a ) === compare) ) {

			// Choose the first element that is related to our preferred document
			if ( a === doc || a.ownerDocument === preferredDoc && contains(preferredDoc, a) ) {
				return -1;
			}
			if ( b === doc || b.ownerDocument === preferredDoc && contains(preferredDoc, b) ) {
				return 1;
			}

			// Maintain original order
			return sortInput ?
				( indexOf.call( sortInput, a ) - indexOf.call( sortInput, b ) ) :
				0;
		}

		return compare & 4 ? -1 : 1;
	} :
	function( a, b ) {
		// Exit early if the nodes are identical
		if ( a === b ) {
			hasDuplicate = true;
			return 0;
		}

		var cur,
			i = 0,
			aup = a.parentNode,
			bup = b.parentNode,
			ap = [ a ],
			bp = [ b ];

		// Parentless nodes are either documents or disconnected
		if ( !aup || !bup ) {
			return a === doc ? -1 :
				b === doc ? 1 :
				aup ? -1 :
				bup ? 1 :
				sortInput ?
				( indexOf.call( sortInput, a ) - indexOf.call( sortInput, b ) ) :
				0;

		// If the nodes are siblings, we can do a quick check
		} else if ( aup === bup ) {
			return siblingCheck( a, b );
		}

		// Otherwise we need full lists of their ancestors for comparison
		cur = a;
		while ( (cur = cur.parentNode) ) {
			ap.unshift( cur );
		}
		cur = b;
		while ( (cur = cur.parentNode) ) {
			bp.unshift( cur );
		}

		// Walk down the tree looking for a discrepancy
		while ( ap[i] === bp[i] ) {
			i++;
		}

		return i ?
			// Do a sibling check if the nodes have a common ancestor
			siblingCheck( ap[i], bp[i] ) :

			// Otherwise nodes in our document sort first
			ap[i] === preferredDoc ? -1 :
			bp[i] === preferredDoc ? 1 :
			0;
	};

	return doc;
};

Sizzle.matches = function( expr, elements ) {
	return Sizzle( expr, null, null, elements );
};

Sizzle.matchesSelector = function( elem, expr ) {
	// Set document vars if needed
	if ( ( elem.ownerDocument || elem ) !== document ) {
		setDocument( elem );
	}

	// Make sure that attribute selectors are quoted
	expr = expr.replace( rattributeQuotes, "='$1']" );

	if ( support.matchesSelector && documentIsHTML &&
		( !rbuggyMatches || !rbuggyMatches.test( expr ) ) &&
		( !rbuggyQSA     || !rbuggyQSA.test( expr ) ) ) {

		try {
			var ret = matches.call( elem, expr );

			// IE 9's matchesSelector returns false on disconnected nodes
			if ( ret || support.disconnectedMatch ||
					// As well, disconnected nodes are said to be in a document
					// fragment in IE 9
					elem.document && elem.document.nodeType !== 11 ) {
				return ret;
			}
		} catch(e) {}
	}

	return Sizzle( expr, document, null, [elem] ).length > 0;
};

Sizzle.contains = function( context, elem ) {
	// Set document vars if needed
	if ( ( context.ownerDocument || context ) !== document ) {
		setDocument( context );
	}
	return contains( context, elem );
};

Sizzle.attr = function( elem, name ) {
	// Set document vars if needed
	if ( ( elem.ownerDocument || elem ) !== document ) {
		setDocument( elem );
	}

	var fn = Expr.attrHandle[ name.toLowerCase() ],
		// Don't get fooled by Object.prototype properties (jQuery #13807)
		val = fn && hasOwn.call( Expr.attrHandle, name.toLowerCase() ) ?
			fn( elem, name, !documentIsHTML ) :
			undefined;

	return val !== undefined ?
		val :
		support.attributes || !documentIsHTML ?
			elem.getAttribute( name ) :
			(val = elem.getAttributeNode(name)) && val.specified ?
				val.value :
				null;
};

Sizzle.error = function( msg ) {
	throw new Error( "Syntax error, unrecognized expression: " + msg );
};

/**
 * Document sorting and removing duplicates
 * @param {ArrayLike} results
 */
Sizzle.uniqueSort = function( results ) {
	var elem,
		duplicates = [],
		j = 0,
		i = 0;

	// Unless we *know* we can detect duplicates, assume their presence
	hasDuplicate = !support.detectDuplicates;
	sortInput = !support.sortStable && results.slice( 0 );
	results.sort( sortOrder );

	if ( hasDuplicate ) {
		while ( (elem = results[i++]) ) {
			if ( elem === results[ i ] ) {
				j = duplicates.push( i );
			}
		}
		while ( j-- ) {
			results.splice( duplicates[ j ], 1 );
		}
	}

	// Clear input after sorting to release objects
	// See https://github.com/jquery/sizzle/pull/225
	sortInput = null;

	return results;
};

/**
 * Utility function for retrieving the text value of an array of DOM nodes
 * @param {Array|Element} elem
 */
getText = Sizzle.getText = function( elem ) {
	var node,
		ret = "",
		i = 0,
		nodeType = elem.nodeType;

	if ( !nodeType ) {
		// If no nodeType, this is expected to be an array
		while ( (node = elem[i++]) ) {
			// Do not traverse comment nodes
			ret += getText( node );
		}
	} else if ( nodeType === 1 || nodeType === 9 || nodeType === 11 ) {
		// Use textContent for elements
		// innerText usage removed for consistency of new lines (jQuery #11153)
		if ( typeof elem.textContent === "string" ) {
			return elem.textContent;
		} else {
			// Traverse its children
			for ( elem = elem.firstChild; elem; elem = elem.nextSibling ) {
				ret += getText( elem );
			}
		}
	} else if ( nodeType === 3 || nodeType === 4 ) {
		return elem.nodeValue;
	}
	// Do not include comment or processing instruction nodes

	return ret;
};

Expr = Sizzle.selectors = {

	// Can be adjusted by the user
	cacheLength: 50,

	createPseudo: markFunction,

	match: matchExpr,

	attrHandle: {},

	find: {},

	relative: {
		">": { dir: "parentNode", first: true },
		" ": { dir: "parentNode" },
		"+": { dir: "previousSibling", first: true },
		"~": { dir: "previousSibling" }
	},

	preFilter: {
		"ATTR": function( match ) {
			match[1] = match[1].replace( runescape, funescape );

			// Move the given value to match[3] whether quoted or unquoted
			match[3] = ( match[4] || match[5] || "" ).replace( runescape, funescape );

			if ( match[2] === "~=" ) {
				match[3] = " " + match[3] + " ";
			}

			return match.slice( 0, 4 );
		},

		"CHILD": function( match ) {
			/* matches from matchExpr["CHILD"]
				1 type (only|nth|...)
				2 what (child|of-type)
				3 argument (even|odd|\d*|\d*n([+-]\d+)?|...)
				4 xn-component of xn+y argument ([+-]?\d*n|)
				5 sign of xn-component
				6 x of xn-component
				7 sign of y-component
				8 y of y-component
			*/
			match[1] = match[1].toLowerCase();

			if ( match[1].slice( 0, 3 ) === "nth" ) {
				// nth-* requires argument
				if ( !match[3] ) {
					Sizzle.error( match[0] );
				}

				// numeric x and y parameters for Expr.filter.CHILD
				// remember that false/true cast respectively to 0/1
				match[4] = +( match[4] ? match[5] + (match[6] || 1) : 2 * ( match[3] === "even" || match[3] === "odd" ) );
				match[5] = +( ( match[7] + match[8] ) || match[3] === "odd" );

			// other types prohibit arguments
			} else if ( match[3] ) {
				Sizzle.error( match[0] );
			}

			return match;
		},

		"PSEUDO": function( match ) {
			var excess,
				unquoted = !match[5] && match[2];

			if ( matchExpr["CHILD"].test( match[0] ) ) {
				return null;
			}

			// Accept quoted arguments as-is
			if ( match[3] && match[4] !== undefined ) {
				match[2] = match[4];

			// Strip excess characters from unquoted arguments
			} else if ( unquoted && rpseudo.test( unquoted ) &&
				// Get excess from tokenize (recursively)
				(excess = tokenize( unquoted, true )) &&
				// advance to the next closing parenthesis
				(excess = unquoted.indexOf( ")", unquoted.length - excess ) - unquoted.length) ) {

				// excess is a negative index
				match[0] = match[0].slice( 0, excess );
				match[2] = unquoted.slice( 0, excess );
			}

			// Return only captures needed by the pseudo filter method (type and argument)
			return match.slice( 0, 3 );
		}
	},

	filter: {

		"TAG": function( nodeNameSelector ) {
			var nodeName = nodeNameSelector.replace( runescape, funescape ).toLowerCase();
			return nodeNameSelector === "*" ?
				function() { return true; } :
				function( elem ) {
					return elem.nodeName && elem.nodeName.toLowerCase() === nodeName;
				};
		},

		"CLASS": function( className ) {
			var pattern = classCache[ className + " " ];

			return pattern ||
				(pattern = new RegExp( "(^|" + whitespace + ")" + className + "(" + whitespace + "|$)" )) &&
				classCache( className, function( elem ) {
					return pattern.test( typeof elem.className === "string" && elem.className || typeof elem.getAttribute !== strundefined && elem.getAttribute("class") || "" );
				});
		},

		"ATTR": function( name, operator, check ) {
			return function( elem ) {
				var result = Sizzle.attr( elem, name );

				if ( result == null ) {
					return operator === "!=";
				}
				if ( !operator ) {
					return true;
				}

				result += "";

				return operator === "=" ? result === check :
					operator === "!=" ? result !== check :
					operator === "^=" ? check && result.indexOf( check ) === 0 :
					operator === "*=" ? check && result.indexOf( check ) > -1 :
					operator === "$=" ? check && result.slice( -check.length ) === check :
					operator === "~=" ? ( " " + result + " " ).indexOf( check ) > -1 :
					operator === "|=" ? result === check || result.slice( 0, check.length + 1 ) === check + "-" :
					false;
			};
		},

		"CHILD": function( type, what, argument, first, last ) {
			var simple = type.slice( 0, 3 ) !== "nth",
				forward = type.slice( -4 ) !== "last",
				ofType = what === "of-type";

			return first === 1 && last === 0 ?

				// Shortcut for :nth-*(n)
				function( elem ) {
					return !!elem.parentNode;
				} :

				function( elem, context, xml ) {
					var cache, outerCache, node, diff, nodeIndex, start,
						dir = simple !== forward ? "nextSibling" : "previousSibling",
						parent = elem.parentNode,
						name = ofType && elem.nodeName.toLowerCase(),
						useCache = !xml && !ofType;

					if ( parent ) {

						// :(first|last|only)-(child|of-type)
						if ( simple ) {
							while ( dir ) {
								node = elem;
								while ( (node = node[ dir ]) ) {
									if ( ofType ? node.nodeName.toLowerCase() === name : node.nodeType === 1 ) {
										return false;
									}
								}
								// Reverse direction for :only-* (if we haven't yet done so)
								start = dir = type === "only" && !start && "nextSibling";
							}
							return true;
						}

						start = [ forward ? parent.firstChild : parent.lastChild ];

						// non-xml :nth-child(...) stores cache data on `parent`
						if ( forward && useCache ) {
							// Seek `elem` from a previously-cached index
							outerCache = parent[ expando ] || (parent[ expando ] = {});
							cache = outerCache[ type ] || [];
							nodeIndex = cache[0] === dirruns && cache[1];
							diff = cache[0] === dirruns && cache[2];
							node = nodeIndex && parent.childNodes[ nodeIndex ];

							while ( (node = ++nodeIndex && node && node[ dir ] ||

								// Fallback to seeking `elem` from the start
								(diff = nodeIndex = 0) || start.pop()) ) {

								// When found, cache indexes on `parent` and break
								if ( node.nodeType === 1 && ++diff && node === elem ) {
									outerCache[ type ] = [ dirruns, nodeIndex, diff ];
									break;
								}
							}

						// Use previously-cached element index if available
						} else if ( useCache && (cache = (elem[ expando ] || (elem[ expando ] = {}))[ type ]) && cache[0] === dirruns ) {
							diff = cache[1];

						// xml :nth-child(...) or :nth-last-child(...) or :nth(-last)?-of-type(...)
						} else {
							// Use the same loop as above to seek `elem` from the start
							while ( (node = ++nodeIndex && node && node[ dir ] ||
								(diff = nodeIndex = 0) || start.pop()) ) {

								if ( ( ofType ? node.nodeName.toLowerCase() === name : node.nodeType === 1 ) && ++diff ) {
									// Cache the index of each encountered element
									if ( useCache ) {
										(node[ expando ] || (node[ expando ] = {}))[ type ] = [ dirruns, diff ];
									}

									if ( node === elem ) {
										break;
									}
								}
							}
						}

						// Incorporate the offset, then check against cycle size
						diff -= last;
						return diff === first || ( diff % first === 0 && diff / first >= 0 );
					}
				};
		},

		"PSEUDO": function( pseudo, argument ) {
			// pseudo-class names are case-insensitive
			// http://www.w3.org/TR/selectors/#pseudo-classes
			// Prioritize by case sensitivity in case custom pseudos are added with uppercase letters
			// Remember that setFilters inherits from pseudos
			var args,
				fn = Expr.pseudos[ pseudo ] || Expr.setFilters[ pseudo.toLowerCase() ] ||
					Sizzle.error( "unsupported pseudo: " + pseudo );

			// The user may use createPseudo to indicate that
			// arguments are needed to create the filter function
			// just as Sizzle does
			if ( fn[ expando ] ) {
				return fn( argument );
			}

			// But maintain support for old signatures
			if ( fn.length > 1 ) {
				args = [ pseudo, pseudo, "", argument ];
				return Expr.setFilters.hasOwnProperty( pseudo.toLowerCase() ) ?
					markFunction(function( seed, matches ) {
						var idx,
							matched = fn( seed, argument ),
							i = matched.length;
						while ( i-- ) {
							idx = indexOf.call( seed, matched[i] );
							seed[ idx ] = !( matches[ idx ] = matched[i] );
						}
					}) :
					function( elem ) {
						return fn( elem, 0, args );
					};
			}

			return fn;
		}
	},

	pseudos: {
		// Potentially complex pseudos
		"not": markFunction(function( selector ) {
			// Trim the selector passed to compile
			// to avoid treating leading and trailing
			// spaces as combinators
			var input = [],
				results = [],
				matcher = compile( selector.replace( rtrim, "$1" ) );

			return matcher[ expando ] ?
				markFunction(function( seed, matches, context, xml ) {
					var elem,
						unmatched = matcher( seed, null, xml, [] ),
						i = seed.length;

					// Match elements unmatched by `matcher`
					while ( i-- ) {
						if ( (elem = unmatched[i]) ) {
							seed[i] = !(matches[i] = elem);
						}
					}
				}) :
				function( elem, context, xml ) {
					input[0] = elem;
					matcher( input, null, xml, results );
					return !results.pop();
				};
		}),

		"has": markFunction(function( selector ) {
			return function( elem ) {
				return Sizzle( selector, elem ).length > 0;
			};
		}),

		"contains": markFunction(function( text ) {
			return function( elem ) {
				return ( elem.textContent || elem.innerText || getText( elem ) ).indexOf( text ) > -1;
			};
		}),

		// "Whether an element is represented by a :lang() selector
		// is based solely on the element's language value
		// being equal to the identifier C,
		// or beginning with the identifier C immediately followed by "-".
		// The matching of C against the element's language value is performed case-insensitively.
		// The identifier C does not have to be a valid language name."
		// http://www.w3.org/TR/selectors/#lang-pseudo
		"lang": markFunction( function( lang ) {
			// lang value must be a valid identifier
			if ( !ridentifier.test(lang || "") ) {
				Sizzle.error( "unsupported lang: " + lang );
			}
			lang = lang.replace( runescape, funescape ).toLowerCase();
			return function( elem ) {
				var elemLang;
				do {
					if ( (elemLang = documentIsHTML ?
						elem.lang :
						elem.getAttribute("xml:lang") || elem.getAttribute("lang")) ) {

						elemLang = elemLang.toLowerCase();
						return elemLang === lang || elemLang.indexOf( lang + "-" ) === 0;
					}
				} while ( (elem = elem.parentNode) && elem.nodeType === 1 );
				return false;
			};
		}),

		// Miscellaneous
		"target": function( elem ) {
			var hash = window.location && window.location.hash;
			return hash && hash.slice( 1 ) === elem.id;
		},

		"root": function( elem ) {
			return elem === docElem;
		},

		"focus": function( elem ) {
			return elem === document.activeElement && (!document.hasFocus || document.hasFocus()) && !!(elem.type || elem.href || ~elem.tabIndex);
		},

		// Boolean properties
		"enabled": function( elem ) {
			return elem.disabled === false;
		},

		"disabled": function( elem ) {
			return elem.disabled === true;
		},

		"checked": function( elem ) {
			// In CSS3, :checked should return both checked and selected elements
			// http://www.w3.org/TR/2011/REC-css3-selectors-20110929/#checked
			var nodeName = elem.nodeName.toLowerCase();
			return (nodeName === "input" && !!elem.checked) || (nodeName === "option" && !!elem.selected);
		},

		"selected": function( elem ) {
			// Accessing this property makes selected-by-default
			// options in Safari work properly
			if ( elem.parentNode ) {
				elem.parentNode.selectedIndex;
			}

			return elem.selected === true;
		},

		// Contents
		"empty": function( elem ) {
			// http://www.w3.org/TR/selectors/#empty-pseudo
			// :empty is negated by element (1) or content nodes (text: 3; cdata: 4; entity ref: 5),
			//   but not by others (comment: 8; processing instruction: 7; etc.)
			// nodeType < 6 works because attributes (2) do not appear as children
			for ( elem = elem.firstChild; elem; elem = elem.nextSibling ) {
				if ( elem.nodeType < 6 ) {
					return false;
				}
			}
			return true;
		},

		"parent": function( elem ) {
			return !Expr.pseudos["empty"]( elem );
		},

		// Element/input types
		"header": function( elem ) {
			return rheader.test( elem.nodeName );
		},

		"input": function( elem ) {
			return rinputs.test( elem.nodeName );
		},

		"button": function( elem ) {
			var name = elem.nodeName.toLowerCase();
			return name === "input" && elem.type === "button" || name === "button";
		},

		"text": function( elem ) {
			var attr;
			return elem.nodeName.toLowerCase() === "input" &&
				elem.type === "text" &&

				// Support: IE<8
				// New HTML5 attribute values (e.g., "search") appear with elem.type === "text"
				( (attr = elem.getAttribute("type")) == null || attr.toLowerCase() === "text" );
		},

		// Position-in-collection
		"first": createPositionalPseudo(function() {
			return [ 0 ];
		}),

		"last": createPositionalPseudo(function( matchIndexes, length ) {
			return [ length - 1 ];
		}),

		"eq": createPositionalPseudo(function( matchIndexes, length, argument ) {
			return [ argument < 0 ? argument + length : argument ];
		}),

		"even": createPositionalPseudo(function( matchIndexes, length ) {
			var i = 0;
			for ( ; i < length; i += 2 ) {
				matchIndexes.push( i );
			}
			return matchIndexes;
		}),

		"odd": createPositionalPseudo(function( matchIndexes, length ) {
			var i = 1;
			for ( ; i < length; i += 2 ) {
				matchIndexes.push( i );
			}
			return matchIndexes;
		}),

		"lt": createPositionalPseudo(function( matchIndexes, length, argument ) {
			var i = argument < 0 ? argument + length : argument;
			for ( ; --i >= 0; ) {
				matchIndexes.push( i );
			}
			return matchIndexes;
		}),

		"gt": createPositionalPseudo(function( matchIndexes, length, argument ) {
			var i = argument < 0 ? argument + length : argument;
			for ( ; ++i < length; ) {
				matchIndexes.push( i );
			}
			return matchIndexes;
		})
	}
};

Expr.pseudos["nth"] = Expr.pseudos["eq"];

// Add button/input type pseudos
for ( i in { radio: true, checkbox: true, file: true, password: true, image: true } ) {
	Expr.pseudos[ i ] = createInputPseudo( i );
}
for ( i in { submit: true, reset: true } ) {
	Expr.pseudos[ i ] = createButtonPseudo( i );
}

// Easy API for creating new setFilters
function setFilters() {}
setFilters.prototype = Expr.filters = Expr.pseudos;
Expr.setFilters = new setFilters();

function tokenize( selector, parseOnly ) {
	var matched, match, tokens, type,
		soFar, groups, preFilters,
		cached = tokenCache[ selector + " " ];

	if ( cached ) {
		return parseOnly ? 0 : cached.slice( 0 );
	}

	soFar = selector;
	groups = [];
	preFilters = Expr.preFilter;

	while ( soFar ) {

		// Comma and first run
		if ( !matched || (match = rcomma.exec( soFar )) ) {
			if ( match ) {
				// Don't consume trailing commas as valid
				soFar = soFar.slice( match[0].length ) || soFar;
			}
			groups.push( (tokens = []) );
		}

		matched = false;

		// Combinators
		if ( (match = rcombinators.exec( soFar )) ) {
			matched = match.shift();
			tokens.push({
				value: matched,
				// Cast descendant combinators to space
				type: match[0].replace( rtrim, " " )
			});
			soFar = soFar.slice( matched.length );
		}

		// Filters
		for ( type in Expr.filter ) {
			if ( (match = matchExpr[ type ].exec( soFar )) && (!preFilters[ type ] ||
				(match = preFilters[ type ]( match ))) ) {
				matched = match.shift();
				tokens.push({
					value: matched,
					type: type,
					matches: match
				});
				soFar = soFar.slice( matched.length );
			}
		}

		if ( !matched ) {
			break;
		}
	}

	// Return the length of the invalid excess
	// if we're just parsing
	// Otherwise, throw an error or return tokens
	return parseOnly ?
		soFar.length :
		soFar ?
			Sizzle.error( selector ) :
			// Cache the tokens
			tokenCache( selector, groups ).slice( 0 );
}

function toSelector( tokens ) {
	var i = 0,
		len = tokens.length,
		selector = "";
	for ( ; i < len; i++ ) {
		selector += tokens[i].value;
	}
	return selector;
}

function addCombinator( matcher, combinator, base ) {
	var dir = combinator.dir,
		checkNonElements = base && dir === "parentNode",
		doneName = done++;

	return combinator.first ?
		// Check against closest ancestor/preceding element
		function( elem, context, xml ) {
			while ( (elem = elem[ dir ]) ) {
				if ( elem.nodeType === 1 || checkNonElements ) {
					return matcher( elem, context, xml );
				}
			}
		} :

		// Check against all ancestor/preceding elements
		function( elem, context, xml ) {
			var oldCache, outerCache,
				newCache = [ dirruns, doneName ];

			// We can't set arbitrary data on XML nodes, so they don't benefit from dir caching
			if ( xml ) {
				while ( (elem = elem[ dir ]) ) {
					if ( elem.nodeType === 1 || checkNonElements ) {
						if ( matcher( elem, context, xml ) ) {
							return true;
						}
					}
				}
			} else {
				while ( (elem = elem[ dir ]) ) {
					if ( elem.nodeType === 1 || checkNonElements ) {
						outerCache = elem[ expando ] || (elem[ expando ] = {});
						if ( (oldCache = outerCache[ dir ]) &&
							oldCache[ 0 ] === dirruns && oldCache[ 1 ] === doneName ) {

							// Assign to newCache so results back-propagate to previous elements
							return (newCache[ 2 ] = oldCache[ 2 ]);
						} else {
							// Reuse newcache so results back-propagate to previous elements
							outerCache[ dir ] = newCache;

							// A match means we're done; a fail means we have to keep checking
							if ( (newCache[ 2 ] = matcher( elem, context, xml )) ) {
								return true;
							}
						}
					}
				}
			}
		};
}

function elementMatcher( matchers ) {
	return matchers.length > 1 ?
		function( elem, context, xml ) {
			var i = matchers.length;
			while ( i-- ) {
				if ( !matchers[i]( elem, context, xml ) ) {
					return false;
				}
			}
			return true;
		} :
		matchers[0];
}

function condense( unmatched, map, filter, context, xml ) {
	var elem,
		newUnmatched = [],
		i = 0,
		len = unmatched.length,
		mapped = map != null;

	for ( ; i < len; i++ ) {
		if ( (elem = unmatched[i]) ) {
			if ( !filter || filter( elem, context, xml ) ) {
				newUnmatched.push( elem );
				if ( mapped ) {
					map.push( i );
				}
			}
		}
	}

	return newUnmatched;
}

function setMatcher( preFilter, selector, matcher, postFilter, postFinder, postSelector ) {
	if ( postFilter && !postFilter[ expando ] ) {
		postFilter = setMatcher( postFilter );
	}
	if ( postFinder && !postFinder[ expando ] ) {
		postFinder = setMatcher( postFinder, postSelector );
	}
	return markFunction(function( seed, results, context, xml ) {
		var temp, i, elem,
			preMap = [],
			postMap = [],
			preexisting = results.length,

			// Get initial elements from seed or context
			elems = seed || multipleContexts( selector || "*", context.nodeType ? [ context ] : context, [] ),

			// Prefilter to get matcher input, preserving a map for seed-results synchronization
			matcherIn = preFilter && ( seed || !selector ) ?
				condense( elems, preMap, preFilter, context, xml ) :
				elems,

			matcherOut = matcher ?
				// If we have a postFinder, or filtered seed, or non-seed postFilter or preexisting results,
				postFinder || ( seed ? preFilter : preexisting || postFilter ) ?

					// ...intermediate processing is necessary
					[] :

					// ...otherwise use results directly
					results :
				matcherIn;

		// Find primary matches
		if ( matcher ) {
			matcher( matcherIn, matcherOut, context, xml );
		}

		// Apply postFilter
		if ( postFilter ) {
			temp = condense( matcherOut, postMap );
			postFilter( temp, [], context, xml );

			// Un-match failing elements by moving them back to matcherIn
			i = temp.length;
			while ( i-- ) {
				if ( (elem = temp[i]) ) {
					matcherOut[ postMap[i] ] = !(matcherIn[ postMap[i] ] = elem);
				}
			}
		}

		if ( seed ) {
			if ( postFinder || preFilter ) {
				if ( postFinder ) {
					// Get the final matcherOut by condensing this intermediate into postFinder contexts
					temp = [];
					i = matcherOut.length;
					while ( i-- ) {
						if ( (elem = matcherOut[i]) ) {
							// Restore matcherIn since elem is not yet a final match
							temp.push( (matcherIn[i] = elem) );
						}
					}
					postFinder( null, (matcherOut = []), temp, xml );
				}

				// Move matched elements from seed to results to keep them synchronized
				i = matcherOut.length;
				while ( i-- ) {
					if ( (elem = matcherOut[i]) &&
						(temp = postFinder ? indexOf.call( seed, elem ) : preMap[i]) > -1 ) {

						seed[temp] = !(results[temp] = elem);
					}
				}
			}

		// Add elements to results, through postFinder if defined
		} else {
			matcherOut = condense(
				matcherOut === results ?
					matcherOut.splice( preexisting, matcherOut.length ) :
					matcherOut
			);
			if ( postFinder ) {
				postFinder( null, results, matcherOut, xml );
			} else {
				push.apply( results, matcherOut );
			}
		}
	});
}

function matcherFromTokens( tokens ) {
	var checkContext, matcher, j,
		len = tokens.length,
		leadingRelative = Expr.relative[ tokens[0].type ],
		implicitRelative = leadingRelative || Expr.relative[" "],
		i = leadingRelative ? 1 : 0,

		// The foundational matcher ensures that elements are reachable from top-level context(s)
		matchContext = addCombinator( function( elem ) {
			return elem === checkContext;
		}, implicitRelative, true ),
		matchAnyContext = addCombinator( function( elem ) {
			return indexOf.call( checkContext, elem ) > -1;
		}, implicitRelative, true ),
		matchers = [ function( elem, context, xml ) {
			return ( !leadingRelative && ( xml || context !== outermostContext ) ) || (
				(checkContext = context).nodeType ?
					matchContext( elem, context, xml ) :
					matchAnyContext( elem, context, xml ) );
		} ];

	for ( ; i < len; i++ ) {
		if ( (matcher = Expr.relative[ tokens[i].type ]) ) {
			matchers = [ addCombinator(elementMatcher( matchers ), matcher) ];
		} else {
			matcher = Expr.filter[ tokens[i].type ].apply( null, tokens[i].matches );

			// Return special upon seeing a positional matcher
			if ( matcher[ expando ] ) {
				// Find the next relative operator (if any) for proper handling
				j = ++i;
				for ( ; j < len; j++ ) {
					if ( Expr.relative[ tokens[j].type ] ) {
						break;
					}
				}
				return setMatcher(
					i > 1 && elementMatcher( matchers ),
					i > 1 && toSelector(
						// If the preceding token was a descendant combinator, insert an implicit any-element `*`
						tokens.slice( 0, i - 1 ).concat({ value: tokens[ i - 2 ].type === " " ? "*" : "" })
					).replace( rtrim, "$1" ),
					matcher,
					i < j && matcherFromTokens( tokens.slice( i, j ) ),
					j < len && matcherFromTokens( (tokens = tokens.slice( j )) ),
					j < len && toSelector( tokens )
				);
			}
			matchers.push( matcher );
		}
	}

	return elementMatcher( matchers );
}

function matcherFromGroupMatchers( elementMatchers, setMatchers ) {
	var bySet = setMatchers.length > 0,
		byElement = elementMatchers.length > 0,
		superMatcher = function( seed, context, xml, results, outermost ) {
			var elem, j, matcher,
				matchedCount = 0,
				i = "0",
				unmatched = seed && [],
				setMatched = [],
				contextBackup = outermostContext,
				// We must always have either seed elements or outermost context
				elems = seed || byElement && Expr.find["TAG"]( "*", outermost ),
				// Use integer dirruns iff this is the outermost matcher
				dirrunsUnique = (dirruns += contextBackup == null ? 1 : Math.random() || 0.1),
				len = elems.length;

			if ( outermost ) {
				outermostContext = context !== document && context;
			}

			// Add elements passing elementMatchers directly to results
			// Keep `i` a string if there are no elements so `matchedCount` will be "00" below
			// Support: IE<9, Safari
			// Tolerate NodeList properties (IE: "length"; Safari: <number>) matching elements by id
			for ( ; i !== len && (elem = elems[i]) != null; i++ ) {
				if ( byElement && elem ) {
					j = 0;
					while ( (matcher = elementMatchers[j++]) ) {
						if ( matcher( elem, context, xml ) ) {
							results.push( elem );
							break;
						}
					}
					if ( outermost ) {
						dirruns = dirrunsUnique;
					}
				}

				// Track unmatched elements for set filters
				if ( bySet ) {
					// They will have gone through all possible matchers
					if ( (elem = !matcher && elem) ) {
						matchedCount--;
					}

					// Lengthen the array for every element, matched or not
					if ( seed ) {
						unmatched.push( elem );
					}
				}
			}

			// Apply set filters to unmatched elements
			matchedCount += i;
			if ( bySet && i !== matchedCount ) {
				j = 0;
				while ( (matcher = setMatchers[j++]) ) {
					matcher( unmatched, setMatched, context, xml );
				}

				if ( seed ) {
					// Reintegrate element matches to eliminate the need for sorting
					if ( matchedCount > 0 ) {
						while ( i-- ) {
							if ( !(unmatched[i] || setMatched[i]) ) {
								setMatched[i] = pop.call( results );
							}
						}
					}

					// Discard index placeholder values to get only actual matches
					setMatched = condense( setMatched );
				}

				// Add matches to results
				push.apply( results, setMatched );

				// Seedless set matches succeeding multiple successful matchers stipulate sorting
				if ( outermost && !seed && setMatched.length > 0 &&
					( matchedCount + setMatchers.length ) > 1 ) {

					Sizzle.uniqueSort( results );
				}
			}

			// Override manipulation of globals by nested matchers
			if ( outermost ) {
				dirruns = dirrunsUnique;
				outermostContext = contextBackup;
			}

			return unmatched;
		};

	return bySet ?
		markFunction( superMatcher ) :
		superMatcher;
}

compile = Sizzle.compile = function( selector, group /* Internal Use Only */ ) {
	var i,
		setMatchers = [],
		elementMatchers = [],
		cached = compilerCache[ selector + " " ];

	if ( !cached ) {
		// Generate a function of recursive functions that can be used to check each element
		if ( !group ) {
			group = tokenize( selector );
		}
		i = group.length;
		while ( i-- ) {
			cached = matcherFromTokens( group[i] );
			if ( cached[ expando ] ) {
				setMatchers.push( cached );
			} else {
				elementMatchers.push( cached );
			}
		}

		// Cache the compiled function
		cached = compilerCache( selector, matcherFromGroupMatchers( elementMatchers, setMatchers ) );
	}
	return cached;
};

function multipleContexts( selector, contexts, results ) {
	var i = 0,
		len = contexts.length;
	for ( ; i < len; i++ ) {
		Sizzle( selector, contexts[i], results );
	}
	return results;
}

function select( selector, context, results, seed ) {
	var i, tokens, token, type, find,
		match = tokenize( selector );

	if ( !seed ) {
		// Try to minimize operations if there is only one group
		if ( match.length === 1 ) {

			// Take a shortcut and set the context if the root selector is an ID
			tokens = match[0] = match[0].slice( 0 );
			if ( tokens.length > 2 && (token = tokens[0]).type === "ID" &&
					support.getById && context.nodeType === 9 && documentIsHTML &&
					Expr.relative[ tokens[1].type ] ) {

				context = ( Expr.find["ID"]( token.matches[0].replace(runescape, funescape), context ) || [] )[0];
				if ( !context ) {
					return results;
				}
				selector = selector.slice( tokens.shift().value.length );
			}

			// Fetch a seed set for right-to-left matching
			i = matchExpr["needsContext"].test( selector ) ? 0 : tokens.length;
			while ( i-- ) {
				token = tokens[i];

				// Abort if we hit a combinator
				if ( Expr.relative[ (type = token.type) ] ) {
					break;
				}
				if ( (find = Expr.find[ type ]) ) {
					// Search, expanding context for leading sibling combinators
					if ( (seed = find(
						token.matches[0].replace( runescape, funescape ),
						rsibling.test( tokens[0].type ) && testContext( context.parentNode ) || context
					)) ) {

						// If seed is empty or no tokens remain, we can return early
						tokens.splice( i, 1 );
						selector = seed.length && toSelector( tokens );
						if ( !selector ) {
							push.apply( results, seed );
							return results;
						}

						break;
					}
				}
			}
		}
	}

	// Compile and execute a filtering function
	// Provide `match` to avoid retokenization if we modified the selector above
	compile( selector, match )(
		seed,
		context,
		!documentIsHTML,
		results,
		rsibling.test( selector ) && testContext( context.parentNode ) || context
	);
	return results;
}

// One-time assignments

// Sort stability
support.sortStable = expando.split("").sort( sortOrder ).join("") === expando;

// Support: Chrome<14
// Always assume duplicates if they aren't passed to the comparison function
support.detectDuplicates = !!hasDuplicate;

// Initialize against the default document
setDocument();

// Support: Webkit<537.32 - Safari 6.0.3/Chrome 25 (fixed in Chrome 27)
// Detached nodes confoundingly follow *each other*
support.sortDetached = assert(function( div1 ) {
	// Should return 1, but returns 4 (following)
	return div1.compareDocumentPosition( document.createElement("div") ) & 1;
});

// Support: IE<8
// Prevent attribute/property "interpolation"
// http://msdn.microsoft.com/en-us/library/ms536429%28VS.85%29.aspx
if ( !assert(function( div ) {
	div.innerHTML = "<a href='#'></a>";
	return div.firstChild.getAttribute("href") === "#" ;
}) ) {
	addHandle( "type|href|height|width", function( elem, name, isXML ) {
		if ( !isXML ) {
			return elem.getAttribute( name, name.toLowerCase() === "type" ? 1 : 2 );
		}
	});
}

// Support: IE<9
// Use defaultValue in place of getAttribute("value")
if ( !support.attributes || !assert(function( div ) {
	div.innerHTML = "<input/>";
	div.firstChild.setAttribute( "value", "" );
	return div.firstChild.getAttribute( "value" ) === "";
}) ) {
	addHandle( "value", function( elem, name, isXML ) {
		if ( !isXML && elem.nodeName.toLowerCase() === "input" ) {
			return elem.defaultValue;
		}
	});
}

// Support: IE<9
// Use getAttributeNode to fetch booleans when getAttribute lies
if ( !assert(function( div ) {
	return div.getAttribute("disabled") == null;
}) ) {
	addHandle( booleans, function( elem, name, isXML ) {
		var val;
		if ( !isXML ) {
			return elem[ name ] === true ? name.toLowerCase() :
					(val = elem.getAttributeNode( name )) && val.specified ?
					val.value :
				null;
		}
	});
}

return Sizzle;

})( window );



jQuery.find = Sizzle;
jQuery.expr = Sizzle.selectors;
jQuery.expr[":"] = jQuery.expr.pseudos;
jQuery.unique = Sizzle.uniqueSort;
jQuery.text = Sizzle.getText;
jQuery.isXMLDoc = Sizzle.isXML;
jQuery.contains = Sizzle.contains;



var rneedsContext = jQuery.expr.match.needsContext;

var rsingleTag = (/^<(\w+)\s*\/?>(?:<\/\1>|)$/);



var risSimple = /^.[^:#\[\.,]*$/;

// Implement the identical functionality for filter and not
function winnow( elements, qualifier, not ) {
	if ( jQuery.isFunction( qualifier ) ) {
		return jQuery.grep( elements, function( elem, i ) {
			/* jshint -W018 */
			return !!qualifier.call( elem, i, elem ) !== not;
		});

	}

	if ( qualifier.nodeType ) {
		return jQuery.grep( elements, function( elem ) {
			return ( elem === qualifier ) !== not;
		});

	}

	if ( typeof qualifier === "string" ) {
		if ( risSimple.test( qualifier ) ) {
			return jQuery.filter( qualifier, elements, not );
		}

		qualifier = jQuery.filter( qualifier, elements );
	}

	return jQuery.grep( elements, function( elem ) {
		return ( indexOf.call( qualifier, elem ) >= 0 ) !== not;
	});
}

jQuery.filter = function( expr, elems, not ) {
	var elem = elems[ 0 ];

	if ( not ) {
		expr = ":not(" + expr + ")";
	}

	return elems.length === 1 && elem.nodeType === 1 ?
		jQuery.find.matchesSelector( elem, expr ) ? [ elem ] : [] :
		jQuery.find.matches( expr, jQuery.grep( elems, function( elem ) {
			return elem.nodeType === 1;
		}));
};

jQuery.fn.extend({
	find: function( selector ) {
		var i,
			len = this.length,
			ret = [],
			self = this;

		if ( typeof selector !== "string" ) {
			return this.pushStack( jQuery( selector ).filter(function() {
				for ( i = 0; i < len; i++ ) {
					if ( jQuery.contains( self[ i ], this ) ) {
						return true;
					}
				}
			}) );
		}

		for ( i = 0; i < len; i++ ) {
			jQuery.find( selector, self[ i ], ret );
		}

		// Needed because $( selector, context ) becomes $( context ).find( selector )
		ret = this.pushStack( len > 1 ? jQuery.unique( ret ) : ret );
		ret.selector = this.selector ? this.selector + " " + selector : selector;
		return ret;
	},
	filter: function( selector ) {
		return this.pushStack( winnow(this, selector || [], false) );
	},
	not: function( selector ) {
		return this.pushStack( winnow(this, selector || [], true) );
	},
	is: function( selector ) {
		return !!winnow(
			this,

			// If this is a positional/relative selector, check membership in the returned set
			// so $("p:first").is("p:last") won't return true for a doc with two "p".
			typeof selector === "string" && rneedsContext.test( selector ) ?
				jQuery( selector ) :
				selector || [],
			false
		).length;
	}
});


// Initialize a jQuery object


// A central reference to the root jQuery(document)
var rootjQuery,

	// A simple way to check for HTML strings
	// Prioritize #id over <tag> to avoid XSS via location.hash (#9521)
	// Strict HTML recognition (#11290: must start with <)
	rquickExpr = /^(?:\s*(<[\w\W]+>)[^>]*|#([\w-]*))$/,

	init = jQuery.fn.init = function( selector, context ) {
		var match, elem;

		// HANDLE: $(""), $(null), $(undefined), $(false)
		if ( !selector ) {
			return this;
		}

		// Handle HTML strings
		if ( typeof selector === "string" ) {
			if ( selector[0] === "<" && selector[ selector.length - 1 ] === ">" && selector.length >= 3 ) {
				// Assume that strings that start and end with <> are HTML and skip the regex check
				match = [ null, selector, null ];

			} else {
				match = rquickExpr.exec( selector );
			}

			// Match html or make sure no context is specified for #id
			if ( match && (match[1] || !context) ) {

				// HANDLE: $(html) -> $(array)
				if ( match[1] ) {
					context = context instanceof jQuery ? context[0] : context;

					// scripts is true for back-compat
					// Intentionally let the error be thrown if parseHTML is not present
					jQuery.merge( this, jQuery.parseHTML(
						match[1],
						context && context.nodeType ? context.ownerDocument || context : document,
						true
					) );

					// HANDLE: $(html, props)
					if ( rsingleTag.test( match[1] ) && jQuery.isPlainObject( context ) ) {
						for ( match in context ) {
							// Properties of context are called as methods if possible
							if ( jQuery.isFunction( this[ match ] ) ) {
								this[ match ]( context[ match ] );

							// ...and otherwise set as attributes
							} else {
								this.attr( match, context[ match ] );
							}
						}
					}

					return this;

				// HANDLE: $(#id)
				} else {
					elem = document.getElementById( match[2] );

					// Check parentNode to catch when Blackberry 4.6 returns
					// nodes that are no longer in the document #6963
					if ( elem && elem.parentNode ) {
						// Inject the element directly into the jQuery object
						this.length = 1;
						this[0] = elem;
					}

					this.context = document;
					this.selector = selector;
					return this;
				}

			// HANDLE: $(expr, $(...))
			} else if ( !context || context.jquery ) {
				return ( context || rootjQuery ).find( selector );

			// HANDLE: $(expr, context)
			// (which is just equivalent to: $(context).find(expr)
			} else {
				return this.constructor( context ).find( selector );
			}

		// HANDLE: $(DOMElement)
		} else if ( selector.nodeType ) {
			this.context = this[0] = selector;
			this.length = 1;
			return this;

		// HANDLE: $(function)
		// Shortcut for document ready
		} else if ( jQuery.isFunction( selector ) ) {
			return typeof rootjQuery.ready !== "undefined" ?
				rootjQuery.ready( selector ) :
				// Execute immediately if ready is not present
				selector( jQuery );
		}

		if ( selector.selector !== undefined ) {
			this.selector = selector.selector;
			this.context = selector.context;
		}

		return jQuery.makeArray( selector, this );
	};

// Give the init function the jQuery prototype for later instantiation
init.prototype = jQuery.fn;

// Initialize central reference
rootjQuery = jQuery( document );


var rparentsprev = /^(?:parents|prev(?:Until|All))/,
	// methods guaranteed to produce a unique set when starting from a unique set
	guaranteedUnique = {
		children: true,
		contents: true,
		next: true,
		prev: true
	};

jQuery.extend({
	dir: function( elem, dir, until ) {
		var matched = [],
			truncate = until !== undefined;

		while ( (elem = elem[ dir ]) && elem.nodeType !== 9 ) {
			if ( elem.nodeType === 1 ) {
				if ( truncate && jQuery( elem ).is( until ) ) {
					break;
				}
				matched.push( elem );
			}
		}
		return matched;
	},

	sibling: function( n, elem ) {
		var matched = [];

		for ( ; n; n = n.nextSibling ) {
			if ( n.nodeType === 1 && n !== elem ) {
				matched.push( n );
			}
		}

		return matched;
	}
});

jQuery.fn.extend({
	has: function( target ) {
		var targets = jQuery( target, this ),
			l = targets.length;

		return this.filter(function() {
			var i = 0;
			for ( ; i < l; i++ ) {
				if ( jQuery.contains( this, targets[i] ) ) {
					return true;
				}
			}
		});
	},

	closest: function( selectors, context ) {
		var cur,
			i = 0,
			l = this.length,
			matched = [],
			pos = rneedsContext.test( selectors ) || typeof selectors !== "string" ?
				jQuery( selectors, context || this.context ) :
				0;

		for ( ; i < l; i++ ) {
			for ( cur = this[i]; cur && cur !== context; cur = cur.parentNode ) {
				// Always skip document fragments
				if ( cur.nodeType < 11 && (pos ?
					pos.index(cur) > -1 :

					// Don't pass non-elements to Sizzle
					cur.nodeType === 1 &&
						jQuery.find.matchesSelector(cur, selectors)) ) {

					matched.push( cur );
					break;
				}
			}
		}

		return this.pushStack( matched.length > 1 ? jQuery.unique( matched ) : matched );
	},

	// Determine the position of an element within
	// the matched set of elements
	index: function( elem ) {

		// No argument, return index in parent
		if ( !elem ) {
			return ( this[ 0 ] && this[ 0 ].parentNode ) ? this.first().prevAll().length : -1;
		}

		// index in selector
		if ( typeof elem === "string" ) {
			return indexOf.call( jQuery( elem ), this[ 0 ] );
		}

		// Locate the position of the desired element
		return indexOf.call( this,

			// If it receives a jQuery object, the first element is used
			elem.jquery ? elem[ 0 ] : elem
		);
	},

	add: function( selector, context ) {
		return this.pushStack(
			jQuery.unique(
				jQuery.merge( this.get(), jQuery( selector, context ) )
			)
		);
	},

	addBack: function( selector ) {
		return this.add( selector == null ?
			this.prevObject : this.prevObject.filter(selector)
		);
	}
});

function sibling( cur, dir ) {
	while ( (cur = cur[dir]) && cur.nodeType !== 1 ) {}
	return cur;
}

jQuery.each({
	parent: function( elem ) {
		var parent = elem.parentNode;
		return parent && parent.nodeType !== 11 ? parent : null;
	},
	parents: function( elem ) {
		return jQuery.dir( elem, "parentNode" );
	},
	parentsUntil: function( elem, i, until ) {
		return jQuery.dir( elem, "parentNode", until );
	},
	next: function( elem ) {
		return sibling( elem, "nextSibling" );
	},
	prev: function( elem ) {
		return sibling( elem, "previousSibling" );
	},
	nextAll: function( elem ) {
		return jQuery.dir( elem, "nextSibling" );
	},
	prevAll: function( elem ) {
		return jQuery.dir( elem, "previousSibling" );
	},
	nextUntil: function( elem, i, until ) {
		return jQuery.dir( elem, "nextSibling", until );
	},
	prevUntil: function( elem, i, until ) {
		return jQuery.dir( elem, "previousSibling", until );
	},
	siblings: function( elem ) {
		return jQuery.sibling( ( elem.parentNode || {} ).firstChild, elem );
	},
	children: function( elem ) {
		return jQuery.sibling( elem.firstChild );
	},
	contents: function( elem ) {
		return elem.contentDocument || jQuery.merge( [], elem.childNodes );
	}
}, function( name, fn ) {
	jQuery.fn[ name ] = function( until, selector ) {
		var matched = jQuery.map( this, fn, until );

		if ( name.slice( -5 ) !== "Until" ) {
			selector = until;
		}

		if ( selector && typeof selector === "string" ) {
			matched = jQuery.filter( selector, matched );
		}

		if ( this.length > 1 ) {
			// Remove duplicates
			if ( !guaranteedUnique[ name ] ) {
				jQuery.unique( matched );
			}

			// Reverse order for parents* and prev-derivatives
			if ( rparentsprev.test( name ) ) {
				matched.reverse();
			}
		}

		return this.pushStack( matched );
	};
});
var rnotwhite = (/\S+/g);



// String to Object options format cache
var optionsCache = {};

// Convert String-formatted options into Object-formatted ones and store in cache
function createOptions( options ) {
	var object = optionsCache[ options ] = {};
	jQuery.each( options.match( rnotwhite ) || [], function( _, flag ) {
		object[ flag ] = true;
	});
	return object;
}

/*
 * Create a callback list using the following parameters:
 *
 *	options: an optional list of space-separated options that will change how
 *			the callback list behaves or a more traditional option object
 *
 * By default a callback list will act like an event callback list and can be
 * "fired" multiple times.
 *
 * Possible options:
 *
 *	once:			will ensure the callback list can only be fired once (like a Deferred)
 *
 *	memory:			will keep track of previous values and will call any callback added
 *					after the list has been fired right away with the latest "memorized"
 *					values (like a Deferred)
 *
 *	unique:			will ensure a callback can only be added once (no duplicate in the list)
 *
 *	stopOnFalse:	interrupt callings when a callback returns false
 *
 */
jQuery.Callbacks = function( options ) {

	// Convert options from String-formatted to Object-formatted if needed
	// (we check in cache first)
	options = typeof options === "string" ?
		( optionsCache[ options ] || createOptions( options ) ) :
		jQuery.extend( {}, options );

	var // Last fire value (for non-forgettable lists)
		memory,
		// Flag to know if list was already fired
		fired,
		// Flag to know if list is currently firing
		firing,
		// First callback to fire (used internally by add and fireWith)
		firingStart,
		// End of the loop when firing
		firingLength,
		// Index of currently firing callback (modified by remove if needed)
		firingIndex,
		// Actual callback list
		list = [],
		// Stack of fire calls for repeatable lists
		stack = !options.once && [],
		// Fire callbacks
		fire = function( data ) {
			memory = options.memory && data;
			fired = true;
			firingIndex = firingStart || 0;
			firingStart = 0;
			firingLength = list.length;
			firing = true;
			for ( ; list && firingIndex < firingLength; firingIndex++ ) {
				if ( list[ firingIndex ].apply( data[ 0 ], data[ 1 ] ) === false && options.stopOnFalse ) {
					memory = false; // To prevent further calls using add
					break;
				}
			}
			firing = false;
			if ( list ) {
				if ( stack ) {
					if ( stack.length ) {
						fire( stack.shift() );
					}
				} else if ( memory ) {
					list = [];
				} else {
					self.disable();
				}
			}
		},
		// Actual Callbacks object
		self = {
			// Add a callback or a collection of callbacks to the list
			add: function() {
				if ( list ) {
					// First, we save the current length
					var start = list.length;
					(function add( args ) {
						jQuery.each( args, function( _, arg ) {
							var type = jQuery.type( arg );
							if ( type === "function" ) {
								if ( !options.unique || !self.has( arg ) ) {
									list.push( arg );
								}
							} else if ( arg && arg.length && type !== "string" ) {
								// Inspect recursively
								add( arg );
							}
						});
					})( arguments );
					// Do we need to add the callbacks to the
					// current firing batch?
					if ( firing ) {
						firingLength = list.length;
					// With memory, if we're not firing then
					// we should call right away
					} else if ( memory ) {
						firingStart = start;
						fire( memory );
					}
				}
				return this;
			},
			// Remove a callback from the list
			remove: function() {
				if ( list ) {
					jQuery.each( arguments, function( _, arg ) {
						var index;
						while ( ( index = jQuery.inArray( arg, list, index ) ) > -1 ) {
							list.splice( index, 1 );
							// Handle firing indexes
							if ( firing ) {
								if ( index <= firingLength ) {
									firingLength--;
								}
								if ( index <= firingIndex ) {
									firingIndex--;
								}
							}
						}
					});
				}
				return this;
			},
			// Check if a given callback is in the list.
			// If no argument is given, return whether or not list has callbacks attached.
			has: function( fn ) {
				return fn ? jQuery.inArray( fn, list ) > -1 : !!( list && list.length );
			},
			// Remove all callbacks from the list
			empty: function() {
				list = [];
				firingLength = 0;
				return this;
			},
			// Have the list do nothing anymore
			disable: function() {
				list = stack = memory = undefined;
				return this;
			},
			// Is it disabled?
			disabled: function() {
				return !list;
			},
			// Lock the list in its current state
			lock: function() {
				stack = undefined;
				if ( !memory ) {
					self.disable();
				}
				return this;
			},
			// Is it locked?
			locked: function() {
				return !stack;
			},
			// Call all callbacks with the given context and arguments
			fireWith: function( context, args ) {
				if ( list && ( !fired || stack ) ) {
					args = args || [];
					args = [ context, args.slice ? args.slice() : args ];
					if ( firing ) {
						stack.push( args );
					} else {
						fire( args );
					}
				}
				return this;
			},
			// Call all the callbacks with the given arguments
			fire: function() {
				self.fireWith( this, arguments );
				return this;
			},
			// To know if the callbacks have already been called at least once
			fired: function() {
				return !!fired;
			}
		};

	return self;
};


jQuery.extend({

	Deferred: function( func ) {
		var tuples = [
				// action, add listener, listener list, final state
				[ "resolve", "done", jQuery.Callbacks("once memory"), "resolved" ],
				[ "reject", "fail", jQuery.Callbacks("once memory"), "rejected" ],
				[ "notify", "progress", jQuery.Callbacks("memory") ]
			],
			state = "pending",
			promise = {
				state: function() {
					return state;
				},
				always: function() {
					deferred.done( arguments ).fail( arguments );
					return this;
				},
				then: function( /* fnDone, fnFail, fnProgress */ ) {
					var fns = arguments;
					return jQuery.Deferred(function( newDefer ) {
						jQuery.each( tuples, function( i, tuple ) {
							var fn = jQuery.isFunction( fns[ i ] ) && fns[ i ];
							// deferred[ done | fail | progress ] for forwarding actions to newDefer
							deferred[ tuple[1] ](function() {
								var returned = fn && fn.apply( this, arguments );
								if ( returned && jQuery.isFunction( returned.promise ) ) {
									returned.promise()
										.done( newDefer.resolve )
										.fail( newDefer.reject )
										.progress( newDefer.notify );
								} else {
									newDefer[ tuple[ 0 ] + "With" ]( this === promise ? newDefer.promise() : this, fn ? [ returned ] : arguments );
								}
							});
						});
						fns = null;
					}).promise();
				},
				// Get a promise for this deferred
				// If obj is provided, the promise aspect is added to the object
				promise: function( obj ) {
					return obj != null ? jQuery.extend( obj, promise ) : promise;
				}
			},
			deferred = {};

		// Keep pipe for back-compat
		promise.pipe = promise.then;

		// Add list-specific methods
		jQuery.each( tuples, function( i, tuple ) {
			var list = tuple[ 2 ],
				stateString = tuple[ 3 ];

			// promise[ done | fail | progress ] = list.add
			promise[ tuple[1] ] = list.add;

			// Handle state
			if ( stateString ) {
				list.add(function() {
					// state = [ resolved | rejected ]
					state = stateString;

				// [ reject_list | resolve_list ].disable; progress_list.lock
				}, tuples[ i ^ 1 ][ 2 ].disable, tuples[ 2 ][ 2 ].lock );
			}

			// deferred[ resolve | reject | notify ]
			deferred[ tuple[0] ] = function() {
				deferred[ tuple[0] + "With" ]( this === deferred ? promise : this, arguments );
				return this;
			};
			deferred[ tuple[0] + "With" ] = list.fireWith;
		});

		// Make the deferred a promise
		promise.promise( deferred );

		// Call given func if any
		if ( func ) {
			func.call( deferred, deferred );
		}

		// All done!
		return deferred;
	},

	// Deferred helper
	when: function( subordinate /* , ..., subordinateN */ ) {
		var i = 0,
			resolveValues = slice.call( arguments ),
			length = resolveValues.length,

			// the count of uncompleted subordinates
			remaining = length !== 1 || ( subordinate && jQuery.isFunction( subordinate.promise ) ) ? length : 0,

			// the master Deferred. If resolveValues consist of only a single Deferred, just use that.
			deferred = remaining === 1 ? subordinate : jQuery.Deferred(),

			// Update function for both resolve and progress values
			updateFunc = function( i, contexts, values ) {
				return function( value ) {
					contexts[ i ] = this;
					values[ i ] = arguments.length > 1 ? slice.call( arguments ) : value;
					if ( values === progressValues ) {
						deferred.notifyWith( contexts, values );
					} else if ( !( --remaining ) ) {
						deferred.resolveWith( contexts, values );
					}
				};
			},

			progressValues, progressContexts, resolveContexts;

		// add listeners to Deferred subordinates; treat others as resolved
		if ( length > 1 ) {
			progressValues = new Array( length );
			progressContexts = new Array( length );
			resolveContexts = new Array( length );
			for ( ; i < length; i++ ) {
				if ( resolveValues[ i ] && jQuery.isFunction( resolveValues[ i ].promise ) ) {
					resolveValues[ i ].promise()
						.done( updateFunc( i, resolveContexts, resolveValues ) )
						.fail( deferred.reject )
						.progress( updateFunc( i, progressContexts, progressValues ) );
				} else {
					--remaining;
				}
			}
		}

		// if we're not waiting on anything, resolve the master
		if ( !remaining ) {
			deferred.resolveWith( resolveContexts, resolveValues );
		}

		return deferred.promise();
	}
});


// The deferred used on DOM ready
var readyList;

jQuery.fn.ready = function( fn ) {
	// Add the callback
	jQuery.ready.promise().done( fn );

	return this;
};

jQuery.extend({
	// Is the DOM ready to be used? Set to true once it occurs.
	isReady: false,

	// A counter to track how many items to wait for before
	// the ready event fires. See #6781
	readyWait: 1,

	// Hold (or release) the ready event
	holdReady: function( hold ) {
		if ( hold ) {
			jQuery.readyWait++;
		} else {
			jQuery.ready( true );
		}
	},

	// Handle when the DOM is ready
	ready: function( wait ) {

		// Abort if there are pending holds or we're already ready
		if ( wait === true ? --jQuery.readyWait : jQuery.isReady ) {
			return;
		}

		// Remember that the DOM is ready
		jQuery.isReady = true;

		// If a normal DOM Ready event fired, decrement, and wait if need be
		if ( wait !== true && --jQuery.readyWait > 0 ) {
			return;
		}

		// If there are functions bound, to execute
		readyList.resolveWith( document, [ jQuery ] );

		// Trigger any bound ready events
		if ( jQuery.fn.trigger ) {
			jQuery( document ).trigger("ready").off("ready");
		}
	}
});

/**
 * The ready event handler and self cleanup method
 */
function completed() {
	document.removeEventListener( "DOMContentLoaded", completed, false );
	window.removeEventListener( "load", completed, false );
	jQuery.ready();
}

jQuery.ready.promise = function( obj ) {
	if ( !readyList ) {

		readyList = jQuery.Deferred();

		// Catch cases where $(document).ready() is called after the browser event has already occurred.
		// we once tried to use readyState "interactive" here, but it caused issues like the one
		// discovered by ChrisS here: http://bugs.jquery.com/ticket/12282#comment:15
		if ( document.readyState === "complete" ) {
			// Handle it asynchronously to allow scripts the opportunity to delay ready
			setTimeout( jQuery.ready );

		} else {

			// Use the handy event callback
			document.addEventListener( "DOMContentLoaded", completed, false );

			// A fallback to window.onload, that will always work
			window.addEventListener( "load", completed, false );
		}
	}
	return readyList.promise( obj );
};

// Kick off the DOM ready check even if the user does not
jQuery.ready.promise();




// Multifunctional method to get and set values of a collection
// The value/s can optionally be executed if it's a function
var access = jQuery.access = function( elems, fn, key, value, chainable, emptyGet, raw ) {
	var i = 0,
		len = elems.length,
		bulk = key == null;

	// Sets many values
	if ( jQuery.type( key ) === "object" ) {
		chainable = true;
		for ( i in key ) {
			jQuery.access( elems, fn, i, key[i], true, emptyGet, raw );
		}

	// Sets one value
	} else if ( value !== undefined ) {
		chainable = true;

		if ( !jQuery.isFunction( value ) ) {
			raw = true;
		}

		if ( bulk ) {
			// Bulk operations run against the entire set
			if ( raw ) {
				fn.call( elems, value );
				fn = null;

			// ...except when executing function values
			} else {
				bulk = fn;
				fn = function( elem, key, value ) {
					return bulk.call( jQuery( elem ), value );
				};
			}
		}

		if ( fn ) {
			for ( ; i < len; i++ ) {
				fn( elems[i], key, raw ? value : value.call( elems[i], i, fn( elems[i], key ) ) );
			}
		}
	}

	return chainable ?
		elems :

		// Gets
		bulk ?
			fn.call( elems ) :
			len ? fn( elems[0], key ) : emptyGet;
};


/**
 * Determines whether an object can have data
 */
jQuery.acceptData = function( owner ) {
	// Accepts only:
	//  - Node
	//    - Node.ELEMENT_NODE
	//    - Node.DOCUMENT_NODE
	//  - Object
	//    - Any
	/* jshint -W018 */
	return owner.nodeType === 1 || owner.nodeType === 9 || !( +owner.nodeType );
};


function Data() {
	// Support: Android < 4,
	// Old WebKit does not have Object.preventExtensions/freeze method,
	// return new empty object instead with no [[set]] accessor
	Object.defineProperty( this.cache = {}, 0, {
		get: function() {
			return {};
		}
	});

	this.expando = jQuery.expando + Math.random();
}

Data.uid = 1;
Data.accepts = jQuery.acceptData;

Data.prototype = {
	key: function( owner ) {
		// We can accept data for non-element nodes in modern browsers,
		// but we should not, see #8335.
		// Always return the key for a frozen object.
		if ( !Data.accepts( owner ) ) {
			return 0;
		}

		var descriptor = {},
			// Check if the owner object already has a cache key
			unlock = owner[ this.expando ];

		// If not, create one
		if ( !unlock ) {
			unlock = Data.uid++;

			// Secure it in a non-enumerable, non-writable property
			try {
				descriptor[ this.expando ] = { value: unlock };
				Object.defineProperties( owner, descriptor );

			// Support: Android < 4
			// Fallback to a less secure definition
			} catch ( e ) {
				descriptor[ this.expando ] = unlock;
				jQuery.extend( owner, descriptor );
			}
		}

		// Ensure the cache object
		if ( !this.cache[ unlock ] ) {
			this.cache[ unlock ] = {};
		}

		return unlock;
	},
	set: function( owner, data, value ) {
		var prop,
			// There may be an unlock assigned to this node,
			// if there is no entry for this "owner", create one inline
			// and set the unlock as though an owner entry had always existed
			unlock = this.key( owner ),
			cache = this.cache[ unlock ];

		// Handle: [ owner, key, value ] args
		if ( typeof data === "string" ) {
			cache[ data ] = value;

		// Handle: [ owner, { properties } ] args
		} else {
			// Fresh assignments by object are shallow copied
			if ( jQuery.isEmptyObject( cache ) ) {
				jQuery.extend( this.cache[ unlock ], data );
			// Otherwise, copy the properties one-by-one to the cache object
			} else {
				for ( prop in data ) {
					cache[ prop ] = data[ prop ];
				}
			}
		}
		return cache;
	},
	get: function( owner, key ) {
		// Either a valid cache is found, or will be created.
		// New caches will be created and the unlock returned,
		// allowing direct access to the newly created
		// empty data object. A valid owner object must be provided.
		var cache = this.cache[ this.key( owner ) ];

		return key === undefined ?
			cache : cache[ key ];
	},
	access: function( owner, key, value ) {
		var stored;
		// In cases where either:
		//
		//   1. No key was specified
		//   2. A string key was specified, but no value provided
		//
		// Take the "read" path and allow the get method to determine
		// which value to return, respectively either:
		//
		//   1. The entire cache object
		//   2. The data stored at the key
		//
		if ( key === undefined ||
				((key && typeof key === "string") && value === undefined) ) {

			stored = this.get( owner, key );

			return stored !== undefined ?
				stored : this.get( owner, jQuery.camelCase(key) );
		}

		// [*]When the key is not a string, or both a key and value
		// are specified, set or extend (existing objects) with either:
		//
		//   1. An object of properties
		//   2. A key and value
		//
		this.set( owner, key, value );

		// Since the "set" path can have two possible entry points
		// return the expected data based on which path was taken[*]
		return value !== undefined ? value : key;
	},
	remove: function( owner, key ) {
		var i, name, camel,
			unlock = this.key( owner ),
			cache = this.cache[ unlock ];

		if ( key === undefined ) {
			this.cache[ unlock ] = {};

		} else {
			// Support array or space separated string of keys
			if ( jQuery.isArray( key ) ) {
				// If "name" is an array of keys...
				// When data is initially created, via ("key", "val") signature,
				// keys will be converted to camelCase.
				// Since there is no way to tell _how_ a key was added, remove
				// both plain key and camelCase key. #12786
				// This will only penalize the array argument path.
				name = key.concat( key.map( jQuery.camelCase ) );
			} else {
				camel = jQuery.camelCase( key );
				// Try the string as a key before any manipulation
				if ( key in cache ) {
					name = [ key, camel ];
				} else {
					// If a key with the spaces exists, use it.
					// Otherwise, create an array by matching non-whitespace
					name = camel;
					name = name in cache ?
						[ name ] : ( name.match( rnotwhite ) || [] );
				}
			}

			i = name.length;
			while ( i-- ) {
				delete cache[ name[ i ] ];
			}
		}
	},
	hasData: function( owner ) {
		return !jQuery.isEmptyObject(
			this.cache[ owner[ this.expando ] ] || {}
		);
	},
	discard: function( owner ) {
		if ( owner[ this.expando ] ) {
			delete this.cache[ owner[ this.expando ] ];
		}
	}
};
var data_priv = new Data();

var data_user = new Data();



/*
	Implementation Summary

	1. Enforce API surface and semantic compatibility with 1.9.x branch
	2. Improve the module's maintainability by reducing the storage
		paths to a single mechanism.
	3. Use the same single mechanism to support "private" and "user" data.
	4. _Never_ expose "private" data to user code (TODO: Drop _data, _removeData)
	5. Avoid exposing implementation details on user objects (eg. expando properties)
	6. Provide a clear path for implementation upgrade to WeakMap in 2014
*/
var rbrace = /^(?:\{[\w\W]*\}|\[[\w\W]*\])$/,
	rmultiDash = /([A-Z])/g;

function dataAttr( elem, key, data ) {
	var name;

	// If nothing was found internally, try to fetch any
	// data from the HTML5 data-* attribute
	if ( data === undefined && elem.nodeType === 1 ) {
		name = "data-" + key.replace( rmultiDash, "-$1" ).toLowerCase();
		data = elem.getAttribute( name );

		if ( typeof data === "string" ) {
			try {
				data = data === "true" ? true :
					data === "false" ? false :
					data === "null" ? null :
					// Only convert to a number if it doesn't change the string
					+data + "" === data ? +data :
					rbrace.test( data ) ? jQuery.parseJSON( data ) :
					data;
			} catch( e ) {}

			// Make sure we set the data so it isn't changed later
			data_user.set( elem, key, data );
		} else {
			data = undefined;
		}
	}
	return data;
}

jQuery.extend({
	hasData: function( elem ) {
		return data_user.hasData( elem ) || data_priv.hasData( elem );
	},

	data: function( elem, name, data ) {
		return data_user.access( elem, name, data );
	},

	removeData: function( elem, name ) {
		data_user.remove( elem, name );
	},

	// TODO: Now that all calls to _data and _removeData have been replaced
	// with direct calls to data_priv methods, these can be deprecated.
	_data: function( elem, name, data ) {
		return data_priv.access( elem, name, data );
	},

	_removeData: function( elem, name ) {
		data_priv.remove( elem, name );
	}
});

jQuery.fn.extend({
	data: function( key, value ) {
		var i, name, data,
			elem = this[ 0 ],
			attrs = elem && elem.attributes;

		// Gets all values
		if ( key === undefined ) {
			if ( this.length ) {
				data = data_user.get( elem );

				if ( elem.nodeType === 1 && !data_priv.get( elem, "hasDataAttrs" ) ) {
					i = attrs.length;
					while ( i-- ) {
						name = attrs[ i ].name;

						if ( name.indexOf( "data-" ) === 0 ) {
							name = jQuery.camelCase( name.slice(5) );
							dataAttr( elem, name, data[ name ] );
						}
					}
					data_priv.set( elem, "hasDataAttrs", true );
				}
			}

			return data;
		}

		// Sets multiple values
		if ( typeof key === "object" ) {
			return this.each(function() {
				data_user.set( this, key );
			});
		}

		return access( this, function( value ) {
			var data,
				camelKey = jQuery.camelCase( key );

			// The calling jQuery object (element matches) is not empty
			// (and therefore has an element appears at this[ 0 ]) and the
			// `value` parameter was not undefined. An empty jQuery object
			// will result in `undefined` for elem = this[ 0 ] which will
			// throw an exception if an attempt to read a data cache is made.
			if ( elem && value === undefined ) {
				// Attempt to get data from the cache
				// with the key as-is
				data = data_user.get( elem, key );
				if ( data !== undefined ) {
					return data;
				}

				// Attempt to get data from the cache
				// with the key camelized
				data = data_user.get( elem, camelKey );
				if ( data !== undefined ) {
					return data;
				}

				// Attempt to "discover" the data in
				// HTML5 custom data-* attrs
				data = dataAttr( elem, camelKey, undefined );
				if ( data !== undefined ) {
					return data;
				}

				// We tried really hard, but the data doesn't exist.
				return;
			}

			// Set the data...
			this.each(function() {
				// First, attempt to store a copy or reference of any
				// data that might've been store with a camelCased key.
				var data = data_user.get( this, camelKey );

				// For HTML5 data-* attribute interop, we have to
				// store property names with dashes in a camelCase form.
				// This might not apply to all properties...*
				data_user.set( this, camelKey, value );

				// *... In the case of properties that might _actually_
				// have dashes, we need to also store a copy of that
				// unchanged property.
				if ( key.indexOf("-") !== -1 && data !== undefined ) {
					data_user.set( this, key, value );
				}
			});
		}, null, value, arguments.length > 1, null, true );
	},

	removeData: function( key ) {
		return this.each(function() {
			data_user.remove( this, key );
		});
	}
});


jQuery.extend({
	queue: function( elem, type, data ) {
		var queue;

		if ( elem ) {
			type = ( type || "fx" ) + "queue";
			queue = data_priv.get( elem, type );

			// Speed up dequeue by getting out quickly if this is just a lookup
			if ( data ) {
				if ( !queue || jQuery.isArray( data ) ) {
					queue = data_priv.access( elem, type, jQuery.makeArray(data) );
				} else {
					queue.push( data );
				}
			}
			return queue || [];
		}
	},

	dequeue: function( elem, type ) {
		type = type || "fx";

		var queue = jQuery.queue( elem, type ),
			startLength = queue.length,
			fn = queue.shift(),
			hooks = jQuery._queueHooks( elem, type ),
			next = function() {
				jQuery.dequeue( elem, type );
			};

		// If the fx queue is dequeued, always remove the progress sentinel
		if ( fn === "inprogress" ) {
			fn = queue.shift();
			startLength--;
		}

		if ( fn ) {

			// Add a progress sentinel to prevent the fx queue from being
			// automatically dequeued
			if ( type === "fx" ) {
				queue.unshift( "inprogress" );
			}

			// clear up the last queue stop function
			delete hooks.stop;
			fn.call( elem, next, hooks );
		}

		if ( !startLength && hooks ) {
			hooks.empty.fire();
		}
	},

	// not intended for public consumption - generates a queueHooks object, or returns the current one
	_queueHooks: function( elem, type ) {
		var key = type + "queueHooks";
		return data_priv.get( elem, key ) || data_priv.access( elem, key, {
			empty: jQuery.Callbacks("once memory").add(function() {
				data_priv.remove( elem, [ type + "queue", key ] );
			})
		});
	}
});

jQuery.fn.extend({
	queue: function( type, data ) {
		var setter = 2;

		if ( typeof type !== "string" ) {
			data = type;
			type = "fx";
			setter--;
		}

		if ( arguments.length < setter ) {
			return jQuery.queue( this[0], type );
		}

		return data === undefined ?
			this :
			this.each(function() {
				var queue = jQuery.queue( this, type, data );

				// ensure a hooks for this queue
				jQuery._queueHooks( this, type );

				if ( type === "fx" && queue[0] !== "inprogress" ) {
					jQuery.dequeue( this, type );
				}
			});
	},
	dequeue: function( type ) {
		return this.each(function() {
			jQuery.dequeue( this, type );
		});
	},
	clearQueue: function( type ) {
		return this.queue( type || "fx", [] );
	},
	// Get a promise resolved when queues of a certain type
	// are emptied (fx is the type by default)
	promise: function( type, obj ) {
		var tmp,
			count = 1,
			defer = jQuery.Deferred(),
			elements = this,
			i = this.length,
			resolve = function() {
				if ( !( --count ) ) {
					defer.resolveWith( elements, [ elements ] );
				}
			};

		if ( typeof type !== "string" ) {
			obj = type;
			type = undefined;
		}
		type = type || "fx";

		while ( i-- ) {
			tmp = data_priv.get( elements[ i ], type + "queueHooks" );
			if ( tmp && tmp.empty ) {
				count++;
				tmp.empty.add( resolve );
			}
		}
		resolve();
		return defer.promise( obj );
	}
});
var pnum = (/[+-]?(?:\d*\.|)\d+(?:[eE][+-]?\d+|)/).source;

var cssExpand = [ "Top", "Right", "Bottom", "Left" ];

var isHidden = function( elem, el ) {
		// isHidden might be called from jQuery#filter function;
		// in that case, element will be second argument
		elem = el || elem;
		return jQuery.css( elem, "display" ) === "none" || !jQuery.contains( elem.ownerDocument, elem );
	};

var rcheckableType = (/^(?:checkbox|radio)$/i);



(function() {
	var fragment = document.createDocumentFragment(),
		div = fragment.appendChild( document.createElement( "div" ) );

	// #11217 - WebKit loses check when the name is after the checked attribute
	div.innerHTML = "<input type='radio' checked='checked' name='t'/>";

	// Support: Safari 5.1, iOS 5.1, Android 4.x, Android 2.3
	// old WebKit doesn't clone checked state correctly in fragments
	support.checkClone = div.cloneNode( true ).cloneNode( true ).lastChild.checked;

	// Make sure textarea (and checkbox) defaultValue is properly cloned
	// Support: IE9-IE11+
	div.innerHTML = "<textarea>x</textarea>";
	support.noCloneChecked = !!div.cloneNode( true ).lastChild.defaultValue;
})();
var strundefined = typeof undefined;



support.focusinBubbles = "onfocusin" in window;


var
	rkeyEvent = /^key/,
	rmouseEvent = /^(?:mouse|contextmenu)|click/,
	rfocusMorph = /^(?:focusinfocus|focusoutblur)$/,
	rtypenamespace = /^([^.]*)(?:\.(.+)|)$/;

function returnTrue() {
	return true;
}

function returnFalse() {
	return false;
}

function safeActiveElement() {
	try {
		return document.activeElement;
	} catch ( err ) { }
}

/*
 * Helper functions for managing events -- not part of the public interface.
 * Props to Dean Edwards' addEvent library for many of the ideas.
 */
jQuery.event = {

	global: {},

	add: function( elem, types, handler, data, selector ) {

		var handleObjIn, eventHandle, tmp,
			events, t, handleObj,
			special, handlers, type, namespaces, origType,
			elemData = data_priv.get( elem );

		// Don't attach events to noData or text/comment nodes (but allow plain objects)
		if ( !elemData ) {
			return;
		}

		// Caller can pass in an object of custom data in lieu of the handler
		if ( handler.handler ) {
			handleObjIn = handler;
			handler = handleObjIn.handler;
			selector = handleObjIn.selector;
		}

		// Make sure that the handler has a unique ID, used to find/remove it later
		if ( !handler.guid ) {
			handler.guid = jQuery.guid++;
		}

		// Init the element's event structure and main handler, if this is the first
		if ( !(events = elemData.events) ) {
			events = elemData.events = {};
		}
		if ( !(eventHandle = elemData.handle) ) {
			eventHandle = elemData.handle = function( e ) {
				// Discard the second event of a jQuery.event.trigger() and
				// when an event is called after a page has unloaded
				return typeof jQuery !== strundefined && jQuery.event.triggered !== e.type ?
					jQuery.event.dispatch.apply( elem, arguments ) : undefined;
			};
		}

		// Handle multiple events separated by a space
		types = ( types || "" ).match( rnotwhite ) || [ "" ];
		t = types.length;
		while ( t-- ) {
			tmp = rtypenamespace.exec( types[t] ) || [];
			type = origType = tmp[1];
			namespaces = ( tmp[2] || "" ).split( "." ).sort();

			// There *must* be a type, no attaching namespace-only handlers
			if ( !type ) {
				continue;
			}

			// If event changes its type, use the special event handlers for the changed type
			special = jQuery.event.special[ type ] || {};

			// If selector defined, determine special event api type, otherwise given type
			type = ( selector ? special.delegateType : special.bindType ) || type;

			// Update special based on newly reset type
			special = jQuery.event.special[ type ] || {};

			// handleObj is passed to all event handlers
			handleObj = jQuery.extend({
				type: type,
				origType: origType,
				data: data,
				handler: handler,
				guid: handler.guid,
				selector: selector,
				needsContext: selector && jQuery.expr.match.needsContext.test( selector ),
				namespace: namespaces.join(".")
			}, handleObjIn );

			// Init the event handler queue if we're the first
			if ( !(handlers = events[ type ]) ) {
				handlers = events[ type ] = [];
				handlers.delegateCount = 0;

				// Only use addEventListener if the special events handler returns false
				if ( !special.setup || special.setup.call( elem, data, namespaces, eventHandle ) === false ) {
					if ( elem.addEventListener ) {
						elem.addEventListener( type, eventHandle, false );
					}
				}
			}

			if ( special.add ) {
				special.add.call( elem, handleObj );

				if ( !handleObj.handler.guid ) {
					handleObj.handler.guid = handler.guid;
				}
			}

			// Add to the element's handler list, delegates in front
			if ( selector ) {
				handlers.splice( handlers.delegateCount++, 0, handleObj );
			} else {
				handlers.push( handleObj );
			}

			// Keep track of which events have ever been used, for event optimization
			jQuery.event.global[ type ] = true;
		}

	},

	// Detach an event or set of events from an element
	remove: function( elem, types, handler, selector, mappedTypes ) {

		var j, origCount, tmp,
			events, t, handleObj,
			special, handlers, type, namespaces, origType,
			elemData = data_priv.hasData( elem ) && data_priv.get( elem );

		if ( !elemData || !(events = elemData.events) ) {
			return;
		}

		// Once for each type.namespace in types; type may be omitted
		types = ( types || "" ).match( rnotwhite ) || [ "" ];
		t = types.length;
		while ( t-- ) {
			tmp = rtypenamespace.exec( types[t] ) || [];
			type = origType = tmp[1];
			namespaces = ( tmp[2] || "" ).split( "." ).sort();

			// Unbind all events (on this namespace, if provided) for the element
			if ( !type ) {
				for ( type in events ) {
					jQuery.event.remove( elem, type + types[ t ], handler, selector, true );
				}
				continue;
			}

			special = jQuery.event.special[ type ] || {};
			type = ( selector ? special.delegateType : special.bindType ) || type;
			handlers = events[ type ] || [];
			tmp = tmp[2] && new RegExp( "(^|\\.)" + namespaces.join("\\.(?:.*\\.|)") + "(\\.|$)" );

			// Remove matching events
			origCount = j = handlers.length;
			while ( j-- ) {
				handleObj = handlers[ j ];

				if ( ( mappedTypes || origType === handleObj.origType ) &&
					( !handler || handler.guid === handleObj.guid ) &&
					( !tmp || tmp.test( handleObj.namespace ) ) &&
					( !selector || selector === handleObj.selector || selector === "**" && handleObj.selector ) ) {
					handlers.splice( j, 1 );

					if ( handleObj.selector ) {
						handlers.delegateCount--;
					}
					if ( special.remove ) {
						special.remove.call( elem, handleObj );
					}
				}
			}

			// Remove generic event handler if we removed something and no more handlers exist
			// (avoids potential for endless recursion during removal of special event handlers)
			if ( origCount && !handlers.length ) {
				if ( !special.teardown || special.teardown.call( elem, namespaces, elemData.handle ) === false ) {
					jQuery.removeEvent( elem, type, elemData.handle );
				}

				delete events[ type ];
			}
		}

		// Remove the expando if it's no longer used
		if ( jQuery.isEmptyObject( events ) ) {
			delete elemData.handle;
			data_priv.remove( elem, "events" );
		}
	},

	trigger: function( event, data, elem, onlyHandlers ) {

		var i, cur, tmp, bubbleType, ontype, handle, special,
			eventPath = [ elem || document ],
			type = hasOwn.call( event, "type" ) ? event.type : event,
			namespaces = hasOwn.call( event, "namespace" ) ? event.namespace.split(".") : [];

		cur = tmp = elem = elem || document;

		// Don't do events on text and comment nodes
		if ( elem.nodeType === 3 || elem.nodeType === 8 ) {
			return;
		}

		// focus/blur morphs to focusin/out; ensure we're not firing them right now
		if ( rfocusMorph.test( type + jQuery.event.triggered ) ) {
			return;
		}

		if ( type.indexOf(".") >= 0 ) {
			// Namespaced trigger; create a regexp to match event type in handle()
			namespaces = type.split(".");
			type = namespaces.shift();
			namespaces.sort();
		}
		ontype = type.indexOf(":") < 0 && "on" + type;

		// Caller can pass in a jQuery.Event object, Object, or just an event type string
		event = event[ jQuery.expando ] ?
			event :
			new jQuery.Event( type, typeof event === "object" && event );

		// Trigger bitmask: & 1 for native handlers; & 2 for jQuery (always true)
		event.isTrigger = onlyHandlers ? 2 : 3;
		event.namespace = namespaces.join(".");
		event.namespace_re = event.namespace ?
			new RegExp( "(^|\\.)" + namespaces.join("\\.(?:.*\\.|)") + "(\\.|$)" ) :
			null;

		// Clean up the event in case it is being reused
		event.result = undefined;
		if ( !event.target ) {
			event.target = elem;
		}

		// Clone any incoming data and prepend the event, creating the handler arg list
		data = data == null ?
			[ event ] :
			jQuery.makeArray( data, [ event ] );

		// Allow special events to draw outside the lines
		special = jQuery.event.special[ type ] || {};
		if ( !onlyHandlers && special.trigger && special.trigger.apply( elem, data ) === false ) {
			return;
		}

		// Determine event propagation path in advance, per W3C events spec (#9951)
		// Bubble up to document, then to window; watch for a global ownerDocument var (#9724)
		if ( !onlyHandlers && !special.noBubble && !jQuery.isWindow( elem ) ) {

			bubbleType = special.delegateType || type;
			if ( !rfocusMorph.test( bubbleType + type ) ) {
				cur = cur.parentNode;
			}
			for ( ; cur; cur = cur.parentNode ) {
				eventPath.push( cur );
				tmp = cur;
			}

			// Only add window if we got to document (e.g., not plain obj or detached DOM)
			if ( tmp === (elem.ownerDocument || document) ) {
				eventPath.push( tmp.defaultView || tmp.parentWindow || window );
			}
		}

		// Fire handlers on the event path
		i = 0;
		while ( (cur = eventPath[i++]) && !event.isPropagationStopped() ) {

			event.type = i > 1 ?
				bubbleType :
				special.bindType || type;

			// jQuery handler
			handle = ( data_priv.get( cur, "events" ) || {} )[ event.type ] && data_priv.get( cur, "handle" );
			if ( handle ) {
				handle.apply( cur, data );
			}

			// Native handler
			handle = ontype && cur[ ontype ];
			if ( handle && handle.apply && jQuery.acceptData( cur ) ) {
				event.result = handle.apply( cur, data );
				if ( event.result === false ) {
					event.preventDefault();
				}
			}
		}
		event.type = type;

		// If nobody prevented the default action, do it now
		if ( !onlyHandlers && !event.isDefaultPrevented() ) {

			if ( (!special._default || special._default.apply( eventPath.pop(), data ) === false) &&
				jQuery.acceptData( elem ) ) {

				// Call a native DOM method on the target with the same name name as the event.
				// Don't do default actions on window, that's where global variables be (#6170)
				if ( ontype && jQuery.isFunction( elem[ type ] ) && !jQuery.isWindow( elem ) ) {

					// Don't re-trigger an onFOO event when we call its FOO() method
					tmp = elem[ ontype ];

					if ( tmp ) {
						elem[ ontype ] = null;
					}

					// Prevent re-triggering of the same event, since we already bubbled it above
					jQuery.event.triggered = type;
					elem[ type ]();
					jQuery.event.triggered = undefined;

					if ( tmp ) {
						elem[ ontype ] = tmp;
					}
				}
			}
		}

		return event.result;
	},

	dispatch: function( event ) {

		// Make a writable jQuery.Event from the native event object
		event = jQuery.event.fix( event );

		var i, j, ret, matched, handleObj,
			handlerQueue = [],
			args = slice.call( arguments ),
			handlers = ( data_priv.get( this, "events" ) || {} )[ event.type ] || [],
			special = jQuery.event.special[ event.type ] || {};

		// Use the fix-ed jQuery.Event rather than the (read-only) native event
		args[0] = event;
		event.delegateTarget = this;

		// Call the preDispatch hook for the mapped type, and let it bail if desired
		if ( special.preDispatch && special.preDispatch.call( this, event ) === false ) {
			return;
		}

		// Determine handlers
		handlerQueue = jQuery.event.handlers.call( this, event, handlers );

		// Run delegates first; they may want to stop propagation beneath us
		i = 0;
		while ( (matched = handlerQueue[ i++ ]) && !event.isPropagationStopped() ) {
			event.currentTarget = matched.elem;

			j = 0;
			while ( (handleObj = matched.handlers[ j++ ]) && !event.isImmediatePropagationStopped() ) {

				// Triggered event must either 1) have no namespace, or
				// 2) have namespace(s) a subset or equal to those in the bound event (both can have no namespace).
				if ( !event.namespace_re || event.namespace_re.test( handleObj.namespace ) ) {

					event.handleObj = handleObj;
					event.data = handleObj.data;

					ret = ( (jQuery.event.special[ handleObj.origType ] || {}).handle || handleObj.handler )
							.apply( matched.elem, args );

					if ( ret !== undefined ) {
						if ( (event.result = ret) === false ) {
							event.preventDefault();
							event.stopPropagation();
						}
					}
				}
			}
		}

		// Call the postDispatch hook for the mapped type
		if ( special.postDispatch ) {
			special.postDispatch.call( this, event );
		}

		return event.result;
	},

	handlers: function( event, handlers ) {
		var i, matches, sel, handleObj,
			handlerQueue = [],
			delegateCount = handlers.delegateCount,
			cur = event.target;

		// Find delegate handlers
		// Black-hole SVG <use> instance trees (#13180)
		// Avoid non-left-click bubbling in Firefox (#3861)
		if ( delegateCount && cur.nodeType && (!event.button || event.type !== "click") ) {

			for ( ; cur !== this; cur = cur.parentNode || this ) {

				// Don't process clicks on disabled elements (#6911, #8165, #11382, #11764)
				if ( cur.disabled !== true || event.type !== "click" ) {
					matches = [];
					for ( i = 0; i < delegateCount; i++ ) {
						handleObj = handlers[ i ];

						// Don't conflict with Object.prototype properties (#13203)
						sel = handleObj.selector + " ";

						if ( matches[ sel ] === undefined ) {
							matches[ sel ] = handleObj.needsContext ?
								jQuery( sel, this ).index( cur ) >= 0 :
								jQuery.find( sel, this, null, [ cur ] ).length;
						}
						if ( matches[ sel ] ) {
							matches.push( handleObj );
						}
					}
					if ( matches.length ) {
						handlerQueue.push({ elem: cur, handlers: matches });
					}
				}
			}
		}

		// Add the remaining (directly-bound) handlers
		if ( delegateCount < handlers.length ) {
			handlerQueue.push({ elem: this, handlers: handlers.slice( delegateCount ) });
		}

		return handlerQueue;
	},

	// Includes some event props shared by KeyEvent and MouseEvent
	props: "altKey bubbles cancelable ctrlKey currentTarget eventPhase metaKey relatedTarget shiftKey target timeStamp view which".split(" "),

	fixHooks: {},

	keyHooks: {
		props: "char charCode key keyCode".split(" "),
		filter: function( event, original ) {

			// Add which for key events
			if ( event.which == null ) {
				event.which = original.charCode != null ? original.charCode : original.keyCode;
			}

			return event;
		}
	},

	mouseHooks: {
		props: "button buttons clientX clientY offsetX offsetY pageX pageY screenX screenY toElement".split(" "),
		filter: function( event, original ) {
			var eventDoc, doc, body,
				button = original.button;

			// Calculate pageX/Y if missing and clientX/Y available
			if ( event.pageX == null && original.clientX != null ) {
				eventDoc = event.target.ownerDocument || document;
				doc = eventDoc.documentElement;
				body = eventDoc.body;

				event.pageX = original.clientX + ( doc && doc.scrollLeft || body && body.scrollLeft || 0 ) - ( doc && doc.clientLeft || body && body.clientLeft || 0 );
				event.pageY = original.clientY + ( doc && doc.scrollTop  || body && body.scrollTop  || 0 ) - ( doc && doc.clientTop  || body && body.clientTop  || 0 );
			}

			// Add which for click: 1 === left; 2 === middle; 3 === right
			// Note: button is not normalized, so don't use it
			if ( !event.which && button !== undefined ) {
				event.which = ( button & 1 ? 1 : ( button & 2 ? 3 : ( button & 4 ? 2 : 0 ) ) );
			}

			return event;
		}
	},

	fix: function( event ) {
		if ( event[ jQuery.expando ] ) {
			return event;
		}

		// Create a writable copy of the event object and normalize some properties
		var i, prop, copy,
			type = event.type,
			originalEvent = event,
			fixHook = this.fixHooks[ type ];

		if ( !fixHook ) {
			this.fixHooks[ type ] = fixHook =
				rmouseEvent.test( type ) ? this.mouseHooks :
				rkeyEvent.test( type ) ? this.keyHooks :
				{};
		}
		copy = fixHook.props ? this.props.concat( fixHook.props ) : this.props;

		event = new jQuery.Event( originalEvent );

		i = copy.length;
		while ( i-- ) {
			prop = copy[ i ];
			event[ prop ] = originalEvent[ prop ];
		}

		// Support: Cordova 2.5 (WebKit) (#13255)
		// All events should have a target; Cordova deviceready doesn't
		if ( !event.target ) {
			event.target = document;
		}

		// Support: Safari 6.0+, Chrome < 28
		// Target should not be a text node (#504, #13143)
		if ( event.target.nodeType === 3 ) {
			event.target = event.target.parentNode;
		}

		return fixHook.filter ? fixHook.filter( event, originalEvent ) : event;
	},

	special: {
		load: {
			// Prevent triggered image.load events from bubbling to window.load
			noBubble: true
		},
		focus: {
			// Fire native event if possible so blur/focus sequence is correct
			trigger: function() {
				if ( this !== safeActiveElement() && this.focus ) {
					this.focus();
					return false;
				}
			},
			delegateType: "focusin"
		},
		blur: {
			trigger: function() {
				if ( this === safeActiveElement() && this.blur ) {
					this.blur();
					return false;
				}
			},
			delegateType: "focusout"
		},
		click: {
			// For checkbox, fire native event so checked state will be right
			trigger: function() {
				if ( this.type === "checkbox" && this.click && jQuery.nodeName( this, "input" ) ) {
					this.click();
					return false;
				}
			},

			// For cross-browser consistency, don't fire native .click() on links
			_default: function( event ) {
				return jQuery.nodeName( event.target, "a" );
			}
		},

		beforeunload: {
			postDispatch: function( event ) {

				// Support: Firefox 20+
				// Firefox doesn't alert if the returnValue field is not set.
				if ( event.result !== undefined ) {
					event.originalEvent.returnValue = event.result;
				}
			}
		}
	},

	simulate: function( type, elem, event, bubble ) {
		// Piggyback on a donor event to simulate a different one.
		// Fake originalEvent to avoid donor's stopPropagation, but if the
		// simulated event prevents default then we do the same on the donor.
		var e = jQuery.extend(
			new jQuery.Event(),
			event,
			{
				type: type,
				isSimulated: true,
				originalEvent: {}
			}
		);
		if ( bubble ) {
			jQuery.event.trigger( e, null, elem );
		} else {
			jQuery.event.dispatch.call( elem, e );
		}
		if ( e.isDefaultPrevented() ) {
			event.preventDefault();
		}
	}
};

jQuery.removeEvent = function( elem, type, handle ) {
	if ( elem.removeEventListener ) {
		elem.removeEventListener( type, handle, false );
	}
};

jQuery.Event = function( src, props ) {
	// Allow instantiation without the 'new' keyword
	if ( !(this instanceof jQuery.Event) ) {
		return new jQuery.Event( src, props );
	}

	// Event object
	if ( src && src.type ) {
		this.originalEvent = src;
		this.type = src.type;

		// Events bubbling up the document may have been marked as prevented
		// by a handler lower down the tree; reflect the correct value.
		this.isDefaultPrevented = src.defaultPrevented ||
				// Support: Android < 4.0
				src.defaultPrevented === undefined &&
				src.getPreventDefault && src.getPreventDefault() ?
			returnTrue :
			returnFalse;

	// Event type
	} else {
		this.type = src;
	}

	// Put explicitly provided properties onto the event object
	if ( props ) {
		jQuery.extend( this, props );
	}

	// Create a timestamp if incoming event doesn't have one
	this.timeStamp = src && src.timeStamp || jQuery.now();

	// Mark it as fixed
	this[ jQuery.expando ] = true;
};

// jQuery.Event is based on DOM3 Events as specified by the ECMAScript Language Binding
// http://www.w3.org/TR/2003/WD-DOM-Level-3-Events-20030331/ecma-script-binding.html
jQuery.Event.prototype = {
	isDefaultPrevented: returnFalse,
	isPropagationStopped: returnFalse,
	isImmediatePropagationStopped: returnFalse,

	preventDefault: function() {
		var e = this.originalEvent;

		this.isDefaultPrevented = returnTrue;

		if ( e && e.preventDefault ) {
			e.preventDefault();
		}
	},
	stopPropagation: function() {
		var e = this.originalEvent;

		this.isPropagationStopped = returnTrue;

		if ( e && e.stopPropagation ) {
			e.stopPropagation();
		}
	},
	stopImmediatePropagation: function() {
		this.isImmediatePropagationStopped = returnTrue;
		this.stopPropagation();
	}
};

// Create mouseenter/leave events using mouseover/out and event-time checks
// Support: Chrome 15+
jQuery.each({
	mouseenter: "mouseover",
	mouseleave: "mouseout"
}, function( orig, fix ) {
	jQuery.event.special[ orig ] = {
		delegateType: fix,
		bindType: fix,

		handle: function( event ) {
			var ret,
				target = this,
				related = event.relatedTarget,
				handleObj = event.handleObj;

			// For mousenter/leave call the handler if related is outside the target.
			// NB: No relatedTarget if the mouse left/entered the browser window
			if ( !related || (related !== target && !jQuery.contains( target, related )) ) {
				event.type = handleObj.origType;
				ret = handleObj.handler.apply( this, arguments );
				event.type = fix;
			}
			return ret;
		}
	};
});

// Create "bubbling" focus and blur events
// Support: Firefox, Chrome, Safari
if ( !support.focusinBubbles ) {
	jQuery.each({ focus: "focusin", blur: "focusout" }, function( orig, fix ) {

		// Attach a single capturing handler on the document while someone wants focusin/focusout
		var handler = function( event ) {
				jQuery.event.simulate( fix, event.target, jQuery.event.fix( event ), true );
			};

		jQuery.event.special[ fix ] = {
			setup: function() {
				var doc = this.ownerDocument || this,
					attaches = data_priv.access( doc, fix );

				if ( !attaches ) {
					doc.addEventListener( orig, handler, true );
				}
				data_priv.access( doc, fix, ( attaches || 0 ) + 1 );
			},
			teardown: function() {
				var doc = this.ownerDocument || this,
					attaches = data_priv.access( doc, fix ) - 1;

				if ( !attaches ) {
					doc.removeEventListener( orig, handler, true );
					data_priv.remove( doc, fix );

				} else {
					data_priv.access( doc, fix, attaches );
				}
			}
		};
	});
}

jQuery.fn.extend({

	on: function( types, selector, data, fn, /*INTERNAL*/ one ) {
		var origFn, type;

		// Types can be a map of types/handlers
		if ( typeof types === "object" ) {
			// ( types-Object, selector, data )
			if ( typeof selector !== "string" ) {
				// ( types-Object, data )
				data = data || selector;
				selector = undefined;
			}
			for ( type in types ) {
				this.on( type, selector, data, types[ type ], one );
			}
			return this;
		}

		if ( data == null && fn == null ) {
			// ( types, fn )
			fn = selector;
			data = selector = undefined;
		} else if ( fn == null ) {
			if ( typeof selector === "string" ) {
				// ( types, selector, fn )
				fn = data;
				data = undefined;
			} else {
				// ( types, data, fn )
				fn = data;
				data = selector;
				selector = undefined;
			}
		}
		if ( fn === false ) {
			fn = returnFalse;
		} else if ( !fn ) {
			return this;
		}

		if ( one === 1 ) {
			origFn = fn;
			fn = function( event ) {
				// Can use an empty set, since event contains the info
				jQuery().off( event );
				return origFn.apply( this, arguments );
			};
			// Use same guid so caller can remove using origFn
			fn.guid = origFn.guid || ( origFn.guid = jQuery.guid++ );
		}
		return this.each( function() {
			jQuery.event.add( this, types, fn, data, selector );
		});
	},
	one: function( types, selector, data, fn ) {
		return this.on( types, selector, data, fn, 1 );
	},
	off: function( types, selector, fn ) {
		var handleObj, type;
		if ( types && types.preventDefault && types.handleObj ) {
			// ( event )  dispatched jQuery.Event
			handleObj = types.handleObj;
			jQuery( types.delegateTarget ).off(
				handleObj.namespace ? handleObj.origType + "." + handleObj.namespace : handleObj.origType,
				handleObj.selector,
				handleObj.handler
			);
			return this;
		}
		if ( typeof types === "object" ) {
			// ( types-object [, selector] )
			for ( type in types ) {
				this.off( type, selector, types[ type ] );
			}
			return this;
		}
		if ( selector === false || typeof selector === "function" ) {
			// ( types [, fn] )
			fn = selector;
			selector = undefined;
		}
		if ( fn === false ) {
			fn = returnFalse;
		}
		return this.each(function() {
			jQuery.event.remove( this, types, fn, selector );
		});
	},

	trigger: function( type, data ) {
		return this.each(function() {
			jQuery.event.trigger( type, data, this );
		});
	},
	triggerHandler: function( type, data ) {
		var elem = this[0];
		if ( elem ) {
			return jQuery.event.trigger( type, data, elem, true );
		}
	}
});


var
	rxhtmlTag = /<(?!area|br|col|embed|hr|img|input|link|meta|param)(([\w:]+)[^>]*)\/>/gi,
	rtagName = /<([\w:]+)/,
	rhtml = /<|&#?\w+;/,
	rnoInnerhtml = /<(?:script|style|link)/i,
	// checked="checked" or checked
	rchecked = /checked\s*(?:[^=]|=\s*.checked.)/i,
	rscriptType = /^$|\/(?:java|ecma)script/i,
	rscriptTypeMasked = /^true\/(.*)/,
	rcleanScript = /^\s*<!(?:\[CDATA\[|--)|(?:\]\]|--)>\s*$/g,

	// We have to close these tags to support XHTML (#13200)
	wrapMap = {

		// Support: IE 9
		option: [ 1, "<select multiple='multiple'>", "</select>" ],

		thead: [ 1, "<table>", "</table>" ],
		col: [ 2, "<table><colgroup>", "</colgroup></table>" ],
		tr: [ 2, "<table><tbody>", "</tbody></table>" ],
		td: [ 3, "<table><tbody><tr>", "</tr></tbody></table>" ],

		_default: [ 0, "", "" ]
	};

// Support: IE 9
wrapMap.optgroup = wrapMap.option;

wrapMap.tbody = wrapMap.tfoot = wrapMap.colgroup = wrapMap.caption = wrapMap.thead;
wrapMap.th = wrapMap.td;

// Support: 1.x compatibility
// Manipulating tables requires a tbody
function manipulationTarget( elem, content ) {
	return jQuery.nodeName( elem, "table" ) &&
		jQuery.nodeName( content.nodeType !== 11 ? content : content.firstChild, "tr" ) ?

		elem.getElementsByTagName("tbody")[0] ||
			elem.appendChild( elem.ownerDocument.createElement("tbody") ) :
		elem;
}

// Replace/restore the type attribute of script elements for safe DOM manipulation
function disableScript( elem ) {
	elem.type = (elem.getAttribute("type") !== null) + "/" + elem.type;
	return elem;
}
function restoreScript( elem ) {
	var match = rscriptTypeMasked.exec( elem.type );

	if ( match ) {
		elem.type = match[ 1 ];
	} else {
		elem.removeAttribute("type");
	}

	return elem;
}

// Mark scripts as having already been evaluated
function setGlobalEval( elems, refElements ) {
	var i = 0,
		l = elems.length;

	for ( ; i < l; i++ ) {
		data_priv.set(
			elems[ i ], "globalEval", !refElements || data_priv.get( refElements[ i ], "globalEval" )
		);
	}
}

function cloneCopyEvent( src, dest ) {
	var i, l, type, pdataOld, pdataCur, udataOld, udataCur, events;

	if ( dest.nodeType !== 1 ) {
		return;
	}

	// 1. Copy private data: events, handlers, etc.
	if ( data_priv.hasData( src ) ) {
		pdataOld = data_priv.access( src );
		pdataCur = data_priv.set( dest, pdataOld );
		events = pdataOld.events;

		if ( events ) {
			delete pdataCur.handle;
			pdataCur.events = {};

			for ( type in events ) {
				for ( i = 0, l = events[ type ].length; i < l; i++ ) {
					jQuery.event.add( dest, type, events[ type ][ i ] );
				}
			}
		}
	}

	// 2. Copy user data
	if ( data_user.hasData( src ) ) {
		udataOld = data_user.access( src );
		udataCur = jQuery.extend( {}, udataOld );

		data_user.set( dest, udataCur );
	}
}

function getAll( context, tag ) {
	var ret = context.getElementsByTagName ? context.getElementsByTagName( tag || "*" ) :
			context.querySelectorAll ? context.querySelectorAll( tag || "*" ) :
			[];

	return tag === undefined || tag && jQuery.nodeName( context, tag ) ?
		jQuery.merge( [ context ], ret ) :
		ret;
}

// Support: IE >= 9
function fixInput( src, dest ) {
	var nodeName = dest.nodeName.toLowerCase();

	// Fails to persist the checked state of a cloned checkbox or radio button.
	if ( nodeName === "input" && rcheckableType.test( src.type ) ) {
		dest.checked = src.checked;

	// Fails to return the selected option to the default selected state when cloning options
	} else if ( nodeName === "input" || nodeName === "textarea" ) {
		dest.defaultValue = src.defaultValue;
	}
}

jQuery.extend({
	clone: function( elem, dataAndEvents, deepDataAndEvents ) {
		var i, l, srcElements, destElements,
			clone = elem.cloneNode( true ),
			inPage = jQuery.contains( elem.ownerDocument, elem );

		// Support: IE >= 9
		// Fix Cloning issues
		if ( !support.noCloneChecked && ( elem.nodeType === 1 || elem.nodeType === 11 ) &&
				!jQuery.isXMLDoc( elem ) ) {

			// We eschew Sizzle here for performance reasons: http://jsperf.com/getall-vs-sizzle/2
			destElements = getAll( clone );
			srcElements = getAll( elem );

			for ( i = 0, l = srcElements.length; i < l; i++ ) {
				fixInput( srcElements[ i ], destElements[ i ] );
			}
		}

		// Copy the events from the original to the clone
		if ( dataAndEvents ) {
			if ( deepDataAndEvents ) {
				srcElements = srcElements || getAll( elem );
				destElements = destElements || getAll( clone );

				for ( i = 0, l = srcElements.length; i < l; i++ ) {
					cloneCopyEvent( srcElements[ i ], destElements[ i ] );
				}
			} else {
				cloneCopyEvent( elem, clone );
			}
		}

		// Preserve script evaluation history
		destElements = getAll( clone, "script" );
		if ( destElements.length > 0 ) {
			setGlobalEval( destElements, !inPage && getAll( elem, "script" ) );
		}

		// Return the cloned set
		return clone;
	},

	buildFragment: function( elems, context, scripts, selection ) {
		var elem, tmp, tag, wrap, contains, j,
			fragment = context.createDocumentFragment(),
			nodes = [],
			i = 0,
			l = elems.length;

		for ( ; i < l; i++ ) {
			elem = elems[ i ];

			if ( elem || elem === 0 ) {

				// Add nodes directly
				if ( jQuery.type( elem ) === "object" ) {
					// Support: QtWebKit
					// jQuery.merge because push.apply(_, arraylike) throws
					jQuery.merge( nodes, elem.nodeType ? [ elem ] : elem );

				// Convert non-html into a text node
				} else if ( !rhtml.test( elem ) ) {
					nodes.push( context.createTextNode( elem ) );

				// Convert html into DOM nodes
				} else {
					tmp = tmp || fragment.appendChild( context.createElement("div") );

					// Deserialize a standard representation
					tag = ( rtagName.exec( elem ) || [ "", "" ] )[ 1 ].toLowerCase();
					wrap = wrapMap[ tag ] || wrapMap._default;
					tmp.innerHTML = wrap[ 1 ] + elem.replace( rxhtmlTag, "<$1></$2>" ) + wrap[ 2 ];

					// Descend through wrappers to the right content
					j = wrap[ 0 ];
					while ( j-- ) {
						tmp = tmp.lastChild;
					}

					// Support: QtWebKit
					// jQuery.merge because push.apply(_, arraylike) throws
					jQuery.merge( nodes, tmp.childNodes );

					// Remember the top-level container
					tmp = fragment.firstChild;

					// Fixes #12346
					// Support: Webkit, IE
					tmp.textContent = "";
				}
			}
		}

		// Remove wrapper from fragment
		fragment.textContent = "";

		i = 0;
		while ( (elem = nodes[ i++ ]) ) {

			// #4087 - If origin and destination elements are the same, and this is
			// that element, do not do anything
			if ( selection && jQuery.inArray( elem, selection ) !== -1 ) {
				continue;
			}

			contains = jQuery.contains( elem.ownerDocument, elem );

			// Append to fragment
			tmp = getAll( fragment.appendChild( elem ), "script" );

			// Preserve script evaluation history
			if ( contains ) {
				setGlobalEval( tmp );
			}

			// Capture executables
			if ( scripts ) {
				j = 0;
				while ( (elem = tmp[ j++ ]) ) {
					if ( rscriptType.test( elem.type || "" ) ) {
						scripts.push( elem );
					}
				}
			}
		}

		return fragment;
	},

	cleanData: function( elems ) {
		var data, elem, events, type, key, j,
			special = jQuery.event.special,
			i = 0;

		for ( ; (elem = elems[ i ]) !== undefined; i++ ) {
			if ( jQuery.acceptData( elem ) ) {
				key = elem[ data_priv.expando ];

				if ( key && (data = data_priv.cache[ key ]) ) {
					events = Object.keys( data.events || {} );
					if ( events.length ) {
						for ( j = 0; (type = events[j]) !== undefined; j++ ) {
							if ( special[ type ] ) {
								jQuery.event.remove( elem, type );

							// This is a shortcut to avoid jQuery.event.remove's overhead
							} else {
								jQuery.removeEvent( elem, type, data.handle );
							}
						}
					}
					if ( data_priv.cache[ key ] ) {
						// Discard any remaining `private` data
						delete data_priv.cache[ key ];
					}
				}
			}
			// Discard any remaining `user` data
			delete data_user.cache[ elem[ data_user.expando ] ];
		}
	}
});

jQuery.fn.extend({
	text: function( value ) {
		return access( this, function( value ) {
			return value === undefined ?
				jQuery.text( this ) :
				this.empty().each(function() {
					if ( this.nodeType === 1 || this.nodeType === 11 || this.nodeType === 9 ) {
						this.textContent = value;
					}
				});
		}, null, value, arguments.length );
	},

	append: function() {
		return this.domManip( arguments, function( elem ) {
			if ( this.nodeType === 1 || this.nodeType === 11 || this.nodeType === 9 ) {
				var target = manipulationTarget( this, elem );
				target.appendChild( elem );
			}
		});
	},

	prepend: function() {
		return this.domManip( arguments, function( elem ) {
			if ( this.nodeType === 1 || this.nodeType === 11 || this.nodeType === 9 ) {
				var target = manipulationTarget( this, elem );
				target.insertBefore( elem, target.firstChild );
			}
		});
	},

	before: function() {
		return this.domManip( arguments, function( elem ) {
			if ( this.parentNode ) {
				this.parentNode.insertBefore( elem, this );
			}
		});
	},

	after: function() {
		return this.domManip( arguments, function( elem ) {
			if ( this.parentNode ) {
				this.parentNode.insertBefore( elem, this.nextSibling );
			}
		});
	},

	remove: function( selector, keepData /* Internal Use Only */ ) {
		var elem,
			elems = selector ? jQuery.filter( selector, this ) : this,
			i = 0;

		for ( ; (elem = elems[i]) != null; i++ ) {
			if ( !keepData && elem.nodeType === 1 ) {
				jQuery.cleanData( getAll( elem ) );
			}

			if ( elem.parentNode ) {
				if ( keepData && jQuery.contains( elem.ownerDocument, elem ) ) {
					setGlobalEval( getAll( elem, "script" ) );
				}
				elem.parentNode.removeChild( elem );
			}
		}

		return this;
	},

	empty: function() {
		var elem,
			i = 0;

		for ( ; (elem = this[i]) != null; i++ ) {
			if ( elem.nodeType === 1 ) {

				// Prevent memory leaks
				jQuery.cleanData( getAll( elem, false ) );

				// Remove any remaining nodes
				elem.textContent = "";
			}
		}

		return this;
	},

	clone: function( dataAndEvents, deepDataAndEvents ) {
		dataAndEvents = dataAndEvents == null ? false : dataAndEvents;
		deepDataAndEvents = deepDataAndEvents == null ? dataAndEvents : deepDataAndEvents;

		return this.map(function() {
			return jQuery.clone( this, dataAndEvents, deepDataAndEvents );
		});
	},

	html: function( value ) {
		return access( this, function( value ) {
			var elem = this[ 0 ] || {},
				i = 0,
				l = this.length;

			if ( value === undefined && elem.nodeType === 1 ) {
				return elem.innerHTML;
			}

			// See if we can take a shortcut and just use innerHTML
			if ( typeof value === "string" && !rnoInnerhtml.test( value ) &&
				!wrapMap[ ( rtagName.exec( value ) || [ "", "" ] )[ 1 ].toLowerCase() ] ) {

				value = value.replace( rxhtmlTag, "<$1></$2>" );

				try {
					for ( ; i < l; i++ ) {
						elem = this[ i ] || {};

						// Remove element nodes and prevent memory leaks
						if ( elem.nodeType === 1 ) {
							jQuery.cleanData( getAll( elem, false ) );
							elem.innerHTML = value;
						}
					}

					elem = 0;

				// If using innerHTML throws an exception, use the fallback method
				} catch( e ) {}
			}

			if ( elem ) {
				this.empty().append( value );
			}
		}, null, value, arguments.length );
	},

	replaceWith: function() {
		var arg = arguments[ 0 ];

		// Make the changes, replacing each context element with the new content
		this.domManip( arguments, function( elem ) {
			arg = this.parentNode;

			jQuery.cleanData( getAll( this ) );

			if ( arg ) {
				arg.replaceChild( elem, this );
			}
		});

		// Force removal if there was no new content (e.g., from empty arguments)
		return arg && (arg.length || arg.nodeType) ? this : this.remove();
	},

	detach: function( selector ) {
		return this.remove( selector, true );
	},

	domManip: function( args, callback ) {

		// Flatten any nested arrays
		args = concat.apply( [], args );

		var fragment, first, scripts, hasScripts, node, doc,
			i = 0,
			l = this.length,
			set = this,
			iNoClone = l - 1,
			value = args[ 0 ],
			isFunction = jQuery.isFunction( value );

		// We can't cloneNode fragments that contain checked, in WebKit
		if ( isFunction ||
				( l > 1 && typeof value === "string" &&
					!support.checkClone && rchecked.test( value ) ) ) {
			return this.each(function( index ) {
				var self = set.eq( index );
				if ( isFunction ) {
					args[ 0 ] = value.call( this, index, self.html() );
				}
				self.domManip( args, callback );
			});
		}

		if ( l ) {
			fragment = jQuery.buildFragment( args, this[ 0 ].ownerDocument, false, this );
			first = fragment.firstChild;

			if ( fragment.childNodes.length === 1 ) {
				fragment = first;
			}

			if ( first ) {
				scripts = jQuery.map( getAll( fragment, "script" ), disableScript );
				hasScripts = scripts.length;

				// Use the original fragment for the last item instead of the first because it can end up
				// being emptied incorrectly in certain situations (#8070).
				for ( ; i < l; i++ ) {
					node = fragment;

					if ( i !== iNoClone ) {
						node = jQuery.clone( node, true, true );

						// Keep references to cloned scripts for later restoration
						if ( hasScripts ) {
							// Support: QtWebKit
							// jQuery.merge because push.apply(_, arraylike) throws
							jQuery.merge( scripts, getAll( node, "script" ) );
						}
					}

					callback.call( this[ i ], node, i );
				}

				if ( hasScripts ) {
					doc = scripts[ scripts.length - 1 ].ownerDocument;

					// Reenable scripts
					jQuery.map( scripts, restoreScript );

					// Evaluate executable scripts on first document insertion
					for ( i = 0; i < hasScripts; i++ ) {
						node = scripts[ i ];
						if ( rscriptType.test( node.type || "" ) &&
							!data_priv.access( node, "globalEval" ) && jQuery.contains( doc, node ) ) {

							if ( node.src ) {
								// Optional AJAX dependency, but won't run scripts if not present
								if ( jQuery._evalUrl ) {
									jQuery._evalUrl( node.src );
								}
							} else {
								jQuery.globalEval( node.textContent.replace( rcleanScript, "" ) );
							}
						}
					}
				}
			}
		}

		return this;
	}
});

jQuery.each({
	appendTo: "append",
	prependTo: "prepend",
	insertBefore: "before",
	insertAfter: "after",
	replaceAll: "replaceWith"
}, function( name, original ) {
	jQuery.fn[ name ] = function( selector ) {
		var elems,
			ret = [],
			insert = jQuery( selector ),
			last = insert.length - 1,
			i = 0;

		for ( ; i <= last; i++ ) {
			elems = i === last ? this : this.clone( true );
			jQuery( insert[ i ] )[ original ]( elems );

			// Support: QtWebKit
			// .get() because push.apply(_, arraylike) throws
			push.apply( ret, elems.get() );
		}

		return this.pushStack( ret );
	};
});


var iframe,
	elemdisplay = {};

/**
 * Retrieve the actual display of a element
 * @param {String} name nodeName of the element
 * @param {Object} doc Document object
 */
// Called only from within defaultDisplay
function actualDisplay( name, doc ) {
	var elem = jQuery( doc.createElement( name ) ).appendTo( doc.body ),

		// getDefaultComputedStyle might be reliably used only on attached element
		display = window.getDefaultComputedStyle ?

			// Use of this method is a temporary fix (more like optmization) until something better comes along,
			// since it was removed from specification and supported only in FF
			window.getDefaultComputedStyle( elem[ 0 ] ).display : jQuery.css( elem[ 0 ], "display" );

	// We don't have any data stored on the element,
	// so use "detach" method as fast way to get rid of the element
	elem.detach();

	return display;
}

/**
 * Try to determine the default display value of an element
 * @param {String} nodeName
 */
function defaultDisplay( nodeName ) {
	var doc = document,
		display = elemdisplay[ nodeName ];

	if ( !display ) {
		display = actualDisplay( nodeName, doc );

		// If the simple way fails, read from inside an iframe
		if ( display === "none" || !display ) {

			// Use the already-created iframe if possible
			iframe = (iframe || jQuery( "<iframe frameborder='0' width='0' height='0'/>" )).appendTo( doc.documentElement );

			// Always write a new HTML skeleton so Webkit and Firefox don't choke on reuse
			doc = iframe[ 0 ].contentDocument;

			// Support: IE
			doc.write();
			doc.close();

			display = actualDisplay( nodeName, doc );
			iframe.detach();
		}

		// Store the correct default display
		elemdisplay[ nodeName ] = display;
	}

	return display;
}
var rmargin = (/^margin/);

var rnumnonpx = new RegExp( "^(" + pnum + ")(?!px)[a-z%]+$", "i" );

var getStyles = function( elem ) {
		return elem.ownerDocument.defaultView.getComputedStyle( elem, null );
	};



function curCSS( elem, name, computed ) {
	var width, minWidth, maxWidth, ret,
		style = elem.style;

	computed = computed || getStyles( elem );

	// Support: IE9
	// getPropertyValue is only needed for .css('filter') in IE9, see #12537
	if ( computed ) {
		ret = computed.getPropertyValue( name ) || computed[ name ];
	}

	if ( computed ) {

		if ( ret === "" && !jQuery.contains( elem.ownerDocument, elem ) ) {
			ret = jQuery.style( elem, name );
		}

		// Support: iOS < 6
		// A tribute to the "awesome hack by Dean Edwards"
		// iOS < 6 (at least) returns percentage for a larger set of values, but width seems to be reliably pixels
		// this is against the CSSOM draft spec: http://dev.w3.org/csswg/cssom/#resolved-values
		if ( rnumnonpx.test( ret ) && rmargin.test( name ) ) {

			// Remember the original values
			width = style.width;
			minWidth = style.minWidth;
			maxWidth = style.maxWidth;

			// Put in the new values to get a computed value out
			style.minWidth = style.maxWidth = style.width = ret;
			ret = computed.width;

			// Revert the changed values
			style.width = width;
			style.minWidth = minWidth;
			style.maxWidth = maxWidth;
		}
	}

	return ret !== undefined ?
		// Support: IE
		// IE returns zIndex value as an integer.
		ret + "" :
		ret;
}


function addGetHookIf( conditionFn, hookFn ) {
	// Define the hook, we'll check on the first run if it's really needed.
	return {
		get: function() {
			if ( conditionFn() ) {
				// Hook not needed (or it's not possible to use it due to missing dependency),
				// remove it.
				// Since there are no other hooks for marginRight, remove the whole object.
				delete this.get;
				return;
			}

			// Hook needed; redefine it so that the support test is not executed again.

			return (this.get = hookFn).apply( this, arguments );
		}
	};
}


(function() {
	var pixelPositionVal, boxSizingReliableVal,
		// Support: Firefox, Android 2.3 (Prefixed box-sizing versions).
		divReset = "padding:0;margin:0;border:0;display:block;-webkit-box-sizing:content-box;" +
			"-moz-box-sizing:content-box;box-sizing:content-box",
		docElem = document.documentElement,
		container = document.createElement( "div" ),
		div = document.createElement( "div" );

	div.style.backgroundClip = "content-box";
	div.cloneNode( true ).style.backgroundClip = "";
	support.clearCloneStyle = div.style.backgroundClip === "content-box";

	container.style.cssText = "border:0;width:0;height:0;position:absolute;top:0;left:-9999px;" +
		"margin-top:1px";
	container.appendChild( div );

	// Executing both pixelPosition & boxSizingReliable tests require only one layout
	// so they're executed at the same time to save the second computation.
	function computePixelPositionAndBoxSizingReliable() {
		// Support: Firefox, Android 2.3 (Prefixed box-sizing versions).
		div.style.cssText = "-webkit-box-sizing:border-box;-moz-box-sizing:border-box;" +
			"box-sizing:border-box;padding:1px;border:1px;display:block;width:4px;margin-top:1%;" +
			"position:absolute;top:1%";
		docElem.appendChild( container );

		var divStyle = window.getComputedStyle( div, null );
		pixelPositionVal = divStyle.top !== "1%";
		boxSizingReliableVal = divStyle.width === "4px";

		docElem.removeChild( container );
	}

	// Use window.getComputedStyle because jsdom on node.js will break without it.
	if ( window.getComputedStyle ) {
		jQuery.extend(support, {
			pixelPosition: function() {
				// This test is executed only once but we still do memoizing
				// since we can use the boxSizingReliable pre-computing.
				// No need to check if the test was already performed, though.
				computePixelPositionAndBoxSizingReliable();
				return pixelPositionVal;
			},
			boxSizingReliable: function() {
				if ( boxSizingReliableVal == null ) {
					computePixelPositionAndBoxSizingReliable();
				}
				return boxSizingReliableVal;
			},
			reliableMarginRight: function() {
				// Support: Android 2.3
				// Check if div with explicit width and no margin-right incorrectly
				// gets computed margin-right based on width of container. (#3333)
				// WebKit Bug 13343 - getComputedStyle returns wrong value for margin-right
				// This support function is only executed once so no memoizing is needed.
				var ret,
					marginDiv = div.appendChild( document.createElement( "div" ) );
				marginDiv.style.cssText = div.style.cssText = divReset;
				marginDiv.style.marginRight = marginDiv.style.width = "0";
				div.style.width = "1px";
				docElem.appendChild( container );

				ret = !parseFloat( window.getComputedStyle( marginDiv, null ).marginRight );

				docElem.removeChild( container );

				// Clean up the div for other support tests.
				div.innerHTML = "";

				return ret;
			}
		});
	}
})();


// A method for quickly swapping in/out CSS properties to get correct calculations.
jQuery.swap = function( elem, options, callback, args ) {
	var ret, name,
		old = {};

	// Remember the old values, and insert the new ones
	for ( name in options ) {
		old[ name ] = elem.style[ name ];
		elem.style[ name ] = options[ name ];
	}

	ret = callback.apply( elem, args || [] );

	// Revert the old values
	for ( name in options ) {
		elem.style[ name ] = old[ name ];
	}

	return ret;
};


var
	// swappable if display is none or starts with table except "table", "table-cell", or "table-caption"
	// see here for display values: https://developer.mozilla.org/en-US/docs/CSS/display
	rdisplayswap = /^(none|table(?!-c[ea]).+)/,
	rnumsplit = new RegExp( "^(" + pnum + ")(.*)$", "i" ),
	rrelNum = new RegExp( "^([+-])=(" + pnum + ")", "i" ),

	cssShow = { position: "absolute", visibility: "hidden", display: "block" },
	cssNormalTransform = {
		letterSpacing: 0,
		fontWeight: 400
	},

	cssPrefixes = [ "Webkit", "O", "Moz", "ms" ];

// return a css property mapped to a potentially vendor prefixed property
function vendorPropName( style, name ) {

	// shortcut for names that are not vendor prefixed
	if ( name in style ) {
		return name;
	}

	// check for vendor prefixed names
	var capName = name[0].toUpperCase() + name.slice(1),
		origName = name,
		i = cssPrefixes.length;

	while ( i-- ) {
		name = cssPrefixes[ i ] + capName;
		if ( name in style ) {
			return name;
		}
	}

	return origName;
}

function setPositiveNumber( elem, value, subtract ) {
	var matches = rnumsplit.exec( value );
	return matches ?
		// Guard against undefined "subtract", e.g., when used as in cssHooks
		Math.max( 0, matches[ 1 ] - ( subtract || 0 ) ) + ( matches[ 2 ] || "px" ) :
		value;
}

function augmentWidthOrHeight( elem, name, extra, isBorderBox, styles ) {
	var i = extra === ( isBorderBox ? "border" : "content" ) ?
		// If we already have the right measurement, avoid augmentation
		4 :
		// Otherwise initialize for horizontal or vertical properties
		name === "width" ? 1 : 0,

		val = 0;

	for ( ; i < 4; i += 2 ) {
		// both box models exclude margin, so add it if we want it
		if ( extra === "margin" ) {
			val += jQuery.css( elem, extra + cssExpand[ i ], true, styles );
		}

		if ( isBorderBox ) {
			// border-box includes padding, so remove it if we want content
			if ( extra === "content" ) {
				val -= jQuery.css( elem, "padding" + cssExpand[ i ], true, styles );
			}

			// at this point, extra isn't border nor margin, so remove border
			if ( extra !== "margin" ) {
				val -= jQuery.css( elem, "border" + cssExpand[ i ] + "Width", true, styles );
			}
		} else {
			// at this point, extra isn't content, so add padding
			val += jQuery.css( elem, "padding" + cssExpand[ i ], true, styles );

			// at this point, extra isn't content nor padding, so add border
			if ( extra !== "padding" ) {
				val += jQuery.css( elem, "border" + cssExpand[ i ] + "Width", true, styles );
			}
		}
	}

	return val;
}

function getWidthOrHeight( elem, name, extra ) {

	// Start with offset property, which is equivalent to the border-box value
	var valueIsBorderBox = true,
		val = name === "width" ? elem.offsetWidth : elem.offsetHeight,
		styles = getStyles( elem ),
		isBorderBox = jQuery.css( elem, "boxSizing", false, styles ) === "border-box";

	// some non-html elements return undefined for offsetWidth, so check for null/undefined
	// svg - https://bugzilla.mozilla.org/show_bug.cgi?id=649285
	// MathML - https://bugzilla.mozilla.org/show_bug.cgi?id=491668
	if ( val <= 0 || val == null ) {
		// Fall back to computed then uncomputed css if necessary
		val = curCSS( elem, name, styles );
		if ( val < 0 || val == null ) {
			val = elem.style[ name ];
		}

		// Computed unit is not pixels. Stop here and return.
		if ( rnumnonpx.test(val) ) {
			return val;
		}

		// we need the check for style in case a browser which returns unreliable values
		// for getComputedStyle silently falls back to the reliable elem.style
		valueIsBorderBox = isBorderBox &&
			( support.boxSizingReliable() || val === elem.style[ name ] );

		// Normalize "", auto, and prepare for extra
		val = parseFloat( val ) || 0;
	}

	// use the active box-sizing model to add/subtract irrelevant styles
	return ( val +
		augmentWidthOrHeight(
			elem,
			name,
			extra || ( isBorderBox ? "border" : "content" ),
			valueIsBorderBox,
			styles
		)
	) + "px";
}

function showHide( elements, show ) {
	var display, elem, hidden,
		values = [],
		index = 0,
		length = elements.length;

	for ( ; index < length; index++ ) {
		elem = elements[ index ];
		if ( !elem.style ) {
			continue;
		}

		values[ index ] = data_priv.get( elem, "olddisplay" );
		display = elem.style.display;
		if ( show ) {
			// Reset the inline display of this element to learn if it is
			// being hidden by cascaded rules or not
			if ( !values[ index ] && display === "none" ) {
				elem.style.display = "";
			}

			// Set elements which have been overridden with display: none
			// in a stylesheet to whatever the default browser style is
			// for such an element
			if ( elem.style.display === "" && isHidden( elem ) ) {
				values[ index ] = data_priv.access( elem, "olddisplay", defaultDisplay(elem.nodeName) );
			}
		} else {

			if ( !values[ index ] ) {
				hidden = isHidden( elem );

				if ( display && display !== "none" || !hidden ) {
					data_priv.set( elem, "olddisplay", hidden ? display : jQuery.css(elem, "display") );
				}
			}
		}
	}

	// Set the display of most of the elements in a second loop
	// to avoid the constant reflow
	for ( index = 0; index < length; index++ ) {
		elem = elements[ index ];
		if ( !elem.style ) {
			continue;
		}
		if ( !show || elem.style.display === "none" || elem.style.display === "" ) {
			elem.style.display = show ? values[ index ] || "" : "none";
		}
	}

	return elements;
}

jQuery.extend({
	// Add in style property hooks for overriding the default
	// behavior of getting and setting a style property
	cssHooks: {
		opacity: {
			get: function( elem, computed ) {
				if ( computed ) {
					// We should always get a number back from opacity
					var ret = curCSS( elem, "opacity" );
					return ret === "" ? "1" : ret;
				}
			}
		}
	},

	// Don't automatically add "px" to these possibly-unitless properties
	cssNumber: {
		"columnCount": true,
		"fillOpacity": true,
		"fontWeight": true,
		"lineHeight": true,
		"opacity": true,
		"order": true,
		"orphans": true,
		"widows": true,
		"zIndex": true,
		"zoom": true
	},

	// Add in properties whose names you wish to fix before
	// setting or getting the value
	cssProps: {
		// normalize float css property
		"float": "cssFloat"
	},

	// Get and set the style property on a DOM Node
	style: function( elem, name, value, extra ) {
		// Don't set styles on text and comment nodes
		if ( !elem || elem.nodeType === 3 || elem.nodeType === 8 || !elem.style ) {
			return;
		}

		// Make sure that we're working with the right name
		var ret, type, hooks,
			origName = jQuery.camelCase( name ),
			style = elem.style;

		name = jQuery.cssProps[ origName ] || ( jQuery.cssProps[ origName ] = vendorPropName( style, origName ) );

		// gets hook for the prefixed version
		// followed by the unprefixed version
		hooks = jQuery.cssHooks[ name ] || jQuery.cssHooks[ origName ];

		// Check if we're setting a value
		if ( value !== undefined ) {
			type = typeof value;

			// convert relative number strings (+= or -=) to relative numbers. #7345
			if ( type === "string" && (ret = rrelNum.exec( value )) ) {
				value = ( ret[1] + 1 ) * ret[2] + parseFloat( jQuery.css( elem, name ) );
				// Fixes bug #9237
				type = "number";
			}

			// Make sure that null and NaN values aren't set. See: #7116
			if ( value == null || value !== value ) {
				return;
			}

			// If a number was passed in, add 'px' to the (except for certain CSS properties)
			if ( type === "number" && !jQuery.cssNumber[ origName ] ) {
				value += "px";
			}

			// Fixes #8908, it can be done more correctly by specifying setters in cssHooks,
			// but it would mean to define eight (for every problematic property) identical functions
			if ( !support.clearCloneStyle && value === "" && name.indexOf( "background" ) === 0 ) {
				style[ name ] = "inherit";
			}

			// If a hook was provided, use that value, otherwise just set the specified value
			if ( !hooks || !("set" in hooks) || (value = hooks.set( elem, value, extra )) !== undefined ) {
				// Support: Chrome, Safari
				// Setting style to blank string required to delete "style: x !important;"
				style[ name ] = "";
				style[ name ] = value;
			}

		} else {
			// If a hook was provided get the non-computed value from there
			if ( hooks && "get" in hooks && (ret = hooks.get( elem, false, extra )) !== undefined ) {
				return ret;
			}

			// Otherwise just get the value from the style object
			return style[ name ];
		}
	},

	css: function( elem, name, extra, styles ) {
		var val, num, hooks,
			origName = jQuery.camelCase( name );

		// Make sure that we're working with the right name
		name = jQuery.cssProps[ origName ] || ( jQuery.cssProps[ origName ] = vendorPropName( elem.style, origName ) );

		// gets hook for the prefixed version
		// followed by the unprefixed version
		hooks = jQuery.cssHooks[ name ] || jQuery.cssHooks[ origName ];

		// If a hook was provided get the computed value from there
		if ( hooks && "get" in hooks ) {
			val = hooks.get( elem, true, extra );
		}

		// Otherwise, if a way to get the computed value exists, use that
		if ( val === undefined ) {
			val = curCSS( elem, name, styles );
		}

		//convert "normal" to computed value
		if ( val === "normal" && name in cssNormalTransform ) {
			val = cssNormalTransform[ name ];
		}

		// Return, converting to number if forced or a qualifier was provided and val looks numeric
		if ( extra === "" || extra ) {
			num = parseFloat( val );
			return extra === true || jQuery.isNumeric( num ) ? num || 0 : val;
		}
		return val;
	}
});

jQuery.each([ "height", "width" ], function( i, name ) {
	jQuery.cssHooks[ name ] = {
		get: function( elem, computed, extra ) {
			if ( computed ) {
				// certain elements can have dimension info if we invisibly show them
				// however, it must have a current display style that would benefit from this
				return elem.offsetWidth === 0 && rdisplayswap.test( jQuery.css( elem, "display" ) ) ?
					jQuery.swap( elem, cssShow, function() {
						return getWidthOrHeight( elem, name, extra );
					}) :
					getWidthOrHeight( elem, name, extra );
			}
		},

		set: function( elem, value, extra ) {
			var styles = extra && getStyles( elem );
			return setPositiveNumber( elem, value, extra ?
				augmentWidthOrHeight(
					elem,
					name,
					extra,
					jQuery.css( elem, "boxSizing", false, styles ) === "border-box",
					styles
				) : 0
			);
		}
	};
});

// Support: Android 2.3
jQuery.cssHooks.marginRight = addGetHookIf( support.reliableMarginRight,
	function( elem, computed ) {
		if ( computed ) {
			// WebKit Bug 13343 - getComputedStyle returns wrong value for margin-right
			// Work around by temporarily setting element display to inline-block
			return jQuery.swap( elem, { "display": "inline-block" },
				curCSS, [ elem, "marginRight" ] );
		}
	}
);

// These hooks are used by animate to expand properties
jQuery.each({
	margin: "",
	padding: "",
	border: "Width"
}, function( prefix, suffix ) {
	jQuery.cssHooks[ prefix + suffix ] = {
		expand: function( value ) {
			var i = 0,
				expanded = {},

				// assumes a single number if not a string
				parts = typeof value === "string" ? value.split(" ") : [ value ];

			for ( ; i < 4; i++ ) {
				expanded[ prefix + cssExpand[ i ] + suffix ] =
					parts[ i ] || parts[ i - 2 ] || parts[ 0 ];
			}

			return expanded;
		}
	};

	if ( !rmargin.test( prefix ) ) {
		jQuery.cssHooks[ prefix + suffix ].set = setPositiveNumber;
	}
});

jQuery.fn.extend({
	css: function( name, value ) {
		return access( this, function( elem, name, value ) {
			var styles, len,
				map = {},
				i = 0;

			if ( jQuery.isArray( name ) ) {
				styles = getStyles( elem );
				len = name.length;

				for ( ; i < len; i++ ) {
					map[ name[ i ] ] = jQuery.css( elem, name[ i ], false, styles );
				}

				return map;
			}

			return value !== undefined ?
				jQuery.style( elem, name, value ) :
				jQuery.css( elem, name );
		}, name, value, arguments.length > 1 );
	},
	show: function() {
		return showHide( this, true );
	},
	hide: function() {
		return showHide( this );
	},
	toggle: function( state ) {
		if ( typeof state === "boolean" ) {
			return state ? this.show() : this.hide();
		}

		return this.each(function() {
			if ( isHidden( this ) ) {
				jQuery( this ).show();
			} else {
				jQuery( this ).hide();
			}
		});
	}
});


function Tween( elem, options, prop, end, easing ) {
	return new Tween.prototype.init( elem, options, prop, end, easing );
}
jQuery.Tween = Tween;

Tween.prototype = {
	constructor: Tween,
	init: function( elem, options, prop, end, easing, unit ) {
		this.elem = elem;
		this.prop = prop;
		this.easing = easing || "swing";
		this.options = options;
		this.start = this.now = this.cur();
		this.end = end;
		this.unit = unit || ( jQuery.cssNumber[ prop ] ? "" : "px" );
	},
	cur: function() {
		var hooks = Tween.propHooks[ this.prop ];

		return hooks && hooks.get ?
			hooks.get( this ) :
			Tween.propHooks._default.get( this );
	},
	run: function( percent ) {
		var eased,
			hooks = Tween.propHooks[ this.prop ];

		if ( this.options.duration ) {
			this.pos = eased = jQuery.easing[ this.easing ](
				percent, this.options.duration * percent, 0, 1, this.options.duration
			);
		} else {
			this.pos = eased = percent;
		}
		this.now = ( this.end - this.start ) * eased + this.start;

		if ( this.options.step ) {
			this.options.step.call( this.elem, this.now, this );
		}

		if ( hooks && hooks.set ) {
			hooks.set( this );
		} else {
			Tween.propHooks._default.set( this );
		}
		return this;
	}
};

Tween.prototype.init.prototype = Tween.prototype;

Tween.propHooks = {
	_default: {
		get: function( tween ) {
			var result;

			if ( tween.elem[ tween.prop ] != null &&
				(!tween.elem.style || tween.elem.style[ tween.prop ] == null) ) {
				return tween.elem[ tween.prop ];
			}

			// passing an empty string as a 3rd parameter to .css will automatically
			// attempt a parseFloat and fallback to a string if the parse fails
			// so, simple values such as "10px" are parsed to Float.
			// complex values such as "rotate(1rad)" are returned as is.
			result = jQuery.css( tween.elem, tween.prop, "" );
			// Empty strings, null, undefined and "auto" are converted to 0.
			return !result || result === "auto" ? 0 : result;
		},
		set: function( tween ) {
			// use step hook for back compat - use cssHook if its there - use .style if its
			// available and use plain properties where available
			if ( jQuery.fx.step[ tween.prop ] ) {
				jQuery.fx.step[ tween.prop ]( tween );
			} else if ( tween.elem.style && ( tween.elem.style[ jQuery.cssProps[ tween.prop ] ] != null || jQuery.cssHooks[ tween.prop ] ) ) {
				jQuery.style( tween.elem, tween.prop, tween.now + tween.unit );
			} else {
				tween.elem[ tween.prop ] = tween.now;
			}
		}
	}
};

// Support: IE9
// Panic based approach to setting things on disconnected nodes

Tween.propHooks.scrollTop = Tween.propHooks.scrollLeft = {
	set: function( tween ) {
		if ( tween.elem.nodeType && tween.elem.parentNode ) {
			tween.elem[ tween.prop ] = tween.now;
		}
	}
};

jQuery.easing = {
	linear: function( p ) {
		return p;
	},
	swing: function( p ) {
		return 0.5 - Math.cos( p * Math.PI ) / 2;
	}
};

jQuery.fx = Tween.prototype.init;

// Back Compat <1.8 extension point
jQuery.fx.step = {};




var
	fxNow, timerId,
	rfxtypes = /^(?:toggle|show|hide)$/,
	rfxnum = new RegExp( "^(?:([+-])=|)(" + pnum + ")([a-z%]*)$", "i" ),
	rrun = /queueHooks$/,
	animationPrefilters = [ defaultPrefilter ],
	tweeners = {
		"*": [ function( prop, value ) {
			var tween = this.createTween( prop, value ),
				target = tween.cur(),
				parts = rfxnum.exec( value ),
				unit = parts && parts[ 3 ] || ( jQuery.cssNumber[ prop ] ? "" : "px" ),

				// Starting value computation is required for potential unit mismatches
				start = ( jQuery.cssNumber[ prop ] || unit !== "px" && +target ) &&
					rfxnum.exec( jQuery.css( tween.elem, prop ) ),
				scale = 1,
				maxIterations = 20;

			if ( start && start[ 3 ] !== unit ) {
				// Trust units reported by jQuery.css
				unit = unit || start[ 3 ];

				// Make sure we update the tween properties later on
				parts = parts || [];

				// Iteratively approximate from a nonzero starting point
				start = +target || 1;

				do {
					// If previous iteration zeroed out, double until we get *something*
					// Use a string for doubling factor so we don't accidentally see scale as unchanged below
					scale = scale || ".5";

					// Adjust and apply
					start = start / scale;
					jQuery.style( tween.elem, prop, start + unit );

				// Update scale, tolerating zero or NaN from tween.cur()
				// And breaking the loop if scale is unchanged or perfect, or if we've just had enough
				} while ( scale !== (scale = tween.cur() / target) && scale !== 1 && --maxIterations );
			}

			// Update tween properties
			if ( parts ) {
				start = tween.start = +start || +target || 0;
				tween.unit = unit;
				// If a +=/-= token was provided, we're doing a relative animation
				tween.end = parts[ 1 ] ?
					start + ( parts[ 1 ] + 1 ) * parts[ 2 ] :
					+parts[ 2 ];
			}

			return tween;
		} ]
	};

// Animations created synchronously will run synchronously
function createFxNow() {
	setTimeout(function() {
		fxNow = undefined;
	});
	return ( fxNow = jQuery.now() );
}

// Generate parameters to create a standard animation
function genFx( type, includeWidth ) {
	var which,
		i = 0,
		attrs = { height: type };

	// if we include width, step value is 1 to do all cssExpand values,
	// if we don't include width, step value is 2 to skip over Left and Right
	includeWidth = includeWidth ? 1 : 0;
	for ( ; i < 4 ; i += 2 - includeWidth ) {
		which = cssExpand[ i ];
		attrs[ "margin" + which ] = attrs[ "padding" + which ] = type;
	}

	if ( includeWidth ) {
		attrs.opacity = attrs.width = type;
	}

	return attrs;
}

function createTween( value, prop, animation ) {
	var tween,
		collection = ( tweeners[ prop ] || [] ).concat( tweeners[ "*" ] ),
		index = 0,
		length = collection.length;
	for ( ; index < length; index++ ) {
		if ( (tween = collection[ index ].call( animation, prop, value )) ) {

			// we're done with this property
			return tween;
		}
	}
}

function defaultPrefilter( elem, props, opts ) {
	/* jshint validthis: true */
	var prop, value, toggle, tween, hooks, oldfire, display,
		anim = this,
		orig = {},
		style = elem.style,
		hidden = elem.nodeType && isHidden( elem ),
		dataShow = data_priv.get( elem, "fxshow" );

	// handle queue: false promises
	if ( !opts.queue ) {
		hooks = jQuery._queueHooks( elem, "fx" );
		if ( hooks.unqueued == null ) {
			hooks.unqueued = 0;
			oldfire = hooks.empty.fire;
			hooks.empty.fire = function() {
				if ( !hooks.unqueued ) {
					oldfire();
				}
			};
		}
		hooks.unqueued++;

		anim.always(function() {
			// doing this makes sure that the complete handler will be called
			// before this completes
			anim.always(function() {
				hooks.unqueued--;
				if ( !jQuery.queue( elem, "fx" ).length ) {
					hooks.empty.fire();
				}
			});
		});
	}

	// height/width overflow pass
	if ( elem.nodeType === 1 && ( "height" in props || "width" in props ) ) {
		// Make sure that nothing sneaks out
		// Record all 3 overflow attributes because IE9-10 do not
		// change the overflow attribute when overflowX and
		// overflowY are set to the same value
		opts.overflow = [ style.overflow, style.overflowX, style.overflowY ];

		// Set display property to inline-block for height/width
		// animations on inline elements that are having width/height animated
		display = jQuery.css( elem, "display" );
		// Get default display if display is currently "none"
		if ( display === "none" ) {
			display = defaultDisplay( elem.nodeName );
		}
		if ( display === "inline" &&
				jQuery.css( elem, "float" ) === "none" ) {

			style.display = "inline-block";
		}
	}

	if ( opts.overflow ) {
		style.overflow = "hidden";
		anim.always(function() {
			style.overflow = opts.overflow[ 0 ];
			style.overflowX = opts.overflow[ 1 ];
			style.overflowY = opts.overflow[ 2 ];
		});
	}

	// show/hide pass
	for ( prop in props ) {
		value = props[ prop ];
		if ( rfxtypes.exec( value ) ) {
			delete props[ prop ];
			toggle = toggle || value === "toggle";
			if ( value === ( hidden ? "hide" : "show" ) ) {

				// If there is dataShow left over from a stopped hide or show and we are going to proceed with show, we should pretend to be hidden
				if ( value === "show" && dataShow && dataShow[ prop ] !== undefined ) {
					hidden = true;
				} else {
					continue;
				}
			}
			orig[ prop ] = dataShow && dataShow[ prop ] || jQuery.style( elem, prop );
		}
	}

	if ( !jQuery.isEmptyObject( orig ) ) {
		if ( dataShow ) {
			if ( "hidden" in dataShow ) {
				hidden = dataShow.hidden;
			}
		} else {
			dataShow = data_priv.access( elem, "fxshow", {} );
		}

		// store state if its toggle - enables .stop().toggle() to "reverse"
		if ( toggle ) {
			dataShow.hidden = !hidden;
		}
		if ( hidden ) {
			jQuery( elem ).show();
		} else {
			anim.done(function() {
				jQuery( elem ).hide();
			});
		}
		anim.done(function() {
			var prop;

			data_priv.remove( elem, "fxshow" );
			for ( prop in orig ) {
				jQuery.style( elem, prop, orig[ prop ] );
			}
		});
		for ( prop in orig ) {
			tween = createTween( hidden ? dataShow[ prop ] : 0, prop, anim );

			if ( !( prop in dataShow ) ) {
				dataShow[ prop ] = tween.start;
				if ( hidden ) {
					tween.end = tween.start;
					tween.start = prop === "width" || prop === "height" ? 1 : 0;
				}
			}
		}
	}
}

function propFilter( props, specialEasing ) {
	var index, name, easing, value, hooks;

	// camelCase, specialEasing and expand cssHook pass
	for ( index in props ) {
		name = jQuery.camelCase( index );
		easing = specialEasing[ name ];
		value = props[ index ];
		if ( jQuery.isArray( value ) ) {
			easing = value[ 1 ];
			value = props[ index ] = value[ 0 ];
		}

		if ( index !== name ) {
			props[ name ] = value;
			delete props[ index ];
		}

		hooks = jQuery.cssHooks[ name ];
		if ( hooks && "expand" in hooks ) {
			value = hooks.expand( value );
			delete props[ name ];

			// not quite $.extend, this wont overwrite keys already present.
			// also - reusing 'index' from above because we have the correct "name"
			for ( index in value ) {
				if ( !( index in props ) ) {
					props[ index ] = value[ index ];
					specialEasing[ index ] = easing;
				}
			}
		} else {
			specialEasing[ name ] = easing;
		}
	}
}

function Animation( elem, properties, options ) {
	var result,
		stopped,
		index = 0,
		length = animationPrefilters.length,
		deferred = jQuery.Deferred().always( function() {
			// don't match elem in the :animated selector
			delete tick.elem;
		}),
		tick = function() {
			if ( stopped ) {
				return false;
			}
			var currentTime = fxNow || createFxNow(),
				remaining = Math.max( 0, animation.startTime + animation.duration - currentTime ),
				// archaic crash bug won't allow us to use 1 - ( 0.5 || 0 ) (#12497)
				temp = remaining / animation.duration || 0,
				percent = 1 - temp,
				index = 0,
				length = animation.tweens.length;

			for ( ; index < length ; index++ ) {
				animation.tweens[ index ].run( percent );
			}

			deferred.notifyWith( elem, [ animation, percent, remaining ]);

			if ( percent < 1 && length ) {
				return remaining;
			} else {
				deferred.resolveWith( elem, [ animation ] );
				return false;
			}
		},
		animation = deferred.promise({
			elem: elem,
			props: jQuery.extend( {}, properties ),
			opts: jQuery.extend( true, { specialEasing: {} }, options ),
			originalProperties: properties,
			originalOptions: options,
			startTime: fxNow || createFxNow(),
			duration: options.duration,
			tweens: [],
			createTween: function( prop, end ) {
				var tween = jQuery.Tween( elem, animation.opts, prop, end,
						animation.opts.specialEasing[ prop ] || animation.opts.easing );
				animation.tweens.push( tween );
				return tween;
			},
			stop: function( gotoEnd ) {
				var index = 0,
					// if we are going to the end, we want to run all the tweens
					// otherwise we skip this part
					length = gotoEnd ? animation.tweens.length : 0;
				if ( stopped ) {
					return this;
				}
				stopped = true;
				for ( ; index < length ; index++ ) {
					animation.tweens[ index ].run( 1 );
				}

				// resolve when we played the last frame
				// otherwise, reject
				if ( gotoEnd ) {
					deferred.resolveWith( elem, [ animation, gotoEnd ] );
				} else {
					deferred.rejectWith( elem, [ animation, gotoEnd ] );
				}
				return this;
			}
		}),
		props = animation.props;

	propFilter( props, animation.opts.specialEasing );

	for ( ; index < length ; index++ ) {
		result = animationPrefilters[ index ].call( animation, elem, props, animation.opts );
		if ( result ) {
			return result;
		}
	}

	jQuery.map( props, createTween, animation );

	if ( jQuery.isFunction( animation.opts.start ) ) {
		animation.opts.start.call( elem, animation );
	}

	jQuery.fx.timer(
		jQuery.extend( tick, {
			elem: elem,
			anim: animation,
			queue: animation.opts.queue
		})
	);

	// attach callbacks from options
	return animation.progress( animation.opts.progress )
		.done( animation.opts.done, animation.opts.complete )
		.fail( animation.opts.fail )
		.always( animation.opts.always );
}

jQuery.Animation = jQuery.extend( Animation, {

	tweener: function( props, callback ) {
		if ( jQuery.isFunction( props ) ) {
			callback = props;
			props = [ "*" ];
		} else {
			props = props.split(" ");
		}

		var prop,
			index = 0,
			length = props.length;

		for ( ; index < length ; index++ ) {
			prop = props[ index ];
			tweeners[ prop ] = tweeners[ prop ] || [];
			tweeners[ prop ].unshift( callback );
		}
	},

	prefilter: function( callback, prepend ) {
		if ( prepend ) {
			animationPrefilters.unshift( callback );
		} else {
			animationPrefilters.push( callback );
		}
	}
});

jQuery.speed = function( speed, easing, fn ) {
	var opt = speed && typeof speed === "object" ? jQuery.extend( {}, speed ) : {
		complete: fn || !fn && easing ||
			jQuery.isFunction( speed ) && speed,
		duration: speed,
		easing: fn && easing || easing && !jQuery.isFunction( easing ) && easing
	};

	opt.duration = jQuery.fx.off ? 0 : typeof opt.duration === "number" ? opt.duration :
		opt.duration in jQuery.fx.speeds ? jQuery.fx.speeds[ opt.duration ] : jQuery.fx.speeds._default;

	// normalize opt.queue - true/undefined/null -> "fx"
	if ( opt.queue == null || opt.queue === true ) {
		opt.queue = "fx";
	}

	// Queueing
	opt.old = opt.complete;

	opt.complete = function() {
		if ( jQuery.isFunction( opt.old ) ) {
			opt.old.call( this );
		}

		if ( opt.queue ) {
			jQuery.dequeue( this, opt.queue );
		}
	};

	return opt;
};

jQuery.fn.extend({
	fadeTo: function( speed, to, easing, callback ) {

		// show any hidden elements after setting opacity to 0
		return this.filter( isHidden ).css( "opacity", 0 ).show()

			// animate to the value specified
			.end().animate({ opacity: to }, speed, easing, callback );
	},
	animate: function( prop, speed, easing, callback ) {
		var empty = jQuery.isEmptyObject( prop ),
			optall = jQuery.speed( speed, easing, callback ),
			doAnimation = function() {
				// Operate on a copy of prop so per-property easing won't be lost
				var anim = Animation( this, jQuery.extend( {}, prop ), optall );

				// Empty animations, or finishing resolves immediately
				if ( empty || data_priv.get( this, "finish" ) ) {
					anim.stop( true );
				}
			};
			doAnimation.finish = doAnimation;

		return empty || optall.queue === false ?
			this.each( doAnimation ) :
			this.queue( optall.queue, doAnimation );
	},
	stop: function( type, clearQueue, gotoEnd ) {
		var stopQueue = function( hooks ) {
			var stop = hooks.stop;
			delete hooks.stop;
			stop( gotoEnd );
		};

		if ( typeof type !== "string" ) {
			gotoEnd = clearQueue;
			clearQueue = type;
			type = undefined;
		}
		if ( clearQueue && type !== false ) {
			this.queue( type || "fx", [] );
		}

		return this.each(function() {
			var dequeue = true,
				index = type != null && type + "queueHooks",
				timers = jQuery.timers,
				data = data_priv.get( this );

			if ( index ) {
				if ( data[ index ] && data[ index ].stop ) {
					stopQueue( data[ index ] );
				}
			} else {
				for ( index in data ) {
					if ( data[ index ] && data[ index ].stop && rrun.test( index ) ) {
						stopQueue( data[ index ] );
					}
				}
			}

			for ( index = timers.length; index--; ) {
				if ( timers[ index ].elem === this && (type == null || timers[ index ].queue === type) ) {
					timers[ index ].anim.stop( gotoEnd );
					dequeue = false;
					timers.splice( index, 1 );
				}
			}

			// start the next in the queue if the last step wasn't forced
			// timers currently will call their complete callbacks, which will dequeue
			// but only if they were gotoEnd
			if ( dequeue || !gotoEnd ) {
				jQuery.dequeue( this, type );
			}
		});
	},
	finish: function( type ) {
		if ( type !== false ) {
			type = type || "fx";
		}
		return this.each(function() {
			var index,
				data = data_priv.get( this ),
				queue = data[ type + "queue" ],
				hooks = data[ type + "queueHooks" ],
				timers = jQuery.timers,
				length = queue ? queue.length : 0;

			// enable finishing flag on private data
			data.finish = true;

			// empty the queue first
			jQuery.queue( this, type, [] );

			if ( hooks && hooks.stop ) {
				hooks.stop.call( this, true );
			}

			// look for any active animations, and finish them
			for ( index = timers.length; index--; ) {
				if ( timers[ index ].elem === this && timers[ index ].queue === type ) {
					timers[ index ].anim.stop( true );
					timers.splice( index, 1 );
				}
			}

			// look for any animations in the old queue and finish them
			for ( index = 0; index < length; index++ ) {
				if ( queue[ index ] && queue[ index ].finish ) {
					queue[ index ].finish.call( this );
				}
			}

			// turn off finishing flag
			delete data.finish;
		});
	}
});

jQuery.each([ "toggle", "show", "hide" ], function( i, name ) {
	var cssFn = jQuery.fn[ name ];
	jQuery.fn[ name ] = function( speed, easing, callback ) {
		return speed == null || typeof speed === "boolean" ?
			cssFn.apply( this, arguments ) :
			this.animate( genFx( name, true ), speed, easing, callback );
	};
});

// Generate shortcuts for custom animations
jQuery.each({
	slideDown: genFx("show"),
	slideUp: genFx("hide"),
	slideToggle: genFx("toggle"),
	fadeIn: { opacity: "show" },
	fadeOut: { opacity: "hide" },
	fadeToggle: { opacity: "toggle" }
}, function( name, props ) {
	jQuery.fn[ name ] = function( speed, easing, callback ) {
		return this.animate( props, speed, easing, callback );
	};
});

jQuery.timers = [];
jQuery.fx.tick = function() {
	var timer,
		i = 0,
		timers = jQuery.timers;

	fxNow = jQuery.now();

	for ( ; i < timers.length; i++ ) {
		timer = timers[ i ];
		// Checks the timer has not already been removed
		if ( !timer() && timers[ i ] === timer ) {
			timers.splice( i--, 1 );
		}
	}

	if ( !timers.length ) {
		jQuery.fx.stop();
	}
	fxNow = undefined;
};

jQuery.fx.timer = function( timer ) {
	jQuery.timers.push( timer );
	if ( timer() ) {
		jQuery.fx.start();
	} else {
		jQuery.timers.pop();
	}
};

jQuery.fx.interval = 13;

jQuery.fx.start = function() {
	if ( !timerId ) {
		timerId = setInterval( jQuery.fx.tick, jQuery.fx.interval );
	}
};

jQuery.fx.stop = function() {
	clearInterval( timerId );
	timerId = null;
};

jQuery.fx.speeds = {
	slow: 600,
	fast: 200,
	// Default speed
	_default: 400
};


// Based off of the plugin by Clint Helfers, with permission.
// http://blindsignals.com/index.php/2009/07/jquery-delay/
jQuery.fn.delay = function( time, type ) {
	time = jQuery.fx ? jQuery.fx.speeds[ time ] || time : time;
	type = type || "fx";

	return this.queue( type, function( next, hooks ) {
		var timeout = setTimeout( next, time );
		hooks.stop = function() {
			clearTimeout( timeout );
		};
	});
};


(function() {
	var input = document.createElement( "input" ),
		select = document.createElement( "select" ),
		opt = select.appendChild( document.createElement( "option" ) );

	input.type = "checkbox";

	// Support: iOS 5.1, Android 4.x, Android 2.3
	// Check the default checkbox/radio value ("" on old WebKit; "on" elsewhere)
	support.checkOn = input.value !== "";

	// Must access the parent to make an option select properly
	// Support: IE9, IE10
	support.optSelected = opt.selected;

	// Make sure that the options inside disabled selects aren't marked as disabled
	// (WebKit marks them as disabled)
	select.disabled = true;
	support.optDisabled = !opt.disabled;

	// Check if an input maintains its value after becoming a radio
	// Support: IE9, IE10
	input = document.createElement( "input" );
	input.value = "t";
	input.type = "radio";
	support.radioValue = input.value === "t";
})();


var nodeHook, boolHook,
	attrHandle = jQuery.expr.attrHandle;

jQuery.fn.extend({
	attr: function( name, value ) {
		return access( this, jQuery.attr, name, value, arguments.length > 1 );
	},

	removeAttr: function( name ) {
		return this.each(function() {
			jQuery.removeAttr( this, name );
		});
	}
});

jQuery.extend({
	attr: function( elem, name, value ) {
		var hooks, ret,
			nType = elem.nodeType;

		// don't get/set attributes on text, comment and attribute nodes
		if ( !elem || nType === 3 || nType === 8 || nType === 2 ) {
			return;
		}

		// Fallback to prop when attributes are not supported
		if ( typeof elem.getAttribute === strundefined ) {
			return jQuery.prop( elem, name, value );
		}

		// All attributes are lowercase
		// Grab necessary hook if one is defined
		if ( nType !== 1 || !jQuery.isXMLDoc( elem ) ) {
			name = name.toLowerCase();
			hooks = jQuery.attrHooks[ name ] ||
				( jQuery.expr.match.bool.test( name ) ? boolHook : nodeHook );
		}

		if ( value !== undefined ) {

			if ( value === null ) {
				jQuery.removeAttr( elem, name );

			} else if ( hooks && "set" in hooks && (ret = hooks.set( elem, value, name )) !== undefined ) {
				return ret;

			} else {
				elem.setAttribute( name, value + "" );
				return value;
			}

		} else if ( hooks && "get" in hooks && (ret = hooks.get( elem, name )) !== null ) {
			return ret;

		} else {
			ret = jQuery.find.attr( elem, name );

			// Non-existent attributes return null, we normalize to undefined
			return ret == null ?
				undefined :
				ret;
		}
	},

	removeAttr: function( elem, value ) {
		var name, propName,
			i = 0,
			attrNames = value && value.match( rnotwhite );

		if ( attrNames && elem.nodeType === 1 ) {
			while ( (name = attrNames[i++]) ) {
				propName = jQuery.propFix[ name ] || name;

				// Boolean attributes get special treatment (#10870)
				if ( jQuery.expr.match.bool.test( name ) ) {
					// Set corresponding property to false
					elem[ propName ] = false;
				}

				elem.removeAttribute( name );
			}
		}
	},

	attrHooks: {
		type: {
			set: function( elem, value ) {
				if ( !support.radioValue && value === "radio" &&
					jQuery.nodeName( elem, "input" ) ) {
					// Setting the type on a radio button after the value resets the value in IE6-9
					// Reset value to default in case type is set after value during creation
					var val = elem.value;
					elem.setAttribute( "type", value );
					if ( val ) {
						elem.value = val;
					}
					return value;
				}
			}
		}
	}
});

// Hooks for boolean attributes
boolHook = {
	set: function( elem, value, name ) {
		if ( value === false ) {
			// Remove boolean attributes when set to false
			jQuery.removeAttr( elem, name );
		} else {
			elem.setAttribute( name, name );
		}
		return name;
	}
};
jQuery.each( jQuery.expr.match.bool.source.match( /\w+/g ), function( i, name ) {
	var getter = attrHandle[ name ] || jQuery.find.attr;

	attrHandle[ name ] = function( elem, name, isXML ) {
		var ret, handle;
		if ( !isXML ) {
			// Avoid an infinite loop by temporarily removing this function from the getter
			handle = attrHandle[ name ];
			attrHandle[ name ] = ret;
			ret = getter( elem, name, isXML ) != null ?
				name.toLowerCase() :
				null;
			attrHandle[ name ] = handle;
		}
		return ret;
	};
});




var rfocusable = /^(?:input|select|textarea|button)$/i;

jQuery.fn.extend({
	prop: function( name, value ) {
		return access( this, jQuery.prop, name, value, arguments.length > 1 );
	},

	removeProp: function( name ) {
		return this.each(function() {
			delete this[ jQuery.propFix[ name ] || name ];
		});
	}
});

jQuery.extend({
	propFix: {
		"for": "htmlFor",
		"class": "className"
	},

	prop: function( elem, name, value ) {
		var ret, hooks, notxml,
			nType = elem.nodeType;

		// don't get/set properties on text, comment and attribute nodes
		if ( !elem || nType === 3 || nType === 8 || nType === 2 ) {
			return;
		}

		notxml = nType !== 1 || !jQuery.isXMLDoc( elem );

		if ( notxml ) {
			// Fix name and attach hooks
			name = jQuery.propFix[ name ] || name;
			hooks = jQuery.propHooks[ name ];
		}

		if ( value !== undefined ) {
			return hooks && "set" in hooks && (ret = hooks.set( elem, value, name )) !== undefined ?
				ret :
				( elem[ name ] = value );

		} else {
			return hooks && "get" in hooks && (ret = hooks.get( elem, name )) !== null ?
				ret :
				elem[ name ];
		}
	},

	propHooks: {
		tabIndex: {
			get: function( elem ) {
				return elem.hasAttribute( "tabindex" ) || rfocusable.test( elem.nodeName ) || elem.href ?
					elem.tabIndex :
					-1;
			}
		}
	}
});

// Support: IE9+
// Selectedness for an option in an optgroup can be inaccurate
if ( !support.optSelected ) {
	jQuery.propHooks.selected = {
		get: function( elem ) {
			var parent = elem.parentNode;
			if ( parent && parent.parentNode ) {
				parent.parentNode.selectedIndex;
			}
			return null;
		}
	};
}

jQuery.each([
	"tabIndex",
	"readOnly",
	"maxLength",
	"cellSpacing",
	"cellPadding",
	"rowSpan",
	"colSpan",
	"useMap",
	"frameBorder",
	"contentEditable"
], function() {
	jQuery.propFix[ this.toLowerCase() ] = this;
});




var rclass = /[\t\r\n\f]/g;

jQuery.fn.extend({
	addClass: function( value ) {
		var classes, elem, cur, clazz, j, finalValue,
			proceed = typeof value === "string" && value,
			i = 0,
			len = this.length;

		if ( jQuery.isFunction( value ) ) {
			return this.each(function( j ) {
				jQuery( this ).addClass( value.call( this, j, this.className ) );
			});
		}

		if ( proceed ) {
			// The disjunction here is for better compressibility (see removeClass)
			classes = ( value || "" ).match( rnotwhite ) || [];

			for ( ; i < len; i++ ) {
				elem = this[ i ];
				cur = elem.nodeType === 1 && ( elem.className ?
					( " " + elem.className + " " ).replace( rclass, " " ) :
					" "
				);

				if ( cur ) {
					j = 0;
					while ( (clazz = classes[j++]) ) {
						if ( cur.indexOf( " " + clazz + " " ) < 0 ) {
							cur += clazz + " ";
						}
					}

					// only assign if different to avoid unneeded rendering.
					finalValue = jQuery.trim( cur );
					if ( elem.className !== finalValue ) {
						elem.className = finalValue;
					}
				}
			}
		}

		return this;
	},

	removeClass: function( value ) {
		var classes, elem, cur, clazz, j, finalValue,
			proceed = arguments.length === 0 || typeof value === "string" && value,
			i = 0,
			len = this.length;

		if ( jQuery.isFunction( value ) ) {
			return this.each(function( j ) {
				jQuery( this ).removeClass( value.call( this, j, this.className ) );
			});
		}
		if ( proceed ) {
			classes = ( value || "" ).match( rnotwhite ) || [];

			for ( ; i < len; i++ ) {
				elem = this[ i ];
				// This expression is here for better compressibility (see addClass)
				cur = elem.nodeType === 1 && ( elem.className ?
					( " " + elem.className + " " ).replace( rclass, " " ) :
					""
				);

				if ( cur ) {
					j = 0;
					while ( (clazz = classes[j++]) ) {
						// Remove *all* instances
						while ( cur.indexOf( " " + clazz + " " ) >= 0 ) {
							cur = cur.replace( " " + clazz + " ", " " );
						}
					}

					// only assign if different to avoid unneeded rendering.
					finalValue = value ? jQuery.trim( cur ) : "";
					if ( elem.className !== finalValue ) {
						elem.className = finalValue;
					}
				}
			}
		}

		return this;
	},

	toggleClass: function( value, stateVal ) {
		var type = typeof value;

		if ( typeof stateVal === "boolean" && type === "string" ) {
			return stateVal ? this.addClass( value ) : this.removeClass( value );
		}

		if ( jQuery.isFunction( value ) ) {
			return this.each(function( i ) {
				jQuery( this ).toggleClass( value.call(this, i, this.className, stateVal), stateVal );
			});
		}

		return this.each(function() {
			if ( type === "string" ) {
				// toggle individual class names
				var className,
					i = 0,
					self = jQuery( this ),
					classNames = value.match( rnotwhite ) || [];

				while ( (className = classNames[ i++ ]) ) {
					// check each className given, space separated list
					if ( self.hasClass( className ) ) {
						self.removeClass( className );
					} else {
						self.addClass( className );
					}
				}

			// Toggle whole class name
			} else if ( type === strundefined || type === "boolean" ) {
				if ( this.className ) {
					// store className if set
					data_priv.set( this, "__className__", this.className );
				}

				// If the element has a class name or if we're passed "false",
				// then remove the whole classname (if there was one, the above saved it).
				// Otherwise bring back whatever was previously saved (if anything),
				// falling back to the empty string if nothing was stored.
				this.className = this.className || value === false ? "" : data_priv.get( this, "__className__" ) || "";
			}
		});
	},

	hasClass: function( selector ) {
		var className = " " + selector + " ",
			i = 0,
			l = this.length;
		for ( ; i < l; i++ ) {
			if ( this[i].nodeType === 1 && (" " + this[i].className + " ").replace(rclass, " ").indexOf( className ) >= 0 ) {
				return true;
			}
		}

		return false;
	}
});




var rreturn = /\r/g;

jQuery.fn.extend({
	val: function( value ) {
		var hooks, ret, isFunction,
			elem = this[0];

		if ( !arguments.length ) {
			if ( elem ) {
				hooks = jQuery.valHooks[ elem.type ] || jQuery.valHooks[ elem.nodeName.toLowerCase() ];

				if ( hooks && "get" in hooks && (ret = hooks.get( elem, "value" )) !== undefined ) {
					return ret;
				}

				ret = elem.value;

				return typeof ret === "string" ?
					// handle most common string cases
					ret.replace(rreturn, "") :
					// handle cases where value is null/undef or number
					ret == null ? "" : ret;
			}

			return;
		}

		isFunction = jQuery.isFunction( value );

		return this.each(function( i ) {
			var val;

			if ( this.nodeType !== 1 ) {
				return;
			}

			if ( isFunction ) {
				val = value.call( this, i, jQuery( this ).val() );
			} else {
				val = value;
			}

			// Treat null/undefined as ""; convert numbers to string
			if ( val == null ) {
				val = "";

			} else if ( typeof val === "number" ) {
				val += "";

			} else if ( jQuery.isArray( val ) ) {
				val = jQuery.map( val, function( value ) {
					return value == null ? "" : value + "";
				});
			}

			hooks = jQuery.valHooks[ this.type ] || jQuery.valHooks[ this.nodeName.toLowerCase() ];

			// If set returns undefined, fall back to normal setting
			if ( !hooks || !("set" in hooks) || hooks.set( this, val, "value" ) === undefined ) {
				this.value = val;
			}
		});
	}
});

jQuery.extend({
	valHooks: {
		select: {
			get: function( elem ) {
				var value, option,
					options = elem.options,
					index = elem.selectedIndex,
					one = elem.type === "select-one" || index < 0,
					values = one ? null : [],
					max = one ? index + 1 : options.length,
					i = index < 0 ?
						max :
						one ? index : 0;

				// Loop through all the selected options
				for ( ; i < max; i++ ) {
					option = options[ i ];

					// IE6-9 doesn't update selected after form reset (#2551)
					if ( ( option.selected || i === index ) &&
							// Don't return options that are disabled or in a disabled optgroup
							( support.optDisabled ? !option.disabled : option.getAttribute( "disabled" ) === null ) &&
							( !option.parentNode.disabled || !jQuery.nodeName( option.parentNode, "optgroup" ) ) ) {

						// Get the specific value for the option
						value = jQuery( option ).val();

						// We don't need an array for one selects
						if ( one ) {
							return value;
						}

						// Multi-Selects return an array
						values.push( value );
					}
				}

				return values;
			},

			set: function( elem, value ) {
				var optionSet, option,
					options = elem.options,
					values = jQuery.makeArray( value ),
					i = options.length;

				while ( i-- ) {
					option = options[ i ];
					if ( (option.selected = jQuery.inArray( jQuery(option).val(), values ) >= 0) ) {
						optionSet = true;
					}
				}

				// force browsers to behave consistently when non-matching value is set
				if ( !optionSet ) {
					elem.selectedIndex = -1;
				}
				return values;
			}
		}
	}
});

// Radios and checkboxes getter/setter
jQuery.each([ "radio", "checkbox" ], function() {
	jQuery.valHooks[ this ] = {
		set: function( elem, value ) {
			if ( jQuery.isArray( value ) ) {
				return ( elem.checked = jQuery.inArray( jQuery(elem).val(), value ) >= 0 );
			}
		}
	};
	if ( !support.checkOn ) {
		jQuery.valHooks[ this ].get = function( elem ) {
			// Support: Webkit
			// "" is returned instead of "on" if a value isn't specified
			return elem.getAttribute("value") === null ? "on" : elem.value;
		};
	}
});




// Return jQuery for attributes-only inclusion


jQuery.each( ("blur focus focusin focusout load resize scroll unload click dblclick " +
	"mousedown mouseup mousemove mouseover mouseout mouseenter mouseleave " +
	"change select submit keydown keypress keyup error contextmenu").split(" "), function( i, name ) {

	// Handle event binding
	jQuery.fn[ name ] = function( data, fn ) {
		return arguments.length > 0 ?
			this.on( name, null, data, fn ) :
			this.trigger( name );
	};
});

jQuery.fn.extend({
	hover: function( fnOver, fnOut ) {
		return this.mouseenter( fnOver ).mouseleave( fnOut || fnOver );
	},

	bind: function( types, data, fn ) {
		return this.on( types, null, data, fn );
	},
	unbind: function( types, fn ) {
		return this.off( types, null, fn );
	},

	delegate: function( selector, types, data, fn ) {
		return this.on( types, selector, data, fn );
	},
	undelegate: function( selector, types, fn ) {
		// ( namespace ) or ( selector, types [, fn] )
		return arguments.length === 1 ? this.off( selector, "**" ) : this.off( types, selector || "**", fn );
	}
});


var nonce = jQuery.now();

var rquery = (/\?/);



// Support: Android 2.3
// Workaround failure to string-cast null input
jQuery.parseJSON = function( data ) {
	return JSON.parse( data + "" );
};


// Cross-browser xml parsing
jQuery.parseXML = function( data ) {
	var xml, tmp;
	if ( !data || typeof data !== "string" ) {
		return null;
	}

	// Support: IE9
	try {
		tmp = new DOMParser();
		xml = tmp.parseFromString( data, "text/xml" );
	} catch ( e ) {
		xml = undefined;
	}

	if ( !xml || xml.getElementsByTagName( "parsererror" ).length ) {
		jQuery.error( "Invalid XML: " + data );
	}
	return xml;
};


var
	// Document location
	ajaxLocParts,
	ajaxLocation,

	rhash = /#.*$/,
	rts = /([?&])_=[^&]*/,
	rheaders = /^(.*?):[ \t]*([^\r\n]*)$/mg,
	// #7653, #8125, #8152: local protocol detection
	rlocalProtocol = /^(?:about|app|app-storage|.+-extension|file|res|widget):$/,
	rnoContent = /^(?:GET|HEAD)$/,
	rprotocol = /^\/\//,
	rurl = /^([\w.+-]+:)(?:\/\/(?:[^\/?#]*@|)([^\/?#:]*)(?::(\d+)|)|)/,

	/* Prefilters
	 * 1) They are useful to introduce custom dataTypes (see ajax/jsonp.js for an example)
	 * 2) These are called:
	 *    - BEFORE asking for a transport
	 *    - AFTER param serialization (s.data is a string if s.processData is true)
	 * 3) key is the dataType
	 * 4) the catchall symbol "*" can be used
	 * 5) execution will start with transport dataType and THEN continue down to "*" if needed
	 */
	prefilters = {},

	/* Transports bindings
	 * 1) key is the dataType
	 * 2) the catchall symbol "*" can be used
	 * 3) selection will start with transport dataType and THEN go to "*" if needed
	 */
	transports = {},

	// Avoid comment-prolog char sequence (#10098); must appease lint and evade compression
	allTypes = "*/".concat("*");

// #8138, IE may throw an exception when accessing
// a field from window.location if document.domain has been set
try {
	ajaxLocation = location.href;
} catch( e ) {
	// Use the href attribute of an A element
	// since IE will modify it given document.location
	ajaxLocation = document.createElement( "a" );
	ajaxLocation.href = "";
	ajaxLocation = ajaxLocation.href;
}

// Segment location into parts
ajaxLocParts = rurl.exec( ajaxLocation.toLowerCase() ) || [];

// Base "constructor" for jQuery.ajaxPrefilter and jQuery.ajaxTransport
function addToPrefiltersOrTransports( structure ) {

	// dataTypeExpression is optional and defaults to "*"
	return function( dataTypeExpression, func ) {

		if ( typeof dataTypeExpression !== "string" ) {
			func = dataTypeExpression;
			dataTypeExpression = "*";
		}

		var dataType,
			i = 0,
			dataTypes = dataTypeExpression.toLowerCase().match( rnotwhite ) || [];

		if ( jQuery.isFunction( func ) ) {
			// For each dataType in the dataTypeExpression
			while ( (dataType = dataTypes[i++]) ) {
				// Prepend if requested
				if ( dataType[0] === "+" ) {
					dataType = dataType.slice( 1 ) || "*";
					(structure[ dataType ] = structure[ dataType ] || []).unshift( func );

				// Otherwise append
				} else {
					(structure[ dataType ] = structure[ dataType ] || []).push( func );
				}
			}
		}
	};
}

// Base inspection function for prefilters and transports
function inspectPrefiltersOrTransports( structure, options, originalOptions, jqXHR ) {

	var inspected = {},
		seekingTransport = ( structure === transports );

	function inspect( dataType ) {
		var selected;
		inspected[ dataType ] = true;
		jQuery.each( structure[ dataType ] || [], function( _, prefilterOrFactory ) {
			var dataTypeOrTransport = prefilterOrFactory( options, originalOptions, jqXHR );
			if ( typeof dataTypeOrTransport === "string" && !seekingTransport && !inspected[ dataTypeOrTransport ] ) {
				options.dataTypes.unshift( dataTypeOrTransport );
				inspect( dataTypeOrTransport );
				return false;
			} else if ( seekingTransport ) {
				return !( selected = dataTypeOrTransport );
			}
		});
		return selected;
	}

	return inspect( options.dataTypes[ 0 ] ) || !inspected[ "*" ] && inspect( "*" );
}

// A special extend for ajax options
// that takes "flat" options (not to be deep extended)
// Fixes #9887
function ajaxExtend( target, src ) {
	var key, deep,
		flatOptions = jQuery.ajaxSettings.flatOptions || {};

	for ( key in src ) {
		if ( src[ key ] !== undefined ) {
			( flatOptions[ key ] ? target : ( deep || (deep = {}) ) )[ key ] = src[ key ];
		}
	}
	if ( deep ) {
		jQuery.extend( true, target, deep );
	}

	return target;
}

/* Handles responses to an ajax request:
 * - finds the right dataType (mediates between content-type and expected dataType)
 * - returns the corresponding response
 */
function ajaxHandleResponses( s, jqXHR, responses ) {

	var ct, type, finalDataType, firstDataType,
		contents = s.contents,
		dataTypes = s.dataTypes;

	// Remove auto dataType and get content-type in the process
	while ( dataTypes[ 0 ] === "*" ) {
		dataTypes.shift();
		if ( ct === undefined ) {
			ct = s.mimeType || jqXHR.getResponseHeader("Content-Type");
		}
	}

	// Check if we're dealing with a known content-type
	if ( ct ) {
		for ( type in contents ) {
			if ( contents[ type ] && contents[ type ].test( ct ) ) {
				dataTypes.unshift( type );
				break;
			}
		}
	}

	// Check to see if we have a response for the expected dataType
	if ( dataTypes[ 0 ] in responses ) {
		finalDataType = dataTypes[ 0 ];
	} else {
		// Try convertible dataTypes
		for ( type in responses ) {
			if ( !dataTypes[ 0 ] || s.converters[ type + " " + dataTypes[0] ] ) {
				finalDataType = type;
				break;
			}
			if ( !firstDataType ) {
				firstDataType = type;
			}
		}
		// Or just use first one
		finalDataType = finalDataType || firstDataType;
	}

	// If we found a dataType
	// We add the dataType to the list if needed
	// and return the corresponding response
	if ( finalDataType ) {
		if ( finalDataType !== dataTypes[ 0 ] ) {
			dataTypes.unshift( finalDataType );
		}
		return responses[ finalDataType ];
	}
}

/* Chain conversions given the request and the original response
 * Also sets the responseXXX fields on the jqXHR instance
 */
function ajaxConvert( s, response, jqXHR, isSuccess ) {
	var conv2, current, conv, tmp, prev,
		converters = {},
		// Work with a copy of dataTypes in case we need to modify it for conversion
		dataTypes = s.dataTypes.slice();

	// Create converters map with lowercased keys
	if ( dataTypes[ 1 ] ) {
		for ( conv in s.converters ) {
			converters[ conv.toLowerCase() ] = s.converters[ conv ];
		}
	}

	current = dataTypes.shift();

	// Convert to each sequential dataType
	while ( current ) {

		if ( s.responseFields[ current ] ) {
			jqXHR[ s.responseFields[ current ] ] = response;
		}

		// Apply the dataFilter if provided
		if ( !prev && isSuccess && s.dataFilter ) {
			response = s.dataFilter( response, s.dataType );
		}

		prev = current;
		current = dataTypes.shift();

		if ( current ) {

		// There's only work to do if current dataType is non-auto
			if ( current === "*" ) {

				current = prev;

			// Convert response if prev dataType is non-auto and differs from current
			} else if ( prev !== "*" && prev !== current ) {

				// Seek a direct converter
				conv = converters[ prev + " " + current ] || converters[ "* " + current ];

				// If none found, seek a pair
				if ( !conv ) {
					for ( conv2 in converters ) {

						// If conv2 outputs current
						tmp = conv2.split( " " );
						if ( tmp[ 1 ] === current ) {

							// If prev can be converted to accepted input
							conv = converters[ prev + " " + tmp[ 0 ] ] ||
								converters[ "* " + tmp[ 0 ] ];
							if ( conv ) {
								// Condense equivalence converters
								if ( conv === true ) {
									conv = converters[ conv2 ];

								// Otherwise, insert the intermediate dataType
								} else if ( converters[ conv2 ] !== true ) {
									current = tmp[ 0 ];
									dataTypes.unshift( tmp[ 1 ] );
								}
								break;
							}
						}
					}
				}

				// Apply converter (if not an equivalence)
				if ( conv !== true ) {

					// Unless errors are allowed to bubble, catch and return them
					if ( conv && s[ "throws" ] ) {
						response = conv( response );
					} else {
						try {
							response = conv( response );
						} catch ( e ) {
							return { state: "parsererror", error: conv ? e : "No conversion from " + prev + " to " + current };
						}
					}
				}
			}
		}
	}

	return { state: "success", data: response };
}

jQuery.extend({

	// Counter for holding the number of active queries
	active: 0,

	// Last-Modified header cache for next request
	lastModified: {},
	etag: {},

	ajaxSettings: {
		url: ajaxLocation,
		type: "GET",
		isLocal: rlocalProtocol.test( ajaxLocParts[ 1 ] ),
		global: true,
		processData: true,
		async: true,
		contentType: "application/x-www-form-urlencoded; charset=UTF-8",
		/*
		timeout: 0,
		data: null,
		dataType: null,
		username: null,
		password: null,
		cache: null,
		throws: false,
		traditional: false,
		headers: {},
		*/

		accepts: {
			"*": allTypes,
			text: "text/plain",
			html: "text/html",
			xml: "application/xml, text/xml",
			json: "application/json, text/javascript"
		},

		contents: {
			xml: /xml/,
			html: /html/,
			json: /json/
		},

		responseFields: {
			xml: "responseXML",
			text: "responseText",
			json: "responseJSON"
		},

		// Data converters
		// Keys separate source (or catchall "*") and destination types with a single space
		converters: {

			// Convert anything to text
			"* text": String,

			// Text to html (true = no transformation)
			"text html": true,

			// Evaluate text as a json expression
			"text json": jQuery.parseJSON,

			// Parse text as xml
			"text xml": jQuery.parseXML
		},

		// For options that shouldn't be deep extended:
		// you can add your own custom options here if
		// and when you create one that shouldn't be
		// deep extended (see ajaxExtend)
		flatOptions: {
			url: true,
			context: true
		}
	},

	// Creates a full fledged settings object into target
	// with both ajaxSettings and settings fields.
	// If target is omitted, writes into ajaxSettings.
	ajaxSetup: function( target, settings ) {
		return settings ?

			// Building a settings object
			ajaxExtend( ajaxExtend( target, jQuery.ajaxSettings ), settings ) :

			// Extending ajaxSettings
			ajaxExtend( jQuery.ajaxSettings, target );
	},

	ajaxPrefilter: addToPrefiltersOrTransports( prefilters ),
	ajaxTransport: addToPrefiltersOrTransports( transports ),

	// Main method
	ajax: function( url, options ) {

		// If url is an object, simulate pre-1.5 signature
		if ( typeof url === "object" ) {
			options = url;
			url = undefined;
		}

		// Force options to be an object
		options = options || {};

		var transport,
			// URL without anti-cache param
			cacheURL,
			// Response headers
			responseHeadersString,
			responseHeaders,
			// timeout handle
			timeoutTimer,
			// Cross-domain detection vars
			parts,
			// To know if global events are to be dispatched
			fireGlobals,
			// Loop variable
			i,
			// Create the final options object
			s = jQuery.ajaxSetup( {}, options ),
			// Callbacks context
			callbackContext = s.context || s,
			// Context for global events is callbackContext if it is a DOM node or jQuery collection
			globalEventContext = s.context && ( callbackContext.nodeType || callbackContext.jquery ) ?
				jQuery( callbackContext ) :
				jQuery.event,
			// Deferreds
			deferred = jQuery.Deferred(),
			completeDeferred = jQuery.Callbacks("once memory"),
			// Status-dependent callbacks
			statusCode = s.statusCode || {},
			// Headers (they are sent all at once)
			requestHeaders = {},
			requestHeadersNames = {},
			// The jqXHR state
			state = 0,
			// Default abort message
			strAbort = "canceled",
			// Fake xhr
			jqXHR = {
				readyState: 0,

				// Builds headers hashtable if needed
				getResponseHeader: function( key ) {
					var match;
					if ( state === 2 ) {
						if ( !responseHeaders ) {
							responseHeaders = {};
							while ( (match = rheaders.exec( responseHeadersString )) ) {
								responseHeaders[ match[1].toLowerCase() ] = match[ 2 ];
							}
						}
						match = responseHeaders[ key.toLowerCase() ];
					}
					return match == null ? null : match;
				},

				// Raw string
				getAllResponseHeaders: function() {
					return state === 2 ? responseHeadersString : null;
				},

				// Caches the header
				setRequestHeader: function( name, value ) {
					var lname = name.toLowerCase();
					if ( !state ) {
						name = requestHeadersNames[ lname ] = requestHeadersNames[ lname ] || name;
						requestHeaders[ name ] = value;
					}
					return this;
				},

				// Overrides response content-type header
				overrideMimeType: function( type ) {
					if ( !state ) {
						s.mimeType = type;
					}
					return this;
				},

				// Status-dependent callbacks
				statusCode: function( map ) {
					var code;
					if ( map ) {
						if ( state < 2 ) {
							for ( code in map ) {
								// Lazy-add the new callback in a way that preserves old ones
								statusCode[ code ] = [ statusCode[ code ], map[ code ] ];
							}
						} else {
							// Execute the appropriate callbacks
							jqXHR.always( map[ jqXHR.status ] );
						}
					}
					return this;
				},

				// Cancel the request
				abort: function( statusText ) {
					var finalText = statusText || strAbort;
					if ( transport ) {
						transport.abort( finalText );
					}
					done( 0, finalText );
					return this;
				}
			};

		// Attach deferreds
		deferred.promise( jqXHR ).complete = completeDeferred.add;
		jqXHR.success = jqXHR.done;
		jqXHR.error = jqXHR.fail;

		// Remove hash character (#7531: and string promotion)
		// Add protocol if not provided (prefilters might expect it)
		// Handle falsy url in the settings object (#10093: consistency with old signature)
		// We also use the url parameter if available
		s.url = ( ( url || s.url || ajaxLocation ) + "" ).replace( rhash, "" )
			.replace( rprotocol, ajaxLocParts[ 1 ] + "//" );

		// Alias method option to type as per ticket #12004
		s.type = options.method || options.type || s.method || s.type;

		// Extract dataTypes list
		s.dataTypes = jQuery.trim( s.dataType || "*" ).toLowerCase().match( rnotwhite ) || [ "" ];

		// A cross-domain request is in order when we have a protocol:host:port mismatch
		if ( s.crossDomain == null ) {
			parts = rurl.exec( s.url.toLowerCase() );
			s.crossDomain = !!( parts &&
				( parts[ 1 ] !== ajaxLocParts[ 1 ] || parts[ 2 ] !== ajaxLocParts[ 2 ] ||
					( parts[ 3 ] || ( parts[ 1 ] === "http:" ? "80" : "443" ) ) !==
						( ajaxLocParts[ 3 ] || ( ajaxLocParts[ 1 ] === "http:" ? "80" : "443" ) ) )
			);
		}

		// Convert data if not already a string
		if ( s.data && s.processData && typeof s.data !== "string" ) {
			s.data = jQuery.param( s.data, s.traditional );
		}

		// Apply prefilters
		inspectPrefiltersOrTransports( prefilters, s, options, jqXHR );

		// If request was aborted inside a prefilter, stop there
		if ( state === 2 ) {
			return jqXHR;
		}

		// We can fire global events as of now if asked to
		fireGlobals = s.global;

		// Watch for a new set of requests
		if ( fireGlobals && jQuery.active++ === 0 ) {
			jQuery.event.trigger("ajaxStart");
		}

		// Uppercase the type
		s.type = s.type.toUpperCase();

		// Determine if request has content
		s.hasContent = !rnoContent.test( s.type );

		// Save the URL in case we're toying with the If-Modified-Since
		// and/or If-None-Match header later on
		cacheURL = s.url;

		// More options handling for requests with no content
		if ( !s.hasContent ) {

			// If data is available, append data to url
			if ( s.data ) {
				cacheURL = ( s.url += ( rquery.test( cacheURL ) ? "&" : "?" ) + s.data );
				// #9682: remove data so that it's not used in an eventual retry
				delete s.data;
			}

			// Add anti-cache in url if needed
			if ( s.cache === false ) {
				s.url = rts.test( cacheURL ) ?

					// If there is already a '_' parameter, set its value
					cacheURL.replace( rts, "$1_=" + nonce++ ) :

					// Otherwise add one to the end
					cacheURL + ( rquery.test( cacheURL ) ? "&" : "?" ) + "_=" + nonce++;
			}
		}

		// Set the If-Modified-Since and/or If-None-Match header, if in ifModified mode.
		if ( s.ifModified ) {
			if ( jQuery.lastModified[ cacheURL ] ) {
				jqXHR.setRequestHeader( "If-Modified-Since", jQuery.lastModified[ cacheURL ] );
			}
			if ( jQuery.etag[ cacheURL ] ) {
				jqXHR.setRequestHeader( "If-None-Match", jQuery.etag[ cacheURL ] );
			}
		}

		// Set the correct header, if data is being sent
		if ( s.data && s.hasContent && s.contentType !== false || options.contentType ) {
			jqXHR.setRequestHeader( "Content-Type", s.contentType );
		}

		// Set the Accepts header for the server, depending on the dataType
		jqXHR.setRequestHeader(
			"Accept",
			s.dataTypes[ 0 ] && s.accepts[ s.dataTypes[0] ] ?
				s.accepts[ s.dataTypes[0] ] + ( s.dataTypes[ 0 ] !== "*" ? ", " + allTypes + "; q=0.01" : "" ) :
				s.accepts[ "*" ]
		);

		// Check for headers option
		for ( i in s.headers ) {
			jqXHR.setRequestHeader( i, s.headers[ i ] );
		}

		// Allow custom headers/mimetypes and early abort
		if ( s.beforeSend && ( s.beforeSend.call( callbackContext, jqXHR, s ) === false || state === 2 ) ) {
			// Abort if not done already and return
			return jqXHR.abort();
		}

		// aborting is no longer a cancellation
		strAbort = "abort";

		// Install callbacks on deferreds
		for ( i in { success: 1, error: 1, complete: 1 } ) {
			jqXHR[ i ]( s[ i ] );
		}

		// Get transport
		transport = inspectPrefiltersOrTransports( transports, s, options, jqXHR );

		// If no transport, we auto-abort
		if ( !transport ) {
			done( -1, "No Transport" );
		} else {
			jqXHR.readyState = 1;

			// Send global event
			if ( fireGlobals ) {
				globalEventContext.trigger( "ajaxSend", [ jqXHR, s ] );
			}
			// Timeout
			if ( s.async && s.timeout > 0 ) {
				timeoutTimer = setTimeout(function() {
					jqXHR.abort("timeout");
				}, s.timeout );
			}

			try {
				state = 1;
				transport.send( requestHeaders, done );
			} catch ( e ) {
				// Propagate exception as error if not done
				if ( state < 2 ) {
					done( -1, e );
				// Simply rethrow otherwise
				} else {
					throw e;
				}
			}
		}

		// Callback for when everything is done
		function done( status, nativeStatusText, responses, headers ) {
			var isSuccess, success, error, response, modified,
				statusText = nativeStatusText;

			// Called once
			if ( state === 2 ) {
				return;
			}

			// State is "done" now
			state = 2;

			// Clear timeout if it exists
			if ( timeoutTimer ) {
				clearTimeout( timeoutTimer );
			}

			// Dereference transport for early garbage collection
			// (no matter how long the jqXHR object will be used)
			transport = undefined;

			// Cache response headers
			responseHeadersString = headers || "";

			// Set readyState
			jqXHR.readyState = status > 0 ? 4 : 0;

			// Determine if successful
			isSuccess = status >= 200 && status < 300 || status === 304;

			// Get response data
			if ( responses ) {
				response = ajaxHandleResponses( s, jqXHR, responses );
			}

			// Convert no matter what (that way responseXXX fields are always set)
			response = ajaxConvert( s, response, jqXHR, isSuccess );

			// If successful, handle type chaining
			if ( isSuccess ) {

				// Set the If-Modified-Since and/or If-None-Match header, if in ifModified mode.
				if ( s.ifModified ) {
					modified = jqXHR.getResponseHeader("Last-Modified");
					if ( modified ) {
						jQuery.lastModified[ cacheURL ] = modified;
					}
					modified = jqXHR.getResponseHeader("etag");
					if ( modified ) {
						jQuery.etag[ cacheURL ] = modified;
					}
				}

				// if no content
				if ( status === 204 || s.type === "HEAD" ) {
					statusText = "nocontent";

				// if not modified
				} else if ( status === 304 ) {
					statusText = "notmodified";

				// If we have data, let's convert it
				} else {
					statusText = response.state;
					success = response.data;
					error = response.error;
					isSuccess = !error;
				}
			} else {
				// We extract error from statusText
				// then normalize statusText and status for non-aborts
				error = statusText;
				if ( status || !statusText ) {
					statusText = "error";
					if ( status < 0 ) {
						status = 0;
					}
				}
			}

			// Set data for the fake xhr object
			jqXHR.status = status;
			jqXHR.statusText = ( nativeStatusText || statusText ) + "";

			// Success/Error
			if ( isSuccess ) {
				deferred.resolveWith( callbackContext, [ success, statusText, jqXHR ] );
			} else {
				deferred.rejectWith( callbackContext, [ jqXHR, statusText, error ] );
			}

			// Status-dependent callbacks
			jqXHR.statusCode( statusCode );
			statusCode = undefined;

			if ( fireGlobals ) {
				globalEventContext.trigger( isSuccess ? "ajaxSuccess" : "ajaxError",
					[ jqXHR, s, isSuccess ? success : error ] );
			}

			// Complete
			completeDeferred.fireWith( callbackContext, [ jqXHR, statusText ] );

			if ( fireGlobals ) {
				globalEventContext.trigger( "ajaxComplete", [ jqXHR, s ] );
				// Handle the global AJAX counter
				if ( !( --jQuery.active ) ) {
					jQuery.event.trigger("ajaxStop");
				}
			}
		}

		return jqXHR;
	},

	getJSON: function( url, data, callback ) {
		return jQuery.get( url, data, callback, "json" );
	},

	getScript: function( url, callback ) {
		return jQuery.get( url, undefined, callback, "script" );
	}
});

jQuery.each( [ "get", "post" ], function( i, method ) {
	jQuery[ method ] = function( url, data, callback, type ) {
		// shift arguments if data argument was omitted
		if ( jQuery.isFunction( data ) ) {
			type = type || callback;
			callback = data;
			data = undefined;
		}

		return jQuery.ajax({
			url: url,
			type: method,
			dataType: type,
			data: data,
			success: callback
		});
	};
});

// Attach a bunch of functions for handling common AJAX events
jQuery.each( [ "ajaxStart", "ajaxStop", "ajaxComplete", "ajaxError", "ajaxSuccess", "ajaxSend" ], function( i, type ) {
	jQuery.fn[ type ] = function( fn ) {
		return this.on( type, fn );
	};
});


jQuery._evalUrl = function( url ) {
	return jQuery.ajax({
		url: url,
		type: "GET",
		dataType: "script",
		async: false,
		global: false,
		"throws": true
	});
};


jQuery.fn.extend({
	wrapAll: function( html ) {
		var wrap;

		if ( jQuery.isFunction( html ) ) {
			return this.each(function( i ) {
				jQuery( this ).wrapAll( html.call(this, i) );
			});
		}

		if ( this[ 0 ] ) {

			// The elements to wrap the target around
			wrap = jQuery( html, this[ 0 ].ownerDocument ).eq( 0 ).clone( true );

			if ( this[ 0 ].parentNode ) {
				wrap.insertBefore( this[ 0 ] );
			}

			wrap.map(function() {
				var elem = this;

				while ( elem.firstElementChild ) {
					elem = elem.firstElementChild;
				}

				return elem;
			}).append( this );
		}

		return this;
	},

	wrapInner: function( html ) {
		if ( jQuery.isFunction( html ) ) {
			return this.each(function( i ) {
				jQuery( this ).wrapInner( html.call(this, i) );
			});
		}

		return this.each(function() {
			var self = jQuery( this ),
				contents = self.contents();

			if ( contents.length ) {
				contents.wrapAll( html );

			} else {
				self.append( html );
			}
		});
	},

	wrap: function( html ) {
		var isFunction = jQuery.isFunction( html );

		return this.each(function( i ) {
			jQuery( this ).wrapAll( isFunction ? html.call(this, i) : html );
		});
	},

	unwrap: function() {
		return this.parent().each(function() {
			if ( !jQuery.nodeName( this, "body" ) ) {
				jQuery( this ).replaceWith( this.childNodes );
			}
		}).end();
	}
});


jQuery.expr.filters.hidden = function( elem ) {
	// Support: Opera <= 12.12
	// Opera reports offsetWidths and offsetHeights less than zero on some elements
	return elem.offsetWidth <= 0 && elem.offsetHeight <= 0;
};
jQuery.expr.filters.visible = function( elem ) {
	return !jQuery.expr.filters.hidden( elem );
};




var r20 = /%20/g,
	rbracket = /\[\]$/,
	rCRLF = /\r?\n/g,
	rsubmitterTypes = /^(?:submit|button|image|reset|file)$/i,
	rsubmittable = /^(?:input|select|textarea|keygen)/i;

function buildParams( prefix, obj, traditional, add ) {
	var name;

	if ( jQuery.isArray( obj ) ) {
		// Serialize array item.
		jQuery.each( obj, function( i, v ) {
			if ( traditional || rbracket.test( prefix ) ) {
				// Treat each array item as a scalar.
				add( prefix, v );

			} else {
				// Item is non-scalar (array or object), encode its numeric index.
				buildParams( prefix + "[" + ( typeof v === "object" ? i : "" ) + "]", v, traditional, add );
			}
		});

	} else if ( !traditional && jQuery.type( obj ) === "object" ) {
		// Serialize object item.
		for ( name in obj ) {
			buildParams( prefix + "[" + name + "]", obj[ name ], traditional, add );
		}

	} else {
		// Serialize scalar item.
		add( prefix, obj );
	}
}

// Serialize an array of form elements or a set of
// key/values into a query string
jQuery.param = function( a, traditional ) {
	var prefix,
		s = [],
		add = function( key, value ) {
			// If value is a function, invoke it and return its value
			value = jQuery.isFunction( value ) ? value() : ( value == null ? "" : value );
			s[ s.length ] = encodeURIComponent( key ) + "=" + encodeURIComponent( value );
		};

	// Set traditional to true for jQuery <= 1.3.2 behavior.
	if ( traditional === undefined ) {
		traditional = jQuery.ajaxSettings && jQuery.ajaxSettings.traditional;
	}

	// If an array was passed in, assume that it is an array of form elements.
	if ( jQuery.isArray( a ) || ( a.jquery && !jQuery.isPlainObject( a ) ) ) {
		// Serialize the form elements
		jQuery.each( a, function() {
			add( this.name, this.value );
		});

	} else {
		// If traditional, encode the "old" way (the way 1.3.2 or older
		// did it), otherwise encode params recursively.
		for ( prefix in a ) {
			buildParams( prefix, a[ prefix ], traditional, add );
		}
	}

	// Return the resulting serialization
	return s.join( "&" ).replace( r20, "+" );
};

jQuery.fn.extend({
	serialize: function() {
		return jQuery.param( this.serializeArray() );
	},
	serializeArray: function() {
		return this.map(function() {
			// Can add propHook for "elements" to filter or add form elements
			var elements = jQuery.prop( this, "elements" );
			return elements ? jQuery.makeArray( elements ) : this;
		})
		.filter(function() {
			var type = this.type;

			// Use .is( ":disabled" ) so that fieldset[disabled] works
			return this.name && !jQuery( this ).is( ":disabled" ) &&
				rsubmittable.test( this.nodeName ) && !rsubmitterTypes.test( type ) &&
				( this.checked || !rcheckableType.test( type ) );
		})
		.map(function( i, elem ) {
			var val = jQuery( this ).val();

			return val == null ?
				null :
				jQuery.isArray( val ) ?
					jQuery.map( val, function( val ) {
						return { name: elem.name, value: val.replace( rCRLF, "\r\n" ) };
					}) :
					{ name: elem.name, value: val.replace( rCRLF, "\r\n" ) };
		}).get();
	}
});


jQuery.ajaxSettings.xhr = function() {
	try {
		return new XMLHttpRequest();
	} catch( e ) {}
};

var xhrId = 0,
	xhrCallbacks = {},
	xhrSuccessStatus = {
		// file protocol always yields status code 0, assume 200
		0: 200,
		// Support: IE9
		// #1450: sometimes IE returns 1223 when it should be 204
		1223: 204
	},
	xhrSupported = jQuery.ajaxSettings.xhr();

// Support: IE9
// Open requests must be manually aborted on unload (#5280)
if ( window.ActiveXObject ) {
	jQuery( window ).on( "unload", function() {
		for ( var key in xhrCallbacks ) {
			xhrCallbacks[ key ]();
		}
	});
}

support.cors = !!xhrSupported && ( "withCredentials" in xhrSupported );
support.ajax = xhrSupported = !!xhrSupported;

jQuery.ajaxTransport(function( options ) {
	var callback;

	// Cross domain only allowed if supported through XMLHttpRequest
	if ( support.cors || xhrSupported && !options.crossDomain ) {
		return {
			send: function( headers, complete ) {
				var i,
					xhr = options.xhr(),
					id = ++xhrId;

				xhr.open( options.type, options.url, options.async, options.username, options.password );

				// Apply custom fields if provided
				if ( options.xhrFields ) {
					for ( i in options.xhrFields ) {
						xhr[ i ] = options.xhrFields[ i ];
					}
				}

				// Override mime type if needed
				if ( options.mimeType && xhr.overrideMimeType ) {
					xhr.overrideMimeType( options.mimeType );
				}

				// X-Requested-With header
				// For cross-domain requests, seeing as conditions for a preflight are
				// akin to a jigsaw puzzle, we simply never set it to be sure.
				// (it can always be set on a per-request basis or even using ajaxSetup)
				// For same-domain requests, won't change header if already provided.
				if ( !options.crossDomain && !headers["X-Requested-With"] ) {
					headers["X-Requested-With"] = "XMLHttpRequest";
				}

				// Set headers
				for ( i in headers ) {
					xhr.setRequestHeader( i, headers[ i ] );
				}

				// Callback
				callback = function( type ) {
					return function() {
						if ( callback ) {
							delete xhrCallbacks[ id ];
							callback = xhr.onload = xhr.onerror = null;

							if ( type === "abort" ) {
								xhr.abort();
							} else if ( type === "error" ) {
								complete(
									// file: protocol always yields status 0; see #8605, #14207
									xhr.status,
									xhr.statusText
								);
							} else {
								complete(
									xhrSuccessStatus[ xhr.status ] || xhr.status,
									xhr.statusText,
									// Support: IE9
									// Accessing binary-data responseText throws an exception
									// (#11426)
									typeof xhr.responseText === "string" ? {
										text: xhr.responseText
									} : undefined,
									xhr.getAllResponseHeaders()
								);
							}
						}
					};
				};

				// Listen to events
				xhr.onload = callback();
				xhr.onerror = callback("error");

				// Create the abort callback
				callback = xhrCallbacks[ id ] = callback("abort");

				// Do send the request
				// This may raise an exception which is actually
				// handled in jQuery.ajax (so no try/catch here)
				xhr.send( options.hasContent && options.data || null );
			},

			abort: function() {
				if ( callback ) {
					callback();
				}
			}
		};
	}
});




// Install script dataType
jQuery.ajaxSetup({
	accepts: {
		script: "text/javascript, application/javascript, application/ecmascript, application/x-ecmascript"
	},
	contents: {
		script: /(?:java|ecma)script/
	},
	converters: {
		"text script": function( text ) {
			jQuery.globalEval( text );
			return text;
		}
	}
});

// Handle cache's special case and crossDomain
jQuery.ajaxPrefilter( "script", function( s ) {
	if ( s.cache === undefined ) {
		s.cache = false;
	}
	if ( s.crossDomain ) {
		s.type = "GET";
	}
});

// Bind script tag hack transport
jQuery.ajaxTransport( "script", function( s ) {
	// This transport only deals with cross domain requests
	if ( s.crossDomain ) {
		var script, callback;
		return {
			send: function( _, complete ) {
				script = jQuery("<script>").prop({
					async: true,
					charset: s.scriptCharset,
					src: s.url
				}).on(
					"load error",
					callback = function( evt ) {
						script.remove();
						callback = null;
						if ( evt ) {
							complete( evt.type === "error" ? 404 : 200, evt.type );
						}
					}
				);
				document.head.appendChild( script[ 0 ] );
			},
			abort: function() {
				if ( callback ) {
					callback();
				}
			}
		};
	}
});




var oldCallbacks = [],
	rjsonp = /(=)\?(?=&|$)|\?\?/;

// Default jsonp settings
jQuery.ajaxSetup({
	jsonp: "callback",
	jsonpCallback: function() {
		var callback = oldCallbacks.pop() || ( jQuery.expando + "_" + ( nonce++ ) );
		this[ callback ] = true;
		return callback;
	}
});

// Detect, normalize options and install callbacks for jsonp requests
jQuery.ajaxPrefilter( "json jsonp", function( s, originalSettings, jqXHR ) {

	var callbackName, overwritten, responseContainer,
		jsonProp = s.jsonp !== false && ( rjsonp.test( s.url ) ?
			"url" :
			typeof s.data === "string" && !( s.contentType || "" ).indexOf("application/x-www-form-urlencoded") && rjsonp.test( s.data ) && "data"
		);

	// Handle iff the expected data type is "jsonp" or we have a parameter to set
	if ( jsonProp || s.dataTypes[ 0 ] === "jsonp" ) {

		// Get callback name, remembering preexisting value associated with it
		callbackName = s.jsonpCallback = jQuery.isFunction( s.jsonpCallback ) ?
			s.jsonpCallback() :
			s.jsonpCallback;

		// Insert callback into url or form data
		if ( jsonProp ) {
			s[ jsonProp ] = s[ jsonProp ].replace( rjsonp, "$1" + callbackName );
		} else if ( s.jsonp !== false ) {
			s.url += ( rquery.test( s.url ) ? "&" : "?" ) + s.jsonp + "=" + callbackName;
		}

		// Use data converter to retrieve json after script execution
		s.converters["script json"] = function() {
			if ( !responseContainer ) {
				jQuery.error( callbackName + " was not called" );
			}
			return responseContainer[ 0 ];
		};

		// force json dataType
		s.dataTypes[ 0 ] = "json";

		// Install callback
		overwritten = window[ callbackName ];
		window[ callbackName ] = function() {
			responseContainer = arguments;
		};

		// Clean-up function (fires after converters)
		jqXHR.always(function() {
			// Restore preexisting value
			window[ callbackName ] = overwritten;

			// Save back as free
			if ( s[ callbackName ] ) {
				// make sure that re-using the options doesn't screw things around
				s.jsonpCallback = originalSettings.jsonpCallback;

				// save the callback name for future use
				oldCallbacks.push( callbackName );
			}

			// Call if it was a function and we have a response
			if ( responseContainer && jQuery.isFunction( overwritten ) ) {
				overwritten( responseContainer[ 0 ] );
			}

			responseContainer = overwritten = undefined;
		});

		// Delegate to script
		return "script";
	}
});




// data: string of html
// context (optional): If specified, the fragment will be created in this context, defaults to document
// keepScripts (optional): If true, will include scripts passed in the html string
jQuery.parseHTML = function( data, context, keepScripts ) {
	if ( !data || typeof data !== "string" ) {
		return null;
	}
	if ( typeof context === "boolean" ) {
		keepScripts = context;
		context = false;
	}
	context = context || document;

	var parsed = rsingleTag.exec( data ),
		scripts = !keepScripts && [];

	// Single tag
	if ( parsed ) {
		return [ context.createElement( parsed[1] ) ];
	}

	parsed = jQuery.buildFragment( [ data ], context, scripts );

	if ( scripts && scripts.length ) {
		jQuery( scripts ).remove();
	}

	return jQuery.merge( [], parsed.childNodes );
};


// Keep a copy of the old load method
var _load = jQuery.fn.load;

/**
 * Load a url into a page
 */
jQuery.fn.load = function( url, params, callback ) {
	if ( typeof url !== "string" && _load ) {
		return _load.apply( this, arguments );
	}

	var selector, type, response,
		self = this,
		off = url.indexOf(" ");

	if ( off >= 0 ) {
		selector = url.slice( off );
		url = url.slice( 0, off );
	}

	// If it's a function
	if ( jQuery.isFunction( params ) ) {

		// We assume that it's the callback
		callback = params;
		params = undefined;

	// Otherwise, build a param string
	} else if ( params && typeof params === "object" ) {
		type = "POST";
	}

	// If we have elements to modify, make the request
	if ( self.length > 0 ) {
		jQuery.ajax({
			url: url,

			// if "type" variable is undefined, then "GET" method will be used
			type: type,
			dataType: "html",
			data: params
		}).done(function( responseText ) {

			// Save response for use in complete callback
			response = arguments;

			self.html( selector ?

				// If a selector was specified, locate the right elements in a dummy div
				// Exclude scripts to avoid IE 'Permission Denied' errors
				jQuery("<div>").append( jQuery.parseHTML( responseText ) ).find( selector ) :

				// Otherwise use the full result
				responseText );

		}).complete( callback && function( jqXHR, status ) {
			self.each( callback, response || [ jqXHR.responseText, status, jqXHR ] );
		});
	}

	return this;
};




jQuery.expr.filters.animated = function( elem ) {
	return jQuery.grep(jQuery.timers, function( fn ) {
		return elem === fn.elem;
	}).length;
};




var docElem = window.document.documentElement;

/**
 * Gets a window from an element
 */
function getWindow( elem ) {
	return jQuery.isWindow( elem ) ? elem : elem.nodeType === 9 && elem.defaultView;
}

jQuery.offset = {
	setOffset: function( elem, options, i ) {
		var curPosition, curLeft, curCSSTop, curTop, curOffset, curCSSLeft, calculatePosition,
			position = jQuery.css( elem, "position" ),
			curElem = jQuery( elem ),
			props = {};

		// Set position first, in-case top/left are set even on static elem
		if ( position === "static" ) {
			elem.style.position = "relative";
		}

		curOffset = curElem.offset();
		curCSSTop = jQuery.css( elem, "top" );
		curCSSLeft = jQuery.css( elem, "left" );
		calculatePosition = ( position === "absolute" || position === "fixed" ) &&
			( curCSSTop + curCSSLeft ).indexOf("auto") > -1;

		// Need to be able to calculate position if either top or left is auto and position is either absolute or fixed
		if ( calculatePosition ) {
			curPosition = curElem.position();
			curTop = curPosition.top;
			curLeft = curPosition.left;

		} else {
			curTop = parseFloat( curCSSTop ) || 0;
			curLeft = parseFloat( curCSSLeft ) || 0;
		}

		if ( jQuery.isFunction( options ) ) {
			options = options.call( elem, i, curOffset );
		}

		if ( options.top != null ) {
			props.top = ( options.top - curOffset.top ) + curTop;
		}
		if ( options.left != null ) {
			props.left = ( options.left - curOffset.left ) + curLeft;
		}

		if ( "using" in options ) {
			options.using.call( elem, props );

		} else {
			curElem.css( props );
		}
	}
};

jQuery.fn.extend({
	offset: function( options ) {
		if ( arguments.length ) {
			return options === undefined ?
				this :
				this.each(function( i ) {
					jQuery.offset.setOffset( this, options, i );
				});
		}

		var docElem, win,
			elem = this[ 0 ],
			box = { top: 0, left: 0 },
			doc = elem && elem.ownerDocument;

		if ( !doc ) {
			return;
		}

		docElem = doc.documentElement;

		// Make sure it's not a disconnected DOM node
		if ( !jQuery.contains( docElem, elem ) ) {
			return box;
		}

		// If we don't have gBCR, just use 0,0 rather than error
		// BlackBerry 5, iOS 3 (original iPhone)
		if ( typeof elem.getBoundingClientRect !== strundefined ) {
			box = elem.getBoundingClientRect();
		}
		win = getWindow( doc );
		return {
			top: box.top + win.pageYOffset - docElem.clientTop,
			left: box.left + win.pageXOffset - docElem.clientLeft
		};
	},

	position: function() {
		if ( !this[ 0 ] ) {
			return;
		}

		var offsetParent, offset,
			elem = this[ 0 ],
			parentOffset = { top: 0, left: 0 };

		// Fixed elements are offset from window (parentOffset = {top:0, left: 0}, because it is its only offset parent
		if ( jQuery.css( elem, "position" ) === "fixed" ) {
			// We assume that getBoundingClientRect is available when computed position is fixed
			offset = elem.getBoundingClientRect();

		} else {
			// Get *real* offsetParent
			offsetParent = this.offsetParent();

			// Get correct offsets
			offset = this.offset();
			if ( !jQuery.nodeName( offsetParent[ 0 ], "html" ) ) {
				parentOffset = offsetParent.offset();
			}

			// Add offsetParent borders
			parentOffset.top += jQuery.css( offsetParent[ 0 ], "borderTopWidth", true );
			parentOffset.left += jQuery.css( offsetParent[ 0 ], "borderLeftWidth", true );
		}

		// Subtract parent offsets and element margins
		return {
			top: offset.top - parentOffset.top - jQuery.css( elem, "marginTop", true ),
			left: offset.left - parentOffset.left - jQuery.css( elem, "marginLeft", true )
		};
	},

	offsetParent: function() {
		return this.map(function() {
			var offsetParent = this.offsetParent || docElem;

			while ( offsetParent && ( !jQuery.nodeName( offsetParent, "html" ) && jQuery.css( offsetParent, "position" ) === "static" ) ) {
				offsetParent = offsetParent.offsetParent;
			}

			return offsetParent || docElem;
		});
	}
});

// Create scrollLeft and scrollTop methods
jQuery.each( { scrollLeft: "pageXOffset", scrollTop: "pageYOffset" }, function( method, prop ) {
	var top = "pageYOffset" === prop;

	jQuery.fn[ method ] = function( val ) {
		return access( this, function( elem, method, val ) {
			var win = getWindow( elem );

			if ( val === undefined ) {
				return win ? win[ prop ] : elem[ method ];
			}

			if ( win ) {
				win.scrollTo(
					!top ? val : window.pageXOffset,
					top ? val : window.pageYOffset
				);

			} else {
				elem[ method ] = val;
			}
		}, method, val, arguments.length, null );
	};
});

// Add the top/left cssHooks using jQuery.fn.position
// Webkit bug: https://bugs.webkit.org/show_bug.cgi?id=29084
// getComputedStyle returns percent when specified for top/left/bottom/right
// rather than make the css module depend on the offset module, we just check for it here
jQuery.each( [ "top", "left" ], function( i, prop ) {
	jQuery.cssHooks[ prop ] = addGetHookIf( support.pixelPosition,
		function( elem, computed ) {
			if ( computed ) {
				computed = curCSS( elem, prop );
				// if curCSS returns percentage, fallback to offset
				return rnumnonpx.test( computed ) ?
					jQuery( elem ).position()[ prop ] + "px" :
					computed;
			}
		}
	);
});


// Create innerHeight, innerWidth, height, width, outerHeight and outerWidth methods
jQuery.each( { Height: "height", Width: "width" }, function( name, type ) {
	jQuery.each( { padding: "inner" + name, content: type, "": "outer" + name }, function( defaultExtra, funcName ) {
		// margin is only for outerHeight, outerWidth
		jQuery.fn[ funcName ] = function( margin, value ) {
			var chainable = arguments.length && ( defaultExtra || typeof margin !== "boolean" ),
				extra = defaultExtra || ( margin === true || value === true ? "margin" : "border" );

			return access( this, function( elem, type, value ) {
				var doc;

				if ( jQuery.isWindow( elem ) ) {
					// As of 5/8/2012 this will yield incorrect results for Mobile Safari, but there
					// isn't a whole lot we can do. See pull request at this URL for discussion:
					// https://github.com/jquery/jquery/pull/764
					return elem.document.documentElement[ "client" + name ];
				}

				// Get document width or height
				if ( elem.nodeType === 9 ) {
					doc = elem.documentElement;

					// Either scroll[Width/Height] or offset[Width/Height] or client[Width/Height],
					// whichever is greatest
					return Math.max(
						elem.body[ "scroll" + name ], doc[ "scroll" + name ],
						elem.body[ "offset" + name ], doc[ "offset" + name ],
						doc[ "client" + name ]
					);
				}

				return value === undefined ?
					// Get width or height on the element, requesting but not forcing parseFloat
					jQuery.css( elem, type, extra ) :

					// Set width or height on the element
					jQuery.style( elem, type, value, extra );
			}, type, chainable ? margin : undefined, chainable, null );
		};
	});
});


// The number of elements contained in the matched element set
jQuery.fn.size = function() {
	return this.length;
};

jQuery.fn.andSelf = jQuery.fn.addBack;




// Register as a named AMD module, since jQuery can be concatenated with other
// files that may use define, but not via a proper concatenation script that
// understands anonymous AMD modules. A named AMD is safest and most robust
// way to register. Lowercase jquery is used because AMD module names are
// derived from file names, and jQuery is normally delivered in a lowercase
// file name. Do this after creating the global so that if an AMD module wants
// to call noConflict to hide this version of jQuery, it will work.
if ( typeof define === "function" && define.amd ) {
	define( "jquery", [], function() {
		return jQuery;
	});
}




var
	// Map over jQuery in case of overwrite
	_jQuery = window.jQuery,

	// Map over the $ in case of overwrite
	_$ = window.$;

jQuery.noConflict = function( deep ) {
	if ( window.$ === jQuery ) {
		window.$ = _$;
	}

	if ( deep && window.jQuery === jQuery ) {
		window.jQuery = _jQuery;
	}

	return jQuery;
};

// Expose jQuery and $ identifiers, even in
// AMD (#7102#comment:10, https://github.com/jquery/jquery/pull/557)
// and CommonJS for browser emulators (#13566)
if ( typeof noGlobal === strundefined ) {
	window.jQuery = window.$ = jQuery;
}




return jQuery;

}));

//     Underscore.js 1.5.2
//     http://underscorejs.org
//     (c) 2009-2013 Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
//     Underscore may be freely distributed under the MIT license.

(function() {

  // Baseline setup
  // --------------

  // Establish the root object, `window` in the browser, or `exports` on the server.
  var root = this;

  // Save the previous value of the `_` variable.
  var previousUnderscore = root._;

  // Establish the object that gets returned to break out of a loop iteration.
  var breaker = {};

  // Save bytes in the minified (but not gzipped) version:
  var ArrayProto = Array.prototype, ObjProto = Object.prototype, FuncProto = Function.prototype;

  // Create quick reference variables for speed access to core prototypes.
  var
    push             = ArrayProto.push,
    slice            = ArrayProto.slice,
    concat           = ArrayProto.concat,
    toString         = ObjProto.toString,
    hasOwnProperty   = ObjProto.hasOwnProperty;

  // All **ECMAScript 5** native function implementations that we hope to use
  // are declared here.
  var
    nativeForEach      = ArrayProto.forEach,
    nativeMap          = ArrayProto.map,
    nativeReduce       = ArrayProto.reduce,
    nativeReduceRight  = ArrayProto.reduceRight,
    nativeFilter       = ArrayProto.filter,
    nativeEvery        = ArrayProto.every,
    nativeSome         = ArrayProto.some,
    nativeIndexOf      = ArrayProto.indexOf,
    nativeLastIndexOf  = ArrayProto.lastIndexOf,
    nativeIsArray      = Array.isArray,
    nativeKeys         = Object.keys,
    nativeBind         = FuncProto.bind;

  // Create a safe reference to the Underscore object for use below.
  var _ = function(obj) {
    if (obj instanceof _) return obj;
    if (!(this instanceof _)) return new _(obj);
    this._wrapped = obj;
  };

  // Export the Underscore object for **Node.js**, with
  // backwards-compatibility for the old `require()` API. If we're in
  // the browser, add `_` as a global object via a string identifier,
  // for Closure Compiler "advanced" mode.
  if (typeof exports !== 'undefined') {
    if (typeof module !== 'undefined' && module.exports) {
      exports = module.exports = _;
    }
    exports._ = _;
  } else {
    root._ = _;
  }

  // Current version.
  _.VERSION = '1.5.2';

  // Collection Functions
  // --------------------

  // The cornerstone, an `each` implementation, aka `forEach`.
  // Handles objects with the built-in `forEach`, arrays, and raw objects.
  // Delegates to **ECMAScript 5**'s native `forEach` if available.
  var each = _.each = _.forEach = function(obj, iterator, context) {
    if (obj == null) return;
    if (nativeForEach && obj.forEach === nativeForEach) {
      obj.forEach(iterator, context);
    } else if (obj.length === +obj.length) {
      for (var i = 0, length = obj.length; i < length; i++) {
        if (iterator.call(context, obj[i], i, obj) === breaker) return;
      }
    } else {
      var keys = _.keys(obj);
      for (var i = 0, length = keys.length; i < length; i++) {
        if (iterator.call(context, obj[keys[i]], keys[i], obj) === breaker) return;
      }
    }
  };

  // Return the results of applying the iterator to each element.
  // Delegates to **ECMAScript 5**'s native `map` if available.
  _.map = _.collect = function(obj, iterator, context) {
    var results = [];
    if (obj == null) return results;
    if (nativeMap && obj.map === nativeMap) return obj.map(iterator, context);
    each(obj, function(value, index, list) {
      results.push(iterator.call(context, value, index, list));
    });
    return results;
  };

  var reduceError = 'Reduce of empty array with no initial value';

  // **Reduce** builds up a single result from a list of values, aka `inject`,
  // or `foldl`. Delegates to **ECMAScript 5**'s native `reduce` if available.
  _.reduce = _.foldl = _.inject = function(obj, iterator, memo, context) {
    var initial = arguments.length > 2;
    if (obj == null) obj = [];
    if (nativeReduce && obj.reduce === nativeReduce) {
      if (context) iterator = _.bind(iterator, context);
      return initial ? obj.reduce(iterator, memo) : obj.reduce(iterator);
    }
    each(obj, function(value, index, list) {
      if (!initial) {
        memo = value;
        initial = true;
      } else {
        memo = iterator.call(context, memo, value, index, list);
      }
    });
    if (!initial) throw new TypeError(reduceError);
    return memo;
  };

  // The right-associative version of reduce, also known as `foldr`.
  // Delegates to **ECMAScript 5**'s native `reduceRight` if available.
  _.reduceRight = _.foldr = function(obj, iterator, memo, context) {
    var initial = arguments.length > 2;
    if (obj == null) obj = [];
    if (nativeReduceRight && obj.reduceRight === nativeReduceRight) {
      if (context) iterator = _.bind(iterator, context);
      return initial ? obj.reduceRight(iterator, memo) : obj.reduceRight(iterator);
    }
    var length = obj.length;
    if (length !== +length) {
      var keys = _.keys(obj);
      length = keys.length;
    }
    each(obj, function(value, index, list) {
      index = keys ? keys[--length] : --length;
      if (!initial) {
        memo = obj[index];
        initial = true;
      } else {
        memo = iterator.call(context, memo, obj[index], index, list);
      }
    });
    if (!initial) throw new TypeError(reduceError);
    return memo;
  };

  // Return the first value which passes a truth test. Aliased as `detect`.
  _.find = _.detect = function(obj, iterator, context) {
    var result;
    any(obj, function(value, index, list) {
      if (iterator.call(context, value, index, list)) {
        result = value;
        return true;
      }
    });
    return result;
  };

  // Return all the elements that pass a truth test.
  // Delegates to **ECMAScript 5**'s native `filter` if available.
  // Aliased as `select`.
  _.filter = _.select = function(obj, iterator, context) {
    var results = [];
    if (obj == null) return results;
    if (nativeFilter && obj.filter === nativeFilter) return obj.filter(iterator, context);
    each(obj, function(value, index, list) {
      if (iterator.call(context, value, index, list)) results.push(value);
    });
    return results;
  };

  // Return all the elements for which a truth test fails.
  _.reject = function(obj, iterator, context) {
    return _.filter(obj, function(value, index, list) {
      return !iterator.call(context, value, index, list);
    }, context);
  };

  // Determine whether all of the elements match a truth test.
  // Delegates to **ECMAScript 5**'s native `every` if available.
  // Aliased as `all`.
  _.every = _.all = function(obj, iterator, context) {
    iterator || (iterator = _.identity);
    var result = true;
    if (obj == null) return result;
    if (nativeEvery && obj.every === nativeEvery) return obj.every(iterator, context);
    each(obj, function(value, index, list) {
      if (!(result = result && iterator.call(context, value, index, list))) return breaker;
    });
    return !!result;
  };

  // Determine if at least one element in the object matches a truth test.
  // Delegates to **ECMAScript 5**'s native `some` if available.
  // Aliased as `any`.
  var any = _.some = _.any = function(obj, iterator, context) {
    iterator || (iterator = _.identity);
    var result = false;
    if (obj == null) return result;
    if (nativeSome && obj.some === nativeSome) return obj.some(iterator, context);
    each(obj, function(value, index, list) {
      if (result || (result = iterator.call(context, value, index, list))) return breaker;
    });
    return !!result;
  };

  // Determine if the array or object contains a given value (using `===`).
  // Aliased as `include`.
  _.contains = _.include = function(obj, target) {
    if (obj == null) return false;
    if (nativeIndexOf && obj.indexOf === nativeIndexOf) return obj.indexOf(target) != -1;
    return any(obj, function(value) {
      return value === target;
    });
  };

  // Invoke a method (with arguments) on every item in a collection.
  _.invoke = function(obj, method) {
    var args = slice.call(arguments, 2);
    var isFunc = _.isFunction(method);
    return _.map(obj, function(value) {
      return (isFunc ? method : value[method]).apply(value, args);
    });
  };

  // Convenience version of a common use case of `map`: fetching a property.
  _.pluck = function(obj, key) {
    return _.map(obj, function(value){ return value[key]; });
  };

  // Convenience version of a common use case of `filter`: selecting only objects
  // containing specific `key:value` pairs.
  _.where = function(obj, attrs, first) {
    if (_.isEmpty(attrs)) return first ? void 0 : [];
    return _[first ? 'find' : 'filter'](obj, function(value) {
      for (var key in attrs) {
        if (attrs[key] !== value[key]) return false;
      }
      return true;
    });
  };

  // Convenience version of a common use case of `find`: getting the first object
  // containing specific `key:value` pairs.
  _.findWhere = function(obj, attrs) {
    return _.where(obj, attrs, true);
  };

  // Return the maximum element or (element-based computation).
  // Can't optimize arrays of integers longer than 65,535 elements.
  // See [WebKit Bug 80797](https://bugs.webkit.org/show_bug.cgi?id=80797)
  _.max = function(obj, iterator, context) {
    if (!iterator && _.isArray(obj) && obj[0] === +obj[0] && obj.length < 65535) {
      return Math.max.apply(Math, obj);
    }
    if (!iterator && _.isEmpty(obj)) return -Infinity;
    var result = {computed : -Infinity, value: -Infinity};
    each(obj, function(value, index, list) {
      var computed = iterator ? iterator.call(context, value, index, list) : value;
      computed > result.computed && (result = {value : value, computed : computed});
    });
    return result.value;
  };

  // Return the minimum element (or element-based computation).
  _.min = function(obj, iterator, context) {
    if (!iterator && _.isArray(obj) && obj[0] === +obj[0] && obj.length < 65535) {
      return Math.min.apply(Math, obj);
    }
    if (!iterator && _.isEmpty(obj)) return Infinity;
    var result = {computed : Infinity, value: Infinity};
    each(obj, function(value, index, list) {
      var computed = iterator ? iterator.call(context, value, index, list) : value;
      computed < result.computed && (result = {value : value, computed : computed});
    });
    return result.value;
  };

  // Shuffle an array, using the modern version of the 
  // [Fisher-Yates shuffle](http://en.wikipedia.org/wiki/Fisher–Yates_shuffle).
  _.shuffle = function(obj) {
    var rand;
    var index = 0;
    var shuffled = [];
    each(obj, function(value) {
      rand = _.random(index++);
      shuffled[index - 1] = shuffled[rand];
      shuffled[rand] = value;
    });
    return shuffled;
  };

  // Sample **n** random values from an array.
  // If **n** is not specified, returns a single random element from the array.
  // The internal `guard` argument allows it to work with `map`.
  _.sample = function(obj, n, guard) {
    if (arguments.length < 2 || guard) {
      return obj[_.random(obj.length - 1)];
    }
    return _.shuffle(obj).slice(0, Math.max(0, n));
  };

  // An internal function to generate lookup iterators.
  var lookupIterator = function(value) {
    return _.isFunction(value) ? value : function(obj){ return obj[value]; };
  };

  // Sort the object's values by a criterion produced by an iterator.
  _.sortBy = function(obj, value, context) {
    var iterator = lookupIterator(value);
    return _.pluck(_.map(obj, function(value, index, list) {
      return {
        value: value,
        index: index,
        criteria: iterator.call(context, value, index, list)
      };
    }).sort(function(left, right) {
      var a = left.criteria;
      var b = right.criteria;
      if (a !== b) {
        if (a > b || a === void 0) return 1;
        if (a < b || b === void 0) return -1;
      }
      return left.index - right.index;
    }), 'value');
  };

  // An internal function used for aggregate "group by" operations.
  var group = function(behavior) {
    return function(obj, value, context) {
      var result = {};
      var iterator = value == null ? _.identity : lookupIterator(value);
      each(obj, function(value, index) {
        var key = iterator.call(context, value, index, obj);
        behavior(result, key, value);
      });
      return result;
    };
  };

  // Groups the object's values by a criterion. Pass either a string attribute
  // to group by, or a function that returns the criterion.
  _.groupBy = group(function(result, key, value) {
    (_.has(result, key) ? result[key] : (result[key] = [])).push(value);
  });

  // Indexes the object's values by a criterion, similar to `groupBy`, but for
  // when you know that your index values will be unique.
  _.indexBy = group(function(result, key, value) {
    result[key] = value;
  });

  // Counts instances of an object that group by a certain criterion. Pass
  // either a string attribute to count by, or a function that returns the
  // criterion.
  _.countBy = group(function(result, key) {
    _.has(result, key) ? result[key]++ : result[key] = 1;
  });

  // Use a comparator function to figure out the smallest index at which
  // an object should be inserted so as to maintain order. Uses binary search.
  _.sortedIndex = function(array, obj, iterator, context) {
    iterator = iterator == null ? _.identity : lookupIterator(iterator);
    var value = iterator.call(context, obj);
    var low = 0, high = array.length;
    while (low < high) {
      var mid = (low + high) >>> 1;
      iterator.call(context, array[mid]) < value ? low = mid + 1 : high = mid;
    }
    return low;
  };

  // Safely create a real, live array from anything iterable.
  _.toArray = function(obj) {
    if (!obj) return [];
    if (_.isArray(obj)) return slice.call(obj);
    if (obj.length === +obj.length) return _.map(obj, _.identity);
    return _.values(obj);
  };

  // Return the number of elements in an object.
  _.size = function(obj) {
    if (obj == null) return 0;
    return (obj.length === +obj.length) ? obj.length : _.keys(obj).length;
  };

  // Array Functions
  // ---------------

  // Get the first element of an array. Passing **n** will return the first N
  // values in the array. Aliased as `head` and `take`. The **guard** check
  // allows it to work with `_.map`.
  _.first = _.head = _.take = function(array, n, guard) {
    if (array == null) return void 0;
    return (n == null) || guard ? array[0] : slice.call(array, 0, n);
  };

  // Returns everything but the last entry of the array. Especially useful on
  // the arguments object. Passing **n** will return all the values in
  // the array, excluding the last N. The **guard** check allows it to work with
  // `_.map`.
  _.initial = function(array, n, guard) {
    return slice.call(array, 0, array.length - ((n == null) || guard ? 1 : n));
  };

  // Get the last element of an array. Passing **n** will return the last N
  // values in the array. The **guard** check allows it to work with `_.map`.
  _.last = function(array, n, guard) {
    if (array == null) return void 0;
    if ((n == null) || guard) {
      return array[array.length - 1];
    } else {
      return slice.call(array, Math.max(array.length - n, 0));
    }
  };

  // Returns everything but the first entry of the array. Aliased as `tail` and `drop`.
  // Especially useful on the arguments object. Passing an **n** will return
  // the rest N values in the array. The **guard**
  // check allows it to work with `_.map`.
  _.rest = _.tail = _.drop = function(array, n, guard) {
    return slice.call(array, (n == null) || guard ? 1 : n);
  };

  // Trim out all falsy values from an array.
  _.compact = function(array) {
    return _.filter(array, _.identity);
  };

  // Internal implementation of a recursive `flatten` function.
  var flatten = function(input, shallow, output) {
    if (shallow && _.every(input, _.isArray)) {
      return concat.apply(output, input);
    }
    each(input, function(value) {
      if (_.isArray(value) || _.isArguments(value)) {
        shallow ? push.apply(output, value) : flatten(value, shallow, output);
      } else {
        output.push(value);
      }
    });
    return output;
  };

  // Flatten out an array, either recursively (by default), or just one level.
  _.flatten = function(array, shallow) {
    return flatten(array, shallow, []);
  };

  // Return a version of the array that does not contain the specified value(s).
  _.without = function(array) {
    return _.difference(array, slice.call(arguments, 1));
  };

  // Produce a duplicate-free version of the array. If the array has already
  // been sorted, you have the option of using a faster algorithm.
  // Aliased as `unique`.
  _.uniq = _.unique = function(array, isSorted, iterator, context) {
    if (_.isFunction(isSorted)) {
      context = iterator;
      iterator = isSorted;
      isSorted = false;
    }
    var initial = iterator ? _.map(array, iterator, context) : array;
    var results = [];
    var seen = [];
    each(initial, function(value, index) {
      if (isSorted ? (!index || seen[seen.length - 1] !== value) : !_.contains(seen, value)) {
        seen.push(value);
        results.push(array[index]);
      }
    });
    return results;
  };

  // Produce an array that contains the union: each distinct element from all of
  // the passed-in arrays.
  _.union = function() {
    return _.uniq(_.flatten(arguments, true));
  };

  // Produce an array that contains every item shared between all the
  // passed-in arrays.
  _.intersection = function(array) {
    var rest = slice.call(arguments, 1);
    return _.filter(_.uniq(array), function(item) {
      return _.every(rest, function(other) {
        return _.indexOf(other, item) >= 0;
      });
    });
  };

  // Take the difference between one array and a number of other arrays.
  // Only the elements present in just the first array will remain.
  _.difference = function(array) {
    var rest = concat.apply(ArrayProto, slice.call(arguments, 1));
    return _.filter(array, function(value){ return !_.contains(rest, value); });
  };

  // Zip together multiple lists into a single array -- elements that share
  // an index go together.
  _.zip = function() {
    var length = _.max(_.pluck(arguments, "length").concat(0));
    var results = new Array(length);
    for (var i = 0; i < length; i++) {
      results[i] = _.pluck(arguments, '' + i);
    }
    return results;
  };

  // Converts lists into objects. Pass either a single array of `[key, value]`
  // pairs, or two parallel arrays of the same length -- one of keys, and one of
  // the corresponding values.
  _.object = function(list, values) {
    if (list == null) return {};
    var result = {};
    for (var i = 0, length = list.length; i < length; i++) {
      if (values) {
        result[list[i]] = values[i];
      } else {
        result[list[i][0]] = list[i][1];
      }
    }
    return result;
  };

  // If the browser doesn't supply us with indexOf (I'm looking at you, **MSIE**),
  // we need this function. Return the position of the first occurrence of an
  // item in an array, or -1 if the item is not included in the array.
  // Delegates to **ECMAScript 5**'s native `indexOf` if available.
  // If the array is large and already in sort order, pass `true`
  // for **isSorted** to use binary search.
  _.indexOf = function(array, item, isSorted) {
    if (array == null) return -1;
    var i = 0, length = array.length;
    if (isSorted) {
      if (typeof isSorted == 'number') {
        i = (isSorted < 0 ? Math.max(0, length + isSorted) : isSorted);
      } else {
        i = _.sortedIndex(array, item);
        return array[i] === item ? i : -1;
      }
    }
    if (nativeIndexOf && array.indexOf === nativeIndexOf) return array.indexOf(item, isSorted);
    for (; i < length; i++) if (array[i] === item) return i;
    return -1;
  };

  // Delegates to **ECMAScript 5**'s native `lastIndexOf` if available.
  _.lastIndexOf = function(array, item, from) {
    if (array == null) return -1;
    var hasIndex = from != null;
    if (nativeLastIndexOf && array.lastIndexOf === nativeLastIndexOf) {
      return hasIndex ? array.lastIndexOf(item, from) : array.lastIndexOf(item);
    }
    var i = (hasIndex ? from : array.length);
    while (i--) if (array[i] === item) return i;
    return -1;
  };

  // Generate an integer Array containing an arithmetic progression. A port of
  // the native Python `range()` function. See
  // [the Python documentation](http://docs.python.org/library/functions.html#range).
  _.range = function(start, stop, step) {
    if (arguments.length <= 1) {
      stop = start || 0;
      start = 0;
    }
    step = arguments[2] || 1;

    var length = Math.max(Math.ceil((stop - start) / step), 0);
    var idx = 0;
    var range = new Array(length);

    while(idx < length) {
      range[idx++] = start;
      start += step;
    }

    return range;
  };

  // Function (ahem) Functions
  // ------------------

  // Reusable constructor function for prototype setting.
  var ctor = function(){};

  // Create a function bound to a given object (assigning `this`, and arguments,
  // optionally). Delegates to **ECMAScript 5**'s native `Function.bind` if
  // available.
  _.bind = function(func, context) {
    var args, bound;
    if (nativeBind && func.bind === nativeBind) return nativeBind.apply(func, slice.call(arguments, 1));
    if (!_.isFunction(func)) throw new TypeError;
    args = slice.call(arguments, 2);
    return bound = function() {
      if (!(this instanceof bound)) return func.apply(context, args.concat(slice.call(arguments)));
      ctor.prototype = func.prototype;
      var self = new ctor;
      ctor.prototype = null;
      var result = func.apply(self, args.concat(slice.call(arguments)));
      if (Object(result) === result) return result;
      return self;
    };
  };

  // Partially apply a function by creating a version that has had some of its
  // arguments pre-filled, without changing its dynamic `this` context.
  _.partial = function(func) {
    var args = slice.call(arguments, 1);
    return function() {
      return func.apply(this, args.concat(slice.call(arguments)));
    };
  };

  // Bind all of an object's methods to that object. Useful for ensuring that
  // all callbacks defined on an object belong to it.
  _.bindAll = function(obj) {
    var funcs = slice.call(arguments, 1);
    if (funcs.length === 0) throw new Error("bindAll must be passed function names");
    each(funcs, function(f) { obj[f] = _.bind(obj[f], obj); });
    return obj;
  };

  // Memoize an expensive function by storing its results.
  _.memoize = function(func, hasher) {
    var memo = {};
    hasher || (hasher = _.identity);
    return function() {
      var key = hasher.apply(this, arguments);
      return _.has(memo, key) ? memo[key] : (memo[key] = func.apply(this, arguments));
    };
  };

  // Delays a function for the given number of milliseconds, and then calls
  // it with the arguments supplied.
  _.delay = function(func, wait) {
    var args = slice.call(arguments, 2);
    return setTimeout(function(){ return func.apply(null, args); }, wait);
  };

  // Defers a function, scheduling it to run after the current call stack has
  // cleared.
  _.defer = function(func) {
    return _.delay.apply(_, [func, 1].concat(slice.call(arguments, 1)));
  };

  // Returns a function, that, when invoked, will only be triggered at most once
  // during a given window of time. Normally, the throttled function will run
  // as much as it can, without ever going more than once per `wait` duration;
  // but if you'd like to disable the execution on the leading edge, pass
  // `{leading: false}`. To disable execution on the trailing edge, ditto.
  _.throttle = function(func, wait, options) {
    var context, args, result;
    var timeout = null;
    var previous = 0;
    options || (options = {});
    var later = function() {
      previous = options.leading === false ? 0 : new Date;
      timeout = null;
      result = func.apply(context, args);
    };
    return function() {
      var now = new Date;
      if (!previous && options.leading === false) previous = now;
      var remaining = wait - (now - previous);
      context = this;
      args = arguments;
      if (remaining <= 0) {
        clearTimeout(timeout);
        timeout = null;
        previous = now;
        result = func.apply(context, args);
      } else if (!timeout && options.trailing !== false) {
        timeout = setTimeout(later, remaining);
      }
      return result;
    };
  };

  // Returns a function, that, as long as it continues to be invoked, will not
  // be triggered. The function will be called after it stops being called for
  // N milliseconds. If `immediate` is passed, trigger the function on the
  // leading edge, instead of the trailing.
  _.debounce = function(func, wait, immediate) {
    var timeout, args, context, timestamp, result;
    return function() {
      context = this;
      args = arguments;
      timestamp = new Date();
      var later = function() {
        var last = (new Date()) - timestamp;
        if (last < wait) {
          timeout = setTimeout(later, wait - last);
        } else {
          timeout = null;
          if (!immediate) result = func.apply(context, args);
        }
      };
      var callNow = immediate && !timeout;
      if (!timeout) {
        timeout = setTimeout(later, wait);
      }
      if (callNow) result = func.apply(context, args);
      return result;
    };
  };

  // Returns a function that will be executed at most one time, no matter how
  // often you call it. Useful for lazy initialization.
  _.once = function(func) {
    var ran = false, memo;
    return function() {
      if (ran) return memo;
      ran = true;
      memo = func.apply(this, arguments);
      func = null;
      return memo;
    };
  };

  // Returns the first function passed as an argument to the second,
  // allowing you to adjust arguments, run code before and after, and
  // conditionally execute the original function.
  _.wrap = function(func, wrapper) {
    return function() {
      var args = [func];
      push.apply(args, arguments);
      return wrapper.apply(this, args);
    };
  };

  // Returns a function that is the composition of a list of functions, each
  // consuming the return value of the function that follows.
  _.compose = function() {
    var funcs = arguments;
    return function() {
      var args = arguments;
      for (var i = funcs.length - 1; i >= 0; i--) {
        args = [funcs[i].apply(this, args)];
      }
      return args[0];
    };
  };

  // Returns a function that will only be executed after being called N times.
  _.after = function(times, func) {
    return function() {
      if (--times < 1) {
        return func.apply(this, arguments);
      }
    };
  };

  // Object Functions
  // ----------------

  // Retrieve the names of an object's properties.
  // Delegates to **ECMAScript 5**'s native `Object.keys`
  _.keys = nativeKeys || function(obj) {
    if (obj !== Object(obj)) throw new TypeError('Invalid object');
    var keys = [];
    for (var key in obj) if (_.has(obj, key)) keys.push(key);
    return keys;
  };

  // Retrieve the values of an object's properties.
  _.values = function(obj) {
    var keys = _.keys(obj);
    var length = keys.length;
    var values = new Array(length);
    for (var i = 0; i < length; i++) {
      values[i] = obj[keys[i]];
    }
    return values;
  };

  // Convert an object into a list of `[key, value]` pairs.
  _.pairs = function(obj) {
    var keys = _.keys(obj);
    var length = keys.length;
    var pairs = new Array(length);
    for (var i = 0; i < length; i++) {
      pairs[i] = [keys[i], obj[keys[i]]];
    }
    return pairs;
  };

  // Invert the keys and values of an object. The values must be serializable.
  _.invert = function(obj) {
    var result = {};
    var keys = _.keys(obj);
    for (var i = 0, length = keys.length; i < length; i++) {
      result[obj[keys[i]]] = keys[i];
    }
    return result;
  };

  // Return a sorted list of the function names available on the object.
  // Aliased as `methods`
  _.functions = _.methods = function(obj) {
    var names = [];
    for (var key in obj) {
      if (_.isFunction(obj[key])) names.push(key);
    }
    return names.sort();
  };

  // Extend a given object with all the properties in passed-in object(s).
  _.extend = function(obj) {
    each(slice.call(arguments, 1), function(source) {
      if (source) {
        for (var prop in source) {
          obj[prop] = source[prop];
        }
      }
    });
    return obj;
  };

  // Return a copy of the object only containing the whitelisted properties.
  _.pick = function(obj) {
    var copy = {};
    var keys = concat.apply(ArrayProto, slice.call(arguments, 1));
    each(keys, function(key) {
      if (key in obj) copy[key] = obj[key];
    });
    return copy;
  };

   // Return a copy of the object without the blacklisted properties.
  _.omit = function(obj) {
    var copy = {};
    var keys = concat.apply(ArrayProto, slice.call(arguments, 1));
    for (var key in obj) {
      if (!_.contains(keys, key)) copy[key] = obj[key];
    }
    return copy;
  };

  // Fill in a given object with default properties.
  _.defaults = function(obj) {
    each(slice.call(arguments, 1), function(source) {
      if (source) {
        for (var prop in source) {
          if (obj[prop] === void 0) obj[prop] = source[prop];
        }
      }
    });
    return obj;
  };

  // Create a (shallow-cloned) duplicate of an object.
  _.clone = function(obj) {
    if (!_.isObject(obj)) return obj;
    return _.isArray(obj) ? obj.slice() : _.extend({}, obj);
  };

  // Invokes interceptor with the obj, and then returns obj.
  // The primary purpose of this method is to "tap into" a method chain, in
  // order to perform operations on intermediate results within the chain.
  _.tap = function(obj, interceptor) {
    interceptor(obj);
    return obj;
  };

  // Internal recursive comparison function for `isEqual`.
  var eq = function(a, b, aStack, bStack) {
    // Identical objects are equal. `0 === -0`, but they aren't identical.
    // See the [Harmony `egal` proposal](http://wiki.ecmascript.org/doku.php?id=harmony:egal).
    if (a === b) return a !== 0 || 1 / a == 1 / b;
    // A strict comparison is necessary because `null == undefined`.
    if (a == null || b == null) return a === b;
    // Unwrap any wrapped objects.
    if (a instanceof _) a = a._wrapped;
    if (b instanceof _) b = b._wrapped;
    // Compare `[[Class]]` names.
    var className = toString.call(a);
    if (className != toString.call(b)) return false;
    switch (className) {
      // Strings, numbers, dates, and booleans are compared by value.
      case '[object String]':
        // Primitives and their corresponding object wrappers are equivalent; thus, `"5"` is
        // equivalent to `new String("5")`.
        return a == String(b);
      case '[object Number]':
        // `NaN`s are equivalent, but non-reflexive. An `egal` comparison is performed for
        // other numeric values.
        return a != +a ? b != +b : (a == 0 ? 1 / a == 1 / b : a == +b);
      case '[object Date]':
      case '[object Boolean]':
        // Coerce dates and booleans to numeric primitive values. Dates are compared by their
        // millisecond representations. Note that invalid dates with millisecond representations
        // of `NaN` are not equivalent.
        return +a == +b;
      // RegExps are compared by their source patterns and flags.
      case '[object RegExp]':
        return a.source == b.source &&
               a.global == b.global &&
               a.multiline == b.multiline &&
               a.ignoreCase == b.ignoreCase;
    }
    if (typeof a != 'object' || typeof b != 'object') return false;
    // Assume equality for cyclic structures. The algorithm for detecting cyclic
    // structures is adapted from ES 5.1 section 15.12.3, abstract operation `JO`.
    var length = aStack.length;
    while (length--) {
      // Linear search. Performance is inversely proportional to the number of
      // unique nested structures.
      if (aStack[length] == a) return bStack[length] == b;
    }
    // Objects with different constructors are not equivalent, but `Object`s
    // from different frames are.
    var aCtor = a.constructor, bCtor = b.constructor;
    if (aCtor !== bCtor && !(_.isFunction(aCtor) && (aCtor instanceof aCtor) &&
                             _.isFunction(bCtor) && (bCtor instanceof bCtor))) {
      return false;
    }
    // Add the first object to the stack of traversed objects.
    aStack.push(a);
    bStack.push(b);
    var size = 0, result = true;
    // Recursively compare objects and arrays.
    if (className == '[object Array]') {
      // Compare array lengths to determine if a deep comparison is necessary.
      size = a.length;
      result = size == b.length;
      if (result) {
        // Deep compare the contents, ignoring non-numeric properties.
        while (size--) {
          if (!(result = eq(a[size], b[size], aStack, bStack))) break;
        }
      }
    } else {
      // Deep compare objects.
      for (var key in a) {
        if (_.has(a, key)) {
          // Count the expected number of properties.
          size++;
          // Deep compare each member.
          if (!(result = _.has(b, key) && eq(a[key], b[key], aStack, bStack))) break;
        }
      }
      // Ensure that both objects contain the same number of properties.
      if (result) {
        for (key in b) {
          if (_.has(b, key) && !(size--)) break;
        }
        result = !size;
      }
    }
    // Remove the first object from the stack of traversed objects.
    aStack.pop();
    bStack.pop();
    return result;
  };

  // Perform a deep comparison to check if two objects are equal.
  _.isEqual = function(a, b) {
    return eq(a, b, [], []);
  };

  // Is a given array, string, or object empty?
  // An "empty" object has no enumerable own-properties.
  _.isEmpty = function(obj) {
    if (obj == null) return true;
    if (_.isArray(obj) || _.isString(obj)) return obj.length === 0;
    for (var key in obj) if (_.has(obj, key)) return false;
    return true;
  };

  // Is a given value a DOM element?
  _.isElement = function(obj) {
    return !!(obj && obj.nodeType === 1);
  };

  // Is a given value an array?
  // Delegates to ECMA5's native Array.isArray
  _.isArray = nativeIsArray || function(obj) {
    return toString.call(obj) == '[object Array]';
  };

  // Is a given variable an object?
  _.isObject = function(obj) {
    return obj === Object(obj);
  };

  // Add some isType methods: isArguments, isFunction, isString, isNumber, isDate, isRegExp.
  each(['Arguments', 'Function', 'String', 'Number', 'Date', 'RegExp'], function(name) {
    _['is' + name] = function(obj) {
      return toString.call(obj) == '[object ' + name + ']';
    };
  });

  // Define a fallback version of the method in browsers (ahem, IE), where
  // there isn't any inspectable "Arguments" type.
  if (!_.isArguments(arguments)) {
    _.isArguments = function(obj) {
      return !!(obj && _.has(obj, 'callee'));
    };
  }

  // Optimize `isFunction` if appropriate.
  if (typeof (/./) !== 'function') {
    _.isFunction = function(obj) {
      return typeof obj === 'function';
    };
  }

  // Is a given object a finite number?
  _.isFinite = function(obj) {
    return isFinite(obj) && !isNaN(parseFloat(obj));
  };

  // Is the given value `NaN`? (NaN is the only number which does not equal itself).
  _.isNaN = function(obj) {
    return _.isNumber(obj) && obj != +obj;
  };

  // Is a given value a boolean?
  _.isBoolean = function(obj) {
    return obj === true || obj === false || toString.call(obj) == '[object Boolean]';
  };

  // Is a given value equal to null?
  _.isNull = function(obj) {
    return obj === null;
  };

  // Is a given variable undefined?
  _.isUndefined = function(obj) {
    return obj === void 0;
  };

  // Shortcut function for checking if an object has a given property directly
  // on itself (in other words, not on a prototype).
  _.has = function(obj, key) {
    return hasOwnProperty.call(obj, key);
  };

  // Utility Functions
  // -----------------

  // Run Underscore.js in *noConflict* mode, returning the `_` variable to its
  // previous owner. Returns a reference to the Underscore object.
  _.noConflict = function() {
    root._ = previousUnderscore;
    return this;
  };

  // Keep the identity function around for default iterators.
  _.identity = function(value) {
    return value;
  };

  // Run a function **n** times.
  _.times = function(n, iterator, context) {
    var accum = Array(Math.max(0, n));
    for (var i = 0; i < n; i++) accum[i] = iterator.call(context, i);
    return accum;
  };

  // Return a random integer between min and max (inclusive).
  _.random = function(min, max) {
    if (max == null) {
      max = min;
      min = 0;
    }
    return min + Math.floor(Math.random() * (max - min + 1));
  };

  // List of HTML entities for escaping.
  var entityMap = {
    escape: {
      '&': '&amp;',
      '<': '&lt;',
      '>': '&gt;',
      '"': '&quot;',
      "'": '&#x27;'
    }
  };
  entityMap.unescape = _.invert(entityMap.escape);

  // Regexes containing the keys and values listed immediately above.
  var entityRegexes = {
    escape:   new RegExp('[' + _.keys(entityMap.escape).join('') + ']', 'g'),
    unescape: new RegExp('(' + _.keys(entityMap.unescape).join('|') + ')', 'g')
  };

  // Functions for escaping and unescaping strings to/from HTML interpolation.
  _.each(['escape', 'unescape'], function(method) {
    _[method] = function(string) {
      if (string == null) return '';
      return ('' + string).replace(entityRegexes[method], function(match) {
        return entityMap[method][match];
      });
    };
  });

  // If the value of the named `property` is a function then invoke it with the
  // `object` as context; otherwise, return it.
  _.result = function(object, property) {
    if (object == null) return void 0;
    var value = object[property];
    return _.isFunction(value) ? value.call(object) : value;
  };

  // Add your own custom functions to the Underscore object.
  _.mixin = function(obj) {
    each(_.functions(obj), function(name) {
      var func = _[name] = obj[name];
      _.prototype[name] = function() {
        var args = [this._wrapped];
        push.apply(args, arguments);
        return result.call(this, func.apply(_, args));
      };
    });
  };

  // Generate a unique integer id (unique within the entire client session).
  // Useful for temporary DOM ids.
  var idCounter = 0;
  _.uniqueId = function(prefix) {
    var id = ++idCounter + '';
    return prefix ? prefix + id : id;
  };

  // By default, Underscore uses ERB-style template delimiters, change the
  // following template settings to use alternative delimiters.
  _.templateSettings = {
    evaluate    : /<%([\s\S]+?)%>/g,
    interpolate : /<%=([\s\S]+?)%>/g,
    escape      : /<%-([\s\S]+?)%>/g
  };

  // When customizing `templateSettings`, if you don't want to define an
  // interpolation, evaluation or escaping regex, we need one that is
  // guaranteed not to match.
  var noMatch = /(.)^/;

  // Certain characters need to be escaped so that they can be put into a
  // string literal.
  var escapes = {
    "'":      "'",
    '\\':     '\\',
    '\r':     'r',
    '\n':     'n',
    '\t':     't',
    '\u2028': 'u2028',
    '\u2029': 'u2029'
  };

  var escaper = /\\|'|\r|\n|\t|\u2028|\u2029/g;

  // JavaScript micro-templating, similar to John Resig's implementation.
  // Underscore templating handles arbitrary delimiters, preserves whitespace,
  // and correctly escapes quotes within interpolated code.
  _.template = function(text, data, settings) {
    var render;
    settings = _.defaults({}, settings, _.templateSettings);

    // Combine delimiters into one regular expression via alternation.
    var matcher = new RegExp([
      (settings.escape || noMatch).source,
      (settings.interpolate || noMatch).source,
      (settings.evaluate || noMatch).source
    ].join('|') + '|$', 'g');

    // Compile the template source, escaping string literals appropriately.
    var index = 0;
    var source = "__p+='";
    text.replace(matcher, function(match, escape, interpolate, evaluate, offset) {
      source += text.slice(index, offset)
        .replace(escaper, function(match) { return '\\' + escapes[match]; });

      if (escape) {
        source += "'+\n((__t=(" + escape + "))==null?'':_.escape(__t))+\n'";
      }
      if (interpolate) {
        source += "'+\n((__t=(" + interpolate + "))==null?'':__t)+\n'";
      }
      if (evaluate) {
        source += "';\n" + evaluate + "\n__p+='";
      }
      index = offset + match.length;
      return match;
    });
    source += "';\n";

    // If a variable is not specified, place data values in local scope.
    if (!settings.variable) source = 'with(obj||{}){\n' + source + '}\n';

    source = "var __t,__p='',__j=Array.prototype.join," +
      "print=function(){__p+=__j.call(arguments,'');};\n" +
      source + "return __p;\n";

    try {
      render = new Function(settings.variable || 'obj', '_', source);
    } catch (e) {
      e.source = source;
      throw e;
    }

    if (data) return render(data, _);
    var template = function(data) {
      return render.call(this, data, _);
    };

    // Provide the compiled function source as a convenience for precompilation.
    template.source = 'function(' + (settings.variable || 'obj') + '){\n' + source + '}';

    return template;
  };

  // Add a "chain" function, which will delegate to the wrapper.
  _.chain = function(obj) {
    return _(obj).chain();
  };

  // OOP
  // ---------------
  // If Underscore is called as a function, it returns a wrapped object that
  // can be used OO-style. This wrapper holds altered versions of all the
  // underscore functions. Wrapped objects may be chained.

  // Helper function to continue chaining intermediate results.
  var result = function(obj) {
    return this._chain ? _(obj).chain() : obj;
  };

  // Add all of the Underscore functions to the wrapper object.
  _.mixin(_);

  // Add all mutator Array functions to the wrapper.
  each(['pop', 'push', 'reverse', 'shift', 'sort', 'splice', 'unshift'], function(name) {
    var method = ArrayProto[name];
    _.prototype[name] = function() {
      var obj = this._wrapped;
      method.apply(obj, arguments);
      if ((name == 'shift' || name == 'splice') && obj.length === 0) delete obj[0];
      return result.call(this, obj);
    };
  });

  // Add all accessor Array functions to the wrapper.
  each(['concat', 'join', 'slice'], function(name) {
    var method = ArrayProto[name];
    _.prototype[name] = function() {
      return result.call(this, method.apply(this._wrapped, arguments));
    };
  });

  _.extend(_.prototype, {

    // Start chaining a wrapped Underscore object.
    chain: function() {
      this._chain = true;
      return this;
    },

    // Extracts the result from a wrapped and chained object.
    value: function() {
      return this._wrapped;
    }

  });

}).call(this);

define("underscore", (function (global) {
    return function () {
        var ret, fn;
        return ret || global._;
    };
}(this)));

//     Backbone.js 1.1.2

//     (c) 2010-2014 Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
//     Backbone may be freely distributed under the MIT license.
//     For all details and documentation:
//     http://backbonejs.org

(function(root, factory) {

  // Set up Backbone appropriately for the environment. Start with AMD.
  if (typeof define === 'function' && define.amd) {
    define('backbone',['underscore', 'jquery', 'exports'], function(_, $, exports) {
      // Export global even in AMD case in case this script is loaded with
      // others that may still expect a global Backbone.
      root.Backbone = factory(root, exports, _, $);
    });

  // Next for Node.js or CommonJS. jQuery may not be needed as a module.
  } else if (typeof exports !== 'undefined') {
    var _ = require('underscore');
    factory(root, exports, _);

  // Finally, as a browser global.
  } else {
    root.Backbone = factory(root, {}, root._, (root.jQuery || root.Zepto || root.ender || root.$));
  }

}(this, function(root, Backbone, _, $) {

  // Initial Setup
  // -------------

  // Save the previous value of the `Backbone` variable, so that it can be
  // restored later on, if `noConflict` is used.
  var previousBackbone = root.Backbone;

  // Create local references to array methods we'll want to use later.
  var array = [];
  var push = array.push;
  var slice = array.slice;
  var splice = array.splice;

  // Current version of the library. Keep in sync with `package.json`.
  Backbone.VERSION = '1.1.2';

  // For Backbone's purposes, jQuery, Zepto, Ender, or My Library (kidding) owns
  // the `$` variable.
  Backbone.$ = $;

  // Runs Backbone.js in *noConflict* mode, returning the `Backbone` variable
  // to its previous owner. Returns a reference to this Backbone object.
  Backbone.noConflict = function() {
    root.Backbone = previousBackbone;
    return this;
  };

  // Turn on `emulateHTTP` to support legacy HTTP servers. Setting this option
  // will fake `"PATCH"`, `"PUT"` and `"DELETE"` requests via the `_method` parameter and
  // set a `X-Http-Method-Override` header.
  Backbone.emulateHTTP = false;

  // Turn on `emulateJSON` to support legacy servers that can't deal with direct
  // `application/json` requests ... will encode the body as
  // `application/x-www-form-urlencoded` instead and will send the model in a
  // form param named `model`.
  Backbone.emulateJSON = false;

  // Backbone.Events
  // ---------------

  // A module that can be mixed in to *any object* in order to provide it with
  // custom events. You may bind with `on` or remove with `off` callback
  // functions to an event; `trigger`-ing an event fires all callbacks in
  // succession.
  //
  //     var object = {};
  //     _.extend(object, Backbone.Events);
  //     object.on('expand', function(){ alert('expanded'); });
  //     object.trigger('expand');
  //
  var Events = Backbone.Events = {

    // Bind an event to a `callback` function. Passing `"all"` will bind
    // the callback to all events fired.
    on: function(name, callback, context) {
      if (!eventsApi(this, 'on', name, [callback, context]) || !callback) return this;
      this._events || (this._events = {});
      var events = this._events[name] || (this._events[name] = []);
      events.push({callback: callback, context: context, ctx: context || this});
      return this;
    },

    // Bind an event to only be triggered a single time. After the first time
    // the callback is invoked, it will be removed.
    once: function(name, callback, context) {
      if (!eventsApi(this, 'once', name, [callback, context]) || !callback) return this;
      var self = this;
      var once = _.once(function() {
        self.off(name, once);
        callback.apply(this, arguments);
      });
      once._callback = callback;
      return this.on(name, once, context);
    },

    // Remove one or many callbacks. If `context` is null, removes all
    // callbacks with that function. If `callback` is null, removes all
    // callbacks for the event. If `name` is null, removes all bound
    // callbacks for all events.
    off: function(name, callback, context) {
      var retain, ev, events, names, i, l, j, k;
      if (!this._events || !eventsApi(this, 'off', name, [callback, context])) return this;
      if (!name && !callback && !context) {
        this._events = void 0;
        return this;
      }
      names = name ? [name] : _.keys(this._events);
      for (i = 0, l = names.length; i < l; i++) {
        name = names[i];
        if (events = this._events[name]) {
          this._events[name] = retain = [];
          if (callback || context) {
            for (j = 0, k = events.length; j < k; j++) {
              ev = events[j];
              if ((callback && callback !== ev.callback && callback !== ev.callback._callback) ||
                  (context && context !== ev.context)) {
                retain.push(ev);
              }
            }
          }
          if (!retain.length) delete this._events[name];
        }
      }

      return this;
    },

    // Trigger one or many events, firing all bound callbacks. Callbacks are
    // passed the same arguments as `trigger` is, apart from the event name
    // (unless you're listening on `"all"`, which will cause your callback to
    // receive the true name of the event as the first argument).
    trigger: function(name) {
      if (!this._events) return this;
      var args = slice.call(arguments, 1);
      if (!eventsApi(this, 'trigger', name, args)) return this;
      var events = this._events[name];
      var allEvents = this._events.all;
      if (events) triggerEvents(events, args);
      if (allEvents) triggerEvents(allEvents, arguments);
      return this;
    },

    // Tell this object to stop listening to either specific events ... or
    // to every object it's currently listening to.
    stopListening: function(obj, name, callback) {
      var listeningTo = this._listeningTo;
      if (!listeningTo) return this;
      var remove = !name && !callback;
      if (!callback && typeof name === 'object') callback = this;
      if (obj) (listeningTo = {})[obj._listenId] = obj;
      for (var id in listeningTo) {
        obj = listeningTo[id];
        obj.off(name, callback, this);
        if (remove || _.isEmpty(obj._events)) delete this._listeningTo[id];
      }
      return this;
    }

  };

  // Regular expression used to split event strings.
  var eventSplitter = /\s+/;

  // Implement fancy features of the Events API such as multiple event
  // names `"change blur"` and jQuery-style event maps `{change: action}`
  // in terms of the existing API.
  var eventsApi = function(obj, action, name, rest) {
    if (!name) return true;

    // Handle event maps.
    if (typeof name === 'object') {
      for (var key in name) {
        obj[action].apply(obj, [key, name[key]].concat(rest));
      }
      return false;
    }

    // Handle space separated event names.
    if (eventSplitter.test(name)) {
      var names = name.split(eventSplitter);
      for (var i = 0, l = names.length; i < l; i++) {
        obj[action].apply(obj, [names[i]].concat(rest));
      }
      return false;
    }

    return true;
  };

  // A difficult-to-believe, but optimized internal dispatch function for
  // triggering events. Tries to keep the usual cases speedy (most internal
  // Backbone events have 3 arguments).
  var triggerEvents = function(events, args) {
    var ev, i = -1, l = events.length, a1 = args[0], a2 = args[1], a3 = args[2];
    switch (args.length) {
      case 0: while (++i < l) (ev = events[i]).callback.call(ev.ctx); return;
      case 1: while (++i < l) (ev = events[i]).callback.call(ev.ctx, a1); return;
      case 2: while (++i < l) (ev = events[i]).callback.call(ev.ctx, a1, a2); return;
      case 3: while (++i < l) (ev = events[i]).callback.call(ev.ctx, a1, a2, a3); return;
      default: while (++i < l) (ev = events[i]).callback.apply(ev.ctx, args); return;
    }
  };

  var listenMethods = {listenTo: 'on', listenToOnce: 'once'};

  // Inversion-of-control versions of `on` and `once`. Tell *this* object to
  // listen to an event in another object ... keeping track of what it's
  // listening to.
  _.each(listenMethods, function(implementation, method) {
    Events[method] = function(obj, name, callback) {
      var listeningTo = this._listeningTo || (this._listeningTo = {});
      var id = obj._listenId || (obj._listenId = _.uniqueId('l'));
      listeningTo[id] = obj;
      if (!callback && typeof name === 'object') callback = this;
      obj[implementation](name, callback, this);
      return this;
    };
  });

  // Aliases for backwards compatibility.
  Events.bind   = Events.on;
  Events.unbind = Events.off;

  // Allow the `Backbone` object to serve as a global event bus, for folks who
  // want global "pubsub" in a convenient place.
  _.extend(Backbone, Events);

  // Backbone.Model
  // --------------

  // Backbone **Models** are the basic data object in the framework --
  // frequently representing a row in a table in a database on your server.
  // A discrete chunk of data and a bunch of useful, related methods for
  // performing computations and transformations on that data.

  // Create a new model with the specified attributes. A client id (`cid`)
  // is automatically generated and assigned for you.
  var Model = Backbone.Model = function(attributes, options) {
    var attrs = attributes || {};
    options || (options = {});
    this.cid = _.uniqueId('c');
    this.attributes = {};
    if (options.collection) this.collection = options.collection;
    if (options.parse) attrs = this.parse(attrs, options) || {};
    attrs = _.defaults({}, attrs, _.result(this, 'defaults'));
    this.set(attrs, options);
    this.changed = {};
    this.initialize.apply(this, arguments);
  };

  // Attach all inheritable methods to the Model prototype.
  _.extend(Model.prototype, Events, {

    // A hash of attributes whose current and previous value differ.
    changed: null,

    // The value returned during the last failed validation.
    validationError: null,

    // The default name for the JSON `id` attribute is `"id"`. MongoDB and
    // CouchDB users may want to set this to `"_id"`.
    idAttribute: 'id',

    // Initialize is an empty function by default. Override it with your own
    // initialization logic.
    initialize: function(){},

    // Return a copy of the model's `attributes` object.
    toJSON: function(options) {
      return _.clone(this.attributes);
    },

    // Proxy `Backbone.sync` by default -- but override this if you need
    // custom syncing semantics for *this* particular model.
    sync: function() {
      return Backbone.sync.apply(this, arguments);
    },

    // Get the value of an attribute.
    get: function(attr) {
      return this.attributes[attr];
    },

    // Get the HTML-escaped value of an attribute.
    escape: function(attr) {
      return _.escape(this.get(attr));
    },

    // Returns `true` if the attribute contains a value that is not null
    // or undefined.
    has: function(attr) {
      return this.get(attr) != null;
    },

    // Set a hash of model attributes on the object, firing `"change"`. This is
    // the core primitive operation of a model, updating the data and notifying
    // anyone who needs to know about the change in state. The heart of the beast.
    set: function(key, val, options) {
      var attr, attrs, unset, changes, silent, changing, prev, current;
      if (key == null) return this;

      // Handle both `"key", value` and `{key: value}` -style arguments.
      if (typeof key === 'object') {
        attrs = key;
        options = val;
      } else {
        (attrs = {})[key] = val;
      }

      options || (options = {});

      // Run validation.
      if (!this._validate(attrs, options)) return false;

      // Extract attributes and options.
      unset           = options.unset;
      silent          = options.silent;
      changes         = [];
      changing        = this._changing;
      this._changing  = true;

      if (!changing) {
        this._previousAttributes = _.clone(this.attributes);
        this.changed = {};
      }
      current = this.attributes, prev = this._previousAttributes;

      // Check for changes of `id`.
      if (this.idAttribute in attrs) this.id = attrs[this.idAttribute];

      // For each `set` attribute, update or delete the current value.
      for (attr in attrs) {
        val = attrs[attr];
        if (!_.isEqual(current[attr], val)) changes.push(attr);
        if (!_.isEqual(prev[attr], val)) {
          this.changed[attr] = val;
        } else {
          delete this.changed[attr];
        }
        unset ? delete current[attr] : current[attr] = val;
      }

      // Trigger all relevant attribute changes.
      if (!silent) {
        if (changes.length) this._pending = options;
        for (var i = 0, l = changes.length; i < l; i++) {
          this.trigger('change:' + changes[i], this, current[changes[i]], options);
        }
      }

      // You might be wondering why there's a `while` loop here. Changes can
      // be recursively nested within `"change"` events.
      if (changing) return this;
      if (!silent) {
        while (this._pending) {
          options = this._pending;
          this._pending = false;
          this.trigger('change', this, options);
        }
      }
      this._pending = false;
      this._changing = false;
      return this;
    },

    // Remove an attribute from the model, firing `"change"`. `unset` is a noop
    // if the attribute doesn't exist.
    unset: function(attr, options) {
      return this.set(attr, void 0, _.extend({}, options, {unset: true}));
    },

    // Clear all attributes on the model, firing `"change"`.
    clear: function(options) {
      var attrs = {};
      for (var key in this.attributes) attrs[key] = void 0;
      return this.set(attrs, _.extend({}, options, {unset: true}));
    },

    // Determine if the model has changed since the last `"change"` event.
    // If you specify an attribute name, determine if that attribute has changed.
    hasChanged: function(attr) {
      if (attr == null) return !_.isEmpty(this.changed);
      return _.has(this.changed, attr);
    },

    // Return an object containing all the attributes that have changed, or
    // false if there are no changed attributes. Useful for determining what
    // parts of a view need to be updated and/or what attributes need to be
    // persisted to the server. Unset attributes will be set to undefined.
    // You can also pass an attributes object to diff against the model,
    // determining if there *would be* a change.
    changedAttributes: function(diff) {
      if (!diff) return this.hasChanged() ? _.clone(this.changed) : false;
      var val, changed = false;
      var old = this._changing ? this._previousAttributes : this.attributes;
      for (var attr in diff) {
        if (_.isEqual(old[attr], (val = diff[attr]))) continue;
        (changed || (changed = {}))[attr] = val;
      }
      return changed;
    },

    // Get the previous value of an attribute, recorded at the time the last
    // `"change"` event was fired.
    previous: function(attr) {
      if (attr == null || !this._previousAttributes) return null;
      return this._previousAttributes[attr];
    },

    // Get all of the attributes of the model at the time of the previous
    // `"change"` event.
    previousAttributes: function() {
      return _.clone(this._previousAttributes);
    },

    // Fetch the model from the server. If the server's representation of the
    // model differs from its current attributes, they will be overridden,
    // triggering a `"change"` event.
    fetch: function(options) {
      options = options ? _.clone(options) : {};
      if (options.parse === void 0) options.parse = true;
      var model = this;
      var success = options.success;
      options.success = function(resp) {
        if (!model.set(model.parse(resp, options), options)) return false;
        if (success) success(model, resp, options);
        model.trigger('sync', model, resp, options);
      };
      wrapError(this, options);
      return this.sync('read', this, options);
    },

    // Set a hash of model attributes, and sync the model to the server.
    // If the server returns an attributes hash that differs, the model's
    // state will be `set` again.
    save: function(key, val, options) {
      var attrs, method, xhr, attributes = this.attributes;

      // Handle both `"key", value` and `{key: value}` -style arguments.
      if (key == null || typeof key === 'object') {
        attrs = key;
        options = val;
      } else {
        (attrs = {})[key] = val;
      }

      options = _.extend({validate: true}, options);

      // If we're not waiting and attributes exist, save acts as
      // `set(attr).save(null, opts)` with validation. Otherwise, check if
      // the model will be valid when the attributes, if any, are set.
      if (attrs && !options.wait) {
        if (!this.set(attrs, options)) return false;
      } else {
        if (!this._validate(attrs, options)) return false;
      }

      // Set temporary attributes if `{wait: true}`.
      if (attrs && options.wait) {
        this.attributes = _.extend({}, attributes, attrs);
      }

      // After a successful server-side save, the client is (optionally)
      // updated with the server-side state.
      if (options.parse === void 0) options.parse = true;
      var model = this;
      var success = options.success;
      options.success = function(resp) {
        // Ensure attributes are restored during synchronous saves.
        model.attributes = attributes;
        var serverAttrs = model.parse(resp, options);
        if (options.wait) serverAttrs = _.extend(attrs || {}, serverAttrs);
        if (_.isObject(serverAttrs) && !model.set(serverAttrs, options)) {
          return false;
        }
        if (success) success(model, resp, options);
        model.trigger('sync', model, resp, options);
      };
      wrapError(this, options);

      method = this.isNew() ? 'create' : (options.patch ? 'patch' : 'update');
      if (method === 'patch') options.attrs = attrs;
      xhr = this.sync(method, this, options);

      // Restore attributes.
      if (attrs && options.wait) this.attributes = attributes;

      return xhr;
    },

    // Destroy this model on the server if it was already persisted.
    // Optimistically removes the model from its collection, if it has one.
    // If `wait: true` is passed, waits for the server to respond before removal.
    destroy: function(options) {
      options = options ? _.clone(options) : {};
      var model = this;
      var success = options.success;

      var destroy = function() {
        model.trigger('destroy', model, model.collection, options);
      };

      options.success = function(resp) {
        if (options.wait || model.isNew()) destroy();
        if (success) success(model, resp, options);
        if (!model.isNew()) model.trigger('sync', model, resp, options);
      };

      if (this.isNew()) {
        options.success();
        return false;
      }
      wrapError(this, options);

      var xhr = this.sync('delete', this, options);
      if (!options.wait) destroy();
      return xhr;
    },

    // Default URL for the model's representation on the server -- if you're
    // using Backbone's restful methods, override this to change the endpoint
    // that will be called.
    url: function() {
      var base =
        _.result(this, 'urlRoot') ||
        _.result(this.collection, 'url') ||
        urlError();
      if (this.isNew()) return base;
      return base.replace(/([^\/])$/, '$1/') + encodeURIComponent(this.id);
    },

    // **parse** converts a response into the hash of attributes to be `set` on
    // the model. The default implementation is just to pass the response along.
    parse: function(resp, options) {
      return resp;
    },

    // Create a new model with identical attributes to this one.
    clone: function() {
      return new this.constructor(this.attributes);
    },

    // A model is new if it has never been saved to the server, and lacks an id.
    isNew: function() {
      return !this.has(this.idAttribute);
    },

    // Check if the model is currently in a valid state.
    isValid: function(options) {
      return this._validate({}, _.extend(options || {}, { validate: true }));
    },

    // Run validation against the next complete set of model attributes,
    // returning `true` if all is well. Otherwise, fire an `"invalid"` event.
    _validate: function(attrs, options) {
      if (!options.validate || !this.validate) return true;
      attrs = _.extend({}, this.attributes, attrs);
      var error = this.validationError = this.validate(attrs, options) || null;
      if (!error) return true;
      this.trigger('invalid', this, error, _.extend(options, {validationError: error}));
      return false;
    }

  });

  // Underscore methods that we want to implement on the Model.
  var modelMethods = ['keys', 'values', 'pairs', 'invert', 'pick', 'omit'];

  // Mix in each Underscore method as a proxy to `Model#attributes`.
  _.each(modelMethods, function(method) {
    Model.prototype[method] = function() {
      var args = slice.call(arguments);
      args.unshift(this.attributes);
      return _[method].apply(_, args);
    };
  });

  // Backbone.Collection
  // -------------------

  // If models tend to represent a single row of data, a Backbone Collection is
  // more analagous to a table full of data ... or a small slice or page of that
  // table, or a collection of rows that belong together for a particular reason
  // -- all of the messages in this particular folder, all of the documents
  // belonging to this particular author, and so on. Collections maintain
  // indexes of their models, both in order, and for lookup by `id`.

  // Create a new **Collection**, perhaps to contain a specific type of `model`.
  // If a `comparator` is specified, the Collection will maintain
  // its models in sort order, as they're added and removed.
  var Collection = Backbone.Collection = function(models, options) {
    options || (options = {});
    if (options.model) this.model = options.model;
    if (options.comparator !== void 0) this.comparator = options.comparator;
    this._reset();
    this.initialize.apply(this, arguments);
    if (models) this.reset(models, _.extend({silent: true}, options));
  };

  // Default options for `Collection#set`.
  var setOptions = {add: true, remove: true, merge: true};
  var addOptions = {add: true, remove: false};

  // Define the Collection's inheritable methods.
  _.extend(Collection.prototype, Events, {

    // The default model for a collection is just a **Backbone.Model**.
    // This should be overridden in most cases.
    model: Model,

    // Initialize is an empty function by default. Override it with your own
    // initialization logic.
    initialize: function(){},

    // The JSON representation of a Collection is an array of the
    // models' attributes.
    toJSON: function(options) {
      return this.map(function(model){ return model.toJSON(options); });
    },

    // Proxy `Backbone.sync` by default.
    sync: function() {
      return Backbone.sync.apply(this, arguments);
    },

    // Add a model, or list of models to the set.
    add: function(models, options) {
      return this.set(models, _.extend({merge: false}, options, addOptions));
    },

    // Remove a model, or a list of models from the set.
    remove: function(models, options) {
      var singular = !_.isArray(models);
      models = singular ? [models] : _.clone(models);
      options || (options = {});
      var i, l, index, model;
      for (i = 0, l = models.length; i < l; i++) {
        model = models[i] = this.get(models[i]);
        if (!model) continue;
        delete this._byId[model.id];
        delete this._byId[model.cid];
        index = this.indexOf(model);
        this.models.splice(index, 1);
        this.length--;
        if (!options.silent) {
          options.index = index;
          model.trigger('remove', model, this, options);
        }
        this._removeReference(model, options);
      }
      return singular ? models[0] : models;
    },

    // Update a collection by `set`-ing a new list of models, adding new ones,
    // removing models that are no longer present, and merging models that
    // already exist in the collection, as necessary. Similar to **Model#set**,
    // the core operation for updating the data contained by the collection.
    set: function(models, options) {
      options = _.defaults({}, options, setOptions);
      if (options.parse) models = this.parse(models, options);
      var singular = !_.isArray(models);
      models = singular ? (models ? [models] : []) : _.clone(models);
      var i, l, id, model, attrs, existing, sort;
      var at = options.at;
      var targetModel = this.model;
      var sortable = this.comparator && (at == null) && options.sort !== false;
      var sortAttr = _.isString(this.comparator) ? this.comparator : null;
      var toAdd = [], toRemove = [], modelMap = {};
      var add = options.add, merge = options.merge, remove = options.remove;
      var order = !sortable && add && remove ? [] : false;

      // Turn bare objects into model references, and prevent invalid models
      // from being added.
      for (i = 0, l = models.length; i < l; i++) {
        attrs = models[i] || {};
        if (attrs instanceof Model) {
          id = model = attrs;
        } else {
          id = attrs[targetModel.prototype.idAttribute || 'id'];
        }

        // If a duplicate is found, prevent it from being added and
        // optionally merge it into the existing model.
        if (existing = this.get(id)) {
          if (remove) modelMap[existing.cid] = true;
          if (merge) {
            attrs = attrs === model ? model.attributes : attrs;
            if (options.parse) attrs = existing.parse(attrs, options);
            existing.set(attrs, options);
            if (sortable && !sort && existing.hasChanged(sortAttr)) sort = true;
          }
          models[i] = existing;

        // If this is a new, valid model, push it to the `toAdd` list.
        } else if (add) {
          model = models[i] = this._prepareModel(attrs, options);
          if (!model) continue;
          toAdd.push(model);
          this._addReference(model, options);
        }

        // Do not add multiple models with the same `id`.
        model = existing || model;
        if (order && (model.isNew() || !modelMap[model.id])) order.push(model);
        modelMap[model.id] = true;
      }

      // Remove nonexistent models if appropriate.
      if (remove) {
        for (i = 0, l = this.length; i < l; ++i) {
          if (!modelMap[(model = this.models[i]).cid]) toRemove.push(model);
        }
        if (toRemove.length) this.remove(toRemove, options);
      }

      // See if sorting is needed, update `length` and splice in new models.
      if (toAdd.length || (order && order.length)) {
        if (sortable) sort = true;
        this.length += toAdd.length;
        if (at != null) {
          for (i = 0, l = toAdd.length; i < l; i++) {
            this.models.splice(at + i, 0, toAdd[i]);
          }
        } else {
          if (order) this.models.length = 0;
          var orderedModels = order || toAdd;
          for (i = 0, l = orderedModels.length; i < l; i++) {
            this.models.push(orderedModels[i]);
          }
        }
      }

      // Silently sort the collection if appropriate.
      if (sort) this.sort({silent: true});

      // Unless silenced, it's time to fire all appropriate add/sort events.
      if (!options.silent) {
        for (i = 0, l = toAdd.length; i < l; i++) {
          (model = toAdd[i]).trigger('add', model, this, options);
        }
        if (sort || (order && order.length)) this.trigger('sort', this, options);
      }

      // Return the added (or merged) model (or models).
      return singular ? models[0] : models;
    },

    // When you have more items than you want to add or remove individually,
    // you can reset the entire set with a new list of models, without firing
    // any granular `add` or `remove` events. Fires `reset` when finished.
    // Useful for bulk operations and optimizations.
    reset: function(models, options) {
      options || (options = {});
      for (var i = 0, l = this.models.length; i < l; i++) {
        this._removeReference(this.models[i], options);
      }
      options.previousModels = this.models;
      this._reset();
      models = this.add(models, _.extend({silent: true}, options));
      if (!options.silent) this.trigger('reset', this, options);
      return models;
    },

    // Add a model to the end of the collection.
    push: function(model, options) {
      return this.add(model, _.extend({at: this.length}, options));
    },

    // Remove a model from the end of the collection.
    pop: function(options) {
      var model = this.at(this.length - 1);
      this.remove(model, options);
      return model;
    },

    // Add a model to the beginning of the collection.
    unshift: function(model, options) {
      return this.add(model, _.extend({at: 0}, options));
    },

    // Remove a model from the beginning of the collection.
    shift: function(options) {
      var model = this.at(0);
      this.remove(model, options);
      return model;
    },

    // Slice out a sub-array of models from the collection.
    slice: function() {
      return slice.apply(this.models, arguments);
    },

    // Get a model from the set by id.
    get: function(obj) {
      if (obj == null) return void 0;
      return this._byId[obj] || this._byId[obj.id] || this._byId[obj.cid];
    },

    // Get the model at the given index.
    at: function(index) {
      return this.models[index];
    },

    // Return models with matching attributes. Useful for simple cases of
    // `filter`.
    where: function(attrs, first) {
      if (_.isEmpty(attrs)) return first ? void 0 : [];
      return this[first ? 'find' : 'filter'](function(model) {
        for (var key in attrs) {
          if (attrs[key] !== model.get(key)) return false;
        }
        return true;
      });
    },

    // Return the first model with matching attributes. Useful for simple cases
    // of `find`.
    findWhere: function(attrs) {
      return this.where(attrs, true);
    },

    // Force the collection to re-sort itself. You don't need to call this under
    // normal circumstances, as the set will maintain sort order as each item
    // is added.
    sort: function(options) {
      if (!this.comparator) throw new Error('Cannot sort a set without a comparator');
      options || (options = {});

      // Run sort based on type of `comparator`.
      if (_.isString(this.comparator) || this.comparator.length === 1) {
        this.models = this.sortBy(this.comparator, this);
      } else {
        this.models.sort(_.bind(this.comparator, this));
      }

      if (!options.silent) this.trigger('sort', this, options);
      return this;
    },

    // Pluck an attribute from each model in the collection.
    pluck: function(attr) {
      return _.invoke(this.models, 'get', attr);
    },

    // Fetch the default set of models for this collection, resetting the
    // collection when they arrive. If `reset: true` is passed, the response
    // data will be passed through the `reset` method instead of `set`.
    fetch: function(options) {
      options = options ? _.clone(options) : {};
      if (options.parse === void 0) options.parse = true;
      var success = options.success;
      var collection = this;
      options.success = function(resp) {
        var method = options.reset ? 'reset' : 'set';
        collection[method](resp, options);
        if (success) success(collection, resp, options);
        collection.trigger('sync', collection, resp, options);
      };
      wrapError(this, options);
      return this.sync('read', this, options);
    },

    // Create a new instance of a model in this collection. Add the model to the
    // collection immediately, unless `wait: true` is passed, in which case we
    // wait for the server to agree.
    create: function(model, options) {
      options = options ? _.clone(options) : {};
      if (!(model = this._prepareModel(model, options))) return false;
      if (!options.wait) this.add(model, options);
      var collection = this;
      var success = options.success;
      options.success = function(model, resp) {
        if (options.wait) collection.add(model, options);
        if (success) success(model, resp, options);
      };
      model.save(null, options);
      return model;
    },

    // **parse** converts a response into a list of models to be added to the
    // collection. The default implementation is just to pass it through.
    parse: function(resp, options) {
      return resp;
    },

    // Create a new collection with an identical list of models as this one.
    clone: function() {
      return new this.constructor(this.models);
    },

    // Private method to reset all internal state. Called when the collection
    // is first initialized or reset.
    _reset: function() {
      this.length = 0;
      this.models = [];
      this._byId  = {};
    },

    // Prepare a hash of attributes (or other model) to be added to this
    // collection.
    _prepareModel: function(attrs, options) {
      if (attrs instanceof Model) return attrs;
      options = options ? _.clone(options) : {};
      options.collection = this;
      var model = new this.model(attrs, options);
      if (!model.validationError) return model;
      this.trigger('invalid', this, model.validationError, options);
      return false;
    },

    // Internal method to create a model's ties to a collection.
    _addReference: function(model, options) {
      this._byId[model.cid] = model;
      if (model.id != null) this._byId[model.id] = model;
      if (!model.collection) model.collection = this;
      model.on('all', this._onModelEvent, this);
    },

    // Internal method to sever a model's ties to a collection.
    _removeReference: function(model, options) {
      if (this === model.collection) delete model.collection;
      model.off('all', this._onModelEvent, this);
    },

    // Internal method called every time a model in the set fires an event.
    // Sets need to update their indexes when models change ids. All other
    // events simply proxy through. "add" and "remove" events that originate
    // in other collections are ignored.
    _onModelEvent: function(event, model, collection, options) {
      if ((event === 'add' || event === 'remove') && collection !== this) return;
      if (event === 'destroy') this.remove(model, options);
      if (model && event === 'change:' + model.idAttribute) {
        delete this._byId[model.previous(model.idAttribute)];
        if (model.id != null) this._byId[model.id] = model;
      }
      this.trigger.apply(this, arguments);
    }

  });

  // Underscore methods that we want to implement on the Collection.
  // 90% of the core usefulness of Backbone Collections is actually implemented
  // right here:
  var methods = ['forEach', 'each', 'map', 'collect', 'reduce', 'foldl',
    'inject', 'reduceRight', 'foldr', 'find', 'detect', 'filter', 'select',
    'reject', 'every', 'all', 'some', 'any', 'include', 'contains', 'invoke',
    'max', 'min', 'toArray', 'size', 'first', 'head', 'take', 'initial', 'rest',
    'tail', 'drop', 'last', 'without', 'difference', 'indexOf', 'shuffle',
    'lastIndexOf', 'isEmpty', 'chain', 'sample'];

  // Mix in each Underscore method as a proxy to `Collection#models`.
  _.each(methods, function(method) {
    Collection.prototype[method] = function() {
      var args = slice.call(arguments);
      args.unshift(this.models);
      return _[method].apply(_, args);
    };
  });

  // Underscore methods that take a property name as an argument.
  var attributeMethods = ['groupBy', 'countBy', 'sortBy', 'indexBy'];

  // Use attributes instead of properties.
  _.each(attributeMethods, function(method) {
    Collection.prototype[method] = function(value, context) {
      var iterator = _.isFunction(value) ? value : function(model) {
        return model.get(value);
      };
      return _[method](this.models, iterator, context);
    };
  });

  // Backbone.View
  // -------------

  // Backbone Views are almost more convention than they are actual code. A View
  // is simply a JavaScript object that represents a logical chunk of UI in the
  // DOM. This might be a single item, an entire list, a sidebar or panel, or
  // even the surrounding frame which wraps your whole app. Defining a chunk of
  // UI as a **View** allows you to define your DOM events declaratively, without
  // having to worry about render order ... and makes it easy for the view to
  // react to specific changes in the state of your models.

  // Creating a Backbone.View creates its initial element outside of the DOM,
  // if an existing element is not provided...
  var View = Backbone.View = function(options) {
    this.cid = _.uniqueId('view');
    options || (options = {});
    _.extend(this, _.pick(options, viewOptions));
    this._ensureElement();
    this.initialize.apply(this, arguments);
    this.delegateEvents();
  };

  // Cached regex to split keys for `delegate`.
  var delegateEventSplitter = /^(\S+)\s*(.*)$/;

  // List of view options to be merged as properties.
  var viewOptions = ['model', 'collection', 'el', 'id', 'attributes', 'className', 'tagName', 'events'];

  // Set up all inheritable **Backbone.View** properties and methods.
  _.extend(View.prototype, Events, {

    // The default `tagName` of a View's element is `"div"`.
    tagName: 'div',

    // jQuery delegate for element lookup, scoped to DOM elements within the
    // current view. This should be preferred to global lookups where possible.
    $: function(selector) {
      return this.$el.find(selector);
    },

    // Initialize is an empty function by default. Override it with your own
    // initialization logic.
    initialize: function(){},

    // **render** is the core function that your view should override, in order
    // to populate its element (`this.el`), with the appropriate HTML. The
    // convention is for **render** to always return `this`.
    render: function() {
      return this;
    },

    // Remove this view by taking the element out of the DOM, and removing any
    // applicable Backbone.Events listeners.
    remove: function() {
      this.$el.remove();
      this.stopListening();
      return this;
    },

    // Change the view's element (`this.el` property), including event
    // re-delegation.
    setElement: function(element, delegate) {
      if (this.$el) this.undelegateEvents();
      this.$el = element instanceof Backbone.$ ? element : Backbone.$(element);
      this.el = this.$el[0];
      if (delegate !== false) this.delegateEvents();
      return this;
    },

    // Set callbacks, where `this.events` is a hash of
    //
    // *{"event selector": "callback"}*
    //
    //     {
    //       'mousedown .title':  'edit',
    //       'click .button':     'save',
    //       'click .open':       function(e) { ... }
    //     }
    //
    // pairs. Callbacks will be bound to the view, with `this` set properly.
    // Uses event delegation for efficiency.
    // Omitting the selector binds the event to `this.el`.
    // This only works for delegate-able events: not `focus`, `blur`, and
    // not `change`, `submit`, and `reset` in Internet Explorer.
    delegateEvents: function(events) {
      if (!(events || (events = _.result(this, 'events')))) return this;
      this.undelegateEvents();
      for (var key in events) {
        var method = events[key];
        if (!_.isFunction(method)) method = this[events[key]];
        if (!method) continue;

        var match = key.match(delegateEventSplitter);
        var eventName = match[1], selector = match[2];
        method = _.bind(method, this);
        eventName += '.delegateEvents' + this.cid;
        if (selector === '') {
          this.$el.on(eventName, method);
        } else {
          this.$el.on(eventName, selector, method);
        }
      }
      return this;
    },

    // Clears all callbacks previously bound to the view with `delegateEvents`.
    // You usually don't need to use this, but may wish to if you have multiple
    // Backbone views attached to the same DOM element.
    undelegateEvents: function() {
      this.$el.off('.delegateEvents' + this.cid);
      return this;
    },

    // Ensure that the View has a DOM element to render into.
    // If `this.el` is a string, pass it through `$()`, take the first
    // matching element, and re-assign it to `el`. Otherwise, create
    // an element from the `id`, `className` and `tagName` properties.
    _ensureElement: function() {
      if (!this.el) {
        var attrs = _.extend({}, _.result(this, 'attributes'));
        if (this.id) attrs.id = _.result(this, 'id');
        if (this.className) attrs['class'] = _.result(this, 'className');
        var $el = Backbone.$('<' + _.result(this, 'tagName') + '>').attr(attrs);
        this.setElement($el, false);
      } else {
        this.setElement(_.result(this, 'el'), false);
      }
    }

  });

  // Backbone.sync
  // -------------

  // Override this function to change the manner in which Backbone persists
  // models to the server. You will be passed the type of request, and the
  // model in question. By default, makes a RESTful Ajax request
  // to the model's `url()`. Some possible customizations could be:
  //
  // * Use `setTimeout` to batch rapid-fire updates into a single request.
  // * Send up the models as XML instead of JSON.
  // * Persist models via WebSockets instead of Ajax.
  //
  // Turn on `Backbone.emulateHTTP` in order to send `PUT` and `DELETE` requests
  // as `POST`, with a `_method` parameter containing the true HTTP method,
  // as well as all requests with the body as `application/x-www-form-urlencoded`
  // instead of `application/json` with the model in a param named `model`.
  // Useful when interfacing with server-side languages like **PHP** that make
  // it difficult to read the body of `PUT` requests.
  Backbone.sync = function(method, model, options) {
    var type = methodMap[method];

    // Default options, unless specified.
    _.defaults(options || (options = {}), {
      emulateHTTP: Backbone.emulateHTTP,
      emulateJSON: Backbone.emulateJSON
    });

    // Default JSON-request options.
    var params = {type: type, dataType: 'json'};

    // Ensure that we have a URL.
    if (!options.url) {
      params.url = _.result(model, 'url') || urlError();
    }

    // Ensure that we have the appropriate request data.
    if (options.data == null && model && (method === 'create' || method === 'update' || method === 'patch')) {
      params.contentType = 'application/json';
      params.data = JSON.stringify(options.attrs || model.toJSON(options));
    }

    // For older servers, emulate JSON by encoding the request into an HTML-form.
    if (options.emulateJSON) {
      params.contentType = 'application/x-www-form-urlencoded';
      params.data = params.data ? {model: params.data} : {};
    }

    // For older servers, emulate HTTP by mimicking the HTTP method with `_method`
    // And an `X-HTTP-Method-Override` header.
    if (options.emulateHTTP && (type === 'PUT' || type === 'DELETE' || type === 'PATCH')) {
      params.type = 'POST';
      if (options.emulateJSON) params.data._method = type;
      var beforeSend = options.beforeSend;
      options.beforeSend = function(xhr) {
        xhr.setRequestHeader('X-HTTP-Method-Override', type);
        if (beforeSend) return beforeSend.apply(this, arguments);
      };
    }

    // Don't process data on a non-GET request.
    if (params.type !== 'GET' && !options.emulateJSON) {
      params.processData = false;
    }

    // If we're sending a `PATCH` request, and we're in an old Internet Explorer
    // that still has ActiveX enabled by default, override jQuery to use that
    // for XHR instead. Remove this line when jQuery supports `PATCH` on IE8.
    if (params.type === 'PATCH' && noXhrPatch) {
      params.xhr = function() {
        return new ActiveXObject("Microsoft.XMLHTTP");
      };
    }

    // Make the request, allowing the user to override any Ajax options.
    var xhr = options.xhr = Backbone.ajax(_.extend(params, options));
    model.trigger('request', model, xhr, options);
    return xhr;
  };

  var noXhrPatch =
    typeof window !== 'undefined' && !!window.ActiveXObject &&
      !(window.XMLHttpRequest && (new XMLHttpRequest).dispatchEvent);

  // Map from CRUD to HTTP for our default `Backbone.sync` implementation.
  var methodMap = {
    'create': 'POST',
    'update': 'PUT',
    'patch':  'PATCH',
    'delete': 'DELETE',
    'read':   'GET'
  };

  // Set the default implementation of `Backbone.ajax` to proxy through to `$`.
  // Override this if you'd like to use a different library.
  Backbone.ajax = function() {
    return Backbone.$.ajax.apply(Backbone.$, arguments);
  };

  // Backbone.Router
  // ---------------

  // Routers map faux-URLs to actions, and fire events when routes are
  // matched. Creating a new one sets its `routes` hash, if not set statically.
  var Router = Backbone.Router = function(options) {
    options || (options = {});
    if (options.routes) this.routes = options.routes;
    this._bindRoutes();
    this.initialize.apply(this, arguments);
  };

  // Cached regular expressions for matching named param parts and splatted
  // parts of route strings.
  var optionalParam = /\((.*?)\)/g;
  var namedParam    = /(\(\?)?:\w+/g;
  var splatParam    = /\*\w+/g;
  var escapeRegExp  = /[\-{}\[\]+?.,\\\^$|#\s]/g;

  // Set up all inheritable **Backbone.Router** properties and methods.
  _.extend(Router.prototype, Events, {

    // Initialize is an empty function by default. Override it with your own
    // initialization logic.
    initialize: function(){},

    // Manually bind a single named route to a callback. For example:
    //
    //     this.route('search/:query/p:num', 'search', function(query, num) {
    //       ...
    //     });
    //
    route: function(route, name, callback) {
      if (!_.isRegExp(route)) route = this._routeToRegExp(route);
      if (_.isFunction(name)) {
        callback = name;
        name = '';
      }
      if (!callback) callback = this[name];
      var router = this;
      Backbone.history.route(route, function(fragment) {
        var args = router._extractParameters(route, fragment);
        router.execute(callback, args);
        router.trigger.apply(router, ['route:' + name].concat(args));
        router.trigger('route', name, args);
        Backbone.history.trigger('route', router, name, args);
      });
      return this;
    },

    // Execute a route handler with the provided parameters.  This is an
    // excellent place to do pre-route setup or post-route cleanup.
    execute: function(callback, args) {
      if (callback) callback.apply(this, args);
    },

    // Simple proxy to `Backbone.history` to save a fragment into the history.
    navigate: function(fragment, options) {
      Backbone.history.navigate(fragment, options);
      return this;
    },

    // Bind all defined routes to `Backbone.history`. We have to reverse the
    // order of the routes here to support behavior where the most general
    // routes can be defined at the bottom of the route map.
    _bindRoutes: function() {
      if (!this.routes) return;
      this.routes = _.result(this, 'routes');
      var route, routes = _.keys(this.routes);
      while ((route = routes.pop()) != null) {
        this.route(route, this.routes[route]);
      }
    },

    // Convert a route string into a regular expression, suitable for matching
    // against the current location hash.
    _routeToRegExp: function(route) {
      route = route.replace(escapeRegExp, '\\$&')
                   .replace(optionalParam, '(?:$1)?')
                   .replace(namedParam, function(match, optional) {
                     return optional ? match : '([^/?]+)';
                   })
                   .replace(splatParam, '([^?]*?)');
      return new RegExp('^' + route + '(?:\\?([\\s\\S]*))?$');
    },

    // Given a route, and a URL fragment that it matches, return the array of
    // extracted decoded parameters. Empty or unmatched parameters will be
    // treated as `null` to normalize cross-browser behavior.
    _extractParameters: function(route, fragment) {
      var params = route.exec(fragment).slice(1);
      return _.map(params, function(param, i) {
        // Don't decode the search params.
        if (i === params.length - 1) return param || null;
        return param ? decodeURIComponent(param) : null;
      });
    }

  });

  // Backbone.History
  // ----------------

  // Handles cross-browser history management, based on either
  // [pushState](http://diveintohtml5.info/history.html) and real URLs, or
  // [onhashchange](https://developer.mozilla.org/en-US/docs/DOM/window.onhashchange)
  // and URL fragments. If the browser supports neither (old IE, natch),
  // falls back to polling.
  var History = Backbone.History = function() {
    this.handlers = [];
    _.bindAll(this, 'checkUrl');

    // Ensure that `History` can be used outside of the browser.
    if (typeof window !== 'undefined') {
      this.location = window.location;
      this.history = window.history;
    }
  };

  // Cached regex for stripping a leading hash/slash and trailing space.
  var routeStripper = /^[#\/]|\s+$/g;

  // Cached regex for stripping leading and trailing slashes.
  var rootStripper = /^\/+|\/+$/g;

  // Cached regex for detecting MSIE.
  var isExplorer = /msie [\w.]+/;

  // Cached regex for removing a trailing slash.
  var trailingSlash = /\/$/;

  // Cached regex for stripping urls of hash.
  var pathStripper = /#.*$/;

  // Has the history handling already been started?
  History.started = false;

  // Set up all inheritable **Backbone.History** properties and methods.
  _.extend(History.prototype, Events, {

    // The default interval to poll for hash changes, if necessary, is
    // twenty times a second.
    interval: 50,

    // Are we at the app root?
    atRoot: function() {
      return this.location.pathname.replace(/[^\/]$/, '$&/') === this.root;
    },

    // Gets the true hash value. Cannot use location.hash directly due to bug
    // in Firefox where location.hash will always be decoded.
    getHash: function(window) {
      var match = (window || this).location.href.match(/#(.*)$/);
      return match ? match[1] : '';
    },

    // Get the cross-browser normalized URL fragment, either from the URL,
    // the hash, or the override.
    getFragment: function(fragment, forcePushState) {
      if (fragment == null) {
        if (this._hasPushState || !this._wantsHashChange || forcePushState) {
          fragment = decodeURI(this.location.pathname + this.location.search);
          var root = this.root.replace(trailingSlash, '');
          if (!fragment.indexOf(root)) fragment = fragment.slice(root.length);
        } else {
          fragment = this.getHash();
        }
      }
      return fragment.replace(routeStripper, '');
    },

    // Start the hash change handling, returning `true` if the current URL matches
    // an existing route, and `false` otherwise.
    start: function(options) {
      if (History.started) throw new Error("Backbone.history has already been started");
      History.started = true;

      // Figure out the initial configuration. Do we need an iframe?
      // Is pushState desired ... is it available?
      this.options          = _.extend({root: '/'}, this.options, options);
      this.root             = this.options.root;
      this._wantsHashChange = this.options.hashChange !== false;
      this._wantsPushState  = !!this.options.pushState;
      this._hasPushState    = !!(this.options.pushState && this.history && this.history.pushState);
      var fragment          = this.getFragment();
      var docMode           = document.documentMode;
      var oldIE             = (isExplorer.exec(navigator.userAgent.toLowerCase()) && (!docMode || docMode <= 7));

      // Normalize root to always include a leading and trailing slash.
      this.root = ('/' + this.root + '/').replace(rootStripper, '/');

      if (oldIE && this._wantsHashChange) {
        var frame = Backbone.$('<iframe src="javascript:0" tabindex="-1">');
        this.iframe = frame.hide().appendTo('body')[0].contentWindow;
        this.navigate(fragment);
      }

      // Depending on whether we're using pushState or hashes, and whether
      // 'onhashchange' is supported, determine how we check the URL state.
      if (this._hasPushState) {
        Backbone.$(window).on('popstate', this.checkUrl);
      } else if (this._wantsHashChange && ('onhashchange' in window) && !oldIE) {
        Backbone.$(window).on('hashchange', this.checkUrl);
      } else if (this._wantsHashChange) {
        this._checkUrlInterval = setInterval(this.checkUrl, this.interval);
      }

      // Determine if we need to change the base url, for a pushState link
      // opened by a non-pushState browser.
      this.fragment = fragment;
      var loc = this.location;

      // Transition from hashChange to pushState or vice versa if both are
      // requested.
      if (this._wantsHashChange && this._wantsPushState) {

        // If we've started off with a route from a `pushState`-enabled
        // browser, but we're currently in a browser that doesn't support it...
        if (!this._hasPushState && !this.atRoot()) {
          this.fragment = this.getFragment(null, true);
          this.location.replace(this.root + '#' + this.fragment);
          // Return immediately as browser will do redirect to new url
          return true;

        // Or if we've started out with a hash-based route, but we're currently
        // in a browser where it could be `pushState`-based instead...
        } else if (this._hasPushState && this.atRoot() && loc.hash) {
          this.fragment = this.getHash().replace(routeStripper, '');
          this.history.replaceState({}, document.title, this.root + this.fragment);
        }

      }

      if (!this.options.silent) return this.loadUrl();
    },

    // Disable Backbone.history, perhaps temporarily. Not useful in a real app,
    // but possibly useful for unit testing Routers.
    stop: function() {
      Backbone.$(window).off('popstate', this.checkUrl).off('hashchange', this.checkUrl);
      if (this._checkUrlInterval) clearInterval(this._checkUrlInterval);
      History.started = false;
    },

    // Add a route to be tested when the fragment changes. Routes added later
    // may override previous routes.
    route: function(route, callback) {
      this.handlers.unshift({route: route, callback: callback});
    },

    // Checks the current URL to see if it has changed, and if it has,
    // calls `loadUrl`, normalizing across the hidden iframe.
    checkUrl: function(e) {
      var current = this.getFragment();
      if (current === this.fragment && this.iframe) {
        current = this.getFragment(this.getHash(this.iframe));
      }
      if (current === this.fragment) return false;
      if (this.iframe) this.navigate(current);
      this.loadUrl();
    },

    // Attempt to load the current URL fragment. If a route succeeds with a
    // match, returns `true`. If no defined routes matches the fragment,
    // returns `false`.
    loadUrl: function(fragment) {
      fragment = this.fragment = this.getFragment(fragment);
      return _.any(this.handlers, function(handler) {
        if (handler.route.test(fragment)) {
          handler.callback(fragment);
          return true;
        }
      });
    },

    // Save a fragment into the hash history, or replace the URL state if the
    // 'replace' option is passed. You are responsible for properly URL-encoding
    // the fragment in advance.
    //
    // The options object can contain `trigger: true` if you wish to have the
    // route callback be fired (not usually desirable), or `replace: true`, if
    // you wish to modify the current URL without adding an entry to the history.
    navigate: function(fragment, options) {
      if (!History.started) return false;
      if (!options || options === true) options = {trigger: !!options};

      var url = this.root + (fragment = this.getFragment(fragment || ''));

      // Strip the hash for matching.
      fragment = fragment.replace(pathStripper, '');

      if (this.fragment === fragment) return;
      this.fragment = fragment;

      // Don't include a trailing slash on the root.
      if (fragment === '' && url !== '/') url = url.slice(0, -1);

      // If pushState is available, we use it to set the fragment as a real URL.
      if (this._hasPushState) {
        this.history[options.replace ? 'replaceState' : 'pushState']({}, document.title, url);

      // If hash changes haven't been explicitly disabled, update the hash
      // fragment to store history.
      } else if (this._wantsHashChange) {
        this._updateHash(this.location, fragment, options.replace);
        if (this.iframe && (fragment !== this.getFragment(this.getHash(this.iframe)))) {
          // Opening and closing the iframe tricks IE7 and earlier to push a
          // history entry on hash-tag change.  When replace is true, we don't
          // want this.
          if(!options.replace) this.iframe.document.open().close();
          this._updateHash(this.iframe.location, fragment, options.replace);
        }

      // If you've told us that you explicitly don't want fallback hashchange-
      // based history, then `navigate` becomes a page refresh.
      } else {
        return this.location.assign(url);
      }
      if (options.trigger) return this.loadUrl(fragment);
    },

    // Update the hash location, either replacing the current entry, or adding
    // a new one to the browser history.
    _updateHash: function(location, fragment, replace) {
      if (replace) {
        var href = location.href.replace(/(javascript:|#).*$/, '');
        location.replace(href + '#' + fragment);
      } else {
        // Some browsers require that `hash` contains a leading #.
        location.hash = '#' + fragment;
      }
    }

  });

  // Create the default Backbone.history.
  Backbone.history = new History;

  // Helpers
  // -------

  // Helper function to correctly set up the prototype chain, for subclasses.
  // Similar to `goog.inherits`, but uses a hash of prototype properties and
  // class properties to be extended.
  var extend = function(protoProps, staticProps) {
    var parent = this;
    var child;

    // The constructor function for the new subclass is either defined by you
    // (the "constructor" property in your `extend` definition), or defaulted
    // by us to simply call the parent's constructor.
    if (protoProps && _.has(protoProps, 'constructor')) {
      child = protoProps.constructor;
    } else {
      child = function(){ return parent.apply(this, arguments); };
    }

    // Add static properties to the constructor function, if supplied.
    _.extend(child, parent, staticProps);

    // Set the prototype chain to inherit from `parent`, without calling
    // `parent`'s constructor function.
    var Surrogate = function(){ this.constructor = child; };
    Surrogate.prototype = parent.prototype;
    child.prototype = new Surrogate;

    // Add prototype properties (instance properties) to the subclass,
    // if supplied.
    if (protoProps) _.extend(child.prototype, protoProps);

    // Set a convenience property in case the parent's prototype is needed
    // later.
    child.__super__ = parent.prototype;

    return child;
  };

  // Set up inheritance for the model, collection, router, view and history.
  Model.extend = Collection.extend = Router.extend = View.extend = History.extend = extend;

  // Throw an error when a URL is needed, and none is supplied.
  var urlError = function() {
    throw new Error('A "url" property or function must be specified');
  };

  // Wrap an optional error callback with a fallback error event.
  var wrapError = function(model, options) {
    var error = options.error;
    options.error = function(resp) {
      if (error) error(model, resp, options);
      model.trigger('error', model, resp, options);
    };
  };

  return Backbone;

}));


define('SE',['backbone'],function(Backbone){
	var SE = {};

	//SE.Emitter
	//----------
	//Emitter is our basic class, with event support & extend function
	SE.Emitter = function () {
	};
	SE.Emitter.extend = Backbone.Model.extend;
	_.extend(SE.Emitter, Backbone.Events);
	_.extend(SE.Emitter.prototype, Backbone.Events);
	return SE;
})
;
define('SE.Engine',['backbone','SE'],function(Backbone,SE){
SE.Engine = {};
//SE.Engine.Base
//--------------
//interface for storage engines which describes a certain `sync` method.
SE.Engine.Base = SE.Emitter.extend({
  //the default implement is proxy to `this[method]` so you can implement `create`, `read`, `update`, and `delete` to describe the sync behavior.
  sync: function (method, model, options) {
    return this[method](model, options);
  }
}, {
  createSync: function () {
    var engine, sync;

    engine = this.construct.apply(this, arguments);
    sync = _.bind(engine.sync, engine);
    sync.engine = engine;

    return sync;
  },
  //implement this method to make an instance of Engine class.  
  //you may want to use `_.memoize` on this method to singleton your Engine.
  construct: function () {
    throw '`construct` factory method not implemented';
  }
});
});

define('SE.Engine.LocalStorage',['backbone','SE','SE.Engine'],function(Backbone,SE){
//SE.Engine.LocalStorage
//--------------
//Basically copy-paste from <https://github.com/jeromegn/Backbone.localStorage>
//
//    Licensed under MIT license
//
//    Copyright (c) 2010 Jerome Gravel-Niquet
//
//    Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
//
//    The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
//    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

SE.Engine.LocalStorage = (function (root, _, Backbone) {

  // A simple module to replace `Backbone.sync` with *localStorage*-based
  // persistence. Models are given GUIDS, and saved into a JSON object. Simple
  // as that.

  // Hold reference to Underscore.js and Backbone.js in the closure in order
  // to make things work even if they are removed from the global namespace

  // Generate a pseudo-GUID by concatenating random hexadecimal.
  function guid() {
    // Generate four random hex digits.
    function s4() {
      return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);
    }

    return (s4() + s4() + "-" + s4() + "-" + s4() + "-" + s4() + "-" + s4() + s4() + s4());
  }

  // Our Store is represented by a single JS object in *localStorage*. Create it
  // with a meaningful name, like the name you'd give a table.
  // window.Store is deprectated, use Backbone.LocalStorage instead
  var Store = function (name) {
    if (!this.localStorage) {
      throw "Backbone.localStorage: Environment does not support localStorage.";
    }
    this.name = name;
    var store = this.localStorage().getItem(this.name);
    this.records = (store && store.split(",")) || [];
  };

  _.extend(Store.prototype, {

    // Save the current state of the **Store** to *localStorage*.
    save: function () {
      this.localStorage().setItem(this.name, this.records.join(","));
    },

    // Add a model, giving it a (hopefully)-unique GUID, if it doesn't already
    // have an id of it's own.
    create: function (model) {
      if (!model.id) {
        model.id = guid();
        model.set(model.idAttribute, model.id);
      }
      this.localStorage().setItem(this.name + "-" + model.id, JSON.stringify(model));
      this.records.push(model.id.toString());
      this.save();
      return this.find(model);
    },

    // Update a model by replacing its copy in `this.data`.
    update: function (model) {
      this.localStorage().setItem(this.name + "-" + model.id, JSON.stringify(model));
      if (!_.include(this.records, model.id.toString())) {
        this.records.push(model.id.toString());
      }
      this.save();
      return this.find(model);
    },

    // Retrieve a model from `this.data` by id.
    find: function (model) {
      return this.jsonData(this.localStorage().getItem(this.name + "-" + model.id));
    },

    // Return the array of all models currently in storage.
    findAll: function () {
      // Lodash removed _#chain in v1.0.0-rc.1
      return (_.chain || _)(this.records)
        .map(function (id) {
          return this.jsonData(this.localStorage().getItem(this.name + "-" + id));
        }, this)
        .compact()
        .value();
    },

    // Delete a model from `this.data`, returning it.
    destroy: function (model) {
      if (model.isNew()) {
        return false;
      }
      this.localStorage().removeItem(this.name + "-" + model.id);
      this.records = _.reject(this.records, function (id) {
        return id === model.id.toString();
      });
      this.save();
      return model;
    },

    localStorage: function () {
      return root.localStorage;
    },

    // fix for "illegal access" error on Android when JSON.parse is passed null
    jsonData: function (data) {
      return data && JSON.parse(data);
    },

    // Clear localStorage for specific collection.
    _clear: function () {
      var local = this.localStorage(),
        itemRe = new RegExp("^" + this.name + "-");

      // Remove id-tracking item (e.g., "foo").
      local.removeItem(this.name);

      // Lodash removed _#chain in v1.0.0-rc.1
      // Match all data items (e.g., "foo-ID") and remove.
      (_.chain || _)(local).keys()
        .filter(function (k) {
          return itemRe.test(k);
        })
        .each(function (k) {
          local.removeItem(k);
        });

      this.records.length = 0;
    },

    // Size of localStorage.
    _storageSize: function () {
      return this.localStorage().length;
    }

  });


  return SE.Engine.Base.extend({
    constructor: function (name) {
      SE.Engine.Base.prototype.constructor.apply(this, arguments);

      this.store = new Store(name);
    },
    // localSync delegate to the model or collection's
    // *localStorage* property, which should be an instance of `Store`.
    // window.Store.sync and Backbone.localSync is deprecated, use Backbone.LocalStorage.sync instead
    sync: function (method, model, options) {
      var store = this.store,
        resp,
        errorMessage,
        syncDfd = Backbone.$.Deferred && Backbone.$.Deferred(); //If $ is having Deferred - use it.

      try {

        switch (method) {
        case "read":
          resp = model.id !== undefined ? store.find(model) : store.findAll();
          break;
        case "create":
          resp = store.create(model);
          break;
        case "update":
          resp = store.update(model);
          break;
        case "delete":
          resp = store.destroy(model);
          break;
        }

      } catch (error) {
        if (error.code === 22 && store._storageSize() === 0) {
          errorMessage = "Private browsing is unsupported";
        } else {
          errorMessage = error.message;
        }
      }

      if (resp) {
        if (options && options.success) {
          if (Backbone.VERSION === "0.9.10") {
            options.success(model, resp, options);
          } else {
            options.success(resp);
          }
        }
        if (syncDfd) {
          syncDfd.resolve(resp);
        }

      } else {
        errorMessage = errorMessage || "Record Not Found";

        if (options && options.error) {
          if (Backbone.VERSION === "0.9.10") {
            options.error(model, errorMessage, options);
          } else {
            options.error(errorMessage);
          }
        }

        if (syncDfd) {
          syncDfd.reject(errorMessage);
        }
      }

      // add compatibility with $.ajax
      // always execute callback for success and error
      if (options && options.complete) {
        options.complete(resp);
      }

      return syncDfd && syncDfd.promise();
    }
  }, {
    construct: _.memoize(function (name) {
      return new this(name);
    })
  });

}(this, _, Backbone));
});

define("StoreEngine",
	[
		'SE',
		'SE.Engine.LocalStorage',
	], 
	function (SE,LocalStorage) {
		
		return SE.Engine.LocalStorage;

	}
);

//! moment.js
//! version : 2.4.0
//! authors : Tim Wood, Iskren Chernev, Moment.js contributors
//! license : MIT
//! momentjs.com

(function (undefined) {

    /************************************
        Constants
    ************************************/

    var moment,
        VERSION = "2.4.0",
        round = Math.round,
        i,

        YEAR = 0,
        MONTH = 1,
        DATE = 2,
        HOUR = 3,
        MINUTE = 4,
        SECOND = 5,
        MILLISECOND = 6,

        // internal storage for language config files
        languages = {},

        // check for nodeJS
        hasModule = (typeof module !== 'undefined' && module.exports),

        // ASP.NET json date format regex
        aspNetJsonRegex = /^\/?Date\((\-?\d+)/i,
        aspNetTimeSpanJsonRegex = /(\-)?(?:(\d*)\.)?(\d+)\:(\d+)(?:\:(\d+)\.?(\d{3})?)?/,

        // from http://docs.closure-library.googlecode.com/git/closure_goog_date_date.js.source.html
        // somewhat more in line with 4.4.3.2 2004 spec, but allows decimal anywhere
        isoDurationRegex = /^(-)?P(?:(?:([0-9,.]*)Y)?(?:([0-9,.]*)M)?(?:([0-9,.]*)D)?(?:T(?:([0-9,.]*)H)?(?:([0-9,.]*)M)?(?:([0-9,.]*)S)?)?|([0-9,.]*)W)$/,

        // format tokens
        formattingTokens = /(\[[^\[]*\])|(\\)?(Mo|MM?M?M?|Do|DDDo|DD?D?D?|ddd?d?|do?|w[o|w]?|W[o|W]?|YYYYY|YYYY|YY|gg(ggg?)?|GG(GGG?)?|e|E|a|A|hh?|HH?|mm?|ss?|S{1,4}|X|zz?|ZZ?|.)/g,
        localFormattingTokens = /(\[[^\[]*\])|(\\)?(LT|LL?L?L?|l{1,4})/g,

        // parsing token regexes
        parseTokenOneOrTwoDigits = /\d\d?/, // 0 - 99
        parseTokenOneToThreeDigits = /\d{1,3}/, // 0 - 999
        parseTokenThreeDigits = /\d{3}/, // 000 - 999
        parseTokenFourDigits = /\d{1,4}/, // 0 - 9999
        parseTokenSixDigits = /[+\-]?\d{1,6}/, // -999,999 - 999,999
        parseTokenDigits = /\d+/, // nonzero number of digits
        parseTokenWord = /[0-9]*['a-z\u00A0-\u05FF\u0700-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]+|[\u0600-\u06FF\/]+(\s*?[\u0600-\u06FF]+){1,2}/i, // any word (or two) characters or numbers including two/three word month in arabic.
        parseTokenTimezone = /Z|[\+\-]\d\d:?\d\d/i, // +00:00 -00:00 +0000 -0000 or Z
        parseTokenT = /T/i, // T (ISO seperator)
        parseTokenTimestampMs = /[\+\-]?\d+(\.\d{1,3})?/, // 123456789 123456789.123

        // preliminary iso regex
        // 0000-00-00 0000-W00 or 0000-W00-0 + T + 00 or 00:00 or 00:00:00 or 00:00:00.000 + +00:00 or +0000)
        isoRegex = /^\s*\d{4}-(?:(\d\d-\d\d)|(W\d\d$)|(W\d\d-\d)|(\d\d\d))((T| )(\d\d(:\d\d(:\d\d(\.\d+)?)?)?)?([\+\-]\d\d:?\d\d|Z)?)?$/,

        isoFormat = 'YYYY-MM-DDTHH:mm:ssZ',

        isoDates = [
            'YYYY-MM-DD',
            'GGGG-[W]WW',
            'GGGG-[W]WW-E',
            'YYYY-DDD'
        ],

        // iso time formats and regexes
        isoTimes = [
            ['HH:mm:ss.SSSS', /(T| )\d\d:\d\d:\d\d\.\d{1,3}/],
            ['HH:mm:ss', /(T| )\d\d:\d\d:\d\d/],
            ['HH:mm', /(T| )\d\d:\d\d/],
            ['HH', /(T| )\d\d/]
        ],

        // timezone chunker "+10:00" > ["10", "00"] or "-1530" > ["-15", "30"]
        parseTimezoneChunker = /([\+\-]|\d\d)/gi,

        // getter and setter names
        proxyGettersAndSetters = 'Date|Hours|Minutes|Seconds|Milliseconds'.split('|'),
        unitMillisecondFactors = {
            'Milliseconds' : 1,
            'Seconds' : 1e3,
            'Minutes' : 6e4,
            'Hours' : 36e5,
            'Days' : 864e5,
            'Months' : 2592e6,
            'Years' : 31536e6
        },

        unitAliases = {
            ms : 'millisecond',
            s : 'second',
            m : 'minute',
            h : 'hour',
            d : 'day',
            D : 'date',
            w : 'week',
            W : 'isoWeek',
            M : 'month',
            y : 'year',
            DDD : 'dayOfYear',
            e : 'weekday',
            E : 'isoWeekday',
            gg: 'weekYear',
            GG: 'isoWeekYear'
        },

        camelFunctions = {
            dayofyear : 'dayOfYear',
            isoweekday : 'isoWeekday',
            isoweek : 'isoWeek',
            weekyear : 'weekYear',
            isoweekyear : 'isoWeekYear'
        },

        // format function strings
        formatFunctions = {},

        // tokens to ordinalize and pad
        ordinalizeTokens = 'DDD w W M D d'.split(' '),
        paddedTokens = 'M D H h m s w W'.split(' '),

        formatTokenFunctions = {
            M    : function () {
                return this.month() + 1;
            },
            MMM  : function (format) {
                return this.lang().monthsShort(this, format);
            },
            MMMM : function (format) {
                return this.lang().months(this, format);
            },
            D    : function () {
                return this.date();
            },
            DDD  : function () {
                return this.dayOfYear();
            },
            d    : function () {
                return this.day();
            },
            dd   : function (format) {
                return this.lang().weekdaysMin(this, format);
            },
            ddd  : function (format) {
                return this.lang().weekdaysShort(this, format);
            },
            dddd : function (format) {
                return this.lang().weekdays(this, format);
            },
            w    : function () {
                return this.week();
            },
            W    : function () {
                return this.isoWeek();
            },
            YY   : function () {
                return leftZeroFill(this.year() % 100, 2);
            },
            YYYY : function () {
                return leftZeroFill(this.year(), 4);
            },
            YYYYY : function () {
                return leftZeroFill(this.year(), 5);
            },
            gg   : function () {
                return leftZeroFill(this.weekYear() % 100, 2);
            },
            gggg : function () {
                return this.weekYear();
            },
            ggggg : function () {
                return leftZeroFill(this.weekYear(), 5);
            },
            GG   : function () {
                return leftZeroFill(this.isoWeekYear() % 100, 2);
            },
            GGGG : function () {
                return this.isoWeekYear();
            },
            GGGGG : function () {
                return leftZeroFill(this.isoWeekYear(), 5);
            },
            e : function () {
                return this.weekday();
            },
            E : function () {
                return this.isoWeekday();
            },
            a    : function () {
                return this.lang().meridiem(this.hours(), this.minutes(), true);
            },
            A    : function () {
                return this.lang().meridiem(this.hours(), this.minutes(), false);
            },
            H    : function () {
                return this.hours();
            },
            h    : function () {
                return this.hours() % 12 || 12;
            },
            m    : function () {
                return this.minutes();
            },
            s    : function () {
                return this.seconds();
            },
            S    : function () {
                return toInt(this.milliseconds() / 100);
            },
            SS   : function () {
                return leftZeroFill(toInt(this.milliseconds() / 10), 2);
            },
            SSS  : function () {
                return leftZeroFill(this.milliseconds(), 3);
            },
            SSSS : function () {
                return leftZeroFill(this.milliseconds(), 3);
            },
            Z    : function () {
                var a = -this.zone(),
                    b = "+";
                if (a < 0) {
                    a = -a;
                    b = "-";
                }
                return b + leftZeroFill(toInt(a / 60), 2) + ":" + leftZeroFill(toInt(a) % 60, 2);
            },
            ZZ   : function () {
                var a = -this.zone(),
                    b = "+";
                if (a < 0) {
                    a = -a;
                    b = "-";
                }
                return b + leftZeroFill(toInt(10 * a / 6), 4);
            },
            z : function () {
                return this.zoneAbbr();
            },
            zz : function () {
                return this.zoneName();
            },
            X    : function () {
                return this.unix();
            }
        },

        lists = ['months', 'monthsShort', 'weekdays', 'weekdaysShort', 'weekdaysMin'];

    function padToken(func, count) {
        return function (a) {
            return leftZeroFill(func.call(this, a), count);
        };
    }
    function ordinalizeToken(func, period) {
        return function (a) {
            return this.lang().ordinal(func.call(this, a), period);
        };
    }

    while (ordinalizeTokens.length) {
        i = ordinalizeTokens.pop();
        formatTokenFunctions[i + 'o'] = ordinalizeToken(formatTokenFunctions[i], i);
    }
    while (paddedTokens.length) {
        i = paddedTokens.pop();
        formatTokenFunctions[i + i] = padToken(formatTokenFunctions[i], 2);
    }
    formatTokenFunctions.DDDD = padToken(formatTokenFunctions.DDD, 3);


    /************************************
        Constructors
    ************************************/

    function Language() {

    }

    // Moment prototype object
    function Moment(config) {
        checkOverflow(config);
        extend(this, config);
    }

    // Duration Constructor
    function Duration(duration) {
        var normalizedInput = normalizeObjectUnits(duration),
            years = normalizedInput.year || 0,
            months = normalizedInput.month || 0,
            weeks = normalizedInput.week || 0,
            days = normalizedInput.day || 0,
            hours = normalizedInput.hour || 0,
            minutes = normalizedInput.minute || 0,
            seconds = normalizedInput.second || 0,
            milliseconds = normalizedInput.millisecond || 0;

        // store reference to input for deterministic cloning
        this._input = duration;

        // representation for dateAddRemove
        this._milliseconds = +milliseconds +
            seconds * 1e3 + // 1000
            minutes * 6e4 + // 1000 * 60
            hours * 36e5; // 1000 * 60 * 60
        // Because of dateAddRemove treats 24 hours as different from a
        // day when working around DST, we need to store them separately
        this._days = +days +
            weeks * 7;
        // It is impossible translate months into days without knowing
        // which months you are are talking about, so we have to store
        // it separately.
        this._months = +months +
            years * 12;

        this._data = {};

        this._bubble();
    }

    /************************************
        Helpers
    ************************************/


    function extend(a, b) {
        for (var i in b) {
            if (b.hasOwnProperty(i)) {
                a[i] = b[i];
            }
        }

        if (b.hasOwnProperty("toString")) {
            a.toString = b.toString;
        }

        if (b.hasOwnProperty("valueOf")) {
            a.valueOf = b.valueOf;
        }

        return a;
    }

    function absRound(number) {
        if (number < 0) {
            return Math.ceil(number);
        } else {
            return Math.floor(number);
        }
    }

    // left zero fill a number
    // see http://jsperf.com/left-zero-filling for performance comparison
    function leftZeroFill(number, targetLength) {
        var output = number + '';
        while (output.length < targetLength) {
            output = '0' + output;
        }
        return output;
    }

    // helper function for _.addTime and _.subtractTime
    function addOrSubtractDurationFromMoment(mom, duration, isAdding, ignoreUpdateOffset) {
        var milliseconds = duration._milliseconds,
            days = duration._days,
            months = duration._months,
            minutes,
            hours;

        if (milliseconds) {
            mom._d.setTime(+mom._d + milliseconds * isAdding);
        }
        // store the minutes and hours so we can restore them
        if (days || months) {
            minutes = mom.minute();
            hours = mom.hour();
        }
        if (days) {
            mom.date(mom.date() + days * isAdding);
        }
        if (months) {
            mom.month(mom.month() + months * isAdding);
        }
        if (milliseconds && !ignoreUpdateOffset) {
            moment.updateOffset(mom);
        }
        // restore the minutes and hours after possibly changing dst
        if (days || months) {
            mom.minute(minutes);
            mom.hour(hours);
        }
    }

    // check if is an array
    function isArray(input) {
        return Object.prototype.toString.call(input) === '[object Array]';
    }

    function isDate(input) {
        return  Object.prototype.toString.call(input) === '[object Date]' ||
                input instanceof Date;
    }

    // compare two arrays, return the number of differences
    function compareArrays(array1, array2, dontConvert) {
        var len = Math.min(array1.length, array2.length),
            lengthDiff = Math.abs(array1.length - array2.length),
            diffs = 0,
            i;
        for (i = 0; i < len; i++) {
            if ((dontConvert && array1[i] !== array2[i]) ||
                (!dontConvert && toInt(array1[i]) !== toInt(array2[i]))) {
                diffs++;
            }
        }
        return diffs + lengthDiff;
    }

    function normalizeUnits(units) {
        if (units) {
            var lowered = units.toLowerCase().replace(/(.)s$/, '$1');
            units = unitAliases[units] || camelFunctions[lowered] || lowered;
        }
        return units;
    }

    function normalizeObjectUnits(inputObject) {
        var normalizedInput = {},
            normalizedProp,
            prop,
            index;

        for (prop in inputObject) {
            if (inputObject.hasOwnProperty(prop)) {
                normalizedProp = normalizeUnits(prop);
                if (normalizedProp) {
                    normalizedInput[normalizedProp] = inputObject[prop];
                }
            }
        }

        return normalizedInput;
    }

    function makeList(field) {
        var count, setter;

        if (field.indexOf('week') === 0) {
            count = 7;
            setter = 'day';
        }
        else if (field.indexOf('month') === 0) {
            count = 12;
            setter = 'month';
        }
        else {
            return;
        }

        moment[field] = function (format, index) {
            var i, getter,
                method = moment.fn._lang[field],
                results = [];

            if (typeof format === 'number') {
                index = format;
                format = undefined;
            }

            getter = function (i) {
                var m = moment().utc().set(setter, i);
                return method.call(moment.fn._lang, m, format || '');
            };

            if (index != null) {
                return getter(index);
            }
            else {
                for (i = 0; i < count; i++) {
                    results.push(getter(i));
                }
                return results;
            }
        };
    }

    function toInt(argumentForCoercion) {
        var coercedNumber = +argumentForCoercion,
            value = 0;

        if (coercedNumber !== 0 && isFinite(coercedNumber)) {
            if (coercedNumber >= 0) {
                value = Math.floor(coercedNumber);
            } else {
                value = Math.ceil(coercedNumber);
            }
        }

        return value;
    }

    function daysInMonth(year, month) {
        return new Date(Date.UTC(year, month + 1, 0)).getUTCDate();
    }

    function daysInYear(year) {
        return isLeapYear(year) ? 366 : 365;
    }

    function isLeapYear(year) {
        return (year % 4 === 0 && year % 100 !== 0) || year % 400 === 0;
    }

    function checkOverflow(m) {
        var overflow;
        if (m._a && m._pf.overflow === -2) {
            overflow =
                m._a[MONTH] < 0 || m._a[MONTH] > 11 ? MONTH :
                m._a[DATE] < 1 || m._a[DATE] > daysInMonth(m._a[YEAR], m._a[MONTH]) ? DATE :
                m._a[HOUR] < 0 || m._a[HOUR] > 23 ? HOUR :
                m._a[MINUTE] < 0 || m._a[MINUTE] > 59 ? MINUTE :
                m._a[SECOND] < 0 || m._a[SECOND] > 59 ? SECOND :
                m._a[MILLISECOND] < 0 || m._a[MILLISECOND] > 999 ? MILLISECOND :
                -1;

            if (m._pf._overflowDayOfYear && (overflow < YEAR || overflow > DATE)) {
                overflow = DATE;
            }

            m._pf.overflow = overflow;
        }
    }

    function initializeParsingFlags(config) {
        config._pf = {
            empty : false,
            unusedTokens : [],
            unusedInput : [],
            overflow : -2,
            charsLeftOver : 0,
            nullInput : false,
            invalidMonth : null,
            invalidFormat : false,
            userInvalidated : false,
            iso: false
        };
    }

    function isValid(m) {
        if (m._isValid == null) {
            m._isValid = !isNaN(m._d.getTime()) &&
                m._pf.overflow < 0 &&
                !m._pf.empty &&
                !m._pf.invalidMonth &&
                !m._pf.nullInput &&
                !m._pf.invalidFormat &&
                !m._pf.userInvalidated;

            if (m._strict) {
                m._isValid = m._isValid &&
                    m._pf.charsLeftOver === 0 &&
                    m._pf.unusedTokens.length === 0;
            }
        }
        return m._isValid;
    }

    function normalizeLanguage(key) {
        return key ? key.toLowerCase().replace('_', '-') : key;
    }

    /************************************
        Languages
    ************************************/


    extend(Language.prototype, {

        set : function (config) {
            var prop, i;
            for (i in config) {
                prop = config[i];
                if (typeof prop === 'function') {
                    this[i] = prop;
                } else {
                    this['_' + i] = prop;
                }
            }
        },

        _months : "January_February_March_April_May_June_July_August_September_October_November_December".split("_"),
        months : function (m) {
            return this._months[m.month()];
        },

        _monthsShort : "Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),
        monthsShort : function (m) {
            return this._monthsShort[m.month()];
        },

        monthsParse : function (monthName) {
            var i, mom, regex;

            if (!this._monthsParse) {
                this._monthsParse = [];
            }

            for (i = 0; i < 12; i++) {
                // make the regex if we don't have it already
                if (!this._monthsParse[i]) {
                    mom = moment.utc([2000, i]);
                    regex = '^' + this.months(mom, '') + '|^' + this.monthsShort(mom, '');
                    this._monthsParse[i] = new RegExp(regex.replace('.', ''), 'i');
                }
                // test the regex
                if (this._monthsParse[i].test(monthName)) {
                    return i;
                }
            }
        },

        _weekdays : "Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),
        weekdays : function (m) {
            return this._weekdays[m.day()];
        },

        _weekdaysShort : "Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),
        weekdaysShort : function (m) {
            return this._weekdaysShort[m.day()];
        },

        _weekdaysMin : "Su_Mo_Tu_We_Th_Fr_Sa".split("_"),
        weekdaysMin : function (m) {
            return this._weekdaysMin[m.day()];
        },

        weekdaysParse : function (weekdayName) {
            var i, mom, regex;

            if (!this._weekdaysParse) {
                this._weekdaysParse = [];
            }

            for (i = 0; i < 7; i++) {
                // make the regex if we don't have it already
                if (!this._weekdaysParse[i]) {
                    mom = moment([2000, 1]).day(i);
                    regex = '^' + this.weekdays(mom, '') + '|^' + this.weekdaysShort(mom, '') + '|^' + this.weekdaysMin(mom, '');
                    this._weekdaysParse[i] = new RegExp(regex.replace('.', ''), 'i');
                }
                // test the regex
                if (this._weekdaysParse[i].test(weekdayName)) {
                    return i;
                }
            }
        },

        _longDateFormat : {
            LT : "h:mm A",
            L : "MM/DD/YYYY",
            LL : "MMMM D YYYY",
            LLL : "MMMM D YYYY LT",
            LLLL : "dddd, MMMM D YYYY LT"
        },
        longDateFormat : function (key) {
            var output = this._longDateFormat[key];
            if (!output && this._longDateFormat[key.toUpperCase()]) {
                output = this._longDateFormat[key.toUpperCase()].replace(/MMMM|MM|DD|dddd/g, function (val) {
                    return val.slice(1);
                });
                this._longDateFormat[key] = output;
            }
            return output;
        },

        isPM : function (input) {
            // IE8 Quirks Mode & IE7 Standards Mode do not allow accessing strings like arrays
            // Using charAt should be more compatible.
            return ((input + '').toLowerCase().charAt(0) === 'p');
        },

        _meridiemParse : /[ap]\.?m?\.?/i,
        meridiem : function (hours, minutes, isLower) {
            if (hours > 11) {
                return isLower ? 'pm' : 'PM';
            } else {
                return isLower ? 'am' : 'AM';
            }
        },

        _calendar : {
            sameDay : '[Today at] LT',
            nextDay : '[Tomorrow at] LT',
            nextWeek : 'dddd [at] LT',
            lastDay : '[Yesterday at] LT',
            lastWeek : '[Last] dddd [at] LT',
            sameElse : 'L'
        },
        calendar : function (key, mom) {
            var output = this._calendar[key];
            return typeof output === 'function' ? output.apply(mom) : output;
        },

        _relativeTime : {
            future : "in %s",
            past : "%s ago",
            s : "a few seconds",
            m : "a minute",
            mm : "%d minutes",
            h : "an hour",
            hh : "%d hours",
            d : "a day",
            dd : "%d days",
            M : "a month",
            MM : "%d months",
            y : "a year",
            yy : "%d years"
        },
        relativeTime : function (number, withoutSuffix, string, isFuture) {
            var output = this._relativeTime[string];
            return (typeof output === 'function') ?
                output(number, withoutSuffix, string, isFuture) :
                output.replace(/%d/i, number);
        },
        pastFuture : function (diff, output) {
            var format = this._relativeTime[diff > 0 ? 'future' : 'past'];
            return typeof format === 'function' ? format(output) : format.replace(/%s/i, output);
        },

        ordinal : function (number) {
            return this._ordinal.replace("%d", number);
        },
        _ordinal : "%d",

        preparse : function (string) {
            return string;
        },

        postformat : function (string) {
            return string;
        },

        week : function (mom) {
            return weekOfYear(mom, this._week.dow, this._week.doy).week;
        },

        _week : {
            dow : 0, // Sunday is the first day of the week.
            doy : 6  // The week that contains Jan 1st is the first week of the year.
        },

        _invalidDate: 'Invalid date',
        invalidDate: function () {
            return this._invalidDate;
        }
    });

    // Loads a language definition into the `languages` cache.  The function
    // takes a key and optionally values.  If not in the browser and no values
    // are provided, it will load the language file module.  As a convenience,
    // this function also returns the language values.
    function loadLang(key, values) {
        values.abbr = key;
        if (!languages[key]) {
            languages[key] = new Language();
        }
        languages[key].set(values);
        return languages[key];
    }

    // Remove a language from the `languages` cache. Mostly useful in tests.
    function unloadLang(key) {
        delete languages[key];
    }

    // Determines which language definition to use and returns it.
    //
    // With no parameters, it will return the global language.  If you
    // pass in a language key, such as 'en', it will return the
    // definition for 'en', so long as 'en' has already been loaded using
    // moment.lang.
    function getLangDefinition(key) {
        var i = 0, j, lang, next, split,
            get = function (k) {
                if (!languages[k] && hasModule) {
                    try {
                        require('./lang/' + k);
                    } catch (e) { }
                }
                return languages[k];
            };

        if (!key) {
            return moment.fn._lang;
        }

        if (!isArray(key)) {
            //short-circuit everything else
            lang = get(key);
            if (lang) {
                return lang;
            }
            key = [key];
        }

        //pick the language from the array
        //try ['en-au', 'en-gb'] as 'en-au', 'en-gb', 'en', as in move through the list trying each
        //substring from most specific to least, but move to the next array item if it's a more specific variant than the current root
        while (i < key.length) {
            split = normalizeLanguage(key[i]).split('-');
            j = split.length;
            next = normalizeLanguage(key[i + 1]);
            next = next ? next.split('-') : null;
            while (j > 0) {
                lang = get(split.slice(0, j).join('-'));
                if (lang) {
                    return lang;
                }
                if (next && next.length >= j && compareArrays(split, next, true) >= j - 1) {
                    //the next array item is better than a shallower substring of this one
                    break;
                }
                j--;
            }
            i++;
        }
        return moment.fn._lang;
    }

    /************************************
        Formatting
    ************************************/


    function removeFormattingTokens(input) {
        if (input.match(/\[[\s\S]/)) {
            return input.replace(/^\[|\]$/g, "");
        }
        return input.replace(/\\/g, "");
    }

    function makeFormatFunction(format) {
        var array = format.match(formattingTokens), i, length;

        for (i = 0, length = array.length; i < length; i++) {
            if (formatTokenFunctions[array[i]]) {
                array[i] = formatTokenFunctions[array[i]];
            } else {
                array[i] = removeFormattingTokens(array[i]);
            }
        }

        return function (mom) {
            var output = "";
            for (i = 0; i < length; i++) {
                output += array[i] instanceof Function ? array[i].call(mom, format) : array[i];
            }
            return output;
        };
    }

    // format date using native date object
    function formatMoment(m, format) {

        if (!m.isValid()) {
            return m.lang().invalidDate();
        }

        format = expandFormat(format, m.lang());

        if (!formatFunctions[format]) {
            formatFunctions[format] = makeFormatFunction(format);
        }

        return formatFunctions[format](m);
    }

    function expandFormat(format, lang) {
        var i = 5;

        function replaceLongDateFormatTokens(input) {
            return lang.longDateFormat(input) || input;
        }

        localFormattingTokens.lastIndex = 0;
        while (i >= 0 && localFormattingTokens.test(format)) {
            format = format.replace(localFormattingTokens, replaceLongDateFormatTokens);
            localFormattingTokens.lastIndex = 0;
            i -= 1;
        }

        return format;
    }


    /************************************
        Parsing
    ************************************/


    // get the regex to find the next token
    function getParseRegexForToken(token, config) {
        var a;
        switch (token) {
        case 'DDDD':
            return parseTokenThreeDigits;
        case 'YYYY':
        case 'GGGG':
        case 'gggg':
            return parseTokenFourDigits;
        case 'YYYYY':
        case 'GGGGG':
        case 'ggggg':
            return parseTokenSixDigits;
        case 'S':
        case 'SS':
        case 'SSS':
        case 'DDD':
            return parseTokenOneToThreeDigits;
        case 'MMM':
        case 'MMMM':
        case 'dd':
        case 'ddd':
        case 'dddd':
            return parseTokenWord;
        case 'a':
        case 'A':
            return getLangDefinition(config._l)._meridiemParse;
        case 'X':
            return parseTokenTimestampMs;
        case 'Z':
        case 'ZZ':
            return parseTokenTimezone;
        case 'T':
            return parseTokenT;
        case 'SSSS':
            return parseTokenDigits;
        case 'MM':
        case 'DD':
        case 'YY':
        case 'GG':
        case 'gg':
        case 'HH':
        case 'hh':
        case 'mm':
        case 'ss':
        case 'M':
        case 'D':
        case 'd':
        case 'H':
        case 'h':
        case 'm':
        case 's':
        case 'w':
        case 'ww':
        case 'W':
        case 'WW':
        case 'e':
        case 'E':
            return parseTokenOneOrTwoDigits;
        default :
            a = new RegExp(regexpEscape(unescapeFormat(token.replace('\\', '')), "i"));
            return a;
        }
    }

    function timezoneMinutesFromString(string) {
        var tzchunk = (parseTokenTimezone.exec(string) || [])[0],
            parts = (tzchunk + '').match(parseTimezoneChunker) || ['-', 0, 0],
            minutes = +(parts[1] * 60) + toInt(parts[2]);

        return parts[0] === '+' ? -minutes : minutes;
    }

    // function to convert string input to date
    function addTimeToArrayFromToken(token, input, config) {
        var a, datePartArray = config._a;

        switch (token) {
        // MONTH
        case 'M' : // fall through to MM
        case 'MM' :
            if (input != null) {
                datePartArray[MONTH] = toInt(input) - 1;
            }
            break;
        case 'MMM' : // fall through to MMMM
        case 'MMMM' :
            a = getLangDefinition(config._l).monthsParse(input);
            // if we didn't find a month name, mark the date as invalid.
            if (a != null) {
                datePartArray[MONTH] = a;
            } else {
                config._pf.invalidMonth = input;
            }
            break;
        // DAY OF MONTH
        case 'D' : // fall through to DD
        case 'DD' :
            if (input != null) {
                datePartArray[DATE] = toInt(input);
            }
            break;
        // DAY OF YEAR
        case 'DDD' : // fall through to DDDD
        case 'DDDD' :
            if (input != null) {
                config._dayOfYear = toInt(input);
            }

            break;
        // YEAR
        case 'YY' :
            datePartArray[YEAR] = toInt(input) + (toInt(input) > 68 ? 1900 : 2000);
            break;
        case 'YYYY' :
        case 'YYYYY' :
            datePartArray[YEAR] = toInt(input);
            break;
        // AM / PM
        case 'a' : // fall through to A
        case 'A' :
            config._isPm = getLangDefinition(config._l).isPM(input);
            break;
        // 24 HOUR
        case 'H' : // fall through to hh
        case 'HH' : // fall through to hh
        case 'h' : // fall through to hh
        case 'hh' :
            datePartArray[HOUR] = toInt(input);
            break;
        // MINUTE
        case 'm' : // fall through to mm
        case 'mm' :
            datePartArray[MINUTE] = toInt(input);
            break;
        // SECOND
        case 's' : // fall through to ss
        case 'ss' :
            datePartArray[SECOND] = toInt(input);
            break;
        // MILLISECOND
        case 'S' :
        case 'SS' :
        case 'SSS' :
        case 'SSSS' :
            datePartArray[MILLISECOND] = toInt(('0.' + input) * 1000);
            break;
        // UNIX TIMESTAMP WITH MS
        case 'X':
            config._d = new Date(parseFloat(input) * 1000);
            break;
        // TIMEZONE
        case 'Z' : // fall through to ZZ
        case 'ZZ' :
            config._useUTC = true;
            config._tzm = timezoneMinutesFromString(input);
            break;
        case 'w':
        case 'ww':
        case 'W':
        case 'WW':
        case 'd':
        case 'dd':
        case 'ddd':
        case 'dddd':
        case 'e':
        case 'E':
            token = token.substr(0, 1);
            /* falls through */
        case 'gg':
        case 'gggg':
        case 'GG':
        case 'GGGG':
        case 'GGGGG':
            token = token.substr(0, 2);
            if (input) {
                config._w = config._w || {};
                config._w[token] = input;
            }
            break;
        }
    }

    // convert an array to a date.
    // the array should mirror the parameters below
    // note: all values past the year are optional and will default to the lowest possible value.
    // [year, month, day , hour, minute, second, millisecond]
    function dateFromConfig(config) {
        var i, date, input = [], currentDate,
            yearToUse, fixYear, w, temp, lang, weekday, week;

        if (config._d) {
            return;
        }

        currentDate = currentDateArray(config);

        //compute day of the year from weeks and weekdays
        if (config._w && config._a[DATE] == null && config._a[MONTH] == null) {
            fixYear = function (val) {
                return val ?
                  (val.length < 3 ? (parseInt(val, 10) > 68 ? '19' + val : '20' + val) : val) :
                  (config._a[YEAR] == null ? moment().weekYear() : config._a[YEAR]);
            };

            w = config._w;
            if (w.GG != null || w.W != null || w.E != null) {
                temp = dayOfYearFromWeeks(fixYear(w.GG), w.W || 1, w.E, 4, 1);
            }
            else {
                lang = getLangDefinition(config._l);
                weekday = w.d != null ?  parseWeekday(w.d, lang) :
                  (w.e != null ?  parseInt(w.e, 10) + lang._week.dow : 0);

                week = parseInt(w.w, 10) || 1;

                //if we're parsing 'd', then the low day numbers may be next week
                if (w.d != null && weekday < lang._week.dow) {
                    week++;
                }

                temp = dayOfYearFromWeeks(fixYear(w.gg), week, weekday, lang._week.doy, lang._week.dow);
            }

            config._a[YEAR] = temp.year;
            config._dayOfYear = temp.dayOfYear;
        }

        //if the day of the year is set, figure out what it is
        if (config._dayOfYear) {
            yearToUse = config._a[YEAR] == null ? currentDate[YEAR] : config._a[YEAR];

            if (config._dayOfYear > daysInYear(yearToUse)) {
                config._pf._overflowDayOfYear = true;
            }

            date = makeUTCDate(yearToUse, 0, config._dayOfYear);
            config._a[MONTH] = date.getUTCMonth();
            config._a[DATE] = date.getUTCDate();
        }

        // Default to current date.
        // * if no year, month, day of month are given, default to today
        // * if day of month is given, default month and year
        // * if month is given, default only year
        // * if year is given, don't default anything
        for (i = 0; i < 3 && config._a[i] == null; ++i) {
            config._a[i] = input[i] = currentDate[i];
        }

        // Zero out whatever was not defaulted, including time
        for (; i < 7; i++) {
            config._a[i] = input[i] = (config._a[i] == null) ? (i === 2 ? 1 : 0) : config._a[i];
        }

        // add the offsets to the time to be parsed so that we can have a clean array for checking isValid
        input[HOUR] += toInt((config._tzm || 0) / 60);
        input[MINUTE] += toInt((config._tzm || 0) % 60);

        config._d = (config._useUTC ? makeUTCDate : makeDate).apply(null, input);
    }

    function dateFromObject(config) {
        var normalizedInput;

        if (config._d) {
            return;
        }

        normalizedInput = normalizeObjectUnits(config._i);
        config._a = [
            normalizedInput.year,
            normalizedInput.month,
            normalizedInput.day,
            normalizedInput.hour,
            normalizedInput.minute,
            normalizedInput.second,
            normalizedInput.millisecond
        ];

        dateFromConfig(config);
    }

    function currentDateArray(config) {
        var now = new Date();
        if (config._useUTC) {
            return [
                now.getUTCFullYear(),
                now.getUTCMonth(),
                now.getUTCDate()
            ];
        } else {
            return [now.getFullYear(), now.getMonth(), now.getDate()];
        }
    }

    // date from string and format string
    function makeDateFromStringAndFormat(config) {

        config._a = [];
        config._pf.empty = true;

        // This array is used to make a Date, either with `new Date` or `Date.UTC`
        var lang = getLangDefinition(config._l),
            string = '' + config._i,
            i, parsedInput, tokens, token, skipped,
            stringLength = string.length,
            totalParsedInputLength = 0;

        tokens = expandFormat(config._f, lang).match(formattingTokens) || [];

        for (i = 0; i < tokens.length; i++) {
            token = tokens[i];
            parsedInput = (getParseRegexForToken(token, config).exec(string) || [])[0];
            if (parsedInput) {
                skipped = string.substr(0, string.indexOf(parsedInput));
                if (skipped.length > 0) {
                    config._pf.unusedInput.push(skipped);
                }
                string = string.slice(string.indexOf(parsedInput) + parsedInput.length);
                totalParsedInputLength += parsedInput.length;
            }
            // don't parse if it's not a known token
            if (formatTokenFunctions[token]) {
                if (parsedInput) {
                    config._pf.empty = false;
                }
                else {
                    config._pf.unusedTokens.push(token);
                }
                addTimeToArrayFromToken(token, parsedInput, config);
            }
            else if (config._strict && !parsedInput) {
                config._pf.unusedTokens.push(token);
            }
        }

        // add remaining unparsed input length to the string
        config._pf.charsLeftOver = stringLength - totalParsedInputLength;
        if (string.length > 0) {
            config._pf.unusedInput.push(string);
        }

        // handle am pm
        if (config._isPm && config._a[HOUR] < 12) {
            config._a[HOUR] += 12;
        }
        // if is 12 am, change hours to 0
        if (config._isPm === false && config._a[HOUR] === 12) {
            config._a[HOUR] = 0;
        }

        dateFromConfig(config);
        checkOverflow(config);
    }

    function unescapeFormat(s) {
        return s.replace(/\\(\[)|\\(\])|\[([^\]\[]*)\]|\\(.)/g, function (matched, p1, p2, p3, p4) {
            return p1 || p2 || p3 || p4;
        });
    }

    // Code from http://stackoverflow.com/questions/3561493/is-there-a-regexp-escape-function-in-javascript
    function regexpEscape(s) {
        return s.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&');
    }

    // date from string and array of format strings
    function makeDateFromStringAndArray(config) {
        var tempConfig,
            bestMoment,

            scoreToBeat,
            i,
            currentScore;

        if (config._f.length === 0) {
            config._pf.invalidFormat = true;
            config._d = new Date(NaN);
            return;
        }

        for (i = 0; i < config._f.length; i++) {
            currentScore = 0;
            tempConfig = extend({}, config);
            initializeParsingFlags(tempConfig);
            tempConfig._f = config._f[i];
            makeDateFromStringAndFormat(tempConfig);

            if (!isValid(tempConfig)) {
                continue;
            }

            // if there is any input that was not parsed add a penalty for that format
            currentScore += tempConfig._pf.charsLeftOver;

            //or tokens
            currentScore += tempConfig._pf.unusedTokens.length * 10;

            tempConfig._pf.score = currentScore;

            if (scoreToBeat == null || currentScore < scoreToBeat) {
                scoreToBeat = currentScore;
                bestMoment = tempConfig;
            }
        }

        extend(config, bestMoment || tempConfig);
    }

    // date from iso format
    function makeDateFromString(config) {
        var i,
            string = config._i,
            match = isoRegex.exec(string);

        if (match) {
            config._pf.iso = true;
            for (i = 4; i > 0; i--) {
                if (match[i]) {
                    // match[5] should be "T" or undefined
                    config._f = isoDates[i - 1] + (match[6] || " ");
                    break;
                }
            }
            for (i = 0; i < 4; i++) {
                if (isoTimes[i][1].exec(string)) {
                    config._f += isoTimes[i][0];
                    break;
                }
            }
            if (parseTokenTimezone.exec(string)) {
                config._f += "Z";
            }
            makeDateFromStringAndFormat(config);
        }
        else {
            config._d = new Date(string);
        }
    }

    function makeDateFromInput(config) {
        var input = config._i,
            matched = aspNetJsonRegex.exec(input);

        if (input === undefined) {
            config._d = new Date();
        } else if (matched) {
            config._d = new Date(+matched[1]);
        } else if (typeof input === 'string') {
            makeDateFromString(config);
        } else if (isArray(input)) {
            config._a = input.slice(0);
            dateFromConfig(config);
        } else if (isDate(input)) {
            config._d = new Date(+input);
        } else if (typeof(input) === 'object') {
            dateFromObject(config);
        } else {
            config._d = new Date(input);
        }
    }

    function makeDate(y, m, d, h, M, s, ms) {
        //can't just apply() to create a date:
        //http://stackoverflow.com/questions/181348/instantiating-a-javascript-object-by-calling-prototype-constructor-apply
        var date = new Date(y, m, d, h, M, s, ms);

        //the date constructor doesn't accept years < 1970
        if (y < 1970) {
            date.setFullYear(y);
        }
        return date;
    }

    function makeUTCDate(y) {
        var date = new Date(Date.UTC.apply(null, arguments));
        if (y < 1970) {
            date.setUTCFullYear(y);
        }
        return date;
    }

    function parseWeekday(input, language) {
        if (typeof input === 'string') {
            if (!isNaN(input)) {
                input = parseInt(input, 10);
            }
            else {
                input = language.weekdaysParse(input);
                if (typeof input !== 'number') {
                    return null;
                }
            }
        }
        return input;
    }

    /************************************
        Relative Time
    ************************************/


    // helper function for moment.fn.from, moment.fn.fromNow, and moment.duration.fn.humanize
    function substituteTimeAgo(string, number, withoutSuffix, isFuture, lang) {
        return lang.relativeTime(number || 1, !!withoutSuffix, string, isFuture);
    }

    function relativeTime(milliseconds, withoutSuffix, lang) {
        var seconds = round(Math.abs(milliseconds) / 1000),
            minutes = round(seconds / 60),
            hours = round(minutes / 60),
            days = round(hours / 24),
            years = round(days / 365),
            args = seconds < 45 && ['s', seconds] ||
                minutes === 1 && ['m'] ||
                minutes < 45 && ['mm', minutes] ||
                hours === 1 && ['h'] ||
                hours < 22 && ['hh', hours] ||
                days === 1 && ['d'] ||
                days <= 25 && ['dd', days] ||
                days <= 45 && ['M'] ||
                days < 345 && ['MM', round(days / 30)] ||
                years === 1 && ['y'] || ['yy', years];
        args[2] = withoutSuffix;
        args[3] = milliseconds > 0;
        args[4] = lang;
        return substituteTimeAgo.apply({}, args);
    }


    /************************************
        Week of Year
    ************************************/


    // firstDayOfWeek       0 = sun, 6 = sat
    //                      the day of the week that starts the week
    //                      (usually sunday or monday)
    // firstDayOfWeekOfYear 0 = sun, 6 = sat
    //                      the first week is the week that contains the first
    //                      of this day of the week
    //                      (eg. ISO weeks use thursday (4))
    function weekOfYear(mom, firstDayOfWeek, firstDayOfWeekOfYear) {
        var end = firstDayOfWeekOfYear - firstDayOfWeek,
            daysToDayOfWeek = firstDayOfWeekOfYear - mom.day(),
            adjustedMoment;


        if (daysToDayOfWeek > end) {
            daysToDayOfWeek -= 7;
        }

        if (daysToDayOfWeek < end - 7) {
            daysToDayOfWeek += 7;
        }

        adjustedMoment = moment(mom).add('d', daysToDayOfWeek);
        return {
            week: Math.ceil(adjustedMoment.dayOfYear() / 7),
            year: adjustedMoment.year()
        };
    }

    //http://en.wikipedia.org/wiki/ISO_week_date#Calculating_a_date_given_the_year.2C_week_number_and_weekday
    function dayOfYearFromWeeks(year, week, weekday, firstDayOfWeekOfYear, firstDayOfWeek) {
        var d = new Date(Date.UTC(year, 0)).getUTCDay(),
            daysToAdd, dayOfYear;

        weekday = weekday != null ? weekday : firstDayOfWeek;
        daysToAdd = firstDayOfWeek - d + (d > firstDayOfWeekOfYear ? 7 : 0);
        dayOfYear = 7 * (week - 1) + (weekday - firstDayOfWeek) + daysToAdd + 1;

        return {
            year: dayOfYear > 0 ? year : year - 1,
            dayOfYear: dayOfYear > 0 ?  dayOfYear : daysInYear(year - 1) + dayOfYear
        };
    }

    /************************************
        Top Level Functions
    ************************************/

    function makeMoment(config) {
        var input = config._i,
            format = config._f;

        if (typeof config._pf === 'undefined') {
            initializeParsingFlags(config);
        }

        if (input === null) {
            return moment.invalid({nullInput: true});
        }

        if (typeof input === 'string') {
            config._i = input = getLangDefinition().preparse(input);
        }

        if (moment.isMoment(input)) {
            config = extend({}, input);

            config._d = new Date(+input._d);
        } else if (format) {
            if (isArray(format)) {
                makeDateFromStringAndArray(config);
            } else {
                makeDateFromStringAndFormat(config);
            }
        } else {
            makeDateFromInput(config);
        }

        return new Moment(config);
    }

    moment = function (input, format, lang, strict) {
        if (typeof(lang) === "boolean") {
            strict = lang;
            lang = undefined;
        }
        return makeMoment({
            _i : input,
            _f : format,
            _l : lang,
            _strict : strict,
            _isUTC : false
        });
    };

    // creating with utc
    moment.utc = function (input, format, lang, strict) {
        var m;

        if (typeof(lang) === "boolean") {
            strict = lang;
            lang = undefined;
        }
        m = makeMoment({
            _useUTC : true,
            _isUTC : true,
            _l : lang,
            _i : input,
            _f : format,
            _strict : strict
        }).utc();

        return m;
    };

    // creating with unix timestamp (in seconds)
    moment.unix = function (input) {
        return moment(input * 1000);
    };

    // duration
    moment.duration = function (input, key) {
        var isDuration = moment.isDuration(input),
            isNumber = (typeof input === 'number'),
            duration = (isDuration ? input._input : (isNumber ? {} : input)),
            // matching against regexp is expensive, do it on demand
            match = null,
            sign,
            ret,
            parseIso,
            timeEmpty,
            dateTimeEmpty;

        if (isNumber) {
            if (key) {
                duration[key] = input;
            } else {
                duration.milliseconds = input;
            }
        } else if (!!(match = aspNetTimeSpanJsonRegex.exec(input))) {
            sign = (match[1] === "-") ? -1 : 1;
            duration = {
                y: 0,
                d: toInt(match[DATE]) * sign,
                h: toInt(match[HOUR]) * sign,
                m: toInt(match[MINUTE]) * sign,
                s: toInt(match[SECOND]) * sign,
                ms: toInt(match[MILLISECOND]) * sign
            };
        } else if (!!(match = isoDurationRegex.exec(input))) {
            sign = (match[1] === "-") ? -1 : 1;
            parseIso = function (inp) {
                // We'd normally use ~~inp for this, but unfortunately it also
                // converts floats to ints.
                // inp may be undefined, so careful calling replace on it.
                var res = inp && parseFloat(inp.replace(',', '.'));
                // apply sign while we're at it
                return (isNaN(res) ? 0 : res) * sign;
            };
            duration = {
                y: parseIso(match[2]),
                M: parseIso(match[3]),
                d: parseIso(match[4]),
                h: parseIso(match[5]),
                m: parseIso(match[6]),
                s: parseIso(match[7]),
                w: parseIso(match[8])
            };
        }

        ret = new Duration(duration);

        if (isDuration && input.hasOwnProperty('_lang')) {
            ret._lang = input._lang;
        }

        return ret;
    };

    // version number
    moment.version = VERSION;

    // default format
    moment.defaultFormat = isoFormat;

    // This function will be called whenever a moment is mutated.
    // It is intended to keep the offset in sync with the timezone.
    moment.updateOffset = function () {};

    // This function will load languages and then set the global language.  If
    // no arguments are passed in, it will simply return the current global
    // language key.
    moment.lang = function (key, values) {
        var r;
        if (!key) {
            return moment.fn._lang._abbr;
        }
        if (values) {
            loadLang(normalizeLanguage(key), values);
        } else if (values === null) {
            unloadLang(key);
            key = 'en';
        } else if (!languages[key]) {
            getLangDefinition(key);
        }
        r = moment.duration.fn._lang = moment.fn._lang = getLangDefinition(key);
        return r._abbr;
    };

    // returns language data
    moment.langData = function (key) {
        if (key && key._lang && key._lang._abbr) {
            key = key._lang._abbr;
        }
        return getLangDefinition(key);
    };

    // compare moment object
    moment.isMoment = function (obj) {
        return obj instanceof Moment;
    };

    // for typechecking Duration objects
    moment.isDuration = function (obj) {
        return obj instanceof Duration;
    };

    for (i = lists.length - 1; i >= 0; --i) {
        makeList(lists[i]);
    }

    moment.normalizeUnits = function (units) {
        return normalizeUnits(units);
    };

    moment.invalid = function (flags) {
        var m = moment.utc(NaN);
        if (flags != null) {
            extend(m._pf, flags);
        }
        else {
            m._pf.userInvalidated = true;
        }

        return m;
    };

    moment.parseZone = function (input) {
        return moment(input).parseZone();
    };

    /************************************
        Moment Prototype
    ************************************/


    extend(moment.fn = Moment.prototype, {

        clone : function () {
            return moment(this);
        },

        valueOf : function () {
            return +this._d + ((this._offset || 0) * 60000);
        },

        unix : function () {
            return Math.floor(+this / 1000);
        },

        toString : function () {
            return this.clone().lang('en').format("ddd MMM DD YYYY HH:mm:ss [GMT]ZZ");
        },

        toDate : function () {
            return this._offset ? new Date(+this) : this._d;
        },

        toISOString : function () {
            return formatMoment(moment(this).utc(), 'YYYY-MM-DD[T]HH:mm:ss.SSS[Z]');
        },

        toArray : function () {
            var m = this;
            return [
                m.year(),
                m.month(),
                m.date(),
                m.hours(),
                m.minutes(),
                m.seconds(),
                m.milliseconds()
            ];
        },

        isValid : function () {
            return isValid(this);
        },

        isDSTShifted : function () {

            if (this._a) {
                return this.isValid() && compareArrays(this._a, (this._isUTC ? moment.utc(this._a) : moment(this._a)).toArray()) > 0;
            }

            return false;
        },

        parsingFlags : function () {
            return extend({}, this._pf);
        },

        invalidAt: function () {
            return this._pf.overflow;
        },

        utc : function () {
            return this.zone(0);
        },

        local : function () {
            this.zone(0);
            this._isUTC = false;
            return this;
        },

        format : function (inputString) {
            var output = formatMoment(this, inputString || moment.defaultFormat);
            return this.lang().postformat(output);
        },

        add : function (input, val) {
            var dur;
            // switch args to support add('s', 1) and add(1, 's')
            if (typeof input === 'string') {
                dur = moment.duration(+val, input);
            } else {
                dur = moment.duration(input, val);
            }
            addOrSubtractDurationFromMoment(this, dur, 1);
            return this;
        },

        subtract : function (input, val) {
            var dur;
            // switch args to support subtract('s', 1) and subtract(1, 's')
            if (typeof input === 'string') {
                dur = moment.duration(+val, input);
            } else {
                dur = moment.duration(input, val);
            }
            addOrSubtractDurationFromMoment(this, dur, -1);
            return this;
        },

        diff : function (input, units, asFloat) {
            var that = this._isUTC ? moment(input).zone(this._offset || 0) : moment(input).local(),
                zoneDiff = (this.zone() - that.zone()) * 6e4,
                diff, output;

            units = normalizeUnits(units);

            if (units === 'year' || units === 'month') {
                // average number of days in the months in the given dates
                diff = (this.daysInMonth() + that.daysInMonth()) * 432e5; // 24 * 60 * 60 * 1000 / 2
                // difference in months
                output = ((this.year() - that.year()) * 12) + (this.month() - that.month());
                // adjust by taking difference in days, average number of days
                // and dst in the given months.
                output += ((this - moment(this).startOf('month')) -
                        (that - moment(that).startOf('month'))) / diff;
                // same as above but with zones, to negate all dst
                output -= ((this.zone() - moment(this).startOf('month').zone()) -
                        (that.zone() - moment(that).startOf('month').zone())) * 6e4 / diff;
                if (units === 'year') {
                    output = output / 12;
                }
            } else {
                diff = (this - that);
                output = units === 'second' ? diff / 1e3 : // 1000
                    units === 'minute' ? diff / 6e4 : // 1000 * 60
                    units === 'hour' ? diff / 36e5 : // 1000 * 60 * 60
                    units === 'day' ? (diff - zoneDiff) / 864e5 : // 1000 * 60 * 60 * 24, negate dst
                    units === 'week' ? (diff - zoneDiff) / 6048e5 : // 1000 * 60 * 60 * 24 * 7, negate dst
                    diff;
            }
            return asFloat ? output : absRound(output);
        },

        from : function (time, withoutSuffix) {
            return moment.duration(this.diff(time)).lang(this.lang()._abbr).humanize(!withoutSuffix);
        },

        fromNow : function (withoutSuffix) {
            return this.from(moment(), withoutSuffix);
        },

        calendar : function () {
            var diff = this.diff(moment().zone(this.zone()).startOf('day'), 'days', true),
                format = diff < -6 ? 'sameElse' :
                diff < -1 ? 'lastWeek' :
                diff < 0 ? 'lastDay' :
                diff < 1 ? 'sameDay' :
                diff < 2 ? 'nextDay' :
                diff < 7 ? 'nextWeek' : 'sameElse';
            return this.format(this.lang().calendar(format, this));
        },

        isLeapYear : function () {
            return isLeapYear(this.year());
        },

        isDST : function () {
            return (this.zone() < this.clone().month(0).zone() ||
                this.zone() < this.clone().month(5).zone());
        },

        day : function (input) {
            var day = this._isUTC ? this._d.getUTCDay() : this._d.getDay();
            if (input != null) {
                input = parseWeekday(input, this.lang());
                return this.add({ d : input - day });
            } else {
                return day;
            }
        },

        month : function (input) {
            var utc = this._isUTC ? 'UTC' : '',
                dayOfMonth;

            if (input != null) {
                if (typeof input === 'string') {
                    input = this.lang().monthsParse(input);
                    if (typeof input !== 'number') {
                        return this;
                    }
                }

                dayOfMonth = this.date();
                this.date(1);
                this._d['set' + utc + 'Month'](input);
                this.date(Math.min(dayOfMonth, this.daysInMonth()));

                moment.updateOffset(this);
                return this;
            } else {
                return this._d['get' + utc + 'Month']();
            }
        },

        startOf: function (units) {
            units = normalizeUnits(units);
            // the following switch intentionally omits break keywords
            // to utilize falling through the cases.
            switch (units) {
            case 'year':
                this.month(0);
                /* falls through */
            case 'month':
                this.date(1);
                /* falls through */
            case 'week':
            case 'isoWeek':
            case 'day':
                this.hours(0);
                /* falls through */
            case 'hour':
                this.minutes(0);
                /* falls through */
            case 'minute':
                this.seconds(0);
                /* falls through */
            case 'second':
                this.milliseconds(0);
                /* falls through */
            }

            // weeks are a special case
            if (units === 'week') {
                this.weekday(0);
            } else if (units === 'isoWeek') {
                this.isoWeekday(1);
            }

            return this;
        },

        endOf: function (units) {
            units = normalizeUnits(units);
            return this.startOf(units).add((units === 'isoWeek' ? 'week' : units), 1).subtract('ms', 1);
        },

        isAfter: function (input, units) {
            units = typeof units !== 'undefined' ? units : 'millisecond';
            return +this.clone().startOf(units) > +moment(input).startOf(units);
        },

        isBefore: function (input, units) {
            units = typeof units !== 'undefined' ? units : 'millisecond';
            return +this.clone().startOf(units) < +moment(input).startOf(units);
        },

        isSame: function (input, units) {
            units = typeof units !== 'undefined' ? units : 'millisecond';
            return +this.clone().startOf(units) === +moment(input).startOf(units);
        },

        min: function (other) {
            other = moment.apply(null, arguments);
            return other < this ? this : other;
        },

        max: function (other) {
            other = moment.apply(null, arguments);
            return other > this ? this : other;
        },

        zone : function (input) {
            var offset = this._offset || 0;
            if (input != null) {
                if (typeof input === "string") {
                    input = timezoneMinutesFromString(input);
                }
                if (Math.abs(input) < 16) {
                    input = input * 60;
                }
                this._offset = input;
                this._isUTC = true;
                if (offset !== input) {
                    addOrSubtractDurationFromMoment(this, moment.duration(offset - input, 'm'), 1, true);
                }
            } else {
                return this._isUTC ? offset : this._d.getTimezoneOffset();
            }
            return this;
        },

        zoneAbbr : function () {
            return this._isUTC ? "UTC" : "";
        },

        zoneName : function () {
            return this._isUTC ? "Coordinated Universal Time" : "";
        },

        parseZone : function () {
            if (typeof this._i === 'string') {
                this.zone(this._i);
            }
            return this;
        },

        hasAlignedHourOffset : function (input) {
            if (!input) {
                input = 0;
            }
            else {
                input = moment(input).zone();
            }

            return (this.zone() - input) % 60 === 0;
        },

        daysInMonth : function () {
            return daysInMonth(this.year(), this.month());
        },

        dayOfYear : function (input) {
            var dayOfYear = round((moment(this).startOf('day') - moment(this).startOf('year')) / 864e5) + 1;
            return input == null ? dayOfYear : this.add("d", (input - dayOfYear));
        },

        weekYear : function (input) {
            var year = weekOfYear(this, this.lang()._week.dow, this.lang()._week.doy).year;
            return input == null ? year : this.add("y", (input - year));
        },

        isoWeekYear : function (input) {
            var year = weekOfYear(this, 1, 4).year;
            return input == null ? year : this.add("y", (input - year));
        },

        week : function (input) {
            var week = this.lang().week(this);
            return input == null ? week : this.add("d", (input - week) * 7);
        },

        isoWeek : function (input) {
            var week = weekOfYear(this, 1, 4).week;
            return input == null ? week : this.add("d", (input - week) * 7);
        },

        weekday : function (input) {
            var weekday = (this.day() + 7 - this.lang()._week.dow) % 7;
            return input == null ? weekday : this.add("d", input - weekday);
        },

        isoWeekday : function (input) {
            // behaves the same as moment#day except
            // as a getter, returns 7 instead of 0 (1-7 range instead of 0-6)
            // as a setter, sunday should belong to the previous week.
            return input == null ? this.day() || 7 : this.day(this.day() % 7 ? input : input - 7);
        },

        get : function (units) {
            units = normalizeUnits(units);
            return this[units]();
        },

        set : function (units, value) {
            units = normalizeUnits(units);
            if (typeof this[units] === 'function') {
                this[units](value);
            }
            return this;
        },

        // If passed a language key, it will set the language for this
        // instance.  Otherwise, it will return the language configuration
        // variables for this instance.
        lang : function (key) {
            if (key === undefined) {
                return this._lang;
            } else {
                this._lang = getLangDefinition(key);
                return this;
            }
        }
    });

    // helper for adding shortcuts
    function makeGetterAndSetter(name, key) {
        moment.fn[name] = moment.fn[name + 's'] = function (input) {
            var utc = this._isUTC ? 'UTC' : '';
            if (input != null) {
                this._d['set' + utc + key](input);
                moment.updateOffset(this);
                return this;
            } else {
                return this._d['get' + utc + key]();
            }
        };
    }

    // loop through and add shortcuts (Month, Date, Hours, Minutes, Seconds, Milliseconds)
    for (i = 0; i < proxyGettersAndSetters.length; i ++) {
        makeGetterAndSetter(proxyGettersAndSetters[i].toLowerCase().replace(/s$/, ''), proxyGettersAndSetters[i]);
    }

    // add shortcut for year (uses different syntax than the getter/setter 'year' == 'FullYear')
    makeGetterAndSetter('year', 'FullYear');

    // add plural methods
    moment.fn.days = moment.fn.day;
    moment.fn.months = moment.fn.month;
    moment.fn.weeks = moment.fn.week;
    moment.fn.isoWeeks = moment.fn.isoWeek;

    // add aliased format methods
    moment.fn.toJSON = moment.fn.toISOString;

    /************************************
        Duration Prototype
    ************************************/


    extend(moment.duration.fn = Duration.prototype, {

        _bubble : function () {
            var milliseconds = this._milliseconds,
                days = this._days,
                months = this._months,
                data = this._data,
                seconds, minutes, hours, years;

            // The following code bubbles up values, see the tests for
            // examples of what that means.
            data.milliseconds = milliseconds % 1000;

            seconds = absRound(milliseconds / 1000);
            data.seconds = seconds % 60;

            minutes = absRound(seconds / 60);
            data.minutes = minutes % 60;

            hours = absRound(minutes / 60);
            data.hours = hours % 24;

            days += absRound(hours / 24);
            data.days = days % 30;

            months += absRound(days / 30);
            data.months = months % 12;

            years = absRound(months / 12);
            data.years = years;
        },

        weeks : function () {
            return absRound(this.days() / 7);
        },

        valueOf : function () {
            return this._milliseconds +
              this._days * 864e5 +
              (this._months % 12) * 2592e6 +
              toInt(this._months / 12) * 31536e6;
        },

        humanize : function (withSuffix) {
            var difference = +this,
                output = relativeTime(difference, !withSuffix, this.lang());

            if (withSuffix) {
                output = this.lang().pastFuture(difference, output);
            }

            return this.lang().postformat(output);
        },

        add : function (input, val) {
            // supports only 2.0-style add(1, 's') or add(moment)
            var dur = moment.duration(input, val);

            this._milliseconds += dur._milliseconds;
            this._days += dur._days;
            this._months += dur._months;

            this._bubble();

            return this;
        },

        subtract : function (input, val) {
            var dur = moment.duration(input, val);

            this._milliseconds -= dur._milliseconds;
            this._days -= dur._days;
            this._months -= dur._months;

            this._bubble();

            return this;
        },

        get : function (units) {
            units = normalizeUnits(units);
            return this[units.toLowerCase() + 's']();
        },

        as : function (units) {
            units = normalizeUnits(units);
            return this['as' + units.charAt(0).toUpperCase() + units.slice(1) + 's']();
        },

        lang : moment.fn.lang,

        toIsoString : function () {
            // inspired by https://github.com/dordille/moment-isoduration/blob/master/moment.isoduration.js
            var years = Math.abs(this.years()),
                months = Math.abs(this.months()),
                days = Math.abs(this.days()),
                hours = Math.abs(this.hours()),
                minutes = Math.abs(this.minutes()),
                seconds = Math.abs(this.seconds() + this.milliseconds() / 1000);

            if (!this.asSeconds()) {
                // this is the same as C#'s (Noda) and python (isodate)...
                // but not other JS (goog.date)
                return 'P0D';
            }

            return (this.asSeconds() < 0 ? '-' : '') +
                'P' +
                (years ? years + 'Y' : '') +
                (months ? months + 'M' : '') +
                (days ? days + 'D' : '') +
                ((hours || minutes || seconds) ? 'T' : '') +
                (hours ? hours + 'H' : '') +
                (minutes ? minutes + 'M' : '') +
                (seconds ? seconds + 'S' : '');
        }
    });

    function makeDurationGetter(name) {
        moment.duration.fn[name] = function () {
            return this._data[name];
        };
    }

    function makeDurationAsGetter(name, factor) {
        moment.duration.fn['as' + name] = function () {
            return +this / factor;
        };
    }

    for (i in unitMillisecondFactors) {
        if (unitMillisecondFactors.hasOwnProperty(i)) {
            makeDurationAsGetter(i, unitMillisecondFactors[i]);
            makeDurationGetter(i.toLowerCase());
        }
    }

    makeDurationAsGetter('Weeks', 6048e5);
    moment.duration.fn.asMonths = function () {
        return (+this - this.years() * 31536e6) / 2592e6 + this.years() * 12;
    };


    /************************************
        Default Lang
    ************************************/


    // Set default language, other languages will inherit from English.
    moment.lang('en', {
        ordinal : function (number) {
            var b = number % 10,
                output = (toInt(number % 100 / 10) === 1) ? 'th' :
                (b === 1) ? 'st' :
                (b === 2) ? 'nd' :
                (b === 3) ? 'rd' : 'th';
            return number + output;
        }
    });

    /* EMBED_LANGUAGES */

    /************************************
        Exposing Moment
    ************************************/

    function makeGlobal(deprecate) {
        var warned = false, local_moment = moment;
        /*global ender:false */
        if (typeof ender !== 'undefined') {
            return;
        }
        // here, `this` means `window` in the browser, or `global` on the server
        // add `moment` as a global object via a string identifier,
        // for Closure Compiler "advanced" mode
        if (deprecate) {
            this.moment = function () {
                if (!warned && console && console.warn) {
                    warned = true;
                    console.warn(
                            "Accessing Moment through the global scope is " +
                            "deprecated, and will be removed in an upcoming " +
                            "release.");
                }
                return local_moment.apply(null, arguments);
            };
        } else {
            this['moment'] = moment;
        }
    }

    // CommonJS module is defined
    if (hasModule) {
        module.exports = moment;
        makeGlobal(true);
    } else if (typeof define === "function" && define.amd) {
        define("moment", ['require','exports','module'],function (require, exports, module) {
            if (module.config().noGlobal !== true) {
                // If user provided noGlobal, he is aware of global
                makeGlobal(module.config().noGlobal === undefined);
            }

            return moment;
        });
    } else {
        makeGlobal();
    }
}).call(this);

/*global define*/
define('models/settings',[
	'underscore',
	'backbone',
	'StoreEngine',
	'moment',
], function (_, Backbone,StoreEngine,moment) {
	

	var isPhoneGap = (document.location.protocol == "file:"),
		SettingsModel = Backbone.Model.extend({
		appVersion:'2.0.0',
		appDate:'2014-06-30',
        serverVersion:'2.0.0',
        serverDate:'2014-06-30',
        
		defaults: {
			id:'settings',
			server:isPhoneGap?'https://hp-soccer.com':window.location.protocol+"//"+window.location.host,
			weinre: false,
			extendedError: false
		},
		sync : StoreEngine.createSync ('wmapp-settings'),
		url:"localstorage://settings"
		
	});

	var settings = new SettingsModel();
	settings.fetch({ cache: false }).done(function(){
		if (!isPhoneGap){
			settings.set("server",window.location.protocol+"//"+window.location.host);
			settings.save();
		}
	});

	return settings;
});

/*global define*/
define('models/player',[
	'underscore',
	'backbone',
	'models/settings'
], function (_, Backbone,Settings) {
	

	var PlayerModel = Backbone.Model.extend({
		  idAttribute: "playerId",
		 getIterationToken:function(){
			Settings.playerIterationToken=Settings.playerIterationToken||{};
			var id=this.get("id");
			if (id && !Settings.playerIterationToken[id]){
				Settings.playerIterationToken[""+id]=new Date().getTime();
			}
			return Settings.playerIterationToken[""+id];
		},
		invalidate:function(){
			Settings.playerIterationToken=Settings.playerIterationToken||{};
			Settings.playerIterationToken[""+this.get("id")]=new Date().getTime();
		},
		reload:function(options){
			this.invalidate();
		  	return this.fetch.apply(this,arguments);
		  },
		  url: function() {  
		  		return  Settings.get("server") + '/rest/player/'+this.get("playerId")+"?_it="+this.getIterationToken()+(this.get('includeTips')?"&includetips=true":""); 
		  },

		  isAdmin:function(){
		  		return this.get('role')==="ADMIN";
		  },
		  logout:function(){
		  		return $.ajax(
				{
					type:"GET",
					url:Settings.get("server") + '/rest/player/'+this.get("playerId")+"/logout",
					contentType: 'application/json'
				}
			);
		  },
		  update:function(){
		  	return $.ajax(
				{
					type:"PUT",
					url:this.url(),
					contentType: 'application/json',
					data:{
						realName:this.get('realName')
					}
				}
			);
		  },
		  isOwner:function(groupId){
			return _.findWhere(this.get('groups'), {groupId: groupId,ownerId:this.get('id')})!== undefined;
		  },
		  isMemberOf:function(groupId){
		  	return _.findWhere(this.get('groups'), {groupId: groupId})!== undefined;
		  },
		  /*get list of join requests for a givven group*/
		  getGroupJoinRequests:function(groupId){
		  	return _.where(this.get('groupJoinRequests'),{status: "join-requested",groupId: parseInt(groupId,10)});
		  },
		  /*Check if player was invited  to join the group*/
		  getGroupInvited:function(groupId){
		  	return _.findWhere(this.get('groupInvitations'),{status: "invited",groupId: parseInt(groupId,10)});
		  },
		  /*Check if player requested to join the group*/
		  getGroupJoinRequested:function(groupId){
		  	return _.findWhere(this.get('groupInvitations'),{status: "join-requested",groupId:parseInt(groupId,10)});
		  }
		  
	});

	return PlayerModel;
});

/*global define*/
define('models/result',[
	'underscore',
	'backbone',
		'models/settings'
], function (_, Backbone,Settings) {
	

	var ResultModel = Backbone.Model.extend({
		url:function (){ 
			Settings.matchIterationToken=Settings.matchIterationToken||new Date().getTime();
			return  Settings.get("server") + "/rest/match/"+this.get("id")+"?_it="+Settings.matchIterationToken
		},
		reload:function(options){
		  	Settings.matchIterationToken=new Date().getTime();
		  	return this.fetch.apply(this,arguments);
		},
		getStats:function(){
			return $.ajax(
				{
					type:"GET",
					url:Settings.get("server") + "/rest/match/"+this.get("id")+"/trend",
					headers:{
						'Cache-Control': 'no-cache'
					}
				}
			);
		},
		getResultStats:function(){
			return $.ajax(
				{
					type:"GET",
					url:Settings.get("server") + "/rest/match/"+this.get("id")+"/stats/all",
					headers:{
						'Cache-Control': 'no-cache'
					}
				}
			);
		}
	});

	return ResultModel;
});

/*global define*/
define('collections/results',[
	'underscore',
	'backbone',
	'models/result',
	'models/settings'
], function (_, Backbone, Result, Settings) {
	

	var ResultsCollection = Backbone.Collection.extend({
		// Reference to this collection's model.
		model: Result,
		url:function (){ 
			Settings.matchListIterationToken=Settings.matchListIterationToken||new Date().getTime();
			var param=$.extend({},this.query,{
				"_it":Settings.matchListIterationToken
			})
			return  Settings.get("server") + "/rest/match?"+$.param(param);
		},
		reload:function(options){
		  	Settings.matchListIterationToken=new Date().getTime();
		  	return this.fetch.apply(this,arguments);
		}
	});

	return new ResultsCollection();
});

/*global define*/
define('models/tip',[
	'underscore',
	'backbone',
	'util/context',
], function (_, Backbone,Context) {
	

	var MatchTipModel = Backbone.Model.extend({
		url: function() {  
			Context.tipIterationToken=Context.tipIterationToken||new Date().getTime();
			var param=$.extend({},this.query,{
				"_it":Context.tipIterationToken
			})
		  	return  Settings.get("server") + '/rest/matchtips/'+this.get("id")+"?_it="+Context.tipIterationToken 
		},
		reload:function(options){
		  	Context.tipIterationToken=new Date().getTime();
		  	return this.fetch.apply(this,arguments);
		}
	});

	return MatchTipModel;
});

/*global define*/
define('collections/tips',[
	'underscore',
	'backbone',
	'models/tip',
	'models/settings'
], function (_, Backbone, MatchTip, Settings) {
	

	var TipsCollection = Backbone.Collection.extend({
		// Reference to this collection's model.
		model: MatchTip,
		url:function (){
			Settings.matchtipListIterationToken=Settings.matchtipListIterationToken||new Date().getTime();
			var param=$.extend({},this.query,{
				"_it":Settings.matchtipListIterationToken
			})
			return  Settings.get("server") + "/rest/matchtips?"+$.param(param);
		},
		reload:function(){
		  	Settings.matchtipListIterationToken=new Date().getTime();
		  	return this.fetch.apply(this,arguments);
		  },
	});

	return new TipsCollection();
});

/*!
 *  Copyright 2011 Twitter, Inc.
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */



var Hogan = {};

(function (Hogan, useArrayBuffer) {
  Hogan.Template = function (codeObj, text, compiler, options) {
    codeObj = codeObj || {};
    this.r = codeObj.code || this.r;
    this.c = compiler;
    this.options = options || {};
    this.text = text || '';
    this.partials = codeObj.partials || {};
    this.subs = codeObj.subs || {};
    this.ib();
  }

  Hogan.Template.prototype = {
    // render: replaced by generated code.
    r: function (context, partials, indent) { return ''; },

    // variable escaping
    v: hoganEscape,

    // triple stache
    t: coerceToString,

    render: function render(context, partials, indent) {
      return this.ri([context], partials || {}, indent);
    },

    // render internal -- a hook for overrides that catches partials too
    ri: function (context, partials, indent) {
      return this.r(context, partials, indent);
    },

    // ensurePartial
    ep: function(symbol, partials) {
      var partial = this.partials[symbol];

      // check to see that if we've instantiated this partial before
      var template = partials[partial.name];
      if (partial.instance && partial.base == template) {
        return partial.instance;
      }

      if (typeof template == 'string') {
        if (!this.c) {
          throw new Error("No compiler available.");
        }
        template = this.c.compile(template, this.options);
      }

      if (!template) {
        return null;
      }

      // We use this to check whether the partials dictionary has changed
      this.partials[symbol].base = template;

      if (partial.subs) {
        // Make sure we consider parent template now
        if (this.activeSub === undefined) {
          // Store parent template text in partials.stackText to perform substitutions in child templates correctly
          partials.stackText  = this.text;
        }
         template = createSpecializedPartial(template, partial.subs, partial.partials, partials.stackText || this.text);
       }
      this.partials[symbol].instance = template;
      return template;
    },

    // tries to find a partial in the current scope and render it
    rp: function(symbol, context, partials, indent) {
      var partial = this.ep(symbol, partials);
      if (!partial) {
        return '';
      }

      return partial.ri(context, partials, indent);
    },

    // render a section
    rs: function(context, partials, section) {
      var tail = context[context.length - 1];

      if (!isArray(tail)) {
        section(context, partials, this);
        return;
      }

      for (var i = 0; i < tail.length; i++) {
        context.push(tail[i]);
        section(context, partials, this);
        context.pop();
      }
    },

    // maybe start a section
    s: function(val, ctx, partials, inverted, start, end, tags) {
      var pass;

      if (isArray(val) && val.length === 0) {
        return false;
      }

      if (typeof val == 'function') {
        val = this.ms(val, ctx, partials, inverted, start, end, tags);
      }

      pass = !!val;

      if (!inverted && pass && ctx) {
        ctx.push((typeof val == 'object') ? val : ctx[ctx.length - 1]);
      }

      return pass;
    },

    // find values with dotted names
    d: function(key, ctx, partials, returnFound) {
      var found,
          names = key.split('.'),
          val = this.f(names[0], ctx, partials, returnFound),
          doModelGet = this.options.modelGet,
          cx = null;

      if (key === '.' && isArray(ctx[ctx.length - 2])) {
        val = ctx[ctx.length - 1];
      } else {
        for (var i = 1; i < names.length; i++) {
          found = findInScope(names[i], val, doModelGet);
          if (found != null) {
            cx = val;
            val = found;
          } else {
            val = '';
          }
        }
      }

      if (returnFound && !val) {
        return false;
      }

      if (!returnFound && typeof val == 'function') {
        ctx.push(cx);
        val = this.mv(val, ctx, partials);
        ctx.pop();
      }

      return val;
    },

    // find values with normal names
    f: function(key, ctx, partials, returnFound) {
      var val = false,
          v = null,
          found = false,
          doModelGet = this.options.modelGet;

      for (var i = ctx.length - 1; i >= 0; i--) {
        v = ctx[i];
        val = findInScope(key, v, doModelGet);
        if (val != null) {
          found = true;
          break;
        }
      }

      if (!found) {
        return (returnFound) ? false : "";
      }

      if (!returnFound && typeof val == 'function') {
        val = this.mv(val, ctx, partials);
      }

      return val;
    },

    // higher order templates
    ls: function(func, cx, partials, text, tags) {
      var oldTags = this.options.delimiters;

      this.options.delimiters = tags;
      this.b(this.ct(coerceToString(func.call(cx, text)), cx, partials));
      this.options.delimiters = oldTags;

      return false;
    },

    // compile text
    ct: function(text, cx, partials) {
      if (this.options.disableLambda) {
        throw new Error('Lambda features disabled.');
      }
      return this.c.compile(text, this.options).render(cx, partials);
    },

    // template result buffering
    b: (useArrayBuffer) ? function(s) { this.buf.push(s); } :
                          function(s) { this.buf += s; },

    fl: (useArrayBuffer) ? function() { var r = this.buf.join(''); this.buf = []; return r; } :
                           function() { var r = this.buf; this.buf = ''; return r; },
    // init the buffer
    ib: function () {
      this.buf = (useArrayBuffer) ? [] : '';
    },

    // method replace section
    ms: function(func, ctx, partials, inverted, start, end, tags) {
      var textSource,
          cx = ctx[ctx.length - 1],
          result = func.call(cx);

      if (typeof result == 'function') {
        if (inverted) {
          return true;
        } else {
          textSource = (this.activeSub && this.subsText[this.activeSub]) ? this.subsText[this.activeSub] : this.text;
          return this.ls(result, cx, partials, textSource.substring(start, end), tags);
        }
      }

      return result;
    },

    // method replace variable
    mv: function(func, ctx, partials) {
      var cx = ctx[ctx.length - 1];
      var result = func.call(cx);

      if (typeof result == 'function') {
        return this.ct(coerceToString(result.call(cx)), cx, partials);
      }

      return result;
    },

    sub: function(name, context, partials, indent) {
      var f = this.subs[name];
      if (f) {
        this.activeSub = name;
        f(context, partials, this, indent);
        this.activeSub = false;
      }
    }

  };

  //Find a key in an object
  function findInScope(key, scope, doModelGet) {
    var val, checkVal;

    if (scope && typeof scope == 'object') {

      if (scope[key] != null) {
        val = scope[key];

      // try lookup with get for backbone or similar model data
      } else if (doModelGet && scope.get && typeof scope.get == 'function') {
        val = scope.get(key);
      }
    }

    return val;
  }

  function createSpecializedPartial(instance, subs, partials, childText) {
    function PartialTemplate() {};
    PartialTemplate.prototype = instance;
    function Substitutions() {};
    Substitutions.prototype = instance.subs;
    var key;
    var partial = new PartialTemplate();
    partial.subs = new Substitutions();
    partial.subsText = {};  //hehe. substext.
    partial.ib();

    for (key in subs) {
      partial.subs[key] = subs[key];
      partial.subsText[key] = childText;
    }

    for (key in partials) {
      partial.partials[key] = partials[key];
    }

    return partial;
  }

  var rAmp = /&/g,
      rLt = /</g,
      rGt = />/g,
      rApos = /\'/g,
      rQuot = /\"/g,
      hChars = /[&<>\"\']/;

  function coerceToString(val) {
    return String((val === null || val === undefined) ? '' : val);
  }

  function hoganEscape(str) {
    str = coerceToString(str);
    return hChars.test(str) ?
      str
        .replace(rAmp, '&amp;')
        .replace(rLt, '&lt;')
        .replace(rGt, '&gt;')
        .replace(rApos, '&#39;')
        .replace(rQuot, '&quot;') :
      str;
  }

  var isArray = Array.isArray || function(a) {
    return Object.prototype.toString.call(a) === '[object Array]';
  };

})(typeof exports !== 'undefined' ? exports : Hogan);




if (typeof define === 'function' && define.amd) {
  define('hogan',Hogan);
}
;
/**
 * @license RequireJS text 2.0.10 Copyright (c) 2010-2012, The Dojo Foundation All Rights Reserved.
 * Available via the MIT or new BSD license.
 * see: http://github.com/requirejs/text for details
 */
/*jslint regexp: true */
/*global require, XMLHttpRequest, ActiveXObject,
  define, window, process, Packages,
  java, location, Components, FileUtils */

define('text',['module'], function (module) {
    

    var text, fs, Cc, Ci, xpcIsWindows,
        progIds = ['Msxml2.XMLHTTP', 'Microsoft.XMLHTTP', 'Msxml2.XMLHTTP.4.0'],
        xmlRegExp = /^\s*<\?xml(\s)+version=[\'\"](\d)*.(\d)*[\'\"](\s)*\?>/im,
        bodyRegExp = /<body[^>]*>\s*([\s\S]+)\s*<\/body>/im,
        hasLocation = typeof location !== 'undefined' && location.href,
        defaultProtocol = hasLocation && location.protocol && location.protocol.replace(/\:/, ''),
        defaultHostName = hasLocation && location.hostname,
        defaultPort = hasLocation && (location.port || undefined),
        buildMap = {},
        masterConfig = (module.config && module.config()) || {};

    text = {
        version: '2.0.10',

        strip: function (content) {
            //Strips <?xml ...?> declarations so that external SVG and XML
            //documents can be added to a document without worry. Also, if the string
            //is an HTML document, only the part inside the body tag is returned.
            if (content) {
                content = content.replace(xmlRegExp, "");
                var matches = content.match(bodyRegExp);
                if (matches) {
                    content = matches[1];
                }
            } else {
                content = "";
            }
            return content;
        },

        jsEscape: function (content) {
            return content.replace(/(['\\])/g, '\\$1')
                .replace(/[\f]/g, "\\f")
                .replace(/[\b]/g, "\\b")
                .replace(/[\n]/g, "\\n")
                .replace(/[\t]/g, "\\t")
                .replace(/[\r]/g, "\\r")
                .replace(/[\u2028]/g, "\\u2028")
                .replace(/[\u2029]/g, "\\u2029");
        },

        createXhr: masterConfig.createXhr || function () {
            //Would love to dump the ActiveX crap in here. Need IE 6 to die first.
            var xhr, i, progId;
            if (typeof XMLHttpRequest !== "undefined") {
                return new XMLHttpRequest();
            } else if (typeof ActiveXObject !== "undefined") {
                for (i = 0; i < 3; i += 1) {
                    progId = progIds[i];
                    try {
                        xhr = new ActiveXObject(progId);
                    } catch (e) {}

                    if (xhr) {
                        progIds = [progId];  // so faster next time
                        break;
                    }
                }
            }

            return xhr;
        },

        /**
         * Parses a resource name into its component parts. Resource names
         * look like: module/name.ext!strip, where the !strip part is
         * optional.
         * @param {String} name the resource name
         * @returns {Object} with properties "moduleName", "ext" and "strip"
         * where strip is a boolean.
         */
        parseName: function (name) {
            var modName, ext, temp,
                strip = false,
                index = name.indexOf("."),
                isRelative = name.indexOf('./') === 0 ||
                             name.indexOf('../') === 0;

            if (index !== -1 && (!isRelative || index > 1)) {
                modName = name.substring(0, index);
                ext = name.substring(index + 1, name.length);
            } else {
                modName = name;
            }

            temp = ext || modName;
            index = temp.indexOf("!");
            if (index !== -1) {
                //Pull off the strip arg.
                strip = temp.substring(index + 1) === "strip";
                temp = temp.substring(0, index);
                if (ext) {
                    ext = temp;
                } else {
                    modName = temp;
                }
            }

            return {
                moduleName: modName,
                ext: ext,
                strip: strip
            };
        },

        xdRegExp: /^((\w+)\:)?\/\/([^\/\\]+)/,

        /**
         * Is an URL on another domain. Only works for browser use, returns
         * false in non-browser environments. Only used to know if an
         * optimized .js version of a text resource should be loaded
         * instead.
         * @param {String} url
         * @returns Boolean
         */
        useXhr: function (url, protocol, hostname, port) {
            var uProtocol, uHostName, uPort,
                match = text.xdRegExp.exec(url);
            if (!match) {
                return true;
            }
            uProtocol = match[2];
            uHostName = match[3];

            uHostName = uHostName.split(':');
            uPort = uHostName[1];
            uHostName = uHostName[0];

            return (!uProtocol || uProtocol === protocol) &&
                   (!uHostName || uHostName.toLowerCase() === hostname.toLowerCase()) &&
                   ((!uPort && !uHostName) || uPort === port);
        },

        finishLoad: function (name, strip, content, onLoad) {
            content = strip ? text.strip(content) : content;
            if (masterConfig.isBuild) {
                buildMap[name] = content;
            }
            onLoad(content);
        },

        load: function (name, req, onLoad, config) {
            //Name has format: some.module.filext!strip
            //The strip part is optional.
            //if strip is present, then that means only get the string contents
            //inside a body tag in an HTML string. For XML/SVG content it means
            //removing the <?xml ...?> declarations so the content can be inserted
            //into the current doc without problems.

            // Do not bother with the work if a build and text will
            // not be inlined.
            if (config.isBuild && !config.inlineText) {
                onLoad();
                return;
            }

            masterConfig.isBuild = config.isBuild;

            var parsed = text.parseName(name),
                nonStripName = parsed.moduleName +
                    (parsed.ext ? '.' + parsed.ext : ''),
                url = req.toUrl(nonStripName),
                useXhr = (masterConfig.useXhr) ||
                         text.useXhr;

            // Do not load if it is an empty: url
            if (url.indexOf('empty:') === 0) {
                onLoad();
                return;
            }

            //Load the text. Use XHR if possible and in a browser.
            if (!hasLocation || useXhr(url, defaultProtocol, defaultHostName, defaultPort)) {
                text.get(url, function (content) {
                    text.finishLoad(name, parsed.strip, content, onLoad);
                }, function (err) {
                    if (onLoad.error) {
                        onLoad.error(err);
                    }
                });
            } else {
                //Need to fetch the resource across domains. Assume
                //the resource has been optimized into a JS module. Fetch
                //by the module name + extension, but do not include the
                //!strip part to avoid file system issues.
                req([nonStripName], function (content) {
                    text.finishLoad(parsed.moduleName + '.' + parsed.ext,
                                    parsed.strip, content, onLoad);
                });
            }
        },

        write: function (pluginName, moduleName, write, config) {
            if (buildMap.hasOwnProperty(moduleName)) {
                var content = text.jsEscape(buildMap[moduleName]);
                write.asModule(pluginName + "!" + moduleName,
                               "define(function () { return '" +
                                   content +
                               "';});\n");
            }
        },

        writeFile: function (pluginName, moduleName, req, write, config) {
            var parsed = text.parseName(moduleName),
                extPart = parsed.ext ? '.' + parsed.ext : '',
                nonStripName = parsed.moduleName + extPart,
                //Use a '.js' file name so that it indicates it is a
                //script that can be loaded across domains.
                fileName = req.toUrl(parsed.moduleName + extPart) + '.js';

            //Leverage own load() method to load plugin value, but only
            //write out values that do not have the strip argument,
            //to avoid any potential issues with ! in file names.
            text.load(nonStripName, req, function (value) {
                //Use own write() method to construct full module value.
                //But need to create shell that translates writeFile's
                //write() to the right interface.
                var textWrite = function (contents) {
                    return write(fileName, contents);
                };
                textWrite.asModule = function (moduleName, contents) {
                    return write.asModule(moduleName, fileName, contents);
                };

                text.write(pluginName, nonStripName, textWrite, config);
            }, config);
        }
    };

    if (masterConfig.env === 'node' || (!masterConfig.env &&
            typeof process !== "undefined" &&
            process.versions &&
            !!process.versions.node &&
            !process.versions['node-webkit'])) {
        //Using special require.nodeRequire, something added by r.js.
        fs = require.nodeRequire('fs');

        text.get = function (url, callback, errback) {
            try {
                var file = fs.readFileSync(url, 'utf8');
                //Remove BOM (Byte Mark Order) from utf8 files if it is there.
                if (file.indexOf('\uFEFF') === 0) {
                    file = file.substring(1);
                }
                callback(file);
            } catch (e) {
                errback(e);
            }
        };
    } else if (masterConfig.env === 'xhr' || (!masterConfig.env &&
            text.createXhr())) {
        text.get = function (url, callback, errback, headers) {
            var xhr = text.createXhr(), header;
            xhr.open('GET', url, true);

            //Allow plugins direct access to xhr headers
            if (headers) {
                for (header in headers) {
                    if (headers.hasOwnProperty(header)) {
                        xhr.setRequestHeader(header.toLowerCase(), headers[header]);
                    }
                }
            }

            //Allow overrides specified in config
            if (masterConfig.onXhr) {
                masterConfig.onXhr(xhr, url);
            }

            xhr.onreadystatechange = function (evt) {
                var status, err;
                //Do not explicitly handle errors, those should be
                //visible via console output in the browser.
                if (xhr.readyState === 4) {
                    status = xhr.status;
                    if (status > 399 && status < 600) {
                        //An http 4xx or 5xx error. Signal an error.
                        err = new Error(url + ' HTTP status: ' + status);
                        err.xhr = xhr;
                        errback(err);
                    } else {
                        callback(xhr.responseText);
                    }

                    if (masterConfig.onXhrComplete) {
                        masterConfig.onXhrComplete(xhr, url);
                    }
                }
            };
            xhr.send(null);
        };
    } else if (masterConfig.env === 'rhino' || (!masterConfig.env &&
            typeof Packages !== 'undefined' && typeof java !== 'undefined')) {
        //Why Java, why is this so awkward?
        text.get = function (url, callback) {
            var stringBuffer, line,
                encoding = "utf-8",
                file = new java.io.File(url),
                lineSeparator = java.lang.System.getProperty("line.separator"),
                input = new java.io.BufferedReader(new java.io.InputStreamReader(new java.io.FileInputStream(file), encoding)),
                content = '';
            try {
                stringBuffer = new java.lang.StringBuffer();
                line = input.readLine();

                // Byte Order Mark (BOM) - The Unicode Standard, version 3.0, page 324
                // http://www.unicode.org/faq/utf_bom.html

                // Note that when we use utf-8, the BOM should appear as "EF BB BF", but it doesn't due to this bug in the JDK:
                // http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4508058
                if (line && line.length() && line.charAt(0) === 0xfeff) {
                    // Eat the BOM, since we've already found the encoding on this file,
                    // and we plan to concatenating this buffer with others; the BOM should
                    // only appear at the top of a file.
                    line = line.substring(1);
                }

                if (line !== null) {
                    stringBuffer.append(line);
                }

                while ((line = input.readLine()) !== null) {
                    stringBuffer.append(lineSeparator);
                    stringBuffer.append(line);
                }
                //Make sure we return a JavaScript string and not a Java string.
                content = String(stringBuffer.toString()); //String
            } finally {
                input.close();
            }
            callback(content);
        };
    } else if (masterConfig.env === 'xpconnect' || (!masterConfig.env &&
            typeof Components !== 'undefined' && Components.classes &&
            Components.interfaces)) {
        //Avert your gaze!
        Cc = Components.classes,
        Ci = Components.interfaces;
        Components.utils['import']('resource://gre/modules/FileUtils.jsm');
        xpcIsWindows = ('@mozilla.org/windows-registry-key;1' in Cc);

        text.get = function (url, callback) {
            var inStream, convertStream, fileObj,
                readData = {};

            if (xpcIsWindows) {
                url = url.replace(/\//g, '\\');
            }

            fileObj = new FileUtils.File(url);

            //XPCOM, you so crazy
            try {
                inStream = Cc['@mozilla.org/network/file-input-stream;1']
                           .createInstance(Ci.nsIFileInputStream);
                inStream.init(fileObj, 1, 0, false);

                convertStream = Cc['@mozilla.org/intl/converter-input-stream;1']
                                .createInstance(Ci.nsIConverterInputStream);
                convertStream.init(inStream, "utf-8", inStream.available(),
                Ci.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER);

                convertStream.readString(inStream.available(), readData);
                convertStream.close();
                inStream.close();
                callback(readData.value);
            } catch (e) {
                throw new Error((fileObj && fileObj.path || '') + ': ' + e);
            }
        };
    }
    return text;
});

define('hgn',{load: function(id){throw new Error("Dynamic load not allowed: " + id);}});

define("hgn!templates/popupmessage", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<h1>");t.b(t.v(t.f("header",c,p,0)));t.b("</h1>\r");t.b("\n" + i);t.b("<p>");t.b(t.v(t.f("message",c,p,0)));t.b("</p>\r");t.b("\n" + i);t.b("<div class=\"form-group\">\r");t.b("\n" + i);t.b("	<input type=\"text\" class=\"form-control\" id=\"popup-message\" placeholder=\"");t.b(t.v(t.f("placeholder",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("<div class=\"btn-group btn-group-justified\">\r");t.b("\n" + i);t.b("	<div class=\"btn-group\">\r");t.b("\n" + i);t.b("		<button type=\"button\" id=\"popup-cancel\" class=\"btn btn-default\">");t.b(t.v(t.d("Strings.COMMON.CANCEL",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("	</div>\r");t.b("\n" + i);t.b("	<div class=\"btn-group\">\r");t.b("\n" + i);t.b("		<button type=\"button\" id=\"popup-ok\" class=\"btn btn-default\">");t.b(t.v(t.d("Strings.COMMON.OK",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("	</div>\r");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});


define("hgn!templates/confirmationmessage", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<h1>");t.b(t.v(t.f("header",c,p,0)));t.b("</h1>\r");t.b("\n" + i);t.b("<p>");t.b(t.v(t.f("message",c,p,0)));t.b("</p>\r");t.b("\n" + i);t.b("<div class=\"btn-group btn-group-justified\">\r");t.b("\n" + i);t.b("	<div class=\"btn-group\">\r");t.b("\n" + i);t.b("		<button type=\"button\" id=\"popup-cancel\" class=\"btn btn-default\">");t.b(t.v(t.d("Strings.COMMON.CANCEL",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("	</div>\r");t.b("\n" + i);t.b("	<div class=\"btn-group\">\r");t.b("\n" + i);t.b("		<button type=\"button\" id=\"popup-ok\" class=\"btn btn-default\">");t.b(t.v(t.d("Strings.COMMON.OK",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("	</div>\r");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/**
 * @license RequireJS i18n 2.0.4 Copyright (c) 2010-2012, The Dojo Foundation All Rights Reserved.
 * Available via the MIT or new BSD license.
 * see: http://github.com/requirejs/i18n for details
 */
/*jslint regexp: true */
/*global require: false, navigator: false, define: false */

/**
 * This plugin handles i18n! prefixed modules. It does the following:
 *
 * 1) A regular module can have a dependency on an i18n bundle, but the regular
 * module does not want to specify what locale to load. So it just specifies
 * the top-level bundle, like "i18n!nls/colors".
 *
 * This plugin will load the i18n bundle at nls/colors, see that it is a root/master
 * bundle since it does not have a locale in its name. It will then try to find
 * the best match locale available in that master bundle, then request all the
 * locale pieces for that best match locale. For instance, if the locale is "en-us",
 * then the plugin will ask for the "en-us", "en" and "root" bundles to be loaded
 * (but only if they are specified on the master bundle).
 *
 * Once all the bundles for the locale pieces load, then it mixes in all those
 * locale pieces into each other, then finally sets the context.defined value
 * for the nls/colors bundle to be that mixed in locale.
 *
 * 2) A regular module specifies a specific locale to load. For instance,
 * i18n!nls/fr-fr/colors. In this case, the plugin needs to load the master bundle
 * first, at nls/colors, then figure out what the best match locale is for fr-fr,
 * since maybe only fr or just root is defined for that locale. Once that best
 * fit is found, all of its locale pieces need to have their bundles loaded.
 *
 * Once all the bundles for the locale pieces load, then it mixes in all those
 * locale pieces into each other, then finally sets the context.defined value
 * for the nls/fr-fr/colors bundle to be that mixed in locale.
 */
(function () {
    

    //regexp for reconstructing the master bundle name from parts of the regexp match
    //nlsRegExp.exec("foo/bar/baz/nls/en-ca/foo") gives:
    //["foo/bar/baz/nls/en-ca/foo", "foo/bar/baz/nls/", "/", "/", "en-ca", "foo"]
    //nlsRegExp.exec("foo/bar/baz/nls/foo") gives:
    //["foo/bar/baz/nls/foo", "foo/bar/baz/nls/", "/", "/", "foo", ""]
    //so, if match[5] is blank, it means this is the top bundle definition.
    var nlsRegExp = /(^.*(^|\/)nls(\/|$))([^\/]*)\/?([^\/]*)/;

    //Helper function to avoid repeating code. Lots of arguments in the
    //desire to stay functional and support RequireJS contexts without having
    //to know about the RequireJS contexts.
    function addPart(locale, master, needed, toLoad, prefix, suffix) {
        if (master[locale]) {
            needed.push(locale);
            if (master[locale] === true || master[locale] === 1) {
                toLoad.push(prefix + locale + '/' + suffix);
            }
        }
    }

    function addIfExists(req, locale, toLoad, prefix, suffix) {
        var fullName = prefix + locale + '/' + suffix;
        if (require._fileExists(req.toUrl(fullName + '.js'))) {
            toLoad.push(fullName);
        }
    }

    /**
     * Simple function to mix in properties from source into target,
     * but only if target does not already have a property of the same name.
     * This is not robust in IE for transferring methods that match
     * Object.prototype names, but the uses of mixin here seem unlikely to
     * trigger a problem related to that.
     */
    function mixin(target, source, force) {
        var prop;
        for (prop in source) {
            if (source.hasOwnProperty(prop) && (!target.hasOwnProperty(prop) || force)) {
                target[prop] = source[prop];
            } else if (typeof source[prop] === 'object') {
                if (!target[prop] && source[prop]) {
                    target[prop] = {};
                }
                mixin(target[prop], source[prop], force);
            }
        }
    }

    define('i18n',['module'], function (module) {
        var masterConfig = module.config ? module.config() : {};

        return {
            version: '2.0.4',
            /**
             * Called when a dependency needs to be loaded.
             */
            load: function (name, req, onLoad, config) {
                config = config || {};

                if (config.locale) {
                    masterConfig.locale = config.locale;
                }

                var masterName,
                    match = nlsRegExp.exec(name),
                    prefix = match[1],
                    locale = match[4],
                    suffix = match[5],
                    parts = locale.split("-"),
                    toLoad = [],
                    value = {},
                    i, part, current = "";

                //If match[5] is blank, it means this is the top bundle definition,
                //so it does not have to be handled. Locale-specific requests
                //will have a match[4] value but no match[5]
                if (match[5]) {
                    //locale-specific bundle
                    prefix = match[1];
                    masterName = prefix + suffix;
                } else {
                    //Top-level bundle.
                    masterName = name;
                    suffix = match[4];
                    locale = masterConfig.locale;
                    if (!locale) {
                        locale = masterConfig.locale =
                            typeof navigator === "undefined" ? "root" :
                            (navigator.language ||
                             navigator.userLanguage || "root").toLowerCase();
                    }
                    parts = locale.split("-");
                }

                if (config.isBuild) {
                    //Check for existence of all locale possible files and
                    //require them if exist.
                    toLoad.push(masterName);
                    addIfExists(req, "root", toLoad, prefix, suffix);
                    for (i = 0; i < parts.length; i++) {
                        part = parts[i];
                        current += (current ? "-" : "") + part;
                        addIfExists(req, current, toLoad, prefix, suffix);
                    }

                    req(toLoad, function () {
                        onLoad();
                    });
                } else {
                    //First, fetch the master bundle, it knows what locales are available.
                    req([masterName], function (master) {
                        //Figure out the best fit
                        var needed = [],
                            part;

                        //Always allow for root, then do the rest of the locale parts.
                        addPart("root", master, needed, toLoad, prefix, suffix);
                        for (i = 0; i < parts.length; i++) {
                            part = parts[i];
                            current += (current ? "-" : "") + part;
                            addPart(current, master, needed, toLoad, prefix, suffix);
                        }

                        //Load all the parts missing.
                        req(toLoad, function () {
                            var i, partBundle, part;
                            for (i = needed.length - 1; i > -1 && needed[i]; i--) {
                                part = needed[i];
                                partBundle = master[part];
                                if (partBundle === true || partBundle === 1) {
                                    partBundle = req(prefix + part + '/' + suffix);
                                }
                                mixin(value, partBundle);
                            }

                            //All done, notify the loader.
                            onLoad(value);
                        });
                    });
                }
            }
        };
    });
}());

define('nls/strings',{
	"de": true,
    "root": true
});
define('nls/root/strings',{
    	COMMON:{
            HELP:"Help",
            SEARCH:"Search",
            SEARCH_GROUPS:"Search for Groups",
            SEARCH_PLAYERS:"Search for Players",
            OVERVIEW:"Overview",
            RANKING:"Ranking",
            OK:"Ok",
            REAL_NAME:"Real Name",
            MY_GROUPS:"My Groups",
    		NICKNAME:"Nickname",
    		EMAIL:"E-Mail",
    		COUNTRY:"Country",
    		ORGANIZATION:"Organization",
    		CANCEL:"Cancel",
			WINS:"wins",
            CHANGE:"Change",
			PREV:"Prev",
			NEXT:"Next",
			CHANGE_PWD:"Change Password",
			DELETE:"Delete",
            SAVE:"Save",
			LOGOUT:"Logout",
			BONUS_TIPS:"Bonus Tips",
			RESULTS:"Results",
			PLAYER_DETAILS:"Player Details",
			NEWS:"News",
			HOME:"Home",
            UNEXPECTED:"Unexpected error",
            EXIT:"Exit",
			ONCE_MORE_TO_EXIT:"Press once more to exit!",
            SETTINGS:"Settings",
            NOT_FOUND:"Not found",
            CONNECTION_REFUSED:"Connection refused",
            SERVER_ERROR:"Internal server error",
            SEQURITY_QUESTION:"Security question",
            SEQURITY_ANSWER:"Your answer",
            NOT_IN_TEAM:"Player is not in team anymore",
            SUMMER_TIME:"DST",
            OFFLINE_MESSAGE:"Application could not connect to server during startup! Some data may be out of date or missing",
            PLAYERS:"Players",
            PSO:"PSO"
    	},
    	LOGIN:{
	    	APP_TITLE:"HP Soccer Contest",
	    	PASSWORD:"Password",
	    	LOGIN_PROGRESS:"Signing in..",
	    	LOGIN:"Sign in",
	    	REGISTER:"Register",
	        FAILED_TO_LOGIN:"Failed to login",
            PASSWORD_RESET:"Forgot password",
	    },
	    REGISTER:{
	    	TITLE:"Registration Form",
	    	ERROR_EMAIL_MISSING:"Email must be specified",
	    	ERROR_EMAIL_INVALID:"Make sure valid email is provided",
	    	ERROR_NICKNAME_MISSING:"Nickname must be specified",
	    	ERROR_COUNTRY_MISSING:"Country must be specified",
	    	REGISTRATION_SENT:"Registration request has been sent. Please check your mails for the initial password",
	    	SEND_REGISTRATION_ERROR:"Failed to send registration request",
            ERROR_SEQ_QUESTION_MISSING:"Security question must be specified",
            ERROR_SEQ_ANSWER_MISSING:"Security answer must be specified"
	    },
	    GAME_TYPE:{
			"GA": "Group Stage - Group A",
			"GB": "Group Stage - Group B",
			"GC": "Group Stage - Group C",
			"GD": "Group Stage - Group D",
			"GE": "Group Stage - Group E",
			"GF": "Group Stage - Group F",
			"GG": "Group Stage - Group G",
			"GH": "Group Stage - Group H",
			"R16":"Second Stage - Round of 16",
			"QRT":"Second Stage - Quarter Finals",
			"SF": "Second Stage - Semi Finals",
			"3RD":"Second Stage - Third Place",
			"FIN":"Second Stage - Final"
	    },
	    ERROR_CODE:{
            "invalid_grant":"Invalid username or password",
            "CHGPWD.WRONG_PLAYER":"The security token does not belong to the requested user",
            "CHGPWD.MISSING_DATA":"Invalid parameters. Please pass old password and new password",
            "CHGPWD.INVALID_OLD_PASSWORD":"The passed old password does not match",
            "CHGPWD.SAME_PASSWORD":"The new password must differ from the currently used password",
            "CHGPWD.INVALID_LENGTH":"The password must be at least {0} characters and most {1} characters long",
            "PLAYER.NOT_FOUND":"Player not found",
            "GROUPS.ERR.FAILED_FETCH":"Failed to load groups",
            "GROUPDETAILS.ERR.FAILED_FETCH":"Failed to load group details",
            "GROUPDETAILS.ERR.FAILED_REMOVE":"Failed to remove group membership",
            "GROUPDETAILS.ERR.FAILED_JOIN":"Failed to join the group",
            "GROUPDETAILS.ERR.FAILED_REQUEST":"Failed to create a group join request",
            "FEEDBACK.ERR.FAILED":"Failed to send feedback",
            "EDITPROFILE.ERR.FAILED_PROFILE":"Failed to save profile",
            "EDITPROFILE.ERR.FAILED_GROUP":"Failed to delete group",
            "CHANGEPWD.ERR.OLDPWD":"Old Password must be specified",
            "CHANGEPWD.ERR.NEWPWD":"New password must be specified",
            "CHANGEPWD.ERR.CONFIRMPWD":"Please confirm new password",
            "CHANGEPWD.ERR.NEWPWD_MATCH":"New Password must match",
            "CHANGEPWD.ERR.FAILED":"Failed to change password",
			"SYSTEM.ERR":"An Error occurred. Please try again",
			"RESOURCE.NOT_FOUND":"The requested resource was not found",
			"RESOURCE.NOT_OWNED":"This Resource is not owned by the calling player",
			"RESOURCE.ALREADY_EXISTS": "Already exists",
			"RESOURCE.ERR.UPDATE":"Failed to update resource",
			"GROUPS.ERR.MISSING_DATA":"Please provide valid data for all fields",
			"SYSTEM.ERR.MISSING_DATA":"Required data missing",
			"SYSTEM.ERR.NOT_ALLOWED":'The user has insufficient privileges to perform this operation.',
			"SEARCH.ERR.INVALID_DATA":'Search parameter missing. Please specify at least one field properly',
			"GROUP.ERR.SEARCH_INVALID_DATA": 'Provide at least one character to search for.',
			"RANKING.SEARCH.ERR.INVALID_DATA": 'Provide at least two characters to search in player ranking.',
			"SYSTEM.ERR.MAX_GROUPS_REACHED":"A player cannot own or join more than overall 5 groups",
			"SYSTEM.ERR.NO_NEW_OWNER_FOUND":"The owner cannot leave the group. There is no player left to take ownership. You may delete the group instead",
			"MATCH.ERR.UPDATE":"Could not update results",
			"MATCHTIPS.ERR_SAVE":"Errors occurred during processing of Bulk MatchTip-Saving",
			"MATCHTIPS.ERR_CREATE":"Error occurred while creating a tip",
			"MATCHTIPS.ERR_UPDATE":"Error occurred while updating a tip",
			"BONUSTIP.ERR.INVALID_DATA":"Missing data or value less than 2 chars",
			"BONUSTIP.ERR.INVALID_KEY":"Unknown bonus tip key",
            "BONUSTIP.ERR.INVALID_PLAYER_KEY":"Unknown bonus tip value",
			"BONUSTIP.ERR.CREATE":"Errors occurred during processing of Bulk BonusTip-Creating",
			"BONUSTIP.ERR.TIP_CLOSED":"You cannot enter this BonusTip anymore. Time is over",
			"COMMON.ERR.INVITEPLAYER":"Errors occurred during processing of InviteFriends",
			"COMMON.ERR.FEEDBACK":"Failed to create feedback",
            "REGISTER.ERR.INVALID_LENGTH":"The nickname must be at least {0} characters and most {1} characters long",
			"REGISTER.ERR.EMAIL_FILTER":"You can only register with an HP email address or get invited by a HP-employee",
			"REGISTER.ERR.MAX_PLAYERS_EXCEEDED":"Sorry, the max number of registered Players is reached. No more registrations possible",
			"REGISTER.ERR.NO_VALID_ADDRESS":"You need a valid email address to register",
			"REGISTER.ERR.EMAIL_EXISTS":"You can only register once with an email address. Your address is already in use",
			"REGISTER.ERR.MISSING_DATA":"Please provide valid data for all fields (nickname, email, countrycode)",
			"REGISTER.ERR.NICKNAME_EXISTS":"The defined Nickname already exists. Please choose a different one",
			"REGISTER.ERR.NICKNAME_NOT_ALLOWED":"The defined Nickname does not meet the name policy",
			"PWRESET.EMAIL_NOT_VALID":"Provided email is not valid. Please enter the email you used for registration",
			"PWRESET.EMAIL_EMAIL_NOT_FOUND":"Provided email not found",
			"PWRESET.EMAIL_NICKNAME_NOT_MATCH":"Provided email and nickname combination is not correct",
			"PWRESET.SECURITY_ANSWER_NOT_MATCH":"PW-Reset not possible. The provided security answer does not match",
			"PWRESET.TOKEN_NOT_FOUND":"The provided PW-Reset Token was not found anymore in the database. Please request a new one",
			"GROUPS.ERR.GROUPNAME_EXISTS":"The defined GroupName already exists",
			"GROUPS.ERR.GROUPNAME_INVALID_LENGTH":"The GroupName must be at least 3characters and most 20 characters long",
			"GROUPS.ERR.GROUPNAME_NOT_ALLOWED":"The defined GroupName does not meet the name policy",
			"GROUPS.ERR.DELETE_REASON_NOT_VALID":"A delete reason must be at least 5 characters and most 2000 characters long",
			"GROUPS.ERR.INVITE_ALREADY_MEMBER_OR_INVITED":"Player is already invited or a member of this group",
			"GROUPS.ERR.DELETE_PLAYER_IS_NOT_MEMBER":"Player is not member of this group - cannot be deleted",
			"PLAYER.ERR.INVALID_REASON":"A delete reason must be provided >= 5 chars",
			"PLAYER.ERR.DELETE_NOT_ALLOWED":"Error, a player cannot delete himself",
			"BONUSTIPS.ERR.FAILED_TO_REFRESH":"Failed to refresh tip",
			"BONUSTIPS.ERR.SAVE_FAILED":"Failed to save tip",
			"RANKINGS.ERR.LOADING_FAILED":"Failed to load ranking details",
			"PLAYER_DETAILS.ERR.FAILED_TO_FETCH":"Failed to fetch player details",
			"PLAYER_DETAILS.ERR.FAILED_TO_DELETE":"Failed to delete user",
			"PLAYER_DETAILS.ERR.LOGG_OUT_FAILED":"Failed to logout user",
			"NEWGROUP.ERR.FAILED_TO_CREATE":"Failed to create a group",
			"MESSAGES.ERR.LOADING_FAILED":"Failed to load message details",
			"MATCHTIP.ERR.FAILED_TO_FETCH_STATS":"Failed to fetch statistics",
			"MATCHTIP.ERR.FAILED_TO_FETCH_UPDATED_TIPS":"Failed to fetch updated tips",
			"MATCHTIP.ERR.FAILED_TO_CREATE":"Failed to create tip",
			"MATCHES.ERR.FAILED_TO_REFRESH":"Failed to refresh",
			"MANAGE_GROUP.ERR.FAILED_TO_FETCH":"Failed to fetch group details",
			"MANAGE_GROUP.ERR.SAVE_FAILED":"Failed to save the group",
			"MANAGE_GROUP.ERR.JOINING_FAILED":"Failed to join the group",
			"MANAGE_GROUP.ERR.JOIN_REQUESTS_CREATION_FAILED":"Failed to create a group join request",
			"GROUP.ERR.INVITATION_FAILED":"Failed to send invitation",
			"HOME.ERR.FAILED_TO_FETCH":"Failed to refresh",
            "MATCHTIP.TIP_CLOSED":"You cannot enter Tips for this match anymore. Betting time is over",
			"MAIN.ERR.UNEXPECTED":"Unexpected Exception. Failed to extract details:"
	    },
        BONUSTIPS:{
            WORLD_CHAMPION:"World Champion",
            TIP_TIME:"Tip time",
            POSSIBLE_POINTS:"Possible points",
            NO_TIP:"No tip created yet",
            VICE_CHAMPION:"Vice World Champion",
            GOLDEN_BALL:"Golden Ball",
            GOLDEN_GLOVE:"Golden Glove",
            GOLDEN_BOOT:"Golden Boot",
			CHANGE_BONUSTIP_WARNING_HEADER:"Warning about bonus tip change",
			CHANGE_BONUSTIP_WARNING_MESSAGE:"You are about to change your previous tip. Changing may result in decreased bonus points. Do you want to continue and overwrite your previous tip?",
            FAILED_TO_LOAD:"Failed to load bonus tips",
            FAILED_TO_REFRESH:"Failed to refresh bonus tips",
            FAILED_TO_LOAD_TEAM_PLAYERS:"Failed to load team players",
            PLACE_TILL_SEMI_FINAL:"Hint: Bonus tips can be placed until kick-off of first semi final match",
            RESULT:"Result:",
            AWARDED:"Awarded to:"
        },
        CHANGEPWD:{
            OLD_PWD:"Old Password",
            NEW_PWD:"New Password",
            REPEAT_PWD:"Repeat New Password",
            CHANGING_PWD:"Changing Password...",
            SUCCESS:"Password successfully changed",
            FAILED_SEQ:"Password successfully changed. But security question could not be saved",
            NO_QUESTION:"Do not use security question"
        },
        FEEDBACK:{
            TELL_US:"Tell us what you think about the app. Do you have questions or found a bug? Just use the text area to send us your feedback. Help us to make the app even better",
            REASON:"Reason",
            PROBLEM:"Problem",
            QUESTION:"Question",
            COMMENT:"Comment",
            OTHER:"Other",
            FEEDBACK:"Feedback",
            PROVIDE_FEEDBACK:"Provide your feedback",
            SEND_FEEDBACK:"Send Feedback",
            SENDING_FEEDBACK:"Sending feedback...",
            SENT:"Feedback successfully saved",
            BETTER_NEXT_TIME:"Suggestion for improvement",
            MISSED_FEATURE:"Missing Feature",
            LIKED_MOST:"I like most"
        },
        GROUPDETAILS:{
            TITLE:"Group Details",
            LOAD_DETAILS:"Loading group details...",
            INVITED:"You were invited to this group on",
            AVG_POINTS:"Avg. Points",
            MEMBERS:"Members",
            MANAGE:"Manage",
            MANAGE_GROUP:"Manage Group",
            LEAVE:"Leave",
            LEAVE_GROUP:"Leave Group",
            LEAVING_GROUP:"Leaving group",
            LEAVING_GROUP_MESSAGE:"Are you sure you want to leave the group?",
            JOIN:"Join",
            JOIN_GROUP:"Join Group",
            ALREADY_REQ:"Request to join this group is sent",
            DELETE_GROUP:"Delete Group",
            DELETE_GROUP_REASON:"Why do you want to delete the group",
            GROUP_REQUEST:"Group Request",
            GROUP_REQUEST_MESSAGE:"Please request message",
            GROUP_DELETED:"Group does not exist anymore",
            JOINED_REFRESH_FAILED:"Request to join this group is sent, but failed to update group details"
        },
        GROUPS:{
            TITLE:"Groups",
            CREATE_GROUP:"Create Group",
            CREATE_GROUP_INVITE:"Create Group & Invite Friends",
            NOTHING_FOUND:"No groups were found"
        },
        GROUPSEARCHRESULTS:{
            TITLE:"Group Search",
            SEARCH_RESULTS:"Search Results",
            NO_GROUPS:"No groups were found",
            MORE_AWAILIABLE:"Too many groups found please refine you search criteria"
        },
        HELP:{
            GENERAL:"General",
            GENERAL_DETAILS:"HP provides a tip competition app for the World Cup 2014. You can bet the results of matches and place bonus tips to earn points. The more correct tips you place, the higher you climb in the global ranking of all registered players. <br /><br />Additionally you can set up own groups (private or public) to run dedicated competitions with your team, your project members or your friends and family.",
            REGISTRATION:"Registration",
            REGISTRATION_DETAILS:"All HP employees can participate in this game. All you need is to register - either with a valid @hp.com email address or being invited by a registered HP-Employee. Registration and participation are completely free and just for enjoying the competition.",
            AFTER_REG:"After successful registration, you will receive an email with your initial password",
            INITIAL_PW:"Use this initial password to logon the first time to the app along with your email address or nickname",
            CHANGE_PW:"Change your password once",
            ENJOY:"Enjoy the app.",
            PLEASE_NOTE:"Please note:",
            NOTE_REGISTRATION:"To ensure fairness to all participants, we do not allow the placing of more than one bet per person (i.e. e-mail address). If we find out that some person is using more than one e-mail address we remove them from the competition.",
            FEEDBACK_QUESTION:"Feedback / Question",
            ISSUE:"If you got issues, questions or comments, please use the",
            APP_FEEDBACK_1:"App-Feedback feature",
            APP_FEEDBACK_2:"of the app or send a mail to",
            APP_FEEDBACK_3:"There is no professional helpdesk behind the app, so please be patient with us.",
            MATCH_TIPS_RULES:"Match Tips Rules",
            PLACE_BETS:"You place bets (match tips) on each of the matches of the soccer World Cup 2014 and you get the following points for each tip.",
            POINTS5:"5 points if you guessed the exact result of the match",
            POINTS3:"3 points if you got the correct goal difference without guessing the exact result",
            POINTS1:"1 point if you guessed the winner of the match, but without guessing the exact result or the goal difference",
            NOTE_MATCH_TIPS_RULES:"The tip entry is closed for each match 30 minutes prior to kickoff. After that no entry is possible anymore. Other players may see your placed tips during the group phase. Tips for final matches (Round of 16, quarter finals, semifinals and finals) are only visible after tips are closed.)",
            IMPORTANT:"Important:",
            IMPORTANT_MATCH_TIPS_RUELS:"If a match ends with a draw after regular and extra playing time a penalty-shootout takes place. In that case the result after the extra playing time will be taken into account for scoring – the penalty-shootout is ignored.",
            BONUS_TIPS_RULES:"Bonus Tips Rules",
            BONUS_TEXT:"Every player can set 5 different bonus bets to gain additional points.",
            WINNER:"Winner (who will be the World Cup Champion)",
            SECOND:"Second (who will be the Runners-up)",
            GOLDEN_BALL:"Golden Ball (best player of the competition)",
            GOLDEN_BOOT:"Golden Boot (best scorer of the competition)",
            GOLDEN_GLOVE:"Golden Glove (best keeper of the competition)",
            BONUS_TEXT_2:"For each bonus tip one can get 10 bonus points dependent on the time the tip is placed (assumed it would be correct of course).",
            NOTE_BONUS:"The bonus tip entries are closed by the start of the first semifinal match. Changing a tip sets a new timestamp and decreases the possible points.",
            EXAMPLE:"Example given: if you place a bonus tip",
            EXAMPLE_1:"In the first days or before start of the world cup, you get 10 points",
            EXAMPLE_2:"around the 22nd of June, you may get 7 points",
            EXAMPLE_3:"around the Second Stage matches, you may get 3 or 4 points",
            EXAMPLE_4:"between quarter and semifinal, you may get 2 or 1 point",
            EXAMPLE_BONUS:"The bonus points are calculated after the final match or after the awardee is officially honored. Bonus points are directly added to the collected points from the match tips.",
            GROUP_HANDLING:"Group handling",
            GROUP_HANDLING_TEXT:"Every player can found own groups or join groups to compete directly with friends and colleagues. Groups can be private or public.<br /> For private groups, the owner manages the members by accepting or declining join requests of other players. This is to keep your group your private competition.<br /> Public groups are intended to support bigger teams or project groups, where no additional restrictions are needed.",
            MANAGE_OWN_GROUP:"Manage own groups:",
            MANAGE_1:"you can “create Group” on the Groups screen.",
            MANAGE_2:"you can manage your group with",
            MANAGE_3:"adding other players,",
            MANAGE_4:"accept join request or",
            MANAGE_5:"remove members from your group.",
            ADDING:"Adding other players provides a convenient player search, where players are searched by nickname and email address.",
            JOINING:"Joining other groups:",
            JOINING_1:"you can browse groups in the ranking or search for groups on the Groups screen.",
            JOINING_2:"On the result list, open a group and tap the join Group button",
            JOINING_3:"If the group is private: A request is sent to the group owner to approve your request.",
            JOINING_4:"If the group is public: you are directly added to this group.",
            NOTE_JOINING:"Member changes of group do not directly affect the ranking or points or groups. Changes are reflected after the next match result. <br/>The global group ranking now only shows groups with at least 3 members (since recent server update)."
        },
        HOMEOVERVIEW:{
            LAST_MATCHDAY:"Last matchday results",
            NO_MATCH:"No match results available yet",
            NEXT_MATCHES:"Next matches",
            NO_MORE:"No more upcoming matches. Display Rank? some Farewell message etc....",
            HOME_WELCOME:"Welcome to the HP Soccer Contest 2014",
            HOME_WELCOME_TEXT:"Compete with players worldwide and become the best in three easy steps:",
            MATCHES:"Matches",
            MATCHES_TEXT:"1. Place your tips for the matches",
            BONUSTIPS:"Bonus Tips",
            BONUSTIPS_TEXT:"2. Place your bonus tips",
            GROUPS:"Groups",
            GROUPS_TEXT:"3. Found competition groups",
            RANKINGS:"Global Rankings",
            RANKINGS_TEXT:"4. Accept the contest: Lead the pack and become the best guesser",
            HOME_FAREWELL:"The finals are over",
            FW_NEW_WC:"Congratulations to the new soccer World Champion 2014 from all of us!",
            FW_PERSONAL_STATS:"Your personal achievements:",
            FEEDBACK:"Feedback",
            FEEDBACK_TEXT:"Thank you for using the HP Soccer Contest App. We would like to enhance and become better for the next contest. Please provide us some feedback with the 3 new categories 'Suggestion for Improvement', 'I like most' and 'Missing Feature':",
            DO_NOT_FORGET_BONUSTIPS:"Hint: Don‘t forget to set bonus tips",
            CD_DAYS:"Days",
            CD_HOURS:"Hours",
            CD_MIN:"Minutes",
            CD_SEC:"Seconds",
            MATCHES_AFTER:"Matches Details",
            BONUSTIPS_AFTER:"Bonus Tips Details",
            RANKINGS_AFTER:"Global Rankings",
            GLOBAL_STATS:"Global App Statistics",
            COUNT_PLAYER_OVERALL:"Registered Users",
            COUNT_COUNTRIES_OVERALL:"Countries",
            COUNT_GROUPS_OVERALL:"Groups",
            COUNT_MATCHTIPS_OVERALL:"Matchtips",
            PREDICTION_ACCURACY:"Prediction Accuracy",
            BEST_MATCH:"Best predicted match",
            BEST_QUOTE:"Highest average points",
            WORST_MATCH:"Worst predicted match",
            WORST_QUOTE:"Lowest average points",
            BEST_WORST_YOU:"Your points:",
            AVG_MATCHTIP_POINTS:"Average Matchtip Points",
            AVG_BONUSTIP_POINTS:"Average Bonus Tip Points",
            COMMUNITY:"Community",
            VS:"vs.",
            YOU:"You",
            OVERALL_POINTS:"Overall Points",
            
        },
        INVITETEXT:{
            SEND_INVITATION:"Send an invitation mail to your friends",
            EMAILS:"Emails",
            EMAIL_ADDRESS:"Email address of invitee",
            SEND:"Send Invitation",
            SENDING:"Sending invitations...",
            AREADY_INVITED:"Player already invited",
            INVALID_MAIL:"Invalid email provided"
        },
        INVITE_TO_MY_GROUP:{
            TITLE:"Invite player",
			INVITE:"Invite",
			INVITE_EXT:"Invite friends",
			SELECT_GROUP:"Select your groups to which you want to invite the user",
			MIN_GROUP_HINT:"Please select at least one group to which you want to invite user",
			SENT:"Invitation sent",
			INVITATIONS_SENT:"Invitations sent"
        },
        ABOUT:{
            TOTAL:"Overall players count",
			ABOUT:"About",
            ABOUT_HP:"About HP",
            ABOUT_HP_TEXT:"HP creates new possibilities for technology to have a meaningful impact on people, businesses, governments and society. With the broadest technology portfolio spanning printing, personal systems, software, services and IT infrastructure, HP delivers solutions for customers’ most complex challenges in every region of the world. More information about HP (NYSE: HPQ) is available at ",
            PRIVACY_STATEMENT:"HP Privacy Statement Worldwide",
            ABOUT_SOCCER:"About HP Soccer Contest",
            DESCRIPTION:"Description",
            DESCRIPTION_TEXT:"The HP community app for tipping the world cup 2014 – Connect and tip the World Cup 2014 games, check results and tip with coworkers and partners in your own groups. Will you or one of your groups win the challenge against the others? Get invited to the app by your HP contact and enjoy the large community. Join today and have fun during the world cup. To join, login with your @hp.com mail address. If you are non-HP employee, please get your personal invitation by your HP contact, e.g. Sales Executives, Account Representatives or others.",
            ORIGIN:"Origin",
            ORIGIN_TEXT:"The soccer contest app continues the heritage started with the \"Fußball-Tipp-Gemeinschaft\" FTG back in 2006. This app is the New Style of IT version of tip community. The app was developed by HP's consultancy unit Industry & Transformation Consulting (ITC) Germany.<br/> ITC is an industry leading consultancy organization helping HP customers to leverage the latest technology in the right way - focused on the business outcomes. ITC enables the transformation to the new style of IT.",
            VERSION_INFORMATION:"Version Information",
            APP_VERSION:"App Version",
            SERVER_VERSION:"Server Version",
            FROM:"from",
            WHATS_NEW:"What's New?",
            INTERNAL:"HP internal players",
            EXTERNAL:"External players",
            V180_VERSION:"Version: 1.8.0 (2014-06-12)",
            V180_BODY:"<ul><li>Fixing matches displayed on overview page (Past and upcoming matches).</li><li>Fixing large groups with more than 50 players (paging to browse and see all players).</li></ul>",
            V190_VERSION:"Version: 1.9.0 (2014-06-20)",
            V190_BODY:"<ul><li>Only groups with 3 or more players are shown in group ranking: invite more players to see your group in the global group ranking.</li><li>Serach and find players: tap players in the menu to open the search dialog.</li><li>Email addresses of non-HP employees are masked to ensure privacy.</li><li>Introducing the release notes about the latest changes.</li></ul>",
            V200_VERSION:"Version: 2.0.0 (2014-06-30)",
            V200_BODY:"<ul><li>Detailed stats for each match.</li><li>Stats and personal achievements on home overview page. Compare yourself againts the community. The stats will be displayed after the final game.</li><li>Optimized view of bonus tips.</li><li>Fixed error to display matches again.</li><li>Fixed error in group management re-enabling editing membership.</li></ul>",
        },
        EDITPROFILE:{
            TITLE:"Edit Profile",
            SAVED:"Details saved",
            DELETE_PLAYER:"Delete player",
            DELETE_REASON:"Please provide reason",
            GROUP_DELETE:"Group deleted",
            GROUP_DELETE_REFRESH_FAILED:"Group deleted but failed to reload player details"
        },
    
        /** BEGIN FLORIAN */
        TIP_STATS:{
			TOTAL_NUMBER_OF_TIPS:"Total number of tips so far",
			DRAW:"Draw" /* "unentschieden" */
		}, 
		RANKINGS_PLAYER:{
			HEADLINE:"Global Player Ranking"
		},
		RANKINGS_GROUP:{
			HEADLINE:"Global Group Ranking"
		},
		RANKINGS_COUNTRY:{
			HEADLINE:"Global Country Ranking"
		},
		PLAYER_DETAILS:{
			MY_PROFILE:"My Profile",
			MATCHTIP_POINTS:"Matchtip Points",
			BONUSTIP_POINTS:"Bonus Points",
			GLOBAL_RANK:"Global Rank",
			GENITIVE_GROUPS:"'s Groups",
			NO_GROUPS:"No groups yet. Create own or join other groups.",
			MY_BONUSTIPS:"My Bonus Tips",
			GENITIVE_BONUSTIPS:"'s Bonus Tips",
			MY_TIPS:"My Tips",
			GENITIVE_TIPS:"'s Tips",
			INVITE:"Invite",
			INVITE_USER:"Invite User",
			LOGOUT_ADMIN:"Logout (Admin)",
			LOGOUT_USER:"Logout User",
			DELETE_USER:"Delete User",
			LOG_ME_OUT:"Log me out",
			DELETE_PLAYER_HEADER:"Delete player",
			DELETE_PLAYER_MESSAGE:"Please provide reason",
			PLAYER_DELETED:"Player deleted",
			LOGGED_OUT:"Player logged out",
            NO_MATCH_TIPS:"No Match Tips yet",
            NO_BONUS_TIPP:"Bonus Tip not set yet",
            FAILED_TO_REFERSH:"Failed to load player details"
		},
		NEWGROUP:{
			TITLE:"New Group",
			CREATE_NEW:"Create New Group",
			CREATE:"Create Group",
			ACTION_DESC:"Add a new group and connect with your friends for a challenge.",
			GROUP_NAME:"Group Name",
			DESC:"Description",
			DESC_PLACEHOLDER:"Enter Description (140 chars)",
			MAKE_GROUP_PRIVATE:"Make group private",
			CREATING:"Creating group...",
            FAILED_TO_REFRESH:"Group is created but failed to refresh player details"
		},
		MESSAGES:{
			TITLE:"Messages",
			YOU_ARE_INVITED_PART1:"You are invited to join group",
			YOU_ARE_INVITED_PART2:". Tap to see details and join.",
			PLAYER_WANTS_TO_JOIN_PART1:"Player",
			PLAYER_WANTS_TO_JOIN_PART2:"wants to join your group",
			PLAYER_WANTS_TO_JOIN_PART3:"Tap to manage this group.",
			NO_MESSAGES:"No incoming game messages",
            UPDATE_HEAD:"What's changing in version 1.9:",
            UPDATE_MSG:"Only groups with 3 or more players are shown in group ranking. Invite more players to see your group in the global group ranking again.",
            READ_MORE:"See all changes ..."
		},
		MATCHTIPS:{
			TITLE:"Matchtip",
			SAVING:"Saving...",
			SAVE:"Save",
			CANNOT_YET:"You cannot put tips yet, tips will be enabled after",
			CANNOT_ANYMORE:"You cannot put tips anymore, tips were disabled since",
			COMMUNITY_STATS:"Community Tip Stats",
			LOADING_STATS:"Loading Statistics",
			NO_STATS_AVAILABLE:"No statistics available",
            TIME_VENUE:"Kickoff and Venue",
            TIPS_ENABLED_TO:"Tips enabled until:",
            STADIUM:"Stadium:",
            LOCAL_KICKOFF:"Kickoff in your Time Zone",
            BRAZIL_KICKOFF:"Kickoff in Brazil",
			ON:"on", /* used at {{KickOffTime}} on {{KickOffDay}} */ 
            WINS:"wins",
            REFRESH_FAILED:"Failed to refresh",
            FAILED_TO_LOAD:"Failed to load match tips",
            OTHER:"other",
            POINTS_5:"5 Points",
            POINTS_3:"3 Points",
            POINTS_1:"1 Point",
            POINTS_0:"0 Points"
		},
		MATCH:{
			TITLE:"Matches",
			POINTS:"Points", /* besser: Point(s)? */
			NOT_TIPPED:"Not tipped",
			NO_TIPS:"No tips",
			TIP_NOW:"Tip Now!",
			CANNOT_ANYMORE:"Cannot put tips anymore",
			ON:"on", /* used at {{KickOffTime}} on {{KickOffDay}} */ 
			GROUP_STAGE_DATE:"Group Stage",
			GROUP_STAGE_DATE_SUBLINE:"by Date",
            GROUP_STAGE_GROUP:"Group Stage",
			GROUP_STAGE_GROUP_SUBLINE:"by Groups",
			SECOND_STAGE:"Second Stage"
		},
		GROUP:{
			JOIN_REQUESTS:"Join Requests",
			APPROVE:"Approve",
			DECLINE:"Decline",
			NO_REQUESTS:"No new request",
			GROUP_MEMBERS:"Group Members",
			MEMBERS:"Members",
			NO_MEMBERS:"No members yet",
			LOADING:"Loading...",
			PRIVATE_GROUP:"Private Group",
			SAVE_CHANGES:"Save Changes",
			ADD_USERS_TO_GROUP:"Add Users to Group",
			ADD_USERS:"Add Users",
			ITS_ME:"It's me",
			INVITE:"Invite",
			NO_PLAYERS_FOUND:"No players were found",
            NO_MORE_RESULTS:"No more players were found",
			GROUP:"Group",
			UPDATED:"Group Updated",
			SEARCH_MIN_CHARS:"Provide at least one character to search for",
			INVITE_PLAYER_HEADER:"Group Invitation",
			INVITE_PLAYER_MESSAGE:"Please provide an invitation message",
            SENT:"Invitation successfully sent",
            JOINED:"You have successfully joined this group",
            SEARCH_PLAYERNICK:"Search player by nickname",
            REMOVE:"Remove",
            REMOVING_FAILED:"Failed to remove player from the group",
            REMOVING_FAILED_REFRESH:"Player removed but failed to refresh group details",
            INVITE_FAILED_REFRESH:"Player invited but failed to refresh group details",
            INVITATION_STATUS_FAILED_REFRESH:"Player invitation was process but failed to refresh group details",
		},
        RESET:{
            RESET:"Request new password",
            TITEL:"Reset password",
            MISSING_EMAIL:"Please enter your email used during registration in the email text field. Your new password will be sent to that email address",
            MISSING_ANSWER:"Please enter your answer on your security question used during registration",
            REQUESTED: "Password reset has been requested!",
            FAILED:"Password reset request has failed:",
            HEADER:"We need some information to verify your ID",
            MESSAGE:"A link to change your password will be sent to your email",
            NONE_USED:"No security question used"
        },
        SEQURITY_QUESTION:{
            Q1:"Who is your all time favorite band",
            Q2:"What is the name of the street on which you grew up?",
            Q3:"What is the name of your favorite sports team?",
            Q4:"What was your first pet's name?",
            Q5:"What is the name of your favorite book?",
            Q6:"Who is your favorite book author?"
        }
    }
);

define('util/feedback',[
	'jquery',
	'models/settings'
], function ($,Settings) {
	
	var metadata=JSON.stringify({
		agent:navigator.userAgent,
		language:navigator.language,
		date:new Date(),
		server:Settings.get("server"),
		version:Settings.appVersion
	});
	var FeedbackHelper={
			sendFeedback:function(category,message){
				return $.ajax({
					type: "POST",
					url:Settings.get("server") +"/rest/common/feedback",
					contentType: 'application/json',
					data: {
						category:category,
						content:message,
						metadata:metadata
					}
				});
			},
			sendLog:function(category,message){
				console.warn("Reporting error")
				return $.ajax({
					type: "POST",
					url:Settings.get("server") +"/rest/common/log",
					contentType: 'application/json',
					data: {
						category:category,
						content:message,
						metadata:metadata
					}
				});
			},
		}
	return FeedbackHelper;

}
);
define('util/message',[
	'jquery',
	'hgn!templates/popupmessage',
	'hgn!templates/confirmationmessage',
	'i18n!nls/strings',
	'util/feedback',
	'models/settings',
], function ($,PopupMessage,ConfirmationMessage,Strings,Feedback,Settings) {
	
	var doNotSend=[
		'GROUPS.ERR.DELETE_REASON_NOT_VALID',
		'GROUPS.ERR.GROUPNAME_INVALID_LENGTH',
		'GROUPS.ERR.GROUPNAME_NOT_ALLOWED',
		'GROUPS.ERR.GROUPNAME_EXISTS',
		'SYSTEM.ERR.MAX_GROUPS_REACHED',
		'SYSTEM.ERR.NO_NEW_OWNER_FOUND',
		'GROUP.ERR.SEARCH_INVALID_DATA',
		'GROUPS.ERR.INVITE_ALREADY_MEMBER_OR_INVITED'
	],
	extractError=function(ajaxResponce,message){
		var errorMessage=null,
			replaceParams=function (str, replacements) {
			    return str.replace(/\{(\d+)\}/g, function() {
			        return replacements[arguments[1]];
			    });
			}

		var code,msg;
		if (ajaxResponce && ajaxResponce.responseJSON){
			code=ajaxResponce.responseJSON.code||ajaxResponce.responseJSON.error;
			msg=ajaxResponce.responseJSON.message||ajaxResponce.responseJSON.error_description;
			if (code){
				errorMessage=Strings.ERROR_CODE[code];
				if (ajaxResponce.responseJSON.parameters && ajaxResponce.responseJSON.parameters.length){
					errorMessage=replaceParams(errorMessage,ajaxResponce.responseJSON.parameters);
				}
			}
			if (!errorMessage && msg){
				errorMessage=msg;
			}
		}

		if (!errorMessage){
			if (ajaxResponce.status == 404){
				errorMessage=message + ": 404 "+Strings.COMMON.NOT_FOUND+"("+ajaxResponce.responseText+")";
			}else if (ajaxResponce.status === 0){
				errorMessage=message+ ": "+Strings.COMMON.CONNECTION_REFUSED;
			}else if (ajaxResponce.status == 500){
				errorMessage=message+ ": 500 "+Strings.COMMON.SERVER_ERROR;
			}else{
				errorMessage=message||Strings.COMMON.UNEXPECTED;
				if (ajaxResponce.status){
					errorMessage=errorMessage+": "+ajaxResponce.status+" "+ajaxResponce.statusText
				}
			}
			
		}
		return errorMessage;
	},
	 extractErrorDetails=function(ajaxResponce,message,ajax){
		var errorMessage;
		var json=ajaxResponce && ajaxResponce.responseJSON;
		if (json){
			errorMessage="responseJSON="+JSON.stringify(json);
			if (json.code && _.contains(doNotSend, json.code)){
				return;
			}
		}
	
		if (ajaxResponce.responseText){
			errorMessage+="; responseText="+ajaxResponce.responseText;
		}

		errorMessage+="; Message="+message;
		errorMessage+="; status="+ajaxResponce.status;
		errorMessage+="; statusText="+ajaxResponce.statusText;
		if (ajax){
			var url=ajax && ajax.url;
			if (url && typeof url === "function"){
				url=url.call(ajax);
			}
			if (url){
				if (url && url.toLowerCase && (
					url.toLowerCase().match('/token')||
					url.toLowerCase().match('/chgpwd')
					)){
					//this is a login call, do not send feedback for it to prevent sending password
					return
				}
			}
			errorMessage+="; Type="+ajax.type; 
			errorMessage+="; Url="+url; 
			errorMessage+="; Data="+JSON.stringify(ajax.data);
		}
		return errorMessage;
	}

	var MessageHelper={
			sendErrorDetails:function(ajaxResponce,message,ajax){
				//if (!Settings.get("server").toUpperCase().match('LOCALHOST')){
					var errorMessage;
					try{
						errorMessage=extractErrorDetails(ajaxResponce,message,ajax);
					}catch(e){
						errorMessage="Unexpected error extracting error details:"+e.message;
					}
					console.log("errorMessage:"+errorMessage);
					if (errorMessage){
						Feedback.sendLog("Service Error",errorMessage);
					}
				//}
			},
			handleError:function(ajaxResponce,message,ajax){
				this.showError(extractError(ajaxResponce,message));
				this.sendErrorDetails(ajaxResponce,message,ajax);
			},
			showInfo:function(options){
				if (typeof options === "string") {
					options={message:options};
				}
				if (window.device ){
					window.plugins.toast.showShortTop(options.message);
					return;
				}
				$("#messages")
					.addClass("alert-success")
					.removeClass("alert-danger")
					.removeClass('hidden')
					.addClass("show")
					.text(options.message);
			},
			showError:function(options){
				if (typeof options === "string") {
					options={message:options};
				}
				//owner.data("content",options.message);
				//owner.data("title","Error");
				//owner.popover('show')
				setTimeout(function(){
				$("#messages")
					.removeClass("alert-success")
					.addClass("alert-danger")
					.removeClass('hidden')
					.addClass("show")
					.text(options.message);
				},0)
			},
			hide:function(){
				$("#messages")
					.addClass('hidden')
					.removeClass("show")
			},
			showInputMessage:function(options){
				var deferred=new $.Deferred();
				if(navigator.notification && navigator.notification.prompt){
					navigator.notification.prompt(
				        options.message,  // message
				        function(result){
				        	if (result.buttonIndex==1){
				        		deferred.resolve(result.input1);
				        	}else{
				        		deferred.reject();
				        	}
				        },                 
				        options.header,        
				        [Strings.COMMON.OK,Strings.COMMON.CANCEL],
				        options.placeholder
				    );
				}else{

					/*var message=prompt(options.message, options.header );
					if (message){
						deferred.resolve(message);
					}else{
						deferred.reject();
					}*/
					var $messages=$("#popup-messages"),
						$messageBackdrop=$("#popup-messages-backdrop");

					$messages.html(PopupMessage($.extend({Strings:Strings},options)));
					$messages.show();
					$messageBackdrop.show();
					$messages.on("click",'#popup-ok',function(event){
						event.preventDefault();
						var message=$messages.find("#popup-message").val();
						$messages.hide();
						$messageBackdrop.hide();
						$messages.empty();
						$messages.off();
						deferred.resolve(message);

					});
					$messages.on("click",'#popup-cancel',function(event){
						event.preventDefault();
						$messages.hide();
						$messageBackdrop.hide();
						$messages.empty();
						$messages.off();
						deferred.reject();
					});
				}
				return deferred.promise();
			},
			showConfirmationMessage:function(options){

				var deferred=new $.Deferred();
				if (!navigator.notification || !navigator.notification.confirm){
					var $messages=$("#popup-messages"),
						$messageBackdrop=$("#popup-messages-backdrop");
					$messages.html(ConfirmationMessage($.extend({Strings:Strings},options)));
					$messages.show();
					$messageBackdrop.show();
					$messages.on("click",'#popup-ok',function(){
						$messages.hide();
						$messageBackdrop.hide();
						$messages.empty();
						deferred.resolve();
					});
					$messages.on("click",'#popup-cancel',function(){
						$messages.hide();
						$messageBackdrop.hide();
						$messages.empty();
						deferred.reject();
					});
				}else{
					 navigator.notification.confirm(
					        options.message, 
					        function(res){
								if (res === 1){
									deferred.resolve()
								}else{
									deferred.reject();
								}
					        },
					        options.header,            
					        Strings.COMMON.OK+","+Strings.COMMON.CANCEL
					    );
					}
					return deferred.promise();
				}
		}
	return MessageHelper;

}
);
define('util/context',[
	'jquery',
	'models/settings',
	'models/player',
	'collections/results',
	'collections/tips',
	'util/message',
	"i18n!nls/strings",
	], function ($,Settings,PlayerModel,Results,Tips,MessageHelper,Strings) {
	
	
	var context= {
		timezoneMap:{
			'Sao Paulo':180,
			'Natal':180,
			'Salvador':180,
			'Cuiaba':240,
			'Belo Horizonte':180,
			'Fortaleza':180,
			'Recife':180,
			'Manaus':240,
			'Brasilia':180,
			'Porto Alegre':180,
			'Rio de Janeiro':180,
			'Curitiba':180
		},
		signedin:false,
		positions:{},
		whenAllDone :function() {
		    var deferreds = [];
		    var result = $.Deferred();

		    $.each(arguments, function(i, current) {
		        var currentDeferred = $.Deferred();
		        current.then(function() {
		            currentDeferred.resolve(false, arguments);
		        }, function() {
		            currentDeferred.resolve(true, arguments);
		        });
		        deferreds.push(currentDeferred);
		    });

		    $.when.apply($, deferreds).then(function() {
		        var failures = [];
		        var successes = [];

		        $.each(arguments, function(i, args) {
		            // If we resolved with `true` as the first parameter
		            // we have a failure, a success otherwise
		            var target = args[0] ? failures : successes;
		            var data = args[1];
		            // Push either all arguments or the only one
		            target.push(data.length === 1 ? data[0] : args);
		        });

		        if(failures.length) {
		            return result.reject.apply(result, failures);
		        }

		        return result.resolve.apply(result, successes);
		    });

		    return result;
		},
		setScrollPosition:function(id,pos){
			this.positions[id]=pos;
		},
		getScrollPosition:function(id){
			return this.positions[id]||0;
		},
		getCurrentDate:function(){
		/*	var dt=Settings.get("currentdate"),
				currentDate=new Date();
			if (dt){
				var parsed=moment(dt);
				if (parsed.isValid()){
					currentDate=parsed;
				}
			}*/
			return new Date();
		},
		sendAuthentication : function (xhr) {
			if (context.access_token){
			  xhr.setRequestHeader('Authorization', "Bearer "+context.access_token)
			}
		},
		cleanLoginData:function(){
			$(".private-menu-item").hide();
			$("#login").css( "display", "block");
			this.signedin=false;
			delete this.access_token;
			Settings.save({
				access_token:undefined,
				expires_in:undefined,
				loggedIn:undefined
			}).always(function(){
				window.location.hash = '#login';
			});
		},
		logoff:function(){
			$.ajax( Settings.get("server") + "/rest/player/logout");
			this.cleanLoginData();
		},
		haveLocalToken:function(){
			return Settings.get("access_token")&&Settings.get("playerId");
		},
		loginUsingToken:function(){
			var self=this;
			return this.loadInitialData()
			.done(function(){
					$(".private-menu-item").css( "display", "block");
					$("#login").css( "display", "none");
			}).fail(function(data){
					var msg=$("#messages").text();
					if (msg){
						self.errorMessage=msg;
					}
			});
		},
		updateMessagesCount:function(){
			if (!this.player){
				return;
			}
			var messages=this.player.get('messages'),
                groupJoinRequests=this.player.get('groupJoinRequests'),
                sum=0;
            if (messages && messages.length){
                sum+=messages.length;
            }
            if (this.player.get('groupInvitations')){
	            var groupInvitations=$.grep(this.player.get('groupInvitations'),function(invitation){
	                	return invitation.status == 'invited';
	                });
	            if (groupInvitations && groupInvitations.length){
	                sum+=groupInvitations.length;
	            }
	        }
            if (groupJoinRequests && groupJoinRequests.length){
                sum+=groupJoinRequests.length;
            }
			$('#new-messages').text(sum);
		},
		getCommonStatistics:function(){
			var dfd = $.Deferred();
 			$.ajax(
				{
					type:"GET",
					url:Settings.get("server") + "/rest/common/stats/"+this.player.get('id'),
					headers:{
						'Cache-Control': 'no-cache'
					}
				}
			).done(function(data){
				var ret={};
				_.each(data,function(stat){
					ret[stat.name]=stat.value;
				});
				dfd.resolve(ret);
			}).fail(function(ajax,error,message){
				MessageHelper.handleError(ajax,Strings.MATCHTIPS.NO_STATS_AVAILABLE,this);
				dfd.reject();
			})
			return dfd.promise();
		},
		loadInitialData:function(){
			var dfd = $.Deferred(),
				self=this,
				preloadData=function(){
					self.updateMessagesCount();
					self.whenAllDone(
						Results.reload({prefill: true})
							.fail(function(results,ajax,error,message){
								MessageHelper.handleError(ajax,Strings.COMMON.OFFLINE_MESSAGE,this);
							}),
						Tips.reload({prefill: true})
							.fail(function(tips,ajax,error,message){	
								MessageHelper.handleError(ajax,Strings.COMMON.OFFLINE_MESSAGE,this);
							})								
					).done(function(){
						dfd.resolve();
					}).fail(function(){
						if (Results.length){
							self.offline=self.offline||true;
							dfd.resolve();
						}else{
							dfd.reject();
						}
					})
				};
			this.access_token=Settings.get("access_token");
			this.signedin=true;
			this.player=new PlayerModel({playerId:Settings.get("playerId")});
			this.player.reload({prefill: true})
				.fail(function(player,ajax,error,message){
					if (ajax.status == 401){
						self.errorMessage="Authentication token is no longer valid";
						//unauthorized clear login data
						self.cleanLoginData();
						dfd.reject();
					}else{
						self.offline=true;
						if (player.get("nickName")){
							//Data was prefetched from cache try to continue
							preloadData();
						}else{
							MessageHelper.handleError(ajax,"Failed loading player details",this);
							dfd.reject();
						}
					}
				})
				.done(preloadData);

			//Clean up login flags if too optimistically assumed that login will succeed
			dfd.fail(function(){
				delete self.access_token;
				delete self.signedin;
				delete self.player
			});

			return dfd.promise();
		},
		player:null

	};
	return context;
}
);

define('util/wait',[
	'jquery',
	'util/context'
], function ($,Context) {
	
	
	var  counter=0,
		refresh,
		usedElement=null;
	

	var Wait=
		{
		start:function(element){
			refresh=refresh||$("#refresh")

			if (counter===0){
				$('.glyphicon',refresh).addClass('animate-spin');
			}

			refresh.show();

			counter++;
		},
		forceStop:function(){
			refresh=refresh||$("#refresh");
			counter = 0;
			$('.glyphicon',refresh).removeClass('animate-spin');
			if (!refresh.hasClass("have-refresh")){
					refresh.hide();
			}
		},
		stop:function(element){
			refresh=refresh||$("#refresh")

			if (counter >0){
				--counter;
			}
			if(counter === 0){
				$('.glyphicon',refresh).removeClass('animate-spin');
				if (!refresh.hasClass("have-refresh")){
					refresh.hide();
				}
			}
		}
	};
	return Wait;

});
/*global define*/
define('views/baseview',[
	'jquery',
	'underscore',
	'backbone',
	'util/context',
], function ($, _, Backbone,Context) {
	

	var BaseView = Backbone.View.extend({
		titel 	: "Undefined",
		subView : true,

		getTitel : function() {
			return this.titel
		},

		isSubView : function(){
			return this.subView;
		},
		trackScrolling:function(element){
			if (this.trackScrollingElement){
				this.trackScrollingElement.off("scroll");
			}
			this.trackScrollingElement=element;
			this.trackScrollingElement.on("scroll",function(){
				var top=element[0].scrollTop;
				Context.setScrollPosition(window.location.hash,top);
			});
		},
		updateScroll:function(){
			if (this.trackScrollingElement && this.trackScrollingElement[0]){
				this.trackScrollingElement[0].scrollTop=Context.getScrollPosition(window.location.hash);
			}
		}
	});

	return BaseView;
});


define("hgn!templates/settings", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class='sub-container'>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);t.b("       <div class=\"form-group\">\r");t.b("\n" + i);t.b("        <label for=\"userid\">Server</label>\r");t.b("\n" + i);t.b("        <input type=\"text\" class=\"form-control search-text\" id=\"server\" placeholder=\"Server\" value=\"");t.b(t.v(t.f("server",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("        <p class=\"help-block\">Server to connect</p>\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("      <div class=\"form-group\">\r");t.b("\n" + i);t.b("            <div class=\"checkbox\">\r");t.b("\n" + i);t.b("                <input id=\"weinre\" type=\"checkbox\" ");if(t.s(t.f("weinre",c,p,1),c,p,0,499,520,"{{ }}")){t.rs(c,p,function(c,p,t){t.b(" checked=\"isChecked\" ");});c.pop();}t.b(">\r");t.b("\n" + i);t.b("                <label for=\"weinre\">Enable Weinre</label>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("            <div class=\"checkbox\">\r");t.b("\n" + i);t.b("                <input id=\"extendedError\" type=\"checkbox\" ");if(t.s(t.f("extendedError",c,p,1),c,p,0,725,746,"{{ }}")){t.rs(c,p,function(c,p,t){t.b(" checked=\"isChecked\" ");});c.pop();}t.b(">\r");t.b("\n" + i);t.b("                <label for=\"extendedError\">Display extended error details</label>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("      <div class=\"form-group\">\r");t.b("\n" + i);t.b("          <input id=\"save-settings\" type=\"button\" class=\"btn btn-primary btn-lg btn-block\" value=\"Save\"/>\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/*!
 * Bootstrap v3.1.1 (http://getbootstrap.com)
 * Copyright 2011-2014 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 */

if (typeof jQuery === 'undefined') { throw new Error('Bootstrap\'s JavaScript requires jQuery') }

/* ========================================================================
 * Bootstrap: transition.js v3.1.1
 * http://getbootstrap.com/javascript/#transitions
 * ========================================================================
 * Copyright 2011-2014 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  

  // CSS TRANSITION SUPPORT (Shoutout: http://www.modernizr.com/)
  // ============================================================

  function transitionEnd() {
    var el = document.createElement('bootstrap')

    var transEndEventNames = {
      'WebkitTransition' : 'webkitTransitionEnd',
      'MozTransition'    : 'transitionend',
      'OTransition'      : 'oTransitionEnd otransitionend',
      'transition'       : 'transitionend'
    }

    for (var name in transEndEventNames) {
      if (el.style[name] !== undefined) {
        return { end: transEndEventNames[name] }
      }
    }

    return false // explicit for ie8 (  ._.)
  }

  // http://blog.alexmaccaw.com/css-transitions
  $.fn.emulateTransitionEnd = function (duration) {
    var called = false, $el = this
    $(this).one($.support.transition.end, function () { called = true })
    var callback = function () { if (!called) $($el).trigger($.support.transition.end) }
    setTimeout(callback, duration)
    return this
  }

  $(function () {
    $.support.transition = transitionEnd()
  })

}(jQuery);

/* ========================================================================
 * Bootstrap: alert.js v3.1.1
 * http://getbootstrap.com/javascript/#alerts
 * ========================================================================
 * Copyright 2011-2014 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  

  // ALERT CLASS DEFINITION
  // ======================

  var dismiss = '[data-dismiss="alert"]'
  var Alert   = function (el) {
    $(el).on('click', dismiss, this.close)
  }

  Alert.prototype.close = function (e) {
    var $this    = $(this)
    var selector = $this.attr('data-target')

    if (!selector) {
      selector = $this.attr('href')
      selector = selector && selector.replace(/.*(?=#[^\s]*$)/, '') // strip for ie7
    }

    var $parent = $(selector)

    if (e) e.preventDefault()

    if (!$parent.length) {
      $parent = $this.hasClass('alert') ? $this : $this.parent()
    }

    $parent.trigger(e = $.Event('close.bs.alert'))

    if (e.isDefaultPrevented()) return

    $parent.removeClass('in')

    function removeElement() {
      $parent.trigger('closed.bs.alert').remove()
    }

    $.support.transition && $parent.hasClass('fade') ?
      $parent
        .one($.support.transition.end, removeElement)
        .emulateTransitionEnd(150) :
      removeElement()
  }


  // ALERT PLUGIN DEFINITION
  // =======================

  var old = $.fn.alert

  $.fn.alert = function (option) {
    return this.each(function () {
      var $this = $(this)
      var data  = $this.data('bs.alert')

      if (!data) $this.data('bs.alert', (data = new Alert(this)))
      if (typeof option == 'string') data[option].call($this)
    })
  }

  $.fn.alert.Constructor = Alert


  // ALERT NO CONFLICT
  // =================

  $.fn.alert.noConflict = function () {
    $.fn.alert = old
    return this
  }


  // ALERT DATA-API
  // ==============

  $(document).on('click.bs.alert.data-api', dismiss, Alert.prototype.close)

}(jQuery);

/* ========================================================================
 * Bootstrap: button.js v3.1.1
 * http://getbootstrap.com/javascript/#buttons
 * ========================================================================
 * Copyright 2011-2014 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  

  // BUTTON PUBLIC CLASS DEFINITION
  // ==============================

  var Button = function (element, options) {
    this.$element  = $(element)
    this.options   = $.extend({}, Button.DEFAULTS, options)
    this.isLoading = false
  }

  Button.DEFAULTS = {
    loadingText: 'loading...'
  }

  Button.prototype.setState = function (state) {
    var d    = 'disabled'
    var $el  = this.$element
    var val  = $el.is('input') ? 'val' : 'html'
    var data = $el.data()

    state = state + 'Text'

    if (!data.resetText) $el.data('resetText', $el[val]())

    $el[val](data[state] || this.options[state])

    // push to event loop to allow forms to submit
    setTimeout($.proxy(function () {
      if (state == 'loadingText') {
        this.isLoading = true
        $el.addClass(d).attr(d, d)
      } else if (this.isLoading) {
        this.isLoading = false
        $el.removeClass(d).removeAttr(d)
      }
    }, this), 0)
  }

  Button.prototype.toggle = function () {
    var changed = true
    var $parent = this.$element.closest('[data-toggle="buttons"]')

    if ($parent.length) {
      var $input = this.$element.find('input')
      if ($input.prop('type') == 'radio') {
        if ($input.prop('checked') && this.$element.hasClass('active')) changed = false
        else $parent.find('.active').removeClass('active')
      }
      if (changed) $input.prop('checked', !this.$element.hasClass('active')).trigger('change')
    }

    if (changed) this.$element.toggleClass('active')
  }


  // BUTTON PLUGIN DEFINITION
  // ========================

  var old = $.fn.button

  $.fn.button = function (option) {
    return this.each(function () {
      var $this   = $(this)
      var data    = $this.data('bs.button')
      var options = typeof option == 'object' && option

      if (!data) $this.data('bs.button', (data = new Button(this, options)))

      if (option == 'toggle') data.toggle()
      else if (option) data.setState(option)
    })
  }

  $.fn.button.Constructor = Button


  // BUTTON NO CONFLICT
  // ==================

  $.fn.button.noConflict = function () {
    $.fn.button = old
    return this
  }


  // BUTTON DATA-API
  // ===============

  $(document).on('click.bs.button.data-api', '[data-toggle^=button]', function (e) {
    var $btn = $(e.target)
    if (!$btn.hasClass('btn')) $btn = $btn.closest('.btn')
    $btn.button('toggle')
    e.preventDefault()
  })

}(jQuery);

/* ========================================================================
 * Bootstrap: carousel.js v3.1.1
 * http://getbootstrap.com/javascript/#carousel
 * ========================================================================
 * Copyright 2011-2014 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  

  // CAROUSEL CLASS DEFINITION
  // =========================

  var Carousel = function (element, options) {
    this.$element    = $(element)
    this.$indicators = this.$element.find('.carousel-indicators')
    this.options     = options
    this.paused      =
    this.sliding     =
    this.interval    =
    this.$active     =
    this.$items      = null

    this.options.pause == 'hover' && this.$element
      .on('mouseenter', $.proxy(this.pause, this))
      .on('mouseleave', $.proxy(this.cycle, this))
  }

  Carousel.DEFAULTS = {
    interval: 5000,
    pause: 'hover',
    wrap: true
  }

  Carousel.prototype.cycle =  function (e) {
    e || (this.paused = false)

    this.interval && clearInterval(this.interval)

    this.options.interval
      && !this.paused
      && (this.interval = setInterval($.proxy(this.next, this), this.options.interval))

    return this
  }

  Carousel.prototype.getActiveIndex = function () {
    this.$active = this.$element.find('.item.active')
    this.$items  = this.$active.parent().children()

    return this.$items.index(this.$active)
  }

  Carousel.prototype.to = function (pos) {
    var that        = this
    var activeIndex = this.getActiveIndex()

    if (pos > (this.$items.length - 1) || pos < 0) return

    if (this.sliding)       return this.$element.one('slid.bs.carousel', function () { that.to(pos) })
    if (activeIndex == pos) return this.pause().cycle()

    return this.slide(pos > activeIndex ? 'next' : 'prev', $(this.$items[pos]))
  }

  Carousel.prototype.pause = function (e) {
    e || (this.paused = true)

    if (this.$element.find('.next, .prev').length && $.support.transition) {
      this.$element.trigger($.support.transition.end)
      this.cycle(true)
    }

    this.interval = clearInterval(this.interval)

    return this
  }

  Carousel.prototype.next = function () {
    if (this.sliding) return
    return this.slide('next')
  }

  Carousel.prototype.prev = function () {
    if (this.sliding) return
    return this.slide('prev')
  }

  Carousel.prototype.slide = function (type, next) {
    var $active   = this.$element.find('.item.active')
    var $next     = next || $active[type]()
    var isCycling = this.interval
    var direction = type == 'next' ? 'left' : 'right'
    var fallback  = type == 'next' ? 'first' : 'last'
    var that      = this

    if (!$next.length) {
      if (!this.options.wrap) return
      $next = this.$element.find('.item')[fallback]()
    }

    if ($next.hasClass('active')) return this.sliding = false

    var e = $.Event('slide.bs.carousel', { relatedTarget: $next[0], direction: direction })
    this.$element.trigger(e)
    if (e.isDefaultPrevented()) return

    this.sliding = true

    isCycling && this.pause()

    if (this.$indicators.length) {
      this.$indicators.find('.active').removeClass('active')
      this.$element.one('slid.bs.carousel', function () {
        var $nextIndicator = $(that.$indicators.children()[that.getActiveIndex()])
        $nextIndicator && $nextIndicator.addClass('active')
      })
    }

    if ($.support.transition && this.$element.hasClass('slide')) {
      $next.addClass(type)
      $next[0].offsetWidth // force reflow
      $active.addClass(direction)
      $next.addClass(direction)
      $active
        .one($.support.transition.end, function () {
          $next.removeClass([type, direction].join(' ')).addClass('active')
          $active.removeClass(['active', direction].join(' '))
          that.sliding = false
          setTimeout(function () { that.$element.trigger('slid.bs.carousel') }, 0)
        })
        .emulateTransitionEnd($active.css('transition-duration').slice(0, -1) * 1000)
    } else {
      $active.removeClass('active')
      $next.addClass('active')
      this.sliding = false
      this.$element.trigger('slid.bs.carousel')
    }

    isCycling && this.cycle()

    return this
  }


  // CAROUSEL PLUGIN DEFINITION
  // ==========================

  var old = $.fn.carousel

  $.fn.carousel = function (option) {
    return this.each(function () {
      var $this   = $(this)
      var data    = $this.data('bs.carousel')
      var options = $.extend({}, Carousel.DEFAULTS, $this.data(), typeof option == 'object' && option)
      var action  = typeof option == 'string' ? option : options.slide

      if (!data) $this.data('bs.carousel', (data = new Carousel(this, options)))
      if (typeof option == 'number') data.to(option)
      else if (action) data[action]()
      else if (options.interval) data.pause().cycle()
    })
  }

  $.fn.carousel.Constructor = Carousel


  // CAROUSEL NO CONFLICT
  // ====================

  $.fn.carousel.noConflict = function () {
    $.fn.carousel = old
    return this
  }


  // CAROUSEL DATA-API
  // =================

  $(document).on('click.bs.carousel.data-api', '[data-slide], [data-slide-to]', function (e) {
    var $this   = $(this), href
    var $target = $($this.attr('data-target') || (href = $this.attr('href')) && href.replace(/.*(?=#[^\s]+$)/, '')) //strip for ie7
    var options = $.extend({}, $target.data(), $this.data())
    var slideIndex = $this.attr('data-slide-to')
    if (slideIndex) options.interval = false

    $target.carousel(options)

    if (slideIndex = $this.attr('data-slide-to')) {
      $target.data('bs.carousel').to(slideIndex)
    }

    e.preventDefault()
  })

  $(window).on('load', function () {
    $('[data-ride="carousel"]').each(function () {
      var $carousel = $(this)
      $carousel.carousel($carousel.data())
    })
  })

}(jQuery);

/* ========================================================================
 * Bootstrap: collapse.js v3.1.1
 * http://getbootstrap.com/javascript/#collapse
 * ========================================================================
 * Copyright 2011-2014 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  

  // COLLAPSE PUBLIC CLASS DEFINITION
  // ================================

  var Collapse = function (element, options) {
    this.$element      = $(element)
    this.options       = $.extend({}, Collapse.DEFAULTS, options)
    this.transitioning = null

    if (this.options.parent) this.$parent = $(this.options.parent)
    if (this.options.toggle) this.toggle()
  }

  Collapse.DEFAULTS = {
    toggle: true
  }

  Collapse.prototype.dimension = function () {
    var hasWidth = this.$element.hasClass('width')
    return hasWidth ? 'width' : 'height'
  }

  Collapse.prototype.show = function () {
    if (this.transitioning || this.$element.hasClass('in')) return

    var startEvent = $.Event('show.bs.collapse')
    this.$element.trigger(startEvent)
    if (startEvent.isDefaultPrevented()) return

    var actives = this.$parent && this.$parent.find('> .panel > .in')

    if (actives && actives.length) {
      var hasData = actives.data('bs.collapse')
      if (hasData && hasData.transitioning) return
      actives.collapse('hide')
      hasData || actives.data('bs.collapse', null)
    }

    var dimension = this.dimension()

    this.$element
      .removeClass('collapse')
      .addClass('collapsing')
      [dimension](0)

    this.transitioning = 1

    var complete = function () {
      this.$element
        .removeClass('collapsing')
        .addClass('collapse in')
        [dimension]('auto')
      this.transitioning = 0
      this.$element.trigger('shown.bs.collapse')
    }

    if (!$.support.transition) return complete.call(this)

    var scrollSize = $.camelCase(['scroll', dimension].join('-'))

    this.$element
      .one($.support.transition.end, $.proxy(complete, this))
      .emulateTransitionEnd(350)
      [dimension](this.$element[0][scrollSize])
  }

  Collapse.prototype.hide = function () {
    if (this.transitioning || !this.$element.hasClass('in')) return

    var startEvent = $.Event('hide.bs.collapse')
    this.$element.trigger(startEvent)
    if (startEvent.isDefaultPrevented()) return

    var dimension = this.dimension()

    this.$element
      [dimension](this.$element[dimension]())
      [0].offsetHeight

    this.$element
      .addClass('collapsing')
      .removeClass('collapse')
      .removeClass('in')

    this.transitioning = 1

    var complete = function () {
      this.transitioning = 0
      this.$element
        .trigger('hidden.bs.collapse')
        .removeClass('collapsing')
        .addClass('collapse')
    }

    if (!$.support.transition) return complete.call(this)

    this.$element
      [dimension](0)
      .one($.support.transition.end, $.proxy(complete, this))
      .emulateTransitionEnd(350)
  }

  Collapse.prototype.toggle = function () {
    this[this.$element.hasClass('in') ? 'hide' : 'show']()
  }


  // COLLAPSE PLUGIN DEFINITION
  // ==========================

  var old = $.fn.collapse

  $.fn.collapse = function (option) {
    return this.each(function () {
      var $this   = $(this)
      var data    = $this.data('bs.collapse')
      var options = $.extend({}, Collapse.DEFAULTS, $this.data(), typeof option == 'object' && option)

      if (!data && options.toggle && option == 'show') option = !option
      if (!data) $this.data('bs.collapse', (data = new Collapse(this, options)))
      if (typeof option == 'string') data[option]()
    })
  }

  $.fn.collapse.Constructor = Collapse


  // COLLAPSE NO CONFLICT
  // ====================

  $.fn.collapse.noConflict = function () {
    $.fn.collapse = old
    return this
  }


  // COLLAPSE DATA-API
  // =================

  $(document).on('click.bs.collapse.data-api', '[data-toggle=collapse]', function (e) {
    var $this   = $(this), href
    var target  = $this.attr('data-target')
        || e.preventDefault()
        || (href = $this.attr('href')) && href.replace(/.*(?=#[^\s]+$)/, '') //strip for ie7
    var $target = $(target)
    var data    = $target.data('bs.collapse')
    var option  = data ? 'toggle' : $this.data()
    var parent  = $this.attr('data-parent')
    var $parent = parent && $(parent)

    if (!data || !data.transitioning) {
      if ($parent) $parent.find('[data-toggle=collapse][data-parent="' + parent + '"]').not($this).addClass('collapsed')
      $this[$target.hasClass('in') ? 'addClass' : 'removeClass']('collapsed')
    }

    $target.collapse(option)
  })

}(jQuery);

/* ========================================================================
 * Bootstrap: dropdown.js v3.1.1
 * http://getbootstrap.com/javascript/#dropdowns
 * ========================================================================
 * Copyright 2011-2014 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  

  // DROPDOWN CLASS DEFINITION
  // =========================

  var backdrop = '.dropdown-backdrop'
  var toggle   = '[data-toggle=dropdown]'
  var Dropdown = function (element) {
    $(element).on('click.bs.dropdown', this.toggle)
  }

  Dropdown.prototype.toggle = function (e) {
    var $this = $(this)

    if ($this.is('.disabled, :disabled')) return

    var $parent  = getParent($this)
    var isActive = $parent.hasClass('open')

    clearMenus()

    if (!isActive) {
      if ('ontouchstart' in document.documentElement && !$parent.closest('.navbar-nav').length) {
        // if mobile we use a backdrop because click events don't delegate
        $('<div class="dropdown-backdrop"/>').insertAfter($(this)).on('click', clearMenus)
      }

      var relatedTarget = { relatedTarget: this }
      $parent.trigger(e = $.Event('show.bs.dropdown', relatedTarget))

      if (e.isDefaultPrevented()) return

      $parent
        .toggleClass('open')
        .trigger('shown.bs.dropdown', relatedTarget)

      $this.focus()
    }

    return false
  }

  Dropdown.prototype.keydown = function (e) {
    if (!/(38|40|27)/.test(e.keyCode)) return

    var $this = $(this)

    e.preventDefault()
    e.stopPropagation()

    if ($this.is('.disabled, :disabled')) return

    var $parent  = getParent($this)
    var isActive = $parent.hasClass('open')

    if (!isActive || (isActive && e.keyCode == 27)) {
      if (e.which == 27) $parent.find(toggle).focus()
      return $this.click()
    }

    var desc = ' li:not(.divider):visible a'
    var $items = $parent.find('[role=menu]' + desc + ', [role=listbox]' + desc)

    if (!$items.length) return

    var index = $items.index($items.filter(':focus'))

    if (e.keyCode == 38 && index > 0)                 index--                        // up
    if (e.keyCode == 40 && index < $items.length - 1) index++                        // down
    if (!~index)                                      index = 0

    $items.eq(index).focus()
  }

  function clearMenus(e) {
    $(backdrop).remove()
    $(toggle).each(function () {
      var $parent = getParent($(this))
      var relatedTarget = { relatedTarget: this }
      if (!$parent.hasClass('open')) return
      $parent.trigger(e = $.Event('hide.bs.dropdown', relatedTarget))
      if (e.isDefaultPrevented()) return
      $parent.removeClass('open').trigger('hidden.bs.dropdown', relatedTarget)
    })
  }

  function getParent($this) {
    var selector = $this.attr('data-target')

    if (!selector) {
      selector = $this.attr('href')
      selector = selector && /#[A-Za-z]/.test(selector) && selector.replace(/.*(?=#[^\s]*$)/, '') //strip for ie7
    }

    var $parent = selector && $(selector)

    return $parent && $parent.length ? $parent : $this.parent()
  }


  // DROPDOWN PLUGIN DEFINITION
  // ==========================

  var old = $.fn.dropdown

  $.fn.dropdown = function (option) {
    return this.each(function () {
      var $this = $(this)
      var data  = $this.data('bs.dropdown')

      if (!data) $this.data('bs.dropdown', (data = new Dropdown(this)))
      if (typeof option == 'string') data[option].call($this)
    })
  }

  $.fn.dropdown.Constructor = Dropdown


  // DROPDOWN NO CONFLICT
  // ====================

  $.fn.dropdown.noConflict = function () {
    $.fn.dropdown = old
    return this
  }


  // APPLY TO STANDARD DROPDOWN ELEMENTS
  // ===================================

  $(document)
    .on('click.bs.dropdown.data-api', clearMenus)
    .on('click.bs.dropdown.data-api', '.dropdown form', function (e) { e.stopPropagation() })
    .on('click.bs.dropdown.data-api', toggle, Dropdown.prototype.toggle)
    .on('keydown.bs.dropdown.data-api', toggle + ', [role=menu], [role=listbox]', Dropdown.prototype.keydown)

}(jQuery);

/* ========================================================================
 * Bootstrap: modal.js v3.1.1
 * http://getbootstrap.com/javascript/#modals
 * ========================================================================
 * Copyright 2011-2014 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  

  // MODAL CLASS DEFINITION
  // ======================

  var Modal = function (element, options) {
    this.options   = options
    this.$element  = $(element)
    this.$backdrop =
    this.isShown   = null

    if (this.options.remote) {
      this.$element
        .find('.modal-content')
        .load(this.options.remote, $.proxy(function () {
          this.$element.trigger('loaded.bs.modal')
        }, this))
    }
  }

  Modal.DEFAULTS = {
    backdrop: true,
    keyboard: true,
    show: true
  }

  Modal.prototype.toggle = function (_relatedTarget) {
    return this[!this.isShown ? 'show' : 'hide'](_relatedTarget)
  }

  Modal.prototype.show = function (_relatedTarget) {
    var that = this
    var e    = $.Event('show.bs.modal', { relatedTarget: _relatedTarget })

    this.$element.trigger(e)

    if (this.isShown || e.isDefaultPrevented()) return

    this.isShown = true

    this.escape()

    this.$element.on('click.dismiss.bs.modal', '[data-dismiss="modal"]', $.proxy(this.hide, this))

    this.backdrop(function () {
      var transition = $.support.transition && that.$element.hasClass('fade')

      if (!that.$element.parent().length) {
        that.$element.appendTo(document.body) // don't move modals dom position
      }

      that.$element
        .show()
        .scrollTop(0)

      if (transition) {
        that.$element[0].offsetWidth // force reflow
      }

      that.$element
        .addClass('in')
        .attr('aria-hidden', false)

      that.enforceFocus()

      var e = $.Event('shown.bs.modal', { relatedTarget: _relatedTarget })

      transition ?
        that.$element.find('.modal-dialog') // wait for modal to slide in
          .one($.support.transition.end, function () {
            that.$element.focus().trigger(e)
          })
          .emulateTransitionEnd(300) :
        that.$element.focus().trigger(e)
    })
  }

  Modal.prototype.hide = function (e) {
    if (e) e.preventDefault()

    e = $.Event('hide.bs.modal')

    this.$element.trigger(e)

    if (!this.isShown || e.isDefaultPrevented()) return

    this.isShown = false

    this.escape()

    $(document).off('focusin.bs.modal')

    this.$element
      .removeClass('in')
      .attr('aria-hidden', true)
      .off('click.dismiss.bs.modal')

    $.support.transition && this.$element.hasClass('fade') ?
      this.$element
        .one($.support.transition.end, $.proxy(this.hideModal, this))
        .emulateTransitionEnd(300) :
      this.hideModal()
  }

  Modal.prototype.enforceFocus = function () {
    $(document)
      .off('focusin.bs.modal') // guard against infinite focus loop
      .on('focusin.bs.modal', $.proxy(function (e) {
        if (this.$element[0] !== e.target && !this.$element.has(e.target).length) {
          this.$element.focus()
        }
      }, this))
  }

  Modal.prototype.escape = function () {
    if (this.isShown && this.options.keyboard) {
      this.$element.on('keyup.dismiss.bs.modal', $.proxy(function (e) {
        e.which == 27 && this.hide()
      }, this))
    } else if (!this.isShown) {
      this.$element.off('keyup.dismiss.bs.modal')
    }
  }

  Modal.prototype.hideModal = function () {
    var that = this
    this.$element.hide()
    this.backdrop(function () {
      that.removeBackdrop()
      that.$element.trigger('hidden.bs.modal')
    })
  }

  Modal.prototype.removeBackdrop = function () {
    this.$backdrop && this.$backdrop.remove()
    this.$backdrop = null
  }

  Modal.prototype.backdrop = function (callback) {
    var animate = this.$element.hasClass('fade') ? 'fade' : ''

    if (this.isShown && this.options.backdrop) {
      var doAnimate = $.support.transition && animate

      this.$backdrop = $('<div class="modal-backdrop ' + animate + '" />')
        .appendTo(document.body)

      this.$element.on('click.dismiss.bs.modal', $.proxy(function (e) {
        if (e.target !== e.currentTarget) return
        this.options.backdrop == 'static'
          ? this.$element[0].focus.call(this.$element[0])
          : this.hide.call(this)
      }, this))

      if (doAnimate) this.$backdrop[0].offsetWidth // force reflow

      this.$backdrop.addClass('in')

      if (!callback) return

      doAnimate ?
        this.$backdrop
          .one($.support.transition.end, callback)
          .emulateTransitionEnd(150) :
        callback()

    } else if (!this.isShown && this.$backdrop) {
      this.$backdrop.removeClass('in')

      $.support.transition && this.$element.hasClass('fade') ?
        this.$backdrop
          .one($.support.transition.end, callback)
          .emulateTransitionEnd(150) :
        callback()

    } else if (callback) {
      callback()
    }
  }


  // MODAL PLUGIN DEFINITION
  // =======================

  var old = $.fn.modal

  $.fn.modal = function (option, _relatedTarget) {
    return this.each(function () {
      var $this   = $(this)
      var data    = $this.data('bs.modal')
      var options = $.extend({}, Modal.DEFAULTS, $this.data(), typeof option == 'object' && option)

      if (!data) $this.data('bs.modal', (data = new Modal(this, options)))
      if (typeof option == 'string') data[option](_relatedTarget)
      else if (options.show) data.show(_relatedTarget)
    })
  }

  $.fn.modal.Constructor = Modal


  // MODAL NO CONFLICT
  // =================

  $.fn.modal.noConflict = function () {
    $.fn.modal = old
    return this
  }


  // MODAL DATA-API
  // ==============

  $(document).on('click.bs.modal.data-api', '[data-toggle="modal"]', function (e) {
    var $this   = $(this)
    var href    = $this.attr('href')
    var $target = $($this.attr('data-target') || (href && href.replace(/.*(?=#[^\s]+$)/, ''))) //strip for ie7
    var option  = $target.data('bs.modal') ? 'toggle' : $.extend({ remote: !/#/.test(href) && href }, $target.data(), $this.data())

    if ($this.is('a')) e.preventDefault()

    $target
      .modal(option, this)
      .one('hide', function () {
        $this.is(':visible') && $this.focus()
      })
  })

  $(document)
    .on('show.bs.modal', '.modal', function () { $(document.body).addClass('modal-open') })
    .on('hidden.bs.modal', '.modal', function () { $(document.body).removeClass('modal-open') })

}(jQuery);

/* ========================================================================
 * Bootstrap: tooltip.js v3.1.1
 * http://getbootstrap.com/javascript/#tooltip
 * Inspired by the original jQuery.tipsy by Jason Frame
 * ========================================================================
 * Copyright 2011-2014 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  

  // TOOLTIP PUBLIC CLASS DEFINITION
  // ===============================

  var Tooltip = function (element, options) {
    this.type       =
    this.options    =
    this.enabled    =
    this.timeout    =
    this.hoverState =
    this.$element   = null

    this.init('tooltip', element, options)
  }

  Tooltip.DEFAULTS = {
    animation: true,
    placement: 'top',
    selector: false,
    template: '<div class="tooltip"><div class="tooltip-arrow"></div><div class="tooltip-inner"></div></div>',
    trigger: 'hover focus',
    title: '',
    delay: 0,
    html: false,
    container: false
  }

  Tooltip.prototype.init = function (type, element, options) {
    this.enabled  = true
    this.type     = type
    this.$element = $(element)
    this.options  = this.getOptions(options)

    var triggers = this.options.trigger.split(' ')

    for (var i = triggers.length; i--;) {
      var trigger = triggers[i]

      if (trigger == 'click') {
        this.$element.on('click.' + this.type, this.options.selector, $.proxy(this.toggle, this))
      } else if (trigger != 'manual') {
        var eventIn  = trigger == 'hover' ? 'mouseenter' : 'focusin'
        var eventOut = trigger == 'hover' ? 'mouseleave' : 'focusout'

        this.$element.on(eventIn  + '.' + this.type, this.options.selector, $.proxy(this.enter, this))
        this.$element.on(eventOut + '.' + this.type, this.options.selector, $.proxy(this.leave, this))
      }
    }

    this.options.selector ?
      (this._options = $.extend({}, this.options, { trigger: 'manual', selector: '' })) :
      this.fixTitle()
  }

  Tooltip.prototype.getDefaults = function () {
    return Tooltip.DEFAULTS
  }

  Tooltip.prototype.getOptions = function (options) {
    options = $.extend({}, this.getDefaults(), this.$element.data(), options)

    if (options.delay && typeof options.delay == 'number') {
      options.delay = {
        show: options.delay,
        hide: options.delay
      }
    }

    return options
  }

  Tooltip.prototype.getDelegateOptions = function () {
    var options  = {}
    var defaults = this.getDefaults()

    this._options && $.each(this._options, function (key, value) {
      if (defaults[key] != value) options[key] = value
    })

    return options
  }

  Tooltip.prototype.enter = function (obj) {
    var self = obj instanceof this.constructor ?
      obj : $(obj.currentTarget)[this.type](this.getDelegateOptions()).data('bs.' + this.type)

    clearTimeout(self.timeout)

    self.hoverState = 'in'

    if (!self.options.delay || !self.options.delay.show) return self.show()

    self.timeout = setTimeout(function () {
      if (self.hoverState == 'in') self.show()
    }, self.options.delay.show)
  }

  Tooltip.prototype.leave = function (obj) {
    var self = obj instanceof this.constructor ?
      obj : $(obj.currentTarget)[this.type](this.getDelegateOptions()).data('bs.' + this.type)

    clearTimeout(self.timeout)

    self.hoverState = 'out'

    if (!self.options.delay || !self.options.delay.hide) return self.hide()

    self.timeout = setTimeout(function () {
      if (self.hoverState == 'out') self.hide()
    }, self.options.delay.hide)
  }

  Tooltip.prototype.show = function () {
    var e = $.Event('show.bs.' + this.type)

    if (this.hasContent() && this.enabled) {
      this.$element.trigger(e)

      if (e.isDefaultPrevented()) return
      var that = this;

      var $tip = this.tip()

      this.setContent()

      if (this.options.animation) $tip.addClass('fade')

      var placement = typeof this.options.placement == 'function' ?
        this.options.placement.call(this, $tip[0], this.$element[0]) :
        this.options.placement

      var autoToken = /\s?auto?\s?/i
      var autoPlace = autoToken.test(placement)
      if (autoPlace) placement = placement.replace(autoToken, '') || 'top'

      $tip
        .detach()
        .css({ top: 0, left: 0, display: 'block' })
        .addClass(placement)

      this.options.container ? $tip.appendTo(this.options.container) : $tip.insertAfter(this.$element)

      var pos          = this.getPosition()
      var actualWidth  = $tip[0].offsetWidth
      var actualHeight = $tip[0].offsetHeight

      if (autoPlace) {
        var $parent = this.$element.parent()

        var orgPlacement = placement
        var docScroll    = document.documentElement.scrollTop || document.body.scrollTop
        var parentWidth  = this.options.container == 'body' ? window.innerWidth  : $parent.outerWidth()
        var parentHeight = this.options.container == 'body' ? window.innerHeight : $parent.outerHeight()
        var parentLeft   = this.options.container == 'body' ? 0 : $parent.offset().left

        placement = placement == 'bottom' && pos.top   + pos.height  + actualHeight - docScroll > parentHeight  ? 'top'    :
                    placement == 'top'    && pos.top   - docScroll   - actualHeight < 0                         ? 'bottom' :
                    placement == 'right'  && pos.right + actualWidth > parentWidth                              ? 'left'   :
                    placement == 'left'   && pos.left  - actualWidth < parentLeft                               ? 'right'  :
                    placement

        $tip
          .removeClass(orgPlacement)
          .addClass(placement)
      }

      var calculatedOffset = this.getCalculatedOffset(placement, pos, actualWidth, actualHeight)

      this.applyPlacement(calculatedOffset, placement)
      this.hoverState = null

      var complete = function() {
        that.$element.trigger('shown.bs.' + that.type)
      }

      $.support.transition && this.$tip.hasClass('fade') ?
        $tip
          .one($.support.transition.end, complete)
          .emulateTransitionEnd(150) :
        complete()
    }
  }

  Tooltip.prototype.applyPlacement = function (offset, placement) {
    var replace
    var $tip   = this.tip()
    var width  = $tip[0].offsetWidth
    var height = $tip[0].offsetHeight

    // manually read margins because getBoundingClientRect includes difference
    var marginTop = parseInt($tip.css('margin-top'), 10)
    var marginLeft = parseInt($tip.css('margin-left'), 10)

    // we must check for NaN for ie 8/9
    if (isNaN(marginTop))  marginTop  = 0
    if (isNaN(marginLeft)) marginLeft = 0

    offset.top  = offset.top  + marginTop
    offset.left = offset.left + marginLeft

    // $.fn.offset doesn't round pixel values
    // so we use setOffset directly with our own function B-0
    $.offset.setOffset($tip[0], $.extend({
      using: function (props) {
        $tip.css({
          top: Math.round(props.top),
          left: Math.round(props.left)
        })
      }
    }, offset), 0)

    $tip.addClass('in')

    // check to see if placing tip in new offset caused the tip to resize itself
    var actualWidth  = $tip[0].offsetWidth
    var actualHeight = $tip[0].offsetHeight

    if (placement == 'top' && actualHeight != height) {
      replace = true
      offset.top = offset.top + height - actualHeight
    }

    if (/bottom|top/.test(placement)) {
      var delta = 0

      if (offset.left < 0) {
        delta       = offset.left * -2
        offset.left = 0

        $tip.offset(offset)

        actualWidth  = $tip[0].offsetWidth
        actualHeight = $tip[0].offsetHeight
      }

      this.replaceArrow(delta - width + actualWidth, actualWidth, 'left')
    } else {
      this.replaceArrow(actualHeight - height, actualHeight, 'top')
    }

    if (replace) $tip.offset(offset)
  }

  Tooltip.prototype.replaceArrow = function (delta, dimension, position) {
    this.arrow().css(position, delta ? (50 * (1 - delta / dimension) + '%') : '')
  }

  Tooltip.prototype.setContent = function () {
    var $tip  = this.tip()
    var title = this.getTitle()

    $tip.find('.tooltip-inner')[this.options.html ? 'html' : 'text'](title)
    $tip.removeClass('fade in top bottom left right')
  }

  Tooltip.prototype.hide = function () {
    var that = this
    var $tip = this.tip()
    var e    = $.Event('hide.bs.' + this.type)

    function complete() {
      if (that.hoverState != 'in') $tip.detach()
      that.$element.trigger('hidden.bs.' + that.type)
    }

    this.$element.trigger(e)

    if (e.isDefaultPrevented()) return

    $tip.removeClass('in')

    $.support.transition && this.$tip.hasClass('fade') ?
      $tip
        .one($.support.transition.end, complete)
        .emulateTransitionEnd(150) :
      complete()

    this.hoverState = null

    return this
  }

  Tooltip.prototype.fixTitle = function () {
    var $e = this.$element
    if ($e.attr('title') || typeof($e.attr('data-original-title')) != 'string') {
      $e.attr('data-original-title', $e.attr('title') || '').attr('title', '')
    }
  }

  Tooltip.prototype.hasContent = function () {
    return this.getTitle()
  }

  Tooltip.prototype.getPosition = function () {
    var el = this.$element[0]
    return $.extend({}, (typeof el.getBoundingClientRect == 'function') ? el.getBoundingClientRect() : {
      width: el.offsetWidth,
      height: el.offsetHeight
    }, this.$element.offset())
  }

  Tooltip.prototype.getCalculatedOffset = function (placement, pos, actualWidth, actualHeight) {
    return placement == 'bottom' ? { top: pos.top + pos.height,   left: pos.left + pos.width / 2 - actualWidth / 2  } :
           placement == 'top'    ? { top: pos.top - actualHeight, left: pos.left + pos.width / 2 - actualWidth / 2  } :
           placement == 'left'   ? { top: pos.top + pos.height / 2 - actualHeight / 2, left: pos.left - actualWidth } :
        /* placement == 'right' */ { top: pos.top + pos.height / 2 - actualHeight / 2, left: pos.left + pos.width   }
  }

  Tooltip.prototype.getTitle = function () {
    var title
    var $e = this.$element
    var o  = this.options

    title = $e.attr('data-original-title')
      || (typeof o.title == 'function' ? o.title.call($e[0]) :  o.title)

    return title
  }

  Tooltip.prototype.tip = function () {
    return this.$tip = this.$tip || $(this.options.template)
  }

  Tooltip.prototype.arrow = function () {
    return this.$arrow = this.$arrow || this.tip().find('.tooltip-arrow')
  }

  Tooltip.prototype.validate = function () {
    if (!this.$element[0].parentNode) {
      this.hide()
      this.$element = null
      this.options  = null
    }
  }

  Tooltip.prototype.enable = function () {
    this.enabled = true
  }

  Tooltip.prototype.disable = function () {
    this.enabled = false
  }

  Tooltip.prototype.toggleEnabled = function () {
    this.enabled = !this.enabled
  }

  Tooltip.prototype.toggle = function (e) {
    var self = e ? $(e.currentTarget)[this.type](this.getDelegateOptions()).data('bs.' + this.type) : this
    self.tip().hasClass('in') ? self.leave(self) : self.enter(self)
  }

  Tooltip.prototype.destroy = function () {
    clearTimeout(this.timeout)
    this.hide().$element.off('.' + this.type).removeData('bs.' + this.type)
  }


  // TOOLTIP PLUGIN DEFINITION
  // =========================

  var old = $.fn.tooltip

  $.fn.tooltip = function (option) {
    return this.each(function () {
      var $this   = $(this)
      var data    = $this.data('bs.tooltip')
      var options = typeof option == 'object' && option

      if (!data && option == 'destroy') return
      if (!data) $this.data('bs.tooltip', (data = new Tooltip(this, options)))
      if (typeof option == 'string') data[option]()
    })
  }

  $.fn.tooltip.Constructor = Tooltip


  // TOOLTIP NO CONFLICT
  // ===================

  $.fn.tooltip.noConflict = function () {
    $.fn.tooltip = old
    return this
  }

}(jQuery);

/* ========================================================================
 * Bootstrap: popover.js v3.1.1
 * http://getbootstrap.com/javascript/#popovers
 * ========================================================================
 * Copyright 2011-2014 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  

  // POPOVER PUBLIC CLASS DEFINITION
  // ===============================

  var Popover = function (element, options) {
    this.init('popover', element, options)
  }

  if (!$.fn.tooltip) throw new Error('Popover requires tooltip.js')

  Popover.DEFAULTS = $.extend({}, $.fn.tooltip.Constructor.DEFAULTS, {
    placement: 'right',
    trigger: 'click',
    content: '',
    template: '<div class="popover"><div class="arrow"></div><h3 class="popover-title"></h3><div class="popover-content"></div></div>'
  })


  // NOTE: POPOVER EXTENDS tooltip.js
  // ================================

  Popover.prototype = $.extend({}, $.fn.tooltip.Constructor.prototype)

  Popover.prototype.constructor = Popover

  Popover.prototype.getDefaults = function () {
    return Popover.DEFAULTS
  }

  Popover.prototype.setContent = function () {
    var $tip    = this.tip()
    var title   = this.getTitle()
    var content = this.getContent()

    $tip.find('.popover-title')[this.options.html ? 'html' : 'text'](title)
    $tip.find('.popover-content')[ // we use append for html objects to maintain js events
      this.options.html ? (typeof content == 'string' ? 'html' : 'append') : 'text'
    ](content)

    $tip.removeClass('fade top bottom left right in')

    // IE8 doesn't accept hiding via the `:empty` pseudo selector, we have to do
    // this manually by checking the contents.
    if (!$tip.find('.popover-title').html()) $tip.find('.popover-title').hide()
  }

  Popover.prototype.hasContent = function () {
    return this.getTitle() || this.getContent()
  }

  Popover.prototype.getContent = function () {
    var $e = this.$element
    var o  = this.options

    return $e.attr('data-content')
      || (typeof o.content == 'function' ?
            o.content.call($e[0]) :
            o.content)
  }

  Popover.prototype.arrow = function () {
    return this.$arrow = this.$arrow || this.tip().find('.arrow')
  }

  Popover.prototype.tip = function () {
    if (!this.$tip) this.$tip = $(this.options.template)
    return this.$tip
  }


  // POPOVER PLUGIN DEFINITION
  // =========================

  var old = $.fn.popover

  $.fn.popover = function (option) {
    return this.each(function () {
      var $this   = $(this)
      var data    = $this.data('bs.popover')
      var options = typeof option == 'object' && option

      if (!data && option == 'destroy') return
      if (!data) $this.data('bs.popover', (data = new Popover(this, options)))
      if (typeof option == 'string') data[option]()
    })
  }

  $.fn.popover.Constructor = Popover


  // POPOVER NO CONFLICT
  // ===================

  $.fn.popover.noConflict = function () {
    $.fn.popover = old
    return this
  }

}(jQuery);

/* ========================================================================
 * Bootstrap: scrollspy.js v3.1.1
 * http://getbootstrap.com/javascript/#scrollspy
 * ========================================================================
 * Copyright 2011-2014 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  

  // SCROLLSPY CLASS DEFINITION
  // ==========================

  function ScrollSpy(element, options) {
    var href
    var process  = $.proxy(this.process, this)

    this.$element       = $(element).is('body') ? $(window) : $(element)
    this.$body          = $('body')
    this.$scrollElement = this.$element.on('scroll.bs.scroll-spy.data-api', process)
    this.options        = $.extend({}, ScrollSpy.DEFAULTS, options)
    this.selector       = (this.options.target
      || ((href = $(element).attr('href')) && href.replace(/.*(?=#[^\s]+$)/, '')) //strip for ie7
      || '') + ' .nav li > a'
    this.offsets        = $([])
    this.targets        = $([])
    this.activeTarget   = null

    this.refresh()
    this.process()
  }

  ScrollSpy.DEFAULTS = {
    offset: 10
  }

  ScrollSpy.prototype.refresh = function () {
    var offsetMethod = this.$element[0] == window ? 'offset' : 'position'

    this.offsets = $([])
    this.targets = $([])

    var self     = this
    var $targets = this.$body
      .find(this.selector)
      .map(function () {
        var $el   = $(this)
        var href  = $el.data('target') || $el.attr('href')
        var $href = /^#./.test(href) && $(href)

        return ($href
          && $href.length
          && $href.is(':visible')
          && [[ $href[offsetMethod]().top + (!$.isWindow(self.$scrollElement.get(0)) && self.$scrollElement.scrollTop()), href ]]) || null
      })
      .sort(function (a, b) { return a[0] - b[0] })
      .each(function () {
        self.offsets.push(this[0])
        self.targets.push(this[1])
      })
  }

  ScrollSpy.prototype.process = function () {
    var scrollTop    = this.$scrollElement.scrollTop() + this.options.offset
    var scrollHeight = this.$scrollElement[0].scrollHeight || this.$body[0].scrollHeight
    var maxScroll    = scrollHeight - this.$scrollElement.height()
    var offsets      = this.offsets
    var targets      = this.targets
    var activeTarget = this.activeTarget
    var i

    if (scrollTop >= maxScroll) {
      return activeTarget != (i = targets.last()[0]) && this.activate(i)
    }

    if (activeTarget && scrollTop <= offsets[0]) {
      return activeTarget != (i = targets[0]) && this.activate(i)
    }

    for (i = offsets.length; i--;) {
      activeTarget != targets[i]
        && scrollTop >= offsets[i]
        && (!offsets[i + 1] || scrollTop <= offsets[i + 1])
        && this.activate( targets[i] )
    }
  }

  ScrollSpy.prototype.activate = function (target) {
    this.activeTarget = target

    $(this.selector)
      .parentsUntil(this.options.target, '.active')
      .removeClass('active')

    var selector = this.selector +
        '[data-target="' + target + '"],' +
        this.selector + '[href="' + target + '"]'

    var active = $(selector)
      .parents('li')
      .addClass('active')

    if (active.parent('.dropdown-menu').length) {
      active = active
        .closest('li.dropdown')
        .addClass('active')
    }

    active.trigger('activate.bs.scrollspy')
  }


  // SCROLLSPY PLUGIN DEFINITION
  // ===========================

  var old = $.fn.scrollspy

  $.fn.scrollspy = function (option) {
    return this.each(function () {
      var $this   = $(this)
      var data    = $this.data('bs.scrollspy')
      var options = typeof option == 'object' && option

      if (!data) $this.data('bs.scrollspy', (data = new ScrollSpy(this, options)))
      if (typeof option == 'string') data[option]()
    })
  }

  $.fn.scrollspy.Constructor = ScrollSpy


  // SCROLLSPY NO CONFLICT
  // =====================

  $.fn.scrollspy.noConflict = function () {
    $.fn.scrollspy = old
    return this
  }


  // SCROLLSPY DATA-API
  // ==================

  $(window).on('load', function () {
    $('[data-spy="scroll"]').each(function () {
      var $spy = $(this)
      $spy.scrollspy($spy.data())
    })
  })

}(jQuery);

/* ========================================================================
 * Bootstrap: tab.js v3.1.1
 * http://getbootstrap.com/javascript/#tabs
 * ========================================================================
 * Copyright 2011-2014 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  

  // TAB CLASS DEFINITION
  // ====================

  var Tab = function (element) {
    this.element = $(element)
  }

  Tab.prototype.show = function () {
    var $this    = this.element
    var $ul      = $this.closest('ul:not(.dropdown-menu)')
    var selector = $this.data('target')

    if (!selector) {
      selector = $this.attr('href')
      selector = selector && selector.replace(/.*(?=#[^\s]*$)/, '') //strip for ie7
    }

    if ($this.parent('li').hasClass('active')) return

    var previous = $ul.find('.active:last a')[0]
    var e        = $.Event('show.bs.tab', {
      relatedTarget: previous
    })

    $this.trigger(e)

    if (e.isDefaultPrevented()) return

    var $target = $(selector)

    this.activate($this.parent('li'), $ul)
    this.activate($target, $target.parent(), function () {
      $this.trigger({
        type: 'shown.bs.tab',
        relatedTarget: previous
      })
    })
  }

  Tab.prototype.activate = function (element, container, callback) {
    var $active    = container.find('> .active')
    var transition = callback
      && $.support.transition
      && $active.hasClass('fade')

    function next() {
      $active
        .removeClass('active')
        .find('> .dropdown-menu > .active')
        .removeClass('active')

      element.addClass('active')

      if (transition) {
        element[0].offsetWidth // reflow for transition
        element.addClass('in')
      } else {
        element.removeClass('fade')
      }

      if (element.parent('.dropdown-menu')) {
        element.closest('li.dropdown').addClass('active')
      }

      callback && callback()
    }

    transition ?
      $active
        .one($.support.transition.end, next)
        .emulateTransitionEnd(150) :
      next()

    $active.removeClass('in')
  }


  // TAB PLUGIN DEFINITION
  // =====================

  var old = $.fn.tab

  $.fn.tab = function ( option ) {
    return this.each(function () {
      var $this = $(this)
      var data  = $this.data('bs.tab')

      if (!data) $this.data('bs.tab', (data = new Tab(this)))
      if (typeof option == 'string') data[option]()
    })
  }

  $.fn.tab.Constructor = Tab


  // TAB NO CONFLICT
  // ===============

  $.fn.tab.noConflict = function () {
    $.fn.tab = old
    return this
  }


  // TAB DATA-API
  // ============

  $(document).on('click.bs.tab.data-api', '[data-toggle="tab"], [data-toggle="pill"]', function (e) {
    e.preventDefault()
    $(this).tab('show')
  })

}(jQuery);

/* ========================================================================
 * Bootstrap: affix.js v3.1.1
 * http://getbootstrap.com/javascript/#affix
 * ========================================================================
 * Copyright 2011-2014 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  

  // AFFIX CLASS DEFINITION
  // ======================

  var Affix = function (element, options) {
    this.options = $.extend({}, Affix.DEFAULTS, options)
    this.$window = $(window)
      .on('scroll.bs.affix.data-api', $.proxy(this.checkPosition, this))
      .on('click.bs.affix.data-api',  $.proxy(this.checkPositionWithEventLoop, this))

    this.$element     = $(element)
    this.affixed      =
    this.unpin        =
    this.pinnedOffset = null

    this.checkPosition()
  }

  Affix.RESET = 'affix affix-top affix-bottom'

  Affix.DEFAULTS = {
    offset: 0
  }

  Affix.prototype.getPinnedOffset = function () {
    if (this.pinnedOffset) return this.pinnedOffset
    this.$element.removeClass(Affix.RESET).addClass('affix')
    var scrollTop = this.$window.scrollTop()
    var position  = this.$element.offset()
    return (this.pinnedOffset = position.top - scrollTop)
  }

  Affix.prototype.checkPositionWithEventLoop = function () {
    setTimeout($.proxy(this.checkPosition, this), 1)
  }

  Affix.prototype.checkPosition = function () {
    if (!this.$element.is(':visible')) return

    var scrollHeight = $(document).height()
    var scrollTop    = this.$window.scrollTop()
    var position     = this.$element.offset()
    var offset       = this.options.offset
    var offsetTop    = offset.top
    var offsetBottom = offset.bottom

    if (this.affixed == 'top') position.top += scrollTop

    if (typeof offset != 'object')         offsetBottom = offsetTop = offset
    if (typeof offsetTop == 'function')    offsetTop    = offset.top(this.$element)
    if (typeof offsetBottom == 'function') offsetBottom = offset.bottom(this.$element)

    var affix = this.unpin   != null && (scrollTop + this.unpin <= position.top) ? false :
                offsetBottom != null && (position.top + this.$element.height() >= scrollHeight - offsetBottom) ? 'bottom' :
                offsetTop    != null && (scrollTop <= offsetTop) ? 'top' : false

    if (this.affixed === affix) return
    if (this.unpin) this.$element.css('top', '')

    var affixType = 'affix' + (affix ? '-' + affix : '')
    var e         = $.Event(affixType + '.bs.affix')

    this.$element.trigger(e)

    if (e.isDefaultPrevented()) return

    this.affixed = affix
    this.unpin = affix == 'bottom' ? this.getPinnedOffset() : null

    this.$element
      .removeClass(Affix.RESET)
      .addClass(affixType)
      .trigger($.Event(affixType.replace('affix', 'affixed')))

    if (affix == 'bottom') {
      this.$element.offset({ top: scrollHeight - offsetBottom - this.$element.height() })
    }
  }


  // AFFIX PLUGIN DEFINITION
  // =======================

  var old = $.fn.affix

  $.fn.affix = function (option) {
    return this.each(function () {
      var $this   = $(this)
      var data    = $this.data('bs.affix')
      var options = typeof option == 'object' && option

      if (!data) $this.data('bs.affix', (data = new Affix(this, options)))
      if (typeof option == 'string') data[option]()
    })
  }

  $.fn.affix.Constructor = Affix


  // AFFIX NO CONFLICT
  // =================

  $.fn.affix.noConflict = function () {
    $.fn.affix = old
    return this
  }


  // AFFIX DATA-API
  // ==============

  $(window).on('load', function () {
    $('[data-spy="affix"]').each(function () {
      var $spy = $(this)
      var data = $spy.data()

      data.offset = data.offset || {}

      if (data.offsetBottom) data.offset.bottom = data.offsetBottom
      if (data.offsetTop)    data.offset.top    = data.offsetTop

      $spy.affix(data)
    })
  })

}(jQuery);

define("bootstrap", ["jquery"], (function (global) {
    return function () {
        var ret, fn;
        return ret || global.Store;
    };
}(this)));

/*global define*/
define('views/settings',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/settings',
	'models/settings',
	'util/context',
	"i18n!nls/strings",
	'bootstrap',
], function ($, _, Backbone, BaseView, SettingsTemplate,SettingsModel,Context,Strings) {
	

	var SettingsView = BaseView.extend({

		titel : "Settings",
		
		// Instead of generating a new element, bind to the existing skeleton of
		// the App already present in the HTML.
		el: '#wmapp',

		// Compile our stats template
		template: SettingsTemplate,

		events: {
			"click #save-settings":"SaveSettings"
		},

		// At initialization we bind to the relevant events on the `Todos`
		// collection, when items are added or changed. Kick things off by
		// loading any preexisting todos that might be saved in *localStorage*.
		initialize: function () {
			this.$el.html(this.template($.extend({Strings:Strings},SettingsModel.toJSON())));
		},
		SaveSettings:function(){
			SettingsModel.set({
				server: this.$("#server").val().trim(),
				weinre: this.$("#weinre").is(':checked'),
				extendedError: this.$("#extendedError").is(':checked')
			});
			SettingsModel.save().done(function(){
				window.location.hash = '#login'
			})
			.fail(function(){
				//copy& paste from message helper to avoid cyclic reference
				$("#messages")
					.removeClass("alert-success")
					.addClass("alert-danger")
					.removeClass('hidden')
					.addClass("show")
					.text("Failed to save settings.");
			});
			
		},
		
	});

	return SettingsView;
});

define("hgn!templates/home", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class=\"btn-group btn-group-lg btn-group-justified sub-header-nav\">\r");t.b("\n" + i);t.b("  <div class=\"btn-group\">\r");t.b("\n" + i);t.b("    <button type=\"button\" id=\"home-overview\" data-tab-id=\"overview\" class=\"btn head-btn active\">");t.b(t.v(t.d("Strings.COMMON.OVERVIEW",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("  <div class=\"btn-group\">\r");t.b("\n" + i);t.b("    <button type=\"button\" id=\"home-ranking\" data-tab-id=\"ranking\" class=\"btn head-btn\">");t.b(t.v(t.d("Strings.COMMON.RANKING",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("<div id=\"subcontainer\" class='sub-container'>\r");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});


define("hgn!templates/homeoverview", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");if(t.s(t.f("hasCountdown",c,p,1),c,p,0,17,126,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("    <div class=\"col-xs-12\">\r");t.b("\n" + i);t.b("        <div class=\"wmCountdown countdown content-center\"></div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);});c.pop();}t.b("\r");t.b("\n" + i);if(t.s(t.f("havePastResults",c,p,1),c,p,0,167,810,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("    <div class=\"col-xs-12 col-sm-6\">\r");t.b("\n" + i);t.b("        <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("            <div class=\"panel-heading panel-heading-light clickable-heading\">\r");t.b("\n" + i);t.b("                <a href=\"#matches/second\"><h3 class=\"panel-title panel-title-light clearfix\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.LAST_MATCHDAY",c,p,0)));t.b(" <span class=\"float-right wmicon icon-right-open-big\"></span></h3></a>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("            <ul class=\"list-group\">\r");t.b("\n" + i);if(t.s(t.f("results",c,p,1),c,p,0,617,749,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                    <li class=\"list-group-item\">\r");t.b("\n" + i);t.b(t.rp("<match0",c,p,"                        "));t.b("                    </li>\r");t.b("\n" + i);});c.pop();}t.b("            </ul>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("havePastResults",c,p,1),c,p,1,0,0,"")){t.b("    <div class=\"col-xs-12 col-sm-6\">\r");t.b("\n" + i);t.b("        <div class=\"panel-title padding-bottom-10\">\r");t.b("\n" + i);t.b("            ");t.b(t.v(t.d("Strings.HOMEOVERVIEW.HOME_WELCOME_TEXT",c,p,0)));t.b("\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("        <div class=\"row\">\r");t.b("\n" + i);t.b("            <div class=\"col-xs-4\">\r");t.b("\n" + i);t.b("                ");t.b(t.v(t.d("Strings.HOMEOVERVIEW.MATCHES_TEXT",c,p,0)));t.b("\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("            <div class=\"col-xs-4\">\r");t.b("\n" + i);t.b("                ");t.b(t.v(t.d("Strings.HOMEOVERVIEW.BONUSTIPS_TEXT",c,p,0)));t.b("\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("            <div class=\"col-xs-4\">\r");t.b("\n" + i);t.b("                ");t.b(t.v(t.d("Strings.HOMEOVERVIEW.GROUPS_TEXT",c,p,0)));t.b("\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("        <div class=\"row padding-bottom-20\">\r");t.b("\n" + i);t.b("            <div class=\"col-xs-4\">\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"matches\" class=\"btn btn-primary text-wrap btn-width100 font-light padding-right-0 padding-left-0\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.MATCHES",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("            <div class=\"col-xs-4\">\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"bonusTips\" class=\"btn btn-primary text-wrap btn-width100 font-light padding-right-0 padding-left-0\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.BONUSTIPS",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("            <div class=\"col-xs-4\">\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"groups\" class=\"btn btn-primary text-wrap btn-width100 font-light padding-right-0 padding-left-0\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.GROUPS",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <!--\r");t.b("\n" + i);t.b("    <div class=\"col-xs-12\">\r");t.b("\n" + i);t.b("        ");t.b(t.v(t.d("Strings.HOMEOVERVIEW.RANKINGS_TEXT",c,p,0)));t.b("\r");t.b("\n" + i);t.b("        <button type=\"button\" id=\"rankings\" data-tab-id=\"ranking\" class=\"btn btn-primary btn-width100\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.RANKINGS",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div>&nbsp;<br/></div>\r");t.b("\n" + i);t.b("    -->\r");t.b("\n" + i);t.b("    <!--\r");t.b("\n" + i);t.b("    <div class=\"col-xs-12 col-md-6\">\r");t.b("\n" + i);t.b("        <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("            <div class=\"panel-heading clickable-heading\">\r");t.b("\n" + i);t.b("                <h3 class=\"panel-title clearfix\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.NO_MATCH",c,p,0)));t.b("</h3>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    -->\r");t.b("\n" + i);};if(t.s(t.f("haveNextMatches",c,p,1),c,p,0,2807,3855,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("    <!--\r");t.b("\n" + i);t.b("        Don‘t miss setting your tips\r");t.b("\n" + i);t.b("    -->\r");t.b("\n" + i);t.b("    <div class=\"col-xs-12 col-sm-6\">\r");t.b("\n" + i);t.b("        <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("            <div class=\"panel-heading clickable-heading panel-heading-light\">\r");t.b("\n" + i);t.b("                <a href=\"#matches/second\"><h3 class=\"panel-title panel-title-light clearfix\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.NEXT_MATCHES",c,p,0)));t.b(" <span class=\"float-right wmicon icon-right-open-big\"></span></h3></a>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("            <ul class=\"list-group\">\r");t.b("\n" + i);if(t.s(t.f("tips",c,p,1),c,p,0,3310,3442,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                    <li class=\"list-group-item\">\r");t.b("\n" + i);t.b(t.rp("<match1",c,p,"                        "));t.b("                    </li>\r");t.b("\n" + i);});c.pop();}t.b("            </ul>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);if(t.s(t.f("canSetBonusTips",c,p,1),c,p,0,3516,3821,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("        <div class=\"row padding-bottom-20\">\r");t.b("\n" + i);t.b("            <div class=\"col-xs-12\">\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"bonusTips\" class=\"btn btn-default btn-lg text-wrap btn-width100 font-light\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.DO_NOT_FORGET_BONUSTIPS",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);});c.pop();}t.b("    </div>\r");t.b("\n" + i);});c.pop();}t.b("\r");t.b("\n" + i);if(!t.s(t.f("haveNextMatches",c,p,1),c,p,1,0,0,"")){t.b("    <div class=\"col-xs-12 col-sm-6\">\r");t.b("\n" + i);t.b("        <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("            <div class=\"panel-heading panel-heading-light\">\r");t.b("\n" + i);t.b("                <h3 class=\"panel-title panel-title-light clearfix\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.FW_PERSONAL_STATS",c,p,0)));t.b("</h3>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("            \r");t.b("\n" + i);t.b("            <div class=\"panel-body font-light\">\r");t.b("\n" + i);t.b("                <div class=\"panel-body content-center\">\r");t.b("\n" + i);t.b("                    <div class=\"font-black font-h2 font-color-primary\">");t.b(t.v(t.f("overallPoints",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                    <div class=\"color-grey-profile font-size-base font-light\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.OVERALL_POINTS",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("                <div class=\"col-xs-4\">\r");t.b("\n" + i);t.b("                    <div class=\"row\">\r");t.b("\n" + i);t.b("                        <span class=\"wmicon icon-dot-3 font-26 color-grey-profile padding-right-5 pull-left\" />\r");t.b("\n" + i);t.b("                        <div class=\"font-h4 font-medium\">");t.b(t.v(t.f("matchPoints",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                    </div>\r");t.b("\n" + i);t.b("                    <div class=\"row\">\r");t.b("\n" + i);t.b("                        <div class=\"font-s font-condensed font-light color-grey-profile\">");t.b(t.v(t.d("Strings.PLAYER_DETAILS.MATCHTIP_POINTS",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                    </div>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("                <div class=\"col-xs-4\">\r");t.b("\n" + i);t.b("                    <div class=\"row\">\r");t.b("\n" + i);t.b("                        <span class=\"wmicon icon-award-1 font-26 color-grey-profile padding-right-5 pull-left\" />\r");t.b("\n" + i);t.b("                        <div class=\"font-h4 font-medium\">");t.b(t.v(t.f("bonusPoints",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                    </div>\r");t.b("\n" + i);t.b("                    <div class=\"row\">\r");t.b("\n" + i);t.b("                        <div class=\"font-s font-condensed font-light color-grey-profile\">");t.b(t.v(t.d("Strings.PLAYER_DETAILS.BONUSTIP_POINTS",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                    </div>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("                <div class=\"col-xs-4\">\r");t.b("\n" + i);t.b("                    <div class=\"row\">\r");t.b("\n" + i);t.b("                        <span class=\"glyphicon glyphicon-sort-by-attributes font-26 color-grey-profile padding-right-5 pull-left\" />\r");t.b("\n" + i);t.b("                        <div class=\"font-h4 font-medium\">");t.b(t.v(t.f("overallRank",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                    </div>\r");t.b("\n" + i);t.b("                    <div class=\"row\">\r");t.b("\n" + i);t.b("                        <div class=\"font-s font-condensed font-light color-grey-profile\">");t.b(t.v(t.d("Strings.PLAYER_DETAILS.GLOBAL_RANK",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                    </div>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("            \r");t.b("\n" + i);t.b("            <div class=\"panel-body\">\r");t.b("\n" + i);t.b("                <div class=\"col-xs-4 padding-right-10 padding-left-0\">\r");t.b("\n" + i);t.b("                    <button type=\"button\" id=\"matches\" class=\"btn btn-primary text-wrap btn-width100 font-light padding-right-0 padding-left-0\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.MATCHES_AFTER",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("                <div class=\"col-xs-4 padding-right-10 padding-left-0\">\r");t.b("\n" + i);t.b("                    <button type=\"button\" id=\"bonusTips\" class=\"btn btn-primary text-wrap btn-width100 font-light padding-right-0 padding-left-0\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.BONUSTIPS_AFTER",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("                <div class=\"col-xs-4 padding-right-0 padding-left-0\">\r");t.b("\n" + i);t.b("                    <button type=\"button\" id=\"rankings\" class=\"btn btn-primary text-wrap btn-width100 font-light padding-right-0 padding-left-0\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.RANKINGS_AFTER",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("            \r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("    <div class=\"col-xs-12 col-sm-6\">\r");t.b("\n" + i);t.b("        <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("            <div class=\"panel-heading panel-heading-light\">\r");t.b("\n" + i);t.b("                <h3 class=\"panel-title panel-title-light clearfix\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.HOME_FAREWELL",c,p,0)));t.b("</h3>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("            <div class=\"panel-body\">\r");t.b("\n" + i);t.b("                <!--<div class=\"list-group font-bold content-center font-h4\">\r");t.b("\n" + i);t.b("                    {Germany}\r");t.b("\n" + i);t.b("                </div>-->\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("                <div class=\"list-group font-light font-h4\">\r");t.b("\n" + i);t.b("                    ");t.b(t.v(t.d("Strings.HOMEOVERVIEW.FW_NEW_WC",c,p,0)));t.b("\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("                <div class=\"list-group font-light\">\r");t.b("\n" + i);t.b("                    ");t.b(t.v(t.d("Strings.HOMEOVERVIEW.FEEDBACK_TEXT",c,p,0)));t.b("\r");t.b("\n" + i);t.b("                    <br></br>\r");t.b("\n" + i);t.b("                    <button type=\"button\" id=\"feedback\" class=\"btn btn-primary btn-width100  text-wrap font-light\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.FEEDBACK",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("    <div class=\"col-xs-12 col-sm-6\">\r");t.b("\n" + i);t.b("        <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("            <div class=\"panel-heading panel-heading-light\">\r");t.b("\n" + i);t.b("                <h3 class=\"panel-title panel-title-light clearfix\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.GLOBAL_STATS",c,p,0)));t.b("</h3>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("            \r");t.b("\n" + i);t.b("            <div class=\"panel-body content-center\">\r");t.b("\n" + i);t.b("                 <div class=\"font-black font-h2 font-color-primary\" id=\"CountPlayerOverall\"></div>\r");t.b("\n" + i);t.b("                 <div class=\"color-grey-profile font-size-base font-light\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.COUNT_PLAYER_OVERALL",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("            \r");t.b("\n" + i);t.b("            <div class=\"panel-body content-center\">\r");t.b("\n" + i);t.b("                <div class=\"col-xs-4 padding-right-0 padding-left-0\">\r");t.b("\n" + i);t.b("                    <div class=\"font-bold font-h3 font-color-primary\" id=\"CountCountriesOverall\"></div>\r");t.b("\n" + i);t.b("                    <div class=\"color-grey-profile font-size-base font-light\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.COUNT_COUNTRIES_OVERALL",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("                <div class=\"col-xs-4 padding-right-0 padding-left-0\">\r");t.b("\n" + i);t.b("                    <div class=\"font-bold font-h3 font-color-primary\" id=\"CountGroupsOverall\"></div>\r");t.b("\n" + i);t.b("                    <div class=\"color-grey-profile font-size-base font-light\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.COUNT_GROUPS_OVERALL",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("                <div class=\"col-xs-4 padding-right-0 padding-left-0\">\r");t.b("\n" + i);t.b("                    <div class=\"font-bold font-h3 font-color-primary\" id=\"CountMatchtipOverall\"></div>\r");t.b("\n" + i);t.b("                    <div class=\"color-grey-profile font-size-base font-light\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.COUNT_MATCHTIPS_OVERALL",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("            \r");t.b("\n" + i);t.b("            <div class=\"panel-body\">\r");t.b("\n" + i);t.b("                <div class=\"row padding-bottom-5\">\r");t.b("\n" + i);t.b("                    <div class=\"col-xs-4 content-left font-h4 font-light\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.COMMUNITY",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                    <div class=\"col-xs-4 content-center font-h4 font-light\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.VS",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                    <div class=\"col-xs-4 content-right font-h4 font-light\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.YOU",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("                <div class=\"\">\r");t.b("\n" + i);t.b("                    <canvas id=\"statsCommunity\" width=\"130\" height=\"130\" style=\"float:left\"></canvas>\r");t.b("\n" + i);t.b("                    <canvas id=\"statsPlayer\" width=\"130\" height=\"130\" style=\"float:right\"></canvas>\r");t.b("\n" + i);t.b("                    <div class=\"clearfix\"></div>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("                <div class=\"row\">\r");t.b("\n" + i);t.b("                    <div class=\"col-xs-2 content-left font-20 font-bold font-color-primary\" id=\"points5overall\"></div>\r");t.b("\n" + i);t.b("                    <div class=\"col-xs-8 content-center font-14 font-light color-grey-profile\">");t.b(t.v(t.d("Strings.MATCHTIPS.POINTS_5",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                    <div class=\"col-xs-2 content-right font-20 font-bold font-color-primary\" id=\"points5player\"></div>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("                <div class=\"row\">\r");t.b("\n" + i);t.b("                    <div class=\"col-xs-2 content-left font-20 font-bold font-color-secondary\" id=\"points3overall\"></div>\r");t.b("\n" + i);t.b("                    <div class=\"col-xs-8 content-center font-14 font-light color-grey-profile\">");t.b(t.v(t.d("Strings.MATCHTIPS.POINTS_3",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                    <div class=\"col-xs-2 content-right font-20 font-bold font-color-secondary\" id=\"points3player\"></div>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("                <div class=\"row\">\r");t.b("\n" + i);t.b("                    <div class=\"col-xs-2 content-left font-20 font-bold color-gray\" id=\"points1overall\"></div>\r");t.b("\n" + i);t.b("                    <div class=\"col-xs-8 content-center font-14 font-light color-grey-profile\">");t.b(t.v(t.d("Strings.MATCHTIPS.POINTS_1",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                    <div class=\"col-xs-2 content-right font-20 font-bold color-gray\" id=\"points1player\"></div>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("                <div class=\"row\">\r");t.b("\n" + i);t.b("                    <div class=\"col-xs-2 content-left font-20 font-bold color-white \" id=\"points0overall\"></div>\r");t.b("\n" + i);t.b("                    <div class=\"col-xs-8 content-center font-14 font-light color-grey-profile\">");t.b(t.v(t.d("Strings.MATCHTIPS.POINTS_0",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                    <div class=\"col-xs-2 content-right font-20 font-bold color-white \" id=\"points0player\"></div>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("    <div class=\"col-xs-12 col-sm-6\">\r");t.b("\n" + i);t.b("        <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("            <div class=\"panel-heading panel-heading-light\">\r");t.b("\n" + i);t.b("                <h3 class=\"panel-title panel-title-light clearfix\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.PREDICTION_ACCURACY",c,p,0)));t.b("</h3>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("            <ul class=\"list-group\">\r");t.b("\n" + i);t.b("                <li class=\"list-group-item\">\r");t.b("\n" + i);t.b("                    <div class=\"\">\r");t.b("\n" + i);t.b("                        <div class=\"row padding-bottom-5\">\r");t.b("\n" + i);t.b("                            <div class=\"col-xs-12 content-left font-h4 font-light\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.BEST_MATCH",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                        </div>\r");t.b("\n" + i);t.b("                        <div class=\"row\">\r");t.b("\n" + i);t.b("                            <div class=\"col-xs-6 content-left font-14\" id=\"BestCountryNameA\">CountryNameA</div>\r");t.b("\n" + i);t.b("                            <div class=\"col-xs-6 content-right font-14\" id=\"BestCountryNameB\">CountryNameB</div>\r");t.b("\n" + i);t.b("                        </div>\r");t.b("\n" + i);t.b("                        <div class=\"row\">\r");t.b("\n" + i);t.b("                            <div class=\"col-xs-3 content-left padding-top-5\"><img id=\"BestCountryNameAFlag\" src=\"");t.b(t.v(t.f("CountryFlagA",c,p,0)));t.b("\" class=\"img-flag\" alt=\"");t.b(t.v(t.f("CountryNameA",c,p,0)));t.b("\" /></div>\r");t.b("\n" + i);t.b("                            <div class=\"col-xs-6 content-center\">\r");t.b("\n" + i);t.b("                                <div class=\"font-bold font-h3 font-color-primary\" id=\"matchtipBestQuote\"></div>\r");t.b("\n" + i);t.b("                                <div class=\"color-grey-profile font-size-base font-light\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.BEST_QUOTE",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                            </div>\r");t.b("\n" + i);t.b("                            <div class=\"col-xs-3 content-right padding-top-5\"><img id=\"BestCountryNameBFlag\" src=\"");t.b(t.v(t.f("CountryFlagB",c,p,0)));t.b("\" class=\"img-flag\" alt=\"");t.b(t.v(t.f("CountryNameB",c,p,0)));t.b("\" /></div>\r");t.b("\n" + i);t.b("                        </div>\r");t.b("\n" + i);t.b("                    </div>\r");t.b("\n" + i);t.b("                </li>\r");t.b("\n" + i);t.b("                <li class=\"list-group-item\">\r");t.b("\n" + i);t.b("                    <div class=\"\">\r");t.b("\n" + i);t.b("                        <div class=\"row padding-bottom-5\">\r");t.b("\n" + i);t.b("                            <div class=\"col-xs-12 content-left font-h4 font-light\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.WORST_MATCH",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                        </div>\r");t.b("\n" + i);t.b("                        <div class=\"row\">\r");t.b("\n" + i);t.b("                            <div class=\"col-xs-6 content-left font-14\" id=\"WorstCountryNameA\">CountryNameA</div>\r");t.b("\n" + i);t.b("                            <div class=\"col-xs-6 content-right font-14\" id=\"WorstCountryNameB\">CountryNameB</div>\r");t.b("\n" + i);t.b("                        </div>\r");t.b("\n" + i);t.b("                        <div class=\"row\">\r");t.b("\n" + i);t.b("                            <div class=\"col-xs-3 content-left padding-top-5\"><img id=\"WorstCountryNameAFlag\" src=\"");t.b(t.v(t.f("CountryFlagA",c,p,0)));t.b("\" class=\"img-flag\" alt=\"");t.b(t.v(t.f("CountryNameA",c,p,0)));t.b("\" /></div>\r");t.b("\n" + i);t.b("                            <div class=\"col-xs-6 content-center\">\r");t.b("\n" + i);t.b("                                <div class=\"font-bold font-h3 font-color-danger\" id=\"matchtipWorstQuote\"></div>\r");t.b("\n" + i);t.b("                                <div class=\"color-grey-profile font-size-base font-light\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.WORST_QUOTE",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                            </div>\r");t.b("\n" + i);t.b("                            <div class=\"col-xs-3 content-right padding-top-5\"><img id=\"WorstCountryNameBFlag\" src=\"");t.b(t.v(t.f("CountryFlagB",c,p,0)));t.b("\" class=\"img-flag\" alt=\"");t.b(t.v(t.f("CountryNameB",c,p,0)));t.b("\" /></div>\r");t.b("\n" + i);t.b("                        </div>\r");t.b("\n" + i);t.b("                    </div>\r");t.b("\n" + i);t.b("                </li>\r");t.b("\n" + i);t.b("                <li class=\"list-group-item\">\r");t.b("\n" + i);t.b("                    <div class=\"\">\r");t.b("\n" + i);t.b("                        <div class=\"row padding-bottom-5\">\r");t.b("\n" + i);t.b("                            <div class=\"col-xs-4 content-left font-h4 font-light\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.COMMUNITY",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                            <div class=\"col-xs-4 content-center font-h4 font-light\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.VS",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                            <div class=\"col-xs-4 content-right font-h4 font-light\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.YOU",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                        </div>\r");t.b("\n" + i);t.b("                        <div class=\"row\">\r");t.b("\n" + i);t.b("                            <div class=\"col-xs-2 content-left font-20 font-bold font-color-primary\" id=\"avgMatchtipOverall\"></div>\r");t.b("\n" + i);t.b("                            <div class=\"col-xs-8 content-center font-14 font-light color-grey-profile\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.AVG_MATCHTIP_POINTS",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                            <div class=\"col-xs-2 content-right font-20 font-bold font-color-primary\" id=\"avgMatchtipPlayer\"></div>\r");t.b("\n" + i);t.b("                        </div>\r");t.b("\n" + i);t.b("                        <div class=\"row\">\r");t.b("\n" + i);t.b("                            <div class=\"col-xs-2 content-left font-20 font-bold font-color-secondary\" id=\"avgBonustipOverall\"></div>\r");t.b("\n" + i);t.b("                            <div class=\"col-xs-8 content-center font-14 font-light color-grey-profile\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.AVG_BONUSTIP_POINTS",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                            <div class=\"col-xs-2 content-right font-20 font-bold font-color-secondary\" id=\"avgBonustipPlayer\"></div>\r");t.b("\n" + i);t.b("                        </div>\r");t.b("\n" + i);t.b("                    </div>\r");t.b("\n" + i);t.b("                </li>\r");t.b("\n" + i);t.b("            </ul>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);};return t.fl(); },partials: {"<match0":{name:"match", partials: {}, subs: {  }},"<match1":{name:"match", partials: {}, subs: {  }}}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});


define("hgn!templates/homeranking", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("ranking\r");t.b("\n");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});


define("hgn!templates/matchsmall", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class=\"match-tip\"  data-match-id=\"");t.b(t.v(t.f("id",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("    <div class=\"row\">\r");t.b("\n" + i);t.b("        <div class=\"col-xs-6 content-left font-16\">");t.b(t.v(t.f("CountryNameA",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("        <div class=\"col-xs-6 content-right font-16\">");t.b(t.v(t.f("CountryNameB",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"row\">\r");t.b("\n" + i);t.b("        <div class=\"col-xs-3 content-left padding-top-5\"><img src=\"");t.b(t.v(t.f("CountryFlagA",c,p,0)));t.b("\" class=\"img-flag\" alt=\"");t.b(t.v(t.f("CountryNameA",c,p,0)));t.b("\" /></div>\r");t.b("\n" + i);t.b("        <div class=\"col-xs-6 content-center\">\r");t.b("\n" + i);t.b("            ");if(t.s(t.f("HaveResults",c,p,1),c,p,0,468,1188,"{{ }}")){t.rs(c,p,function(c,p,t){t.b(" <!-- Results -->\r");t.b("\n" + i);t.b("                <div class=\"match-goal-results\">");t.b(t.v(t.f("scoreA",c,p,0)));t.b(":");t.b(t.v(t.f("scoreB",c,p,0)));if(t.s(t.f("extention",c,p,1),c,p,0,570,591,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("&nbsp;(");t.b(t.v(t.f("extention",c,p,0)));t.b(")");});c.pop();}t.b("<!-- <span class=\"wmicon icon-soccer pull-left\"></span>--></div>\r");t.b("\n" + i);if(t.s(t.f("Tip",c,p,1),c,p,0,695,984,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                    <div class=\"match-goal-tip font-digital\">");t.b(t.v(t.f("scoreA",c,p,0)));t.b(":");t.b(t.v(t.f("scoreB",c,p,0)));t.b("<!-- <span class=\"wmicon icon-exchange pull-left\"></span>--></div>\r");t.b("\n" + i);t.b("                    <div class=\"\"><span class=\"badge badge-bg-primary\">");t.b(t.v(t.f("points",c,p,0)));t.b(" ");t.b(t.v(t.d("Strings.MATCH.POINTS",c,p,0)));t.b("</span></div>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("Tip",c,p,1),c,p,1,0,0,"")){t.b("                    <div class=\"\">");t.b(t.v(t.d("Strings.MATCH.NOT_TIPPED",c,p,0)));t.b("<!-- <span class=\"wmicon icon-exchange pull-left\"></span>--></div>\r");t.b("\n" + i);};});c.pop();}t.b("\r");t.b("\n" + i);t.b("            ");if(!t.s(t.f("HaveResults",c,p,1),c,p,1,0,0,"")){t.b(" <!-- No Results -->\r");t.b("\n" + i);t.b("                ");if(t.s(t.f("CanCreateTips",c,p,1),c,p,0,1294,2013,"{{ }}")){t.rs(c,p,function(c,p,t){t.b(" <!-- Yes -->\r");t.b("\n" + i);if(t.s(t.f("Tip",c,p,1),c,p,0,1337,1739,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                        <div class=\"match-goal-results\">- : -<!-- <span class=\"wmicon icon-soccer pull-left\"></span>--></div>\r");t.b("\n" + i);t.b("                        <div><button type=\"button\" data-match-id=\"");t.b(t.v(t.f("id",c,p,0)));t.b("\" class=\"create-tip btn btn-default active match-goal-results font-digital\">");t.b(t.v(t.f("scoreA",c,p,0)));t.b("\r");t.b("\n" + i);t.b("						<span class=\"font-roboto font-bold font-26\">:</span>\r");t.b("\n" + i);t.b("						");t.b(t.v(t.f("scoreB",c,p,0)));t.b("</button></div>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("Tip",c,p,1),c,p,1,0,0,"")){t.b("                        <button type=\"button\" class=\"btn btn-default active match-goal-results\">");t.b(t.v(t.d("Strings.MATCH.TIP_NOW",c,p,0)));t.b("<!-- <span class=\"wmicon icon-edit pull-left\"></span>--></button>\r");t.b("\n" + i);};});c.pop();}t.b("                ");if(!t.s(t.f("CanCreateTips",c,p,1),c,p,1,0,0,"")){t.b(" <!-- No -->\r");t.b("\n" + i);if(t.s(t.f("Tip",c,p,1),c,p,0,2109,2567,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                        <div class=\"match-goal-results\">- : -<!-- <span class=\"wmicon icon-soccer pull-left\"></span>--></div>\r");t.b("\n" + i);t.b("                        <div class=\"match-goal-tip font-digital\">");t.b(t.v(t.f("scoreA",c,p,0)));t.b("\r");t.b("\n" + i);t.b("						<span class=\"font-roboto font-bold font-26\">:</span>\r");t.b("\n" + i);t.b("						");t.b(t.v(t.f("scoreB",c,p,0)));t.b("<!-- <span class=\"wmicon icon-exchange pull-left\"></span>--></div>\r");t.b("\n" + i);t.b("                        <div class=\"font-light\">");t.b(t.v(t.d("Strings.MATCH.CANNOT_ANYMORE",c,p,0)));t.b("</div>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("Tip",c,p,1),c,p,1,0,0,"")){t.b("                        <div class=\"match-goal-results\">- : -<!-- <span class=\"wmicon icon-soccer pull-left\"></span>--></div>\r");t.b("\n" + i);t.b("                        <div class=\"match-goal-tip font-digital\">");t.b(t.v(t.d("Strings.MATCH.NO_TIPS",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                        <div class=\"font-light\">");t.b(t.v(t.d("Strings.MATCH.CANNOT_ANYMORE",c,p,0)));t.b("</div>\r");t.b("\n" + i);};};};t.b("        </div>\r");t.b("\n" + i);t.b("        <div class=\"col-xs-3 content-right padding-top-5\"><img src=\"");t.b(t.v(t.f("CountryFlagB",c,p,0)));t.b("\" class=\"img-flag\" alt=\"");t.b(t.v(t.f("CountryNameB",c,p,0)));t.b("\" /></div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <!--{^HaveResuls}}-->\r");t.b("\n" + i);t.b("        <div class=\"row\">\r");t.b("\n" + i);t.b("            <div class=\"col-xs-12 content-center content-font-small font-light\">");t.b(t.v(t.f("KickOffTime",c,p,0)));t.b(" ");t.b(t.v(t.d("Strings.MATCH.ON",c,p,0)));t.b(" ");t.b(t.v(t.f("KickOffDay",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("        <div class=\"row\">\r");t.b("\n" + i);t.b("            <div class=\"col-xs-12 content-center content-font-small font-light\">");t.b(t.v(t.f("matchType",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("        \r");t.b("\n" + i);t.b("   <!-- {/HaveResuls}} -->\r");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

define('nls/country',{
	"de": true,
    "root": true
});
define('nls/root/country',{
"AL":"Albania",
"DZ":"Algeria",
"AO":"Angola",
"AR":"Argentina",
"AU":"Australia",
"AT":"Austria",
"AZ":"Azerbaijan",
"BH":"Bahrain",
"BD":"Bangladesh",
"BY":"Belarus",
"BE":"Belgium",
"BA":"Bosnia and Herzegovina",
"BW":"Botswana",
"BR":"Brazil",
"BG":"Bulgaria",
"BF":"Burkina Faso",
"CM":"Cameroon",
"CA":"Canada",
"CL":"Chile",
"CN":"China",
"CO":"Colombia",
"CG":"Congo",
"CR":"Costa Rica",
"CI":"Cote D'Ivoire",
"HR":"Croatia",
"CY":"Cyprus",
"CZ":"Czech Republic",
"DK":"Denmark",
"DO":"Dominican Republic",
"EC":"Ecuador",
"EG":"Egypt",
"SV":"El Salvador",
"EN":"England",
"EE":"Estonia",
"ET":"Ethiopia",
"FI":"Finland",
"FR":"France",
"DE":"Germany",
"GH":"Ghana",
"GR":"Greece",
"GT":"Guatemala",
"HN":"Honduras",
"HK":"Hong Kong",
"HU":"Hungary",
"IN":"India",
"ID":"Indonesia",
"IR":"Iran",
"IE":"Ireland",
"IL":"Israel",
"IT":"Italy",
"JP":"Japan",
"JO":"Jordan",
"KZ":"Kazakhstan",
"KE":"Kenya",
"KR":"Republic of Korea",
"KW":"Kuwait",
"LV":"Latvia",
"LB":"Lebanon",
"LY":"Libyan Arab Jamahiriya",
"LT":"Lithuania",
"LU":"Luxembourg",
"MK":"F.Y.R. of Macedonia",
"MG":"Madagascar",
"MY":"Malaysia",
"ML":"Mali",
"MT":"Malta",
"MU":"Mauritius",
"MX":"Mexico",
"MD":"Republic of Moldova",
"MC":"Monaco",
"MA":"Morocco",
"MZ":"Mozambique",
"NL":"Netherlands",
"NZ":"New Zealand",
"NI":"Nicaragua",
"NG":"Nigeria",
"NO":"Norway",
"OM":"Oman",
"PK":"Pakistan",
"PA":"Panama",
"PE":"Peru",
"PH":"Philippines",
"PL":"Poland",
"PT":"Portugal",
"PR":"Puerto Rico",
"QA":"Qatar",
"RO":"Romania",
"RU":"Russian Federation",
"SA":"Saudi Arabia",
"SN":"Senegal",
"RS":"Republic of Serbia",
"SG":"Singapore",
"SK":"Slovakia",
"SI":"Slovenia",
"ZA":"South Africa",
"ES":"Spain",
"LK":"Sri Lanka",
"SE":"Sweden",
"CH":"Switzerland",
"TW":"Taiwan",
"TZ":"United Republic of Tanzania",
"TH":"Thailand",
"TT":"Trinidad and Tobago",
"TN":"Tunisia",
"TR":"Turkey",
"UG":"Uganda",
"UA":"Ukraine",
"AE":"United Arab Emirates",
"GB":"United Kingdom",
"US":"United States",
"UY":"Uruguay",
"UZ":"Uzbekistan",
"VE":"Venezuela",
"VN":"Vietnam",
"ZZ":"Other"

});

define('util/country',[
	"i18n!nls/country",
], function (countriesMap) {
	
	var countries=[];
		var i=0;
	 for (var country in countriesMap) {
	 		//if (i++>10) break;
	        if (!countriesMap.hasOwnProperty(country) || typeof(countriesMap[country]) === "function"){ continue;}
	        countries.push({
	        	ccode:country,
	        	name:countriesMap[country]
	        });
	      }
	var CountryMapper={
			CountryMap:countriesMap,
			CountryList:countries,
			getCountryName:function(CountryCode){
				if (!CountryCode){
					console.log("Called getCountryName with undefined")
                    return "";
                }
				return this.CountryMap[CountryCode.toUpperCase()];
			},
			getCountryFlag:function(CountryCode){
				if (!CountryCode){
					console.log("Called getCountryFlag with undefined")
                    return "";
                }
				var cc=CountryCode?CountryCode.toLowerCase():"xx";
				return "media/flagsPNG-1x1/"+cc+".png";
			},
			getHelperMethods:function(){
				return {
					CountryNameA:function (){
						return CountryMapper.getCountryName(this.countryA)
					},
					CountryNameB:function (){
						return CountryMapper.getCountryName(this.countryB)
					},
					CountryFlagA:function (){
						return CountryMapper.getCountryFlag(this.countryA)
					},
					CountryFlagB:function (){
						return CountryMapper.getCountryFlag(this.countryB)
					}
				};
			}
		}
	return CountryMapper;

}
);


define("hgn!templates/rankings", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<!--<div class='sub-container'>-->\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("     <div class=\"col-xs-12 col-sm-4\">\r");t.b("\n" + i);t.b("            <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("                <div class=\"panel-heading\">\r");t.b("\n" + i);t.b("                    <a href=\"#rankingsplayer\"><h3 class=\"panel-title clearfix\">");t.b(t.v(t.d("Strings.RANKINGS_PLAYER.HEADLINE",c,p,0)));t.b(" <span class=\"float-right wmicon icon-right-open-big\"></span></h3></a>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("                <table class=\"table\">\r");t.b("\n" + i);t.b("                    <tbody>\r");t.b("\n" + i);if(t.s(t.f("playerRank",c,p,1),c,p,0,487,1726,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                        <tr class='player-rank' data-player-id=\"");t.b(t.v(t.f("playerId",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("                            <td>\r");t.b("\n" + i);t.b("                                <div class=\"min-height-lists\">\r");t.b("\n" + i);if(t.s(t.f("hasPlayerRank",c,p,1),c,p,0,717,797,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                                    ");t.b(t.v(t.f("rank",c,p,0)));t.b("\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("hasPlayerRank",c,p,1),c,p,1,0,0,"")){t.b("                                    &nbsp;\r");t.b("\n" + i);};t.b("                                </div>\r");t.b("\n" + i);t.b("                            </td>\r");t.b("\n" + i);t.b("                            <td>\r");t.b("\n" + i);t.b("                                <div class=\"min-height-lists\">\r");t.b("\n" + i);t.b("                                ");t.b(t.v(t.f("nickName",c,p,0)));t.b("\r");t.b("\n" + i);if(t.s(t.f("hasPlayerRank",c,p,1),c,p,0,1234,1378,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                                    <span class=\"badge badge-bg-primary pull-right\">");t.b(t.v(t.f("overallPoints",c,p,0)));t.b("</span>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("hasPlayerRank",c,p,1),c,p,1,0,0,"")){t.b("                                    <span class=\"badge badge-bg-primary pull-right\">0</span>\r");t.b("\n" + i);};t.b("                                </div>\r");t.b("\n" + i);t.b("                            </td>\r");t.b("\n" + i);t.b("                        </tr>\r");t.b("\n" + i);});c.pop();}t.b("                    </tbody>\r");t.b("\n" + i);t.b("                </table>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("    <div class=\"col-xs-12 col-sm-4\">\r");t.b("\n" + i);t.b("            <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("                <div class=\"panel-heading\">\r");t.b("\n" + i);t.b("                    <a href=\"#rankingsgroup\"><h3 class=\"panel-title clearfix\">");t.b(t.v(t.d("Strings.RANKINGS_GROUP.HEADLINE",c,p,0)));t.b(" <span class=\"float-right wmicon icon-right-open-big\"></span></h3></a>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("                <table class=\"table\">\r");t.b("\n" + i);t.b("                    <tbody>\r");t.b("\n" + i);if(t.s(t.f("groupRank",c,p,1),c,p,0,2278,3597,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                        <tr class='group-rank' data-group-id=\"");t.b(t.v(t.f("id",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("                            <td>\r");t.b("\n" + i);t.b("                                <div class=\"min-height-lists\">\r");t.b("\n" + i);t.b("                                ");t.b(t.v(t.f("groupIndex",c,p,0)));t.b("\r");t.b("\n" + i);t.b("                                </div>\r");t.b("\n" + i);t.b("                            </td>\r");t.b("\n" + i);t.b("                            <td>\r");t.b("\n" + i);t.b("                                <div class=\"min-height-lists\">\r");t.b("\n" + i);if(t.s(t.f("hasGroupPoints",c,p,1),c,p,0,2722,2900,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                                    <span class=\"badge badge-bg-primary pull-right\"><span class=\"wmicon icon-avg\" />&nbsp;");t.b(t.v(t.f("avgPoints",c,p,0)));t.b("</span>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("hasGroupPoints",c,p,1),c,p,1,0,0,"")){t.b("                                    <span class=\"badge badge-bg-primary pull-right\"><span class=\"wmicon icon-avg\" />&nbsp;0</span>\r");t.b("\n" + i);};t.b("                                <span class=\"pull-right wmicon icon-users-2 content-font-small\" >&nbsp;");t.b(t.v(t.f("memberCount",c,p,0)));t.b("&nbsp;&nbsp;</span>\r");t.b("\n" + i);t.b("                                ");t.b(t.v(t.f("name",c,p,0)));t.b(" ");if(!t.s(t.f("publicGroup",c,p,1),c,p,1,0,0,"")){t.b("\r");t.b("\n" + i);t.b("        <span class=\"wmicon icon-lock  color-grey-profile padding-left-10\"></span>\r");t.b("\n" + i);};t.b("                                </div>\r");t.b("\n" + i);t.b("                            </td>\r");t.b("\n" + i);t.b("                        </tr>\r");t.b("\n" + i);});c.pop();}t.b("                    </tbody>\r");t.b("\n" + i);t.b("                </table>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    \r");t.b("\n" + i);t.b("    <div class=\"col-xs-12 col-sm-4\">\r");t.b("\n" + i);t.b("            <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("                <div class=\"panel-heading\">\r");t.b("\n" + i);t.b("                    <a href=\"#rankingscountry\"><h3 class=\"panel-title clearfix\">");t.b(t.v(t.d("Strings.RANKINGS_COUNTRY.HEADLINE",c,p,0)));t.b(" <span class=\"float-right wmicon icon-right-open-big\"></span></h3></a>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("                <table class=\"table\">\r");t.b("\n" + i);t.b("                    <tbody>\r");t.b("\n" + i);if(t.s(t.f("countryRank",c,p,1),c,p,0,4158,5494,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                        <tr class='group-rank' data-group-id=\"");t.b(t.v(t.f("id",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("                            <td>\r");t.b("\n" + i);t.b("                                <div class=\"min-height-lists\">\r");t.b("\n" + i);t.b("                                ");t.b(t.v(t.f("countryIndex",c,p,0)));t.b("\r");t.b("\n" + i);t.b("                                </div>\r");t.b("\n" + i);t.b("                            </td>\r");t.b("\n" + i);t.b("                            <td>\r");t.b("\n" + i);t.b("                                <div class=\"min-height-lists\">\r");t.b("\n" + i);t.b("                                <img src=\"");t.b(t.v(t.f("countryGroupFlag",c,p,0)));t.b("\" class=\"img-flag20 pull-left\" alt=\"");t.b(t.v(t.f("countryGroupFlag",c,p,0)));t.b("\" />&nbsp;\r");t.b("\n" + i);if(t.s(t.f("hasGroupPoints",c,p,1),c,p,0,4734,4912,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                                    <span class=\"badge badge-bg-primary pull-right\"><span class=\"wmicon icon-avg\" />&nbsp;");t.b(t.v(t.f("avgPoints",c,p,0)));t.b("</span>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("hasGroupPoints",c,p,1),c,p,1,0,0,"")){t.b("                                    <span class=\"badge badge-bg-primary pull-right\"><span class=\"wmicon icon-avg\" />&nbsp;0</span>\r");t.b("\n" + i);};t.b("                                <span class=\"pull-right wmicon icon-users-2 content-font-small\" >&nbsp;");t.b(t.v(t.f("memberCount",c,p,0)));t.b("&nbsp;&nbsp;</span>\r");t.b("\n" + i);t.b("                                ");t.b(t.v(t.f("countryGroupName",c,p,0)));t.b("\r");t.b("\n" + i);t.b("                                </div>\r");t.b("\n" + i);t.b("                            </td>\r");t.b("\n" + i);t.b("                        </tr>\r");t.b("\n" + i);});c.pop();}t.b("                    </tbody>\r");t.b("\n" + i);t.b("                </table>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("<!--</div>-->");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/*global define*/
define('views/rankings',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/rankings',
	'models/settings',
	'util/context',
	'util/message',
	'util/country',
	"i18n!nls/strings",
	'bootstrap'
], function ($, _, Backbone, BaseView, rankingsTemplate,
	Settings,Context,MessageHelper,
	CountryMapper,Strings
	) {
	

	var View = BaseView.extend({
		titel : Strings.COMMON.RANKING,
		el: '#wmapp',

		template: rankingsTemplate,

		events: {
			'click #back':		'back',
			'click .group-rank': 'openGroupDetails',
			'click .player-rank': 'openPlayerDetails'
		},
		
		openGroupDetails:function(event){
			var groupId=$(event.currentTarget).data('group-id');
			Context.Router.navigate("#group/"+groupId+"/0", {trigger: true, replace: false});
		},
		openPlayerDetails:function(event){
			var playerId=$(event.currentTarget).data('player-id');
			Context.Router.navigate("#player/"+playerId, {trigger: true, replace: false});
		},
		initialize: function (options) {
			var groupIndex=1,
				countryIndex=1,
				readyDeferred=new $.Deferred();
				this.readyDeferred=readyDeferred.promise();
				Context.rankingOverviewIterationToken=Context.rankingOverviewIterationToken||new Date().getTime();
				if (options.reload){
					Context.rankingOverviewIterationToken=new Date().getTime();
				}
			$.when(
				this.loadGlobalPlayerRanking(),
				this.loadGlobalGroupRanking(),
				this.loadGlobalCountriesRanking()
			).done(_.bind(function(playerRank,groupRank,countryRank){
				this.viewModel={
					playerRank:playerRank[0],
					groupRank:groupRank[0],
					countryRank:countryRank[0],
					player:Context.player.toJSON(),
					myTopGroup:{},
					myCountryRank:{name:"Germany",rank:1},
					hasPlayerRank:function(){
						return this.rank!==2147483647;
					},
					hasGroupPoints:function(){
						return this.avgPoints!==0;
					},
					groupIndex:function(){
						return groupIndex++;
					},
					countryIndex:function(){
						return countryIndex++;
					},
					countryGroupFlag:function(){
						var country=this.name.substr(8);
						return CountryMapper.getCountryFlag(country);
					},
					countryGroupName:function(){
						var country=this.name.substr(8);
						return CountryMapper.getCountryName(country)
					}
				};
				this.$el.html(this.template(
					$.extend({
						Strings:Strings},
						this.viewModel
					)
				));
				readyDeferred.resolve();
			},this))
			.fail(function(ajax,error,message){
				readyDeferred.reject();
				MessageHelper.handleError(ajax,Strings.ERROR_CODE["RANKINGS.ERR.LOADING_FAILED"],this);
			});
		},
		loadGlobalPlayerRanking:function(){
			
			var options={
			  type: "GET",
			  url:Settings.get("server") +"/rest/player/top10?from=0&size=3&_it="+Context.rankingOverviewIterationToken 
			};
			return $.ajax(options).fail(function(ajax,error,message){
				MessageHelper.handleError(ajax,Strings.ERROR_CODE["RANKINGS.ERR.LOADING_FAILED"],this);
			});
		},
		loadGlobalGroupRanking:function(){
			var options={
			  type: "GET",
			  url:Settings.get("server") +"/rest/groups/top10?from=0&size=3&_it="+Context.rankingOverviewIterationToken 
			};
			return $.ajax(options).fail(function(ajax,error,message){
				MessageHelper.handleError(ajax,Strings.ERROR_CODE["RANKINGS.ERR.LOADING_FAILED"],this);
			});

		},
		loadGlobalCountriesRanking:function(){
			var options={
			  type: "GET",
			  url:Settings.get("server") +"/rest/groups/ranking/countries?from=0&size=3&_it="+Context.rankingOverviewIterationToken 
			};
			return $.ajax(options).fail(function(ajax,error,message){
				MessageHelper.handleError(ajax,Strings.ERROR_CODE["RANKINGS.ERR.LOADING_FAILED"],this);
			});
		},
	});

	return View;
});
define('util/tabs',[
		'jquery',
		'util/context'
], function ($,Context) {
	
	
	var TabsHelper={
			HeaderClickHandler:function(view,event){
				if (event && event.target){
					var target=$(event.target);
					if (target.hasClass('active')){
						return;
					}
					view.$(".btn.active").removeClass('active');
					target.addClass('active');
					var tabId=target.data('tab-id');
					Context.Router.navigate(view.getBaseUrl()+"/"+tabId, {trigger: true,replace:true});
				}
			},
			ActivateTab:function(view,id){
				var active=view.$(".btn.active");
				if (active){
					if (active.data("tab-id")!==id){
						active.removeClass('active');
						view.$(".btn[data-tab-id='"+id+"']").addClass('active');
					}
				}else{
					view.$(".btn[data-tab-id='"+id+"']").addClass('active');
				}
			}
		}
	return TabsHelper;

}
);

// Generated by CoffeeScript 1.4.0

/*
countdown is a simple jquery plugin for countdowns

Dual licensed under the MIT (http://www.opensource.org/licenses/mit-license.php)
and GPL-3.0 (http://opensource.org/licenses/GPL-3.0) licenses.

@source: http://github.com/rendro/countdown/
@autor: Robert Fleischmann
@version: 1.0.1
*/


(function() {

  (function($) {
    $.countdown = function(el, options) {
      var getDateData,
        _this = this;
      this.el = el;
      this.$el = $(el);
      this.$el.data("countdown", this);
      this.init = function() {
        _this.options = $.extend({}, $.countdown.defaultOptions, options);
        if (_this.options.refresh) {
          _this.interval = setInterval(function() {
            return _this.render();
          }, _this.options.refresh);
        }
        _this.render();
        return _this;
      };
      getDateData = function(endDate) {
        var dateData, diff;
        endDate = Date.parse($.isPlainObject(_this.options.date) ? _this.options.date : new Date(_this.options.date));
        diff = (endDate - Date.parse(new Date)) / 1000;
        if (diff <= 0) {
          diff = 0;
          if (_this.interval) {
            _this.stop();
          }
          _this.options.onEnd.apply(_this);
        }
        dateData = {
          years: 0,
          days: 0,
          hours: 0,
          min: 0,
          sec: 0,
          millisec: 0
        };
        if (diff >= (365.25 * 86400)) {
          dateData.years = Math.floor(diff / (365.25 * 86400));
          diff -= dateData.years * 365.25 * 86400;
        }
        if (diff >= 86400) {
          dateData.days = Math.floor(diff / 86400);
          diff -= dateData.days * 86400;
        }
        if (diff >= 3600) {
          dateData.hours = Math.floor(diff / 3600);
          diff -= dateData.hours * 3600;
        }
        if (diff >= 60) {
          dateData.min = Math.floor(diff / 60);
          diff -= dateData.min * 60;
        }
        dateData.sec = diff;
        return dateData;
      };
      this.leadingZeros = function(num, length) {
        if (length == null) {
          length = 2;
        }
        num = String(num);
        while (num.length < length) {
          num = "0" + num;
        }
        return num;
      };
      this.update = function(newDate) {
        _this.options.date = newDate;
        return _this;
      };
      this.render = function() {
        _this.options.render.apply(_this, [getDateData(_this.options.date)]);
        return _this;
      };
      this.stop = function() {
        if (_this.interval) {
          clearInterval(_this.interval);
        }
        _this.interval = null;
        return _this;
      };
      this.start = function(refresh) {
        if (refresh == null) {
          refresh = _this.options.refresh || $.countdown.defaultOptions.refresh;
        }
        if (_this.interval) {
          clearInterval(_this.interval);
        }
        _this.render();
        _this.options.refresh = refresh;
        _this.interval = setInterval(function() {
          return _this.render();
        }, _this.options.refresh);
        return _this;
      };
      return this.init();
    };
    $.countdown.defaultOptions = {
      date: "June 12, 2014 20:00:00",
      refresh: 1000,
      onEnd: $.noop,
      render: function(date) {
        return $(this.el).html("<div class='btn-countdown btn-default-countdown'>" + date.days + "<span>Days</span></div> " + 
                "<div class='btn-countdown btn-default-countdown'>" + (this.leadingZeros(date.hours)) + "<span>hours</span></div> " + 
                "<div class='btn-countdown btn-default-countdown'>" + (this.leadingZeros(date.min)) + "<span>min</span></div> " + 
                "<div class='btn-countdown btn-default-countdown'>" + (this.leadingZeros(date.sec)) + "<span>sec</span></div>");
        //{{Strings.HOMEOVERVIEW.CD_DAYS}}
        //{{Strings.HOMEOVERVIEW.CD_HOURS}}
        //{{Strings.HOMEOVERVIEW.CD_MIN}}
        //{{Strings.HOMEOVERVIEW.CD_SEC}}
      }
    };
    $.fn.countdown = function(options) {
      return $.each(this, function(i, el) {
        var $el;
        $el = $(el);
        if (!$el.data('countdown')) {
          return $el.data('countdown', new $.countdown(el, options));
        }
      });
    };
    return void 0;
  })(jQuery);

}).call(this);

define("util/countdown", function(){});

/*global define*/  
define('views/home',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/home',
	'hgn!templates/homeoverview',
	'hgn!templates/homeranking',
	'hgn!templates/matchsmall',
	'collections/results',
	'collections/tips',
	'util/country',
	'util/context',
	'models/settings',
    'models/player',
	'views/rankings',
	'util/tabs',
    'util/countdown',
	'i18n!nls/strings',
	'moment',
	'util/wait',
	'util/message',
], function ($, _, Backbone, BaseView, 
	homeTemplate,homeOverviewTemplate, homeRankingTemplate,matchSmallTemplate,
	Results,Tips,CountryMapper,Context,Settings,PlayerModel,RankingsView,TabsHelper,
	CoutdownUtil,Strings,moment,Wait,MessageHelper
	) {
	

	var HomeView = BaseView.extend({

		titel : Strings.COMMON.HOME,
		subView : false,
		// Instead of generating a new element, bind to the existing skeleton of
		// the App already present in the HTML.
		el: '#wmapp',
		events:{
			'click #home-overview':'handleTabClick',
			'click #home-ranking':'handleTabClick',
            'click .match-tip' : 'openTip',
            'click #matches' : 'openMatches',
            'click #bonusTips' : 'openBonusTips',
            'click #groups' : 'openGroups',
            'click #rankings' : 'handleTabClick',
            'click #feedback' : 'openFeedback'
		},
		// Compile our stats template
		template: homeTemplate,
		homeOverviewTemplate:homeOverviewTemplate,
		homeRankingTemplate:homeRankingTemplate,
		getBaseUrl:function(){
			return "#home";
		},
		handleTabClick:function(event){
			TabsHelper.HeaderClickHandler(this,event);
		},
		openTip:function(element){
			var matchId=$(element.target).closest('.match-tip').data("match-id");
			window.location.hash = '#matchtip/'+matchId;
		},
        openMatches:function(){
            window.location.hash = '#matches';
        },
        openBonusTips:function(){
            window.location.hash = '#bonustips';
        },
        openGroups:function(){
            window.location.hash = '#groups';
        },
        openFeedback:function(){
            window.location.hash = '#feedback';
        },
		homeOverviewClick:function(){
			var buttons=$(".sub-header-nav button")
			buttons.prop('disabled', true);
			Wait.start();
			this.renderOverview();
			this.updateScroll();
			buttons.prop('disabled', false);
			Wait.stop();
		},
		homeRankingViewClick:function(event){
			var buttons=$(".sub-header-nav button")
			buttons.prop('disabled', true);
			Wait.start();
			var view =new RankingsView({el:this.$("#subcontainer"),reload:event.reload});
			view.readyDeferred.done(_.bind(function(){
				this.updateScroll();
				buttons.prop('disabled', false);
				Wait.stop();
			},this));
		},

		refresh:function(){
			if ( window.location.hash == '#home/ranking'){
				this.homeRankingViewClick({reload:true});
			}else{
				Wait.start();
				$.when(
					Tips.reload(),
					Results.reload()
				).done(_.bind(function(){
					this.homeOverviewClick();
				},this))
				.fail(function(data,ajax,error,message){
					MessageHelper.handleError(ajax,Strings.ERROR_CODE["HOME.ERR.FAILED_TO_FETCH"],this);
				})
				.always(function(){Wait.stop();});
			}
		},
		initialize: function (options) {
			
			var self=this;
			
			this.$el.html(this.template({
				Strings:Strings,
				currentDate:moment(Context.getCurrentDate())
			}));
			this.trackScrolling(this.$("#subcontainer"));
			this.updateOptions(options,true);
		},
		renderStatistics:function(){
			var nextMatches=this.nextMatches();
            if (!nextMatches || nextMatches.length===0){
				Context.getCommonStatistics().done(function(statistics){
                var pointsCommunity=[], pointsPlayer=[],
                    pointsCommunity5=0,pointsCommunity3=0,pointsCommunity1=0,pointsCommunity0=0,
                    pointsPlayer5=0,pointsPlayer3=0,pointsPlayer1=0,pointsPlayer0=0,
                    sumPlayerPoints=0,sumCommunityPoints=0;
                
				pointsCommunity5=parseInt(statistics.COUNT_POINTS_5_OVERALL,10);
				pointsCommunity3=parseInt(statistics.COUNT_POINTS_3_OVERALL,10);
				pointsCommunity1=parseInt(statistics.COUNT_POINTS_1_OVERALL,10);
				pointsCommunity0=parseInt(statistics.COUNT_POINTS_0_OVERALL,10);
				
                pointsCommunity.push({value:pointsCommunity5,color:"#00a859"});
				pointsCommunity.push({value:pointsCommunity3,color:"#ffcc29"});
				pointsCommunity.push({value:pointsCommunity1,color:"#999999"});
				pointsCommunity.push({value:pointsCommunity0,color:"#ffffff"});
                
                var cCommunity=document.getElementById("statsCommunity");
				var ctxCommunity=cCommunity.getContext("2d");
				var myNewCommunityChart = new Chart(ctxCommunity).Pie(pointsCommunity,{animation :false});

				sumCommunityPoints=pointsCommunity5+pointsCommunity3+pointsCommunity1+pointsCommunity0;
                var points5overall=Math.round(pointsCommunity5/sumCommunityPoints*100);
                var points3overall=Math.round(pointsCommunity3/sumCommunityPoints*100);
                var points1overall=Math.round(pointsCommunity1/sumCommunityPoints*100);
                var points0overall=Math.round(100-points5overall-points3overall-points1overall);
                self.$("#points5overall").text(points5overall + "%");
                self.$("#points3overall").text(points3overall + "%");
                self.$("#points1overall").text(points1overall + "%");
                self.$("#points0overall").text(points0overall + "%");
                
                pointsPlayer5=parseInt(statistics.COUNT_POINTS_5,10);
				pointsPlayer3=parseInt(statistics.COUNT_POINTS_3,10);
				pointsPlayer1=parseInt(statistics.COUNT_POINTS_1,10);
				pointsPlayer0=parseInt(statistics.COUNT_POINTS_0,10);
                
                pointsPlayer.push({value:pointsPlayer5,color:"#00a859"});
				pointsPlayer.push({value:pointsPlayer3,color:"#ffcc29"});
				pointsPlayer.push({value:pointsPlayer1,color:"#999999"});
				pointsPlayer.push({value:pointsPlayer0,color:"#ffffff"});
                
                var cPlayer=document.getElementById("statsPlayer");
                if (!cPlayer){
                	return;
                }
				var ctxPlayer=cPlayer.getContext("2d");
				var myNewPlayerChart = new Chart(ctxPlayer).Pie(pointsPlayer,{animation :false});
                
				sumPlayerPoints=pointsPlayer5+pointsPlayer3+pointsPlayer1+pointsPlayer0;
                var points5player=Math.round(pointsPlayer5/sumPlayerPoints*100);
                var points3player=Math.round(pointsPlayer3/sumPlayerPoints*100);
                var points1player=Math.round(pointsPlayer1/sumPlayerPoints*100);
                var points0player=Math.round(100-points5player-points3player-points1player);
                self.$("#points5player").text(points5player + "%");
                self.$("#points3player").text(points3player + "%");
                self.$("#points1player").text(points1player + "%");
                self.$("#points0player").text(points0player + "%");
                
                var countPlayer = 0;
                countPlayer=parseInt(statistics.COUNT_PLAYER_OVERALL,10);
                self.$("#CountPlayerOverall").text(countPlayer);
                
                var countCountries = 0;
                countCountries=parseInt(statistics.COUNT_COUNTRIES_OVERALL,10);
                self.$("#CountCountriesOverall").text(countCountries);
                
                var countGroups = 0;
                countGroups=parseInt(statistics.COUNT_GROUPS_OVERALL,10);
                self.$("#CountGroupsOverall").text(countGroups);
                
                var countMatchtips = 0;
                countMatchtips=parseInt(statistics.COUNT_MATCHTIPS_OVERALL,10);
                self.$("#CountMatchtipOverall").text(countMatchtips);
                
                var matchtipBestId = 0;
                matchtipBestId=parseInt(statistics.MATCH_TIP_BEST_MATCHID,10);
                self.$("#matchtipBestId").text(matchtipBestId);
                self.$("#matchtipBestQuote").text(statistics.MATCH_TIP_BEST_QUOTE);
                
                var matchtipWorstId = 0;
                matchtipWorstId=parseInt(statistics.MATCH_TIP_WORST_MATCHID,10);
                self.$("#matchtipWorstId").text(matchtipWorstId);
                self.$("#matchtipWorstQuote").text(statistics.MATCH_TIP_WORST_QUOTE);
                var res=Results.toJSON();
                var bestMatch=_.findWhere(res,{
                	id :parseInt(statistics.MATCH_TIP_BEST_MATCHID,10)
                }),worstMatch=_.findWhere(res,{
                	id :parseInt(statistics.MATCH_TIP_WORST_MATCHID,10)
                });

                var countryA=
                self.$("#BestCountryNameA").text(CountryMapper.getCountryName(bestMatch.countryA));
                self.$("#BestCountryNameB").text(CountryMapper.getCountryName(bestMatch.countryB));
                self.$("#WorstCountryNameA").text(CountryMapper.getCountryName(worstMatch.countryA));
                self.$("#WorstCountryNameB").text(CountryMapper.getCountryName(worstMatch.countryB));
                self.$("#WorstCountryNameAFlag").attr("src",CountryMapper.getCountryFlag(worstMatch.countryA));
                self.$("#WorstCountryNameBFlag").attr("src",CountryMapper.getCountryFlag(worstMatch.countryB));
                self.$("#BestCountryNameAFlag").attr("src",CountryMapper.getCountryFlag(bestMatch.countryA));
                self.$("#BestCountryNameBFlag").attr("src",CountryMapper.getCountryFlag(bestMatch.countryB));

                self.$("#avgMatchtipOverall").text(statistics.AVG_MATCHTIP_POINTS_OVERALL);
                self.$("#avgMatchtipPlayer").text(statistics.AVG_MATCHTIP_POINTS);
                self.$("#avgBonustipOverall").text(statistics.AVG_BONUSTIP_SUMPOINTS_OVERALL);
                self.$("#avgBonustipPlayer").text(statistics.AVG_BONUSTIP_SUMPOINTS);
			})
            }
		},
		//Special method called by router to apply new options
		updateOptions:function(options,replace){
			if (!options||!options.tab||options.tab === 'overview'){
				this.homeOverviewClick({target:this.$("#home-overview"),replace:replace});
			}else{
				this.homeRankingViewClick({target:this.$("#home-ranking"),replace:replace});
			}
			TabsHelper.ActivateTab(this,options.tab);
		},
		pastDayResults:function(){
			var matches=Results.toJSON()||[],
				currentDate=moment(Context.getCurrentDate()),
				pastResults=_.filter(matches,function(match){
					var resultDate=moment(new Date(match.kickOff+"Z"));
					return resultDate.isBefore(currentDate);
				}),
				res=[],i=pastResults.length-1,lastDate=null;
				for (i;i>=0;i--){
					var r=pastResults[i],
						day=moment(new Date(r.kickOff+"Z")).zone(Context.timezoneMap[r.location]).format('YYMMDD')
					if (lastDate==null){
						res.unshift(r);
						lastDate=day;
					}else if (lastDate === day){
						res.unshift(r);
					}else{
						break;
					}
				}
				return res;
		},
		nextMatches:function(){
			var matches=Results.toJSON()||[],
				currentDate=moment(Context.getCurrentDate()),
				futureMatches=_.filter(matches,function(match){
					var resultDate=moment(new Date(match.kickOff+"Z"));
					return resultDate.isAfter(currentDate) && match.scoreA == -1 && match.scoreB == -1;
				}),
				res=[],i=0,lastDate=null;
				for (i;i<futureMatches.length;i++){
					var r=futureMatches[i],
						day=moment(new Date(r.kickOff+"Z")).zone(Context.timezoneMap[r.location]).format('YYMMDD')
					if (lastDate==null){
						res.push(r);
						lastDate=day;
					}else if (lastDate === day){
						res.push(r);
					}else{
						break;
					}
				}
				return res;
		},
        countdownTimer:function(){
        	var startDate="2014-06-12T22:00:00Z",self=this,
        	currentDate=moment(Context.getCurrentDate());
				
        	if (Results.at(0)){
        		startDate=Results.at(0).get('kickOff')+"Z";
        	}
        	var firstKickOff=moment(new Date(startDate));
        	if (firstKickOff.isAfter(currentDate)){
	            $('.wmCountdown').countdown({
	                date: firstKickOff.toDate().getTime(),
	                onEnd:function(){
	                	self.refresh();
	                }
	            });
	        }

        },
		renderOverview:function(){
			var	yesterdayResults=this.pastDayResults(),
				tommorowTips=this.nextMatches(),
				self=this,
				tipsMap={};
				Tips.each(function(tip){
					tipsMap[tip.get('matchId')]=tip.toJSON();
				});
				var res=Results.toJSON();
				var firstSemifinal= _.findWhere(res,{
                	id :61
                })
                var firstSemifinalDate=moment(new Date(firstSemifinal.kickOff+"Z"));
			this.$("#subcontainer").html(this.homeOverviewTemplate(
				$.extend({},
                    Context.player.toJSON(),
					CountryMapper.getHelperMethods(),
					{
						Strings:Strings,
						results:yesterdayResults,
						tips:tommorowTips,
						groups:Context.player.get("groups"),
						KickOffTime:function(){
						return moment(new Date(this.kickOff+"Z")).format("LT")
						},
                        matchPoints: Context.player.get('overallPoints')-Context.player.get('bonusPoints'),
						KickOffDay:function(){
							return moment(new Date(this.kickOff+"Z")).format("LL")
						},
						CanCreateTips:function(){
						var tipsEnabledFrom=moment(new Date(this.tipsEnabledFrom+"Z")),
							tipsEnabledTo=moment(new Date(this.tipsEnabledTo+"Z")),
							currentDate=moment(Context.getCurrentDate());
						//	console.log(tipsEnabledFrom.format()+" "+ tipsEnabledTo.format()+ ' '+currentDate.format()+ (currentDate.isBefore(tipsEnabledTo) && currentDate.isAfter(tipsEnabledFrom)))
							return currentDate.isBefore(tipsEnabledTo) && currentDate.isAfter(tipsEnabledFrom);
						},
						haveNextMatches:tommorowTips && tommorowTips.length,
						havePastResults:yesterdayResults && yesterdayResults.length,
						HaveResults:function(){
							return this.scoreA!=-1 && this.scoreB!=-1;
						},
						Tip:function(){
							return tipsMap[parseInt(this.id,10)];
						},
						matchType:function(){
							return Strings.GAME_TYPE[this.type];
                        },
                        hasCountdown:function(){
                            return moment(Context.getCurrentDate()).isBefore(1402603200000);
                        },
                        canSetBonusTips:function(){
                            return moment(Context.getCurrentDate()).isBefore(firstSemifinalDate);
                        },
                        extention:function(){
                        	var x=this.scoreExtensionDetail
                        	if (x !=  null && x.psoScoreText != null) {
								return x.psoScoreA+" : "+x.psoScoreB +" "+Strings.COMMON.PSO;
							}
                        }
					}),
				{match:matchSmallTemplate.template}
				)
			);
			this.renderStatistics();
            this.countdownTimer();
		}
	});

	return HomeView;
});


define("hgn!templates/login", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class='sub-container'>\r");t.b("\n" + i);t.b("<div id=\"hp-logo\"></div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-3 col-md-4\"></div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-6 col-md-4\">\r");t.b("\n" + i);t.b("    <h2>");t.b(t.v(t.d("Strings.LOGIN.APP_TITLE",c,p,0)));t.b("</h2>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("    <div class=\"form-group\">\r");t.b("\n" + i);t.b("        <input type=\"email\" class=\"form-control search-text\" id=\"userid\" placeholder=\"");t.b(t.v(t.d("Strings.COMMON.NICKNAME",c,p,0)));t.b("\" value=\"");t.b(t.v(t.f("nickName",c,p,0)));t.b("\" required autofocus>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"form-group\">\r");t.b("\n" + i);t.b("        <input type=\"password\" class=\"form-control search-text\" id=\"password\" placeholder=\"");t.b(t.v(t.d("Strings.LOGIN.PASSWORD",c,p,0)));t.b("\" value=\"");t.b(t.v(t.f("password",c,p,0)));t.b("\" required>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"form-group\">\r");t.b("\n" + i);t.b("         <input id=\"login\" type=\"button\" class=\"btn btn-primary btn-lg btn-block\" \r");t.b("\n" + i);t.b("        data-loading-text=\"");t.b(t.v(t.d("Strings.LOGIN.LOGIN_PROGRESS",c,p,0)));t.b("\" \r");t.b("\n" + i);t.b("        data-placement=\"top auto\"\r");t.b("\n" + i);t.b("        data-animation=\"false\"\r");t.b("\n" + i);t.b("        data-popover=\"true\"\r");t.b("\n" + i);t.b("        value=\"");t.b(t.v(t.d("Strings.LOGIN.LOGIN",c,p,0)));t.b("\"/>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"form-group\">\r");t.b("\n" + i);t.b("        <input id=\"register\" type=\"button\" class=\"btn btn-lg btn-default btn-block\"\r");t.b("\n" + i);t.b("        data-placement=\"top auto\"\r");t.b("\n" + i);t.b("        data-animation=\"false\"\r");t.b("\n" + i);t.b("        data-popover=\"true\"\r");t.b("\n" + i);t.b("         value=\"");t.b(t.v(t.d("Strings.LOGIN.REGISTER",c,p,0)));t.b("\"/>\r");t.b("\n" + i);t.b("        <input id=\"passwordreset\" type=\"button\" class=\"btn btn-sm btn-default btn-block\" value=\"");t.b(t.v(t.d("Strings.LOGIN.PASSWORD_RESET",c,p,0)));t.b("\" />\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-3 col-md-4\"></div>\r");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/*global define*/
define('views/login',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/login',
	'models/settings',
	'util/context',
	'util/message',
	'models/player',
	'util/wait',
	"i18n!nls/strings",
	'bootstrap'
], function ($, _, Backbone, BaseView, loginTemplate,SettingsModel,Context,MessageHelper,PlayerModel,Wait,Strings) {
	
	var LoginView = BaseView.extend({

		titel : Strings.LOGIN.LOGIN,		
		// Instead of generating a new element, bind to the existing skeleton of
		// the App already present in the HTML.
		el: '#wmapp',

		// Compile our stats template
		template: loginTemplate,

		events: {
			'click #login':				'LoginUser',
			'click #passwordreset':		'RestPassword',
			'click #register':			'Register',
			'keypress #password': 'keyboardLogin'

		},
		keyboardLogin:function(e){
			if (e.keyCode == 13 ){
				this.LoginUser();
			}
		},
		initialize: function () {
			var self=this;
			SettingsModel.fetch().always(function(){
				self.$el.html(self.template($.extend(SettingsModel.toJSON(),{
					Strings:Strings,
					password:Context.offline?"xxxx":""//enable relogin using token when offline, let user think the password is going to be used
				})));

				//Display error message from the initialization process if there were any
				if (Context.errorMessage){
					setTimeout(function(){MessageHelper.showError(Context.errorMessage);
						delete Context.errorMessage;
						},0);
				}
			})
			
		},
		Register:function(){
			window.location.hash = '#register';
		},
		RestPassword:function(){
			window.location.hash = '#passwordreset';
		},
		clearCachedData:function(){
			localStorage.removeItem("backboneCache");

			if (window.cache && window.cache.clear){
				var success = function(status) {
		            console.log('Cache cleared');
		        }

		        var error = function(status) {
		            console.log('Errro clearing cache');
		        }

		        window.cache.clear( success, error );
		    }
		},

		LoginUser:function(event){
			if (this.$("#userid").val() === "!#SETTINGS#!"){
				window.location.hash = '#settings';
				return;
			}
			var self =this;
			Wait.start();
			MessageHelper.hide();
			if (Context.offline && Context.haveLocalToken()){
				Context.loginUsingToken().done(function(){
					Context.Router.navigate("#home/overview", {trigger: true, replace: true});
				}).always(function(){
					Wait.stop();
				})
				return;
			}
			$.ajax({
			  type: "POST",
			  url:SettingsModel.get("server") +"/rest/token",
			  contentType: 'application/x-www-form-urlencoded',
			  processData : true,
			  data: {
							username:this.$("#userid").val(),
							password:this.$("#password").val(),
							response_type:"token",
							redirect_uri:window.location.protocol+"//"+window.location.host,
							grant_type:"password",
							client_id:"WMApp",
							client_secret:"WMApp",
							scope:"test"
						}
			}).done (function(data){
				if (SettingsModel.get("playerId")!== data.playerId){
					self.clearCachedData();
				}
				SettingsModel.save({
						access_token:data.access_token,
						playerId:data.playerId,
						nickName:self.$("#userid").val(),
						expires_in:data.expires_in,
						loggedIn:new Date()

				})
				Context.loadInitialData()
					.done(function(){
						
						Wait.stop()
						self.remove();
						if (Context.player.get("pwChangeRequired")){
							Context.requestLoginOnChangePassword=false;
							Context.password=self.$("#password").val();
							Context.Router.navigate("#changepwd", {trigger: true});
							
						}else{
							$(".private-menu-item").css( "display", "block");
							$("#login").css( "display", "none");
							Context.Router.navigate("#home/overview", {trigger: true, replace: true});
						}
					}).fail(function(data){
						Wait.stop();
						Context.signedin=false;
						MessageHelper.handleError(data,Strings.LOGIN.FAILED_TO_LOGIN,this);
					});
					
				
			}).fail(function(data){
				Wait.stop();
				Context.signedin=false;
				MessageHelper.handleError(data,Strings.LOGIN.FAILED_TO_LOGIN,this);
			})
		}
	});

	return LoginView;
});


define("hgn!templates/register", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class='sub-container'>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);t.b("      <div class=\"form-group\">\r");t.b("\n" + i);t.b("        <!--<label for=\"inputEmail3\" class=\"col-sm-2 control-label\">Email</label>-->\r");t.b("\n" + i);t.b("          ");t.b(t.v(t.d("Strings.COMMON.EMAIL",c,p,0)));t.b("\r");t.b("\n" + i);t.b("          <input type=\"text\" class=\"form-control search-text\" id=\"email\" placeholder=\"");t.b(t.v(t.d("Strings.COMMON.EMAIL",c,p,0)));t.b("\" value=\"\">\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);t.b("      <div class=\"form-group\">\r");t.b("\n" + i);t.b("        <!--<label for=\"inputEmail3\" class=\"col-sm-2 control-label\">Nickname</label>-->\r");t.b("\n" + i);t.b("          ");t.b(t.v(t.d("Strings.COMMON.NICKNAME",c,p,0)));t.b("\r");t.b("\n" + i);t.b("          <input type=\"text\" class=\"form-control search-text\" id=\"nickname\" placeholder=\"");t.b(t.v(t.d("Strings.COMMON.NICKNAME",c,p,0)));t.b("\" value=\"\">\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);t.b("      <div class=\"form-group\">\r");t.b("\n" + i);t.b("        ");t.b(t.v(t.d("Strings.COMMON.COUNTRY",c,p,0)));t.b("\r");t.b("\n" + i);t.b("        <!--<label for=\"inputEmail3\" class=\"col-sm-2 control-label\">Country</label>-->\r");t.b("\n" + i);t.b("        <select class=\"form-control search-text\" id=\"country\" placeholder=\"");t.b(t.v(t.d("Strings.COMMON.COUNTRY",c,p,0)));t.b("\">\r");t.b("\n" + i);if(t.s(t.f("countries",c,p,1),c,p,0,988,1057,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <option value=\"");t.b(t.v(t.f("ccode",c,p,0)));t.b("\">");t.b(t.v(t.f("name",c,p,0)));t.b("</option>\r");t.b("\n" + i);});c.pop();}t.b("        </select>\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);t.b("      <div class=\"form-group\">\r");t.b("\n" + i);t.b("        <!--<label for=\"inputEmail3\" class=\"col-sm-2 control-label\">Organization</label>-->\r");t.b("\n" + i);t.b("           ");t.b(t.v(t.d("Strings.COMMON.ORGANIZATION",c,p,0)));t.b("\r");t.b("\n" + i);t.b("          <input type=\"text\" class=\"form-control search-text\" id=\"org\" placeholder=\"");t.b(t.v(t.d("Strings.COMMON.ORGANIZATION",c,p,0)));t.b("\" value=\"\">\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);t.b("       <div class=\"form-group\">\r");t.b("\n" + i);t.b("          <input id=\"register\" type=\"button\" class=\"btn btn-default btn-primary btn-lg btn-block\" value=\"");t.b(t.v(t.d("Strings.LOGIN.REGISTER",c,p,0)));t.b("\"/>\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);t.b("       <div class=\"form-group\">\r");t.b("\n" + i);t.b("          <input id=\"cancel\" type=\"button\" class=\"btn btn-default btn-lg btn-block\" value=\"");t.b(t.v(t.d("Strings.COMMON.CANCEL",c,p,0)));t.b("\"/>\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);t.b("      <div class=\"form-group\">\r");t.b("\n" + i);t.b("        <input  type=\"button\" class=\"btn btn-primary btn-lg btn-block\" id=\"goto-login\" value=\"");t.b(t.v(t.d("Strings.LOGIN.LOGIN",c,p,0)));t.b("\"/>\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);t.b("      <div class=\"hidden alert\"></div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/*global define*/
define('views/register',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/register',
	'models/settings',
	'util/context',
	'util/message',
	'util/country',
	'util/wait',
	"i18n!nls/strings",
	'bootstrap'
], function ($, _, Backbone, BaseView, RegisterTemplate,Settings,Context,MessageHelper,CountryMapper,Wait,Strings) {
	

	var RegisterView = BaseView.extend({

		titel : Strings.REGISTER.TITLE,
		// Instead of generating a new element, bind to the existing skeleton of
		// the App already present in the HTML.
		el: '#wmapp',

		// Compile our stats template
		template: RegisterTemplate,

		events: {
			'click #register':		'register',
			'click #cancel':		'gotologin',
			'click #goto-login': 	'gotologin'
		},

		gotologin:function(){
			window.location.hash = '#login';
		},
		initialize: function () {
			
			this.$el.append(this.template({
				Strings:Strings,
				countries:CountryMapper.CountryList
			}));
			this.$('#country option[value="DE"]').prop('selected', true);
			this.$('#country option[value="EN"]').remove();
		},
		register:function(event){
			var email=this.$("#email").val(),
				nick=this.$("#nickname").val(),
				country=this.$("#country").val(),
				org=this.$("#org").val(),
				isBlank=function (str) {
				    return (!str || /^\s*$/.test(str));
				},
				self=this;
			if (isBlank(email)){
				MessageHelper.showError(Strings.REGISTER.ERROR_EMAIL_MISSING);
				return;
			}
			if (!/@/.test(email)){
				MessageHelper.showError(Strings.REGISTER.ERROR_EMAIL_INVALID);
				return;
			}
			if (isBlank(nickname)){
				MessageHelper.showError(Strings.REGISTER.ERROR_NICKNAME_MISSING);
				return;
			}
			if (isBlank(country)){
				MessageHelper.showError(Strings.REGISTER.ERROR_COUNTRY_MISSING);
				return;
			}
			
			Wait.start(event.target)
			MessageHelper.hide();
			$.ajax({
				type: "PUT",
				  url: Settings.get("server") + "/rest/register",
				  data: { 
				  	nickName:nick.trim(), 
				  	email: email.trim(),
				  	countryCode:country,
				  	organization:org && org.trim()
				  }
			}).done(function(){
				MessageHelper.showInfo(Strings.REGISTER.REGISTRATION_SENT);
				self.$(".btn").hide();
				self.$("#goto-login").show();
				Wait.stop();

			}).fail(function(ajax,error,message){
				Wait.stop();
				MessageHelper.handleError(ajax,Strings.REGISTER.SEND_REGISTRATION_ERROR,this);
			});
		}
	});

	return RegisterView;
});


define("hgn!templates/changepwd", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class='sub-container'>\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);if(t.s(t.f("displayExtra",c,p,1),c,p,0,118,407,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("  <div class=\"form-group\">\r");t.b("\n" + i);t.b("    <!--<label for=\"inputEmail3\" class=\"col-sm-2 control-label\">Old Password</label>-->\r");t.b("\n" + i);t.b("      ");t.b(t.v(t.d("Strings.CHANGEPWD.OLD_PWD",c,p,0)));t.b("\r");t.b("\n" + i);t.b("    <input type=\"password\" class=\"form-control\" id=\"old-password\" placeholder=\"");t.b(t.v(t.d("Strings.CHANGEPWD.OLD_PWD",c,p,0)));t.b("\" value=\"\">\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);});c.pop();}t.b("  <div class=\"form-group\">\r");t.b("\n" + i);t.b("      <!--<label for=\"inputEmail3\" class=\"col-sm-2 control-label\">New Password</label>-->\r");t.b("\n" + i);t.b("      ");t.b(t.v(t.d("Strings.CHANGEPWD.NEW_PWD",c,p,0)));t.b("\r");t.b("\n" + i);t.b("    <input type=\"password\" class=\"form-control\" id=\"new-password\" placeholder=\"");t.b(t.v(t.d("Strings.CHANGEPWD.NEW_PWD",c,p,0)));t.b("\" value=\"\">\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("  <div class=\"form-group\">\r");t.b("\n" + i);t.b("    <!--<label for=\"inputEmail3\" class=\"col-sm-2 control-label\">Repeat New Password</label>-->\r");t.b("\n" + i);t.b("    ");t.b(t.v(t.d("Strings.CHANGEPWD.REPEAT_PWD",c,p,0)));t.b("\r");t.b("\n" + i);t.b("    <input type=\"password\" class=\"form-control\" id=\"new-password2\" placeholder=\"");t.b(t.v(t.d("Strings.CHANGEPWD.REPEAT_PWD",c,p,0)));t.b("\" value=\"\">\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);if(!t.s(t.f("displayExtra",c,p,1),c,p,1,0,0,"")){t.b("    <div class=\"form-group\">\r");t.b("\n" + i);t.b("      <h2>");t.b(t.v(t.d("Strings.COMMON.SEQURITY_QUESTION",c,p,0)));t.b("</h2>\r");t.b("\n" + i);t.b("      <select class=\"form-control search-text\" id=\"sequrityquestion\" placeholder=\"");t.b(t.v(t.d("Strings.COMMON.SEQURITY_QUESTION",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("         <option value=\"\">");t.b(t.v(t.d("Strings.CHANGEPWD.NO_QUESTION",c,p,0)));t.b("</option>\r");t.b("\n" + i);if(t.s(t.f("questions",c,p,1),c,p,0,1328,1392,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("          <option value=\"");t.b(t.v(t.f("key",c,p,0)));t.b("\">");t.b(t.v(t.f("value",c,p,0)));t.b("</option>\r");t.b("\n" + i);});c.pop();}t.b("      </select>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"form-group\">\r");t.b("\n" + i);t.b("        ");t.b(t.v(t.d("Strings.COMMON.SEQURITY_ANSWER",c,p,0)));t.b("\r");t.b("\n" + i);t.b("        <input type=\"text\" class=\"form-control search-text\" id=\"sequrityanswer\" placeholder=\"");t.b(t.v(t.d("Strings.COMMON.SEQURITY_ANSWER",c,p,0)));t.b("\" value=\"\">\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);};t.b("   <div class=\"form-group\">\r");t.b("\n" + i);t.b("      <input id=\"change\" type=\"button\" class=\"btn btn-primary btn-lg btn-block\" data-loading-text=\"");t.b(t.v(t.d("Strings.CHANGEPWD.CHANGING_PWD",c,p,0)));t.b("\" value=\"");t.b(t.v(t.d("Strings.COMMON.CHANGE",c,p,0)));t.b("\"/>\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("   <div class=\"form-group\">\r");t.b("\n" + i);t.b("      <input id=\"cancel\" type=\"button\" class=\"btn btn-default btn-lg btn-block\" value=\"");t.b(t.v(t.d("Strings.COMMON.CANCEL",c,p,0)));t.b("\"/>\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("  <div class=\"hidden  alert\"></div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/*global define*/
define('views/changepwd',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/changepwd',
	'models/settings',
	'util/context',
	'util/message',
	'util/wait',
	"i18n!nls/strings",
	'bootstrap'
], function ($, _, Backbone, BaseView, ChangePwdTemplate,SettingsModel,Context,MessageHelper,Wait,Strings) {
	

	var RegisterView = BaseView.extend({

		titel : Strings.COMMON.CHANGE_PWD,
		subView: true,
		// Instead of generating a new element, bind to the existing skeleton of
		// the App already present in the HTML.
		el: '#wmapp',

		// Compile our stats template
		template: ChangePwdTemplate,

		events: {
			'click #change':		'change',
			'click #cancel':		'cancel',
			'blur select':'focusLost',
			'focus select':'focusGot'
		},

		focusLost:function(event){
			console.log("focusLost")
			if (new Date().getTime()-this.focusSet<1000){
				console.log("resetfocus")
				$(this.lastFocused).focus();
			}
		},
		focusGot:function(event){
			console.log("focusGot")
			this.lastFocused=event.target;
			this.focusSet=new Date().getTime();
		},
		initialize: function () {
			var questions=[];
			_.each(Strings.SEQURITY_QUESTION,function(value, key){
				questions.push({key:key,value:value});
			})
			this.$el.html(this.template({
				Strings:Strings,
				questions:questions,
				displayExtra:!Context.signedin||Context.requestLoginOnChangePassword
			}));
			
		},
		change:function(event){
			var OldPassword=this.$("#old-password").val(),
				NewPassword=this.$("#new-password").val(),
				NewPassword2=this.$("#new-password2").val(),
				seqQuestion=this.$("#sequrityquestion").val(),
				seqAnswer=this.$("#sequrityanswer").val(),
				isBlank=function (str) {
				    return (!str || /^\s*$/.test(str));
				};
			Wait.start(event.target);
			if (!Context.signedin||Context.requestLoginOnChangePassword){
				if (isBlank(OldPassword)){
					MessageHelper.showError(Strings.ERROR_CODE["CHANGEPWD.ERR.OLDPWD"]);
					return;
				}
			}
			if (isBlank(NewPassword)){
				MessageHelper.showError(Strings.ERROR_CODE["CHANGEPWD.ERR.NEWPWD"]);
				return;
			}
			if (isBlank(NewPassword2)){
				MessageHelper.showError(Strings.ERROR_CODE["CHANGEPWD.ERR.CONFIRMPWD"]);
				return;
			}
			if (NewPassword!==NewPassword2){
				MessageHelper.showError(Strings.ERROR_CODE["CHANGEPWD.ERR.NEWPWD_MATCH"]);
				return;
			}
			if (Context.signedin){
				/*if (){
					MessageHelper.showError(Strings.REGISTER.ERROR_SEQ_QUESTION_MISSING);
					return;
				}*/
				if (!isBlank(seqQuestion)&& isBlank(seqAnswer)){
					MessageHelper.showError(Strings.REGISTER.ERROR_SEQ_ANSWER_MISSING);
					return;
				}
			}
			if (!Context.signedin||Context.requestLoginOnChangePassword){
			$.ajax({
			  type: "POST",
			  url:SettingsModel.get("server") +"/rest/token",
			  contentType: 'application/x-www-form-urlencoded',
			  processData : true,
			  data: {
							username:Context.player.get('nickName'),
							password:OldPassword,
							response_type:"token",
							redirect_uri:window.location.protocol+"//"+window.location.host,
							grant_type:"password",
							client_id:"WMApp",
							client_secret:"WMApp",
							scope:"test"
						}
			}).done (function(data){

				SettingsModel.save({
						access_token:data.access_token,
						playerId:data.playerId,
						expires_in:data.expires_in,
						loggedIn:new Date()

				})
				Context.loadInitialData()
				.done(function(){
					$.ajax({
					  type: "PUT",
					  url: SettingsModel.get("server") + "/rest/player/"+Context.player.id+"/chgpwd",
					  data: { 
					  	oldPassword:OldPassword, 
					  	newPassword: NewPassword
					  }
					}).done(function(){
						Wait.stop(event.target);
						$(".private-menu-item").css( "display", "block");
						$("#login").css( "display", "none");
						window.history.back();
						//if (Context.requestLoginOnChangePassword){
						//	Context.Router.navigate("#profile", {trigger: true, replace: false});
						//}else{
						//	Context.Router.navigate("#home/overview", {trigger: true, replace: false});
						//}
						MessageHelper.showInfo(Strings.CHANGEPWD.SUCCESS);
					}).fail(function(ajax,error,message){
						Wait.stop(event.target);
						MessageHelper.handleError(ajax,Strings.ERROR_CODE["CHANGEPWD.ERR.FAILED"],this);
						});		
				}).fail(function(data){
					Wait.stop(event.target);
					Context.signedin=false;
					MessageHelper.handleError(data,Strings.LOGIN.FAILED_TO_LOGIN,this);
				});
				
					
				
			}).fail(function(data){
				if (data.status === 401){
					data.status = 402;//prevent redirect to login page implemented centrally in main.js
				}
				Wait.stop(event.target);
				Context.signedin=true;
				MessageHelper.handleError(data,Strings.LOGIN.FAILED_TO_LOGIN,this);
			})
		}else{
			var changepwd=function(){
				SettingsModel.save({
					SECURITY_QUESTION:seqQuestion
				});
				$.ajax({
					  type: "PUT",
					  url: SettingsModel.get("server") + "/rest/player/"+Context.player.id+"/chgpwd",
					  data: { 
					  	oldPassword:Context.password, 
					  	newPassword: NewPassword
					  }
				}).done(function(){
					Wait.stop(event.target);
					$(".private-menu-item").css( "display", "block");
					$("#login").css( "display", "none");
					Context.Router.navigate("#home/overview", {trigger: true, replace: true});
					MessageHelper.showInfo(Strings.CHANGEPWD.SUCCESS);
				}).fail(function(data){
					Wait.stop(event.target);
					MessageHelper.handleError(data,Strings.ERROR_CODE["CHANGEPWD.ERR.FAILED"],this);
				});
			}
			if (!isBlank(seqQuestion)){
				$.ajax({
				  type: "PUT",
				  url: SettingsModel.get("server") + "/rest/player/settings",
				  data: [{ 
				  	name:"SECURITY_QUESTION", 
				  	value: seqQuestion
				  },
				  { 
				  	name:"SECURITY_ANSWER", 
				  	value: seqAnswer
				  }]
				}).done(changepwd)
				.fail(function(data){
					Wait.stop(event.target);
					MessageHelper.handleError(data,Strings.ERROR_CODE["CHANGEPWD.ERR.FAILED"],this);
				});
			}else{
				changepwd();
			}
		}
		},
		cancel:function(){
			if (Context.signedin && Context.requestLoginOnChangePassword){
				window.history.back();
			}else{
				Context.cleanLoginData();
				window.location.hash = '#login';
			}
		}
	});

	return RegisterView;
});


define("hgn!templates/playerdetails", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class='sub-container have-footer'>\r");t.b("\n" + i);t.b("    <div>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);t.b("        <div class=\"padding-bottom-10\">\r");t.b("\n" + i);if(t.s(t.f("isOwner",c,p,1),c,p,0,199,449,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("              <h2><span class=\"badge badge-bg-primary badge-lg-font pull-right\">");t.b(t.v(t.f("overallPoints",c,p,0)));t.b("</span>");t.b(t.v(t.d("Strings.PLAYER_DETAILS.MY_PROFILE",c,p,0)));t.b("</h2>\r");t.b("\n" + i);t.b("              <div class=\"form-control-static color-grey-profile\">");t.b(t.v(t.f("nickName",c,p,0)));t.b("</div>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("isOwner",c,p,1),c,p,1,0,0,"")){t.b("                <h2><span class=\"badge badge-bg-primary badge-lg-font pull-right\">");t.b(t.v(t.f("overallPoints",c,p,0)));t.b("</span>");t.b(t.v(t.f("nickName",c,p,0)));t.b("</h2>\r");t.b("\n" + i);};t.b("              <!--<div class=\"form-control-static\">");t.b(t.v(t.f("realName",c,p,0)));t.b("</div>-->\r");t.b("\n" + i);t.b("          </div>\r");t.b("\n" + i);t.b("          <div class=\"padding-bottom-10\">\r");t.b("\n" + i);t.b("              <img src=\"");t.b(t.v(t.f("CountryFlag",c,p,0)));t.b("\" class=\"img-flag40 pull-right\" alt=\"");t.b(t.v(t.f("CountryName",c,p,0)));t.b("\" />\r");t.b("\n" + i);t.b("              <div><span class=\"wmicon icon-mail-alt color-grey-profile\"></span> ");t.b(t.v(t.f("email",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("              <div><span class=\"wmicon icon-sitemap color-grey-profile\"></span> ");t.b(t.v(t.f("organization",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("          </div>\r");t.b("\n" + i);t.b("          <div class=\"padding-bottom-50\">\r");t.b("\n" + i);t.b("                <div class=\"col-xs-4\">\r");t.b("\n" + i);t.b("                    <div class=\"row\">\r");t.b("\n" + i);t.b("                        <span class=\"wmicon icon-dot-3 font-26 color-grey-profile padding-right-5 pull-left\" />\r");t.b("\n" + i);t.b("                        <div class=\"font-h4 font-medium\">");t.b(t.v(t.f("matchPoints",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                    </div>\r");t.b("\n" + i);t.b("                    <div class=\"row\">\r");t.b("\n" + i);t.b("                        <div class=\"font-s font-condensed font-light color-grey-profile\">");t.b(t.v(t.d("Strings.PLAYER_DETAILS.MATCHTIP_POINTS",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                    </div>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("                <div class=\"col-xs-4\">\r");t.b("\n" + i);t.b("                    <div class=\"row\">\r");t.b("\n" + i);t.b("                        <span class=\"wmicon icon-award-1 font-26 color-grey-profile padding-right-5 pull-left\" />\r");t.b("\n" + i);t.b("                        <div class=\"font-h4 font-medium\">");t.b(t.v(t.f("bonusPoints",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                    </div>\r");t.b("\n" + i);t.b("                    <div class=\"row\">\r");t.b("\n" + i);t.b("                        <div class=\"font-s font-condensed font-light color-grey-profile\">");t.b(t.v(t.d("Strings.PLAYER_DETAILS.BONUSTIP_POINTS",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                    </div>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("                <div class=\"col-xs-4\">\r");t.b("\n" + i);t.b("                    <div class=\"row\">\r");t.b("\n" + i);t.b("                        <span class=\"glyphicon glyphicon-sort-by-attributes font-26 color-grey-profile padding-right-5 pull-left\" />\r");t.b("\n" + i);t.b("                        <div class=\"font-h4 font-medium\">");t.b(t.v(t.f("overallRank",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                    </div>\r");t.b("\n" + i);t.b("                    <div class=\"row\">\r");t.b("\n" + i);t.b("                        <div class=\"font-s font-condensed font-light color-grey-profile\">");t.b(t.v(t.d("Strings.PLAYER_DETAILS.GLOBAL_RANK",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                    </div>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("          </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-6\">\r");t.b("\n" + i);t.b("            <div class=\"form-group\">\r");t.b("\n" + i);t.b("              <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("                <div class=\"panel-heading\">\r");t.b("\n" + i);if(t.s(t.f("isOwner",c,p,1),c,p,0,3023,3129,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                  <h3 class=\"panel-title clearfix\">");t.b(t.v(t.d("Strings.COMMON.MY_GROUPS",c,p,0)));t.b("</h3>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("isOwner",c,p,1),c,p,1,0,0,"")){t.b("                    <h3 class=\"panel-title clearfix\">");t.b(t.v(t.f("nickName",c,p,0)));t.b(t.v(t.d("Strings.PLAYER_DETAILS.GENITIVE_GROUPS",c,p,0)));t.b("</h3>\r");t.b("\n" + i);};t.b("                </div>\r");t.b("\n" + i);t.b("                <div class=\"list-group\">\r");t.b("\n" + i);if(t.s(t.f("groups",c,p,1),c,p,0,3414,4076,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                    <a href=\"#group/");t.b(t.v(t.f("groupId",c,p,0)));t.b("\" class=\"list-group-item min-height-lists\">\r");t.b("\n" + i);t.b("                      <div class=\"row\">\r");t.b("\n" + i);t.b("                        <div class=\"col-xs-8\">");t.b(t.v(t.f("groupName",c,p,0)));if(t.s(t.f("isPrivate",c,p,1),c,p,0,3622,3716,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("\r");t.b("\n" + i);t.b("        <span class=\"wmicon icon-lock  color-grey-profile padding-left-10\"></span>\r");t.b("\n" + i);t.b("        ");});c.pop();}t.b("</div>\r");t.b("\n" + i);t.b("                        <div class=\"col-xs-2\">");t.b(t.v(t.f("groupRank",c,p,0)));t.b("</div>\r");t.b("\n" + i);if(t.s(t.f("isGroupOwner",c,p,1),c,p,0,3846,3985,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                        <div class=\"col-xs-1 player-owner\"><span class=\"glyphicon glyphicon-flag\"></span></div>\r");t.b("\n" + i);});c.pop();}t.b("                      </div>\r");t.b("\n" + i);t.b("                    </a>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("hasGroups",c,p,1),c,p,1,0,0,"")){t.b("                <a href=\"#groups\" class=\"list-group-item\">\r");t.b("\n" + i);t.b("                  ");t.b(t.v(t.d("Strings.PLAYER_DETAILS.NO_GROUPS",c,p,0)));t.b("\r");t.b("\n" + i);t.b("                </a>\r");t.b("\n" + i);};t.b("                </div>\r");t.b("\n" + i);t.b("              </div>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("          <div class=\"panel panel-default\" >\r");t.b("\n" + i);t.b("            <div class=\"panel-heading\">\r");t.b("\n" + i);t.b("                <h3 class=\"panel-title clearfix\">\r");t.b("\n" + i);if(t.s(t.f("isOwner",c,p,1),c,p,0,4521,4682,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                ");t.b(t.v(t.d("Strings.PLAYER_DETAILS.MY_BONUSTIPS",c,p,0)));t.b(" <a href=\"#bonustips\" class=\"float-right\"><span class=\"wmicon icon-edit-alt\"></span></a>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("isOwner",c,p,1),c,p,1,0,0,"")){t.b("               ");t.b(t.v(t.f("nickName",c,p,0)));t.b(t.v(t.d("Strings.PLAYER_DETAILS.GENITIVE_BONUSTIPS",c,p,0)));t.b("\r");t.b("\n" + i);};t.b("               </h3>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("            <div class=\"panel-body\">\r");t.b("\n" + i);t.b("                <div>\r");t.b("\n" + i);t.b("                    <span class=\"wmicon icon-award color-gold pull-left\" />&nbsp;\r");t.b("\n" + i);if(t.s(t.d("bonusTips.winner",c,p,1),c,p,0,5053,5158,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                        <span class='bonustip-value'>");t.b(t.v(t.f("WinnerCountryName",c,p,0)));t.b("</span>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.d("bonusTips.winner",c,p,1),c,p,1,0,0,"")){t.b("                        <span class=\"font-light color-grey-profile\">");t.b(t.v(t.d("Strings.PLAYER_DETAILS.NO_BONUS_TIPP",c,p,0)));t.b("</span>\r");t.b("\n" + i);};t.b("                </div>\r");t.b("\n" + i);t.b("                <div>\r");t.b("\n" + i);t.b("                    <span class=\"wmicon icon-award color-silver\" />&nbsp;\r");t.b("\n" + i);if(t.s(t.d("bonusTips.second",c,p,1),c,p,0,5547,5652,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                        <span class='bonustip-value'>");t.b(t.v(t.f("SecondCountryName",c,p,0)));t.b("</span>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.d("bonusTips.second",c,p,1),c,p,1,0,0,"")){t.b("                        <span class=\"font-light color-grey-profile\">");t.b(t.v(t.d("Strings.PLAYER_DETAILS.NO_BONUS_TIPP",c,p,0)));t.b("</span>\r");t.b("\n" + i);};t.b("                </div>\r");t.b("\n" + i);t.b("                <div>\r");t.b("\n" + i);t.b("                    <span class=\"wmicon icon-soccer color-gold\" />&nbsp;\r");t.b("\n" + i);if(t.s(t.d("bonusTips.gldball",c,p,1),c,p,0,6041,6505,"{{ }}")){t.rs(c,p,function(c,p,t){if(t.s(t.f("WinnerGoldenBallName",c,p,1),c,p,0,6090,6458,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                        <span class='bonustip-value'>");t.b(t.v(t.f("name",c,p,0)));t.b(" (");t.b(t.v(t.f("PlayerCountry",c,p,0)));t.b(")</span>\r");t.b("\n" + i);if(!t.s(t.f("isSet",c,p,1),c,p,1,0,0,"")){t.b("                        <span class=\"glyphicon glyphicon-warning-sign\"></span>\r");t.b("\n" + i);t.b("                        <span class='bonustip-value-removed'>");t.b(t.v(t.d("Strings.COMMON.NOT_IN_TEAM",c,p,0)));t.b("</span>\r");t.b("\n" + i);};});c.pop();}});c.pop();}if(!t.s(t.d("bonusTips.gldball",c,p,1),c,p,1,0,0,"")){t.b("                        <span class=\"font-light color-grey-profile\">");t.b(t.v(t.d("Strings.PLAYER_DETAILS.NO_BONUS_TIPP",c,p,0)));t.b("</span>\r");t.b("\n" + i);};t.b("                </div>\r");t.b("\n" + i);t.b("                <div>\r");t.b("\n" + i);t.b("                    <span class=\"wmicon icon-glove color-gold\" />&nbsp;\r");t.b("\n" + i);if(t.s(t.d("bonusTips.gldglove",c,p,1),c,p,0,6897,7362,"{{ }}")){t.rs(c,p,function(c,p,t){if(t.s(t.f("WinnerGoldenGloveName",c,p,1),c,p,0,6945,7314,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                        <span class='bonustip-value'>");t.b(t.v(t.f("name",c,p,0)));t.b(" (");t.b(t.v(t.f("PlayerCountry",c,p,0)));t.b(")</span>\r");t.b("\n" + i);if(!t.s(t.f("isSet",c,p,1),c,p,1,0,0,"")){t.b("                        <span class=\"glyphicon glyphicon-warning-sign\"></span>\r");t.b("\n" + i);t.b("                        <span class='bonustip-value-removed'>");t.b(t.v(t.d("Strings.COMMON.NOT_IN_TEAM",c,p,0)));t.b("</span>\r");t.b("\n" + i);};});c.pop();}});c.pop();}if(!t.s(t.d("bonusTips.gldglove",c,p,1),c,p,1,0,0,"")){t.b("                        <span class=\"font-light color-grey-profile\">");t.b(t.v(t.d("Strings.PLAYER_DETAILS.NO_BONUS_TIPP",c,p,0)));t.b("</span>\r");t.b("\n" + i);};t.b("                </div>\r");t.b("\n" + i);t.b("                <div>\r");t.b("\n" + i);t.b("                    <span class=\"wmicon icon-boot color-gold\" />&nbsp;\r");t.b("\n" + i);if(t.s(t.d("bonusTips.gldboot",c,p,1),c,p,0,7755,8218,"{{ }}")){t.rs(c,p,function(c,p,t){if(t.s(t.f("WinnerGoldenBootName",c,p,1),c,p,0,7802,8171,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                        <span class='bonustip-value'>");t.b(t.v(t.f("name",c,p,0)));t.b(" (");t.b(t.v(t.f("PlayerCountry",c,p,0)));t.b(")</span>\r");t.b("\n" + i);if(!t.s(t.f("isSet",c,p,1),c,p,1,0,0,"")){t.b("                        <span class=\"glyphicon glyphicon-warning-sign\"></span>\r");t.b("\n" + i);t.b("                        <span class='bonustip-value-removed'>");t.b(t.v(t.d("Strings.COMMON.NOT_IN_TEAM",c,p,0)));t.b("</span>\r");t.b("\n" + i);};});c.pop();}});c.pop();}if(!t.s(t.d("bonusTips.gldboot",c,p,1),c,p,1,0,0,"")){t.b("                        <span class=\"font-light color-grey-profile\">");t.b(t.v(t.d("Strings.PLAYER_DETAILS.NO_BONUS_TIPP",c,p,0)));t.b("</span>\r");t.b("\n" + i);};t.b("                </div>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-6\">\r");t.b("\n" + i);if(t.s(t.f("hasTips",c,p,1),c,p,0,8569,9734,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("                <div class=\"panel-heading\">\r");t.b("\n" + i);t.b("                    <h3 class=\"panel-title clearfix\">\r");t.b("\n" + i);if(t.s(t.f("isOwner",c,p,1),c,p,0,8752,8922,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                        ");t.b(t.v(t.d("Strings.PLAYER_DETAILS.MY_TIPS",c,p,0)));t.b(" <a href=\"#matches\" class=\"float-right\"><span class=\"wmicon icon-edit-alt\"></span></a>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("isOwner",c,p,1),c,p,1,0,0,"")){t.b("                        ");t.b(t.v(t.f("nickName",c,p,0)));t.b(t.v(t.d("Strings.PLAYER_DETAILS.GENITIVE_TIPS",c,p,0)));t.b("\r");t.b("\n" + i);};t.b("                   </h3>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("                <ul class=\"list-group\">\r");t.b("\n" + i);if(t.s(t.f("matches",c,p,1),c,p,0,9207,9667,"{{ }}")){t.rs(c,p,function(c,p,t){if(!t.s(t.f("isOwner",c,p,1),c,p,1,0,0,"")){t.b("                            <li class=\"list-group-item\">\r");t.b("\n" + i);t.b(t.rp("<tiptemplate0",c,p,"                                "));t.b("                            </li>\r");t.b("\n" + i);};if(t.s(t.f("isOwner",c,p,1),c,p,0,9465,9635,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                            <li class=\"list-group-item\">\r");t.b("\n" + i);t.b(t.rp("<tiptemplate1",c,p,"                                "));t.b("                            </li>\r");t.b("\n" + i);});c.pop();}});c.pop();}t.b("                </ul>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("hasTips",c,p,1),c,p,1,0,0,"")){t.b("            <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("                <div class=\"panel-heading\">\r");t.b("\n" + i);t.b("                    <h3 class=\"panel-title clearfix\">\r");t.b("\n" + i);t.b("                        ");t.b(t.v(t.d("Strings.PLAYER_DETAILS.NO_MATCH_TIPS",c,p,0)));t.b("\r");t.b("\n" + i);t.b("                   </h3>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);};t.b("    </div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("<div class='sub-footer'>\r");t.b("\n" + i);t.b("    <div class=\"btn-group btn-group-lg btn-group-justified\">\r");t.b("\n" + i);if(!t.s(t.f("isOwner",c,p,1),c,p,1,0,0,"")){t.b("            <div class=\"btn-group\">\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"invite\" class=\"btn footer-btn active\" value=\"");t.b(t.v(t.d("Strings.PLAYER_DETAILS.INVITE",c,p,0)));t.b("\"><span class=\"font-h4 glyphicon glyphicon-plus-sign\"></span><br/><span class=\"content-font-small\">");t.b(t.v(t.d("Strings.PLAYER_DETAILS.INVITE_USER",c,p,0)));t.b("</span></button>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);};if(t.s(t.f("isOwner",c,p,1),c,p,0,10585,11228,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"btn-group\">\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"change-password\" class=\"btn footer-btn active\" value=\"");t.b(t.v(t.d("Strings.COMMON.CHANGE_PWD",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("                    <span class=\"font-h4 wmicon icon-key\"></span><br/><span class=\"content-font-small\">");t.b(t.v(t.d("Strings.COMMON.CHANGE_PWD",c,p,0)));t.b("</span>\r");t.b("\n" + i);t.b("                </button>\r");t.b("\n" + i);t.b("                <!-- EDIT Profile\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"edit\" class=\"btn footer-btn active\" value=\"Edit\">\r");t.b("\n" + i);t.b("                    <span class=\"font-h4 wmicon icon-edit-alt\"></span><br/><span class=\"content-font-small\">Edit</span>\r");t.b("\n" + i);t.b("                </button>-->\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);});c.pop();}if(t.s(t.f("isOwner",c,p,1),c,p,0,11262,11606,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"btn-group\">\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"logout\" class=\"btn navbar-fixed-bottom footer-btn active\" value=\"");t.b(t.v(t.d("Strings.COMMON.LOGOUT",c,p,0)));t.b("\"><span class=\"font-h4 wmicon icon-logout-1\"></span><br/><span class=\"content-font-small\">");t.b(t.v(t.d("Strings.PLAYER_DETAILS.LOG_ME_OUT",c,p,0)));t.b("</span></button>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);});c.pop();}if(t.s(t.f("isAdmin",c,p,1),c,p,0,11640,12337,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"btn-group\">\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"delete\" class=\"btn navbar-fixed-bottom footer-btn active\" value=\"");t.b(t.v(t.d("Strings.COMMON.DELETE",c,p,0)));t.b("\"><span class=\"font-h4 wmicon icon-user-delete\"></span><br/><span class=\"content-font-small\">");t.b(t.v(t.d("Strings.PLAYER_DETAILS.DELETE_USER",c,p,0)));t.b("</span></button>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("            <div class=\"btn-group\">\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"logout\" class=\"btn navbar-fixed-bottom footer-btn active\" value=\"");t.b(t.v(t.d("Strings.PLAYER_DETAILS.LOGOUT_ADMIN",c,p,0)));t.b("\"><span class=\"font-h4 wmicon icon-logout-1\"></span><br/><span class=\"content-font-small\">");t.b(t.v(t.d("Strings.PLAYER_DETAILS.LOGOUT_USER",c,p,0)));t.b("</span></button>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);});c.pop();}t.b("    </div>\r");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {"<tiptemplate0":{name:"tiptemplate", partials: {}, subs: {  }},"<tiptemplate1":{name:"tiptemplate", partials: {}, subs: {  }}}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/*global define*/
define('models/bonustip',[
	'underscore',
	'backbone',
	'models/settings'
], function (_, Backbone,Settings) {
	

	var BonusTipModel = Backbone.Model.extend({
		url: function() {  
		  	return  Settings.get("server") + '/rest/v2/bonustips'+(this.get("id")?'/'+this.get("id"):""); 
		}
	});

	return BonusTipModel;
});

/*global define*/
define('collections/bonustips',[
	'underscore',
	'backbone',
	'models/bonustip',
	'models/settings',
	'util/context',
], function (_, Backbone, BonusTip, Settings,Context) {
	

	var BonusTipsCollection = Backbone.Collection.extend({
		// Reference to this collection's model.
		model: BonusTip,
		url:function (){ 
			Context.bonusTipsIterationToken=Context.bonusTipsIterationToken||new Date().getTime();
			return  Settings.get("server") + "/rest/v2/bonustips?_it="+Context.bonusTipsIterationToken
		},
		reload:function(options){
			Context.bonusTipsIterationToken=new Date().getTime();
		  	return this.fetch.apply(this,arguments);
		  },
	});

	return BonusTipsCollection;
});

/*global define*/
define('collections/teamplayers',[
	'underscore',
	'backbone',
	'models/settings'
], function (_, Backbone, Settings) {
	

	var TeamPlayerCollection = Backbone.Collection.extend({
		url:function (){ return  Settings.get("server") + "/rest/common/teamplayer"},
	});

	return new TeamPlayerCollection();
});

/*global define*/
define('views/playerdetails',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/playerdetails',
	'models/settings',
	'collections/tips',
	'collections/bonustips',
	'collections/results',
	'collections/teamplayers',
	'util/context',
	'util/message',
	'models/player',
	'util/country',
	'hgn!templates/matchsmall',
	'moment',
	'util/wait',
	"i18n!nls/strings",
	'bootstrap'
], function ($, _, Backbone, BaseView, UserDetailsTemplate,SettingsModel,
	TipsCollection,	BonusTipsCollection, Results,TeamPlayers,Context,MessageHelper,PlayerModel,CountryMapper,matchSmallTemplate,moment,Wait,Strings) {
	

	var View = BaseView.extend({

		titel : Strings.COMMON.PLAYER_DETAILS,
		subView: true,
		el: '#wmapp',

		template: UserDetailsTemplate,

		events: {
			'click #edit':		'edit',
			'click #delete':	'deleteUser',
			'click #logout':	'logoutUser',
			'load #avatar': 	'avatarLoaded',
			'click #invite':    'invite',
			'click #change-password':'changePassword',
			'click .match-tip' : 'openTip'
		},
		openTip:function(element){
			if (this.player !== Context.player){
				//Can not edit tips for others
				return;
			}
			var matchId=$(element.target).closest('.match-tip').data("match-id");
		//	if (this.canCreateTip(matchId)){
				window.location.hash = '#matchtip/'+matchId;
		//	}
		},
		changePassword:function(){
			Context.requestLoginOnChangePassword=true;
			Context.Router.navigate("#changepwd", {trigger: true, replace: false});
		},
		invite:function(){
			window.location.hash = '#invitetomygroup/'+this.player.get('id');
		},
		avatarLoaded:function(){
			 window.URL.revokeObjectURL(this.$("#avatar")[0].src);
		},
		refresh:function(){
			this.updateOptions($.extend({reload:true},this.options));
		},
		updateOptions:function(options){
			Wait.start();
			var self=this,
				fetchName=options.reload?"reload":"fetch";
			this.options=options;
			if (!options || !options.id||options.id==Context.player.get('id')){
				var deferreds=[];
				this.player=Context.player;
				this.bonusTips=new BonusTipsCollection();
				if (options.reload){
					deferreds.push(Context.player.reload().fail(function(data,ajax,error,message){
						MessageHelper.handleError(ajax,Strings.ERROR_CODE["PLAYER_DETAILS.ERR.FAILED_TO_FETCH"],this);
					}));
				}
				deferreds.push(this.bonusTips[fetchName]().fail(function(data,ajax,error,message){
						MessageHelper.handleError(ajax,Strings.BONUSTIPS.FAILED_TO_LOAD,this);
				}));
				deferreds.push(TipsCollection[fetchName]().fail(function(data,ajax,error,message){
						MessageHelper.handleError(ajax,Strings.MATCHTIPS.FAILED_TO_LOAD,this);
				}));
				if (!TeamPlayers.length){
					deferreds.push(TeamPlayers.fetch().fail(function(data,ajax,error,message){
						MessageHelper.handleError(ajax,Strings.BONUSTIPS.FAILED_TO_LOAD_TEAM_PLAYERS,this);
				}));
				}
				$.when.apply($,deferreds)
				.done(_.bind(this.initForPlayer,this))
				.always(function(){Wait.stop();});
			}else{
				TeamPlayers.fetch().done(_.bind(function(){
					this.player=new PlayerModel({playerId:options.id,includeTips:true});
					this.player.reload({ 
							prefill: true
					})
					.progress(_.bind(this.initForPlayer,this))
					.done(_.bind(this.initForPlayer,this))
					.fail(function(data,ajax,error,message){
						MessageHelper.handleError(ajax,Strings.ERROR_CODE["PLAYER_DETAILS.ERR.FAILED_TO_FETCH"],this);
					}).always(function(){Wait.stop();});
				},this))
				.fail(function(data,ajax,error,message){
					MessageHelper.handleError(ajax,Strings.BONUSTIPS.FAILED_TO_LOAD_TEAM_PLAYERS,this);
					Wait.stop();
				})
			}
		},
		initialize: function (options) {
			this.updateOptions(options);
		},

		canViewPlayerTips:function(){
			return this.isMyself();
		},
		isMyself:function(){
		  		return this.player.get("playerId")==Context.player.get("playerId");
		},
		initForPlayer:function(){
			var matchtips=null,bonusTips={},self=this,tippedMatches=[];
			this.tipsMap={};
			if (this.player === Context.player){
				matchtips=TipsCollection.toJSON();
				_.each(this.bonusTips.toJSON(),function(bonusTip){
					bonusTips[bonusTip.tipKey]=bonusTip;
				},this);
			}else{
				matchtips=this.player.get("matchTips");
				_.each(this.player.get('bonusTips'),function(bonusTip){
					bonusTips[bonusTip.tipKey]=bonusTip;
				},this);
			}
			_.each(matchtips,function(tip){
				self.tipsMap[tip.matchId]=tip;
			});
			

			this.matchMap={};
			Results.each(function(match){
				var id=match.get('id');
				self.matchMap[id]=match.toJSON();
				if (self.tipsMap[id]){
					tippedMatches.push(self.matchMap[id]);
				}
			});

			this.$el.html(
				this.template(
					$.extend({},
						this.player.toJSON(),
						CountryMapper.getHelperMethods(),
						{
							matchPoints:this.player.get('overallPoints')-this.player.get('bonusPoints'),
							Strings:Strings,
							isAdmin:Context.player.isAdmin(),
							isOwner:this.isMyself(),
							canViewPlayerTips:this.canViewPlayerTips(),
							matches:tippedMatches,
							bonusTips:bonusTips,
							hasTips:matchtips&& matchtips.length>0,
							hasGroups:this.player.get('groups') &&  this.player.get('groups').length>0,
							CountryName:CountryMapper.getCountryName(this.player.get('countryCode')),
							CountryFlag:CountryMapper.getCountryFlag(this.player.get('countryCode')),
							KickOffTime:function(){
								return moment(new Date(this.kickOff+"Z")).format("LT")
							},
							KickOffDay:function(){
								return moment(new Date(this.kickOff+"Z")).format("LL")
							},
							WinnerCountryName:function(){
								return bonusTips.winner && CountryMapper.getCountryName(bonusTips.winner.tipValue)
							},
							SecondCountryName:function(){
								return bonusTips.second && CountryMapper.getCountryName(bonusTips.second.tipValue)
							},
                            WinnerGoldenBallName:function(){
								var name= bonusTips.gldball && TeamPlayers.findWhere({id:bonusTips.gldball.tipValue});
								return name && name.toJSON()
							},
                            WinnerGoldenGloveName:function(){
								var name= bonusTips.gldglove && TeamPlayers.findWhere({id:bonusTips.gldglove.tipValue});
								return name && name.toJSON()
							},
                            WinnerGoldenBootName:function(){
								var name= bonusTips.gldboot && TeamPlayers.findWhere({id:bonusTips.gldboot.tipValue});
								return name && name.toJSON()
							},
							PlayerCountry:function(){
								return CountryMapper.getCountryName(this.team)
							},
							isSet:function(){
								return this.status ==="set"
							},
							CanCreateTips:function(){
								return self.canCreateTip(this.id)
							},
							Tip:function(){
								return self.tipsMap[parseInt(this.id,10)];
							},
							HaveResults:function(){
								return this.scoreA!=-1 && this.scoreB!=-1;
							},
							isGroupOwner:function(){
								return this.ownerId == self.player.get('id');
							},
							groupRank:function(){
								return this.rank ===  2147483647 ?"":this.rank
							},
							isPrivate:function(){
								return this.publicGroup === false;
							}
						}
					),
					{tiptemplate:matchSmallTemplate.template}
				)
			);
			if(!this.trackScrollingElement) {
				this.trackScrolling(this.$(".sub-container"));
			}
			this.updateScroll();
			/*$.ajax({
				  type: "GET",
				  url:SettingsModel.get("server") +"/rest/player/"+this.player.get('id')+"/avatar",	
				  responseType:"blob"	  
				}).done(function(data){
					$("#avatar")[0].src=window.URL.createObjectURL(this.response);
				});
*/
		},
		canCreateTip:function(matchId){
			var match=this.matchMap[matchId], 
				tipsEnabledFrom=moment(new Date(match.tipsEnabledFrom+"Z")),
				tipsEnabledTo=moment(new Date(match.tipsEnabledTo+"Z")),
				currentDate=moment(Context.getCurrentDate());
			return currentDate.isBefore(tipsEnabledTo) && currentDate.isAfter(tipsEnabledFrom);
		},
		edit:function(){
			window.location.hash = '#editprofile';
		},
		deleteUser:function(){
			var self=this;
			MessageHelper.showInputMessage({header:Strings.PLAYER_DETAILS.DELETE_PLAYER_HEADER,message:Strings.PLAYER_DETAILS.DELETE_PLAYER_MESSAGE})
			.done(function(message){
				Wait.start();
				$.ajax({
					  type: "DELETE",
					  url:self.player.url(),
					  contentType: 'application/json',
					  data: {text:message}
				}).done(function(){
					MessageHelper.showInfo(Strings.PLAYER_DETAILS.PLAYER_DELETED);
				}).fail(function(ajax,error,message){
					MessageHelper.handleError(ajax,Strings.ERROR_CODE["PLAYER_DETAILS.ERR.FAILED_TO_DELETE"],this);
				}).always(function(){Wait.stop();});
			});
		},
		logoutUser:function(){
            if  (this.player.get("playerId")==Context.player.get("playerId")) {
                Context.logoff();
            } else {
                Wait.start();
                this.player.logout()
                    .done(function(){ MessageHelper.showInfo(Strings.PLAYER_DETAILS.LOGGED_OUT); })
                    .fail(function(ajax,error,message){
                    	MessageHelper.handleError(ajax,Strings.ERROR_CODE["PLAYER_DETAILS.ERR.LOGG_OUT_FAILED"],this);
                    })
                    .always(function(){Wait.stop();});
            }
		}
	});
	return View;
});


define("hgn!templates/groupdetails", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("\r");t.b("\n" + i);if(t.s(t.f("loading",c,p,1),c,p,0,14,91,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("<div class='loading-message'> ");t.b(t.v(t.d("Strings.GROUPDETAILS.LOAD_DETAILS",c,p,0)));t.b("</div>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("loading",c,p,1),c,p,1,0,0,"")){t.b("<div class=\"sub-header\">\r");t.b("\n" + i);t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);if(t.s(t.f("isCountryGroup",c,p,1),c,p,0,211,466,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("    <div class=\"col-xs-3 col-sm-3 col-md-2\">\r");t.b("\n" + i);t.b("        <img src=\"");t.b(t.v(t.f("countryGroupFlag",c,p,0)));t.b("\" class=\"img-flag\" alt=\"");t.b(t.v(t.f("countryGroupFlag",c,p,0)));t.b("\" />\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"col-xs-9 col-sm-5 col-md-4 country-name\">\r");t.b("\n" + i);t.b("        <h2>");t.b(t.v(t.f("groupName",c,p,0)));t.b("</h2>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("isCountryGroup",c,p,1),c,p,1,0,0,"")){t.b("    <div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);t.b("        <h2>");t.b(t.v(t.f("name",c,p,0)));t.b("\r");t.b("\n" + i);if(!t.s(t.f("publicGroup",c,p,1),c,p,1,0,0,"")){t.b("        <span class=\"wmicon icon-lock font-26 color-grey-profile padding-right-5\"></span>\r");t.b("\n" + i);};t.b("        </h2>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);};t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("<div class=\"sub-container ");if(!t.s(t.f("isCountryGroup",c,p,1),c,p,1,0,0,"")){t.b(" have-footer ");};t.b("\">\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);if(t.s(t.f("invitation",c,p,1),c,p,0,991,1117,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("        <div class=\"invitation-message padding-bottom-5\">");t.b(t.v(t.d("Strings.GROUPDETAILS.INVITED",c,p,0)));t.b(" ");t.b(t.v(t.f("created",c,p,0)));t.b(".<br/><br/></div>\r");t.b("\n" + i);});c.pop();}t.b("    \r");t.b("\n" + i);t.b("    <div class=\"padding-bottom-50\">\r");t.b("\n" + i);t.b("        <div class=\"col-xs-6\">\r");t.b("\n" + i);t.b("            <div class=\"row\">\r");t.b("\n" + i);t.b("                <span class=\"wmicon icon-avg font-26 color-grey-profile padding-right-5 pull-left\" />\r");t.b("\n" + i);if(t.s(t.f("hasGroupPoints",c,p,1),c,p,0,1378,1470,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                    <div class=\"font-h4 font-medium\">");t.b(t.v(t.f("avgPoints",c,p,0)));t.b("</div>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("hasGroupPoints",c,p,1),c,p,1,0,0,"")){t.b("                    <div class=\"font-h4 font-medium\">");t.b(t.v(t.f("avgPoints",c,p,0)));t.b("</div>\r");t.b("\n" + i);};t.b("            </div>\r");t.b("\n" + i);t.b("            <div class=\"row\">\r");t.b("\n" + i);t.b("                <div class=\"font-s font-condensed font-light color-grey-profile\">");t.b(t.v(t.d("Strings.GROUPDETAILS.AVG_POINTS",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("        <div class=\"col-xs-6\">\r");t.b("\n" + i);t.b("            <div class=\"row\">\r");t.b("\n" + i);t.b("                <span class=\"wmicon icon-users-2 font-26 color-grey-profile padding-right-5 pull-left\" />\r");t.b("\n" + i);t.b("                <div class=\"font-h4 font-medium\">");t.b(t.v(t.f("memberCount",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("            <div class=\"row\">\r");t.b("\n" + i);t.b("                <div class=\"font-s font-condensed font-light color-grey-profile\">");t.b(t.v(t.d("Strings.GROUPDETAILS.MEMBERS",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);if(!t.s(t.f("isCountryGroup",c,p,1),c,p,1,0,0,"")){t.b("        <div class=\"font-light padding-bottom-10\">");t.b(t.v(t.f("description",c,p,0)));t.b("</div>\r");t.b("\n" + i);};t.b("    <div class=\"panel panel-alpha\">\r");t.b("\n" + i);t.b("        <div class=\"panel-heading\">\r");t.b("\n" + i);t.b("            <h3 class=\"panel-title clearfix\">");t.b(t.v(t.d("Strings.GROUPDETAILS.MEMBERS",c,p,0)));t.b("</h3>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("        <div class=\"list-group\">\r");t.b("\n" + i);if(t.s(t.f("pageData",c,p,1),c,p,0,2671,3054,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                <a href=\"#player/");t.b(t.v(t.f("playerId",c,p,0)));t.b("\" class=\"list-group-item\">\r");t.b("\n" + i);t.b("                    <div class=\"row\">\r");t.b("\n" + i);t.b("                        <div class=\"col-xs-2\">");t.b(t.v(t.f("groupRank",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                        <div class=\"col-xs-10\">");t.b(t.v(t.f("nickName",c,p,0)));t.b(" <span class=\"badge badge-bg-primary pull-right\">");t.b(t.v(t.f("overallPoints",c,p,0)));t.b("</span></div>\r");t.b("\n" + i);t.b("                    </div>\r");t.b("\n" + i);t.b("                </a>\r");t.b("\n" + i);});c.pop();}t.b("        </div>\r");t.b("\n" + i);t.b("        <div class=\"btn-group btn-lg btn-group-justified\">\r");t.b("\n" + i);if(t.s(t.f("hasLess",c,p,1),c,p,0,3177,3458,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                        <div class=\"btn-group\">\r");t.b("\n" + i);t.b("                            <button type=\"button\" id=\"load-prev\" class=\"btn btn-lg btn-primary\"><span class=\"wmicon icon-left-open-big\"></span> ");t.b(t.v(t.d("Strings.COMMON.PREV",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("                        </div>\r");t.b("\n" + i);});c.pop();}t.b("\r");t.b("\n" + i);if(t.s(t.f("hasMore",c,p,1),c,p,0,3506,3788,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                        <div class=\"btn-group\">\r");t.b("\n" + i);t.b("                            <button type=\"button\" id=\"load-next\" class=\"btn btn-lg btn-primary\">");t.b(t.v(t.d("Strings.COMMON.NEXT",c,p,0)));t.b(" <span class=\"wmicon icon-right-open-big\"></span></button>\r");t.b("\n" + i);t.b("                        </div>\r");t.b("\n" + i);});c.pop();}t.b("                </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("<div class=\"sub-footer\">\r");t.b("\n" + i);t.b("    <div class=\"btn-group btn-group-lg btn-group-justified\">\r");t.b("\n" + i);if(t.s(t.f("isOwner",c,p,1),c,p,0,4003,4382,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"btn-group\">\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"manage-group\" class=\"btn footer-btn active\" value=\"");t.b(t.v(t.d("Strings.GROUPDETAILS.MANAGE",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("                    <span class=\"font-h4 glyphicon glyphicon-cog\"></span><br/><span class=\"content-font-small\">");t.b(t.v(t.d("Strings.GROUPDETAILS.MANAGE_GROUP",c,p,0)));t.b("</span>\r");t.b("\n" + i);t.b("                </button>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);});c.pop();}if(t.s(t.f("canLeaveGroup",c,p,1),c,p,0,4422,4805,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"btn-group\">\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"leave-group\" class=\"btn footer-btn active\" value=\"");t.b(t.v(t.d("Strings.GROUPDETAILS.LEAVE",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("                    <span class=\"font-h4 glyphicon glyphicon-minus-sign\"></span><br/><span class=\"content-font-small\">");t.b(t.v(t.d("Strings.GROUPDETAILS.LEAVE_GROUP",c,p,0)));t.b("</span>\r");t.b("\n" + i);t.b("                </button>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);});c.pop();}if(t.s(t.f("canJoinGroup",c,p,1),c,p,0,4850,5229,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"btn-group\">\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"join-group\" class=\"btn footer-btn active\" value=\"");t.b(t.v(t.d("Strings.GROUPDETAILS.JOIN",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("                    <span class=\"font-h4 glyphicon glyphicon-plus-sign\"></span><br/><span class=\"content-font-small\">");t.b(t.v(t.d("Strings.GROUPDETAILS.JOIN_GROUP",c,p,0)));t.b("</span>\r");t.b("\n" + i);t.b("                </button>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);});c.pop();}if(t.s(t.f("haveGroupJoinRequested",c,p,1),c,p,0,5283,5475,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"btn-group\">\r");t.b("\n" + i);t.b("                <div class=\"footer-btn active content-font-small content-center\">");t.b(t.v(t.d("Strings.GROUPDETAILS.ALREADY_REQ",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);});c.pop();}if(t.s(t.f("canDeleteGroup",c,p,1),c,p,0,5531,5938,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"btn-group\">\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"delete\" class=\"btn footer-btn active\" value=\"");t.b(t.v(t.d("Strings.COMMON.DELETE",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("                    <span class=\"font-h4 glyphicon glyphicon-remove-sign\"></span><br/><span class=\"content-font-small\">");t.b(t.v(t.d("Strings.GROUPDETAILS.DELETE_GROUP",c,p,0)));if(t.s(t.f("isAdmin",c,p,1),c,p,0,5854,5862,"{{ }}")){t.rs(c,p,function(c,p,t){t.b(" (Admin)");});c.pop();}t.b("</span>\r");t.b("\n" + i);t.b("                </button>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);});c.pop();}t.b("    </div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);};return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/*global define*/
define('models/group',[
	'underscore',
	'backbone',
	'models/settings',
	'util/context',
	'util/message',
	'i18n!nls/strings'
], function (_, Backbone,Settings,Context,MessageHelper,Strings) {
	

	var GroupModel = Backbone.Model.extend({

		groupIterationToken:function(){
			Context.groupIterationToken=Context.groupIterationToken||{};
			var id=this.get("id");
			if (id && !Context.groupIterationToken[id]){
				Context.groupIterationToken[""+id]=new Date().getTime();
			}
			return Context.groupIterationToken[""+id];
		},
		invalidate:function(){
			Context.groupIterationToken=Context.groupIterationToken||{};
			Context.groupIterationToken[""+this.get("id")]=new Date().getTime();
		},
		reload:function(options){
			this.invalidate();
		  	return this.fetch.apply(this,arguments);
		  },
		url: function() { 
		  	return  Settings.get("server") + '/rest/groups/'+this.get("id")+"?_it="+this.groupIterationToken()+"&includemembers=false"; 
		},
		
		isMember:function(){
	  		return Context.player.isMemberOf(this.get("id"))
		},
		removePlayer:function(playerId){
			return $.ajax({
			  type: "DELETE",
			  url:Settings.get("server") +"/rest/groups/"+this.get('id')+"/members/"+playerId,
			  contentType: 'application/json'
			});
			
		},
		/*Check if i send a request to join the group*/
		invitationRequested:function(){
			return Context.player.getGroupJoinRequested();
		},
		/* Check if i was invited to this group*/
		invitationRecieved:function(){
			return  Context.player.getGroupInvited(this.get("id"));
		},
		isOwner:function(){
			return this.get('ownerId')===Context.player.get('id');
		},
		isCountryGroup:function(){
			return this.get('name').substring(0, 8) === 'country_'
		},
		isOrgGroup:function(){
			return this.get('name').substring(0, 4) === 'org_'
		},
		invitePlayer:function(playerId,message){
			var def=new $.Deferred(),
				self=this,
				reload=function(){
					self.reload()
						.done(function(){def.resolve();})
						.fail(function(data,ajax,error,message){
							def.resolve();
							MessageHelper.handleError(ajax,Strings.GROUP.INVITE_FAILED_REFRESH,this);
						});
				};
			 $.ajax({
			  type: "POST",
			  url:Settings.get("server") +"/rest/groups/"+this.get('id')+"/members/"+playerId,
			  contentType: 'application/json',
			  data: {text:message}
			}).done(function(){
				reload();
			})
			.fail(function(ajax,error,message){
				if (ajax.status === 201){
					//invitation sent
					reload();
				}else{
					def.reject.apply(this,arguments);
				}
			});
			return def.promise();
		},
		getGroupMembershipRequest:function(playerId){
		  	var foundInvitation=null;
		  	_.each(this.get('groupJoinRequests'),function(invitation){
		  		//do not use === since input is string
		  		if (invitation.playerId == playerId){
		  			foundInvitation=invitation;
		  		}
		  	});
		  	return foundInvitation;

		  }
	});

	return GroupModel;
});

/*global define*/
define('views/groupdetails',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/groupdetails',
	'models/settings',
	'util/context',
	'util/message',
	'util/country',
	'models/group',
	'util/wait',
	"i18n!nls/strings",
	'bootstrap'
], function ($, _, Backbone,  BaseView, groupdetailsTemplate,Settings,Context,
	MessageHelper,CountryMapper,GroupModel,Wait,Strings) {
	

	var View = BaseView.extend({

		titel : Strings.GROUPDETAILS.TITLE,
		subView: true,
		el: '#wmapp',

		template: groupdetailsTemplate,

		events: {
			'click #delete':	'deleteGroup',
			'click #manage-group':'manageGroup',
			'click #leave-group':'leaveGroup',
			'click #join-group':'joinGroup',
			'click #load-next':'loadNext',
			'click #load-prev':'loadPrev'

		},
		currentStart:0,
		pageSize:10,
		initialize: function (options) {
			this.group=new GroupModel({id:options.id});
			if (options.currentStart){
				this.currentStart=parseInt(options.currentStart,10);
			}
		},
		loadData:function(){
			return 	$.when(
				this.loadPage().done(_.bind(function(pageData){
				this.pageData=pageData;
			},this)),
				this.loadGroupDetails()
			);
		},
		start:function(){
			this.renderGroupDetails();
		},
		updateOptions:function(options){
			if (options.currentStart){
				this.currentStart=parseInt(options.currentStart,10);
			}
			if (options.id !=this.group.get('id')){
				this.group=new GroupModel({id:options.id});
				this.loadData().done(_.bind(this.renderGroupDetails,this));
			}else{
				this.loadAndRenderPage();
			}

		},
		loadNext:function(event){
			this.currentStart+=this.pageSize;
			this.loadAndRenderPage();
		},
		loadPrev:function(event){
			if (this.currentStart>=this.pageSize){
				this.currentStart-=this.pageSize;
				this.loadAndRenderPage();
			}
		},
		loadGroupDetails:function(){
			return this.group.fetch()
			.fail(function(data,ajax,error,message){
				MessageHelper.handleError(ajax,Strings.ERROR_CODE["GROUPDETAILS.ERR.FAILED_FETCH"],this);
			});
		},
		loadAndRenderPage:function(){
			Wait.start();
				this.loadPage().always(function(){Wait.stop();})
				.done(_.bind(function(pageData){
					this.pageData=pageData;
					this.renderGroupDetails();
				},this));
		},
		loadPage: function () {
			return $.ajax({
				type: "GET",
				url:Settings.get("server") +"/rest/groups/"+this.group.get("id")+"/members?from="+this.currentStart+"&size="+this.pageSize+"&_it="+this.group.groupIterationToken()

			})
			.fail(function(ajax,error,message){
				MessageHelper.handleError(ajax,Strings.ERROR_CODE["RANKINGS.ERR.LOADING_FAILED"],this);
			});
		},
		refresh:function(){
			this.group.invalidate();
			Wait.start();
			return this.loadData()
			.done(_.bind(function(pageData){
				this.renderGroupDetails();
			},this))
			.always(function(){Wait.stop();});
		},

		renderGroupDetails:function(){

			if (this.pageData.length>0){
					this.viewModel=$.extend(
					{},
					this.group.toJSON(),
					{
						pageData:this.pageData,
						hasMore:this.pageData.length===this.pageSize,
						hasLess:this.currentStart!==0,
						Strings:Strings,
						isAdmin:Context.player.isAdmin(),
						isMember:this.group.isMember(),
						isOwner:this.group.isOwner(),
						accepted:_.where(this.group.get('members'), {memberStatus: "accepted"}),
						invitation:Context.player.getGroupInvited(this.group.get('id')),
						isCountryGroup:this.isCountryGroup(),
						canLeaveGroup:this.canLeaveGroup(),
						canJoinGroup:this.canJoinGroup(),
						countryGroupFlag:this.getCountryFlag(),
						countryGroupName:this.getCountryName(),
						canDeleteGroup:this.canDeleteGroup(),
						groupName:this.getGroupName(),
						haveGroupJoinRequested:this.haveGroupJoinRequested(),
						groupRank:function(){
							return this.rank ===  2147483647 ?"-":this.rank
						}
					}
				);
					this.$el.html(this.template(this.viewModel));
					Context.Router.navigate("#group/"+this.group.get('id')+"/"+this.currentStart, {trigger: false, replace: false});
			}else{
				if (this.currentStart>=this.pageSize){
					this.currentStart-=this.pageSize;
					this.viewModel.hasMore=false;
					this.$el.html(this.template(this.viewModel));
				}
			}
		},
		getGroupName:function(){
			return this.isCountryGroup()?this.getCountryName():this.group.get('name');
		},
		canDeleteGroup:function(){
			return !this.isCountryGroup() && (Context.player.isAdmin() || this.group.isOwner());
		},
		getCountryName:function (){
			return CountryMapper.getCountryName(this.group.get('name').substring(8));
		},
		getCountryFlag:function (){
			return CountryMapper.getCountryFlag(this.group.get('name').substring(8));
		},
		haveGroupJoinRequested:function(){
			return Context.player.getGroupJoinRequested(this.group.get('id'));
		},
		isCountryGroup:function(){
			return this.group.isCountryGroup();
		},
		isOrgGroup:function(){
			return this.group.isOrgGroup();
		},
		canLeaveGroup:function(){
			return !this.isCountryGroup() &&
				 !this.group.isOwner() &&
				 this.group.isMember();
		},
		canJoinGroup:function(){
			return !this.isCountryGroup() && !this.group.isMember()&& !this.haveGroupJoinRequested()
					&&!this.group.isOwner();
		},
		deleteGroup:function(event){
			var self=this;
			MessageHelper.showInputMessage({header:Strings.GROUPDETAILS.DELETE_GROUP,message:Strings.GROUPDETAILS.DELETE_GROUP_REASON})
			.done(function(message){
				Wait.start(event.target);
				$.ajax({
				  type: "DELETE",
				  url:self.group.url(),	
				  contentType: 'application/json',
				  data: {text:message}	  
				})
					.done(function(){ 
						//invalidate possibly cached ranking results
						Context.groupRankIterationToken=new Date().getTime();
						Context.player.reload().done(function(){
							Wait.stop();
							window.location.hash='#groups';
							MessageHelper.showInfo(Strings.EDITPROFILE.GROUP_DELETE); 
						})
						.fail(function(ajax,error,message){
							Wait.stop();
							window.location.hash='#groups';
							MessageHelper.handleError(ajax,Strings.EDITPROFILE.GROUP_DELETE_REFRESH_FAILED,this);
						})
						
						
					})
					.fail(function(ajax,error,message){
						Wait.stop();
						MessageHelper.handleError(ajax,Strings.ERROR_CODE["EDITPROFILE.ERR.FAILED_GROUP"],this);
					});
			});
		},
		manageGroup:function(){
			Context.Router.navigate("#managegroup/"+this.group.id+"/home", {trigger: true,replace: false});
		},
		leaveGroup:function(event){
			var self=this;
			MessageHelper.showConfirmationMessage({header:Strings.GROUPDETAILS.LEAVING_GROUP,message:Strings.GROUPDETAILS.LEAVING_GROUP_MESSAGE})
			.done(function(){
				Wait.start(event.target);
				$.ajax({
				  type: "DELETE",
				  url:Settings.get("server") +"/rest/groups/"+self.group.get('id')+"/members/"+Context.player.get('id'),		  
				}).done(function(data){
					Context.groupsSearch={};
					var myGroups=Context.player.get('groups'),
						existingGroup=_.findWhere(myGroups,{groupId:self.group.get('id')});
					Context.player.set('groups',_.without(myGroups,existingGroup));
					self.refresh().always(function(){
						Wait.stop();
					});
				}).fail(function(ajax,error,message){
					Wait.stop();
					MessageHelper.handleError(ajax,Strings.ERROR_CODE["GROUPDETAILS.ERR.FAILED_REMOVE"],this);
				});
			});

		},
		updateInternals:function(){
			var self=this;
			Context.player.get('groups').push({
				groupId:this.group.get('id'), 
				groupName:this.group.get('name')
			})
			return $.when(
				Context.player.reload().fail(function(data,ajax,error,message){
					MessageHelper.handleError(ajax,Strings.PLAYER_DETAILS.FAILED_TO_REFERSH,this);
				}),
				self.refresh()
				);
		},
		joinGroup:function(event){
			var self=this,
				invitation=Context.player.getGroupInvited(this.group.get("id"));
			
			if (invitation){
				var tempInvitation=_.clone(invitation)
				tempInvitation.status="accepted";
				Wait.start(event.target);
				$.ajax({
					  type: "PUT",
					  url:Settings.get("server") +"/rest/groups/"+self.group.get('id')+"/members/"+Context.player.get('id'),
					  contentType: 'application/json',
					  data: tempInvitation
					}).done(function(data){
						self.updateInternals().always(function(){
							Wait.stop();
						});
					}).fail(function(ajax,error,message){
						if (ajax.status === 201){
							//Created
							self.updateInternals().always(function(){
								Wait.stop();
							});
						}else if(ajax.status === 404 && ajax.responseJSON.code==='RESOURCE.NOT_FOUND'){
							MessageHelper.showError(Strings.GROUPDETAILS.GROUP_DELETED);
							Wait.stop();
						}
						else{
							Wait.stop();
							MessageHelper.handleError(ajax,Strings.ERROR_CODE["GROUPDETAILS.ERR.FAILED_JOIN"],this);
						}
					});
			}else{
			//	MessageHelper.showInputMessage({header:Strings.GROUPDETAILS.GROUP_REQUEST,message:Strings.GROUPDETAILS.GROUP_REQUEST_MESSAGE})
			//	.done(function(message){
					var message="",
						reload=function(){
							Context.player.reload().always(function(){
								self.renderGroupDetails();
								Wait.stop();
								
							}).done(function(){
								MessageHelper.showInfo(Strings.GROUPDETAILS.ALREADY_REQ);
							}).fail(function(data,ajax,error,message){
								MessageHelper.handleError(ajax,Strings.GROUPDETAILS.JOINED_REFRESH_FAILED,this);
							});
						}
					Wait.start(event.target);
					$.ajax({
					  type: "POST",
					  url:Settings.get("server") +"/rest/groups/"+self.group.get('id')+"/members/"+Context.player.get('id'),
					  contentType: 'application/json',
					  data: {text:message}
					}).done(function(data){
							reload();
					}).fail(function(ajax,error,message){
						if (ajax.status === 201){
							//Created
							
							if (self.group.get('publicGroup')===true){
								self.updateInternals().always(function(){
									Wait.stop();
									MessageHelper.showInfo(Strings.GROUP.JOINED);
									Context.groupsSearch={};
								});
								

							}else{
								reload();
							}
						}else{
							Wait.stop();
							MessageHelper.handleError(ajax,Strings.ERROR_CODE["GROUPDETAILS.ERR.FAILED_REQUEST"],this);
						}
					});
			//	});
			}
		}
		
	});

	return View;
});

define("hgn!templates/matches", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class=\"btn-group btn-group-lg btn-group-justified sub-header-nav\">\r");t.b("\n" + i);t.b("  <div class=\"btn-group\">\r");t.b("\n" + i);t.b("    <button type=\"button\" id=\"matches-by-date\" class=\"btn head-btn\">");t.b(t.v(t.d("Strings.MATCH.GROUP_STAGE_DATE",c,p,0)));t.b("<br /><span class=\"content-font-small\">");t.b(t.v(t.d("Strings.MATCH.GROUP_STAGE_DATE_SUBLINE",c,p,0)));t.b("</span></button>\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("  <div class=\"btn-group\">\r");t.b("\n" + i);t.b("    <button type=\"button\" id=\"matches-by-group\" class=\"btn head-btn\">");t.b(t.v(t.d("Strings.MATCH.GROUP_STAGE_GROUP",c,p,0)));t.b("<br /><span class=\"content-font-small\">");t.b(t.v(t.d("Strings.MATCH.GROUP_STAGE_GROUP_SUBLINE",c,p,0)));t.b("</span></button>\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("  <div class=\"btn-group\">\r");t.b("\n" + i);t.b("    <button type=\"button\" id=\"matches-second-stage\" class=\"btn head-btn\">");t.b(t.v(t.d("Strings.MATCH.SECOND_STAGE",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("<div id=\"subcontainer\" class='sub-container'>\r");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});


define("hgn!templates/matchesbygroup", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");if(t.s(t.f("results",c,p,1),c,p,0,12,553,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("    <div class=\"col-xs-12 col-sm-6 col-md-4\" data-group=\"");t.b(t.v(t.f("groupBy",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("        <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("            <div class=\"panel-heading panel-heading-light\">\r");t.b("\n" + i);t.b("                <h3 class=\"panel-title panel-title-light clearfix\">");t.b(t.v(t.f("groupBy",c,p,0)));t.b("</h3>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("            <ul class=\"list-group\">\r");t.b("\n" + i);if(t.s(t.f("matches",c,p,1),c,p,0,360,492,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                    <li class=\"list-group-item\">\r");t.b("\n" + i);t.b(t.rp("<match0",c,p,"                        "));t.b("                    </li>\r");t.b("\n" + i);});c.pop();}t.b("            </ul>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);});c.pop();}return t.fl(); },partials: {"<match0":{name:"match", partials: {}, subs: {  }}}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/*global define*/
define('views/matches',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/matches',
	'hgn!templates/matchesbygroup',
	'hgn!templates/matchsmall',
	'models/settings',
	'util/context',
	'util/message',
	'collections/results',
	'collections/tips',
	'util/country',
	'moment',
	'i18n!nls/strings',
	'util/wait',
	'bootstrap'
], function ($, _, Backbone, BaseView, 
	matchesTemplate,matchseByGroupTemplate,matchSmallTemplate,
	SettingsModel,Context,MessageHelper,Results,Tips,CountryMapper,moment,Strings,Wait) {
	

	var View = BaseView.extend({
		titel : Strings.MATCH.TITLE,
		subView: true,
		el: '#wmapp',

		template: matchesTemplate,
		byGroupTemplate:matchseByGroupTemplate,
		events: {
			'click #matches-by-date':		'matchesByDate',
			'click #matches-by-group':		'matchesByGroup',
			'click #matches-second-stage':	'matchesSecondStage',
			'click .match-tip': 			'openTip',
		},
		refresh:function(){
			Wait.start();
			$.when(
				Tips.reload(),
				Results.reload()
			).done(_.bind(function(){
				this.prepareTipsMap();
				this.updateOptions({tab:this.options.tab},true);
			},this))
			.fail(function(ajax,error,message){
				MessageHelper.handleError(ajax,Strings.ERROR_CODE["MATCHES.ERR.FAILED_TO_REFRESH"],this);
				Wait.stop();
			})
			.always(function(){Wait.stop();});
		},
		openTip:function(element){
			var matchId=$(element.target).closest('.match-tip').data("match-id");
			window.location.hash = '#matchtip/'+matchId;
		},
		matchesByDate:function(event){
			if (event){
				//if ($(event.target).hasClass('active')){
				//	return;
				//}
				this.$(".btn.active").removeClass('active');
				this.$(event.target).addClass('active');
			}
			var results=Results.toJSON()||[],
				lastPlayed=null,
				currentDate=moment(Context.getCurrentDate()).zone(240),
				groupedByDate=_.groupBy(results,function(match){
					var resultDate=moment(new Date(match.kickOff+"Z")).zone(Context.timezoneMap[match.location]),
						resultDay=resultDate.format("L");
					if (resultDate.isBefore(currentDate)){
						lastPlayed=resultDay;
					}
					return resultDay;
				}),
				groupedByDateList=[];
			
			_.each(groupedByDate,function(matches,date){
				if (!_.contains(['R16','QRT','SF','3RD','FIN'], matches[0].type)){
					groupedByDateList.push({groupBy:date,matches:matches});
				}
			})

			
			this.renderDetails(groupedByDateList);
			Context.Router.navigate("#matches/byDate", {trigger: false, replace: event.replace});
			if (Context.getScrollPosition(window.location.hash)){
				this.updateScroll();
			}else{
				var el=this.$("[data-group='"+lastPlayed+"']")
				if (el.length>0){
					el[0].scrollIntoView( true );				
				}
			}
			
			this.options.tab = 'byDate';
		},
		matchesByGroup:function(event){
			if (event){
				//if ($(event.target).hasClass('active')){
				//	return;
				//}
				this.$(".btn.active").removeClass('active');
				this.$(event.target).addClass('active');
			}
			var self=this;
			var results=Results.toJSON()||[],
				groupedByGroup=_.groupBy(results,function(match){
					return Strings.GAME_TYPE[match.type];
				}),
				groupedByGroupList=[];
			
			_.each(groupedByGroup,function(matches,group){
				if (!_.contains(['R16','QRT','SF','3RD','FIN'], matches[0].type)){
					groupedByGroupList.push({groupBy:group,matches:matches});
				}
			});
			this.renderDetails(groupedByGroupList);
			Context.Router.navigate("#matches/byGroup", {trigger: false, replace: event.replace});
			this.updateScroll();
			this.options.tab = 'byGroup';
		},
		prepareTipsMap:function(){
			this.tipsMap={};
			var self=this;
			Tips.each(function(tip){
				self.tipsMap[tip.get('matchId')]=tip.toJSON();
			});
		},
		renderDetails:function(results){
			var self=this;
			var data=$.extend({Strings:Strings},
				CountryMapper.getHelperMethods(),
				{
					results:results,
					CanCreateTips:function(){
						var tipsEnabledFrom=moment(new Date(this.tipsEnabledFrom+"Z")),
							tipsEnabledTo=moment(new Date(this.tipsEnabledTo+"Z")),
							currentDate=moment(Context.getCurrentDate());
							return currentDate.isBefore(tipsEnabledTo) && currentDate.isAfter(tipsEnabledFrom);
					},
					KickOffTime:function(){
						return moment(new Date(this.kickOff+"Z")).format("LT")
					},
					KickOffDay:function(){
						return moment(new Date(this.kickOff+"Z")).format("LL")
					},
					HaveResults:function(){
						return this.scoreA!=-1 && this.scoreB!=-1;
					},
					Tip:function(){
						return self.tipsMap[parseInt(this.id,10)];
					},
					matchType:function(){
							return Strings.GAME_TYPE[this.type];
					},
					 extention:function(){
                        	var x=this.scoreExtensionDetail
                        	if (x !=  null && x.psoScoreText != null) {
								return x.psoScoreA+":"+x.psoScoreB +" "+Strings.COMMON.PSO;
							}
                        },
					currentDate:Context.getCurrentDate()
				});
			this.$("#subcontainer").html(this.byGroupTemplate(data,{
				Strings:Strings,
				match:matchSmallTemplate.template
			}));
		},
		matchesSecondStage:function(event){
			if (event){
				//if ($(event.target).hasClass('active')){
				//	return;
				//}
				this.$(".btn.active").removeClass('active');
				this.$(event.target).addClass('active');
			}
			var self=this;
			var results=Results.toJSON()||[],matchid,
				currentDate=moment(Context.getCurrentDate()).zone(240),
				groupedByGroup=_.groupBy(results,function(match){
					var resultDate=moment(new Date(match.kickOff+"Z")).zone(Context.timezoneMap[match.location]);
					if (resultDate.isBefore(currentDate)){
						matchid=match.id;
					}
					return Strings.GAME_TYPE[match.type];
				}),
				groupedByGroupList=[];
			
			_.each(groupedByGroup,function(matches,group){
				if (_.contains(['R16','QRT','SF','3RD','FIN'], matches[0].type)){
					groupedByGroupList.push({groupBy:group,matches:matches});
				}
			});
			this.renderDetails(groupedByGroupList);
			Context.Router.navigate("#matches/second", {trigger: false, replace: event.replace});
			if (Context.getScrollPosition(window.location.hash)){
				this.updateScroll();
			}else{
				var el=this.$("[data-match-id='"+matchid+"']")
				if (el.length>0){
					el[0].scrollIntoView( true );				
				}
			}
			this.options.tab = 'second';
		},
		initialize: function (options) {
			this.options=options;
			this.prepareTipsMap();
			this.$el.html(this.template({Strings:Strings}));
			this.trackScrolling(this.$("#subcontainer"));
			this.updateOptions(options,true);
		},
		//Special method called by router to apply new options
		updateOptions:function(options,replace){
			
			if (!options||!options.tab||options.tab === 'byDate'){
				this.matchesByDate({target:this.$("#matches-by-date"),replace:replace});
			}else if (!options||!options.tab||options.tab === 'byGroup'){
				this.matchesByGroup({target:this.$("#matches-by-group"),replace:replace});
			}else{
				this.matchesSecondStage({target:this.$("#matches-second-stage"),replace:replace});
			}
		},
		
	});

	return View;
});

define("hgn!templates/matchtip", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div id=\"subcontainer\" class='sub-container'>\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);if(!t.s(t.f("hasResult",c,p,1),c,p,1,0,0,"")){t.b("    <div class=\"row\">\r");t.b("\n" + i);t.b("        <div class=\"col-xs-6 content-left match-goal-tip\">");t.b(t.v(t.f("CountryNameA",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("        <div class=\"col-xs-6 content-right match-goal-tip\">");t.b(t.v(t.f("CountryNameB",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"row\">&nbsp;</div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("    <div class=\"row\">\r");t.b("\n" + i);t.b("        <div class=\"col-xs-3 content-left\"><img src=\"");t.b(t.v(t.f("CountryFlagA",c,p,0)));t.b("\" class=\"img-flag\" alt=\"");t.b(t.v(t.f("CountryNameA",c,p,0)));t.b("\" /></div>\r");t.b("\n" + i);t.b("        <div class=\"col-xs-3 content-center\">\r");t.b("\n" + i);t.b("            <select id='goals-a' class=\"form-control font-digital\" ");if(!t.s(t.f("canDoTips",c,p,1),c,p,1,0,0,"")){t.b("disabled");};t.b(">\r");t.b("\n" + i);t.b("                  <option value=\"\"> </option>\r");t.b("\n" + i);t.b("                  <option value=\"0\">0</option>\r");t.b("\n" + i);t.b("                  <option value=\"1\">1</option>\r");t.b("\n" + i);t.b("                  <option value=\"2\">2</option>\r");t.b("\n" + i);t.b("                  <option value=\"3\">3</option>\r");t.b("\n" + i);t.b("                  <option value=\"4\">4</option>\r");t.b("\n" + i);t.b("                  <option value=\"5\">5</option>\r");t.b("\n" + i);t.b("                  <option value=\"6\">6</option>\r");t.b("\n" + i);t.b("                  <option value=\"7\">7</option>\r");t.b("\n" + i);t.b("                  <option value=\"8\">8</option>\r");t.b("\n" + i);t.b("                  <option value=\"9\">9</option>\r");t.b("\n" + i);t.b("            </select>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("        <div class=\"col-xs-3 content-center\">\r");t.b("\n" + i);t.b("            <select id='goals-b' class=\"form-control font-digital\" ");if(!t.s(t.f("canDoTips",c,p,1),c,p,1,0,0,"")){t.b("disabled");};t.b(">\r");t.b("\n" + i);t.b("                  <option value=\"\"> </option>\r");t.b("\n" + i);t.b("                  <option value=\"0\">0</option>\r");t.b("\n" + i);t.b("                  <option value=\"1\">1</option>\r");t.b("\n" + i);t.b("                  <option value=\"2\">2</option>\r");t.b("\n" + i);t.b("                  <option value=\"3\">3</option>\r");t.b("\n" + i);t.b("                  <option value=\"4\">4</option>\r");t.b("\n" + i);t.b("                  <option value=\"5\">5</option>\r");t.b("\n" + i);t.b("                  <option value=\"6\">6</option>\r");t.b("\n" + i);t.b("                  <option value=\"7\">7</option>\r");t.b("\n" + i);t.b("                  <option value=\"8\">8</option>\r");t.b("\n" + i);t.b("                  <option value=\"9\">9</option>\r");t.b("\n" + i);t.b("            </select>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("        <div class=\"col-xs-3 content-right\"><img src=\"");t.b(t.v(t.f("CountryFlagB",c,p,0)));t.b("\" class=\"img-flag\" alt=\"");t.b(t.v(t.f("CountryNameB",c,p,0)));t.b("\" /></div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"row\">&nbsp;</div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("    <div class=\"form-group\">\r");t.b("\n" + i);if(t.s(t.f("canDoTips",c,p,1),c,p,0,2179,2364,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <input id=\"save-tip\" type=\"button\" data-loading-text=\"");t.b(t.v(t.d("Strings.MATCHTIPS.SAVING",c,p,0)));t.b("\" class=\"btn btn-primary btn-lg btn-block\" value=\"");t.b(t.v(t.d("Strings.MATCHTIPS.SAVE",c,p,0)));t.b("\"/>\r");t.b("\n" + i);});c.pop();}if(t.s(t.f("isTipsToEarly",c,p,1),c,p,0,2406,2489,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div>");t.b(t.v(t.d("Strings.MATCHTIPS.CANNOT_YET",c,p,0)));t.b(" ");t.b(t.v(t.f("enabledFrom",c,p,0)));t.b("</div>\r");t.b("\n" + i);});c.pop();}if(t.s(t.f("isTipsToLate",c,p,1),c,p,0,2534,2619,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div>");t.b(t.v(t.d("Strings.MATCHTIPS.CANNOT_ANYMORE",c,p,0)));t.b(" ");t.b(t.v(t.f("enabledTo",c,p,0)));t.b("</div>\r");t.b("\n" + i);});c.pop();}t.b("    </div>\r");t.b("\n" + i);};if(t.s(t.f("hasResult",c,p,1),c,p,0,2680,3960,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("   <div class=\"row\">\r");t.b("\n" + i);t.b("        <div class=\"col-xs-6 content-left font-16\">");t.b(t.v(t.f("CountryNameA",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("        <div class=\"col-xs-6 content-right font-16\">");t.b(t.v(t.f("CountryNameB",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"row\">\r");t.b("\n" + i);t.b("        <div class=\"col-xs-3 content-left padding-top-5\"><img src=\"");t.b(t.v(t.f("CountryFlagA",c,p,0)));t.b("\" class=\"img-flag\" alt=\"");t.b(t.v(t.f("CountryNameA",c,p,0)));t.b("\" /></div>\r");t.b("\n" + i);t.b("        <div class=\"col-xs-6 content-center\">\r");t.b("\n" + i);t.b("                <div class=\"match-goal-results\">");t.b(t.v(t.f("scoreA",c,p,0)));t.b(":");t.b(t.v(t.f("scoreB",c,p,0)));if(t.s(t.f("extention",c,p,1),c,p,0,3155,3176,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("&nbsp;(");t.b(t.v(t.f("extention",c,p,0)));t.b(")");});c.pop();}t.b("<!-- <span class=\"wmicon icon-soccer pull-left\"></span>--></div>\r");t.b("\n" + i);if(t.s(t.f("tip",c,p,1),c,p,0,3280,3569,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                    <div class=\"match-goal-tip font-digital\">");t.b(t.v(t.f("scoreA",c,p,0)));t.b(":");t.b(t.v(t.f("scoreB",c,p,0)));t.b("<!-- <span class=\"wmicon icon-exchange pull-left\"></span>--></div>\r");t.b("\n" + i);t.b("                    <div class=\"\"><span class=\"badge badge-bg-primary\">");t.b(t.v(t.f("points",c,p,0)));t.b(" ");t.b(t.v(t.d("Strings.MATCH.POINTS",c,p,0)));t.b("</span></div>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("tip",c,p,1),c,p,1,0,0,"")){t.b("                    <div class=\"\">");t.b(t.v(t.d("Strings.MATCH.NOT_TIPPED",c,p,0)));t.b("<!-- <span class=\"wmicon icon-exchange pull-left\"></span>--></div>\r");t.b("\n" + i);};t.b("        </div>\r");t.b("\n" + i);t.b("        <div class=\"col-xs-3 content-right padding-top-5\"><img src=\"");t.b(t.v(t.f("CountryFlagB",c,p,0)));t.b("\" class=\"img-flag\" alt=\"");t.b(t.v(t.f("CountryNameB",c,p,0)));t.b("\" /></div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"row\">&nbsp;</div>\r");t.b("\n" + i);});c.pop();}t.b("\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("    <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("        <div class=\"panel-heading\">\r");t.b("\n" + i);t.b("            <h3 class=\"panel-title clearfix\">");t.b(t.v(t.d("Strings.MATCHTIPS.COMMUNITY_STATS",c,p,0)));t.b("</h3>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("        <div id=\"statistics\" class=\"panel-body\">\r");t.b("\n" + i);t.b("            ");t.b(t.v(t.d("Strings.MATCHTIPS.LOADING_STATS",c,p,0)));t.b(" <span class=\"font-h4 glyphicon glyphicon-refresh animate-spin\"></span>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);if(!t.s(t.f("canDoTips",c,p,1),c,p,1,0,0,"")){t.b("        <div id=\"charts\"  style=\"display:none\">\r");t.b("\n" + i);t.b("          <div>\r");t.b("\n" + i);t.b("        <canvas id=\"points\" class=\"points-chart\" width=\"150\" height=\"150\" style=\"float:left\"></canvas>\r");t.b("\n" + i);t.b("        <div style=\"float:left;height: 150px;\" class=\"points-chart\">\r");t.b("\n" + i);t.b("          <div style=\"color:#00a859\">\r");t.b("\n" + i);t.b("            <div class='font-light' style=\"float:left\">");t.b(t.v(t.d("Strings.MATCHTIPS.POINTS_5",c,p,0)));t.b(": &nbsp;</div>\r");t.b("\n" + i);t.b("            <div id=\"exact\" style=\"width:100px;text-align:right;\"></div>\r");t.b("\n" + i);t.b("          </div>\r");t.b("\n" + i);t.b("          <div style=\"color:#ffcc29\">\r");t.b("\n" + i);t.b("            <div class='font-light' style=\"float:left\">");t.b(t.v(t.d("Strings.MATCHTIPS.POINTS_3",c,p,0)));t.b(": &nbsp;</div>\r");t.b("\n" + i);t.b("            <div id=\"goals\" style=\"width:100px;text-align:right;\" ></div>\r");t.b("\n" + i);t.b("          </div>\r");t.b("\n" + i);t.b("          <div style=\"color:#a0a0a0\">\r");t.b("\n" + i);t.b("            <div class='font-light' style=\"float:left\">");t.b(t.v(t.d("Strings.MATCHTIPS.POINTS_1",c,p,0)));t.b(": &nbsp;</div>\r");t.b("\n" + i);t.b("            <div id=\"winner\" style=\"width:100px;text-align:right;\" ></div>\r");t.b("\n" + i);t.b("          </div>\r");t.b("\n" + i);t.b("           <div style=\"color:#ffffff\">\r");t.b("\n" + i);t.b("            <div class='font-light' style=\"float:left\">");t.b(t.v(t.d("Strings.MATCHTIPS.POINTS_0",c,p,0)));t.b(": &nbsp;</div>\r");t.b("\n" + i);t.b("            <div id=\"lost\" style=\"width:100px;text-align:right;\" ></div>\r");t.b("\n" + i);t.b("          </div>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);t.b("        <canvas id=\"tips\" style='height:150px;float:right'></canvas>\r");t.b("\n" + i);t.b("        <div style=\"clear:both\"></div>\r");t.b("\n" + i);t.b("       </div>\r");t.b("\n" + i);};t.b("    </div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("    <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("        <div class=\"panel-heading\">\r");t.b("\n" + i);t.b("            <h3 class=\"panel-title clearfix\">");t.b(t.v(t.d("Strings.MATCHTIPS.TIME_VENUE",c,p,0)));t.b("</h3>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("        <div class=\"panel-body\">\r");t.b("\n" + i);t.b("            <div class=\"padding-bottom-10\"><span class='font-light'>");t.b(t.v(t.d("Strings.MATCHTIPS.STADIUM",c,p,0)));t.b("</span>&nbsp;<span class=''>");t.b(t.v(t.f("location",c,p,0)));t.b("</span></div>\r");t.b("\n" + i);t.b("            <div>\r");t.b("\n" + i);t.b("              <div class='font-light'>");t.b(t.v(t.d("Strings.MATCHTIPS.LOCAL_KICKOFF",c,p,0)));t.b(" ");t.b(t.v(t.f("TimeZone",c,p,0)));t.b(":</div>\r");t.b("\n" + i);t.b("              <div class='font-color-primary'>");t.b(t.v(t.f("KickOffTime",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("            <div class=\"padding-bottom-10\">\r");t.b("\n" + i);t.b("              <div class='font-light'>");t.b(t.v(t.d("Strings.MATCHTIPS.BRAZIL_KICKOFF",c,p,0)));t.b(":</div>\r");t.b("\n" + i);t.b("              <div class='font-color-primary'>");t.b(t.v(t.f("LocalKickOffTime",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("            <div><span class='font-light'>");t.b(t.v(t.d("Strings.MATCHTIPS.TIPS_ENABLED_TO",c,p,0)));t.b("</span>&nbsp;<span class=''>");t.b(t.v(t.f("enabledTo",c,p,0)));t.b("</span></div>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});


define("hgn!templates/tipstatistic", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class=\"row\">\r");t.b("\n" + i);t.b("    <div class=\"col-xs-4 font-light content-left\">\r");t.b("\n" + i);t.b("        ");t.b(t.v(t.f("CountryNameA",c,p,0)));t.b(" ");t.b(t.v(t.d("Strings.MATCHTIPS.WINS",c,p,0)));t.b("\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"col-xs-4 font-light content-center\">\r");t.b("\n" + i);t.b("        ");t.b(t.v(t.d("Strings.TIP_STATS.DRAW",c,p,0)));t.b("\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"col-xs-4 font-light content-right\">\r");t.b("\n" + i);t.b("        ");t.b(t.v(t.f("CountryNameB",c,p,0)));t.b(" ");t.b(t.v(t.d("Strings.MATCHTIPS.WINS",c,p,0)));t.b("\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("<div class=\"row padding-bottom-10\">\r");t.b("\n" + i);t.b("    <div class=\"col-xs-4 font-bold content-left font-h3 font-color-primary\">\r");t.b("\n" + i);t.b("        ");t.b(t.v(t.f("aWinner",c,p,0)));t.b("%\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"col-xs-4 font-bold content-center font-h3 font-color-secondary\">\r");t.b("\n" + i);t.b("        ");t.b(t.v(t.f("draw",c,p,0)));t.b("%\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"col-xs-4 font-bold content-right font-h3 font-color-primary\">\r");t.b("\n" + i);t.b("        ");t.b(t.v(t.f("bWinner",c,p,0)));t.b("%\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("<div>\r");t.b("\n" + i);t.b("    <span class=\"font-light\">");t.b(t.v(t.d("Strings.TIP_STATS.TOTAL_NUMBER_OF_TIPS",c,p,0)));t.b(":</span> <span class=\"font-16\">");t.b(t.v(t.f("tipCountAll",c,p,0)));t.b("</span>\r");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/*!
  backbone.fetch-cache v1.4.0
  by Andy Appleton - https://github.com/mrappleton/backbone-fetch-cache.git
 */

// AMD wrapper from https://github.com/umdjs/umd/blob/master/amdWebGlobal.js

(function (root, factory) {
  if (typeof define === 'function' && define.amd) {
    // AMD. Register as an anonymous module and set browser global
    define('fetchcache',['underscore', 'backbone', 'jquery'], function (_, Backbone, $) {
      return (root.Backbone = factory(_, Backbone, $));
    });
  } else {
    // Browser globals
    root.Backbone = factory(root._, root.Backbone, root.jQuery);
  }
}(this, function (_, Backbone, $) {

  // Setup
  var superMethods = {
    modelFetch: Backbone.Model.prototype.fetch,
    modelSync: Backbone.Model.prototype.sync,
    collectionFetch: Backbone.Collection.prototype.fetch
  },
  supportLocalStorage = (function() {
    var supported = typeof window.localStorage !== 'undefined';
    if (supported) {
      try {
        // impossible to write on some platforms when private browsing is on and
        // throws an exception = local storage not supported.
        localStorage.setItem("test_support", "test_support");
        localStorage.removeItem("test_support");
      } catch (e) {
        supported = false;
      }
    }
    return supported;
  })();

  Backbone.fetchCache = (Backbone.fetchCache || {});
  Backbone.fetchCache._cache = (Backbone.fetchCache._cache || {});

  Backbone.fetchCache.priorityFn = function(a, b) {
    if (!a || !a.expires || !b || !b.expires) {
      return a;
    }

    return a.expires - b.expires;
  };

  Backbone.fetchCache._prioritize = function() {
    var sorted = _.values(this._cache).sort(this.priorityFn);
    var index = _.indexOf(_.values(this._cache), sorted[0]);
    return _.keys(this._cache)[index];
  };

  Backbone.fetchCache._deleteCacheWithPriority = function() {
    Backbone.fetchCache._cache[this._prioritize()] = null;
    delete Backbone.fetchCache._cache[this._prioritize()];
    Backbone.fetchCache.setLocalStorage();
  };

  Backbone.fetchCache.getLocalStorageKey = function() {
    return 'backboneCache';
  };

  if (typeof Backbone.fetchCache.localStorage === 'undefined') {
    Backbone.fetchCache.localStorage = true;
  }

  // Shared methods
  function getCacheKey(instance, opts) {
    var url;

    if(opts && opts.url) {
      url = opts.url;
    } else {
      url = _.isFunction(instance.url) ? instance.url() : instance.url;
    }

    // Need url to use as cache key so return if we can't get it
    if(!url) { return; }

    if(opts && opts.data) {
      return url + "?" + $.param(opts.data);
    }
    return url;
  }

  function setCache(instance, opts, attrs) {
    opts = (opts || {});
    var key = Backbone.fetchCache.getCacheKey(instance, opts),
        expires = false;

    // Need url to use as cache key so return if we can't get it
    if (!key) { return; }

    // Never set the cache if user has explicitly said not to
    if (opts.cache === false) { return; }

    // Don't set the cache unless cache: true or prefill: true option is passed
    if (!(opts.cache || opts.prefill)) { return; }

    if (opts.expires !== false) {
      expires = (new Date()).getTime() + ((opts.expires || 5 * 60) * 1000);
    }

    Backbone.fetchCache._cache[key] = {
      expires: expires,
      value: attrs
    };

    Backbone.fetchCache.setLocalStorage();
  }

  function clearItem(key) {
    if (_.isFunction(key)) { key = key(); }
    delete Backbone.fetchCache._cache[key];
    Backbone.fetchCache.setLocalStorage();
  }

  function setLocalStorage() {
    if (!supportLocalStorage || !Backbone.fetchCache.localStorage) { return; }
    try {
      localStorage.setItem(Backbone.fetchCache.getLocalStorageKey(), JSON.stringify(Backbone.fetchCache._cache));
    } catch (err) {
      var code = err.code || err.number || err.message;
      if (code === 22) {
        this._deleteCacheWithPriority();
      } else {
        throw(err);
      }
    }
  }

  function getLocalStorage() {
    if (!supportLocalStorage || !Backbone.fetchCache.localStorage) { return; }
    var json = localStorage.getItem(Backbone.fetchCache.getLocalStorageKey()) || '{}';
    Backbone.fetchCache._cache = JSON.parse(json);
  }

  function nextTick(fn) {
    return window.setTimeout(fn, 0);
  }

  // Instance methods
  Backbone.Model.prototype.fetch = function(opts) {
    opts = _.defaults(opts || {}, { parse: true });
    var key = Backbone.fetchCache.getCacheKey(this, opts),
        data = Backbone.fetchCache._cache[key],
        expired = false,
        attributes = false,
        deferred = new $.Deferred(),
        self = this;

    function setData() {
      if (opts.parse) {
        attributes = self.parse(attributes, opts);
      }

      self.set(attributes, opts);
      if (_.isFunction(opts.prefillSuccess)) { opts.prefillSuccess(self, attributes, opts); }

      // Trigger sync events
      self.trigger('cachesync', self, attributes, opts);
      self.trigger('sync', self, attributes, opts);

      // Notify progress if we're still waiting for an AJAX call to happen...
      if (opts.prefill) { deferred.notify(self); }
      // ...finish and return if we're not
      else {
        if (_.isFunction(opts.success)) { opts.success(self, attributes, opts); }
        deferred.resolve(self);
      }
    }

    if (data) {
      expired = data.expires;
      expired = expired && data.expires < (new Date()).getTime();
      attributes = data.value;
    }

    if (!expired && (opts.cache || opts.prefill) && attributes) {
      // Ensure that cache resolution adhers to async option, defaults to true.
      if (opts.async == null) { opts.async = true; }

      if (opts.async) {
        nextTick(setData);
      } else {
        setData();
      }

      if (!opts.prefill) {
        return deferred;
      }
    }

    // Delegate to the actual fetch method and store the attributes in the cache
    superMethods.modelFetch.apply(this, arguments)
      // resolve the returned promise when the AJAX call completes
      .done( _.bind(deferred.resolve, this, this) )
      // Set the new data in the cache
      .done( _.bind(Backbone.fetchCache.setCache, null, this, opts) )
      // Reject the promise on fail
      .fail( _.bind(deferred.reject, this, this) );

    // return a promise which provides the same methods as a jqXHR object
    return deferred;
  };

  // Override Model.prototype.sync and try to clear cache items if it looks
  // like they are being updated.
  Backbone.Model.prototype.sync = function(method, model, options) {
    // Only empty the cache if we're doing a create, update, patch or delete.
    if (method === 'read') {
      return superMethods.modelSync.apply(this, arguments);
    }

    var collection = model.collection,
        keys = [],
        i, len;

    // Build up a list of keys to delete from the cache, starting with this
    keys.push(Backbone.fetchCache.getCacheKey(model, options));

    // If this model has a collection, also try to delete the cache for that
    if (!!collection) {
      keys.push(Backbone.fetchCache.getCacheKey(collection));
    }

    // Empty cache for all found keys
    for (i = 0, len = keys.length; i < len; i++) { clearItem(keys[i]); }

    return superMethods.modelSync.apply(this, arguments);
  };

  Backbone.Collection.prototype.fetch = function(opts) {
    opts = _.defaults(opts || {}, { parse: true });
    var key = Backbone.fetchCache.getCacheKey(this, opts),
        data = Backbone.fetchCache._cache[key],
        expired = false,
        attributes = false,
        deferred = new $.Deferred(),
        self = this;

    function setData() {
      self[opts.reset ? 'reset' : 'set'](attributes, opts);
      if (_.isFunction(opts.prefillSuccess)) { opts.prefillSuccess(self); }

      // Trigger sync events
      self.trigger('cachesync', self, attributes, opts);
      self.trigger('sync', self, attributes, opts);

      // Notify progress if we're still waiting for an AJAX call to happen...
      if (opts.prefill) { deferred.notify(self); }
      // ...finish and return if we're not
      else {
        if (_.isFunction(opts.success)) { opts.success(self, attributes, opts); }
        deferred.resolve(self);
      }
    }

    if (data) {
      expired = data.expires;
      expired = expired && data.expires < (new Date()).getTime();
      attributes = data.value;
    }

    if (!expired && (opts.cache || opts.prefill) && attributes) {
      // Ensure that cache resolution adhers to async option, defaults to true.
      if (opts.async == null) { opts.async = true; }

      if (opts.async) {
        nextTick(setData);
      } else {
        setData();
      }

      if (!opts.prefill) {
        return deferred;
      }
    }

    // Delegate to the actual fetch method and store the attributes in the cache
    superMethods.collectionFetch.apply(this, arguments)
      // resolve the returned promise when the AJAX call completes
      .done( _.bind(deferred.resolve, this, this) )
      // Set the new data in the cache
      .done( _.bind(Backbone.fetchCache.setCache, null, this, opts) )
      // Reject the promise on fail
      .fail( _.bind(deferred.reject, this, this) );

    // return a promise which provides the same methods as a jqXHR object
    return deferred;
  };

  // Prime the cache from localStorage on initialization
  getLocalStorage();

  // Exports

  Backbone.fetchCache._superMethods = superMethods;
  Backbone.fetchCache.setCache = setCache;
  Backbone.fetchCache.getCacheKey = getCacheKey;
  Backbone.fetchCache.clearItem = clearItem;
  Backbone.fetchCache.setLocalStorage = setLocalStorage;
  Backbone.fetchCache.getLocalStorage = getLocalStorage;

  return Backbone;
}));

/*!
 * Chart.js
 * http://chartjs.org/
 *
 * Copyright 2013 Nick Downie
 * Released under the MIT license
 * https://github.com/nnnick/Chart.js/blob/master/LICENSE.md
 */

//Define the global Chart Variable as a class.
window.Chart = function(context){

	var chart = this;
	
	
	//Easing functions adapted from Robert Penner's easing equations
	//http://www.robertpenner.com/easing/
	
	var animationOptions = {
		linear : function (t){
			return t;
		},
		easeInQuad: function (t) {
			return t*t;
		},
		easeOutQuad: function (t) {
			return -1 *t*(t-2);
		},
		easeInOutQuad: function (t) {
			if ((t/=1/2) < 1) return 1/2*t*t;
			return -1/2 * ((--t)*(t-2) - 1);
		},
		easeInCubic: function (t) {
			return t*t*t;
		},
		easeOutCubic: function (t) {
			return 1*((t=t/1-1)*t*t + 1);
		},
		easeInOutCubic: function (t) {
			if ((t/=1/2) < 1) return 1/2*t*t*t;
			return 1/2*((t-=2)*t*t + 2);
		},
		easeInQuart: function (t) {
			return t*t*t*t;
		},
		easeOutQuart: function (t) {
			return -1 * ((t=t/1-1)*t*t*t - 1);
		},
		easeInOutQuart: function (t) {
			if ((t/=1/2) < 1) return 1/2*t*t*t*t;
			return -1/2 * ((t-=2)*t*t*t - 2);
		},
		easeInQuint: function (t) {
			return 1*(t/=1)*t*t*t*t;
		},
		easeOutQuint: function (t) {
			return 1*((t=t/1-1)*t*t*t*t + 1);
		},
		easeInOutQuint: function (t) {
			if ((t/=1/2) < 1) return 1/2*t*t*t*t*t;
			return 1/2*((t-=2)*t*t*t*t + 2);
		},
		easeInSine: function (t) {
			return -1 * Math.cos(t/1 * (Math.PI/2)) + 1;
		},
		easeOutSine: function (t) {
			return 1 * Math.sin(t/1 * (Math.PI/2));
		},
		easeInOutSine: function (t) {
			return -1/2 * (Math.cos(Math.PI*t/1) - 1);
		},
		easeInExpo: function (t) {
			return (t==0) ? 1 : 1 * Math.pow(2, 10 * (t/1 - 1));
		},
		easeOutExpo: function (t) {
			return (t==1) ? 1 : 1 * (-Math.pow(2, -10 * t/1) + 1);
		},
		easeInOutExpo: function (t) {
			if (t==0) return 0;
			if (t==1) return 1;
			if ((t/=1/2) < 1) return 1/2 * Math.pow(2, 10 * (t - 1));
			return 1/2 * (-Math.pow(2, -10 * --t) + 2);
			},
		easeInCirc: function (t) {
			if (t>=1) return t;
			return -1 * (Math.sqrt(1 - (t/=1)*t) - 1);
		},
		easeOutCirc: function (t) {
			return 1 * Math.sqrt(1 - (t=t/1-1)*t);
		},
		easeInOutCirc: function (t) {
			if ((t/=1/2) < 1) return -1/2 * (Math.sqrt(1 - t*t) - 1);
			return 1/2 * (Math.sqrt(1 - (t-=2)*t) + 1);
		},
		easeInElastic: function (t) {
			var s=1.70158;var p=0;var a=1;
			if (t==0) return 0;  if ((t/=1)==1) return 1;  if (!p) p=1*.3;
			if (a < Math.abs(1)) { a=1; var s=p/4; }
			else var s = p/(2*Math.PI) * Math.asin (1/a);
			return -(a*Math.pow(2,10*(t-=1)) * Math.sin( (t*1-s)*(2*Math.PI)/p ));
		},
		easeOutElastic: function (t) {
			var s=1.70158;var p=0;var a=1;
			if (t==0) return 0;  if ((t/=1)==1) return 1;  if (!p) p=1*.3;
			if (a < Math.abs(1)) { a=1; var s=p/4; }
			else var s = p/(2*Math.PI) * Math.asin (1/a);
			return a*Math.pow(2,-10*t) * Math.sin( (t*1-s)*(2*Math.PI)/p ) + 1;
		},
		easeInOutElastic: function (t) {
			var s=1.70158;var p=0;var a=1;
			if (t==0) return 0;  if ((t/=1/2)==2) return 1;  if (!p) p=1*(.3*1.5);
			if (a < Math.abs(1)) { a=1; var s=p/4; }
			else var s = p/(2*Math.PI) * Math.asin (1/a);
			if (t < 1) return -.5*(a*Math.pow(2,10*(t-=1)) * Math.sin( (t*1-s)*(2*Math.PI)/p ));
			return a*Math.pow(2,-10*(t-=1)) * Math.sin( (t*1-s)*(2*Math.PI)/p )*.5 + 1;
		},
		easeInBack: function (t) {
			var s = 1.70158;
			return 1*(t/=1)*t*((s+1)*t - s);
		},
		easeOutBack: function (t) {
			var s = 1.70158;
			return 1*((t=t/1-1)*t*((s+1)*t + s) + 1);
		},
		easeInOutBack: function (t) {
			var s = 1.70158; 
			if ((t/=1/2) < 1) return 1/2*(t*t*(((s*=(1.525))+1)*t - s));
			return 1/2*((t-=2)*t*(((s*=(1.525))+1)*t + s) + 2);
		},
		easeInBounce: function (t) {
			return 1 - animationOptions.easeOutBounce (1-t);
		},
		easeOutBounce: function (t) {
			if ((t/=1) < (1/2.75)) {
				return 1*(7.5625*t*t);
			} else if (t < (2/2.75)) {
				return 1*(7.5625*(t-=(1.5/2.75))*t + .75);
			} else if (t < (2.5/2.75)) {
				return 1*(7.5625*(t-=(2.25/2.75))*t + .9375);
			} else {
				return 1*(7.5625*(t-=(2.625/2.75))*t + .984375);
			}
		},
		easeInOutBounce: function (t) {
			if (t < 1/2) return animationOptions.easeInBounce (t*2) * .5;
			return animationOptions.easeOutBounce (t*2-1) * .5 + 1*.5;
		}
	};

	//Variables global to the chart
	var width = context.canvas.width;
	var height = context.canvas.height;


	//High pixel density displays - multiply the size of the canvas height/width by the device pixel ratio, then scale.
	if (window.devicePixelRatio) {
		context.canvas.style.width = width + "px";
		context.canvas.style.height = height + "px";
		context.canvas.height = height * window.devicePixelRatio;
		context.canvas.width = width * window.devicePixelRatio;
		context.scale(window.devicePixelRatio, window.devicePixelRatio);
	}

	this.PolarArea = function(data,options){
	
		chart.PolarArea.defaults = {
			scaleOverlay : true,
			scaleOverride : false,
			scaleSteps : null,
			scaleStepWidth : null,
			scaleStartValue : null,
			scaleShowLine : true,
			scaleLineColor : "rgba(0,0,0,.1)",
			scaleLineWidth : 1,
			scaleShowLabels : true,
			scaleLabel : "<%=value%>",
			scaleFontFamily : "'Arial'",
			scaleFontSize : 12,
			scaleFontStyle : "normal",
			scaleFontColor : "#666",
			scaleShowLabelBackdrop : true,
			scaleBackdropColor : "rgba(255,255,255,0.75)",
			scaleBackdropPaddingY : 2,
			scaleBackdropPaddingX : 2,
			segmentShowStroke : true,
			segmentStrokeColor : "#fff",
			segmentStrokeWidth : 2,
			animation : true,
			animationSteps : 100,
			animationEasing : "easeOutBounce",
			animateRotate : true,
			animateScale : false,
			onAnimationComplete : null
		};
		
		var config = (options)? mergeChartConfig(chart.PolarArea.defaults,options) : chart.PolarArea.defaults;
		
		return new PolarArea(data,config,context);
	};

	this.Radar = function(data,options){
	
		chart.Radar.defaults = {
			scaleOverlay : false,
			scaleOverride : false,
			scaleSteps : null,
			scaleStepWidth : null,
			scaleStartValue : null,
			scaleShowLine : true,
			scaleLineColor : "rgba(0,0,0,.1)",
			scaleLineWidth : 1,
			scaleShowLabels : false,
			scaleLabel : "<%=value%>",
			scaleFontFamily : "'Arial'",
			scaleFontSize : 12,
			scaleFontStyle : "normal",
			scaleFontColor : "#666",
			scaleShowLabelBackdrop : true,
			scaleBackdropColor : "rgba(255,255,255,0.75)",
			scaleBackdropPaddingY : 2,
			scaleBackdropPaddingX : 2,
			angleShowLineOut : true,
			angleLineColor : "rgba(0,0,0,.1)",
			angleLineWidth : 1,			
			pointLabelFontFamily : "'Arial'",
			pointLabelFontStyle : "normal",
			pointLabelFontSize : 12,
			pointLabelFontColor : "#666",
			pointDot : true,
			pointDotRadius : 3,
			pointDotStrokeWidth : 1,
			datasetStroke : true,
			datasetStrokeWidth : 2,
			datasetFill : true,
			animation : true,
			animationSteps : 60,
			animationEasing : "easeOutQuart",
			onAnimationComplete : null
		};
		
		var config = (options)? mergeChartConfig(chart.Radar.defaults,options) : chart.Radar.defaults;

		return new Radar(data,config,context);
	};
	
	this.Pie = function(data,options){
		chart.Pie.defaults = {
			segmentShowStroke : true,
			segmentStrokeColor : "#fff",
			segmentStrokeWidth : 2,
			animation : true,
			animationSteps : 100,
			animationEasing : "easeOutBounce",
			animateRotate : true,
			animateScale : false,
			onAnimationComplete : null
		};		

		var config = (options)? mergeChartConfig(chart.Pie.defaults,options) : chart.Pie.defaults;
		
		return new Pie(data,config,context);				
	};
	
	this.Doughnut = function(data,options){
	
		chart.Doughnut.defaults = {
			segmentShowStroke : true,
			segmentStrokeColor : "#fff",
			segmentStrokeWidth : 2,
			percentageInnerCutout : 50,
			animation : true,
			animationSteps : 100,
			animationEasing : "easeOutBounce",
			animateRotate : true,
			animateScale : false,
			onAnimationComplete : null
		};		

		var config = (options)? mergeChartConfig(chart.Doughnut.defaults,options) : chart.Doughnut.defaults;
		
		return new Doughnut(data,config,context);			
		
	};

	this.Line = function(data,options){
	
		chart.Line.defaults = {
			scaleOverlay : false,
			scaleOverride : false,
			scaleSteps : null,
			scaleStepWidth : null,
			scaleStartValue : null,
			scaleLineColor : "rgba(0,0,0,.1)",
			scaleLineWidth : 1,
			scaleShowLabels : true,
			scaleLabel : "<%=value%>",
			scaleFontFamily : "'Arial'",
			scaleFontSize : 12,
			scaleFontStyle : "normal",
			scaleFontColor : "#666",
			scaleShowGridLines : true,
			scaleGridLineColor : "rgba(0,0,0,.05)",
			scaleGridLineWidth : 1,
			bezierCurve : true,
			pointDot : true,
			pointDotRadius : 4,
			pointDotStrokeWidth : 2,
			datasetStroke : true,
			datasetStrokeWidth : 2,
			datasetFill : true,
			animation : true,
			animationSteps : 60,
			animationEasing : "easeOutQuart",
			onAnimationComplete : null
		};		
		var config = (options) ? mergeChartConfig(chart.Line.defaults,options) : chart.Line.defaults;
		
		return new Line(data,config,context);
	}
	
	this.Bar = function(data,options){
		chart.Bar.defaults = {
			scaleOverlay : false,
			scaleOverride : false,
			scaleSteps : null,
			scaleStepWidth : null,
			scaleStartValue : null,
			scaleLineColor : "rgba(0,0,0,.1)",
			scaleLineWidth : 1,
			scaleShowLabels : true,
			scaleLabel : "<%=value%>",
			scaleFontFamily : "'Arial'",
			scaleFontSize : 12,
			scaleFontStyle : "normal",
			scaleFontColor : "#666",
			scaleShowGridLines : true,
			scaleGridLineColor : "rgba(0,0,0,.05)",
			scaleGridLineWidth : 1,
			barShowStroke : true,
			barStrokeWidth : 2,
			barValueSpacing : 5,
			barDatasetSpacing : 1,
			animation : true,
			animationSteps : 60,
			animationEasing : "easeOutQuart",
			onAnimationComplete : null
		};		
		var config = (options) ? mergeChartConfig(chart.Bar.defaults,options) : chart.Bar.defaults;
		
		return new Bar(data,config,context);		
	}
	
	var clear = function(c){
		c.clearRect(0, 0, width, height);
	};

	var PolarArea = function(data,config,ctx){
		var maxSize, scaleHop, calculatedScale, labelHeight, scaleHeight, valueBounds, labelTemplateString;		
		
		
		calculateDrawingSizes();
		
		valueBounds = getValueBounds();

		labelTemplateString = (config.scaleShowLabels)? config.scaleLabel : null;

		//Check and set the scale
		if (!config.scaleOverride){
			
			calculatedScale = calculateScale(scaleHeight,valueBounds.maxSteps,valueBounds.minSteps,valueBounds.maxValue,valueBounds.minValue,labelTemplateString);
		}
		else {
			calculatedScale = {
				steps : config.scaleSteps,
				stepValue : config.scaleStepWidth,
				graphMin : config.scaleStartValue,
				labels : []
			}
			populateLabels(labelTemplateString, calculatedScale.labels,calculatedScale.steps,config.scaleStartValue,config.scaleStepWidth);
		}
		
		scaleHop = maxSize/(calculatedScale.steps);

		//Wrap in an animation loop wrapper
		animationLoop(config,drawScale,drawAllSegments,ctx);

		function calculateDrawingSizes(){
			maxSize = (Min([width,height])/2);
			//Remove whatever is larger - the font size or line width.
			
			maxSize -= Max([config.scaleFontSize*0.5,config.scaleLineWidth*0.5]);
			
			labelHeight = config.scaleFontSize*2;
			//If we're drawing the backdrop - add the Y padding to the label height and remove from drawing region.
			if (config.scaleShowLabelBackdrop){
				labelHeight += (2 * config.scaleBackdropPaddingY);
				maxSize -= config.scaleBackdropPaddingY*1.5;
			}
			
			scaleHeight = maxSize;
			//If the label height is less than 5, set it to 5 so we don't have lines on top of each other.
			labelHeight = Default(labelHeight,5);
		}
		function drawScale(){
			for (var i=0; i<calculatedScale.steps; i++){
				//If the line object is there
				if (config.scaleShowLine){
					ctx.beginPath();
					ctx.arc(width/2, height/2, scaleHop * (i + 1), 0, (Math.PI * 2), true);
					ctx.strokeStyle = config.scaleLineColor;
					ctx.lineWidth = config.scaleLineWidth;
					ctx.stroke();
				}

				if (config.scaleShowLabels){
					ctx.textAlign = "center";
					ctx.font = config.scaleFontStyle + " " + config.scaleFontSize + "px " + config.scaleFontFamily;
 					var label =  calculatedScale.labels[i];
					//If the backdrop object is within the font object
					if (config.scaleShowLabelBackdrop){
						var textWidth = ctx.measureText(label).width;
						ctx.fillStyle = config.scaleBackdropColor;
						ctx.beginPath();
						ctx.rect(
							Math.round(width/2 - textWidth/2 - config.scaleBackdropPaddingX),     //X
							Math.round(height/2 - (scaleHop * (i + 1)) - config.scaleFontSize*0.5 - config.scaleBackdropPaddingY),//Y
							Math.round(textWidth + (config.scaleBackdropPaddingX*2)), //Width
							Math.round(config.scaleFontSize + (config.scaleBackdropPaddingY*2)) //Height
						);
						ctx.fill();
					}
					ctx.textBaseline = "middle";
					ctx.fillStyle = config.scaleFontColor;
					ctx.fillText(label,width/2,height/2 - (scaleHop * (i + 1)));
				}
			}
		}
		function drawAllSegments(animationDecimal){
			var startAngle = -Math.PI/2,
			angleStep = (Math.PI*2)/data.length,
			scaleAnimation = 1,
			rotateAnimation = 1;
			if (config.animation) {
				if (config.animateScale) {
					scaleAnimation = animationDecimal;
				}
				if (config.animateRotate){
					rotateAnimation = animationDecimal;
				}
			}

			for (var i=0; i<data.length; i++){

				ctx.beginPath();
				ctx.arc(width/2,height/2,scaleAnimation * calculateOffset(data[i].value,calculatedScale,scaleHop),startAngle, startAngle + rotateAnimation*angleStep, false);
				ctx.lineTo(width/2,height/2);
				ctx.closePath();
				ctx.fillStyle = data[i].color;
				ctx.fill();

				if(config.segmentShowStroke){
					ctx.strokeStyle = config.segmentStrokeColor;
					ctx.lineWidth = config.segmentStrokeWidth;
					ctx.stroke();
				}
				startAngle += rotateAnimation*angleStep;
			}
		}
		function getValueBounds() {
			var upperValue = Number.MIN_VALUE;
			var lowerValue = Number.MAX_VALUE;
			for (var i=0; i<data.length; i++){
				if (data[i].value > upperValue) {upperValue = data[i].value;}
				if (data[i].value < lowerValue) {lowerValue = data[i].value;}
			};

			var maxSteps = Math.floor((scaleHeight / (labelHeight*0.66)));
			var minSteps = Math.floor((scaleHeight / labelHeight*0.5));
			
			return {
				maxValue : upperValue,
				minValue : lowerValue,
				maxSteps : maxSteps,
				minSteps : minSteps
			};
			

		}
	}

	var Radar = function (data,config,ctx) {
		var maxSize, scaleHop, calculatedScale, labelHeight, scaleHeight, valueBounds, labelTemplateString;	
			
		//If no labels are defined set to an empty array, so referencing length for looping doesn't blow up.
		if (!data.labels) data.labels = [];
		
		calculateDrawingSizes();

		var valueBounds = getValueBounds();

		labelTemplateString = (config.scaleShowLabels)? config.scaleLabel : null;

		//Check and set the scale
		if (!config.scaleOverride){
			
			calculatedScale = calculateScale(scaleHeight,valueBounds.maxSteps,valueBounds.minSteps,valueBounds.maxValue,valueBounds.minValue,labelTemplateString);
		}
		else {
			calculatedScale = {
				steps : config.scaleSteps,
				stepValue : config.scaleStepWidth,
				graphMin : config.scaleStartValue,
				labels : []
			}
			populateLabels(labelTemplateString, calculatedScale.labels,calculatedScale.steps,config.scaleStartValue,config.scaleStepWidth);
		}
		
		scaleHop = maxSize/(calculatedScale.steps);
		
		animationLoop(config,drawScale,drawAllDataPoints,ctx);
		
		//Radar specific functions.
		function drawAllDataPoints(animationDecimal){
			var rotationDegree = (2*Math.PI)/data.datasets[0].data.length;

			ctx.save();
			//translate to the centre of the canvas.
			ctx.translate(width/2,height/2);
			
			//We accept multiple data sets for radar charts, so show loop through each set
			for (var i=0; i<data.datasets.length; i++){
				ctx.beginPath();

				ctx.moveTo(0,animationDecimal*(-1*calculateOffset(data.datasets[i].data[0],calculatedScale,scaleHop)));
				for (var j=1; j<data.datasets[i].data.length; j++){
					ctx.rotate(rotationDegree);	
					ctx.lineTo(0,animationDecimal*(-1*calculateOffset(data.datasets[i].data[j],calculatedScale,scaleHop)));
			
				}
				ctx.closePath();
				
				
				ctx.fillStyle = data.datasets[i].fillColor;
				ctx.strokeStyle = data.datasets[i].strokeColor;
				ctx.lineWidth = config.datasetStrokeWidth;
				ctx.fill();
				ctx.stroke();
				
								
				if (config.pointDot){
					ctx.fillStyle = data.datasets[i].pointColor;
					ctx.strokeStyle = data.datasets[i].pointStrokeColor;
					ctx.lineWidth = config.pointDotStrokeWidth;
					for (var k=0; k<data.datasets[i].data.length; k++){
						ctx.rotate(rotationDegree);
						ctx.beginPath();
						ctx.arc(0,animationDecimal*(-1*calculateOffset(data.datasets[i].data[k],calculatedScale,scaleHop)),config.pointDotRadius,2*Math.PI,false);
						ctx.fill();
						ctx.stroke();
					}					
					
				}
				ctx.rotate(rotationDegree);
				
			}
			ctx.restore();
			
			
		}
		function drawScale(){
			var rotationDegree = (2*Math.PI)/data.datasets[0].data.length;
			ctx.save();
		    ctx.translate(width / 2, height / 2);	
			
			if (config.angleShowLineOut){
				ctx.strokeStyle = config.angleLineColor;		    	    
				ctx.lineWidth = config.angleLineWidth;
				for (var h=0; h<data.datasets[0].data.length; h++){
					
				    ctx.rotate(rotationDegree);
					ctx.beginPath();
					ctx.moveTo(0,0);
					ctx.lineTo(0,-maxSize);
					ctx.stroke();
				}
			}

			for (var i=0; i<calculatedScale.steps; i++){
				ctx.beginPath();
				
				if(config.scaleShowLine){
					ctx.strokeStyle = config.scaleLineColor;
					ctx.lineWidth = config.scaleLineWidth;
					ctx.moveTo(0,-scaleHop * (i+1));					
					for (var j=0; j<data.datasets[0].data.length; j++){
					    ctx.rotate(rotationDegree);
						ctx.lineTo(0,-scaleHop * (i+1));
					}
					ctx.closePath();
					ctx.stroke();			
							
				}
				
				if (config.scaleShowLabels){				
					ctx.textAlign = 'center';
					ctx.font = config.scaleFontStyle + " " + config.scaleFontSize+"px " + config.scaleFontFamily; 
					ctx.textBaseline = "middle";
					
					if (config.scaleShowLabelBackdrop){
						var textWidth = ctx.measureText(calculatedScale.labels[i]).width;
						ctx.fillStyle = config.scaleBackdropColor;
						ctx.beginPath();
						ctx.rect(
							Math.round(- textWidth/2 - config.scaleBackdropPaddingX),     //X
							Math.round((-scaleHop * (i + 1)) - config.scaleFontSize*0.5 - config.scaleBackdropPaddingY),//Y
							Math.round(textWidth + (config.scaleBackdropPaddingX*2)), //Width
							Math.round(config.scaleFontSize + (config.scaleBackdropPaddingY*2)) //Height
						);
						ctx.fill();
					}						
					ctx.fillStyle = config.scaleFontColor;
					ctx.fillText(calculatedScale.labels[i],0,-scaleHop*(i+1));
				}

			}
			for (var k=0; k<data.labels.length; k++){				
			ctx.font = config.pointLabelFontStyle + " " + config.pointLabelFontSize+"px " + config.pointLabelFontFamily;
			ctx.fillStyle = config.pointLabelFontColor;
				var opposite = Math.sin(rotationDegree*k) * (maxSize + config.pointLabelFontSize);
				var adjacent = Math.cos(rotationDegree*k) * (maxSize + config.pointLabelFontSize);
				
				if(rotationDegree*k == Math.PI || rotationDegree*k == 0){
					ctx.textAlign = "center";
				}
				else if(rotationDegree*k > Math.PI){
					ctx.textAlign = "right";
				}
				else{
					ctx.textAlign = "left";
				}
				
				ctx.textBaseline = "middle";
				
				ctx.fillText(data.labels[k],opposite,-adjacent);
				
			}
			ctx.restore();
		};
		function calculateDrawingSizes(){
			maxSize = (Min([width,height])/2);

			labelHeight = config.scaleFontSize*2;
			
			var labelLength = 0;
			for (var i=0; i<data.labels.length; i++){
				ctx.font = config.pointLabelFontStyle + " " + config.pointLabelFontSize+"px " + config.pointLabelFontFamily;
				var textMeasurement = ctx.measureText(data.labels[i]).width;
				if(textMeasurement>labelLength) labelLength = textMeasurement;
			}
			
			//Figure out whats the largest - the height of the text or the width of what's there, and minus it from the maximum usable size.
			maxSize -= Max([labelLength,((config.pointLabelFontSize/2)*1.5)]);				
			
			maxSize -= config.pointLabelFontSize;
			maxSize = CapValue(maxSize, null, 0);
			scaleHeight = maxSize;
			//If the label height is less than 5, set it to 5 so we don't have lines on top of each other.
			labelHeight = Default(labelHeight,5);
		};
		function getValueBounds() {
			var upperValue = Number.MIN_VALUE;
			var lowerValue = Number.MAX_VALUE;
			
			for (var i=0; i<data.datasets.length; i++){
				for (var j=0; j<data.datasets[i].data.length; j++){
					if (data.datasets[i].data[j] > upperValue){upperValue = data.datasets[i].data[j]}
					if (data.datasets[i].data[j] < lowerValue){lowerValue = data.datasets[i].data[j]}
				}
			}

			var maxSteps = Math.floor((scaleHeight / (labelHeight*0.66)));
			var minSteps = Math.floor((scaleHeight / labelHeight*0.5));
			
			return {
				maxValue : upperValue,
				minValue : lowerValue,
				maxSteps : maxSteps,
				minSteps : minSteps
			};
			

		}
	}

	var Pie = function(data,config,ctx){
		var segmentTotal = 0;
		
		//In case we have a canvas that is not a square. Minus 5 pixels as padding round the edge.
		var pieRadius = Min([height/2,width/2]) - 5;
		
		for (var i=0; i<data.length; i++){
			segmentTotal += data[i].value;
		}
		
		
		animationLoop(config,null,drawPieSegments,ctx);
				
		function drawPieSegments (animationDecimal){
			var cumulativeAngle = -Math.PI/2,
			scaleAnimation = 1,
			rotateAnimation = 1;
			if (config.animation) {
				if (config.animateScale) {
					scaleAnimation = animationDecimal;
				}
				if (config.animateRotate){
					rotateAnimation = animationDecimal;
				}
			}
			for (var i=0; i<data.length; i++){
				var segmentAngle = rotateAnimation * ((data[i].value/segmentTotal) * (Math.PI*2));
				ctx.beginPath();
				ctx.arc(width/2,height/2,scaleAnimation * pieRadius,cumulativeAngle,cumulativeAngle + segmentAngle);
				ctx.lineTo(width/2,height/2);
				ctx.closePath();
				ctx.fillStyle = data[i].color;
				ctx.fill();
				
				if(config.segmentShowStroke){
					ctx.lineWidth = config.segmentStrokeWidth;
					ctx.strokeStyle = config.segmentStrokeColor;
					ctx.stroke();
				}
				cumulativeAngle += segmentAngle;
			}			
		}		
	}

	var Doughnut = function(data,config,ctx){
		var segmentTotal = 0;
		
		//In case we have a canvas that is not a square. Minus 5 pixels as padding round the edge.
		var doughnutRadius = Min([height/2,width/2]) - 5;
		
		var cutoutRadius = doughnutRadius * (config.percentageInnerCutout/100);
		
		for (var i=0; i<data.length; i++){
			segmentTotal += data[i].value;
		}
		
		
		animationLoop(config,null,drawPieSegments,ctx);
		
		
		function drawPieSegments (animationDecimal){
			var cumulativeAngle = -Math.PI/2,
			scaleAnimation = 1,
			rotateAnimation = 1;
			if (config.animation) {
				if (config.animateScale) {
					scaleAnimation = animationDecimal;
				}
				if (config.animateRotate){
					rotateAnimation = animationDecimal;
				}
			}
			for (var i=0; i<data.length; i++){
				var segmentAngle = rotateAnimation * ((data[i].value/segmentTotal) * (Math.PI*2));
				ctx.beginPath();
				ctx.arc(width/2,height/2,scaleAnimation * doughnutRadius,cumulativeAngle,cumulativeAngle + segmentAngle,false);
				ctx.arc(width/2,height/2,scaleAnimation * cutoutRadius,cumulativeAngle + segmentAngle,cumulativeAngle,true);
				ctx.closePath();
				ctx.fillStyle = data[i].color;
				ctx.fill();
				
				if(config.segmentShowStroke){
					ctx.lineWidth = config.segmentStrokeWidth;
					ctx.strokeStyle = config.segmentStrokeColor;
					ctx.stroke();
				}
				cumulativeAngle += segmentAngle;
			}			
		}			
		
		
		
	}

	var Line = function(data,config,ctx){
		var maxSize, scaleHop, calculatedScale, labelHeight, scaleHeight, valueBounds, labelTemplateString, valueHop,widestXLabel, xAxisLength,yAxisPosX,xAxisPosY, rotateLabels = 0;
			
		calculateDrawingSizes();
		
		valueBounds = getValueBounds();
		//Check and set the scale
		labelTemplateString = (config.scaleShowLabels)? config.scaleLabel : "";
		if (!config.scaleOverride){
			
			calculatedScale = calculateScale(scaleHeight,valueBounds.maxSteps,valueBounds.minSteps,valueBounds.maxValue,valueBounds.minValue,labelTemplateString);
		}
		else {
			calculatedScale = {
				steps : config.scaleSteps,
				stepValue : config.scaleStepWidth,
				graphMin : config.scaleStartValue,
				labels : []
			}
			populateLabels(labelTemplateString, calculatedScale.labels,calculatedScale.steps,config.scaleStartValue,config.scaleStepWidth);
		}
		
		scaleHop = Math.floor(scaleHeight/calculatedScale.steps);
		calculateXAxisSize();
		animationLoop(config,drawScale,drawLines,ctx);		
		
		function drawLines(animPc){
			for (var i=0; i<data.datasets.length; i++){
				ctx.strokeStyle = data.datasets[i].strokeColor;
				ctx.lineWidth = config.datasetStrokeWidth;
				ctx.beginPath();
				ctx.moveTo(yAxisPosX, xAxisPosY - animPc*(calculateOffset(data.datasets[i].data[0],calculatedScale,scaleHop)))

				for (var j=1; j<data.datasets[i].data.length; j++){
					if (config.bezierCurve){
						ctx.bezierCurveTo(xPos(j-0.5),yPos(i,j-1),xPos(j-0.5),yPos(i,j),xPos(j),yPos(i,j));
					}
					else{
						ctx.lineTo(xPos(j),yPos(i,j));
					}
				}
				ctx.stroke();
				if (config.datasetFill){
					ctx.lineTo(yAxisPosX + (valueHop*(data.datasets[i].data.length-1)),xAxisPosY);
					ctx.lineTo(yAxisPosX,xAxisPosY);
					ctx.closePath();
					ctx.fillStyle = data.datasets[i].fillColor;
					ctx.fill();
				}
				else{
					ctx.closePath();
				}
				if(config.pointDot){
					ctx.fillStyle = data.datasets[i].pointColor;
					ctx.strokeStyle = data.datasets[i].pointStrokeColor;
					ctx.lineWidth = config.pointDotStrokeWidth;
					for (var k=0; k<data.datasets[i].data.length; k++){
						ctx.beginPath();
						ctx.arc(yAxisPosX + (valueHop *k),xAxisPosY - animPc*(calculateOffset(data.datasets[i].data[k],calculatedScale,scaleHop)),config.pointDotRadius,0,Math.PI*2,true);
						ctx.fill();
						ctx.stroke();
					}
				}
			}
			
			function yPos(dataSet,iteration){
				return xAxisPosY - animPc*(calculateOffset(data.datasets[dataSet].data[iteration],calculatedScale,scaleHop));			
			}
			function xPos(iteration){
				return yAxisPosX + (valueHop * iteration);
			}
		}
		function drawScale(){
			//X axis line
			ctx.lineWidth = config.scaleLineWidth;
			ctx.strokeStyle = config.scaleLineColor;
			ctx.beginPath();
			ctx.moveTo(width-widestXLabel/2+5,xAxisPosY);
			ctx.lineTo(width-(widestXLabel/2)-xAxisLength-5,xAxisPosY);
			ctx.stroke();
			
			
			if (rotateLabels > 0){
				ctx.save();
				ctx.textAlign = "right";
			}
			else{
				ctx.textAlign = "center";
			}
			ctx.fillStyle = config.scaleFontColor;
			for (var i=0; i<data.labels.length; i++){
				ctx.save();
				if (rotateLabels > 0){
					ctx.translate(yAxisPosX + i*valueHop,xAxisPosY + config.scaleFontSize);
					ctx.rotate(-(rotateLabels * (Math.PI/180)));
					ctx.fillText(data.labels[i], 0,0);
					ctx.restore();
				}
				
				else{
					ctx.fillText(data.labels[i], yAxisPosX + i*valueHop,xAxisPosY + config.scaleFontSize+3);					
				}

				ctx.beginPath();
				ctx.moveTo(yAxisPosX + i * valueHop, xAxisPosY+3);
				
				//Check i isnt 0, so we dont go over the Y axis twice.
				if(config.scaleShowGridLines && i>0){
					ctx.lineWidth = config.scaleGridLineWidth;
					ctx.strokeStyle = config.scaleGridLineColor;					
					ctx.lineTo(yAxisPosX + i * valueHop, 5);
				}
				else{
					ctx.lineTo(yAxisPosX + i * valueHop, xAxisPosY+3);				
				}
				ctx.stroke();
			}
			
			//Y axis
			ctx.lineWidth = config.scaleLineWidth;
			ctx.strokeStyle = config.scaleLineColor;
			ctx.beginPath();
			ctx.moveTo(yAxisPosX,xAxisPosY+5);
			ctx.lineTo(yAxisPosX,5);
			ctx.stroke();
			
			ctx.textAlign = "right";
			ctx.textBaseline = "middle";
			for (var j=0; j<calculatedScale.steps; j++){
				ctx.beginPath();
				ctx.moveTo(yAxisPosX-3,xAxisPosY - ((j+1) * scaleHop));
				if (config.scaleShowGridLines){
					ctx.lineWidth = config.scaleGridLineWidth;
					ctx.strokeStyle = config.scaleGridLineColor;
					ctx.lineTo(yAxisPosX + xAxisLength + 5,xAxisPosY - ((j+1) * scaleHop));					
				}
				else{
					ctx.lineTo(yAxisPosX-0.5,xAxisPosY - ((j+1) * scaleHop));
				}
				
				ctx.stroke();
				
				if (config.scaleShowLabels){
					ctx.fillText(calculatedScale.labels[j],yAxisPosX-8,xAxisPosY - ((j+1) * scaleHop));
				}
			}
			
			
		}
		function calculateXAxisSize(){
			var longestText = 1;
			//if we are showing the labels
			if (config.scaleShowLabels){
				ctx.font = config.scaleFontStyle + " " + config.scaleFontSize+"px " + config.scaleFontFamily;
				for (var i=0; i<calculatedScale.labels.length; i++){
					var measuredText = ctx.measureText(calculatedScale.labels[i]).width;
					longestText = (measuredText > longestText)? measuredText : longestText;
				}
				//Add a little extra padding from the y axis
				longestText +=10;
			}
			xAxisLength = width - longestText - widestXLabel;
			valueHop = Math.floor(xAxisLength/(data.labels.length-1));	
				
			yAxisPosX = width-widestXLabel/2-xAxisLength;
			xAxisPosY = scaleHeight + config.scaleFontSize/2;				
		}		
		function calculateDrawingSizes(){
			maxSize = height;

			//Need to check the X axis first - measure the length of each text metric, and figure out if we need to rotate by 45 degrees.
			ctx.font = config.scaleFontStyle + " " + config.scaleFontSize+"px " + config.scaleFontFamily;
			widestXLabel = 1;
			for (var i=0; i<data.labels.length; i++){
				var textLength = ctx.measureText(data.labels[i]).width;
				//If the text length is longer - make that equal to longest text!
				widestXLabel = (textLength > widestXLabel)? textLength : widestXLabel;
			}
			if (width/data.labels.length < widestXLabel){
				rotateLabels = 45;
				if (width/data.labels.length < Math.cos(rotateLabels) * widestXLabel){
					rotateLabels = 90;
					maxSize -= widestXLabel; 
				}
				else{
					maxSize -= Math.sin(rotateLabels) * widestXLabel;
				}
			}
			else{
				maxSize -= config.scaleFontSize;
			}
			
			//Add a little padding between the x line and the text
			maxSize -= 5;
			
			
			labelHeight = config.scaleFontSize;
			
			maxSize -= labelHeight;
			//Set 5 pixels greater than the font size to allow for a little padding from the X axis.
			
			scaleHeight = maxSize;
			
			//Then get the area above we can safely draw on.
			
		}		
		function getValueBounds() {
			var upperValue = Number.MIN_VALUE;
			var lowerValue = Number.MAX_VALUE;
			for (var i=0; i<data.datasets.length; i++){
				for (var j=0; j<data.datasets[i].data.length; j++){
					if ( data.datasets[i].data[j] > upperValue) { upperValue = data.datasets[i].data[j] };
					if ( data.datasets[i].data[j] < lowerValue) { lowerValue = data.datasets[i].data[j] };
				}
			};
	
			var maxSteps = Math.floor((scaleHeight / (labelHeight*0.66)));
			var minSteps = Math.floor((scaleHeight / labelHeight*0.5));
			
			return {
				maxValue : upperValue,
				minValue : lowerValue,
				maxSteps : maxSteps,
				minSteps : minSteps
			};
			
	
		}

		
	}
	
	var Bar = function(data,config,ctx){
		var maxSize, scaleHop, calculatedScale, labelHeight, scaleHeight, valueBounds, labelTemplateString, valueHop,widestXLabel, xAxisLength,yAxisPosX,xAxisPosY,barWidth, rotateLabels = 0;
			
		calculateDrawingSizes();
		
		valueBounds = getValueBounds();
		//Check and set the scale
		labelTemplateString = (config.scaleShowLabels)? config.scaleLabel : "";
		if (!config.scaleOverride){
			
			calculatedScale = calculateScale(scaleHeight,valueBounds.maxSteps,valueBounds.minSteps,valueBounds.maxValue,valueBounds.minValue,labelTemplateString);
		}
		else {
			calculatedScale = {
				steps : config.scaleSteps,
				stepValue : config.scaleStepWidth,
				graphMin : config.scaleStartValue,
				labels : []
			}
			populateLabels(labelTemplateString, calculatedScale.labels,calculatedScale.steps,config.scaleStartValue,config.scaleStepWidth);
		}
		
		scaleHop = Math.floor(scaleHeight/calculatedScale.steps);
		calculateXAxisSize();
		animationLoop(config,drawScale,drawBars,ctx);		
		
		function drawBars(animPc){
			ctx.lineWidth = config.barStrokeWidth;
			for (var i=0; i<data.datasets.length; i++){
					ctx.fillStyle = data.datasets[i].fillColor;
					ctx.strokeStyle = data.datasets[i].strokeColor;
				for (var j=0; j<data.datasets[i].data.length; j++){
					var barOffset = yAxisPosX + config.barValueSpacing + valueHop*j + barWidth*i + config.barDatasetSpacing*i + config.barStrokeWidth*i;
					
					ctx.beginPath();
					ctx.moveTo(barOffset, xAxisPosY);
					ctx.lineTo(barOffset, xAxisPosY - animPc*calculateOffset(data.datasets[i].data[j],calculatedScale,scaleHop)+(config.barStrokeWidth/2));
					ctx.lineTo(barOffset + barWidth, xAxisPosY - animPc*calculateOffset(data.datasets[i].data[j],calculatedScale,scaleHop)+(config.barStrokeWidth/2));
					ctx.lineTo(barOffset + barWidth, xAxisPosY);
					if(config.barShowStroke){
						ctx.stroke();
					}
					ctx.closePath();
					ctx.fill();
				}
			}
			
		}
		function drawScale(){
			//X axis line
			ctx.lineWidth = config.scaleLineWidth;
			ctx.strokeStyle = config.scaleLineColor;
			ctx.beginPath();
			ctx.moveTo(width-widestXLabel/2+5,xAxisPosY);
			ctx.lineTo(width-(widestXLabel/2)-xAxisLength-5,xAxisPosY);
			ctx.stroke();
			
			
			if (rotateLabels > 0){
				ctx.save();
				ctx.textAlign = "right";
			}
			else{
				ctx.textAlign = "center";
			}
			ctx.fillStyle = config.scaleFontColor;
			for (var i=0; i<data.labels.length; i++){
				ctx.save();
				if (rotateLabels > 0){
					ctx.translate(yAxisPosX + i*valueHop,xAxisPosY + config.scaleFontSize);
					ctx.rotate(-(rotateLabels * (Math.PI/180)));
					ctx.fillText(data.labels[i], 0,0);
					ctx.restore();
				}
				
				else{
					ctx.fillText(data.labels[i], yAxisPosX + i*valueHop + valueHop/2,xAxisPosY + config.scaleFontSize+3);					
				}

				ctx.beginPath();
				ctx.moveTo(yAxisPosX + (i+1) * valueHop, xAxisPosY+3);
				
				//Check i isnt 0, so we dont go over the Y axis twice.
					ctx.lineWidth = config.scaleGridLineWidth;
					ctx.strokeStyle = config.scaleGridLineColor;					
					ctx.lineTo(yAxisPosX + (i+1) * valueHop, 5);
				ctx.stroke();
			}
			
			//Y axis
			ctx.lineWidth = config.scaleLineWidth;
			ctx.strokeStyle = config.scaleLineColor;
			ctx.beginPath();
			ctx.moveTo(yAxisPosX,xAxisPosY+5);
			ctx.lineTo(yAxisPosX,5);
			ctx.stroke();
			
			ctx.textAlign = "right";
			ctx.textBaseline = "middle";
			for (var j=0; j<calculatedScale.steps; j++){
				ctx.beginPath();
				ctx.moveTo(yAxisPosX-3,xAxisPosY - ((j+1) * scaleHop));
				if (config.scaleShowGridLines){
					ctx.lineWidth = config.scaleGridLineWidth;
					ctx.strokeStyle = config.scaleGridLineColor;
					ctx.lineTo(yAxisPosX + xAxisLength + 5,xAxisPosY - ((j+1) * scaleHop));					
				}
				else{
					ctx.lineTo(yAxisPosX-0.5,xAxisPosY - ((j+1) * scaleHop));
				}
				
				ctx.stroke();
				if (config.scaleShowLabels){
					ctx.fillText(calculatedScale.labels[j],yAxisPosX-8,xAxisPosY - ((j+1) * scaleHop));
				}
			}
			
			
		}
		function calculateXAxisSize(){
			var longestText = 1;
			//if we are showing the labels
			if (config.scaleShowLabels){
				ctx.font = config.scaleFontStyle + " " + config.scaleFontSize+"px " + config.scaleFontFamily;
				for (var i=0; i<calculatedScale.labels.length; i++){
					var measuredText = ctx.measureText(calculatedScale.labels[i]).width;
					longestText = (measuredText > longestText)? measuredText : longestText;
				}
				//Add a little extra padding from the y axis
				longestText +=10;
			}
			xAxisLength = width - longestText - widestXLabel;
			valueHop = Math.floor(xAxisLength/(data.labels.length));	
			
			barWidth = (valueHop - config.scaleGridLineWidth*2 - (config.barValueSpacing*2) - (config.barDatasetSpacing*data.datasets.length-1) - ((config.barStrokeWidth/2)*data.datasets.length-1))/data.datasets.length;
			
			yAxisPosX = width-widestXLabel/2-xAxisLength;
			xAxisPosY = scaleHeight + config.scaleFontSize/2;				
		}		
		function calculateDrawingSizes(){
			maxSize = height;

			//Need to check the X axis first - measure the length of each text metric, and figure out if we need to rotate by 45 degrees.
			ctx.font = config.scaleFontStyle + " " + config.scaleFontSize+"px " + config.scaleFontFamily;
			widestXLabel = 1;
			for (var i=0; i<data.labels.length; i++){
				var textLength = ctx.measureText(data.labels[i]).width;
				//If the text length is longer - make that equal to longest text!
				widestXLabel = (textLength > widestXLabel)? textLength : widestXLabel;
			}
			if (width/data.labels.length < widestXLabel){
				rotateLabels = 45;
				if (width/data.labels.length < Math.cos(rotateLabels) * widestXLabel){
					rotateLabels = 90;
					maxSize -= widestXLabel; 
				}
				else{
					maxSize -= Math.sin(rotateLabels) * widestXLabel;
				}
			}
			else{
				maxSize -= config.scaleFontSize;
			}
			
			//Add a little padding between the x line and the text
			maxSize -= 5;
			
			
			labelHeight = config.scaleFontSize;
			
			maxSize -= labelHeight;
			//Set 5 pixels greater than the font size to allow for a little padding from the X axis.
			
			scaleHeight = maxSize;
			
			//Then get the area above we can safely draw on.
			
		}		
		function getValueBounds() {
			var upperValue = Number.MIN_VALUE;
			var lowerValue = Number.MAX_VALUE;
			for (var i=0; i<data.datasets.length; i++){
				for (var j=0; j<data.datasets[i].data.length; j++){
					if ( data.datasets[i].data[j] > upperValue) { upperValue = data.datasets[i].data[j] };
					if ( data.datasets[i].data[j] < lowerValue) { lowerValue = data.datasets[i].data[j] };
				}
			};
	
			var maxSteps = Math.floor((scaleHeight / (labelHeight*0.66)));
			var minSteps = Math.floor((scaleHeight / labelHeight*0.5));
			
			return {
				maxValue : upperValue,
				minValue : lowerValue,
				maxSteps : maxSteps,
				minSteps : minSteps
			};
			
	
		}
	}
	
	function calculateOffset(val,calculatedScale,scaleHop){
		var outerValue = calculatedScale.steps * calculatedScale.stepValue;
		var adjustedValue = val - calculatedScale.graphMin;
		var scalingFactor = CapValue(adjustedValue/outerValue,1,0);
		return (scaleHop*calculatedScale.steps) * scalingFactor;
	}
	
	function animationLoop(config,drawScale,drawData,ctx){
		var animFrameAmount = (config.animation)? 1/CapValue(config.animationSteps,Number.MAX_VALUE,1) : 1,
			easingFunction = animationOptions[config.animationEasing],
			percentAnimComplete =(config.animation)? 0 : 1;
		
	
		
		if (typeof drawScale !== "function") drawScale = function(){};
		
		requestAnimFrame(animLoop);
		
		function animateFrame(){
			var easeAdjustedAnimationPercent =(config.animation)? CapValue(easingFunction(percentAnimComplete),null,0) : 1;
			clear(ctx);
			if(config.scaleOverlay){
				drawData(easeAdjustedAnimationPercent);
				drawScale();
			} else {
				drawScale();
				drawData(easeAdjustedAnimationPercent);
			}				
		}
		function animLoop(){
			//We need to check if the animation is incomplete (less than 1), or complete (1).
				percentAnimComplete += animFrameAmount;
				animateFrame();	
				//Stop the loop continuing forever
				if (percentAnimComplete <= 1){
					requestAnimFrame(animLoop);
				}
				else{
					if (typeof config.onAnimationComplete == "function") config.onAnimationComplete();
				}
			
		}		
		
	}

	//Declare global functions to be called within this namespace here.
	
	
	// shim layer with setTimeout fallback
	var requestAnimFrame = (function(){
		return window.requestAnimationFrame ||
			window.webkitRequestAnimationFrame ||
			window.mozRequestAnimationFrame ||
			window.oRequestAnimationFrame ||
			window.msRequestAnimationFrame ||
			function(callback) {
				window.setTimeout(callback, 1000 / 60);
			};
	})();

	function calculateScale(drawingHeight,maxSteps,minSteps,maxValue,minValue,labelTemplateString){
			var graphMin,graphMax,graphRange,stepValue,numberOfSteps,valueRange,rangeOrderOfMagnitude,decimalNum;
			
			valueRange = maxValue - minValue;
			
			rangeOrderOfMagnitude = calculateOrderOfMagnitude(valueRange);

        	graphMin = Math.floor(minValue / (1 * Math.pow(10, rangeOrderOfMagnitude))) * Math.pow(10, rangeOrderOfMagnitude);
            
            graphMax = Math.ceil(maxValue / (1 * Math.pow(10, rangeOrderOfMagnitude))) * Math.pow(10, rangeOrderOfMagnitude);
            
            graphRange = graphMax - graphMin;
            
            stepValue = Math.pow(10, rangeOrderOfMagnitude);
            
	        numberOfSteps = Math.round(graphRange / stepValue);
	        
	        //Compare number of steps to the max and min for that size graph, and add in half steps if need be.	        
	        while(numberOfSteps < minSteps || numberOfSteps > maxSteps) {
	        	if (numberOfSteps < minSteps){
			        stepValue /= 2;
			        numberOfSteps = Math.round(graphRange/stepValue);
		        }
		        else{
			        stepValue *=2;
			        numberOfSteps = Math.round(graphRange/stepValue);
		        }
	        };

	        var labels = [];
	        populateLabels(labelTemplateString, labels, numberOfSteps, graphMin, stepValue);
		
	        return {
		        steps : numberOfSteps,
				stepValue : stepValue,
				graphMin : graphMin,
				labels : labels		        
		        
	        }
		
			function calculateOrderOfMagnitude(val){
			  return Math.floor(Math.log(val) / Math.LN10);
			}		


	}

    //Populate an array of all the labels by interpolating the string.
    function populateLabels(labelTemplateString, labels, numberOfSteps, graphMin, stepValue) {
        if (labelTemplateString) {
            //Fix floating point errors by setting to fixed the on the same decimal as the stepValue.
            for (var i = 1; i < numberOfSteps + 1; i++) {
                labels.push(tmpl(labelTemplateString, {value: (graphMin + (stepValue * i)).toFixed(getDecimalPlaces(stepValue))}));
            }
        }
    }
	
	//Max value from array
	function Max( array ){
		return Math.max.apply( Math, array );
	};
	//Min value from array
	function Min( array ){
		return Math.min.apply( Math, array );
	};
	//Default if undefined
	function Default(userDeclared,valueIfFalse){
		if(!userDeclared){
			return valueIfFalse;
		} else {
			return userDeclared;
		}
	};
	//Is a number function
	function isNumber(n) {
		return !isNaN(parseFloat(n)) && isFinite(n);
	}
	//Apply cap a value at a high or low number
	function CapValue(valueToCap, maxValue, minValue){
		if(isNumber(maxValue)) {
			if( valueToCap > maxValue ) {
				return maxValue;
			}
		}
		if(isNumber(minValue)){
			if ( valueToCap < minValue ){
				return minValue;
			}
		}
		return valueToCap;
	}
	function getDecimalPlaces (num){
		var numberOfDecimalPlaces;
		if (num%1!=0){
			return num.toString().split(".")[1].length
		}
		else{
			return 0;
		}
		
	} 
	
	function mergeChartConfig(defaults,userDefined){
		var returnObj = {};
	    for (var attrname in defaults) { returnObj[attrname] = defaults[attrname]; }
	    for (var attrname in userDefined) { returnObj[attrname] = userDefined[attrname]; }
	    return returnObj;
	}
	
	//Javascript micro templating by John Resig - source at http://ejohn.org/blog/javascript-micro-templating/
	  var cache = {};
	 
	  function tmpl(str, data){
	    // Figure out if we're getting a template, or if we need to
	    // load the template - and be sure to cache the result.
	    var fn = !/\W/.test(str) ?
	      cache[str] = cache[str] ||
	        tmpl(document.getElementById(str).innerHTML) :
	     
	      // Generate a reusable function that will serve as a template
	      // generator (and which will be cached).
	      new Function("obj",
	        "var p=[],print=function(){p.push.apply(p,arguments);};" +
	       
	        // Introduce the data as local variables using with(){}
	        "with(obj){p.push('" +
	       
	        // Convert the template into pure JavaScript
	        str
	          .replace(/[\r\t\n]/g, " ")
	          .split("<%").join("\t")
	          .replace(/((^|%>)[^\t]*)'/g, "$1\r")
	          .replace(/\t=(.*?)%>/g, "',$1,'")
	          .split("\t").join("');")
	          .split("%>").join("p.push('")
	          .split("\r").join("\\'")
	      + "');}return p.join('');");
	   
	    // Provide some basic currying to the user
	    return data ? fn( data ) : fn;
	  };
}


;
define("lib/chart", function(){});

/*global define*/
define('views/matchtip',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/matchtip',
	'hgn!templates/tipstatistic',
	'hgn!templates/matchsmall',
	'models/settings',
	'util/context',
	'util/message',
	'collections/results',
	'collections/tips',
	'util/country',
	'models/tip',
	'moment',
	'fetchcache',
	'util/wait',
	"i18n!nls/strings",
	'lib/chart',
	'bootstrap'
], function ($, _, Backbone, BaseView, matchtipTemplate,TipStatisticTemplate,matchSmallTemplate,SettingsModel,Context,MessageHelper,Results,
	Tips,CountryMapper,TipModel,moment,fetchcache,Wait,Strings
	) {
	


	var stdTimezoneOffset = function(date) {
	    var jan = new Date(date.getFullYear(), 0, 1);
	    var jul = new Date(date.getFullYear(), 6, 1);
	    return Math.max(jan.getTimezoneOffset(), jul.getTimezoneOffset());
	},
	dst = function(date) {
	    return date.getTimezoneOffset() < stdTimezoneOffset(date);
	},
	pad=function (number, length){
	    var str = "" + number
	    if (str.charAt(0)=='-'){
	    	length+=1;
	    }
	    while (str.length < length) {
	        str = '0'+str
	    }
	    return str
	},
	label =function(){
		var date=new Date();
		var hours=Math.floor(date.getTimezoneOffset()/60);
		var negative=hours <0
		if (dst(date)){
			hours+=1;
		}
		hours=Math.abs(hours);
		var minutest=date.getTimezoneOffset()%60;
		 return "UTC "+(negative?"+":"-")+pad(hours,2)+":"+pad(minutest,2)+(dst(date)?" ("+Strings.COMMON.SUMMER_TIME+")":"")
	}
	var View = BaseView.extend({

		titel : Strings.MATCHTIPS.TITLE,
		subView: true,
		el: '#wmapp',

		template: matchtipTemplate,

		events: {
			'click #save-tip':		'saveTip'
		},
		refresh:function(){
			Wait.start();
			var deferreds=[];
			deferreds.push(Tips.reload().fail(function(data,ajax,error,message){
						MessageHelper.handleError(ajax,Strings.MATCHTIPS.REFRESH_FAILED,this)
			}));
			
			if (this.match.get("scoreA")==-1){
				var kickoff=moment(new Date(this.match.get("kickOff")+"Z"))
				if (this.currentDate.isAfter(kickoff)){
					deferreds.push(this.match.reload().fail(function(data,ajax,error,message){
						MessageHelper.handleError(ajax,Strings.ERROR_CODE["MATCHES.ERR.FAILED_TO_REFRESH"],this);
					}));
				}
			}
			$.when.apply($,deferreds).done(_.bind(function(){
				this.initialize({id:this.match.get('id')});
			},this))
			.always(function(){
				Wait.stop();
			})
		},

		initialize: function (options) {
			var self=this;
			this.currentDate=moment(Context.getCurrentDate());
			this.match=Results.get(options.id);
			this.tip=Tips.findWhere({matchId:parseInt(options.id,10)});
			if (!this.canDoTips()){
			this.match.getResultStats().done(function(data){
				var points=[],bets=[],names=[],values=[];
				
				_.each(data,function(stat){
						if (stat.name.indexOf(":")==-1){
							if (stat.name == 1){
								points.push({value:parseInt(stat.value,10),color:"#a0a0a0"})
								self.$("#winner").text(stat.value)
							}else if (stat.name == 3){
								points.push({value:parseInt(stat.value,10),color:"#ffcc29"})
								self.$("#goals").text(stat.value)
							}else if (stat.name == 5){
								points.push({value:parseInt(stat.value,10),color:"#00a859"})
								self.$("#exact").text(stat.value)
							}else if (stat.name === "0"){
								self.$("#lost").text(stat.value)
							}
						}else{
							bets.push(stat);
						}
				});
				if (self.match.get("scoreA") === self.match.get("scoreB")){
					self.$("#winner").parent().hide();
				}
				if (points.length===0){
					self.$(".points-chart").hide();
				}else{
					self.$(".points-chart").show();
				}
				var bets2=_.sortBy(bets,function(v){
					return -1*parseInt(v.value,10);
				});
				var bets3=bets2.slice(0,8);
				if (bets2.length>8){
					var sum=0;
					for(var i=8;i<bets2.length;++i){
						sum+=parseInt(bets2[i].value,10)
					} 
					bets3.push({
						value:sum,
						name:Strings.MATCHTIPS.OTHER
					})
				}

				bets=_.sortBy(bets3,function(v){
					return v.name;
				});
				_.each(bets,function(val){
					names.push(val.name);
					values.push(parseInt(val.value,10));
				});

				var c=document.getElementById("points");
				var ctx=c.getContext("2d");
				
				//This will get the first returned node in the jQuery collection.
				
				var myNewChart = new Chart(ctx).Pie(points,{animation :false});
				

				var c2=document.getElementById("tips");
				var ctx2=c2.getContext("2d");
				
				//This will get the first returned node in the jQuery collection.
				var data2 = {
					labels : names,
					datasets : [
						{
							fillColor : "rgba(220,220,220,0.5)",
							strokeColor : "rgba(220,220,220,1)",
							//pointColor : "rgba(220,220,220,1)",
							//pointStrokeColor : "#fff",
							data : values
						}
					]
				}
				var myNewChart2 = new Chart(ctx2).Bar(data2,	{barShowStroke :false,animation :false,scaleFontColor:"#ffcc29"});
				self.$("#charts").show();
			})
}
			this.match.getStats().done(function(data){
				var aWinner=Math.round( data.tipCountAll?100/data.tipCountAll*data.tipCountWinnerA:0),
					bWinner=Math.round(data.tipCountAll?100/data.tipCountAll*data.tipCountWinnerB:0),
					draw=data.tipCountAll?100-aWinner-bWinner:0;
				self.$("#statistics").html(TipStatisticTemplate(
					$.extend({
							Strings:Strings,
							tipCountAll:data.tipCountAll,
							aWinner:aWinner,
							bWinner:bWinner,
							draw:draw
						},
						self.match.toJSON(),
						CountryMapper.getHelperMethods()
					)
				));
			}).fail(function(data){
				self.$("#statistics").text(Strings.MATCHTIPS.NO_STATS_AVAILABLE);
				MessageHelper.sendErrorDetails(data,Strings.ERROR_CODE["MATCHTIP.ERR.FAILED_TO_FETCH_STATS"],this);
			});
			//this.match.getResultStats().done(function(datax){
				//Get context with jQuery - using jQuery's .get() method.

		//	}).fail(function(data){
		//		self.$("#statistics").text(Strings.MATCHTIPS.NO_STATS_AVAILABLE);
		//		MessageHelper.sendErrorDetails(data,Strings.ERROR_CODE["MATCHTIP.ERR.FAILED_TO_FETCH_STATS"],this);
		//	});
			this.$el.html(this.template($.extend({},
				CountryMapper.getHelperMethods(),
				this.match.toJSON(),
				{
					hasResult:self.match.get("scoreA")!=-1,
					Strings:Strings,
					tip:this.tip&&this.tip.toJSON(),
					TimeZone:label(),
					KickOffTime:function(){
						return moment(new Date(this.kickOff+"Z")).format("LLLL");
					},
					LocalKickOffTime:function(){
						return moment(new Date(this.kickOff+"Z")).zone(Context.timezoneMap[this.location]).format("LLLL");
					},
					canDoTips:this.canDoTips(),
					isTipsToEarly:this.isTipsToEarly(),
					isTipsToLate:this.isTipsToLate(),
					"enabledFrom": function () {
						var dt=moment(new Date(this.tipsEnabledFrom+"Z"));
					      return  dt.format("LL")+" " +dt.format("LT");
					  },
					 "enabledTo": function () {
						var dt=moment(new Date(this.tipsEnabledTo+"Z"));
					      return  dt.format("LL")+" " +dt.format("LT");
					  }, extention:function(){
                        	var x=this.scoreExtensionDetail
                        	if (x !=  null && x.psoScoreText != null) {
								return x.psoScoreA+":"+x.psoScoreB +" "+Strings.COMMON.PSO;
							}
                        }
				}),
				{match:matchSmallTemplate.template}
			));
				
			if (this.tip){
				$('#goals-a option[value="'+this.tip.get("scoreA")+'"]').prop('selected', true);
				$('#goals-b option[value="'+this.tip.get("scoreB")+'"]').prop('selected', true);
			}

		},
		isTipsToEarly:function(){
			var tipsEnabledFrom=moment(new Date(this.match.get("tipsEnabledFrom")+"Z"));
				
				return this.currentDate.isBefore(tipsEnabledFrom);
		},
		isTipsToLate:function(){
			var tipsEnabledTo=moment(new Date(this.match.get("tipsEnabledTo")+"Z"));
				return this.currentDate.isAfter(tipsEnabledTo);
		},
		canDoTips:function(){
			return !this.isTipsToEarly() && !this.isTipsToLate();
		},
		saveTip:function(event){
			var verb;
			if (this.tip && this.tip.get('id')){
				verb="PUT";
			}else{
				verb="POST";
				this.tip=new TipModel({
					matchId:this.match.get('id')
				});
			}
			this.tip.set('scoreA',this.$("#goals-a").val());
			this.tip.set('scoreB',this.$("#goals-b").val());
			Wait.start(event.target)
			$(event.target).prop('disabled', true);
			$.ajax({
			  type: verb,
			  url:SettingsModel.get("server") +"/rest/matchtips",
			  contentType: 'application/json',
			  data: this.tip.toJSON()
			}).done(function(data){
				Tips.reload()
				.done(function(){
					console.log("Tips reloaded. going back;");
					window.history.back();
					Wait.stop();
					$(event.target).prop('disabled', false);
				})
				.fail(function(data,ajax,error,message){
					MessageHelper.handleError(ajax,Strings.ERROR_CODE["MATCHTIP.ERR.FAILED_TO_FETCH_UPDATED_TIPS"],this);
					Wait.stop();
					$(event.target).prop('disabled', false);
				});
				
			}).fail(function(ajax,error,message){
				Wait.stop();
				MessageHelper.handleError(ajax,Strings.ERROR_CODE["MATCHTIP.ERR.FAILED_TO_CREATE"],this);
				$(event.target).prop('disabled', false);
			});
			
		}
		
	});

	return View;
});

define("hgn!templates/rankingsplayer", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class='sub-container ");if(t.s(t.f("hasMoreOrLess",c,p,1),c,p,0,44,55,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("have-footer");});c.pop();}t.b("'>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("     <div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);t.b("            <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("                <table class=\"table\">\r");t.b("\n" + i);t.b("                    <tbody>\r");t.b("\n" + i);if(t.s(t.f("playerRank",c,p,1),c,p,0,312,1546,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                        <tr class='player-rank' data-player-id=\"");t.b(t.v(t.f("playerId",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("                            <td>\r");t.b("\n" + i);t.b("                                <div class=\"min-height-lists\">\r");t.b("\n" + i);if(t.s(t.f("hasPlayerRank",c,p,1),c,p,0,542,622,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                                    ");t.b(t.v(t.f("rank",c,p,0)));t.b("\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("hasPlayerRank",c,p,1),c,p,1,0,0,"")){t.b("                                    0\r");t.b("\n" + i);};t.b("                                </div>\r");t.b("\n" + i);t.b("                            </td>\r");t.b("\n" + i);t.b("                            <td>\r");t.b("\n" + i);t.b("                                <div class=\"min-height-lists\">\r");t.b("\n" + i);if(t.s(t.f("hasPlayerRank",c,p,1),c,p,0,1008,1152,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                                    <span class=\"badge badge-bg-primary pull-right\">");t.b(t.v(t.f("overallPoints",c,p,0)));t.b("</span>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("hasPlayerRank",c,p,1),c,p,1,0,0,"")){t.b("                                    <span class=\"badge badge-bg-primary pull-right\">0</span>\r");t.b("\n" + i);};t.b("                                ");t.b(t.v(t.f("nickName",c,p,0)));t.b("\r");t.b("\n" + i);t.b("                                </div>\r");t.b("\n" + i);t.b("                            </td>\r");t.b("\n" + i);t.b("                        </tr>\r");t.b("\n" + i);});c.pop();}t.b("                    </tbody>\r");t.b("\n" + i);t.b("                </table>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);if(t.s(t.f("hasMoreOrLess",c,p,1),c,p,0,1722,2366,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("<div class=\"sub-footer\">\r");t.b("\n" + i);t.b("    <div class=\"btn-group btn-group-lg btn-group-justified\">\r");t.b("\n" + i);if(t.s(t.f("hasLess",c,p,1),c,p,0,1831,2063,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"btn-group\">\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"load-prev\" class=\"btn footer-btn active\"><span class=\"wmicon icon-left-open-big\"></span> ");t.b(t.v(t.d("Strings.COMMON.PREV",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);});c.pop();}t.b("\r");t.b("\n" + i);if(t.s(t.f("hasMore",c,p,1),c,p,0,2099,2332,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"btn-group\">\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"load-next\" class=\"btn footer-btn active\">");t.b(t.v(t.d("Strings.COMMON.NEXT",c,p,0)));t.b(" <span class=\"wmicon icon-right-open-big\"></span></button>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);});c.pop();}t.b("    </div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);});c.pop();}return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/*global define*/
define('views/rankingsplayer',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/rankingsplayer',
	'models/settings',
	'util/context',
	'util/message',
	'util/wait',
	"i18n!nls/strings",
	'bootstrap'
], function ($, _, Backbone, BaseView, rankingsplayerTemplate,Settings,Context,MessageHelper,Wait,Strings) {
	

	var View = BaseView.extend({
		titel : Strings.RANKINGS_PLAYER.HEADLINE,
		el: '#wmapp',

		template: rankingsplayerTemplate,

		events: {
			'click #load-next':'loadNext',
			'click #load-prev':'loadPrev',
			'click .player-rank': 'openPlayerDetails'
		},

		currentStart:0,
		pageSize:10,
		initialize: function (options) {
			Context.playerIterationToken=Context.playerIterationToken||new Date().getTime();
			this.updateOptions(options);
		},
		openPlayerDetails:function(event){
			var playerId=$(event.currentTarget).data('player-id');
			Context.Router.navigate("#player/"+playerId, {trigger: true, replace: false});
		},
		updateOptions:function(options){
			if (options.currentStart){
				this.currentStart=parseInt(options.currentStart,10);
			}
			Wait.start();
			this.loadPage(true).always(function(){Wait.stop();});
		},
		refresh:function(){
			Wait.start();
			Context.playerIterationToken=new Date().getTime();
			this.loadPage(true,true).always(function(){Wait.stop();});
		},
		loadPage: function (replaceUrl,reload) {
			var options={
				type: "GET",
				url:Settings.get("server") +"/rest/player/globalranking?from="+this.currentStart+"&size="+this.pageSize+"&_it="+Context.playerIterationToken

			};
			return $.ajax(options)
			.done(_.bind(function(playerRank){
				if (playerRank.length>0){
					this.viewModel={
						playerRank:playerRank,
						hasMore:playerRank.length===this.pageSize,
						hasLess:this.currentStart!==0,
						hasMoreOrLess:this.currentStart!==0 ||playerRank.length===this.pageSize,
						hasPlayerRank:function(){
							return this.rank!==2147483647;
						},
					};
					this.$el.html(this.template($.extend({Strings:Strings},this.viewModel)));
					Context.Router.navigate("#rankingsplayer/"+this.currentStart, {trigger: false, replace: replaceUrl});
				}else{
					if (this.currentStart>=this.pageSize){
						this.currentStart-=this.pageSize;
						if (this.viewModel){
							this.viewModel.hasMore=false;
							this.groupIndex-=this.pageSize;
							this.$el.html(this.template($.extend({Strings:Strings},this.viewModel)));
						}else{
							MessageHelper.showError("Invalid page requested");
						}
					}
				}
			},this))
			.fail(function(ajax,error,message){
				MessageHelper.handleError(ajax,Strings.ERROR_CODE["RANKINGS.ERR.LOADING_FAILED"],this);
			});
		},
		loadNext:function(event){
			this.currentStart+=this.pageSize;
			Wait.start(event.target);
			this.loadPage().always(function(){Wait.stop();});
		},
		loadPrev:function(event){
			if (this.currentStart>=this.pageSize){
				this.currentStart-=this.pageSize;
				Wait.start(event.target);
				this.loadPage().always(function(){Wait.stop();});
			}
		}
	});

	return View;
});

define("hgn!templates/rankingsgroup", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class='sub-container ");if(t.s(t.f("hasMoreOrLess",c,p,1),c,p,0,44,55,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("have-footer");});c.pop();}t.b("'>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("     <div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);t.b("            <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("                <table class=\"table\">\r");t.b("\n" + i);t.b("                    <tbody>\r");t.b("\n" + i);if(t.s(t.f("groupRank",c,p,1),c,p,0,311,1505,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                        <tr class='group-rank' data-group-id=\"");t.b(t.v(t.f("id",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("                            <td><div class=\"min-height-lists\">");t.b(t.v(t.f("groupIndex",c,p,0)));t.b("</div></td>\r");t.b("\n" + i);t.b("                            <td>\r");t.b("\n" + i);t.b("                                <div class=\"min-height-lists\">\r");t.b("\n" + i);if(t.s(t.f("hasGroupPoints",c,p,1),c,p,0,623,801,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                                    <span class=\"badge badge-bg-primary pull-right\"><span class=\"wmicon icon-avg\" />&nbsp;");t.b(t.v(t.f("avgPoints",c,p,0)));t.b("</span>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("hasGroupPoints",c,p,1),c,p,1,0,0,"")){t.b("                                    <span class=\"badge badge-bg-primary pull-right\"><span class=\"wmicon icon-avg\" />&nbsp;0</span>\r");t.b("\n" + i);};t.b("                                <span class=\"pull-right wmicon icon-users-2 content-font-small\" >&nbsp;");t.b(t.v(t.f("memberCount",c,p,0)));t.b("&nbsp;&nbsp;</span>\r");t.b("\n" + i);t.b("                                ");t.b(t.v(t.f("name",c,p,0)));t.b("        ");if(!t.s(t.f("publicGroup",c,p,1),c,p,1,0,0,"")){t.b("\r");t.b("\n" + i);t.b("        <span class=\"wmicon icon-lock  color-grey-profile padding-left-10\"></span>\r");t.b("\n" + i);};t.b("                                </div>\r");t.b("\n" + i);t.b("                            </td>\r");t.b("\n" + i);t.b("                        </tr>\r");t.b("\n" + i);});c.pop();}t.b("                    </tbody>\r");t.b("\n" + i);t.b("                </table>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);if(t.s(t.f("hasMoreOrLess",c,p,1),c,p,0,1680,2324,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("<div class=\"sub-footer\">\r");t.b("\n" + i);t.b("    <div class=\"btn-group btn-group-lg btn-group-justified\">\r");t.b("\n" + i);if(t.s(t.f("hasLess",c,p,1),c,p,0,1789,2021,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"btn-group\">\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"load-prev\" class=\"btn footer-btn active\"><span class=\"wmicon icon-left-open-big\"></span> ");t.b(t.v(t.d("Strings.COMMON.PREV",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);});c.pop();}t.b("\r");t.b("\n" + i);if(t.s(t.f("hasMore",c,p,1),c,p,0,2057,2290,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"btn-group\">\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"load-next\" class=\"btn footer-btn active\">");t.b(t.v(t.d("Strings.COMMON.NEXT",c,p,0)));t.b(" <span class=\"wmicon icon-right-open-big\"></span></button>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);});c.pop();}t.b("    </div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);});c.pop();}return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/*global define*/
define('views/rankingsgroup',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/rankingsgroup',
	'models/settings',
	'util/context',
	'util/message',
	'util/wait',
	"i18n!nls/strings",
	'bootstrap'
], function ($, _, Backbone, BaseView, rankingsgroupTemplate,Settings,Context,MessageHelper,Wait,Strings) {
	

	var View = BaseView.extend({
		titel : Strings.RANKINGS_GROUP.HEADLINE,
		el: '#wmapp',

		template: rankingsgroupTemplate,

		events: {
			'click #load-next':'loadNext',
			'click #load-prev':'loadPrev',
			'click .group-rank': 'openGroupDetails'
		},
		currentStart:0,
		pageSize:10,
		groupIndex:1,

		initialize: function (options) {
			Context.groupRankIterationToken=Context.groupRankIterationToken||new Date().getTime();
			this.updateOptions(options);
			
		},
		openGroupDetails:function(event){
			var groupId=$(event.currentTarget).data('group-id');
			Context.Router.navigate("#group/"+groupId+"/0", {trigger: true, replace: false});
		},
		updateOptions:function(options){
			if (options.currentStart){
				this.currentStart=parseInt(options.currentStart,10);
			}
			this.loadPage(true)
		},
		refresh:function(){
    		Wait.start();
    		Context.groupRankIterationToken=new Date().getTime();
    		this.loadPage(true,true).always(function(){Wait.stop();});
   		},
		loadPage: function (replaceUrl,reload) {
			Wait.start();
			var options={
				type: "GET",
				url:Settings.get("server") +"/rest/groups/ranking/usergroups?from="+this.currentStart+"&size="+this.pageSize+"&_it="+Context.groupRankIterationToken
			};
			
			return $.ajax(options)
			.done(_.bind(function(groupRank){
				if (groupRank.length>0){
					this.groupIndex=this.currentStart+1;
					this.viewModel={
						groupRank:groupRank,
						hasMore:groupRank.length===this.pageSize,
						hasLess:this.currentStart!==0,
						hasMoreOrLess:this.currentStart!==0 ||groupRank.length===this.pageSize,
						groupIndex:_.bind(function(){
							return this.groupIndex++;
						},this),
						hasGroupPoints:function(){
							return this.avgPoints!==0;
						},

					};
					this.$el.html(this.template($.extend({Strings:Strings},this.viewModel)));
					Context.Router.navigate("#rankingsgroup/"+this.currentStart, {trigger: false, replace: replaceUrl});
				}else{
					if (this.currentStart>=this.pageSize){
						this.currentStart-=this.pageSize;
						if (this.viewModel){
							this.viewModel.hasMore=false;
							this.groupIndex-=this.pageSize;
							this.$el.html(this.template($.extend({Strings:Strings},this.viewModel)));
						}else{
							MessageHelper.showError("Invalid page requested");
						}
					}
				}
				Wait.stop();
			},this))
			.fail(function(ajax,error,message){
				Wait.stop();
				MessageHelper.handleError(ajax,Strings.ERROR_CODE["RANKINGS.ERR.LOADING_FAILED"],this);
			})
		},
		loadNext:function(event){
			this.currentStart+=this.pageSize;
			Wait.start(event.target);
			this.loadPage().always(function(){Wait.stop();});
		},
		loadPrev:function(event){
			if (this.currentStart>=this.pageSize){
				this.currentStart-=this.pageSize;
				Wait.start(event.target);
				this.loadPage().always(function(){Wait.stop();});
			}
		}
		
	});

	return View;
});

define("hgn!templates/rankingscountry", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class='sub-container ");if(t.s(t.f("hasMoreOrLess",c,p,1),c,p,0,44,55,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("have-footer");});c.pop();}t.b("'>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("     <div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);t.b("            <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("                <table class=\"table\">\r");t.b("\n" + i);t.b("                    <tbody>\r");t.b("\n" + i);if(t.s(t.f("countryRank",c,p,1),c,p,0,313,1517,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                        <tr class='group-rank' data-group-id=\"");t.b(t.v(t.f("id",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("                            <td><div class=\"min-height-lists\">");t.b(t.v(t.f("countryIndex",c,p,0)));t.b("</div></td>\r");t.b("\n" + i);t.b("                            <td>\r");t.b("\n" + i);t.b("                                <div class=\"min-height-lists\">\r");t.b("\n" + i);t.b("                                <img src=\"");t.b(t.v(t.f("countryGroupFlag",c,p,0)));t.b("\" class=\"img-flag20 pull-left\" alt=\"");t.b(t.v(t.f("countryGroupFlag",c,p,0)));t.b("\" />&nbsp;\r");t.b("\n" + i);if(t.s(t.f("hasGroupPoints",c,p,1),c,p,0,757,935,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                                    <span class=\"badge badge-bg-primary pull-right\"><span class=\"wmicon icon-avg\" />&nbsp;");t.b(t.v(t.f("avgPoints",c,p,0)));t.b("</span>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("hasGroupPoints",c,p,1),c,p,1,0,0,"")){t.b("                                    <span class=\"badge badge-bg-primary pull-right\"><span class=\"wmicon icon-avg\" />&nbsp;0</span>\r");t.b("\n" + i);};t.b("                                <span class=\"pull-right wmicon icon-users-2 content-font-small\" >&nbsp;");t.b(t.v(t.f("memberCount",c,p,0)));t.b("&nbsp;&nbsp;</span>\r");t.b("\n" + i);t.b("                                ");t.b(t.v(t.f("countryGroupName",c,p,0)));t.b("\r");t.b("\n" + i);t.b("                                </div>\r");t.b("\n" + i);t.b("                            </td>\r");t.b("\n" + i);t.b("                        </tr>\r");t.b("\n" + i);});c.pop();}t.b("                    </tbody>\r");t.b("\n" + i);t.b("                </table>\r");t.b("\n" + i);t.b("                            \r");t.b("\n" + i);t.b("               \r");t.b("\n" + i);t.b("                \r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);if(t.s(t.f("hasMoreOrLess",c,p,1),c,p,0,1759,2403,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("<div class=\"sub-footer\">\r");t.b("\n" + i);t.b("    <div class=\"btn-group btn-group-lg btn-group-justified\">\r");t.b("\n" + i);if(t.s(t.f("hasLess",c,p,1),c,p,0,1868,2100,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"btn-group\">\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"load-prev\" class=\"btn footer-btn active\"><span class=\"wmicon icon-left-open-big\"></span> ");t.b(t.v(t.d("Strings.COMMON.PREV",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);});c.pop();}t.b("\r");t.b("\n" + i);if(t.s(t.f("hasMore",c,p,1),c,p,0,2136,2369,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"btn-group\">\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"load-next\" class=\"btn footer-btn active\">");t.b(t.v(t.d("Strings.COMMON.NEXT",c,p,0)));t.b(" <span class=\"wmicon icon-right-open-big\"></span></button>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);});c.pop();}t.b("    </div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);});c.pop();}return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/*global define*/
define('views/rankingscountry',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/rankingscountry',
	'models/settings',
	'util/context',
	'util/message',
	'util/wait',
	'util/country',
	"i18n!nls/strings",
	'bootstrap'
], function ($, _, Backbone, BaseView, rankingscountryTemplate,Settings,Context,MessageHelper,Wait,CountryMapper,Strings) {
	

	var View = BaseView.extend({
		titel : Strings.RANKINGS_COUNTRY.HEADLINE,
		el: '#wmapp',

		template: rankingscountryTemplate,

		events: {
			'click #load-next':'loadNext',
			'click #load-prev':'loadPrev',
			'click .group-rank': 'openGroupDetails'
		},
		currentStart:0,
		pageSize:10,
		countryIndex:1,
		openGroupDetails:function(event){
			var groupId=$(event.currentTarget).data('group-id');
			Context.Router.navigate("#group/"+groupId+"/0", {trigger: true, replace: false});
		},
		initialize: function (options) {
			Context.countryIterationToken=Context.countryIterationToken||new Date().getTime();
			this.updateOptions(options);
		},
		updateOptions:function(options){
			if (options.currentStart){
				this.currentStart=parseInt(options.currentStart,10);
			}
			Wait.start();
			this.loadPage(true).always(function(){Wait.stop();});
		},
		refresh:function(){
			Wait.start();
			Context.countryIterationToken=new Date().getTime();
			this.loadPage(true,true).always(function(){Wait.stop();});
		},
		loadPage: function (replaceUrl,reload) {
			var options={
			  type: "GET",
			  url:Settings.get("server") +"/rest/groups/ranking/countries?from="+this.currentStart+"&size="+this.pageSize+"&_it="+Context.countryIterationToken
			};
			return $.ajax(options)
			.done(_.bind(function(countryRank){
				if (countryRank.length>0){
					this.countryIndex=this.currentStart+1;
					this.viewModel={
						countryRank:countryRank,
						hasMore:countryRank.length===this.pageSize,
						hasLess:this.currentStart!==0,
						hasMoreOrLess:this.currentStart!==0 ||countryRank.length===this.pageSize,
						countryIndex:_.bind(function(){
							return this.countryIndex++;
						},this),
						countryGroupFlag:function(){
							var country=this.name.substr(8);
							return CountryMapper.getCountryFlag(country);
						},
						countryGroupName:function(){
							var country=this.name.substr(8);
							return CountryMapper.getCountryName(country)
						},
						hasGroupPoints:function(){
							return this.avgPoints!==0;
						},
					};
					this.$el.html(this.template($.extend({Strings:Strings},this.viewModel)));
					Context.Router.navigate("#rankingscountry/"+this.currentStart, {trigger: false, replace: replaceUrl});
				}else{
					if (this.currentStart>=this.pageSize){
						this.currentStart-=this.pageSize;
						this.countryIndex-=this.pageSize;
						if (this.viewModel){
							this.viewModel.hasMore=false;
							this.$el.html(this.template($.extend({Strings:Strings},this.viewModel)));
						}else{
							MessageHelper.showError("Invalid page requested");
						}
					}
				}
			},this))
			.fail(function(ajax,error,message){
				MessageHelper.handleError(ajax,Strings.ERROR_CODE["RANKINGS.ERR.LOADING_FAILED"],this);
			});
		},
		loadNext:function(event){
			this.currentStart+=this.pageSize;
			Wait.start(event.target);
			this.loadPage().always(function(){Wait.stop();});
		},
		loadPrev:function(event){
			if (this.currentStart>=this.pageSize){
				this.currentStart-=this.pageSize;
				Wait.start(event.target);
				this.loadPage().always(function(){Wait.stop();});
			}
		}
		
	});

	return View;
});

define("hgn!templates/help", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class='sub-container'>\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);t.b("<div class=\"panel-group\" id=\"accordion\">\r");t.b("\n" + i);t.b("  <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("      <div class=\"panel-heading clickable-heading\">\r");t.b("\n" + i);t.b("        <h4 class=\"panel-title\">\r");t.b("\n" + i);t.b("            <a data-toggle=\"collapse\" data-parent=\"#accordion\" href=\"#collapseGameplay\">\r");t.b("\n" + i);t.b("                ");t.b(t.v(t.d("Strings.HELP.GENERAL",c,p,0)));t.b("\r");t.b("\n" + i);t.b("            </a>\r");t.b("\n" + i);t.b("        </h4>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("    <div id=\"collapseGameplay\" class=\"panel-collapse collapse in\">\r");t.b("\n" + i);t.b("      <div class=\"panel-body\">\r");t.b("\n" + i);t.b("          ");t.b(t.t(t.d("Strings.HELP.GENERAL_DETAILS",c,p,0)));t.b("\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("  <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("    <div class=\"panel-heading clickable-heading\">\r");t.b("\n" + i);t.b("      <h4 class=\"panel-title\">\r");t.b("\n" + i);t.b("        <a data-toggle=\"collapse\" data-parent=\"#accordion\" href=\"#collapseRules\">\r");t.b("\n" + i);t.b("          ");t.b(t.v(t.d("Strings.HELP.REGISTRATION",c,p,0)));t.b("\r");t.b("\n" + i);t.b("        </a>\r");t.b("\n" + i);t.b("      </h4>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div id=\"collapseRules\" class=\"panel-collapse collapse\">\r");t.b("\n" + i);t.b("      <div class=\"panel-body\">\r");t.b("\n" + i);t.b("            ");t.b(t.v(t.d("Strings.HELP.REGISTRATION_DETAILS",c,p,0)));t.b("\r");t.b("\n" + i);t.b("          <br /><br />\r");t.b("\n" + i);t.b("          <ul type=\"disc\">\r");t.b("\n" + i);t.b("            <li>");t.b(t.v(t.d("Strings.HELP.AFTER_REG",c,p,0)));t.b("</li>\r");t.b("\n" + i);t.b("            <li>");t.b(t.v(t.d("Strings.HELP.INITIAL_PW",c,p,0)));t.b("</li>\r");t.b("\n" + i);t.b("            <li>");t.b(t.v(t.d("Strings.HELP.CHANGE_PW",c,p,0)));t.b("</li>\r");t.b("\n" + i);t.b("            <li>");t.b(t.v(t.d("Strings.HELP.ENJOY",c,p,0)));t.b("</li>\r");t.b("\n" + i);t.b("          </ul>\r");t.b("\n" + i);t.b("          <span class=\"font-bold\">");t.b(t.v(t.d("Strings.HELP.PLEASE_NOTE",c,p,0)));t.b("</span><br/>\r");t.b("\n" + i);t.b("          ");t.b(t.v(t.d("Strings.HELP.NOTE_REGISTRATION",c,p,0)));t.b("\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("  <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("    <div class=\"panel-heading\">\r");t.b("\n" + i);t.b("      <h4 class=\"panel-title\">\r");t.b("\n" + i);t.b("        <a data-toggle=\"collapse\" data-parent=\"#accordion\" href=\"#collapsePoints\">\r");t.b("\n" + i);t.b("          ");t.b(t.v(t.d("Strings.HELP.FEEDBACK_QUESTION",c,p,0)));t.b("\r");t.b("\n" + i);t.b("        </a>\r");t.b("\n" + i);t.b("      </h4>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div id=\"collapsePoints\" class=\"panel-collapse collapse\">\r");t.b("\n" + i);t.b("      <div class=\"panel-body\">\r");t.b("\n" + i);t.b("        ");t.b(t.v(t.d("Strings.HELP.ISSUE",c,p,0)));t.b(" <a href=\"#feedback\">");t.b(t.v(t.d("Strings.HELP.APP_FEEDBACK_1",c,p,0)));t.b("</a> ");t.b(t.v(t.d("Strings.HELP.APP_FEEDBACK_2",c,p,0)));t.b(" <a href=\"mailto:soccer2014@hp.com\">soccer2014@hp.com</a>. ");t.b(t.v(t.d("Strings.HELP.APP_FEEDBACK_3",c,p,0)));t.b("\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("  <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("    <div class=\"panel-heading\">\r");t.b("\n" + i);t.b("      <h4 class=\"panel-title\">\r");t.b("\n" + i);t.b("        <a data-toggle=\"collapse\" data-parent=\"#accordion\" href=\"#collapseBonus\">\r");t.b("\n" + i);t.b("          ");t.b(t.v(t.d("Strings.HELP.MATCH_TIPS_RULES",c,p,0)));t.b("\r");t.b("\n" + i);t.b("        </a>\r");t.b("\n" + i);t.b("      </h4>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div id=\"collapseBonus\" class=\"panel-collapse collapse\">\r");t.b("\n" + i);t.b("      <div class=\"panel-body\">\r");t.b("\n" + i);t.b("        ");t.b(t.v(t.d("Strings.HELP.PLACE_BETS",c,p,0)));t.b("\r");t.b("\n" + i);t.b("          <br /><br />\r");t.b("\n" + i);t.b("          <ul type=\"disc\">\r");t.b("\n" + i);t.b("            <li>");t.b(t.v(t.d("Strings.HELP.POINTS5",c,p,0)));t.b("</li>\r");t.b("\n" + i);t.b("            <li>");t.b(t.v(t.d("Strings.HELP.POINTS3",c,p,0)));t.b("</li>\r");t.b("\n" + i);t.b("            <li>");t.b(t.v(t.d("Strings.HELP.POINTS1",c,p,0)));t.b("</li>\r");t.b("\n" + i);t.b("          </ul>\r");t.b("\n" + i);t.b("          <span class=\"font-bold\">");t.b(t.v(t.d("Strings.HELP.PLEASE_NOTE",c,p,0)));t.b("</span><br/>\r");t.b("\n" + i);t.b("          ");t.b(t.v(t.d("Strings.HELP.NOTE_MATCH_TIPS_RULES",c,p,0)));t.b("\r");t.b("\n" + i);t.b("<br /><br />\r");t.b("\n" + i);t.b("          <span class=\"font-bold\">");t.b(t.v(t.d("Strings.HELP.IMPORTANT",c,p,0)));t.b("</span><br/>\r");t.b("\n" + i);t.b("          ");t.b(t.v(t.d("Strings.HELP.IMPORTANT_MATCH_TIPS_RUELS",c,p,0)));t.b("\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("  <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("    <div class=\"panel-heading\">\r");t.b("\n" + i);t.b("      <h4 class=\"panel-title\">\r");t.b("\n" + i);t.b("        <a data-toggle=\"collapse\" data-parent=\"#accordion\" href=\"#collapseGroup\">\r");t.b("\n" + i);t.b("          ");t.b(t.v(t.d("Strings.HELP.BONUS_TIPS_RULES",c,p,0)));t.b("\r");t.b("\n" + i);t.b("        </a>\r");t.b("\n" + i);t.b("      </h4>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div id=\"collapseGroup\" class=\"panel-collapse collapse\">\r");t.b("\n" + i);t.b("      <div class=\"panel-body\">\r");t.b("\n" + i);t.b("        ");t.b(t.v(t.d("Strings.HELP.BONUS_TEXT",c,p,0)));t.b("\r");t.b("\n" + i);t.b("          <br /><br />\r");t.b("\n" + i);t.b("          <ul type=\"disc\">\r");t.b("\n" + i);t.b("            <li>");t.b(t.v(t.d("Strings.HELP.WINNER",c,p,0)));t.b("</li>\r");t.b("\n" + i);t.b("            <li>");t.b(t.v(t.d("Strings.HELP.SECOND",c,p,0)));t.b("</li>\r");t.b("\n" + i);t.b("            <li>");t.b(t.v(t.d("Strings.HELP.GOLDEN_BALL",c,p,0)));t.b("</li>\r");t.b("\n" + i);t.b("            <li>");t.b(t.v(t.d("Strings.HELP.GOLDEN_BOOT",c,p,0)));t.b("</li>\r");t.b("\n" + i);t.b("            <li>");t.b(t.v(t.d("Strings.HELP.GOLDEN_GLOVE",c,p,0)));t.b("</li>\r");t.b("\n" + i);t.b("          </ul>\r");t.b("\n" + i);t.b("          ");t.b(t.v(t.d("Strings.HELP.BONUS_TEXT_2",c,p,0)));t.b("\r");t.b("\n" + i);t.b("          <br /><br />\r");t.b("\n" + i);t.b("          <span class=\"font-bold\">");t.b(t.v(t.d("Strings.HELP.PLEASE_NOTE",c,p,0)));t.b("</span><br/>\r");t.b("\n" + i);t.b("          ");t.b(t.v(t.d("Strings.HELP.NOTE_BONUS",c,p,0)));t.b("\r");t.b("\n" + i);t.b("          <br /><br />\r");t.b("\n" + i);t.b("          ");t.b(t.v(t.d("Strings.HELP.EXAMPLE",c,p,0)));t.b("\r");t.b("\n" + i);t.b("          <br /><br />\r");t.b("\n" + i);t.b("          <ul type=\"disc\">\r");t.b("\n" + i);t.b("            <li>");t.b(t.v(t.d("Strings.HELP.EXAMPLE_1",c,p,0)));t.b("</li>\r");t.b("\n" + i);t.b("            <li>");t.b(t.v(t.d("Strings.HELP.EXAMPLE_2",c,p,0)));t.b("</li>\r");t.b("\n" + i);t.b("            <li>");t.b(t.v(t.d("Strings.HELP.EXAMPLE_3",c,p,0)));t.b("</li>\r");t.b("\n" + i);t.b("            <li>");t.b(t.v(t.d("Strings.HELP.EXAMPLE_4",c,p,0)));t.b("</li>\r");t.b("\n" + i);t.b("          </ul>\r");t.b("\n" + i);t.b("              ");t.b(t.v(t.d("Strings.HELP.EXAMPLE_BONUS",c,p,0)));t.b("\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("  <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("    <div class=\"panel-heading\">\r");t.b("\n" + i);t.b("      <h4 class=\"panel-title\">\r");t.b("\n" + i);t.b("        <a data-toggle=\"collapse\" data-parent=\"#accordion\" href=\"#collapseRegister\">\r");t.b("\n" + i);t.b("          ");t.b(t.v(t.d("Strings.HELP.GROUP_HANDLING",c,p,0)));t.b("\r");t.b("\n" + i);t.b("        </a>\r");t.b("\n" + i);t.b("      </h4>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div id=\"collapseRegister\" class=\"panel-collapse collapse\">\r");t.b("\n" + i);t.b("      <div class=\"panel-body\">\r");t.b("\n" + i);t.b("          ");t.b(t.t(t.d("Strings.HELP.GROUP_HANDLING_TEXT",c,p,0)));t.b("\r");t.b("\n" + i);t.b("          <br /><br />\r");t.b("\n" + i);t.b("          <span class=\"font-bold\">");t.b(t.v(t.d("Strings.HELP.MANAGE_OWN_GROUP",c,p,0)));t.b("</span><br/>\r");t.b("\n" + i);t.b("          <ul type=\"disc\">\r");t.b("\n" + i);t.b("            <li>");t.b(t.v(t.d("Strings.HELP.MANAGE_1",c,p,0)));t.b("</li>\r");t.b("\n" + i);t.b("            <li>");t.b(t.v(t.d("Strings.HELP.MANAGE_2",c,p,0)));t.b("</li>\r");t.b("\n" + i);t.b("              <ul type=\"circle\">\r");t.b("\n" + i);t.b("                <li>");t.b(t.v(t.d("Strings.HELP.MANAGE_3",c,p,0)));t.b("</li>\r");t.b("\n" + i);t.b("                <li>");t.b(t.v(t.d("Strings.HELP.MANAGE_4",c,p,0)));t.b("</li>\r");t.b("\n" + i);t.b("                <li>");t.b(t.v(t.d("Strings.HELP.MANAGE_5",c,p,0)));t.b("</li>\r");t.b("\n" + i);t.b("              </ul>\r");t.b("\n" + i);t.b("          </ul>\r");t.b("\n" + i);t.b("          ");t.b(t.v(t.d("Strings.HELP.ADDING",c,p,0)));t.b("\r");t.b("\n" + i);t.b("          <br /><br />\r");t.b("\n" + i);t.b("          <span class=\"font-bold\">");t.b(t.v(t.d("Strings.HELP.JOINING",c,p,0)));t.b("</span><br/>\r");t.b("\n" + i);t.b("          <ul type=\"disc\">\r");t.b("\n" + i);t.b("            <li>");t.b(t.v(t.d("Strings.HELP.JOINING_1",c,p,0)));t.b("</li>\r");t.b("\n" + i);t.b("            <li>");t.b(t.v(t.d("Strings.HELP.JOINING_2",c,p,0)));t.b("</li>\r");t.b("\n" + i);t.b("            <li>");t.b(t.v(t.d("Strings.HELP.JOINING_3",c,p,0)));t.b("</li>\r");t.b("\n" + i);t.b("            <li>");t.b(t.v(t.d("Strings.HELP.JOINING_4",c,p,0)));t.b("</li>\r");t.b("\n" + i);t.b("          </ul>\r");t.b("\n" + i);t.b("          <span class=\"font-bold\">");t.b(t.v(t.d("Strings.HELP.PLEASE_NOTE",c,p,0)));t.b("</span><br/>\r");t.b("\n" + i);t.b("          ");t.b(t.t(t.d("Strings.HELP.NOTE_JOINING",c,p,0)));t.b("\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/*global define*/
define('views/help',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/help',
	'models/settings',
	'util/context',
	'util/message',
	"i18n!nls/strings",
	'bootstrap'
], function ($, _, Backbone, BaseView, helpTemplate, SettingsModel, Context, MessageHelper,Strings) {
	

	var View = BaseView.extend({
		titel : Strings.COMMON.HELP,
		el: '#wmapp',

		template: helpTemplate,

		events: {
		},
        
		initialize: function () {
			this.$el.html(this.template({Strings:Strings}));
		}
	});
    
    $('.collapse').collapse();
        
	return View;
});



define("hgn!templates/about", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class='sub-container'>\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);t.b("    <div class=\"form-group\">\r");t.b("\n" + i);t.b("      <h2>");t.b(t.v(t.d("Strings.ABOUT.ABOUT_HP",c,p,0)));t.b("</h2>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("     <div class=\"form-group btn btn-default text-wrap content-left padding-bottom-10\">\r");t.b("\n" + i);t.b("         ");t.b(t.v(t.d("Strings.ABOUT.ABOUT_HP_TEXT",c,p,0)));t.b("<a onclick=\"system_browser('http://www.hp.com');\">http://www.hp.com</a>\r");t.b("\n" + i);t.b("    </div>   \r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("    <div class=\"form-group padding-top-10\">\r");t.b("\n" + i);t.b("      <div class=\"panel panel-alpha\">\r");t.b("\n" + i);t.b("        <div class=\"list-group\">\r");t.b("\n" + i);t.b("            <div class=\"list-group-item\">\r");t.b("\n" + i);t.b("                <a onclick=\"system_browser('http://m.hp.com/us/en/privacy/ww-privacy.html');\">");t.b(t.v(t.d("Strings.ABOUT.PRIVACY_STATEMENT",c,p,0)));t.b("</a>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    \r");t.b("\n" + i);t.b("    <div class=\"form-group\">\r");t.b("\n" + i);t.b("      <h2>");t.b(t.v(t.d("Strings.ABOUT.ABOUT_SOCCER",c,p,0)));t.b("</h2>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    \r");t.b("\n" + i);t.b("    <div class=\"panel-group\" id=\"accordion\">\r");t.b("\n" + i);t.b("      <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("        <div class=\"panel-heading\">\r");t.b("\n" + i);t.b("          <h4 class=\"panel-title\">\r");t.b("\n" + i);t.b("            <a data-toggle=\"collapse\" data-parent=\"#accordion\" href=\"#collapseDesc\">\r");t.b("\n" + i);t.b("              ");t.b(t.v(t.d("Strings.ABOUT.DESCRIPTION",c,p,0)));t.b("\r");t.b("\n" + i);t.b("            </a>\r");t.b("\n" + i);t.b("          </h4>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("        <div id=\"collapseDesc\" class=\"panel-collapse collapse\">\r");t.b("\n" + i);t.b("          <div class=\"panel-body\">\r");t.b("\n" + i);t.b("              ");t.b(t.v(t.d("Strings.ABOUT.DESCRIPTION_TEXT",c,p,0)));t.b("\r");t.b("\n" + i);t.b("          </div>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("    <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("        <div class=\"panel-heading\">\r");t.b("\n" + i);t.b("          <h4 class=\"panel-title\">\r");t.b("\n" + i);t.b("            <a data-toggle=\"collapse\" data-parent=\"#accordion\" href=\"#collapseOrigin\">\r");t.b("\n" + i);t.b("                ");t.b(t.v(t.d("Strings.ABOUT.ORIGIN",c,p,0)));t.b("\r");t.b("\n" + i);t.b("            </a>\r");t.b("\n" + i);t.b("          </h4>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("        <div id=\"collapseOrigin\" class=\"panel-collapse collapse\">\r");t.b("\n" + i);t.b("          <div class=\"panel-body\">\r");t.b("\n" + i);t.b("              ");t.b(t.t(t.d("Strings.ABOUT.ORIGIN_TEXT",c,p,0)));t.b("\r");t.b("\n" + i);t.b("          </div>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("    <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("        <div class=\"panel-heading\">\r");t.b("\n" + i);t.b("          <h4 class=\"panel-title\">\r");t.b("\n" + i);t.b("            <a data-toggle=\"collapse\" data-parent=\"#accordion\" href=\"#collapseVersion\">\r");t.b("\n" + i);t.b("                ");t.b(t.v(t.d("Strings.ABOUT.VERSION_INFORMATION",c,p,0)));t.b("\r");t.b("\n" + i);t.b("            </a>\r");t.b("\n" + i);t.b("          </h4>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("        <div id=\"collapseVersion\" class=\"panel-collapse collapse in\">\r");t.b("\n" + i);t.b("          <div class=\"panel-body\">\r");t.b("\n" + i);t.b("                <div><span class=\"wmicon icon-tablet\" /> ");t.b(t.v(t.d("Strings.ABOUT.APP_VERSION",c,p,0)));t.b(": ");t.b(t.v(t.f("appVersion",c,p,0)));t.b(" ");t.b(t.v(t.d("Strings.ABOUT.FROM",c,p,0)));t.b(" ");t.b(t.v(t.f("appDate",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                <div><span class=\"wmicon icon-cloud-1\" /> ");t.b(t.v(t.d("Strings.ABOUT.SERVER_VERSION",c,p,0)));t.b(": ");t.b(t.v(t.f("serverVersion",c,p,0)));t.b(" ");t.b(t.v(t.d("Strings.ABOUT.FROM",c,p,0)));t.b(" ");t.b(t.v(t.f("serverDate",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("          </div>\r");t.b("\n" + i);t.b("          <div class=\"panel-title font-h4 padding-left-15 padding-bottom-10\">");t.b(t.v(t.d("Strings.ABOUT.WHATS_NEW",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("          <div class=\"panel-body\" id=\"v200\"><div>");t.b(t.v(t.d("Strings.ABOUT.V200_VERSION",c,p,0)));t.b("</div><div class=\"font-light\">");t.b(t.t(t.d("Strings.ABOUT.V200_BODY",c,p,0)));t.b("</div></div>\r");t.b("\n" + i);t.b("          <div class=\"panel-body\" id=\"v190\"><div>");t.b(t.v(t.d("Strings.ABOUT.V190_VERSION",c,p,0)));t.b("</div><div class=\"font-light\">");t.b(t.t(t.d("Strings.ABOUT.V190_BODY",c,p,0)));t.b("</div></div>\r");t.b("\n" + i);t.b("          <div class=\"panel-body\" id=\"v180\"><div>");t.b(t.v(t.d("Strings.ABOUT.V180_VERSION",c,p,0)));t.b("</div><div class=\"font-light\">");t.b(t.t(t.d("Strings.ABOUT.V180_BODY",c,p,0)));t.b("</div></div>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/*global define*/
define('views/about',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/about',
	'models/settings',
	'util/context',
	'util/message',
	"i18n!nls/strings",
	'bootstrap'
], function ($, _, Backbone, BaseView, aboutTemplate,SettingsModel,Context,MessageHelper,Strings) {
	

	var View = BaseView.extend({
		titel : Strings.ABOUT.ABOUT,
		el: '#wmapp',

		template: aboutTemplate,
        model:  SettingsModel,
		events: {
			'click #back':		'back'
		},


		initialize: function () {
            this.$el.html(this.template($.extend({
            	Strings:Strings,
            	Language:navigator.language
            },SettingsModel)));
		},
		
		back:function(){
			window.history.back();
		}
		
	});

	return View;
});

    function system_browser(url) {
    	var ref ;
        if(isAndroid() || isiOS()){
            ref= window.open(url, '_system', 'location=yes');
        }else {
            ref = window.open(url, '_blank', 'location=yes');
        }
    }

    function isAndroid(){
        return navigator.userAgent.indexOf("Android") > 0;
    }

    function isiOS(){
        return ( navigator.userAgent.indexOf("iPhone") > 0 || navigator.userAgent.indexOf("iPad") > 0 || navigator.userAgent.indexOf("iPod") > 0);
    };

define("hgn!templates/bonustips", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class=\"sub-container\">\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("    \r");t.b("\n" + i);t.b("<div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);t.b("    <div class=\"row padding-bottom-20\">\r");t.b("\n" + i);t.b("        <div class=\"col-xs-12\">\r");t.b("\n" + i);t.b("            <div class=\"content-center btn-default btn-lg font-16 font-light\">");t.b(t.v(t.d("Strings.BONUSTIPS.PLACE_TILL_SEMI_FINAL",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("    <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("      <div class=\"panel-heading\"><h3 class=\"panel-title clearfix\">");t.b(t.v(t.d("Strings.BONUSTIPS.WORLD_CHAMPION",c,p,0)));t.b("<span class=\"wmicon icon-award color-gold pull-right\" /></h3></div>\r");t.b("\n" + i);t.b("      <div class=\"panel-body winner\">\r");t.b("\n" + i);if(t.s(t.f("canSetTips",c,p,1),c,p,0,612,687,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <select class=\"form-control\" data-tip-key=\"winner\">\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("canSetTips",c,p,1),c,p,1,0,0,"")){t.b("          <select class=\"form-control\" data-tip-key=\"winner\" disabled>\r");t.b("\n" + i);};t.b("            <option value=\"\"></option>\r");t.b("\n" + i);if(t.s(t.f("playingCountries",c,p,1),c,p,0,898,974,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <option value=\"");t.b(t.v(t.f("ccode",c,p,0)));t.b("\">");t.b(t.v(t.f("countryName",c,p,0)));t.b("</option>\r");t.b("\n" + i);});c.pop();}t.b("        </select>\r");t.b("\n" + i);if(t.s(t.f("winner",c,p,1),c,p,0,1035,1258,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("        <div class=\"content-font-small tip-time\">");t.b(t.v(t.d("Strings.BONUSTIPS.TIP_TIME",c,p,0)));t.b(": ");t.b(t.v(t.f("time",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("        <div class=\"content-font-small tip-points\">");t.b(t.v(t.d("Strings.BONUSTIPS.POSSIBLE_POINTS",c,p,0)));t.b(": ");t.b(t.v(t.f("possiblePoints",c,p,0)));t.b("</div>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("winner",c,p,1),c,p,1,0,0,"")){t.b("        <div class=\"content-font-small tip-time\">");t.b(t.v(t.d("Strings.BONUSTIPS.NO_TIP",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("        <div class=\"content-font-small tip-points\"></div>\r");t.b("\n" + i);};if(t.s(t.f("haveWinnerResults",c,p,1),c,p,0,1487,1787,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"padding-top-10 font-h4\"><span class=\"font-light\">");t.b(t.v(t.d("Strings.BONUSTIPS.RESULT",c,p,0)));t.b("</span><span class=\"badge badge-bg-primary font-h4 pull-right points\">");t.b(t.v(t.f("winnerPoints",c,p,0)));t.b("</span> <span class=\"font-color-secondary font-medium actualValue\">");t.b(t.v(t.f("winnerActualValue",c,p,0)));t.b("</span></div>\r");t.b("\n" + i);});c.pop();}t.b("      </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    \r");t.b("\n" + i);t.b("     <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("      <div class=\"panel-heading\"><h3 class=\"panel-title clearfix\">");t.b(t.v(t.d("Strings.BONUSTIPS.VICE_CHAMPION",c,p,0)));t.b("<span class=\"wmicon icon-award color-silver pull-right\" /></h3></div>\r");t.b("\n" + i);t.b("      <div class=\"panel-body second\">\r");t.b("\n" + i);if(t.s(t.f("canSetTips",c,p,1),c,p,0,2117,2192,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <select class=\"form-control\" data-tip-key=\"second\">\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("canSetTips",c,p,1),c,p,1,0,0,"")){t.b("          <select class=\"form-control\" data-tip-key=\"second\" disabled>\r");t.b("\n" + i);};t.b("          <option value=\"\"></option>\r");t.b("\n" + i);if(t.s(t.f("playingCountries",c,p,1),c,p,0,2401,2477,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <option value=\"");t.b(t.v(t.f("ccode",c,p,0)));t.b("\">");t.b(t.v(t.f("countryName",c,p,0)));t.b("</option>\r");t.b("\n" + i);});c.pop();}t.b("        </select>\r");t.b("\n" + i);if(t.s(t.f("second",c,p,1),c,p,0,2538,2761,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("        <div class=\"content-font-small tip-time\">");t.b(t.v(t.d("Strings.BONUSTIPS.TIP_TIME",c,p,0)));t.b(": ");t.b(t.v(t.f("time",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("        <div class=\"content-font-small tip-points\">");t.b(t.v(t.d("Strings.BONUSTIPS.POSSIBLE_POINTS",c,p,0)));t.b(": ");t.b(t.v(t.f("possiblePoints",c,p,0)));t.b("</div>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("second",c,p,1),c,p,1,0,0,"")){t.b("        <div class=\"content-font-small tip-time\">");t.b(t.v(t.d("Strings.BONUSTIPS.NO_TIP",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("        <div class=\"content-font-small tip-points\"></div>\r");t.b("\n" + i);};if(t.s(t.f("haveSecondResults",c,p,1),c,p,0,2990,3290,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"padding-top-10 font-h4\"><span class=\"font-light\">");t.b(t.v(t.d("Strings.BONUSTIPS.RESULT",c,p,0)));t.b("</span><span class=\"badge badge-bg-primary font-h4 pull-right points\">");t.b(t.v(t.f("secondPoints",c,p,0)));t.b("</span> <span class=\"font-color-secondary font-medium actualValue\">");t.b(t.v(t.f("secondActualValue",c,p,0)));t.b("</span></div>\r");t.b("\n" + i);});c.pop();}t.b("      </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("    <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("      <div class=\"panel-heading\"><h3 class=\"panel-title clearfix\">");t.b(t.v(t.d("Strings.BONUSTIPS.GOLDEN_BALL",c,p,0)));t.b("<span class=\"wmicon icon-soccer color-gold pull-right\" /></h3></div>\r");t.b("\n" + i);t.b("      <div class=\"panel-body gldball\">\r");t.b("\n" + i);if(t.s(t.f("canSetTips",c,p,1),c,p,0,3613,3687,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("          <select class=\"form-control\" data-tip-key=\"gldball\">\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("canSetTips",c,p,1),c,p,1,0,0,"")){t.b("          <select class=\"form-control\" data-tip-key=\"gldball\" disabled>\r");t.b("\n" + i);};t.b("          <option value=\"\"></option>\r");t.b("\n" + i);if(t.s(t.f("allplayers",c,p,1),c,p,0,3889,3974,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("           <option value=\"");t.b(t.v(t.f("id",c,p,0)));t.b("\">");t.b(t.v(t.f("playerCountry",c,p,0)));t.b(" - ");t.b(t.v(t.f("name",c,p,0)));t.b("</option>\r");t.b("\n" + i);});c.pop();}t.b("        </select>\r");t.b("\n" + i);if(t.s(t.f("gldball",c,p,1),c,p,0,4030,4506,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("        <div class=\"content-font-small tip-time\">");t.b(t.v(t.d("Strings.BONUSTIPS.TIP_TIME",c,p,0)));t.b(": ");t.b(t.v(t.f("time",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("        <div class=\"content-font-small tip-points\">");t.b(t.v(t.d("Strings.BONUSTIPS.POSSIBLE_POINTS",c,p,0)));t.b(": ");t.b(t.v(t.f("possiblePoints",c,p,0)));t.b("</div>\r");t.b("\n" + i);if(!t.s(t.f("isSet",c,p,1),c,p,1,0,0,"")){t.b("                        <span class=\"glyphicon glyphicon-warning-sign\"></span>\r");t.b("\n" + i);t.b("                        <span class='bonustip-value-removed'>");t.b(t.v(t.d("Strings.COMMON.NOT_IN_TEAM",c,p,0)));t.b("</span>\r");t.b("\n" + i);};});c.pop();}if(!t.s(t.f("gldball",c,p,1),c,p,1,0,0,"")){t.b("        <div class=\"content-font-small tip-time\">");t.b(t.v(t.d("Strings.BONUSTIPS.NO_TIP",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("        <div class=\"content-font-small tip-points\"></div>\r");t.b("\n" + i);};if(t.s(t.f("haveGldballResults",c,p,1),c,p,0,4739,5041,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"padding-top-10 font-h4\"><span class=\"font-light\">");t.b(t.v(t.d("Strings.BONUSTIPS.RESULT",c,p,0)));t.b("</span><span class=\"badge badge-bg-primary font-h4 pull-right points\">");t.b(t.v(t.f("gldballPoints",c,p,0)));t.b("</span> <span class=\"font-color-secondary font-medium actualValue\">");t.b(t.v(t.f("gldballActualValue",c,p,0)));t.b("</span></div>\r");t.b("\n" + i);});c.pop();}t.b("      </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("    <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("      <div class=\"panel-heading\"><h3 class=\"panel-title clearfix\">");t.b(t.v(t.d("Strings.BONUSTIPS.GOLDEN_GLOVE",c,p,0)));t.b("<span class=\"wmicon icon-glove color-gold pull-right\" /></h3></div>\r");t.b("\n" + i);t.b("      <div class=\"panel-body gldglove\">\r");t.b("\n" + i);if(t.s(t.f("canSetTips",c,p,1),c,p,0,5366,5441,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("          <select class=\"form-control\" data-tip-key=\"gldglove\">\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("canSetTips",c,p,1),c,p,1,0,0,"")){t.b("          <select class=\"form-control\" data-tip-key=\"gldglove\" disabled>\r");t.b("\n" + i);};t.b("          <option value=\"\"></option>\r");t.b("\n" + i);if(t.s(t.f("keepers",c,p,1),c,p,0,5641,5726,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("           <option value=\"");t.b(t.v(t.f("id",c,p,0)));t.b("\">");t.b(t.v(t.f("playerCountry",c,p,0)));t.b(" - ");t.b(t.v(t.f("name",c,p,0)));t.b("</option>\r");t.b("\n" + i);});c.pop();}t.b("        </select>\r");t.b("\n" + i);if(t.s(t.f("gldglove",c,p,1),c,p,0,5780,6240,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("        <div class=\"content-font-small tip-time\">");t.b(t.v(t.d("Strings.BONUSTIPS.TIP_TIME",c,p,0)));t.b(": ");t.b(t.v(t.f("time",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("        <div class=\"content-font-small tip-points\">");t.b(t.v(t.d("Strings.BONUSTIPS.POSSIBLE_POINTS",c,p,0)));t.b(": ");t.b(t.v(t.f("possiblePoints",c,p,0)));t.b("</div>\r");t.b("\n" + i);if(!t.s(t.f("isSet",c,p,1),c,p,1,0,0,"")){t.b("                        <span class=\"glyphicon glyphicon-warning-sign\"></span>\r");t.b("\n" + i);t.b("                        <span class='bonustip-value-removed'>");t.b(t.v(t.d("Strings.COMMON.NOT_IN_TEAM",c,p,0)));t.b("</span>\r");t.b("\n" + i);};});c.pop();}if(!t.s(t.f("gldglove",c,p,1),c,p,1,0,0,"")){t.b("        <div class=\"content-font-small tip-time\">");t.b(t.v(t.d("Strings.BONUSTIPS.NO_TIP",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("        <div class=\"content-font-small tip-points\"></div>\r");t.b("\n" + i);};if(t.s(t.f("haveGldgloveResults",c,p,1),c,p,0,6477,6781,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"padding-top-10 font-h4\"><span class=\"font-light\">");t.b(t.v(t.d("Strings.BONUSTIPS.RESULT",c,p,0)));t.b("</span><span class=\"badge badge-bg-primary font-h4 pull-right points\">");t.b(t.v(t.f("gldglovePoints",c,p,0)));t.b("</span> <span class=\"font-color-secondary font-medium actualValue\">");t.b(t.v(t.f("gldgloveActualValue",c,p,0)));t.b("</span></div>\r");t.b("\n" + i);});c.pop();}t.b("      </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("    <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("      <div class=\"panel-heading\"><h3 class=\"panel-title clearfix\">");t.b(t.v(t.d("Strings.BONUSTIPS.GOLDEN_BOOT",c,p,0)));t.b("<span class=\"wmicon icon-boot color-gold pull-right\" /></h3></div>\r");t.b("\n" + i);t.b("      <div class=\"panel-body gldboot\">\r");t.b("\n" + i);if(t.s(t.f("canSetTips",c,p,1),c,p,0,7104,7178,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("          <select class=\"form-control\" data-tip-key=\"gldboot\">\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("canSetTips",c,p,1),c,p,1,0,0,"")){t.b("          <select class=\"form-control\" data-tip-key=\"gldboot\" disabled>\r");t.b("\n" + i);};t.b("          <option value=\"\"></option>\r");t.b("\n" + i);if(t.s(t.f("strikers",c,p,1),c,p,0,7378,7463,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("           <option value=\"");t.b(t.v(t.f("id",c,p,0)));t.b("\">");t.b(t.v(t.f("playerCountry",c,p,0)));t.b(" - ");t.b(t.v(t.f("name",c,p,0)));t.b("</option>\r");t.b("\n" + i);});c.pop();}t.b("        </select>\r");t.b("\n" + i);if(t.s(t.f("gldboot",c,p,1),c,p,0,7517,7977,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("        <div class=\"content-font-small tip-time\">");t.b(t.v(t.d("Strings.BONUSTIPS.TIP_TIME",c,p,0)));t.b(": ");t.b(t.v(t.f("time",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("        <div class=\"content-font-small tip-points\">");t.b(t.v(t.d("Strings.BONUSTIPS.POSSIBLE_POINTS",c,p,0)));t.b(": ");t.b(t.v(t.f("possiblePoints",c,p,0)));t.b("</div>\r");t.b("\n" + i);if(!t.s(t.f("isSet",c,p,1),c,p,1,0,0,"")){t.b("                        <span class=\"glyphicon glyphicon-warning-sign\"></span>\r");t.b("\n" + i);t.b("                        <span class='bonustip-value-removed'>");t.b(t.v(t.d("Strings.COMMON.NOT_IN_TEAM",c,p,0)));t.b("</span>\r");t.b("\n" + i);};});c.pop();}if(!t.s(t.f("gldboot",c,p,1),c,p,1,0,0,"")){t.b("        <div class=\"content-font-small tip-time\">");t.b(t.v(t.d("Strings.BONUSTIPS.NO_TIP",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("        <div class=\"content-font-small tip-points\"></div>\r");t.b("\n" + i);};if(t.s(t.f("haveGldbootResults",c,p,1),c,p,0,8210,8512,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"padding-top-10 font-h4\"><span class=\"font-light\">");t.b(t.v(t.d("Strings.BONUSTIPS.RESULT",c,p,0)));t.b("</span><span class=\"badge badge-bg-primary font-h4 pull-right points\">");t.b(t.v(t.f("gldbootPoints",c,p,0)));t.b("</span> <span class=\"font-color-secondary font-medium actualValue\">");t.b(t.v(t.f("gldbootActualValue",c,p,0)));t.b("</span></div>\r");t.b("\n" + i);});c.pop();}t.b("      </div>\r");t.b("\n" + i);t.b("    </div>    \r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

define('util/playingCountries',[],function(){
	return [
		{
			"id": 1,
			"group_id": 1,
			"code": "BRA",
			"ccode":"BR"
		},
		{
			"id": 2,
			"group_id": 1,
			"code": "HRV",
			"ccode":"HR"
		},
		{
			"id": 3,
			"group_id": 1,
			"code": "MEX",
			"ccode":"MX"

		},
		{
			"id": 4,
			"group_id": 1,
			"code": "CMR",
			"ccode": "CM"
		},
		{
			"id": 5,
			"group_id": 2,
			"code": "ESP",
			"ccode": "ES"
		},
		{
			"id": 6,
			"group_id": 2,
			"code": "NLD",
			"ccode": "NL"
		},
		{
			"id": 7,
			"group_id": 2,
			"code": "CHL",
			"ccode": "CL"
		},
		{
			"id": 8,
			"group_id": 2,
			"code": "AUS",
			"ccode": "AU"
		},
		{
			"id": 9,
			"group_id": 3,
			"code": "COL",
			"ccode": "CO"
		},
		{
			"id": 10,
			"group_id": 3,
			"code": "GRC",
			"ccode": "GR"
		},
		{
			"id": 11,
			"group_id": 3,
			"code": "CIV",
			"ccode": "CI"
		},
		{
			"id": 12,
			"group_id": 3,
			"code": "JPN",
			"ccode": "JP"
		},
		{
			"id": 13,
			"group_id": 4,
			"code": "URY",
			"ccode": "UY"
		},
		{
			"id": 14,
			"group_id": 4,
			"code": "CRC",
			"ccode": "CR"
		},
		{
			"id": 15,
			"group_id": 4,
			"code": "ENG",
			"ccode": "EN"
		},
		{
			"id": 16,
			"group_id": 4,
			"code": "ITA",
			"ccode": "IT"
		},
		{
			"id": 17,
			"group_id": 5,
			"code": "CHE",
			"ccode": "CH"
		},
		{
			"id": 18,
			"group_id": 5,
			"code": "ECU",
			"ccode": "EC"
		},
		{
			"id": 19,
			"group_id": 5,
			"code": "FRA",
			"ccode": "FR"
		},
		{
			"id": 20,
			"group_id": 5,
			"code": "HND",
			"ccode": "HN"
		},
		{
			"id": 21,
			"group_id": 6,
			"code": "ARG",
			"ccode": "AR"
		},
		{
			"id": 22,
			"group_id": 6,
			"code": "BIH",
			"ccode": "BA"
		},
		{
			"id": 23,
			"group_id": 6,
			"code": "IRN",
			"ccode": "IR"
		},
		{
			"id": 24,
			"group_id": 6,
			"code": "NGA",
			"ccode": "NG"
		},
		{
			"id": 25,
			"group_id": 7,
			"code": "DEU",
			"ccode": "DE"
		},
		{
			"id": 26,
			"group_id": 7,
			"code": "PRT",
			"ccode": "PT"
		},
		{
			"id": 27,
			"group_id": 7,
			"code": "GHA",
			"ccode": "GH"
		},
		{
			"id": 28,
			"group_id": 7,
			"code": "USA",
			"ccode": "US"
		},
		{
			"id": 29,
			"group_id": 8,
			"code": "BEL",
			"ccode": "BE"
		},
		{
			"id": 30,
			"group_id": 8,
			"code": "DZA",
			"ccode": "DZ"
		},
		{
			"id": 31,
			"group_id": 8,
			"code": "RUS",
			"ccode": "RU"
		},
		{
			"id": 32,
			"group_id": 8,
			"code": "KOR",
			"ccode": "KR"
		}
	];
});
/*global define*/
define('views/bonustips',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/bonustips',
	'models/settings',
	'util/context',
	'util/message',
	'collections/bonustips',
	'models/bonustip',
	'collections/teamplayers',
	'util/playingCountries',
	'moment',
	'util/wait',
	"i18n!nls/country",
	"i18n!nls/strings",
	'bootstrap'
], function ($, _, Backbone, BaseView, bonustipsTemplate,Settings,
	Context,MessageHelper,BonusTipsCollection,BonusTipModel,TeamPlayers,
	playingCountries,moment,Wait,CountryNames,Strings
	) {
	

	var View = BaseView.extend({
		titel : Strings.COMMON.BONUS_TIPS,
		el: '#wmapp',
		template: bonustipsTemplate,

		events: {
			'click #back':		'back',
			'change select':    'bonusTipChanged',
			'change input':     'bonusTipChanged',
		},
		initialize: function () {
			this.bonusTips=new BonusTipsCollection();
		},
		sort:function(){
			if (!TeamPlayers.sorted){
				TeamPlayers.models=TeamPlayers.sortBy(function(tm){
					return tm.get('team')+tm.get('name');
				})
				TeamPlayers.sorted=true;
			}
		},
		loadData:function(reload){
			var action=reload?"reload":"fetch";
			if (TeamPlayers.length){
				this.sort();
				return	this.bonusTips[action]({ prefill: true});
			}else{
				var def=new $.Deferred();
				TeamPlayers.fetch()
				.done(_.bind(function(){
						this.sort();
						this.bonusTips[action]({ prefill: true}).done(function(){
							def.resolve.apply(this,arguments);
						}).progress(function(){
							def.notify.apply(this,arguments);
						}).fail(function(){
							def.reject.apply(this,arguments);
						})
					},this))
				.fail(function(data,ajax,error,message){
						MessageHelper.handleError(ajax,Strings.BONUSTIPS.FAILED_TO_LOAD_TEAM_PLAYERS,this);
						Wait.stop();
						def.reject.apply(this,arguments);
					});
	
				return def.promise();
			}
			
		},
		refresh:function(){
			Wait.start();
			return this.loadData(true)
				.progress(_.bind(this.start,this))
				.done(_.bind(function(){
					this.start();
					Wait.stop();
				},this))
				.fail(function(data,ajax,error,message){
					MessageHelper.handleError(ajax,Strings.BONUSTIPS.FAILED_TO_REFRESH,this);
					Wait.stop();
				});

		},
		start:function(){
			if (!TeamPlayers.length){
				return;
			}
			var playerList=TeamPlayers.toJSON(),
				self=this;
			this.bonusTipConverted={};
			this.bonusTips.each(function(bonusTip){
				this.bonusTipConverted[bonusTip.get('tipKey')]=bonusTip.toJSON();
			},this)
			
            var firstSemifinalDate=moment(new Date("2014-07-08T20:00:00Z"));
			$.extend(this.bonusTipConverted,{
				time:function(){
						return moment(new Date(this.lastUpdated)).format('lll');
				},
				keepers:$.grep(playerList,function(e){
					return e.position==='Keeper' && (e.status ==="set"||(self.bonusTipConverted.gldglove && e.id===self.bonusTipConverted.gldglove.tipValue));
				}),
				strikers:$.grep(playerList,function(e){
					return (e.position==='Striker'|| e.position==='Midfielder' ) && (e.status ==="set"||(self.bonusTipConverted.gldboot && e.id===self.bonusTipConverted.gldboot.tipValue));
				}),
				allplayers:$.grep(playerList,function(e){
					return e.status ==="set"||(self.bonusTipConverted.gldball && e.id===self.bonusTipConverted.gldball.tipValue);
				}),
				playingCountries:_.sortBy(playingCountries, function(country){return CountryNames[country.ccode]}),
				countryName:function(){
					return CountryNames[this.ccode];
				},
				playerCountry:function(){
					return this.team;
				},
				isSet:function(){
					var tip=TeamPlayers.findWhere({id:this.tipValue});
					return tip&& tip.get("status") ==="set"
				},
                canSetTips:function(){
                    return moment(Context.getCurrentDate()).isBefore(firstSemifinalDate);
                },
                // WINNER
                haveWinnerResults:function(){
                    if (self.bonusTipConverted.winner) {
                        if (self.bonusTipConverted.winner.actualValue) {
                            return true;
                        } else { return false; }
                    } else { return false; }
                },
                winnerPoints:function(){
                    if (self.bonusTipConverted.winner) {
                        if (self.bonusTipConverted.winner.actualValue) {
                            return self.bonusTipConverted.winner.points;
                        } else { return 0; }
                    } else { return 0; }
				},
                winnerActualValue:function(){
                    if (self.bonusTipConverted.winner) {
                        if (self.bonusTipConverted.winner.actualValue) {
                            return CountryNames[self.bonusTipConverted.winner.actualValue];
                        } else { return false; }
                    } else { return false; }
				},
                // SECOND
                haveSecondResults:function(){
                    if (self.bonusTipConverted.second) {
                        if (self.bonusTipConverted.second.actualValue) {
                            return true;
                        } else { return false; }
                    } else { return false; }
                },
                secondPoints:function(){
                    if (self.bonusTipConverted.second) {
                        if (self.bonusTipConverted.second.actualValue) {
                            return self.bonusTipConverted.second.points;
                        } else { return 0; }
                    } else { return 0; }
				},
                secondActualValue:function(){
                    if (self.bonusTipConverted.second) {
                        if (self.bonusTipConverted.second.actualValue) {
                            return CountryNames[self.bonusTipConverted.second.actualValue];
                        } else { return false; }
                    } else { return false; }
				},
                // GOLDBALL
                haveGldballResults:function(){
                    if (self.bonusTipConverted.gldball) {
                        if (self.bonusTipConverted.gldball.actualValue) {
                            return true;
                        } else { return false; }
                    } else { return false; }
                },
                gldballPoints:function(){
                    if (self.bonusTipConverted.gldball) {
                        if (self.bonusTipConverted.gldball.actualValue) {
                            return self.bonusTipConverted.gldball.points;
                        } else { return 0; }
                    } else { return 0; }
				},
                gldballActualValue:function(){
                    if (self.bonusTipConverted.gldball) {
                        if (self.bonusTipConverted.gldball.actualValue) {
                            var name= self.bonusTipConverted.gldball && TeamPlayers.findWhere({id:self.bonusTipConverted.gldball.actualValue});
							return name && name.get("name")
                        } else { return false; }
                    } else { return false; }
				},
                // GOLDENGLOVE
                //{{playerCountry}} - {{name}}
                haveGldgloveResults:function(){
                    if (self.bonusTipConverted.gldglove) {
                        if (self.bonusTipConverted.gldglove.actualValue) {
                            return true;
                        } else { return false; }
                    } else { return false; }
                },
                gldglovePoints:function(){
                    if (self.bonusTipConverted.gldglove) {
                        if (self.bonusTipConverted.gldglove.actualValue) {
                            return self.bonusTipConverted.gldglove.points;
                        } else { return 0; }
                    } else { return 0; }
				},
                gldgloveActualValue:function(){
                    if (self.bonusTipConverted.gldglove) {
                        if (self.bonusTipConverted.gldglove.actualValue) {
                        	var name= self.bonusTipConverted.gldglove && TeamPlayers.findWhere({id:self.bonusTipConverted.gldglove.actualValue});
							return name && name.get("name")
                        } else { return false; }
                    } else { return false; }
				},
                // GOLDENBOOT
                haveGldbootResults:function(){
                    if (self.bonusTipConverted.gldboot) {
                        if (self.bonusTipConverted.gldboot.actualValue) {
                            return true;
                        } else { return false; }
                    } else { return false; }
                },
                gldbootPoints:function(){
                    if (self.bonusTipConverted.gldboot) {
                        if (self.bonusTipConverted.gldboot.actualValue) {
                            return self.bonusTipConverted.gldboot.points;
                        } else { return 0; }
                    } else { return 0; }
				},
                gldbootActualValue:function(){
                    if (self.bonusTipConverted.gldboot) {
                        if (self.bonusTipConverted.gldboot.actualValue) {
                            var name= self.bonusTipConverted.gldboot && TeamPlayers.findWhere({id:self.bonusTipConverted.gldboot.actualValue});
							return name && name.get("name");
                        } else { return false; }
                    } else { return false; }
				}
			});
	
			this.renderScreen()
		},
		renderScreen:function(){
			console.log("Rendering")
			this.$el.html(this.template($.extend({Strings:Strings},this.bonusTipConverted)));
			this.bindTips();
		},
		bindTips:function(){
			if (this.bonusTipConverted.winner){
				$('select[data-tip-key="winner"] option[value="'+this.bonusTipConverted.winner.tipValue+'"]').prop('selected', true);
			}
			if (this.bonusTipConverted.second){
				$('select[data-tip-key="second"] option[value="'+this.bonusTipConverted.second.tipValue+'"]').prop('selected', true);
			}
			if (this.bonusTipConverted.gldball){
				$('select[data-tip-key="gldball"] option[value="'+this.bonusTipConverted.gldball.tipValue+'"]').prop('selected', true);
			}
			if (this.bonusTipConverted.gldglove){
				$('select[data-tip-key="gldglove"] option[value="'+this.bonusTipConverted.gldglove.tipValue+'"]').prop('selected', true);
			}
			if (this.bonusTipConverted.gldboot){
				$('select[data-tip-key="gldboot"] option[value="'+this.bonusTipConverted.gldboot.tipValue+'"]').prop('selected', true);
			}
		},
		back:function(){
			window.history.back();
		},
		bonusTipChanged:function(event){
			var $el=$(event.target),
				selectedValue=$el.val(),
				tipKey=$el.data('tip-key'),
				bonusTip=new BonusTipModel(this.bonusTipConverted[tipKey]||{
					tipKey:tipKey
				}),
				oldValue=bonusTip.get('tipValue');
				if (oldValue){
					MessageHelper.showConfirmationMessage({header:Strings.BONUSTIPS.CHANGE_BONUSTIP_WARNING_HEADER,
						message:Strings.BONUSTIPS.CHANGE_BONUSTIP_WARNING_MESSAGE})
					.done(_.bind(function(){
						Wait.start();
						this.$("select").prop('disabled', true);
						bonusTip.set('tipValue',selectedValue);
						bonusTip.save()
							.done(_.bind(_.partial(this.bonusTipSaved,bonusTip),this))
							.fail(_.partial(this.bonusTipSaveFailed,oldValue,tipKey))
					},this))
					.fail(function(){
						$('select[data-tip-key="'+tipKey+'"] option[value="'+oldValue+'"]').prop('selected', true);
					});
				}else{
					Wait.start();
					$el.prop('disabled', true);
					bonusTip.set('tipValue',selectedValue);
					bonusTip.save()
						.done(_.bind(_.partial(this.bonusTipSaved,bonusTip),this))
						.fail(_.partial(this.bonusTipSaveFailed,oldValue,tipKey))
				}
		},
		bonusTipSaveFailed:function(oldValue,tipKey,ajax,error,message){
			MessageHelper.handleError(ajax,Strings.ERROR_CODE["BONUSTIPS.ERR.SAVE_FAILED"],this);
			if (oldValue){
				$('select[data-tip-key="'+tipKey+'"] option[value="'+oldValue+'"]').prop('selected', true);
			}else{
				$('select[data-tip-key="'+tipKey+'"] option[value=""]').prop('selected', true);
			}
			$("select").prop('disabled', false);
			Wait.stop();
		},
		bonusTipSaved:function(bonusTip,model){
			var tipKey=bonusTip.get('tipKey');
			this.bonusTipConverted[tipKey]=model;
			this.$('select[data-tip-key="'+tipKey+'"] option[value="'+model.tipValue+'"]').prop('selected', true);
			this.$('.'+tipKey+ ' .tip-time').text(Strings.BONUSTIPS.TIP_TIME+': '+moment(new Date(model.lastUpdated)).format('lll'));
			this.$('.'+tipKey+ ' .tip-points').text(Strings.BONUSTIPS.POSSIBLE_POINTS+': '+model.possiblePoints);
			this.refresh().always(_.bind(function(){
				Wait.stop();
				this.$("select").prop('disabled', false);
			},this));
			
			
		}
		
	});

	return View;
});

define("hgn!templates/groups", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class='sub-header-nav'>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);t.b("    <fieldset>\r");t.b("\n" + i);t.b("        <div class=\"input-group\">\r");t.b("\n" + i);t.b("          <input id=\"search-group-text\" type=\"text\" class=\"form-control search-text\" placeholder=\"");t.b(t.v(t.d("Strings.COMMON.SEARCH_GROUPS",c,p,0)));t.b("\" value=\"");t.b(t.v(t.f("searchText",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("          <span class=\"input-group-btn\">\r");t.b("\n" + i);t.b("            <button id=\"search-group\" class=\"btn btn-default btn-search\" type=\"button\" value=\"");t.b(t.v(t.d("Strings.COMMON.SEARCH",c,p,0)));t.b("\"><span class=\"glyphicon glyphicon-search\"></span></button>\r");t.b("\n" + i);t.b("          </span>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("        <div id=\"group-messages\" class=\"font-h4 font-light font-color-danger\"></div>\r");t.b("\n" + i);t.b("    </fieldset>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("<div class='sub-container have-footer'>\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);t.b("    <div class=\"form-group\">\r");t.b("\n" + i);t.b("      <h2>");t.b(t.v(t.d("Strings.COMMON.MY_GROUPS",c,p,0)));t.b("</h2>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"form-group\">\r");t.b("\n" + i);t.b("      <div class=\"panel panel-alpha\">\r");t.b("\n" + i);t.b("        <div class=\"list-group\">\r");t.b("\n" + i);if(t.s(t.f("groups",c,p,1),c,p,0,1065,1666,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <a href=\"#group/");t.b(t.v(t.f("groupId",c,p,0)));t.b("\" data-group-id='");t.b(t.v(t.f("groupId",c,p,0)));t.b("' class=\"list-group-item min-height-lists\">\r");t.b("\n" + i);t.b("              <div class=\"row\">\r");t.b("\n" + i);t.b("                <div class=\"col-xs-8\">");t.b(t.v(t.f("groupName",c,p,0)));if(t.s(t.f("isPrivate",c,p,1),c,p,0,1277,1371,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("\r");t.b("\n" + i);t.b("        <span class=\"wmicon icon-lock  color-grey-profile padding-left-10\"></span>\r");t.b("\n" + i);t.b("        ");});c.pop();}t.b("</div>\r");t.b("\n" + i);t.b("                <div class=\"col-xs-2 player-position\">");t.b(t.v(t.f("groupRank",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                <div class=\"col-xs-1 player-owner\" ");if(!t.s(t.f("isOwner",c,p,1),c,p,1,0,0,"")){t.b("style=\"display:none\"");};t.b("><span class=\"glyphicon glyphicon-flag\"></span></div>\r");t.b("\n" + i);t.b("              </div>\r");t.b("\n" + i);t.b("            </a>\r");t.b("\n" + i);});c.pop();}t.b("        </div>\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("<div class=\"sub-footer\">\r");t.b("\n" + i);t.b("    <div class=\"btn-group btn-group-lg btn-group-justified\">\r");t.b("\n" + i);t.b("        <div class=\"btn-group\">\r");t.b("\n" + i);t.b("            <button type=\"button\" id=\"create-group\" class=\"btn footer-btn active\" value=\"");t.b(t.v(t.d("Strings.GROUPS.CREATE_GROUP",c,p,0)));t.b("\"><span class=\"font-h4 glyphicon glyphicon-plus-sign\"></span><br/><span class=\"content-font-small\">");t.b(t.v(t.d("Strings.GROUPS.CREATE_GROUP_INVITE",c,p,0)));t.b("</span></button>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/*global define*/
define('collections/groups',['jquery',
	'underscore',
	'backbone',
	'models/group',
	'models/settings',
	'util/context',
], function ($,_, Backbone, GroupModel, Settings,Context) {
	

	var GroupModelCollection = Backbone.Collection.extend({
		// Reference to this collection's model.
		model: GroupModel,
		initialize : function(models, options){
	        this.query = options;
	    },
		url:function (){ 
			Context.groupsIterationToken=Context.groupsIterationToken||new Date().getTime();
			var param=$.extend({},this.query,{
				"_it":Context.groupsIterationToken
			})
			return  Settings.get("server") + "/rest/groups?"+$.param(param);
		},
		reload:function(){
			Context.groupsIterationToken=new Date().getTime();
		  	return this.fetch.apply(this,arguments);
		  },

	});

	return GroupModelCollection;
});

/*global define*/
define('views/groups',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/groups',
	'models/settings',
	'util/context',
	'util/message',
	'collections/groups',
	'util/wait',
	'models/group',
	"i18n!nls/strings",
	'bootstrap'
], function ($, _, Backbone, BaseView, groupsTemplate,SettingsModel,Context,MessageHelper,GroupCollection,
	Wait,GroupModel,Strings) {
	

	var View = BaseView.extend({

		titel : Strings.GROUPS.TITLE,
		el: '#wmapp',

		template: groupsTemplate,

		events: {
			'click #search-group': 'searchGroup',
			'click #create-group': 'createGroup',
			'keypress #search-group-text': 'keyboardSearch'
		},

		keyboardSearch:function(e){
			if (e.keyCode == 13 ){
				this.searchGroup();
			}
		},

		searchGroup:function(){
			MessageHelper.hide();
			this.searchText=$("#search-group-text").val();
			var	groupsSearch = new GroupCollection([], {
					find:this.searchText.trim(),
					from:0,
					size:10
				});
			Wait.start();
			groupsSearch.reload()
			.done(_.bind(this.searchCallback,this))
			.fail(function(data,ajax,error,message){
				MessageHelper.handleError(ajax,Strings.ERROR_CODE["GROUPS.ERR.FAILED_FETCH"],this);
			})
			.always(function(){Wait.stop();});
		},
		createGroup:function(){
			Context.Router.navigate("#newgroup", {trigger: true,replace: false});
		},
		refresh:function(){
			Wait.start();
			Context.player.reload()
			.done(_.bind(this.initialize,this))
			.fail(function(data,ajax,error,message){
					MessageHelper.handleError(ajax,Strings.PLAYER_DETAILS.FAILED_TO_REFERSH,this);
			})
			.always(function(){Wait.stop();});
		},
		initialize: function () {
			var playerJSON=Context.player.toJSON();
			this.$el.html(this.template($.extend({},playerJSON,{
				Strings:Strings,
				isAdmin:Context.player.isAdmin(),
				isOwner:function(){
					return Context.player.get('id') === this.ownerId;
				},
				groupRank:function(){
					return this.rank ===  2147483647 ?"-":this.rank
				},
				isPrivate:function(){
					return this.publicGroup === false;
				}
			})));
		},
		searchCallback:function(data){
			if (data && data.length>0){
				Context.groupsSearch={};
				Context.groupsSearch[this.searchText]=data;
				window.location.hash = '#groupsearchresults/'+encodeURIComponent(this.searchText);
			}else{
				this.$("#group-messages").text(Strings.GROUPS.NOTHING_FOUND);
			}
			
		}
	});

	return View;
});

define("hgn!templates/newgroup", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class='sub-container have-footer'>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);t.b("    <div class=\"form-group\"><h2>");t.b(t.v(t.d("Strings.NEWGROUP.CREATE_NEW",c,p,0)));t.b("</h2></div>\r");t.b("\n" + i);t.b("      <fieldset>\r");t.b("\n" + i);t.b("        <div>");t.b(t.v(t.d("Strings.NEWGROUP.ACTION_DESC",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("          <div class=\"form-group\">\r");t.b("\n" + i);t.b("            <!--<label class=\"control-label\">Group Name</label>-->\r");t.b("\n" + i);t.b("              ");t.b(t.v(t.d("Strings.NEWGROUP.GROUP_NAME",c,p,0)));t.b("\r");t.b("\n" + i);t.b("            <input id=\"group-name\" type=\"text\" class=\"form-control search-text\" placeholder=\"");t.b(t.v(t.d("Strings.NEWGROUP.GROUP_NAME",c,p,0)));t.b("\" maxlength=\"20\">\r");t.b("\n" + i);t.b("          </div>\r");t.b("\n" + i);t.b("          <div class=\"form-group\">\r");t.b("\n" + i);t.b("            <!--<label class=\"control-label\">Description</label>-->\r");t.b("\n" + i);t.b("              ");t.b(t.v(t.d("Strings.NEWGROUP.DESC",c,p,0)));t.b("\r");t.b("\n" + i);t.b("             <textarea id=\"group-description\" class=\"form-control search-text\" rows=\"4\" placeholder=\"");t.b(t.v(t.d("Strings.NEWGROUP.DESC_PLACEHOLDER",c,p,0)));t.b("\" maxlength=\"140\"></textarea>\r");t.b("\n" + i);t.b("          </div> \r");t.b("\n" + i);t.b("        <div class=\"form-group\">\r");t.b("\n" + i);t.b("           <div class=\"checkbox\">\r");t.b("\n" + i);t.b("               <input id=\"is-group-private\" type=\"checkbox\">\r");t.b("\n" + i);t.b("                <label for=\"is-group-private\">");t.b(t.v(t.d("Strings.NEWGROUP.MAKE_GROUP_PRIVATE",c,p,0)));t.b("</label>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("    </fieldset>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("<div class=\"sub-footer\">\r");t.b("\n" + i);t.b("    <div class=\"btn-group btn-group-lg btn-group-justified\">\r");t.b("\n" + i);t.b("        <div class=\"btn-group\">\r");t.b("\n" + i);t.b("            <button type=\"button\" id=\"cancel\" class=\"btn footer-btn active\" value=\"");t.b(t.v(t.d("Strings.COMMON.CANCEL",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("                <span class=\"glyphicon glyphicon-remove-sign\"></span><br/><span class=\"content-font-small\">");t.b(t.v(t.d("Strings.COMMON.CANCEL",c,p,0)));t.b("</span>\r");t.b("\n" + i);t.b("            </button>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("        <div class=\"btn-group\">\r");t.b("\n" + i);t.b("            <button type=\"button\" id=\"create-group\" class=\"btn footer-btn active\" \r");t.b("\n" + i);t.b("                    data-loading-text='");t.b(t.v(t.d("Strings.NEWGROUP.CREATING",c,p,0)));t.b("' value=\"");t.b(t.v(t.d("Strings.NEWGROUP.CREATE",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("                <span class=\"glyphicon glyphicon-ok-sign\"></span><br/><span class=\"content-font-small\">");t.b(t.v(t.d("Strings.NEWGROUP.CREATE",c,p,0)));t.b("</span>\r");t.b("\n" + i);t.b("            </button>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/*global define*/
define('views/newgroup',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/newgroup',
	'models/settings',
	'util/context',
	'util/message',
	'collections/groups',
	'util/wait',
	"i18n!nls/strings",
	'bootstrap'
], function ($, _, Backbone, BaseView, newGroupTemplate,SettingsModel,Context,MessageHelper,GroupCollection,Wait,Strings) {
	

	var View = BaseView.extend({

		titel : Strings.NEWGROUP.TITLE,
		el: '#wmapp',

		template: newGroupTemplate,

		events: {
			'click #create-group':'createGroup',
            'click #cancel':		'cancel'
		},

		'createGroup':function(event){
			Wait.start(event.target);
			$.ajax({
			  type: "POST",
			  url:SettingsModel.get("server") +"/rest/groups",
			  contentType: 'application/json',
			  data: {
							description:this.$("#group-description").val(),
							name:this.$("#group-name").val(),
							publicGroup:!this.$("#is-group-private").prop('checked')
							
						}
			}).done(function(data){
				//invalidate possibly cached ranking results
				Context.groupRankIterationToken=new Date().getTime();

				Context.player.reload().done(function(){
					Wait.stop();
					Context.Router.navigate('#group/'+data.id, {trigger: true, replace: true});
				}).fail(function(data,ajax,error,message){
					Wait.stop();
					Context.Router.navigate('#group/'+data.id, {trigger: true, replace: true});
					MessageHelper.handleError(ajax,Strings.NEWGROUP.FAILED_TO_REFRESH,this);
				})
				
			}).fail(function(ajax,error,message){
				Wait.stop();
				MessageHelper.handleError(ajax,Strings.ERROR_CODE["NEWGROUP.ERR.FAILED_TO_CREATE"],this);
			});
		},
		initialize: function () {
			this.$el.html(this.template($.extend({},Context.player.toJSON(),{
				Strings:Strings,
				isAdmin:Context.player.isAdmin()
			})));
		},
        cancel:function(){
		  Context.Router.navigate('#groups', {trigger: true, replace: true});
        }
	});

	return View;
});

define("hgn!templates/managegroup", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class=\"btn-group btn-group-lg btn-group-justified sub-header-nav\">\r");t.b("\n" + i);t.b("  <div class=\"btn-group\">\r");t.b("\n" + i);t.b("    <button type=\"button\" id=\"manage-group-home\" data-tab-id=\"home\" class=\"btn head-btn active\">");t.b(t.v(t.d("Strings.GROUP.GROUP",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("  <div class=\"btn-group\">\r");t.b("\n" + i);t.b("    <button type=\"button\" id=\"manage-group-requests\" data-tab-id=\"requests\" class=\"btn head-btn\">");t.b(t.v(t.d("Strings.GROUP.JOIN_REQUESTS",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("  <div class=\"btn-group\">\r");t.b("\n" + i);t.b("    <button type=\"button\" id=\"manage-group-members\" data-tab-id=\"members\" class=\"btn head-btn\">");t.b(t.v(t.d("Strings.GROUP.MEMBERS",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("  <div class=\"btn-group\">\r");t.b("\n" + i);t.b("    <button type=\"button\" id=\"manage-group-add\" data-tab-id=\"add\" class=\"btn head-btn\">");t.b(t.v(t.d("Strings.GROUP.ADD_USERS",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("<div id='tab-container'>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("\r");t.b("\n");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});


define("hgn!templates/managegrouprequests", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class='sub-container'>\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);t.b("    <div class=\"list-group col-sm-12\">\r");t.b("\n" + i);if(t.s(t.f("requests",c,p,1),c,p,0,162,737,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <a href=\"#player/");t.b(t.v(t.d("player.id",c,p,0)));t.b("\" class=\"list-group-item min-height-lists\">\r");t.b("\n" + i);t.b("                <div class=\"\">");t.b(t.v(t.d("player.nickName",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("            </a>\r");t.b("\n" + i);t.b("            <button type=\"button\" data-player-id=\"");t.b(t.v(t.d("player.playerId",c,p,0)));t.b("\" class=\"approve-membership btn btn-success\"><span class=\"glyphicon glyphicon-ok-sign\"></span> ");t.b(t.v(t.d("Strings.GROUP.APPROVE",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("            <button type=\"button\" data-player-id=\"");t.b(t.v(t.d("player.playerId",c,p,0)));t.b("\" class=\"decline-membership btn btn-danger\"><span class=\"glyphicon glyphicon-remove-sign\"></span> ");t.b(t.v(t.d("Strings.GROUP.DECLINE",c,p,0)));t.b("</button>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("requests",c,p,1),c,p,1,0,0,"")){t.b("            <div>");t.b(t.v(t.d("Strings.GROUP.NO_REQUESTS",c,p,0)));t.b("</div>\r");t.b("\n" + i);};t.b("    </div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});


define("hgn!templates/managegroupadd", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class='sub-container ");if(t.s(t.f("hasMoreOrLess",c,p,1),c,p,0,44,55,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("have-footer");});c.pop();}t.b("'>\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);t.b("<div class=\"form-group\">\r");t.b("\n" + i);t.b("      <!--Search users-->\r");t.b("\n" + i);t.b("    <div class=\"input-group\">\r");t.b("\n" + i);t.b("        <input id=\"search-player-text\" type=\"text\" class=\"form-control search-text\" value=\"");t.b(t.v(t.f("searchText",c,p,0)));t.b("\" placeholder=\"");t.b(t.v(t.d("Strings.GROUP.SEARCH_PLAYERNICK",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("        <span class=\"input-group-btn\">\r");t.b("\n" + i);t.b("          <button id=\"search-player\" class=\"btn btn-default btn-search\" type=\"button\" value=\"Search\">\r");t.b("\n" + i);t.b("            <span class=\"glyphicon glyphicon-search hide-when-working\"></span>\r");t.b("\n" + i);t.b("            <div class=\"loading\">\r");t.b("\n" + i);t.b("                <div class=\"spinner\">\r");t.b("\n" + i);t.b("                    <div class=\"mask\">\r");t.b("\n" + i);t.b("                        <div class=\"maskedCircle\"></div>\r");t.b("\n" + i);t.b("                    </div>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("          </button>\r");t.b("\n" + i);t.b("        </span>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);if(t.s(t.f("somethingFound",c,p,1),c,p,0,938,2638,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("  <div class=\"form-group\">\r");t.b("\n" + i);t.b("      <div class=\"panel panel-alpha\">\r");t.b("\n" + i);t.b("          <div class=\"list-group\">\r");t.b("\n" + i);if(t.s(t.f("searchResults",c,p,1),c,p,0,1069,2416,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"list-group-item min-height-lists\">\r");t.b("\n" + i);t.b("              <!--<button type=\"button\" class=\"btn btn-primary badge-button btn-sm invite-player\" data-player-id=\"");t.b(t.v(t.f("id",c,p,0)));t.b("\"><span class=\"glyphicon glyphicon-share-alt\"></span> Invite</button>\r");t.b("\n" + i);t.b("              ");t.b(t.v(t.f("nickName",c,p,0)));t.b("-->\r");t.b("\n" + i);t.b("                  <div class=\"row\">\r");t.b("\n" + i);t.b("                      <div class=\"col-xs-9\"><div>");if(t.s(t.f("isMyself",c,p,1),c,p,0,1452,1491,"{{ }}")){t.rs(c,p,function(c,p,t){t.b(t.v(t.d("Strings.GROUP.ITS_ME",c,p,0)));t.b(" (");t.b(t.v(t.f("nickName",c,p,0)));t.b(")");});c.pop();}if(!t.s(t.f("isMyself",c,p,1),c,p,1,0,0,"")){t.b(t.v(t.f("nickName",c,p,0)));};t.b("</div><div>");t.b(t.v(t.f("email",c,p,0)));t.b("</div></div>\r");t.b("\n" + i);t.b("                      <div class=\"col-xs-3\">\r");t.b("\n" + i);if(!t.s(t.f("isMyself",c,p,1),c,p,1,0,0,"")){if(t.s(t.f("isAlreadyInvited",c,p,1),c,p,0,1706,1809,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                        <span class=\"glyphicon glyphicon-share-alt\"></span>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("isAlreadyInvited",c,p,1),c,p,1,0,0,"")){t.b("                        <button type=\"button\" class=\"btn btn-primary badge-button btn-sm invite-player\" data-player-id=\"");t.b(t.v(t.f("id",c,p,0)));t.b("\"><span class=\"glyphicon glyphicon-share-alt\"></span> ");t.b(t.v(t.d("Strings.GROUP.INVITE",c,p,0)));t.b("</button>\r");t.b("\n" + i);};};if(t.s(t.f("isMyself",c,p,1),c,p,0,2217,2317,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                          <span class=\"glyphicon glyphicon-user\"></span>\r");t.b("\n" + i);});c.pop();}t.b("                      </div>\r");t.b("\n" + i);t.b("                  </div>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);});c.pop();}if(t.s(t.f("hasNoMore",c,p,1),c,p,0,2460,2580,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("           <div class=\"list-group-item\">\r");t.b("\n" + i);t.b("            ");t.b(t.v(t.d("Strings.GROUP.NO_MORE_RESULTS",c,p,0)));t.b("\r");t.b("\n" + i);t.b("           </div>\r");t.b("\n" + i);});c.pop();}t.b("          </div>\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);});c.pop();}if(t.s(t.f("nothingFound",c,p,1),c,p,0,2676,2764,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("    <h4 class=\"font-light font-color-danger\">");t.b(t.v(t.d("Strings.GROUP.NO_PLAYERS_FOUND",c,p,0)));t.b("</h4>\r");t.b("\n" + i);});c.pop();}t.b("</div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);if(t.s(t.f("hasMoreOrLess",c,p,1),c,p,0,2860,3504,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("<div class=\"sub-footer\">\r");t.b("\n" + i);t.b("    <div class=\"btn-group btn-group-lg btn-group-justified\">\r");t.b("\n" + i);if(t.s(t.f("hasLess",c,p,1),c,p,0,2969,3201,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"btn-group\">\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"load-prev\" class=\"btn footer-btn active\"><span class=\"wmicon icon-left-open-big\"></span> ");t.b(t.v(t.d("Strings.COMMON.PREV",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);});c.pop();}t.b("\r");t.b("\n" + i);if(t.s(t.f("hasMore",c,p,1),c,p,0,3237,3470,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"btn-group\">\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"load-next\" class=\"btn footer-btn active\">");t.b(t.v(t.d("Strings.COMMON.NEXT",c,p,0)));t.b(" <span class=\"wmicon icon-right-open-big\"></span></button>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);});c.pop();}t.b("    </div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);});c.pop();}return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});


define("hgn!templates/managegroupmembers", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class='sub-container ");if(t.s(t.f("hasMoreOrLess",c,p,1),c,p,0,44,55,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("have-footer");});c.pop();}t.b("'>\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);t.b("    <div class=\"list-group col-sm-12\">\r");t.b("\n" + i);t.b("            <div class=\"panel panel-default\">\r");t.b("\n" + i);t.b("                <ul class=\"list-group\">\r");t.b("\n" + i);if(t.s(t.f("accepted",c,p,1),c,p,0,310,791,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                        <a href=\"#player/");t.b(t.v(t.f("playerId",c,p,0)));t.b("\" data-player-id='");t.b(t.v(t.f("playerId",c,p,0)));t.b("' class=\"list-group-item min-height-lists\">\r");t.b("\n" + i);t.b("                            <div class=\"\">");t.b(t.v(t.f("nickName",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                            <button type=\"button\" data-player-id=\"");t.b(t.v(t.d("player.playerId",c,p,0)));t.b("\" class=\"remove-member btn btn-danger\"><span class=\"glyphicon glyphicon-remove-sign\"></span> ");t.b(t.v(t.d("Strings.GROUP.REMOVE",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("                        </a>\r");t.b("\n" + i);t.b("                        \r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("accepted",c,p,1),c,p,1,0,0,"")){t.b("                        <div class=\"list-group-item min-height-lists\">");t.b(t.v(t.d("Strings.GROUP.NO_MEMBERS",c,p,0)));t.b("</div>\r");t.b("\n" + i);};t.b("                </ul>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);if(t.s(t.f("hasMoreOrLess",c,p,1),c,p,0,1110,1768,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("<div class=\"sub-footer\">\r");t.b("\n" + i);t.b("    <div class=\"btn-group btn-group-lg btn-group-justified\">\r");t.b("\n" + i);if(t.s(t.f("hasLess",c,p,1),c,p,0,1219,1458,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"btn-group\">\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"load-member-prev\" class=\"btn footer-btn active\"><span class=\"wmicon icon-left-open-big\"></span> ");t.b(t.v(t.d("Strings.COMMON.PREV",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);});c.pop();}t.b("\r");t.b("\n" + i);if(t.s(t.f("hasMore",c,p,1),c,p,0,1494,1734,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"btn-group\">\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"load-member-next\" class=\"btn footer-btn active\">");t.b(t.v(t.d("Strings.COMMON.NEXT",c,p,0)));t.b(" <span class=\"wmicon icon-right-open-big\"></span></button>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);});c.pop();}t.b("    </div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);});c.pop();}return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});


define("hgn!templates/managegrouphome", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class='sub-container have-footer'>\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);t.b("    <h3 class=\"font-light\">");t.b(t.v(t.f("name",c,p,0)));t.b("&nbsp;");if(!t.s(t.f("name",c,p,1),c,p,1,0,0,"")){t.b(t.v(t.d("Strings.GROUP.LOADING",c,p,0)));};t.b("</h3>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("    <div class=\"form-group\"> \r");t.b("\n" + i);t.b("      <textarea id=\"group-description\" class=\"form-control font-16\" rows=\"5\" placeholder=\"Enter Description\" >");t.b(t.v(t.f("description",c,p,0)));t.b("</textarea>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("    <div class=\"checkbox\">\r");t.b("\n" + i);t.b("        <input id='is-group-private' type=\"checkbox\" ");if(!t.s(t.f("publicGroup",c,p,1),c,p,1,0,0,"")){t.b("checked=\"checked\"");};t.b(">\r");t.b("\n" + i);t.b("        <label for='is-group-private'>");t.b(t.v(t.d("Strings.GROUP.PRIVATE_GROUP",c,p,0)));t.b("</label>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("<div class=\"form-group\"> \r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("<div class=\"sub-footer\">\r");t.b("\n" + i);t.b("    <div class=\"btn-group btn-group-lg btn-group-justified\">\r");t.b("\n" + i);t.b("        <div class=\"btn-group\">\r");t.b("\n" + i);t.b("            <button type=\"button\" id=\"save-group\" class=\"btn footer-btn active\">\r");t.b("\n" + i);t.b("                <span class=\"font-h4 glyphicon glyphicon-floppy-disk\"></span><br/><span class=\"content-font-small\">");t.b(t.v(t.d("Strings.GROUP.SAVE_CHANGES",c,p,0)));t.b("</span>\r");t.b("\n" + i);t.b("            </button>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/*global define*/
define('collections/players',['jquery',
	'underscore',
	'backbone',
	'models/player',
	'models/settings',
	'util/context',
], function ($,_, Backbone, PlayerModel, Settings,Context) {
	

	var PlayerModelCollection = Backbone.Collection.extend({
		// Reference to this collection's model.
		model: PlayerModel,
		initialize : function(models, options){
	        this.query = options;
	    },
		url:function (){ 
			Context.playersListIterationToken=Context.playersListIterationToken||new Date().getTime();
			var param=$.extend({},this.query,{
				"_it":Context.playersListIterationToken
			})
			return  Settings.get("server") + "/rest/player?"+$.param(param);
		},
		reload:function(options){
		  	Context.playersListIterationToken=new Date().getTime();
		  	return this.fetch.apply(this,arguments);
		  },

	});

	return PlayerModelCollection;
});
/*global define*/
define('views/managegroup',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/managegroup',
	'hgn!templates/managegrouprequests',
	'hgn!templates/managegroupadd',
	'hgn!templates/managegroupmembers',
	'hgn!templates/managegrouphome',
	'models/settings',
	'util/context',
	'util/message',
	'models/group',
	'models/player',
	'collections/players',
	'util/tabs',
	'util/wait',
	"i18n!nls/strings",
	'bootstrap'
], function ($, _, Backbone,  BaseView, managegroupTemplate,
	requestsTemplate, addTemplate, membersTemplate, homeTemplate,
	SettingsModel,
	Context,MessageHelper,GroupModel,PlayerModel,
	PlayerCollection,TabsHelper,Wait,Strings) {
	


	var View = BaseView.extend({

		titel : Strings.GROUPDETAILS.MANAGE_GROUP,
		subView: true,
		el: '#wmapp',

		template: managegroupTemplate,
		currentStart:0,
		pageSize:10,
		events: {
			'click #manage-group-home':'handleTabClick',
			'click #manage-group-requests':'handleTabClick',
			'click #manage-group-members' :'handleTabClick',
			'click #manage-group-add':'handleTabClick',
			'click #save-group'	:'saveGroup',
			'click #search-player':'searchPlayer',
			'click .invite-player':'invitePlayer',
			'click .approve-membership':'approiveMembership',
			'click .decline-membership':'declineMembership',
			'click #load-prev':'prevClick',
			'click #load-next':'nextClick',
			'click .remove-member':'removeMember',
			'click #load-member-next':'loadNext',
			'click #load-member-prev':'loadPrev'
		},
		getBaseUrl:function(){
			return "#managegroup/"+this.group.get('id');
		},
		handleTabClick:function(event){
			TabsHelper.HeaderClickHandler(this,event);
		},
		initialize: function (options) {
			this.$el.html(this.template({Strings:Strings}));
			this.tabContainer=this.$("#tab-container");
			this.group=new GroupModel({id:options.id});
			Wait.start();
			this.group.reload( {prefill: true})
				.progress(_.bind(function(){
					this.updateOptions({tab:options.tab,replace:true});
				}, this))
				.done(_.bind(function(){
					this.updateOptions({tab:options.tab,replace:true});
					Wait.stop();
				}, this))
				.fail(function(data,ajax,error,message){
					Wait.stop();
					MessageHelper.handleError(ajax,Strings.ERROR_CODE["MANAGE_GROUP.ERR.FAILED_TO_FETCH"],this);
				});
		},
		refresh:function(){
			Wait.start();
			this.group.reload( )
				.done(_.bind(function(){
					this.updateOptions({tab:this.activeTab,replace:true});
					Wait.stop();
				}, this))
				.fail(function(data,ajax,error,message){
					Wait.stop();
					MessageHelper.handleError(ajax,Strings.ERROR_CODE["MANAGE_GROUP.ERR.FAILED_TO_FETCH"],this);
				});
		},
		updateOptions:function(options,replace){
			if (!options||!options.tab||options.tab === 'home'){
				this.manageGroupHome({target:this.$("#manage-group-home"),replace:replace});
			}else if (!options||!options.tab||options.tab === 'requests'){
				this.manageGroupRequests({target:this.$("#manage-group-requests"),replace:replace});
			}else if (!options||!options.tab||options.tab === 'members'){
				this.manageGroupMemebers({target:this.$("#manage-group-members"),replace:replace});
			}else{
				this.manageGroupAdd({target:this.$("#manage-group-add"),replace:replace});
			}
			TabsHelper.ActivateTab(this,options.tab);
			this.activeTab=options.tab;
		},
/*---------------- Group Home ----------------*/
		manageGroupHome:function(event){
			this.tabContainer.html(homeTemplate(
			$.extend(
				{},
				this.group.toJSON(),
				{
					Strings:Strings,
					isAdmin:Context.player.isAdmin(),
				}
			)));
			Context.Router.navigate(this.getBaseUrl()+"/home", {trigger: false,replace: event.replace});
			Wait.stop();
		},
		saveGroup:function(event){
			Wait.start(event.target);
			this.group.set('description',this.$('#group-description').val());
			this.group.set('publicGroup',!this.$("#is-group-private").prop('checked'));
			this.group.save().done(function(){
				Wait.stop(event.target);
				MessageHelper.showInfo(Strings.GROUP.UPDATED);
			}).fail(function(data,ajax,error,message){
				MessageHelper.handleError(ajax,Strings.ERROR_CODE["MANAGE_GROUP.ERR.SAVE_FAILED"],this);
				Wait.stop();
			});

			
		},
		
/*---------------- Group requests ----------------*/
		manageGroupRequests:function(event){
			TabsHelper.HeaderClickHandler(this,event);
			this.prepareMembershipRequests(event);
			
		},
		prepareMembershipRequests:function(event){
			var requestDeferreds=[],
				requests=[],
				self=this,
				target=event && event.target;
				Wait.start(target);
			_.each(this.group.get("groupJoinRequests"),function(request){
					var player=new PlayerModel({playerId:request.playerId});
					requestDeferreds.push(player.fetch().fail(function(data,ajax,error,message){
						MessageHelper.handleError(ajax,Strings.PLAYER_DETAILS.FAILED_TO_REFERSH,this);
					}));
					requests.push({
						request:request,
						player:player
					});
			})
			$.when.apply(this,requestDeferreds).done(function(){
				self.plainRequests=[]
				_.each(requests,function(request){
					self.plainRequests.push({
						request:request.request,
						player:request.player.toJSON()
					});
				});
				self.tabContainer.html(requestsTemplate({requests:self.plainRequests,Strings:Strings}));
			}).always(function(){
				Wait.stop();
				Context.Router.navigate("#managegroup/"+self.group.get('id')+"/requests", {trigger: false,replace: event.replace});
			});
		},
		setMembershipStatus:function(playerId, status){
			var self=this;
			var groupRequest=this.group.getGroupMembershipRequest(playerId);
			Wait.start();
			groupRequest.status=status;
				$.ajax({
					  type: "PUT",
					  url:SettingsModel.get("server") +"/rest/groups/"+self.group.get('id')+"/members/"+groupRequest.playerId,
					  contentType: 'application/json',
					  data: groupRequest
				}).done(function(data){
					//Manually change localy, just in case reload will fail
					var requests=self.group.get("groupJoinRequests"),
						playerRequest=_.findWhere(requests,{playerId:playerId});
					self.group.set("groupJoinRequests",_.without(requests,playerRequest));
					self.group.reload().always(function(){
						self.prepareMembershipRequests({replace:true});
						Wait.stop();
					}).fail(function(data,ajax,error,message){
						MessageHelper.handleError(ajax,Strings.GROUP.INVITATION_STATUS_FAILED_REFRESH,this);
					})
					
				}).fail(function(ajax,error,message){
					MessageHelper.handleError(ajax,Strings.ERROR_CODE["MANAGE_GROUP.ERR.JOINING_FAILED"],this);
					Wait.stop();
				});
		},
		approiveMembership:function(event){
			this.setMembershipStatus($(event.target).closest("button").data("player-id"),'accepted');
		},
		declineMembership:function(event){
			this.setMembershipStatus($(event.target).closest("button").data("player-id"),'declined');
		},
/*---------------- Group members ----------------*/
		manageGroupMemebers:function(event){
			this.loadAndRenderPage();
			
			//Context.Router.navigate("#managegroup/"+this.group.get('id')+"/members", {trigger: false,replace: event.replace});
		},
		renderMembers:function(){
			if (this.pageData.length>0){
				this.viewModel={
					Strings:Strings,
					accepted:this.pageData,
					hasMore:this.pageData.length===this.pageSize,
					hasLess:this.currentStart!==0,
					hasMoreOrLess:this.currentStart!==0 ||this.pageData.length===this.pageSize,
				};
				this.tabContainer.html(membersTemplate(this.viewModel));
			}else{
				if (this.currentStart>=this.pageSize){
					this.currentStart-=this.pageSize;
					if (this.viewModel){
						this.viewModel.hasMore=false;
						this.viewModel.hasMoreOrLess=this.viewModel.hasLess;
						this.tabContainer.html(membersTemplate(this.viewModel));
					}else{
						MessageHelper.showError("Invalid page requested");
					}
				}
			}

		},
		loadNext:function(event){
			this.currentStart+=this.pageSize;
			this.loadAndRenderPage();
		},
		loadPrev:function(event){
			if (this.currentStart>=this.pageSize){
				this.currentStart-=this.pageSize;
				this.loadAndRenderPage();
			}
		},
		loadAndRenderPage:function(){
			Wait.start();
				this.loadPage().always(function(){Wait.stop();})
				.done(_.bind(function(pageData){
					this.pageData=pageData;
					this.renderMembers();
				},this));
		},
		loadPage: function () {
			return $.ajax({
				type: "GET",
				url:SettingsModel.get("server") +"/rest/groups/"+this.group.get("id")+"/members?from="+this.currentStart+"&size="+this.pageSize+"&_it="+this.group.groupIterationToken()

			})
			.fail(function(ajax,error,message){
				MessageHelper.handleError(ajax,Strings.ERROR_CODE["RANKINGS.ERR.LOADING_FAILED"],this);
			});
		},
		removeMember:function(event){
			event.stopPropagation();
			var self=this,
				reload=function(){
					self.group.reload()
						.done(function(){self.manageGroupMemebers()})
						.fail(function(ajax,error,message){
							MessageHelper.handleError(ajax,Strings.GROUP.REMOVING_FAILED_REFRESH,this);
						});
				},
				playerId=$(event.target).closest('.list-group-item').data('player-id');
			this.group.removePlayer(playerId)
			.done(function(ajax,error,message){
				reload();
			})
			.fail(function(ajax,error,message){
				if (ajax.status === 201){
					//invitation sent
					reload();
				}else{
					MessageHelper.handleError(ajax,Strings.GROUP.REMOVING_FAILED,this);
				}
			});
			return false;
		},
		
/*---------------- Group Add ----------------*/
		manageGroupAdd:function(event){
			this.from=0;
			this.rendersearchScreen();
			Context.Router.navigate("#managegroup/"+this.group.get('id')+"/add", {trigger: false,replace: event.replace});
		},
		nextClick:function(){
			this.from+=10;
			this.searchPlayerImpl();
		},
		prevClick:function(){
			this.from-=10;
			this.searchPlayerImpl();
		},

		rendersearchScreen:function(){
			var self=this,
				searchResultsJSON=this.playerSearch?this.playerSearch.toJSON():[],
				invitedPlayer={};
			_.each(this.group.get("groupInvitations"),function(invitation){
					invitedPlayer[invitation.playerId]=invitation;
			});
			if (searchResultsJSON.length){
				this.prevResults=searchResultsJSON;
				this.hasMore=searchResultsJSON.length ===10;
			}else{
				if (this.from>0){
					this.from-=10;
				}
				this.hasMore=false;
			}
			
			this.tabContainer.html(addTemplate({
				Strings:Strings,
				searchText:this.searchText,
				searchResults:this.prevResults,
				nothingFound:this.playerSearch&&this.prevResults.length === 0,
				somethingFound:this.playerSearch&&this.prevResults.length > 0,
				isAlreadyInvited:function(){
					return invitedPlayer[this.id];
				},
				isMyself:function(){
					return Context.player.get("playerId")==this.id;
				},
				hasLess:this.from>0,
				hasMore:this.hasMore,
				hasMoreOrLess:this.from>0 ||this.hasMore,
				hasNoMore:this.from>0 && !this.hasMore
			}));
			Wait.stop();
		},
		searchPlayer:function(event){
			this.prevResults=[];
			this.from=0;
			this.searchPlayerImpl();
		},
		searchPlayerImpl:function(event){
			MessageHelper.hide();
			this.searchText=$("#search-player-text").val().trim();
			if (!this.searchText || this.searchText===""){
				MessageHelper.showError(Strings.GROUP.SEARCH_MIN_CHARS);
				return;
			}
			this.playerSearch = new PlayerCollection([], {
					anyname:this.searchText,
					from:this.from,
					size:10
			});

			Wait.start();
			this.playerSearch.reload()
				.done(_.bind(this.rendersearchScreen, this))
				.fail(function(data,ajax,error,message){
					Wait.stop();
					MessageHelper.handleError(ajax,Strings.ERROR_CODE["MANAGE_GROUP.ERR.FAILED_TO_FETCH"],this);
			});
		},
		invitePlayer:function(event){
			var playerId=$(event.target).closest("button").data("player-id"),
				self=this;
			//MessageHelper.showInputMessage({header:Strings.GROUP.INVITE_PLAYER_HEADER,message:Strings.GROUP.INVITE_PLAYER_MESSAGE})
			//.done(function(message){
				var message="";
				Wait.start();
				self.group.invitePlayer(playerId,message).done(function(data){
					self.group.get("groupInvitations").push({
						"created":new Date(),
						"groupId":self.group.get('id'),
						"groupName":self.group.get('name'),
						"id":-1,
						"ownerId":-1,
						"playerId":playerId,
						"status":"invited"
					})
					self.rendersearchScreen();
					MessageHelper.showInfo(Strings.GROUP.SENT);
					Wait.stop();
				}).fail(function(ajax,error,message){
					if (ajax.status === 201){
						self.group.get("groupInvitations").push({
							"created":new Date(),
							"groupId":self.group.get('id'),
							"groupName":self.group.get('name'),
							"id":-1,
							"ownerId":-1,
							"playerId":playerId,
							"status":"invited"
						})
						self.rendersearchScreen();
						MessageHelper.showInfo(Strings.GROUP.SENT);
					}else{
						MessageHelper.handleError(ajax,Strings.ERROR_CODE["MANAGE_GROUP.ERR.JOIN_REQUESTS_CREATION_FAILED"],this);
					}
					Wait.stop();
				});
			//});
		}
		
	});

	return View;
});

define("hgn!templates/messages", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class='sub-container'>\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);if(t.s(t.f("invitations",c,p,1),c,p,0,117,386,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("<div class=\"well well-sm\">\r");t.b("\n" + i);t.b("	<button type=\"button\" data-group-id=\"");t.b(t.v(t.d("request.groupId",c,p,0)));t.b("\" class=\"btn btn-primary group-invitation text-wrap\">\r");t.b("\n" + i);t.b("		");t.b(t.v(t.d("Strings.MESSAGES.YOU_ARE_INVITED_PART1",c,p,0)));t.b(" '");t.b(t.v(t.d("group.name",c,p,0)));t.b("'. ");t.b(t.v(t.d("Strings.MESSAGES.YOU_ARE_INVITED_PART2",c,p,0)));t.b("\r");t.b("\n" + i);t.b("	</button>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);});c.pop();}if(t.s(t.f("joinRequests",c,p,1),c,p,0,421,770,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("<div class=\"well well-sm\">\r");t.b("\n" + i);t.b("	<button type=\"button\" data-group-id=\"");t.b(t.v(t.d("request.groupId",c,p,0)));t.b("\" class=\"btn btn-primary group-join-request text-wrap\">\r");t.b("\n" + i);t.b("		");t.b(t.v(t.d("Strings.MESSAGES.PLAYER_WANTS_TO_JOIN_PART1",c,p,0)));t.b(" ");t.b(t.v(t.d("player.nickName",c,p,0)));t.b(" ");t.b(t.v(t.d("Strings.MESSAGES.PLAYER_WANTS_TO_JOIN_PART2",c,p,0)));t.b(" '");t.b(t.v(t.d("group.name",c,p,0)));t.b("'. ");t.b(t.v(t.d("Strings.MESSAGES.PLAYER_WANTS_TO_JOIN_PART3",c,p,0)));t.b("\r");t.b("\n" + i);t.b("	</button>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);});c.pop();}if(t.s(t.f("messages",c,p,1),c,p,0,802,851,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("<div class=\"well well-sm\">\r");t.b("\n" + i);t.b("	");t.b(t.v(t.f("text",c,p,0)));t.b("\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("haveMessages",c,p,1),c,p,1,0,0,"")){t.b("<div class=\"well well-sm\">\r");t.b("\n" + i);t.b("	");t.b(t.v(t.d("Strings.MESSAGES.NO_MESSAGES",c,p,0)));t.b("\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);};t.b("<div class=\"well well-sm\">\r");t.b("\n" + i);t.b("    ");t.b(t.v(t.d("Strings.MESSAGES.UPDATE_HEAD",c,p,0)));t.b("<br/>\r");t.b("\n" + i);t.b("    <span class=\"font-light\">");t.b(t.v(t.d("Strings.MESSAGES.UPDATE_MSG",c,p,0)));t.b("</span> <a href=\"#about\">");t.b(t.v(t.d("Strings.MESSAGES.READ_MORE",c,p,0)));t.b("</a>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/*global define*/
define('views/messages',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/messages',
	'models/settings',
	'util/context',
	'util/message',
	'models/group',
	'models/player',
	'util/wait',
	"i18n!nls/strings",
	'bootstrap'
], function ($, _, Backbone, BaseView, messagesTemplate,SettingsModel,
	Context,MessageHelper,GroupModel,PlayerModel,Wait,Strings) {
	

	var View = BaseView.extend({
		titel : Strings.MESSAGES.TITLE,
		el: '#wmapp',

		template: messagesTemplate,

		events: {
			'click .group-invitation':'GroupInvitation',
			'click .group-join-request':'GroupRequest'
		},
		GroupInvitation:function(event){
			var groupId=$(event.target).data('group-id');
			window.location.hash = '#group/'+groupId;
		},
		GroupRequest:function(event){
			var groupId=$(event.target).data('group-id');
			window.location.hash = '#managegroup/'+groupId+'/requests';
		},
		refresh:function(){
			Wait.start();
			Context.player.reload()
			.done(_.bind(this.initialize,this))
			.fail(function(data,ajax,error,message){
					MessageHelper.handleError(ajax,Strings.PLAYER_DETAILS.FAILED_TO_REFERSH,this);
			})
			.always(function(){Wait.stop();});
		},
		initialize: function () {
			var requestDeferreds=[],
				invitations=[],
				joinRequests=[],
				self=this;
			Wait.start();
			_.each(Context.player.get("groupInvitations"),function(request){
					var group=new GroupModel({id:request.groupId}),
						player=new PlayerModel({playerId:request.playerId}),
						groupDeferred=group.fetch().fail(function(data,ajax,error,message){
							MessageHelper.handleError(ajax,Strings.ERROR_CODE["GROUPDETAILS.ERR.FAILED_FETCH"],this);
						}),
						playerDeferred=player.fetch().fail(function(data,ajax,error,message){
								MessageHelper.handleError(ajax,Strings.PLAYER_DETAILS.FAILED_TO_REFERSH,this);
						});

					requestDeferreds.push(groupDeferred);
					requestDeferreds.push(playerDeferred);
					invitations.push({
						request:request,
						player:player,
						group:group,
						deferreds:[groupDeferred,playerDeferred]
					});
			});
			_.each(Context.player.get("groupJoinRequests"),function(request){
					var group=new GroupModel({id:request.groupId}),
						player=new PlayerModel({playerId:request.playerId}),
						groupDeferred=group.fetch().fail(function(data,ajax,error,message){
							MessageHelper.handleError(ajax,Strings.ERROR_CODE["GROUPDETAILS.ERR.FAILED_FETCH"],this);
						}),
						playerDeferred=player.fetch().fail(function(data,ajax,error,message){
								MessageHelper.handleError(ajax,Strings.PLAYER_DETAILS.FAILED_TO_REFERSH,this);
						});

					requestDeferreds.push(groupDeferred);
					requestDeferreds.push(playerDeferred);
					joinRequests.push({
						request:request,
						player:player,
						group:group,
						deferreds:[groupDeferred,playerDeferred]
					});
			})
			Context.whenAllDone.apply(this,requestDeferreds).always(function(){
				var plainInvitations=[],
					plainJoinRequests=[];
				_.each(invitations,function(request){
					if (request.deferreds[0].state()==='resolved' && request.deferreds[0].state()==='resolved'){
						if (request.request.status === 'invited'){
							plainInvitations.push({
								request:request.request,
								player:request.player.toJSON(),
								group:request.group.toJSON(),
							})
						}
					}
				});
				_.each(joinRequests,function(request){
					if (request.deferreds[0].state()==='resolved' && request.deferreds[0].state()==='resolved'){
						plainJoinRequests.push({
							request:request.request,
							player:request.player.toJSON(),
							group:request.group.toJSON(),
						})
					}
				});
				var messages=Context.player.toJSON().messages;
				self.$el.html(self.template({
						Strings:Strings,
						messages:messages,
						invitations:plainInvitations,
						joinRequests:plainJoinRequests,
						haveMessages:(invitations && invitations.length)||(plainJoinRequests && plainJoinRequests.length)||(messages && messages.length)
					}
				));
				Wait.stop();
			});
		}
		
	});

	return View;
});

define("hgn!templates/feedback", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class='sub-container'>\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);t.b("    <p>");t.b(t.v(t.d("Strings.FEEDBACK.TELL_US",c,p,0)));t.b("</p>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("<form role=\"form\">\r");t.b("\n" + i);t.b("  <div class=\"form-group\">\r");t.b("\n" + i);t.b("    <label for=\"exampleInputEmail1\">");t.b(t.v(t.d("Strings.FEEDBACK.REASON",c,p,0)));t.b("</label>\r");t.b("\n" + i);t.b("    <select id=\"feedback-type\" class=\"form-control\" placeholder=\"");t.b(t.v(t.d("Strings.FEEDBACK.REASON",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("    	<option>");t.b(t.v(t.d("Strings.FEEDBACK.PROBLEM",c,p,0)));t.b("</option>\r");t.b("\n" + i);t.b("    	<option>");t.b(t.v(t.d("Strings.FEEDBACK.QUESTION",c,p,0)));t.b("</option>\r");t.b("\n" + i);t.b("    	<option>");t.b(t.v(t.d("Strings.FEEDBACK.COMMENT",c,p,0)));t.b("</option>\r");t.b("\n" + i);t.b("    	<option>");t.b(t.v(t.d("Strings.FEEDBACK.BETTER_NEXT_TIME",c,p,0)));t.b("</option>\r");t.b("\n" + i);t.b("    	<option>");t.b(t.v(t.d("Strings.FEEDBACK.MISSED_FEATURE",c,p,0)));t.b("</option>\r");t.b("\n" + i);t.b("    	<option>");t.b(t.v(t.d("Strings.FEEDBACK.LIKED_MOST",c,p,0)));t.b("</option>\r");t.b("\n" + i);t.b("    	<option>");t.b(t.v(t.d("Strings.FEEDBACK.OTHER",c,p,0)));t.b("</option>\r");t.b("\n" + i);t.b("    </select>\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("  <div class=\"form-group\">\r");t.b("\n" + i);t.b("    <label for=\"exampleInputPassword1\">");t.b(t.v(t.d("Strings.FEEDBACK.FEEDBACK",c,p,0)));t.b("</label>\r");t.b("\n" + i);t.b("    <textarea id=\"feedback-text\" class=\"form-control\" rows=\"5\" placeholder=\"");t.b(t.v(t.d("Strings.FEEDBACK.PROVIDE_FEEDBACK",c,p,0)));t.b("\" >\r");t.b("\n" + i);t.b("    </textarea>\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("   <div class=\"form-group\">\r");t.b("\n" + i);t.b("      <input id=\"send\" type=\"button\" class=\"btn btn-primary btn-lg btn-block\" value=\"");t.b(t.v(t.d("Strings.FEEDBACK.SEND_FEEDBACK",c,p,0)));t.b("\"\r");t.b("\n" + i);t.b("      data-loading-text=\"");t.b(t.v(t.d("Strings.FEEDBACK.SENDING_FEEDBACK",c,p,0)));t.b("\"\r");t.b("\n" + i);t.b("      />\r");t.b("\n" + i);t.b("       <!--<button type=\"button\" id=\"send\" class=\"btn btn-primary btn-lg btn-block\" value=\"Send Feedback\"><span class=\"glyphicon glyphicon-comment\"></span> Send Feedback</button>-->\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/*global define*/
define('views/feedback',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/feedback',
	'models/settings',
	'util/context',
	'util/message',
	'collections/results',
	'collections/tips',
	'util/country',
	'util/wait',
	'util/feedback',
	"i18n!nls/strings",
	'bootstrap'
], function ($, _, Backbone, BaseView, FeedbackTemplate,SettingsModel,Context,MessageHelper,
	Results,Tips,CountryMapper,Wait,Feedback,Strings) {
	

	var View = BaseView.extend({

		titel : Strings.FEEDBACK.FEEDBACK,	
		subView : true,	
	
		el: '#wmapp',

		template: FeedbackTemplate,

		events: {
			'click #send':		'send'
		},


		initialize: function () {
			this.$el.html(this.template({Strings:Strings}));
		},
		send:function(event){
			Wait.start(event.target);
			Feedback.sendFeedback(this.$("#feedback-type").val(),this.$("#feedback-text").val())
				.done(_.bind(function(data){
					MessageHelper.showInfo(Strings.FEEDBACK.SENT);
					this.$("#feedback-type").val("");
					this.$("#feedback-text").val("");
					Wait.stop(event.target);
				},this)).fail(function(ajax,error,message){
					Wait.stop(event.target);
					MessageHelper.handleError(ajax,Strings.ERROR_CODE["FEEDBACK.ERR.FAILED"],this);
				});
		}
		
	});

	return View;
});

define("hgn!templates/editprofile", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class='sub-header'>\r");t.b("\n" + i);t.b("  <div class=\"row\">\r");t.b("\n" + i);t.b("    <div class=\"col-xs-9 left\">\r");t.b("\n" + i);t.b("        <h2>");t.b(t.v(t.f("nickName",c,p,0)));t.b(" <span class=\"badge badge-bg-primary badge-lg-font\">");t.b(t.v(t.f("overallPoints",c,p,0)));t.b("</span></h2>\r");t.b("\n" + i);t.b("        <div class=\"form-control-static\">");t.b(t.v(t.f("realName",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("        <div class=\"row\">\r");t.b("\n" + i);t.b("            <label class=\"col-sm-2 control-label\">");t.b(t.v(t.d("Strings.COMMON.EMAIL",c,p,0)));t.b("</label>\r");t.b("\n" + i);t.b("            <content class=\"form-control-static text-wrap\">");t.b(t.v(t.f("email",c,p,0)));t.b("</content>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("        <div class=\"row\">\r");t.b("\n" + i);t.b("            <label class=\"col-sm-2 control-label\">");t.b(t.v(t.d("Strings.COMMON.ORGANIZATION",c,p,0)));t.b("</label>\r");t.b("\n" + i);t.b("            <content class=\"form-control-static text-wrap\">");t.b(t.v(t.f("organization",c,p,0)));t.b("</content>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"col-xs-3\"><img src=\"");t.b(t.v(t.f("CountryFlag",c,p,0)));t.b("\" class=\"img-responsive\" alt=\"Responsive image\"/>");t.b(t.v(t.f("CountryName",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("  <div class=\"row col-xs-12\">\r");t.b("\n" + i);t.b("      <!--<label class=\"col-sm-2 control-label\">realName</label>-->\r");t.b("\n" + i);t.b("        ");t.b(t.v(t.d("Strings.COMMON.REAL_NAME",c,p,0)));t.b("\r");t.b("\n" + i);t.b("        <input type=\"text\" class=\"form-control\" id=\"real-name\"  value=\"");t.b(t.v(t.f("realName",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("    <div class=\"form-group sub-container have-footer\">\r");t.b("\n" + i);t.b("      <div class=\"panel panel-alpha\">\r");t.b("\n" + i);t.b("        <div class=\"panel-heading\">\r");t.b("\n" + i);t.b("            <h3 class=\"panel-title clearfix\">");t.b(t.v(t.d("Strings.COMMON.MY_GROUPS",c,p,0)));t.b("</h3>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("        <div class=\"list-group\">\r");t.b("\n" + i);if(t.s(t.f("groups",c,p,1),c,p,0,1347,1807,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <a href=\"#managegroup/");t.b(t.v(t.f("groupId",c,p,0)));t.b("\" class=\"list-group-item\">\r");t.b("\n" + i);t.b("              <div class=\"row\">\r");t.b("\n" + i);t.b("                <div class=\"col-xs-9\">");t.b(t.v(t.f("groupName",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                <div class=\"col-xs-3\"><button id=\"delete-group\" type=\"button\" class=\"btn btn-danger badge-button btn-sm\" data-group-id=\"");t.b(t.v(t.f("groupId",c,p,0)));t.b("\"><span class=\"glyphicon glyphicon-remove-sign\"></span> ");t.b(t.v(t.d("Strings.COMMON.DELETE",c,p,0)));t.b("</button></div>\r");t.b("\n" + i);t.b("              </div>\r");t.b("\n" + i);t.b("            </a>\r");t.b("\n" + i);});c.pop();}t.b("        </div>\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);t.b("  </div>  \r");t.b("\n" + i);t.b("    <div class=\"sub-footer\">   \r");t.b("\n" + i);t.b("       <div class=\"form-group\">\r");t.b("\n" + i);t.b("        <input id=\"update\" type=\"button\" class=\"btn btn-primary btn-lg btn-block\" value=\"");t.b(t.v(t.d("Strings.COMMON.SAVE",c,p,0)));t.b("\"/>\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);t.b("      <div class=\"form-group\">\r");t.b("\n" + i);t.b("          <input id=\"change-password\" type=\"button\" class=\"btn btn-default btn-lg btn-block\" value=\"");t.b(t.v(t.d("Strings.COMMON.CHANGE_PWD",c,p,0)));t.b("\"/>\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);t.b("  </div>");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/*global define*/
define('views/editprofile',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/editprofile',
	'models/settings',
	'util/context',
	'util/message',
	'models/group',
	'util/country',
	'util/wait',
	"i18n!nls/strings",
	'bootstrap'
], function ($, _, Backbone, BaseView, editProfileTemplate,SettingsModel,
	Context,MessageHelper,GroupModel,CountryMapper,Wait,Strings) {
	

	var View = BaseView.extend({

		titel : Strings.EDITPROFILE.TITLE,
		el: '#wmapp',

		template: editProfileTemplate,

		events: {
			'click #update':			'update',
			'click #change-password':   "ChangePassword",
			'click #delete-group': 		"DeleteGroup",
		},


		initialize: function () {
			this.$el.html(this.template($.extend({},Context.player.toJSON(),{
				isAdmin:Context.player.isAdmin(),
				Strings:Strings,
				CountryName:CountryMapper.getCountryName(Context.player.get('countryCode')),
				CountryFlag:CountryMapper.getCountryFlag(Context.player.get('countryCode')),
			})));
			
		},
		update:function(event){
			Wait.start(event.target);
			Context.player.set("realName",this.$("#real-name").val());
			Context.player.update().done(function(){
				Wait.stop();
				MessageHelper.showInfo(Strings.EDITPROFILE.SAVED);
			}).fail(function(data,ajax,error,message){
				Wait.stop();
				MessageHelper.handleError(ajax,Strings.ERROR_CODE["EDITPROFILE.ERR.FAILED_PROFILE"],this);
			});
		},
		ChangePassword:function(){
			Context.requestLoginOnChangePassword=true;
			window.location.hash = '#changepwd';
		},
		DeleteGroup:function(event){
			var groupId=$(event.target).data("group-id");
			MessageHelper.showInputMessage({header:Strings.EDITPROFILE.DELETE_PLAYER,message:Strings.EDITPROFILE.DELETE_REASON})
			.done(function(message){
				Wait.start(event.target);
				$.ajax({
					  type: "DELETE",
					  url:new GroupModel({id:groupId}).url(),
					  contentType: 'application/json',
					  data: {text:message}
				}).done(function(){
					Wait.stop();
					MessageHelper.showInfo(Strings.EDITPROFILE.GROUP_DELETE);
				}).fail(function(ajax,error,message){
					Wait.stop();
					MessageHelper.handleError(ajax,Strings.ERROR_CODE["EDITPROFILE.ERR.FAILED_GROUP"],this);
				})
			});
			return false;
		}
	});

	return View;
});

define("hgn!templates/groupsearchresults", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class='sub-header-nav'>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);t.b("        <div class=\"input-group\">\r");t.b("\n" + i);t.b("            <input id=\"search-group-text\" type=\"text\" class=\"form-control search-text\" value=\"");t.b(t.v(t.f("searchText",c,p,0)));t.b("\" placeholder=\"");t.b(t.v(t.d("Strings.COMMON.SEARCH_GROUPS",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("            <span class=\"input-group-btn\">\r");t.b("\n" + i);t.b("                <button id=\"search-group\" class=\"btn btn-default btn-search\" type=\"button\" value=\"");t.b(t.v(t.d("Strings.COMMON.SEARCH",c,p,0)));t.b("\"><span class=\"glyphicon glyphicon-search\"></span></button>\r");t.b("\n" + i);t.b("            </span>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("<div class='sub-container'>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);t.b("        <div class=\"form-group\">\r");t.b("\n" + i);t.b("            <h2>");t.b(t.v(t.d("Strings.GROUPSEARCHRESULTS.SEARCH_RESULTS",c,p,0)));t.b("</h2>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);if(!t.s(t.f("nothingFound",c,p,1),c,p,1,0,0,"")){if(t.s(t.f("searchText",c,p,1),c,p,0,915,2504,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                <div class=\"form-group\">\r");t.b("\n" + i);t.b("                    <div class=\"panel panel-alpha\">\r");t.b("\n" + i);t.b("                        <div class=\"list-group\">\r");t.b("\n" + i);if(t.s(t.f("searchResults",c,p,1),c,p,0,1108,2181,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                            <a href=\"#group/");t.b(t.v(t.f("id",c,p,0)));t.b("\" class=\"list-group-item\">\r");t.b("\n" + i);t.b("                                <div class=\"row\">\r");t.b("\n" + i);t.b("                                    <div class=\"col-xs-9\">");t.b(t.v(t.f("name",c,p,0)));if(!t.s(t.f("publicGroup",c,p,1),c,p,1,0,0,"")){t.b("\r");t.b("\n" + i);t.b("        <span class=\"wmicon icon-lock  color-grey-profile padding-left-10\"></span>\r");t.b("\n" + i);t.b("        ");};t.b("</div>\r");t.b("\n" + i);t.b("                                    <div class=\"col-xs-3\">\r");t.b("\n" + i);t.b("                                        <span class=\"wmicon icon-users-2 content-font-small\" />&nbsp;");t.b(t.v(t.f("memberCount",c,p,0)));t.b("&nbsp;&nbsp;\r");t.b("\n" + i);if(t.s(t.f("iconClass",c,p,1),c,p,0,1683,1816,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                                            <span class=\"glyphicon ");t.b(t.v(t.f("iconClass",c,p,0)));t.b("\"></span>\r");t.b("\n" + i);});c.pop();}t.b("                                    </div>\r");t.b("\n" + i);t.b("                                </div>\r");t.b("\n" + i);t.b("                                <div class=\"row\">\r");t.b("\n" + i);t.b("                                    <div class=\"col-xs-12 content-font-small font-light\">");t.b(t.v(t.f("description",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("                                </div>\r");t.b("\n" + i);t.b("                            </a>\r");t.b("\n" + i);});c.pop();}if(t.s(t.f("canHaveMore",c,p,1),c,p,0,2245,2390,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                                <div class=\"list-group-item\">");t.b(t.v(t.d("Strings.GROUPSEARCHRESULTS.MORE_AWAILIABLE",c,p,0)));t.b("</div>\r");t.b("\n" + i);});c.pop();}t.b("                        </div>\r");t.b("\n" + i);t.b("                    </div>\r");t.b("\n" + i);t.b("                </div>\r");t.b("\n" + i);});c.pop();}};t.b("\r");t.b("\n" + i);if(t.s(t.f("nothingFound",c,p,1),c,p,0,2575,2689,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                <h4 class=\"font-light font-color-danger\">");t.b(t.v(t.d("Strings.GROUPSEARCHRESULTS.NO_GROUPS",c,p,0)));t.b("</h4>\r");t.b("\n" + i);});c.pop();}t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/*global define*/
define('views/groupsearchresults',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/groupsearchresults',
	'models/settings',
	'util/context',
	'util/message',
	'collections/groups',
	'util/wait',
	"i18n!nls/strings",
	'bootstrap'
], function ($, _, Backbone, BaseView, groupSearchResultsTemplate,SettingsModel,Context,MessageHelper,GroupCollection,Wait,Strings) {
	

	var View = BaseView.extend({

		titel : Strings.GROUPSEARCHRESULTS.TITLE,
		el: '#wmapp',

		template: groupSearchResultsTemplate,

		events: {
			'click #search-group': 'searchGroup',
			'keypress #search-group-text': 'keyboardSearch'
		},

		keyboardSearch:function(e){
			if (e.keyCode == 13 ){
				this.searchGroup();
			}
		},
		searchGroup:function(){
			MessageHelper.hide();
			Context.groupsSearch={};
			var searchText=$("#search-group-text").val();	
			if (!searchText){
				return;
			}
			window.location.hash = '#groupsearchresults/'+encodeURIComponent(searchText);
			Wait.start();
			this.doSearch(searchText).always(function(){
				Wait.stop();
			});
			
		},
		doSearch:function(searchText){
			this.request=searchText;
			Context.groupsSearch[this.request] = new GroupCollection([], {
					find:this.request.trim(),
					from:0,
					size:30
					});
			return Context.groupsSearch[this.request].reload()
				.done(_.bind(this.resultsFound,this))
				.fail(function(data,ajax,error,message){
					MessageHelper.handleError(ajax,Strings.ERROR_CODE["GROUPS.ERR.FAILED_FETCH"],this);
				})
		},
		initialize: function (options) {
			Context.groupsSearch=Context.groupsSearch||{};
			this.request=options.request;
			if (Context.groupsSearch && Context.groupsSearch[options.request]){
				this.resultsFound(Context.groupsSearch[this.request]);
			}else{
				this.doSearch(this.request)
			}
			
		},
		resultsFound:function(){
			var self=this,
				searchResults=Context.groupsSearch[this.request] && Context.groupsSearch[this.request].toJSON()
			this.$el.html(this.template(
				$.extend(
					{
						Strings:Strings,
						iconClass:function(){
							if (Context.player.isMemberOf(this.id)){
								return 'glyphicon-ok'
							}else if (Context.player.getGroupJoinRequested(this.id)){
								return 'glyphicon-send'
							}else if (Context.player.getGroupInvited(this.id)){
								return 'glyphicon-flag'
							}
							return undefined;
						},
						canHaveMore:searchResults && searchResults.length===30,
						searchResults:searchResults,
						searchText:this.request,
						nothingFound:!Context.groupsSearch||!Context.groupsSearch[this.request]||Context.groupsSearch[this.request].length === 0
					},
					Context.player.toJSON(),
					{isAdmin:Context.player.isAdmin()})
			));
		}
	});

	return View;
});

define("hgn!templates/invitetomygroup", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<ul class=\"list-group sub-container have-footer\">\r");t.b("\n" + i);t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);t.b("         <div><h4 class=\"font-light\">");t.b(t.v(t.d("Strings.INVITE_TO_MY_GROUP.SELECT_GROUP",c,p,0)));t.b("</h4></div>\r");t.b("\n" + i);if(t.s(t.f("myGroups",c,p,1),c,p,0,247,775,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <li class=\"list-group-item ");if(t.s(t.f("isMember",c,p,1),c,p,0,301,316,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("is-group-member");});c.pop();}t.b(" ");if(t.s(t.f("hasInvitation",c,p,1),c,p,0,348,366,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("is-already-invited");});c.pop();}t.b(" min-height-lists\" data-group-id=\"");t.b(t.v(t.f("id",c,p,0)));t.b("\"> \r");t.b("\n" + i);t.b("              <div>");t.b(t.v(t.f("name",c,p,0)));t.b(" \r");t.b("\n" + i);t.b("                <span class=\"glyphicon glyphicon-ok form-control-feedback\"></span>\r");t.b("\n" + i);t.b("                    <span class=\"glyphicon glyphicon-plus form-control-feedback\"></span>\r");t.b("\n" + i);t.b("                <span class=\"glyphicon glyphicon-share-alt form-control-feedback\"></span>\r");t.b("\n" + i);t.b("              </div>\r");t.b("\n" + i);t.b("            </li>\r");t.b("\n" + i);});c.pop();}t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</ul>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("<div class=\"sub-footer\">\r");t.b("\n" + i);t.b("    <div class=\"btn-group btn-group-lg btn-group-justified\">\r");t.b("\n" + i);t.b("        <div class=\"btn-group\">\r");t.b("\n" + i);t.b("            <button type=\"button\" id=\"invite\" class=\"btn footer-btn active\" value=\"Invite\"><span class=\"glyphicon glyphicon-plus-sign\" />\r");t.b("\n" + i);t.b("                <br/><span class=\"content-font-small\">");t.b(t.v(t.d("Strings.INVITE_TO_MY_GROUP.INVITE",c,p,0)));t.b("&nbsp;");t.b(t.v(t.d("player.nickName",c,p,0)));t.b("</span>\r");t.b("\n" + i);t.b("            </button>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/*global define*/
define('views/invitetomygroup',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/invitetomygroup',
	'models/settings',
	'models/player',
	'models/group',
	'util/context',
	'util/message',
	'collections/groups',
	'util/wait',
	"i18n!nls/strings",
	'bootstrap'
], function ($, _, Backbone, BaseView, inviteToMyGroupTemplate,
	SettingsModel,PlayerModel,GroupModel,
	Context,MessageHelper,GroupCollection,Wait,Strings) {
	

	var View = BaseView.extend({

		titel : Strings.INVITE_TO_MY_GROUP.TITLE,
		el: '#wmapp',

		template: inviteToMyGroupTemplate,

		events: {
			'click #invite':'invite',
			'click .list-group-item':'selectGroup'
		},
		selectGroup:function(event){
			var element=$(event.target).closest('.list-group-item');
			if (!element.hasClass('is-group-member') && !element.hasClass('is-already-invited')){
				element.toggleClass( 'group-selected' );
			}
			
		},
		invite:function(event){
			this.selectedGroups=[];
			this.$('.group-selected').each(_.bind(function(index,el){
				this.selectedGroups.push($(el).data('group-id'));
			},this));
			if (this.selectedGroups.length>0){
			//	MessageHelper.showInputMessage({
			//		header:Strings.GROUP.INVITE_PLAYER_HEADER,
			//		message:Strings.GROUP.INVITE_PLAYER_MESSAGE
			//	})
			//	.done(_.bind(
			//		function(){
						Wait.start(event.target);
						this.doInvitePlayer().always(function(){
							Wait.stop();
						});
			//		}
			//	,this));
			}else{
				MessageHelper.showError(Strings.INVITE_TO_MY_GROUP.MIN_GROUP_HINT);
			}
		},
		doInvitePlayer:function(message){
			var playerId=this.player.get('playerId');
			var invitations=[]

			_.each(this.selectedGroups,_.bind(function(groupId){
				var group=new GroupModel({id:groupId});
				invitations.push(group.invitePlayer(playerId,message))
			},this));
					
			return $.when.apply(this,invitations).done(_.bind(function(data){
				Context.Router.navigate('#player/'+playerId, {trigger: true, replace: true});
					MessageHelper.showInfo(Strings.GROUP.SENT);
			},this)).fail(function(ajax,error,message){
				if (ajax.status === 201){
					Context.Router.navigate('#player/'+playerId, {trigger: true, replace: true});
					MessageHelper.showInfo(Strings.GROUP.SENT);
				}else{
					MessageHelper.handleError(ajax,Strings.ERROR_CODE["MANAGE_GROUP.ERR.JOIN_REQUESTS_CREATION_FAILED"],this);
				}
			});
		},
		loadData:function(){
			var self=this,
				deferreds=[];
			this.myGroups=_.map(
					Context.player.get('groups').filter(
						function(group){
							return group.ownerId==Context.player.get('id')
						}
					),
					function(group){
						var model= new GroupModel({id:group.groupId})
						deferreds.push(model.fetch().fail(function(data,ajax,error,message){
							MessageHelper.handleError(ajax,Strings.ERROR_CODE["GROUPDETAILS.ERR.FAILED_FETCH"],this);
						}));
						return model;
					}
				);
				

			this.player=new PlayerModel({playerId:this.options.playerId,includeTips:true});
			deferreds.push(this.player.fetch().fail(function(data,ajax,error,message){
					MessageHelper.handleError(ajax,Strings.PLAYER_DETAILS.FAILED_TO_REFERSH,this);
			}))

			return $.when.apply(window,deferreds);
		},
		start:function(){
			var self=this;
			this.$el.html(
					this.template(
						$.extend(
							{},
							Context.player.toJSON(),
							{
								Strings:Strings,
								isAdmin:Context.player.isAdmin(),
								player:this.player.toJSON(),
								isMember:function(){
									return self.player.isMemberOf(this.id);
								},
								myGroups:_.map(this.myGroups,function(gr){return gr.toJSON()}),
								hasInvitation:function(){
									return _.find(this.groupInvitations,function(invitation){
											return invitation.playerId == self.options.playerId;
										});
								}
							}
						)
					)
				);
		},
		initialize: function (options) {
			this.options=options;
		}
		
	});

	return View;
});

define("hgn!templates/inviteext", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class='sub-container'>\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);t.b("    <p>");t.b(t.v(t.d("Strings.INVITETEXT.SEND_INVITATION",c,p,0)));t.b("</p>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("  <div class=\"form-group\">\r");t.b("\n" + i);t.b("    <label for=\"exampleInputPassword1\">");t.b(t.v(t.d("Strings.INVITETEXT.EMAILS",c,p,0)));t.b("</label>\r");t.b("\n" + i);t.b("    <input type=\"email\" class=\"form-control email search-text\" placeholder=\"");t.b(t.v(t.d("Strings.INVITETEXT.EMAIL_ADDRESS",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("    <div class=\"error-text\"></div>\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("  <div class=\"form-group\">\r");t.b("\n" + i);t.b("    <input type=\"email\" class=\"form-control email search-text\" placeholder=\"");t.b(t.v(t.d("Strings.INVITETEXT.EMAIL_ADDRESS",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("    <div class=\"error-text\"></div>\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("    <div class=\"form-group\">\r");t.b("\n" + i);t.b("    <input type=\"email\" class=\"form-control email search-text\" placeholder=\"");t.b(t.v(t.d("Strings.INVITETEXT.EMAIL_ADDRESS",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("    <div class=\"error-text\"></div>\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("   <div class=\"form-group\">\r");t.b("\n" + i);t.b("      <input id=\"send\" type=\"button\" class=\"btn btn-primary btn-lg btn-block\" value=\"");t.b(t.v(t.d("Strings.INVITETEXT.SEND",c,p,0)));t.b("\"\r");t.b("\n" + i);t.b("      data-loading-text=\"");t.b(t.v(t.d("Strings.INVITETEXT.SENDING",c,p,0)));t.b("\"\r");t.b("\n" + i);t.b("      />\r");t.b("\n" + i);t.b("  </div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("<div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/*global define*/
define('views/inviteext',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/inviteext',
	'models/settings',
	'util/context',
	'util/message',
	'util/wait',
	"i18n!nls/strings",
	'bootstrap'
], function ($, _, Backbone, BaseView, inviteExternalTemplate,SettingsModel,Context,MessageHelper,Wait,Strings) {
	

	var View = BaseView.extend({
		titel : Strings.INVITE_TO_MY_GROUP.INVITE_EXT,
		el: '#wmapp',

		template: inviteExternalTemplate,

		events: {
			'click #send':"send"
		},


		initialize: function () {
			this.$el.html(this.template({Strings:Strings}));
			
		},
		send:function(){
			var invitations=[],refs=[],self=this,emails=this.$(".email");
			emails.each(function(index,mail){
				var jm=$(mail);
				jm.next().text("");
				jm.parent().removeClass("success").removeClass("error");
				if (jm.val()&& jm.val().trim()){
					invitations.push({email:mail.value.trim()});
					refs.push(jm);
				}
			})
			Wait.start();
			$.ajax({
				type: "POST",
				url:SettingsModel.get("server") +"/rest/common/invite",
				contentType: 'application/json',
				data: invitations
			}).done(function(data){
				MessageHelper.showInfo(Strings.INVITE_TO_MY_GROUP.INVITATIONS_SENT);
				self.$(".email").val("");
				Wait.stop();
			}).fail(function(ajax,error,message){
				if (ajax && ajax.responseJSON && ajax.responseJSON.code === "COMMON.ERR.INVITEPLAYER"){
					_.each(ajax.responseJSON.messages,function(message, index){
						var x=refs[index];
						if (message.indexOf("Successfully")===0){
							x.parent().addClass("success");
							x.next().text(x.val()+" "+Strings.INVITE_TO_MY_GROUP.INVITATIONS_SENT);
							x.val("");
						}else if (message.indexOf("Could")===0){
							x.parent().addClass("error");
							x.next().text(Strings.INVITETEXT.AREADY_INVITED);
						}else {
							x.parent().addClass("error");
							x.next().text(Strings.INVITETEXT.INVALID_MAIL);
						}

					});
				}
				Wait.stop();
				MessageHelper.handleError(ajax,Strings.ERROR_CODE["GROUP.ERR.INVITATION_FAILED"],this);
			});
		}
		
	});

	return View;
});

define("hgn!templates/passwordreset", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class='sub-container'>\r");t.b("\n" + i);t.b("	<div class=\"col-sm-3 col-md-4\"></div>\r");t.b("\n" + i);t.b("	<div class=\"col-sm-6 col-md-4\">\r");t.b("\n" + i);t.b("		<h2>");t.b(t.v(t.d("Strings.RESET.HEADER",c,p,0)));t.b("</h2>\r");t.b("\n" + i);t.b("		<div>");t.b(t.v(t.d("Strings.RESET.MESSAGE",c,p,0)));t.b("</div>\r");t.b("\n" + i);t.b("	    <div class=\"form-group\">\r");t.b("\n" + i);t.b("	        <input type=\"email\" class=\"form-control\" id=\"email\" placeholder=\"");t.b(t.v(t.d("Strings.COMMON.EMAIL",c,p,0)));t.b("\"  required autofocus>\r");t.b("\n" + i);t.b("	    </div>\r");t.b("\n" + i);t.b("		    <div class=\"form-group\">\r");t.b("\n" + i);t.b("		    	");t.b(t.v(t.d("Strings.COMMON.SEQURITY_QUESTION",c,p,0)));t.b("\r");t.b("\n" + i);t.b("		      <select class=\"form-control search-text\" id=\"sequrityquestion\" placeholder=\"");t.b(t.v(t.d("Strings.COMMON.SEQURITY_QUESTION",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("		         <option value=\"\">");t.b(t.v(t.d("Strings.RESET.NONE_USED",c,p,0)));t.b("</option>\r");t.b("\n" + i);if(t.s(t.f("questions",c,p,1),c,p,0,637,705,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("		          <option value=\"");t.b(t.v(t.f("key",c,p,0)));t.b("\">");t.b(t.v(t.f("value",c,p,0)));t.b("</option>\r");t.b("\n" + i);});c.pop();}t.b("		      </select>\r");t.b("\n" + i);t.b("		    </div>\r");t.b("\n" + i);t.b("		    <div class=\"form-group\">\r");t.b("\n" + i);t.b("		        ");t.b(t.v(t.d("Strings.COMMON.SEQURITY_ANSWER",c,p,0)));t.b("\r");t.b("\n" + i);t.b("		        <input type=\"text\" class=\"form-control search-text\" id=\"sequrityanswer\" placeholder=\"");t.b(t.v(t.d("Strings.COMMON.SEQURITY_ANSWER",c,p,0)));t.b("\" value=\"\">\r");t.b("\n" + i);t.b("		    </div>\r");t.b("\n" + i);t.b("	    <div class=\"form-group\">\r");t.b("\n" + i);t.b("	        <input id=\"reset\" type=\"button\" class=\"btn btn-primary btn-lg btn-block\" \r");t.b("\n" + i);t.b("	        data-loading-text=\"");t.b(t.v(t.d("Strings.LOGIN.LOGIN_PROGRESS",c,p,0)));t.b("\" \r");t.b("\n" + i);t.b("	        data-placement=\"top auto\"\r");t.b("\n" + i);t.b("	        data-animation=\"false\"\r");t.b("\n" + i);t.b("	        data-popover=\"true\"\r");t.b("\n" + i);t.b("	        value=\"");t.b(t.v(t.d("Strings.RESET.RESET",c,p,0)));t.b("\"/>\r");t.b("\n" + i);t.b("	    </div>\r");t.b("\n" + i);t.b("	    <div class=\"form-group\">\r");t.b("\n" + i);t.b("        	<input  type=\"button\" class=\"btn btn-primary btn-lg btn-block\" id=\"goto-login\" value=\"");t.b(t.v(t.d("Strings.LOGIN.LOGIN",c,p,0)));t.b("\"/>\r");t.b("\n" + i);t.b("      	</div>\r");t.b("\n" + i);t.b("	</div>\r");t.b("\n" + i);t.b("	<div class=\"col-sm-3 col-md-4\"></div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/*global define*/
define('views/passwordreset',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/passwordreset',
	'models/settings',
	'util/context',
	'util/message',
	"i18n!nls/strings",
	'util/wait',
	'bootstrap'
], function ($, _, Backbone, BaseView, aboutTemplate,SettingsModel,Context,MessageHelper,Strings,Wait) {
	

	var View = BaseView.extend({
		titel : Strings.RESET.TITEL,
		el: '#wmapp',

		template: aboutTemplate,
        model:  SettingsModel,
		events: {
			'click #reset':				'reset',
			'click #goto-login': 	'gotologin'
		},
		initialize: function () {
			var questions=[];
			_.each(Strings.SEQURITY_QUESTION,function(value, key){
				questions.push({key:key,value:value});
			});
            this.$el.html(this.template($.extend({
            	Strings:Strings,
            	questions:questions,
            	Language:navigator.language
            },SettingsModel)));
		},
		gotologin:function(){
			window.location.hash = '#login';
		},
		reset:function(event){
			
			MessageHelper.hide();
			var email=this.$("#email").val(),
				sequrityquestion=this.$("#sequrityquestion").val(),
				sequrityanswer=this.$("#sequrityanswer").val(),
				isBlank=function (str) {
				    return (!str || /^\s*$/.test(str));
				};
			if(isBlank(email)){
				MessageHelper.showError(Strings.RESET.MISSING_EMAIL);
				return;
			} 
			if (!isBlank(sequrityquestion)){
				if(!sequrityanswer){
					MessageHelper.showError(Strings.RESET.MISSING_ANSWER);
					return;
				} 
			}
			Wait.start(event.target);
			$.ajax({
			  type: "POST",
			  url:SettingsModel.get("server") +"/rest/token/pwreset",
			  contentType: 'application/x-www-form-urlencoded',
			  processData : true,
			  data: {
					email:_.escape(email),
					answer:_.escape(sequrityanswer)
			  }
			}).done (function(data){
				MessageHelper.showInfo(Strings.RESET.REQUESTED);
				self.$(".btn").hide();
				self.$("#goto-login").show();
				Wait.stop();
			}).fail(function(ajax,error,message){
				Context.signedin=false;
				MessageHelper.handleError(ajax,Strings.RESET.FAILED,this);
				Wait.stop();
			})
		}

		
	});

	return View;
});

    

define("hgn!templates/playersearch", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class='sub-header-nav'>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);t.b("        <div class=\"form-group\">\r");t.b("\n" + i);t.b("              <!--Search users-->\r");t.b("\n" + i);t.b("            <div class=\"input-group\">\r");t.b("\n" + i);t.b("                <input id=\"search-player-text\" type=\"text\" class=\"form-control search-text\" value=\"");t.b(t.v(t.f("searchText",c,p,0)));t.b("\" placeholder=\"");t.b(t.v(t.d("Strings.GROUP.SEARCH_PLAYERNICK",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("                <span class=\"input-group-btn\">\r");t.b("\n" + i);t.b("                  <button id=\"search-player\" class=\"btn btn-default btn-search\" type=\"button\" value=\"Search\">\r");t.b("\n" + i);t.b("                    <span class=\"glyphicon glyphicon-search hide-when-working\"></span>\r");t.b("\n" + i);t.b("                    <div class=\"loading\">\r");t.b("\n" + i);t.b("                        <div class=\"spinner\">\r");t.b("\n" + i);t.b("                            <div class=\"mask\">\r");t.b("\n" + i);t.b("                                <div class=\"maskedCircle\"></div>\r");t.b("\n" + i);t.b("                            </div>\r");t.b("\n" + i);t.b("                        </div>\r");t.b("\n" + i);t.b("                    </div>\r");t.b("\n" + i);t.b("                  </button>\r");t.b("\n" + i);t.b("                </span>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);t.b("        </div>\r");t.b("\n" + i);t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);t.b("<div class='sub-container  ");if(t.s(t.f("hasMoreOrLess",c,p,1),c,p,0,1132,1143,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("have-footer");});c.pop();}t.b("'>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-8 col-md-6\">\r");t.b("\n" + i);if(t.s(t.f("somethingFound",c,p,1),c,p,0,1268,3144,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("      <div class=\"form-group\">\r");t.b("\n" + i);t.b("          <div class=\"panel panel-alpha\">\r");t.b("\n" + i);t.b("              <div class=\"list-group\">\r");t.b("\n" + i);t.b("    <table class=\"table\">\r");t.b("\n" + i);t.b("                        <tbody>\r");t.b("\n" + i);if(t.s(t.f("searchResults",c,p,1),c,p,0,1491,2822,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                            <tr class='player-rank' data-player-id=\"");t.b(t.v(t.f("id",c,p,0)));t.b("\">\r");t.b("\n" + i);t.b("                                <td>\r");t.b("\n" + i);t.b("                                    <div class=\"min-height-lists\">\r");t.b("\n" + i);if(t.s(t.f("hasPlayerRank",c,p,1),c,p,0,1731,1826,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                                        ");t.b(t.v(t.f("overallRank",c,p,0)));t.b("\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("hasPlayerRank",c,p,1),c,p,1,0,0,"")){t.b("                                        0\r");t.b("\n" + i);};t.b("                                    </div>\r");t.b("\n" + i);t.b("                                </td>\r");t.b("\n" + i);t.b("                                <td>\r");t.b("\n" + i);t.b("                                    <div class=\"min-height-lists\">\r");t.b("\n" + i);if(t.s(t.f("hasPlayerRank",c,p,1),c,p,0,2244,2396,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("                                        <span class=\"badge badge-bg-primary pull-right\">");t.b(t.v(t.f("overallPoints",c,p,0)));t.b("</span>\r");t.b("\n" + i);});c.pop();}if(!t.s(t.f("hasPlayerRank",c,p,1),c,p,1,0,0,"")){t.b("                                        <span class=\"badge badge-bg-primary pull-right\">0</span>\r");t.b("\n" + i);};t.b("                                    ");t.b(t.v(t.f("nickName",c,p,0)));t.b("\r");t.b("\n" + i);t.b("                                    </div>\r");t.b("\n" + i);t.b("                                </td>\r");t.b("\n" + i);t.b("                            </tr>\r");t.b("\n" + i);});c.pop();}t.b("                        </tbody>\r");t.b("\n" + i);t.b("                    </table>\r");t.b("\n" + i);if(t.s(t.f("hasNoMore",c,p,1),c,p,0,2934,3070,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("               <div class=\"list-group-item\">\r");t.b("\n" + i);t.b("                ");t.b(t.v(t.d("Strings.GROUP.NO_MORE_RESULTS",c,p,0)));t.b("\r");t.b("\n" + i);t.b("               </div>\r");t.b("\n" + i);});c.pop();}t.b("              </div>\r");t.b("\n" + i);t.b("          </div>\r");t.b("\n" + i);t.b("      </div>\r");t.b("\n" + i);});c.pop();}if(t.s(t.f("nothingFound",c,p,1),c,p,0,3186,3282,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("        <h4 class=\"font-light font-color-danger\">");t.b(t.v(t.d("Strings.GROUP.NO_PLAYERS_FOUND",c,p,0)));t.b("</h4>\r");t.b("\n" + i);});c.pop();}t.b("    </div>\r");t.b("\n" + i);t.b("    <div class=\"col-sm-2 col-md-3\"></div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);t.b("\r");t.b("\n" + i);if(t.s(t.f("hasMoreOrLess",c,p,1),c,p,0,3385,4030,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("<div class=\"sub-footer\">\r");t.b("\n" + i);t.b("    <div class=\"btn-group btn-group-lg btn-group-justified\">\r");t.b("\n" + i);if(t.s(t.f("hasLess",c,p,1),c,p,0,3495,3727,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"btn-group\">\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"load-prev\" class=\"btn footer-btn active\"><span class=\"wmicon icon-left-open-big\"></span> ");t.b(t.v(t.d("Strings.COMMON.PREV",c,p,0)));t.b("</button>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);});c.pop();}t.b("\r");t.b("\n" + i);if(t.s(t.f("hasMore",c,p,1),c,p,0,3763,3996,"{{ }}")){t.rs(c,p,function(c,p,t){t.b("            <div class=\"btn-group\">\r");t.b("\n" + i);t.b("                <button type=\"button\" id=\"load-next\" class=\"btn footer-btn active\">");t.b(t.v(t.d("Strings.COMMON.NEXT",c,p,0)));t.b(" <span class=\"wmicon icon-right-open-big\"></span></button>\r");t.b("\n" + i);t.b("            </div>\r");t.b("\n" + i);});c.pop();}t.b("    </div>\r");t.b("\n" + i);t.b("</div>\r");t.b("\n" + i);});c.pop();}return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

/*global define*/
define('views/playersearch',[
	'jquery',
	'underscore',
	'backbone',
	'views/baseview',
	'hgn!templates/playersearch',
	'models/settings',
	'util/context',
	'util/message',
	'collections/groups',
	'util/wait',
	"i18n!nls/strings",
	'collections/players',
	'bootstrap'
], function ($, _, Backbone, BaseView, playerSearchTemplate,SettingsModel,Context,MessageHelper,GroupCollection,Wait,Strings,PlayerCollection) {
	

	var View = BaseView.extend({

		titel : Strings.COMMON.SEARCH_PLAYERS,
		el: '#wmapp',

		template: playerSearchTemplate,

		events: {
			'click #search-player':'searchPlayer',
			'click #load-prev':'prevClick',
			'click #load-next':'nextClick',
			'click .player-rank': 'openPlayerDetails',
			'keypress #search-player-text': 'keyboardSearch'
		},

		keyboardSearch:function(e){
			if (e.keyCode == 13 ){
				this.searchPlayer();
			}
		},
		initialize: function (options) {
			if (typeof Context.from === 'undefined'){
				Context.from=0;
			}
			this.rendersearchScreen();
		},
		nextClick:function(){
			Context.from+=10;
			this.searchPlayerImpl();
		},
		prevClick:function(){
			Context.from-=10;
			this.searchPlayerImpl();
		},
		openPlayerDetails:function(event){
			var playerId=$(event.currentTarget).data('player-id');
			Context.Router.navigate("#player/"+playerId, {trigger: true, replace: false});
		},
		rendersearchScreen:function(){
			var self=this,
				searchResultsJSON=Context.playerSearch?Context.playerSearch.toJSON():[];

			if (searchResultsJSON.length){
				this.prevResults=searchResultsJSON;
				this.hasMore=searchResultsJSON.length ===10;
			}else{
				if (Context.from>0){
					Context.from-=10;
				}
				this.hasMore=false;
			}
			
			this.$el.html(this.template({
				Strings:Strings,
				searchText:Context.searchText,
				searchResults:this.prevResults,
				nothingFound:this.prevResults&&this.prevResults.length === 0,
				somethingFound:this.prevResults&&this.prevResults.length > 0,
				hasPlayerRank:function(){
							return this.overallRank!==2147483647;
				},
				isMyself:function(){
					return Context.player.get("playerId")==this.id;
				},
				hasLess:Context.from>0,
				hasMore:this.hasMore,
				hasMoreOrLess:Context.from>0 ||this.hasMore,
				hasNoMore:Context.from>0 && !this.hasMore
			}));
			Wait.stop();
		},
		searchPlayer:function(event){
			this.prevResults=[];
			Context.from=0;
			this.searchPlayerImpl();
		},
		searchPlayerImpl:function(event){
			MessageHelper.hide();
			Context.searchText=$("#search-player-text").val();
			if (!Context.searchText){
				MessageHelper.showError(Strings.GROUP.SEARCH_MIN_CHARS);
				return;
			}
			Context.playerSearch = new PlayerCollection([], {
					anyname:Context.searchText.trim(),
					from:Context.from,
					size:10
			});

			Wait.start();
			Context.playerSearch.reload()
				.done(_.bind(this.rendersearchScreen, this))
				.fail(function(data,ajax,error,message){
					Wait.stop();
					MessageHelper.handleError(ajax,Strings.ERROR_CODE["MANAGE_GROUP.ERR.FAILED_TO_FETCH"],this);
			});
		}
	});

	return View;
});
/*global define*/
/*jshint nonew:false*/
define('routers/router',[
	'jquery',
	'backbone',
	'util/context',
	'models/settings',
	'util/wait',
	'views/settings',
	'views/home',
	'views/login',
	'views/register',
	'views/changepwd',
	'views/playerdetails',
	'views/groupdetails',
	'views/matches',
	'views/matchtip',
	'views/rankings',
	'views/rankingsplayer',
	'views/rankingsgroup',
	'views/rankingscountry',
	'views/help',
	'views/about',
	'views/bonustips',
	'views/groups',
	'views/newgroup',
	'views/managegroup',
	'views/messages',
	'views/feedback',
	'views/editprofile',
	'views/groupsearchresults',
	'views/invitetomygroup',
	'views/inviteext',
	'views/passwordreset',
	'views/playersearch'
], function ($, Backbone,Context,Settings,Wait) {
	

 	var container,
        currentPage,
        currentOptions,
        stateHistory = [],
        openInProgress=false,
        openViewAnimationQueue=[];
	var Workspace = Backbone.Router.extend({
		routes: {
			'home': 'home',
			'home/:tab': 'home',
			'login': 'login',
			'settings':'settings',
			'register':'register',
			'changepwd':'changepwd',
			'player/:nick':'playerdetails',
			'profile':'playerdetails',
			'group/:group/:currentStart':'groupdetails',
			'group/:group':'groupdetails',
			'matches':'matches',
			'matches/:tab':'matches',
			'matchtip/:match':"matchtip",
			'help':'help',
			'about':'about',
			'bonustips':'bonustips',
			'rankings':'rankings',
			'rankingsplayer':'rankingsplayer',
			'rankingsgroup':'rankingsgroup',
			'rankingscountry':'rankingscountry',
			'rankingsplayer/:currentStart':'rankingsplayer',
			'rankingsgroup/:currentStart':'rankingsgroup',
			'rankingscountry/:currentStart':'rankingscountry',
			'groups':'groups',
			'newgroup':'newgroup',
			'managegroup/:group/:tab':'managegroup',
			'managegroup/:group/':'managegroup',
			'messages':'messages',
			'feedback':'feedback',
			'editprofile':'editprofile',
			'groupsearchresults/:request':'groupsearchresults',
			'invitetomygroup/:playerId':'invitetomygroup',
			'inviteext':'inviteext',
			'passwordreset':'passwordreset',
			'playersearch':'playersearch'
		},
		noLogin:["settings-view","login-view","register-view","about-view", "help-view","passwordreset-view"],

		home: function (tab) {
			this.openView("home-view", "views/home",{tab:tab});
		},
		login: function () {
			if (Context.signedin){
				this.navigate("home/overview", {trigger: true});
			}else{
				this.openView("login-view", "views/login");
			}
		},
		settings: function () {
			this.openView("settings-view", "views/settings");
		},
		register: function() {
			this.openView("register-view", "views/register");
		},
		changepwd: function() {
			this.openView("change-password-view", "views/changepwd");
		},
		playerdetails: function(id) {
			this.openView("userdetails-view", "views/playerdetails",{id:id});
		},
		groupdetails: function(id,currentStart) {
			this.openView("groupdetails-view", "views/groupdetails",{id:id,currentStart:currentStart||0});
		},
		matches: function(tab) {
			this.openView("gameresults-view", "views/matches",{tab:tab});
		},
		matchtip: function(id) {
			this.openView("matchtip-view", "views/matchtip",{id:id});
		},
		help: function() {
			this.openView("help-view", "views/help");
		},
		about: function() {
			this.openView("about-view", "views/about");
		},
        bonustips: function() {
			this.openView("bonustips-view", "views/bonustips");
		},
		rankings: function() {
			this.openView("rankings-view", "views/rankings");
		},
		rankingsplayer: function(currentStart) {
			this.openView("rankingsplayer-view", "views/rankingsplayer",{currentStart:currentStart});
		},
		rankingsgroup: function(currentStart) {
			this.openView("rankingsgroup-view", "views/rankingsgroup",{currentStart:currentStart});
		},
		rankingscountry: function(currentStart) {
			this.openView("rankingscountry-view", "views/rankingscountry",{currentStart:currentStart});
		},
		news: function() {
			this.openView("news-view", "views/news");
		},
		groups: function() {
			this.openView("groups-view", "views/groups");
		},
		newgroup: function() {
			this.openView("newgroup-view", "views/newgroup");
		},
		managegroup: function(id,tab) {
			this.openView("managegroup-view", "views/managegroup",{id:id,tab:tab});
		},
		messages: function() {
			this.openView("messages-view", "views/messages");
		},
		feedback: function() {
			this.openView("feedback-view", "views/feedback");
		},
		editprofile: function() {
			this.openView("editprofile-view", "views/editprofile");
		},
		groupsearchresults: function(request) {
			this.openView("groupsearchresults-view", "views/groupsearchresults",{request:request});
		},
		invitetomygroup: function(playerId) {
			this.openView("invitetomygroup-view", "views/invitetomygroup",{playerId:playerId});
		},
		inviteext:function(){
			this.openView("inviteext-view", "views/inviteext");
		},
		passwordreset:function(){
			this.openView("passwordreset-view", "views/passwordreset");
		},
		playersearch:function(){
			this.openView("playersearch-view", "views/playersearch");
		},
		slideDirection:function(){
	        var l = stateHistory.length,
	            state = window.location.hash;

	        if (l === 0) {
	            stateHistory.push(state);
	            return "right";
	        }
	        if (state === stateHistory[l-2]) {
	            stateHistory.pop();
	            return 'left';
	        } else {
	            stateHistory.push(state);
	            return 'right';
	        }
		},
		openView: function(idName, viewName,options) {
			//redirect to login if not logged in
			if (!Context.signedin && !_.contains(this.noLogin,idName)){
				window.location.hash = '#login';
				return;
			}
			//close menu if open
			$("#cbp-spmenu-s1").removeClass("cbp-spmenu-open");
			//do not do anything  if the same 
			if (currentPage && currentPage[0].id===idName){
				if (_.isEqual(currentOptions,options)){
					return;
				}else{
					if (Context.currentView.updateOptions){
						Context.currentView.updateOptions(options);
						currentOptions=options;
					}
					return;
				}
				
			}
			this.openViewAnimation(idName, viewName,options);
		},
		openViewAnimation : function(idName, viewName,options,from) {
			
			var deferred=new $.Deferred(),
				self=this;
			from=from||this.slideDirection();
			console.log("openViewAnimation("+idName+","+viewName+","+JSON.stringify(options)+","+from+")");
			/*if (openInProgress){
				openViewAnimationQueue.push({
					idName:idName, 
					viewName:viewName,
					options:options,
					from:from
				});
				var processQueue=function(){
					if (openViewAnimationQueue.length){
						var o=openViewAnimationQueue.shift();
						self.openViewAnimation(o.idName,o.viewName,o.options,o.from).done(processQueue);
					}
				}
				deferred.done(processQueue);
			}*/
			container=container||$("#wmapp");
			Wait.start();
			openInProgress=true;
            var page=$("<div id='" + idName + "'>").appendTo(container);
            // Position the page at the starting position of the animation
            page.attr("class", "page " + from);

			var newOptions=$.extend({el:"#" + idName},	options);

			var view = new (requirejs(viewName))(newOptions);	
			var progressCalled=false;
			if (view.refresh){
				$("#refresh").addClass('have-refresh');
			}else{
				$("#refresh").removeClass('have-refresh');
			}
			
			$("#viewName").text( view.getTitel() );	
			if (view.isSubView()) {
				$("#back").css( "display", "block");
			} else {
				$("#back").css( "display", "none");
			}

            if (!currentPage ) {
            	var activate=function(){
            		page.attr("class", "page center");
	                currentPage = page;
	                currentOptions=options;
	                Context.currentView=view;
	                openInProgress=false;
	                Wait.stop();
	                deferred.resolve();
	                console.log("First Page("+idName+","+viewName+","+JSON.stringify(options)+","+from+")");
            	}
            	if (view.loadData){
	            	view.loadData()
	            		.done(function(){
	            			view.start();
	            			activate();
	            		}).fail(function(){
	            			Wait.stop();
	            			deferred.reject();
	            		});
	            }else{
	            	//This is the first page jsut open it
	               	activate()
	                
	            }
	            return deferred.promise();
            }
            var transitionEnd=_.partial(function(def,e) {
            	if (!$(e.target).hasClass('remove-in-progress')){
            		console.log("Unexpected end transition event:[type:"+e.type+"; target:"+e.target.id+"]");
            		return;
            	}
            	var target=$(e.target);
            	if (target.hasClass('page')){
					console.log("Removing:"+target.id);
					target.remove();
            	}
                openInProgress=false;
                def.resolve();
                console.log("Finished[type:"+e.type+"; target:"+e.target.id+"]("+idName+","+viewName+","+JSON.stringify(options)+","+from+")");
            },deferred);
            currentPage.on('webkitTransitionEnd', transitionEnd);
            currentPage.on('transitionend', transitionEnd);

            // Force reflow. More information here: http://www.phpied.com/rendering-repaint-reflowrelayout-restyle/
            var x=container[0].offsetWidth;
            // Position the new page and the current page at the ending position of their animation with a transition class indicating the duration of the animation
            if (view.loadData){
            	view.loadData()
            		.progress(function(){
            			view.start();
            			page.attr("class", "page transition center");
			            currentPage.attr("class", " remove-in-progress page transition " + (from === "left" ? "right" : "left"));
			            currentPage = page;
			            currentOptions=options;
			            progressCalled=true;
            		})
            		.done(function(){
            			view.start();
            			if (!progressCalled){
	            			page.attr("class", "page transition center");
				            currentPage.attr("class", "remove-in-progress page transition " + (from === "left" ? "right" : "left"));
				            currentPage = page;
				            currentOptions=options;
				        }
				        Wait.stop();
				        Context.currentView=view;
            		}).fail(function(){
            			Wait.stop();
            			currentPage.off();
            			page.remove();
            			Context.doNotRemoveNextMessages=true;
            			window.history.back();
            		});
            }else{
            	page.attr("class", "page transition center");
	            currentPage.attr("class", " remove-in-progress page transition " + (from === "left" ? "right" : "left"));
	            currentPage = page;
	            currentOptions=options;
	            Context.currentView=view;
	            Wait.stop();
            }
            
           return deferred.promise();
		}
	});
	Context.Router=new Workspace();
	return Context.Router;
});

/*! Hammer.JS - v1.0.6 - 2014-01-02
 * http://eightmedia.github.com/hammer.js
 *
 * Copyright (c) 2014 Jorik Tangelder <j.tangelder@gmail.com>;
 * Licensed under the MIT license */

(function(window, undefined) {
  

/**
 * Hammer
 * use this to create instances
 * @param   {HTMLElement}   element
 * @param   {Object}        options
 * @returns {Hammer.Instance}
 * @constructor
 */
var Hammer = function(element, options) {
  return new Hammer.Instance(element, options || {});
};

// default settings
Hammer.defaults = {
  // add styles and attributes to the element to prevent the browser from doing
  // its native behavior. this doesnt prevent the scrolling, but cancels
  // the contextmenu, tap highlighting etc
  // set to false to disable this
  stop_browser_behavior: {
    // this also triggers onselectstart=false for IE
    userSelect       : 'none',
    // this makes the element blocking in IE10 >, you could experiment with the value
    // see for more options this issue; https://github.com/EightMedia/hammer.js/issues/241
    touchAction      : 'none',
    touchCallout     : 'none',
    contentZooming   : 'none',
    userDrag         : 'none',
    tapHighlightColor: 'rgba(0,0,0,0)'
  }

  //
  // more settings are defined per gesture at gestures.js
  //
};

// detect touchevents
Hammer.HAS_POINTEREVENTS = window.navigator.pointerEnabled || window.navigator.msPointerEnabled;
Hammer.HAS_TOUCHEVENTS = ('ontouchstart' in window);

// dont use mouseevents on mobile devices
Hammer.MOBILE_REGEX = /mobile|tablet|ip(ad|hone|od)|android|silk/i;
Hammer.NO_MOUSEEVENTS = Hammer.HAS_TOUCHEVENTS && window.navigator.userAgent.match(Hammer.MOBILE_REGEX);

// eventtypes per touchevent (start, move, end)
// are filled by Hammer.event.determineEventTypes on setup
Hammer.EVENT_TYPES = {};

// direction defines
Hammer.DIRECTION_DOWN = 'down';
Hammer.DIRECTION_LEFT = 'left';
Hammer.DIRECTION_UP = 'up';
Hammer.DIRECTION_RIGHT = 'right';

// pointer type
Hammer.POINTER_MOUSE = 'mouse';
Hammer.POINTER_TOUCH = 'touch';
Hammer.POINTER_PEN = 'pen';

// touch event defines
Hammer.EVENT_START = 'start';
Hammer.EVENT_MOVE = 'move';
Hammer.EVENT_END = 'end';

// hammer document where the base events are added at
Hammer.DOCUMENT = window.document;

// plugins and gestures namespaces
Hammer.plugins = Hammer.plugins || {};
Hammer.gestures = Hammer.gestures || {};

// if the window events are set...
Hammer.READY = false;

/**
 * setup events to detect gestures on the document
 */
function setup() {
  if(Hammer.READY) {
    return;
  }

  // find what eventtypes we add listeners to
  Hammer.event.determineEventTypes();

  // Register all gestures inside Hammer.gestures
  Hammer.utils.each(Hammer.gestures, function(gesture){
    Hammer.detection.register(gesture);
  });

  // Add touch events on the document
  Hammer.event.onTouch(Hammer.DOCUMENT, Hammer.EVENT_MOVE, Hammer.detection.detect);
  Hammer.event.onTouch(Hammer.DOCUMENT, Hammer.EVENT_END, Hammer.detection.detect);

  // Hammer is ready...!
  Hammer.READY = true;
}

Hammer.utils = {
  /**
   * extend method,
   * also used for cloning when dest is an empty object
   * @param   {Object}    dest
   * @param   {Object}    src
   * @parm  {Boolean}  merge    do a merge
   * @returns {Object}    dest
   */
  extend: function extend(dest, src, merge) {
    for(var key in src) {
      if(dest[key] !== undefined && merge) {
        continue;
      }
      dest[key] = src[key];
    }
    return dest;
  },


  /**
   * for each
   * @param obj
   * @param iterator
   */
  each: function(obj, iterator, context) {
    var i, length;
    // native forEach on arrays
    if ('forEach' in obj) {
      obj.forEach(iterator, context);
    }
    // arrays
    else if(obj.length !== undefined) {
      for (i = 0, length = obj.length; i < length; i++) {
        if (iterator.call(context, obj[i], i, obj) === false) {
          return;
        }
      }
    }
    // objects
    else {
      for (i in obj) {
        if (obj.hasOwnProperty(i) && iterator.call(context, obj[i], i, obj) === false) {
          return;
        }
      }
    }
  },

  /**
   * find if a node is in the given parent
   * used for event delegation tricks
   * @param   {HTMLElement}   node
   * @param   {HTMLElement}   parent
   * @returns {boolean}       has_parent
   */
  hasParent: function(node, parent) {
    while(node) {
      if(node == parent) {
        return true;
      }
      node = node.parentNode;
    }
    return false;
  },


  /**
   * get the center of all the touches
   * @param   {Array}     touches
   * @returns {Object}    center
   */
  getCenter: function getCenter(touches) {
    var valuesX = [], valuesY = [];

    Hammer.utils.each(touches, function(touch) {
      // I prefer clientX because it ignore the scrolling position
      valuesX.push(typeof touch.clientX !== 'undefined' ? touch.clientX : touch.pageX );
      valuesY.push(typeof touch.clientY !== 'undefined' ? touch.clientY : touch.pageY );
    });

    return {
      pageX: ((Math.min.apply(Math, valuesX) + Math.max.apply(Math, valuesX)) / 2),
      pageY: ((Math.min.apply(Math, valuesY) + Math.max.apply(Math, valuesY)) / 2)
    };
  },


  /**
   * calculate the velocity between two points
   * @param   {Number}    delta_time
   * @param   {Number}    delta_x
   * @param   {Number}    delta_y
   * @returns {Object}    velocity
   */
  getVelocity: function getVelocity(delta_time, delta_x, delta_y) {
    return {
      x: Math.abs(delta_x / delta_time) || 0,
      y: Math.abs(delta_y / delta_time) || 0
    };
  },


  /**
   * calculate the angle between two coordinates
   * @param   {Touch}     touch1
   * @param   {Touch}     touch2
   * @returns {Number}    angle
   */
  getAngle: function getAngle(touch1, touch2) {
    var y = touch2.pageY - touch1.pageY,
      x = touch2.pageX - touch1.pageX;
    return Math.atan2(y, x) * 180 / Math.PI;
  },


  /**
   * angle to direction define
   * @param   {Touch}     touch1
   * @param   {Touch}     touch2
   * @returns {String}    direction constant, like Hammer.DIRECTION_LEFT
   */
  getDirection: function getDirection(touch1, touch2) {
    var x = Math.abs(touch1.pageX - touch2.pageX),
      y = Math.abs(touch1.pageY - touch2.pageY);

    if(x >= y) {
      return touch1.pageX - touch2.pageX > 0 ? Hammer.DIRECTION_LEFT : Hammer.DIRECTION_RIGHT;
    }
    else {
      return touch1.pageY - touch2.pageY > 0 ? Hammer.DIRECTION_UP : Hammer.DIRECTION_DOWN;
    }
  },


  /**
   * calculate the distance between two touches
   * @param   {Touch}     touch1
   * @param   {Touch}     touch2
   * @returns {Number}    distance
   */
  getDistance: function getDistance(touch1, touch2) {
    var x = touch2.pageX - touch1.pageX,
      y = touch2.pageY - touch1.pageY;
    return Math.sqrt((x * x) + (y * y));
  },


  /**
   * calculate the scale factor between two touchLists (fingers)
   * no scale is 1, and goes down to 0 when pinched together, and bigger when pinched out
   * @param   {Array}     start
   * @param   {Array}     end
   * @returns {Number}    scale
   */
  getScale: function getScale(start, end) {
    // need two fingers...
    if(start.length >= 2 && end.length >= 2) {
      return this.getDistance(end[0], end[1]) /
        this.getDistance(start[0], start[1]);
    }
    return 1;
  },


  /**
   * calculate the rotation degrees between two touchLists (fingers)
   * @param   {Array}     start
   * @param   {Array}     end
   * @returns {Number}    rotation
   */
  getRotation: function getRotation(start, end) {
    // need two fingers
    if(start.length >= 2 && end.length >= 2) {
      return this.getAngle(end[1], end[0]) -
        this.getAngle(start[1], start[0]);
    }
    return 0;
  },


  /**
   * boolean if the direction is vertical
   * @param    {String}    direction
   * @returns  {Boolean}   is_vertical
   */
  isVertical: function isVertical(direction) {
    return (direction == Hammer.DIRECTION_UP || direction == Hammer.DIRECTION_DOWN);
  },


  /**
   * stop browser default behavior with css props
   * @param   {HtmlElement}   element
   * @param   {Object}        css_props
   */
  stopDefaultBrowserBehavior: function stopDefaultBrowserBehavior(element, css_props) {
    if(!css_props || !element || !element.style) {
      return;
    }

    // with css properties for modern browsers
    Hammer.utils.each(['webkit', 'khtml', 'moz', 'Moz', 'ms', 'o', ''], function(vendor) {
      Hammer.utils.each(css_props, function(prop) {
          // vender prefix at the property
          if(vendor) {
            prop = vendor + prop.substring(0, 1).toUpperCase() + prop.substring(1);
          }
          // set the style
          if(prop in element.style) {
            element.style[prop] = prop;
          }
      });
    });

    // also the disable onselectstart
    if(css_props.userSelect == 'none') {
      element.onselectstart = function() {
        return false;
      };
    }

    // and disable ondragstart
    if(css_props.userDrag == 'none') {
      element.ondragstart = function() {
        return false;
      };
    }
  }
};


/**
 * create new hammer instance
 * all methods should return the instance itself, so it is chainable.
 * @param   {HTMLElement}       element
 * @param   {Object}            [options={}]
 * @returns {Hammer.Instance}
 * @constructor
 */
Hammer.Instance = function(element, options) {
  var self = this;

  // setup HammerJS window events and register all gestures
  // this also sets up the default options
  setup();

  this.element = element;

  // start/stop detection option
  this.enabled = true;

  // merge options
  this.options = Hammer.utils.extend(
    Hammer.utils.extend({}, Hammer.defaults),
    options || {});

  // add some css to the element to prevent the browser from doing its native behavoir
  if(this.options.stop_browser_behavior) {
    Hammer.utils.stopDefaultBrowserBehavior(this.element, this.options.stop_browser_behavior);
  }

  // start detection on touchstart
  Hammer.event.onTouch(element, Hammer.EVENT_START, function(ev) {
    if(self.enabled) {
      Hammer.detection.startDetect(self, ev);
    }
  });

  // return instance
  return this;
};


Hammer.Instance.prototype = {
  /**
   * bind events to the instance
   * @param   {String}      gesture
   * @param   {Function}    handler
   * @returns {Hammer.Instance}
   */
  on: function onEvent(gesture, handler) {
    var gestures = gesture.split(' ');
    Hammer.utils.each(gestures, function(gesture) {
      this.element.addEventListener(gesture, handler, false);
    }, this);
    return this;
  },


  /**
   * unbind events to the instance
   * @param   {String}      gesture
   * @param   {Function}    handler
   * @returns {Hammer.Instance}
   */
  off: function offEvent(gesture, handler) {
    var gestures = gesture.split(' ');
    Hammer.utils.each(gestures, function(gesture) {
      this.element.removeEventListener(gesture, handler, false);
    }, this);
    return this;
  },


  /**
   * trigger gesture event
   * @param   {String}      gesture
   * @param   {Object}      [eventData]
   * @returns {Hammer.Instance}
   */
  trigger: function triggerEvent(gesture, eventData) {
    // optional
    if(!eventData) {
      eventData = {};
    }

    // create DOM event
    var event = Hammer.DOCUMENT.createEvent('Event');
    event.initEvent(gesture, true, true);
    event.gesture = eventData;

    // trigger on the target if it is in the instance element,
    // this is for event delegation tricks
    var element = this.element;
    if(Hammer.utils.hasParent(eventData.target, element)) {
      element = eventData.target;
    }

    element.dispatchEvent(event);
    return this;
  },


  /**
   * enable of disable hammer.js detection
   * @param   {Boolean}   state
   * @returns {Hammer.Instance}
   */
  enable: function enable(state) {
    this.enabled = state;
    return this;
  }
};


/**
 * this holds the last move event,
 * used to fix empty touchend issue
 * see the onTouch event for an explanation
 * @type {Object}
 */
var last_move_event = null;


/**
 * when the mouse is hold down, this is true
 * @type {Boolean}
 */
var enable_detect = false;


/**
 * when touch events have been fired, this is true
 * @type {Boolean}
 */
var touch_triggered = false;


Hammer.event = {
  /**
   * simple addEventListener
   * @param   {HTMLElement}   element
   * @param   {String}        type
   * @param   {Function}      handler
   */
  bindDom: function(element, type, handler) {
    var types = type.split(' ');
    Hammer.utils.each(types, function(type){
      element.addEventListener(type, handler, false);
    });
  },


  /**
   * touch events with mouse fallback
   * @param   {HTMLElement}   element
   * @param   {String}        eventType        like Hammer.EVENT_MOVE
   * @param   {Function}      handler
   */
  onTouch: function onTouch(element, eventType, handler) {
    var self = this;

    this.bindDom(element, Hammer.EVENT_TYPES[eventType], function bindDomOnTouch(ev) {
      var sourceEventType = ev.type.toLowerCase();

      // onmouseup, but when touchend has been fired we do nothing.
      // this is for touchdevices which also fire a mouseup on touchend
      if(sourceEventType.match(/mouse/) && touch_triggered) {
        return;
      }

      // mousebutton must be down or a touch event
      else if(sourceEventType.match(/touch/) ||   // touch events are always on screen
        sourceEventType.match(/pointerdown/) || // pointerevents touch
        (sourceEventType.match(/mouse/) && ev.which === 1)   // mouse is pressed
        ) {
        enable_detect = true;
      }

      // mouse isn't pressed
      else if(sourceEventType.match(/mouse/) && !ev.which) {
        enable_detect = false;
      }


      // we are in a touch event, set the touch triggered bool to true,
      // this for the conflicts that may occur on ios and android
      if(sourceEventType.match(/touch|pointer/)) {
        touch_triggered = true;
      }

      // count the total touches on the screen
      var count_touches = 0;

      // when touch has been triggered in this detection session
      // and we are now handling a mouse event, we stop that to prevent conflicts
      if(enable_detect) {
        // update pointerevent
        if(Hammer.HAS_POINTEREVENTS && eventType != Hammer.EVENT_END) {
          count_touches = Hammer.PointerEvent.updatePointer(eventType, ev);
        }
        // touch
        else if(sourceEventType.match(/touch/)) {
          count_touches = ev.touches.length;
        }
        // mouse
        else if(!touch_triggered) {
          count_touches = sourceEventType.match(/up/) ? 0 : 1;
        }

        // if we are in a end event, but when we remove one touch and
        // we still have enough, set eventType to move
        if(count_touches > 0 && eventType == Hammer.EVENT_END) {
          eventType = Hammer.EVENT_MOVE;
        }
        // no touches, force the end event
        else if(!count_touches) {
          eventType = Hammer.EVENT_END;
        }

        // store the last move event
        if(count_touches || last_move_event === null) {
          last_move_event = ev;
        }

        // trigger the handler
        handler.call(Hammer.detection, self.collectEventData(element, eventType, self.getTouchList(last_move_event, eventType), ev));

        // remove pointerevent from list
        if(Hammer.HAS_POINTEREVENTS && eventType == Hammer.EVENT_END) {
          count_touches = Hammer.PointerEvent.updatePointer(eventType, ev);
        }
      }

      // on the end we reset everything
      if(!count_touches) {
        last_move_event = null;
        enable_detect = false;
        touch_triggered = false;
        Hammer.PointerEvent.reset();
      }
    });
  },


  /**
   * we have different events for each device/browser
   * determine what we need and set them in the Hammer.EVENT_TYPES constant
   */
  determineEventTypes: function determineEventTypes() {
    // determine the eventtype we want to set
    var types;

    // pointerEvents magic
    if(Hammer.HAS_POINTEREVENTS) {
      types = Hammer.PointerEvent.getEvents();
    }
    // on Android, iOS, blackberry, windows mobile we dont want any mouseevents
    else if(Hammer.NO_MOUSEEVENTS) {
      types = [
        'touchstart',
        'touchmove',
        'touchend touchcancel'];
    }
    // for non pointer events browsers and mixed browsers,
    // like chrome on windows8 touch laptop
    else {
      types = [
        'touchstart mousedown',
        'touchmove mousemove',
        'touchend touchcancel mouseup'];
    }

    Hammer.EVENT_TYPES[Hammer.EVENT_START] = types[0];
    Hammer.EVENT_TYPES[Hammer.EVENT_MOVE] = types[1];
    Hammer.EVENT_TYPES[Hammer.EVENT_END] = types[2];
  },


  /**
   * create touchlist depending on the event
   * @param   {Object}    ev
   * @param   {String}    eventType   used by the fakemultitouch plugin
   */
  getTouchList: function getTouchList(ev/*, eventType*/) {
    // get the fake pointerEvent touchlist
    if(Hammer.HAS_POINTEREVENTS) {
      return Hammer.PointerEvent.getTouchList();
    }
    // get the touchlist
    else if(ev.touches) {
      return ev.touches;
    }
    // make fake touchlist from mouse position
    else {
      ev.identifier = 1;
      return [ev];
    }
  },


  /**
   * collect event data for Hammer js
   * @param   {HTMLElement}   element
   * @param   {String}        eventType        like Hammer.EVENT_MOVE
   * @param   {Object}        eventData
   */
  collectEventData: function collectEventData(element, eventType, touches, ev) {
    // find out pointerType
    var pointerType = Hammer.POINTER_TOUCH;
    if(ev.type.match(/mouse/) || Hammer.PointerEvent.matchType(Hammer.POINTER_MOUSE, ev)) {
      pointerType = Hammer.POINTER_MOUSE;
    }

    return {
      center     : Hammer.utils.getCenter(touches),
      timeStamp  : new Date().getTime(),
      target     : ev.target,
      touches    : touches,
      eventType  : eventType,
      pointerType: pointerType,
      srcEvent   : ev,

      /**
       * prevent the browser default actions
       * mostly used to disable scrolling of the browser
       */
      preventDefault: function() {
        if(this.srcEvent.preventManipulation) {
          this.srcEvent.preventManipulation();
        }

        if(this.srcEvent.preventDefault) {
          this.srcEvent.preventDefault();
        }
      },

      /**
       * stop bubbling the event up to its parents
       */
      stopPropagation: function() {
        this.srcEvent.stopPropagation();
      },

      /**
       * immediately stop gesture detection
       * might be useful after a swipe was detected
       * @return {*}
       */
      stopDetect: function() {
        return Hammer.detection.stopDetect();
      }
    };
  }
};

Hammer.PointerEvent = {
  /**
   * holds all pointers
   * @type {Object}
   */
  pointers: {},

  /**
   * get a list of pointers
   * @returns {Array}     touchlist
   */
  getTouchList: function() {
    var self = this;
    var touchlist = [];

    // we can use forEach since pointerEvents only is in IE10
    Hammer.utils.each(self.pointers, function(pointer){
      touchlist.push(pointer);
    });
    
    return touchlist;
  },

  /**
   * update the position of a pointer
   * @param   {String}   type             Hammer.EVENT_END
   * @param   {Object}   pointerEvent
   */
  updatePointer: function(type, pointerEvent) {
    if(type == Hammer.EVENT_END) {
      this.pointers = {};
    }
    else {
      pointerEvent.identifier = pointerEvent.pointerId;
      this.pointers[pointerEvent.pointerId] = pointerEvent;
    }

    return Object.keys(this.pointers).length;
  },

  /**
   * check if ev matches pointertype
   * @param   {String}        pointerType     Hammer.POINTER_MOUSE
   * @param   {PointerEvent}  ev
   */
  matchType: function(pointerType, ev) {
    if(!ev.pointerType) {
      return false;
    }

    var pt = ev.pointerType,
      types = {};
    types[Hammer.POINTER_MOUSE] = (pt === ev.MSPOINTER_TYPE_MOUSE || pt === Hammer.POINTER_MOUSE);
    types[Hammer.POINTER_TOUCH] = (pt === ev.MSPOINTER_TYPE_TOUCH || pt === Hammer.POINTER_TOUCH);
    types[Hammer.POINTER_PEN] = (pt === ev.MSPOINTER_TYPE_PEN || pt === Hammer.POINTER_PEN);
    return types[pointerType];
  },


  /**
   * get events
   */
  getEvents: function() {
    return [
      'pointerdown MSPointerDown',
      'pointermove MSPointerMove',
      'pointerup pointercancel MSPointerUp MSPointerCancel'
    ];
  },

  /**
   * reset the list
   */
  reset: function() {
    this.pointers = {};
  }
};


Hammer.detection = {
  // contains all registred Hammer.gestures in the correct order
  gestures: [],

  // data of the current Hammer.gesture detection session
  current : null,

  // the previous Hammer.gesture session data
  // is a full clone of the previous gesture.current object
  previous: null,

  // when this becomes true, no gestures are fired
  stopped : false,


  /**
   * start Hammer.gesture detection
   * @param   {Hammer.Instance}   inst
   * @param   {Object}            eventData
   */
  startDetect: function startDetect(inst, eventData) {
    // already busy with a Hammer.gesture detection on an element
    if(this.current) {
      return;
    }

    this.stopped = false;

    this.current = {
      inst      : inst, // reference to HammerInstance we're working for
      startEvent: Hammer.utils.extend({}, eventData), // start eventData for distances, timing etc
      lastEvent : false, // last eventData
      name      : '' // current gesture we're in/detected, can be 'tap', 'hold' etc
    };

    this.detect(eventData);
  },


  /**
   * Hammer.gesture detection
   * @param   {Object}    eventData
   */
  detect: function detect(eventData) {
    if(!this.current || this.stopped) {
      return;
    }

    // extend event data with calculations about scale, distance etc
    eventData = this.extendEventData(eventData);

    // instance options
    var inst_options = this.current.inst.options;

    // call Hammer.gesture handlers
    Hammer.utils.each(this.gestures, function(gesture) {
      // only when the instance options have enabled this gesture
      if(!this.stopped && inst_options[gesture.name] !== false) {
        // if a handler returns false, we stop with the detection
        if(gesture.handler.call(gesture, eventData, this.current.inst) === false) {
          this.stopDetect();
          return false;
        }
      }
    }, this);

    // store as previous event event
    if(this.current) {
      this.current.lastEvent = eventData;
    }

    // endevent, but not the last touch, so dont stop
    if(eventData.eventType == Hammer.EVENT_END && !eventData.touches.length - 1) {
      this.stopDetect();
    }

    return eventData;
  },


  /**
   * clear the Hammer.gesture vars
   * this is called on endDetect, but can also be used when a final Hammer.gesture has been detected
   * to stop other Hammer.gestures from being fired
   */
  stopDetect: function stopDetect() {
    // clone current data to the store as the previous gesture
    // used for the double tap gesture, since this is an other gesture detect session
    this.previous = Hammer.utils.extend({}, this.current);

    // reset the current
    this.current = null;

    // stopped!
    this.stopped = true;
  },


  /**
   * extend eventData for Hammer.gestures
   * @param   {Object}   ev
   * @returns {Object}   ev
   */
  extendEventData: function extendEventData(ev) {
    var startEv = this.current.startEvent;

    // if the touches change, set the new touches over the startEvent touches
    // this because touchevents don't have all the touches on touchstart, or the
    // user must place his fingers at the EXACT same time on the screen, which is not realistic
    // but, sometimes it happens that both fingers are touching at the EXACT same time
    if(startEv && (ev.touches.length != startEv.touches.length || ev.touches === startEv.touches)) {
      // extend 1 level deep to get the touchlist with the touch objects
      startEv.touches = [];
      Hammer.utils.each(ev.touches, function(touch) {
        startEv.touches.push(Hammer.utils.extend({}, touch));
      });
    }

    var delta_time = ev.timeStamp - startEv.timeStamp
      , delta_x = ev.center.pageX - startEv.center.pageX
      , delta_y = ev.center.pageY - startEv.center.pageY
      , velocity = Hammer.utils.getVelocity(delta_time, delta_x, delta_y)
      , interimAngle
      , interimDirection;

    // end events (e.g. dragend) don't have useful values for interimDirection & interimAngle
    // because the previous event has exactly the same coordinates
    // so for end events, take the previous values of interimDirection & interimAngle
    // instead of recalculating them and getting a spurious '0'
    if(ev.eventType === 'end') {
      interimAngle = this.current.lastEvent && this.current.lastEvent.interimAngle;
      interimDirection = this.current.lastEvent && this.current.lastEvent.interimDirection;
    }
    else {
      interimAngle = this.current.lastEvent && Hammer.utils.getAngle(this.current.lastEvent.center, ev.center);
      interimDirection = this.current.lastEvent && Hammer.utils.getDirection(this.current.lastEvent.center, ev.center);
    }

    Hammer.utils.extend(ev, {
      deltaTime: delta_time,

      deltaX: delta_x,
      deltaY: delta_y,

      velocityX: velocity.x,
      velocityY: velocity.y,

      distance: Hammer.utils.getDistance(startEv.center, ev.center),

      angle: Hammer.utils.getAngle(startEv.center, ev.center),
      interimAngle: interimAngle,

      direction: Hammer.utils.getDirection(startEv.center, ev.center),
      interimDirection: interimDirection,

      scale: Hammer.utils.getScale(startEv.touches, ev.touches),
      rotation: Hammer.utils.getRotation(startEv.touches, ev.touches),

      startEvent: startEv
    });

    return ev;
  },


  /**
   * register new gesture
   * @param   {Object}    gesture object, see gestures.js for documentation
   * @returns {Array}     gestures
   */
  register: function register(gesture) {
    // add an enable gesture options if there is no given
    var options = gesture.defaults || {};
    if(options[gesture.name] === undefined) {
      options[gesture.name] = true;
    }

    // extend Hammer default options with the Hammer.gesture options
    Hammer.utils.extend(Hammer.defaults, options, true);

    // set its index
    gesture.index = gesture.index || 1000;

    // add Hammer.gesture to the list
    this.gestures.push(gesture);

    // sort the list by index
    this.gestures.sort(function(a, b) {
      if(a.index < b.index) { return -1; }
      if(a.index > b.index) { return 1; }
      return 0;
    });

    return this.gestures;
  }
};


/**
 * Drag
 * Move with x fingers (default 1) around on the page. Blocking the scrolling when
 * moving left and right is a good practice. When all the drag events are blocking
 * you disable scrolling on that area.
 * @events  drag, drapleft, dragright, dragup, dragdown
 */
Hammer.gestures.Drag = {
  name     : 'drag',
  index    : 50,
  defaults : {
    drag_min_distance            : 10,
    
    // Set correct_for_drag_min_distance to true to make the starting point of the drag
    // be calculated from where the drag was triggered, not from where the touch started.
    // Useful to avoid a jerk-starting drag, which can make fine-adjustments
    // through dragging difficult, and be visually unappealing.
    correct_for_drag_min_distance: true,
    
    // set 0 for unlimited, but this can conflict with transform
    drag_max_touches             : 1,
    
    // prevent default browser behavior when dragging occurs
    // be careful with it, it makes the element a blocking element
    // when you are using the drag gesture, it is a good practice to set this true
    drag_block_horizontal        : false,
    drag_block_vertical          : false,
    
    // drag_lock_to_axis keeps the drag gesture on the axis that it started on,
    // It disallows vertical directions if the initial direction was horizontal, and vice versa.
    drag_lock_to_axis            : false,
    
    // drag lock only kicks in when distance > drag_lock_min_distance
    // This way, locking occurs only when the distance has become large enough to reliably determine the direction
    drag_lock_min_distance       : 25
  },
  
  triggered: false,
  handler  : function dragGesture(ev, inst) {
    // current gesture isnt drag, but dragged is true
    // this means an other gesture is busy. now call dragend
    if(Hammer.detection.current.name != this.name && this.triggered) {
      inst.trigger(this.name + 'end', ev);
      this.triggered = false;
      return;
    }

    // max touches
    if(inst.options.drag_max_touches > 0 &&
      ev.touches.length > inst.options.drag_max_touches) {
      return;
    }

    switch(ev.eventType) {
      case Hammer.EVENT_START:
        this.triggered = false;
        break;

      case Hammer.EVENT_MOVE:
        // when the distance we moved is too small we skip this gesture
        // or we can be already in dragging
        if(ev.distance < inst.options.drag_min_distance &&
          Hammer.detection.current.name != this.name) {
          return;
        }

        // we are dragging!
        if(Hammer.detection.current.name != this.name) {
          Hammer.detection.current.name = this.name;
          if(inst.options.correct_for_drag_min_distance && ev.distance > 0) {
            // When a drag is triggered, set the event center to drag_min_distance pixels from the original event center.
            // Without this correction, the dragged distance would jumpstart at drag_min_distance pixels instead of at 0.
            // It might be useful to save the original start point somewhere
            var factor = Math.abs(inst.options.drag_min_distance / ev.distance);
            Hammer.detection.current.startEvent.center.pageX += ev.deltaX * factor;
            Hammer.detection.current.startEvent.center.pageY += ev.deltaY * factor;

            // recalculate event data using new start point
            ev = Hammer.detection.extendEventData(ev);
          }
        }

        // lock drag to axis?
        if(Hammer.detection.current.lastEvent.drag_locked_to_axis || (inst.options.drag_lock_to_axis && inst.options.drag_lock_min_distance <= ev.distance)) {
          ev.drag_locked_to_axis = true;
        }
        var last_direction = Hammer.detection.current.lastEvent.direction;
        if(ev.drag_locked_to_axis && last_direction !== ev.direction) {
          // keep direction on the axis that the drag gesture started on
          if(Hammer.utils.isVertical(last_direction)) {
            ev.direction = (ev.deltaY < 0) ? Hammer.DIRECTION_UP : Hammer.DIRECTION_DOWN;
          }
          else {
            ev.direction = (ev.deltaX < 0) ? Hammer.DIRECTION_LEFT : Hammer.DIRECTION_RIGHT;
          }
        }

        // first time, trigger dragstart event
        if(!this.triggered) {
          inst.trigger(this.name + 'start', ev);
          this.triggered = true;
        }

        // trigger normal event
        inst.trigger(this.name, ev);

        // direction event, like dragdown
        inst.trigger(this.name + ev.direction, ev);

        // block the browser events
        if((inst.options.drag_block_vertical && Hammer.utils.isVertical(ev.direction)) ||
          (inst.options.drag_block_horizontal && !Hammer.utils.isVertical(ev.direction))) {
          ev.preventDefault();
        }
        break;

      case Hammer.EVENT_END:
        // trigger dragend
        if(this.triggered) {
          inst.trigger(this.name + 'end', ev);
        }

        this.triggered = false;
        break;
    }
  }
};

/**
 * Hold
 * Touch stays at the same place for x time
 * @events  hold
 */
Hammer.gestures.Hold = {
  name    : 'hold',
  index   : 10,
  defaults: {
    hold_timeout  : 500,
    hold_threshold: 1
  },
  timer   : null,
  handler : function holdGesture(ev, inst) {
    switch(ev.eventType) {
      case Hammer.EVENT_START:
        // clear any running timers
        clearTimeout(this.timer);

        // set the gesture so we can check in the timeout if it still is
        Hammer.detection.current.name = this.name;

        // set timer and if after the timeout it still is hold,
        // we trigger the hold event
        this.timer = setTimeout(function() {
          if(Hammer.detection.current.name == 'hold') {
            inst.trigger('hold', ev);
          }
        }, inst.options.hold_timeout);
        break;

      // when you move or end we clear the timer
      case Hammer.EVENT_MOVE:
        if(ev.distance > inst.options.hold_threshold) {
          clearTimeout(this.timer);
        }
        break;

      case Hammer.EVENT_END:
        clearTimeout(this.timer);
        break;
    }
  }
};

/**
 * Release
 * Called as last, tells the user has released the screen
 * @events  release
 */
Hammer.gestures.Release = {
  name   : 'release',
  index  : Infinity,
  handler: function releaseGesture(ev, inst) {
    if(ev.eventType == Hammer.EVENT_END) {
      inst.trigger(this.name, ev);
    }
  }
};

/**
 * Swipe
 * triggers swipe events when the end velocity is above the threshold
 * @events  swipe, swipeleft, swiperight, swipeup, swipedown
 */
Hammer.gestures.Swipe = {
  name    : 'swipe',
  index   : 40,
  defaults: {
    // set 0 for unlimited, but this can conflict with transform
    swipe_min_touches: 1,
    swipe_max_touches: 1,
    swipe_velocity   : 0.7
  },
  handler : function swipeGesture(ev, inst) {
    if(ev.eventType == Hammer.EVENT_END) {
      // max touches
      if(inst.options.swipe_max_touches > 0 &&
        ev.touches.length < inst.options.swipe_min_touches &&
        ev.touches.length > inst.options.swipe_max_touches) {
        return;
      }

      // when the distance we moved is too small we skip this gesture
      // or we can be already in dragging
      if(ev.velocityX > inst.options.swipe_velocity ||
        ev.velocityY > inst.options.swipe_velocity) {
        // trigger swipe events
        inst.trigger(this.name, ev);
        inst.trigger(this.name + ev.direction, ev);
      }
    }
  }
};

/**
 * Tap/DoubleTap
 * Quick touch at a place or double at the same place
 * @events  tap, doubletap
 */
Hammer.gestures.Tap = {
  name    : 'tap',
  index   : 100,
  defaults: {
    tap_max_touchtime : 250,
    tap_max_distance  : 10,
    tap_always        : true,
    doubletap_distance: 20,
    doubletap_interval: 300
  },
  handler : function tapGesture(ev, inst) {
    if(ev.eventType == Hammer.EVENT_END && ev.srcEvent.type != 'touchcancel') {
      // previous gesture, for the double tap since these are two different gesture detections
      var prev = Hammer.detection.previous,
        did_doubletap = false;

      // when the touchtime is higher then the max touch time
      // or when the moving distance is too much
      if(ev.deltaTime > inst.options.tap_max_touchtime ||
        ev.distance > inst.options.tap_max_distance) {
        return;
      }

      // check if double tap
      if(prev && prev.name == 'tap' &&
        (ev.timeStamp - prev.lastEvent.timeStamp) < inst.options.doubletap_interval &&
        ev.distance < inst.options.doubletap_distance) {
        inst.trigger('doubletap', ev);
        did_doubletap = true;
      }

      // do a single tap
      if(!did_doubletap || inst.options.tap_always) {
        Hammer.detection.current.name = 'tap';
        inst.trigger(Hammer.detection.current.name, ev);
      }
    }
  }
};

/**
 * Touch
 * Called as first, tells the user has touched the screen
 * @events  touch
 */
Hammer.gestures.Touch = {
  name    : 'touch',
  index   : -Infinity,
  defaults: {
    // call preventDefault at touchstart, and makes the element blocking by
    // disabling the scrolling of the page, but it improves gestures like
    // transforming and dragging.
    // be careful with using this, it can be very annoying for users to be stuck
    // on the page
    prevent_default    : false,

    // disable mouse events, so only touch (or pen!) input triggers events
    prevent_mouseevents: false
  },
  handler : function touchGesture(ev, inst) {
    if(inst.options.prevent_mouseevents && ev.pointerType == Hammer.POINTER_MOUSE) {
      ev.stopDetect();
      return;
    }

    if(inst.options.prevent_default) {
      ev.preventDefault();
    }

    if(ev.eventType == Hammer.EVENT_START) {
      inst.trigger(this.name, ev);
    }
  }
};

/**
 * Transform
 * User want to scale or rotate with 2 fingers
 * @events  transform, pinch, pinchin, pinchout, rotate
 */
Hammer.gestures.Transform = {
  name     : 'transform',
  index    : 45,
  defaults : {
    // factor, no scale is 1, zoomin is to 0 and zoomout until higher then 1
    transform_min_scale   : 0.01,
    // rotation in degrees
    transform_min_rotation: 1,
    // prevent default browser behavior when two touches are on the screen
    // but it makes the element a blocking element
    // when you are using the transform gesture, it is a good practice to set this true
    transform_always_block: false
  },
  triggered: false,
  handler  : function transformGesture(ev, inst) {
    // current gesture isnt drag, but dragged is true
    // this means an other gesture is busy. now call dragend
    if(Hammer.detection.current.name != this.name && this.triggered) {
      inst.trigger(this.name + 'end', ev);
      this.triggered = false;
      return;
    }

    // atleast multitouch
    if(ev.touches.length < 2) {
      return;
    }

    // prevent default when two fingers are on the screen
    if(inst.options.transform_always_block) {
      ev.preventDefault();
    }

    switch(ev.eventType) {
      case Hammer.EVENT_START:
        this.triggered = false;
        break;

      case Hammer.EVENT_MOVE:
        var scale_threshold = Math.abs(1 - ev.scale);
        var rotation_threshold = Math.abs(ev.rotation);

        // when the distance we moved is too small we skip this gesture
        // or we can be already in dragging
        if(scale_threshold < inst.options.transform_min_scale &&
          rotation_threshold < inst.options.transform_min_rotation) {
          return;
        }

        // we are transforming!
        Hammer.detection.current.name = this.name;

        // first time, trigger dragstart event
        if(!this.triggered) {
          inst.trigger(this.name + 'start', ev);
          this.triggered = true;
        }

        inst.trigger(this.name, ev); // basic transform event

        // trigger rotate event
        if(rotation_threshold > inst.options.transform_min_rotation) {
          inst.trigger('rotate', ev);
        }

        // trigger pinch event
        if(scale_threshold > inst.options.transform_min_scale) {
          inst.trigger('pinch', ev);
          inst.trigger('pinch' + ((ev.scale < 1) ? 'in' : 'out'), ev);
        }
        break;

      case Hammer.EVENT_END:
        // trigger dragend
        if(this.triggered) {
          inst.trigger(this.name + 'end', ev);
        }

        this.triggered = false;
        break;
    }
  }
};

  // Based off Lo-Dash's excellent UMD wrapper (slightly modified) - https://github.com/bestiejs/lodash/blob/master/lodash.js#L5515-L5543
  // some AMD build optimizers, like r.js, check for specific condition patterns like the following:
  if(typeof define == 'function' && typeof define.amd == 'object' && define.amd) {
    // define as an anonymous module
    define('hammer',[],function() {
      return Hammer;
    });
    // check for `exports` after `define` in case a build optimizer adds an `exports` object
  }
  else if(typeof module === 'object' && typeof module.exports === 'object') {
    module.exports = Hammer;
  }
  else {
    window.Hammer = Hammer;
  }
})(this);
/*! jQuery plugin for Hammer.JS - v1.0.0 - 2014-01-02
 * http://eightmedia.github.com/hammer.js
 *
 * Copyright (c) 2014 Jorik Tangelder <j.tangelder@gmail.com>;
 * Licensed under the MIT license */(function(window, undefined) {
  

function setup(Hammer, $) {
  /**
   * bind dom events
   * this overwrites addEventListener
   * @param   {HTMLElement}   element
   * @param   {String}        eventTypes
   * @param   {Function}      handler
   */
  Hammer.event.bindDom = function(element, eventTypes, handler) {
    $(element).on(eventTypes, function(ev) {
      var data = ev.originalEvent || ev;

      if(data.pageX === undefined) {
        data.pageX = ev.pageX;
        data.pageY = ev.pageY;
      }

      if(!data.target) {
        data.target = ev.target;
      }

      if(data.which === undefined) {
        data.which = data.button;
      }

      if(!data.preventDefault) {
        data.preventDefault = ev.preventDefault;
      }

      if(!data.stopPropagation) {
        data.stopPropagation = ev.stopPropagation;
      }

      handler.call(this, data);
    });
  };

  /**
   * the methods are called by the instance, but with the jquery plugin
   * we use the jquery event methods instead.
   * @this    {Hammer.Instance}
   * @return  {jQuery}
   */
  Hammer.Instance.prototype.on = function(types, handler) {
    return $(this.element).on(types, handler);
  };
  Hammer.Instance.prototype.off = function(types, handler) {
    return $(this.element).off(types, handler);
  };


  /**
   * trigger events
   * this is called by the gestures to trigger an event like 'tap'
   * @this    {Hammer.Instance}
   * @param   {String}    gesture
   * @param   {Object}    eventData
   * @return  {jQuery}
   */
  Hammer.Instance.prototype.trigger = function(gesture, eventData) {
    var el = $(this.element);
    if(el.has(eventData.target).length) {
      el = $(eventData.target);
    }

    return el.trigger({
      type   : gesture,
      gesture: eventData
    });
  };


  /**
   * jQuery plugin
   * create instance of Hammer and watch for gestures,
   * and when called again you can change the options
   * @param   {Object}    [options={}]
   * @return  {jQuery}
   */
  $.fn.hammer = function(options) {
    return this.each(function() {
      var el = $(this);
      var inst = el.data('hammer');
      // start new hammer instance
      if(!inst) {
        el.data('hammer', new Hammer(this, options || {}));
      }
      // change the options
      else if(inst && options) {
        Hammer.utils.extend(inst.options, options);
      }
    });
  };
}

  // Based off Lo-Dash's excellent UMD wrapper (slightly modified) - https://github.com/bestiejs/lodash/blob/master/lodash.js#L5515-L5543
  // some AMD build optimizers, like r.js, check for specific condition patterns like the following:
  if(typeof define == 'function' && typeof define.amd == 'object' && define.amd) {
    // define as an anonymous module
    define('jqhammer',['hammer', 'jquery'], setup);
  
  }
  else {
    setup(window.Hammer, window.jQuery || window.Zepto);
  }
})(this);
/**
 * @preserve FastClick: polyfill to remove click delays on browsers with touch UIs.
 *
 * @version 1.0.1
 * @codingstandard ftlabs-jsv2
 * @copyright The Financial Times Limited [All Rights Reserved]
 * @license MIT License (see LICENSE.txt)
 */

/*jslint browser:true, node:true*/
/*global define, Event, Node*/


/**
 * Instantiate fast-clicking listeners on the specificed layer.
 *
 * @constructor
 * @param {Element} layer The layer to listen on
 * @param {Object} options The options to override the defaults
 */
function FastClick(layer, options) {
	
	var oldOnClick;

	options = options || {};

	/**
	 * Whether a click is currently being tracked.
	 *
	 * @type boolean
	 */
	this.trackingClick = false;


	/**
	 * Timestamp for when click tracking started.
	 *
	 * @type number
	 */
	this.trackingClickStart = 0;


	/**
	 * The element being tracked for a click.
	 *
	 * @type EventTarget
	 */
	this.targetElement = null;


	/**
	 * X-coordinate of touch start event.
	 *
	 * @type number
	 */
	this.touchStartX = 0;


	/**
	 * Y-coordinate of touch start event.
	 *
	 * @type number
	 */
	this.touchStartY = 0;


	/**
	 * ID of the last touch, retrieved from Touch.identifier.
	 *
	 * @type number
	 */
	this.lastTouchIdentifier = 0;


	/**
	 * Touchmove boundary, beyond which a click will be cancelled.
	 *
	 * @type number
	 */
	this.touchBoundary = options.touchBoundary || 10;


	/**
	 * The FastClick layer.
	 *
	 * @type Element
	 */
	this.layer = layer;

	/**
	 * The minimum time between tap(touchstart and touchend) events
	 *
	 * @type number
	 */
	this.tapDelay = options.tapDelay || 200;

	if (FastClick.notNeeded(layer)) {
		return;
	}

	// Some old versions of Android don't have Function.prototype.bind
	function bind(method, context) {
		return function() { return method.apply(context, arguments); };
	}


	var methods = ['onMouse', 'onClick', 'onTouchStart', 'onTouchMove', 'onTouchEnd', 'onTouchCancel'];
	var context = this;
	for (var i = 0, l = methods.length; i < l; i++) {
		context[methods[i]] = bind(context[methods[i]], context);
	}

	// Set up event handlers as required
	if (deviceIsAndroid) {
		layer.addEventListener('mouseover', this.onMouse, true);
		layer.addEventListener('mousedown', this.onMouse, true);
		layer.addEventListener('mouseup', this.onMouse, true);
	}

	layer.addEventListener('click', this.onClick, true);
	layer.addEventListener('touchstart', this.onTouchStart, false);
	layer.addEventListener('touchmove', this.onTouchMove, false);
	layer.addEventListener('touchend', this.onTouchEnd, false);
	layer.addEventListener('touchcancel', this.onTouchCancel, false);

	// Hack is required for browsers that don't support Event#stopImmediatePropagation (e.g. Android 2)
	// which is how FastClick normally stops click events bubbling to callbacks registered on the FastClick
	// layer when they are cancelled.
	if (!Event.prototype.stopImmediatePropagation) {
		layer.removeEventListener = function(type, callback, capture) {
			var rmv = Node.prototype.removeEventListener;
			if (type === 'click') {
				rmv.call(layer, type, callback.hijacked || callback, capture);
			} else {
				rmv.call(layer, type, callback, capture);
			}
		};

		layer.addEventListener = function(type, callback, capture) {
			var adv = Node.prototype.addEventListener;
			if (type === 'click') {
				adv.call(layer, type, callback.hijacked || (callback.hijacked = function(event) {
					if (!event.propagationStopped) {
						callback(event);
					}
				}), capture);
			} else {
				adv.call(layer, type, callback, capture);
			}
		};
	}

	// If a handler is already declared in the element's onclick attribute, it will be fired before
	// FastClick's onClick handler. Fix this by pulling out the user-defined handler function and
	// adding it as listener.
	if (typeof layer.onclick === 'function') {

		// Android browser on at least 3.2 requires a new reference to the function in layer.onclick
		// - the old one won't work if passed to addEventListener directly.
		oldOnClick = layer.onclick;
		layer.addEventListener('click', function(event) {
			oldOnClick(event);
		}, false);
		layer.onclick = null;
	}
}


/**
 * Android requires exceptions.
 *
 * @type boolean
 */
var deviceIsAndroid = navigator.userAgent.indexOf('Android') > 0;


/**
 * iOS requires exceptions.
 *
 * @type boolean
 */
var deviceIsIOS = /iP(ad|hone|od)/.test(navigator.userAgent);


/**
 * iOS 4 requires an exception for select elements.
 *
 * @type boolean
 */
var deviceIsIOS4 = deviceIsIOS && (/OS 4_\d(_\d)?/).test(navigator.userAgent);


/**
 * iOS 6.0(+?) requires the target element to be manually derived
 *
 * @type boolean
 */
var deviceIsIOSWithBadTarget = deviceIsIOS && (/OS ([6-9]|\d{2})_\d/).test(navigator.userAgent);


/**
 * Determine whether a given element requires a native click.
 *
 * @param {EventTarget|Element} target Target DOM element
 * @returns {boolean} Returns true if the element needs a native click
 */
FastClick.prototype.needsClick = function(target) {
	
	if (!target.nodeName){
		return false;
	}
	switch (target.nodeName.toLowerCase()) {

	// Don't send a synthetic click to disabled inputs (issue #62)
	case 'button':
	case 'select':
	case 'textarea':
		if (target.disabled) {
			return true;
		}

		break;
	case 'input':

		// File inputs need real clicks on iOS 6 due to a browser bug (issue #68)
		if ((deviceIsIOS && target.type === 'file') || target.disabled) {
			return true;
		}

		break;
	case 'label':
	case 'video':
		return true;
	}

	return (/\bneedsclick\b/).test(target.className);
};


/**
 * Determine whether a given element requires a call to focus to simulate click into element.
 *
 * @param {EventTarget|Element} target Target DOM element
 * @returns {boolean} Returns true if the element requires a call to focus to simulate native click.
 */
FastClick.prototype.needsFocus = function(target) {
	
	switch (target.nodeName.toLowerCase()) {
	case 'textarea':
		return true;
	case 'select':
		return !deviceIsAndroid;
	case 'input':
		switch (target.type) {
		case 'button':
		case 'checkbox':
		case 'file':
		case 'image':
		case 'radio':
		case 'submit':
			return false;
		}

		// No point in attempting to focus disabled inputs
		return !target.disabled && !target.readOnly;
	default:
		return (/\bneedsfocus\b/).test(target.className);
	}
};


/**
 * Send a click event to the specified element.
 *
 * @param {EventTarget|Element} targetElement
 * @param {Event} event
 */
FastClick.prototype.sendClick = function(targetElement, event) {
	
	var clickEvent, touch;

	// On some Android devices activeElement needs to be blurred otherwise the synthetic click will have no effect (#24)
	if (document.activeElement && document.activeElement !== targetElement) {
		document.activeElement.blur();
	}

	touch = event.changedTouches[0];

	// Synthesise a click event, with an extra attribute so it can be tracked
	clickEvent = document.createEvent('MouseEvents');
	clickEvent.initMouseEvent(this.determineEventType(targetElement), true, true, window, 1, touch.screenX, touch.screenY, touch.clientX, touch.clientY, false, false, false, false, 0, null);
	clickEvent.forwardedTouchEvent = true;
	targetElement.dispatchEvent(clickEvent);
};

FastClick.prototype.determineEventType = function(targetElement) {
	
	if (!targetElement.tagName){
		return 'click';
	}
	//Issue #159: Android Chrome Select Box does not open with a synthetic click event
	if (deviceIsAndroid && targetElement.tagName.toLowerCase() === 'select') {
		return 'mousedown';
	}

	return 'click';
};


/**
 * @param {EventTarget|Element} targetElement
 */
FastClick.prototype.focus = function(targetElement) {
	
	var length;

	// Issue #160: on iOS 7, some input elements (e.g. date datetime) throw a vague TypeError on setSelectionRange. These elements don't have an integer value for the selectionStart and selectionEnd properties, but unfortunately that can't be used for detection because accessing the properties also throws a TypeError. Just check the type instead. Filed as Apple bug #15122724.
	if (deviceIsIOS && targetElement.setSelectionRange && targetElement.type.indexOf('date') !== 0 && targetElement.type !== 'time') {
		length = targetElement.value.length;
		targetElement.setSelectionRange(length, length);
	} else {
		targetElement.focus();
	}
};


/**
 * Check whether the given target element is a child of a scrollable layer and if so, set a flag on it.
 *
 * @param {EventTarget|Element} targetElement
 */
FastClick.prototype.updateScrollParent = function(targetElement) {
	
	var scrollParent, parentElement;

	scrollParent = targetElement.fastClickScrollParent;

	// Attempt to discover whether the target element is contained within a scrollable layer. Re-check if the
	// target element was moved to another parent.
	if (!scrollParent || !scrollParent.contains(targetElement)) {
		parentElement = targetElement;
		do {
			if (parentElement.scrollHeight > parentElement.offsetHeight) {
				scrollParent = parentElement;
				targetElement.fastClickScrollParent = parentElement;
				break;
			}

			parentElement = parentElement.parentElement;
		} while (parentElement);
	}

	// Always update the scroll top tracker if possible.
	if (scrollParent) {
		scrollParent.fastClickLastScrollTop = scrollParent.scrollTop;
	}
};


/**
 * @param {EventTarget} targetElement
 * @returns {Element|EventTarget}
 */
FastClick.prototype.getTargetElementFromEventTarget = function(eventTarget) {
	

	// On some older browsers (notably Safari on iOS 4.1 - see issue #56) the event target may be a text node.
	if (eventTarget.nodeType === Node.TEXT_NODE) {
		return eventTarget.parentNode;
	}

	return eventTarget;
};


/**
 * On touch start, record the position and scroll offset.
 *
 * @param {Event} event
 * @returns {boolean}
 */
FastClick.prototype.onTouchStart = function(event) {
	
	var targetElement, touch, selection;

	// Ignore multiple touches, otherwise pinch-to-zoom is prevented if both fingers are on the FastClick element (issue #111).
	if (event.targetTouches.length > 1) {
		return true;
	}

	targetElement = this.getTargetElementFromEventTarget(event.target);
	touch = event.targetTouches[0];

	if (deviceIsIOS) {

		// Only trusted events will deselect text on iOS (issue #49)
		selection = window.getSelection();
		if (selection.rangeCount && !selection.isCollapsed) {
			return true;
		}

		if (!deviceIsIOS4) {

			// Weird things happen on iOS when an alert or confirm dialog is opened from a click event callback (issue #23):
			// when the user next taps anywhere else on the page, new touchstart and touchend events are dispatched
			// with the same identifier as the touch event that previously triggered the click that triggered the alert.
			// Sadly, there is an issue on iOS 4 that causes some normal touch events to have the same identifier as an
			// immediately preceeding touch event (issue #52), so this fix is unavailable on that platform.
			if (touch.identifier === this.lastTouchIdentifier) {
				event.preventDefault();
				return false;
			}

			this.lastTouchIdentifier = touch.identifier;

			// If the target element is a child of a scrollable layer (using -webkit-overflow-scrolling: touch) and:
			// 1) the user does a fling scroll on the scrollable layer
			// 2) the user stops the fling scroll with another tap
			// then the event.target of the last 'touchend' event will be the element that was under the user's finger
			// when the fling scroll was started, causing FastClick to send a click event to that layer - unless a check
			// is made to ensure that a parent layer was not scrolled before sending a synthetic click (issue #42).
			this.updateScrollParent(targetElement);
		}
	}

	this.trackingClick = true;
	this.trackingClickStart = event.timeStamp;
	this.targetElement = targetElement;

	this.touchStartX = touch.pageX;
	this.touchStartY = touch.pageY;

	// Prevent phantom clicks on fast double-tap (issue #36)
	if ((event.timeStamp - this.lastClickTime) < this.tapDelay) {
		event.preventDefault();
	}

	return true;
};


/**
 * Based on a touchmove event object, check whether the touch has moved past a boundary since it started.
 *
 * @param {Event} event
 * @returns {boolean}
 */
FastClick.prototype.touchHasMoved = function(event) {
	
	var touch = event.changedTouches[0], boundary = this.touchBoundary;

	if (Math.abs(touch.pageX - this.touchStartX) > boundary || Math.abs(touch.pageY - this.touchStartY) > boundary) {
		return true;
	}

	return false;
};


/**
 * Update the last position.
 *
 * @param {Event} event
 * @returns {boolean}
 */
FastClick.prototype.onTouchMove = function(event) {
	
	if (!this.trackingClick) {
		return true;
	}

	// If the touch has moved, cancel the click tracking
	if (this.targetElement !== this.getTargetElementFromEventTarget(event.target) || this.touchHasMoved(event)) {
		this.trackingClick = false;
		this.targetElement = null;
	}

	return true;
};


/**
 * Attempt to find the labelled control for the given label element.
 *
 * @param {EventTarget|HTMLLabelElement} labelElement
 * @returns {Element|null}
 */
FastClick.prototype.findControl = function(labelElement) {
	

	// Fast path for newer browsers supporting the HTML5 control attribute
	if (labelElement.control !== undefined) {
		return labelElement.control;
	}

	// All browsers under test that support touch events also support the HTML5 htmlFor attribute
	if (labelElement.htmlFor) {
		return document.getElementById(labelElement.htmlFor);
	}

	// If no for attribute exists, attempt to retrieve the first labellable descendant element
	// the list of which is defined here: http://www.w3.org/TR/html5/forms.html#category-label
	return labelElement.querySelector('button, input:not([type=hidden]), keygen, meter, output, progress, select, textarea');
};


/**
 * On touch end, determine whether to send a click event at once.
 *
 * @param {Event} event
 * @returns {boolean}
 */
FastClick.prototype.onTouchEnd = function(event) {
	
	var forElement, trackingClickStart, targetTagName, scrollParent, touch, targetElement = this.targetElement;

	if (!this.trackingClick) {
		return true;
	}
	if (! targetElement.tagName){
		return false;
	}
	// Prevent phantom clicks on fast double-tap (issue #36)
	if ((event.timeStamp - this.lastClickTime) < this.tapDelay) {
		this.cancelNextClick = true;
		return true;
	}

	// Reset to prevent wrong click cancel on input (issue #156).
	this.cancelNextClick = false;

	this.lastClickTime = event.timeStamp;

	trackingClickStart = this.trackingClickStart;
	this.trackingClick = false;
	this.trackingClickStart = 0;

	// On some iOS devices, the targetElement supplied with the event is invalid if the layer
	// is performing a transition or scroll, and has to be re-detected manually. Note that
	// for this to function correctly, it must be called *after* the event target is checked!
	// See issue #57; also filed as rdar://13048589 .
	if (deviceIsIOSWithBadTarget) {
		touch = event.changedTouches[0];

		// In certain cases arguments of elementFromPoint can be negative, so prevent setting targetElement to null
		targetElement = document.elementFromPoint(touch.pageX - window.pageXOffset, touch.pageY - window.pageYOffset) || targetElement;
		targetElement.fastClickScrollParent = this.targetElement.fastClickScrollParent;
	}

	targetTagName = targetElement.tagName.toLowerCase();
	if (targetTagName === 'label') {
		forElement = this.findControl(targetElement);
		if (forElement) {
			this.focus(targetElement);
			if (deviceIsAndroid) {
				return false;
			}

			targetElement = forElement;
		}
	} else if (this.needsFocus(targetElement)) {

		// Case 1: If the touch started a while ago (best guess is 100ms based on tests for issue #36) then focus will be triggered anyway. Return early and unset the target element reference so that the subsequent click will be allowed through.
		// Case 2: Without this exception for input elements tapped when the document is contained in an iframe, then any inputted text won't be visible even though the value attribute is updated as the user types (issue #37).
		if ((event.timeStamp - trackingClickStart) > 100 || (deviceIsIOS && window.top !== window && targetTagName === 'input')) {
			this.targetElement = null;
			return false;
		}

		this.focus(targetElement);
		this.sendClick(targetElement, event);

		// Select elements need the event to go through on iOS 4, otherwise the selector menu won't open.
		// Also this breaks opening selects when VoiceOver is active on iOS6, iOS7 (and possibly others)
		if (!deviceIsIOS || targetTagName !== 'select') {
			this.targetElement = null;
			event.preventDefault();
		}

		return false;
	}

	if (deviceIsIOS && !deviceIsIOS4) {

		// Don't send a synthetic click event if the target element is contained within a parent layer that was scrolled
		// and this tap is being used to stop the scrolling (usually initiated by a fling - issue #42).
		scrollParent = targetElement.fastClickScrollParent;
		if (scrollParent && scrollParent.fastClickLastScrollTop !== scrollParent.scrollTop) {
			return true;
		}
	}

	// Prevent the actual click from going though - unless the target node is marked as requiring
	// real clicks or if it is in the whitelist in which case only non-programmatic clicks are permitted.
	if (!this.needsClick(targetElement)) {
		event.preventDefault();
		this.sendClick(targetElement, event);
	}

	return false;
};


/**
 * On touch cancel, stop tracking the click.
 *
 * @returns {void}
 */
FastClick.prototype.onTouchCancel = function() {
	
	this.trackingClick = false;
	this.targetElement = null;
};


/**
 * Determine mouse events which should be permitted.
 *
 * @param {Event} event
 * @returns {boolean}
 */
FastClick.prototype.onMouse = function(event) {
	

	// If a target element was never set (because a touch event was never fired) allow the event
	if (!this.targetElement) {
		return true;
	}

	if (event.forwardedTouchEvent) {
		return true;
	}

	// Programmatically generated events targeting a specific element should be permitted
	if (!event.cancelable) {
		return true;
	}

	// Derive and check the target element to see whether the mouse event needs to be permitted;
	// unless explicitly enabled, prevent non-touch click events from triggering actions,
	// to prevent ghost/doubleclicks.
	if (!this.needsClick(this.targetElement) || this.cancelNextClick) {

		// Prevent any user-added listeners declared on FastClick element from being fired.
		if (event.stopImmediatePropagation) {
			event.stopImmediatePropagation();
		} else {

			// Part of the hack for browsers that don't support Event#stopImmediatePropagation (e.g. Android 2)
			event.propagationStopped = true;
		}

		// Cancel the event
		event.stopPropagation();
		event.preventDefault();

		return false;
	}

	// If the mouse event is permitted, return true for the action to go through.
	return true;
};


/**
 * On actual clicks, determine whether this is a touch-generated click, a click action occurring
 * naturally after a delay after a touch (which needs to be cancelled to avoid duplication), or
 * an actual click which should be permitted.
 *
 * @param {Event} event
 * @returns {boolean}
 */
FastClick.prototype.onClick = function(event) {
	
	var permitted;

	// It's possible for another FastClick-like library delivered with third-party code to fire a click event before FastClick does (issue #44). In that case, set the click-tracking flag back to false and return early. This will cause onTouchEnd to return early.
	if (this.trackingClick) {
		this.targetElement = null;
		this.trackingClick = false;
		return true;
	}

	// Very odd behaviour on iOS (issue #18): if a submit element is present inside a form and the user hits enter in the iOS simulator or clicks the Go button on the pop-up OS keyboard the a kind of 'fake' click event will be triggered with the submit-type input element as the target.
	if (event.target.type === 'submit' && event.detail === 0) {
		return true;
	}

	permitted = this.onMouse(event);

	// Only unset targetElement if the click is not permitted. This will ensure that the check for !targetElement in onMouse fails and the browser's click doesn't go through.
	if (!permitted) {
		this.targetElement = null;
	}

	// If clicks are permitted, return true for the action to go through.
	return permitted;
};


/**
 * Remove all FastClick's event listeners.
 *
 * @returns {void}
 */
FastClick.prototype.destroy = function() {
	
	var layer = this.layer;

	if (deviceIsAndroid) {
		layer.removeEventListener('mouseover', this.onMouse, true);
		layer.removeEventListener('mousedown', this.onMouse, true);
		layer.removeEventListener('mouseup', this.onMouse, true);
	}

	layer.removeEventListener('click', this.onClick, true);
	layer.removeEventListener('touchstart', this.onTouchStart, false);
	layer.removeEventListener('touchmove', this.onTouchMove, false);
	layer.removeEventListener('touchend', this.onTouchEnd, false);
	layer.removeEventListener('touchcancel', this.onTouchCancel, false);
};


/**
 * Check whether FastClick is needed.
 *
 * @param {Element} layer The layer to listen on
 */
FastClick.notNeeded = function(layer) {
	
	var metaViewport;
	var chromeVersion;

	// Devices that don't support touch don't need FastClick
	if (typeof window.ontouchstart === 'undefined') {
		return true;
	}

	// Chrome version - zero for other browsers
	chromeVersion = +(/Chrome\/([0-9]+)/.exec(navigator.userAgent) || [,0])[1];

	if (chromeVersion) {

		if (deviceIsAndroid) {
			metaViewport = document.querySelector('meta[name=viewport]');

			if (metaViewport) {
				// Chrome on Android with user-scalable="no" doesn't need FastClick (issue #89)
				if (metaViewport.content.indexOf('user-scalable=no') !== -1) {
					return true;
				}
				// Chrome 32 and above with width=device-width or less don't need FastClick
				if (chromeVersion > 31 && window.innerWidth <= window.screen.width) {
					return true;
				}
			}

		// Chrome desktop doesn't need FastClick (issue #15)
		} else {
			return true;
		}
	}

	// IE10 with -ms-touch-action: none, which disables double-tap-to-zoom (issue #97)
	if (layer.style.msTouchAction === 'none') {
		return true;
	}

	return false;
};


/**
 * Factory method for creating a FastClick object
 *
 * @param {Element} layer The layer to listen on
 * @param {Object} options The options to override the defaults
 */
FastClick.attach = function(layer, options) {
	
	return new FastClick(layer, options);
};


if (typeof define !== 'undefined' && define.amd) {

	// AMD. Register as an anonymous module.
	define('fastclick',[],function() {
		
		return FastClick;
	});
} else if (typeof module !== 'undefined' && module.exports) {
	module.exports = FastClick.attach;
	module.exports.FastClick = FastClick;
} else {
	window.FastClick = FastClick;
}
;

define("hgn!templates/index", ["hogan"], function(hogan){  var tmpl = new hogan.Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("		<nav class=\"navbar navbar-inverse navbar-fixed-top\" role=\"navigation\">\r");t.b("\n" + i);t.b("              <ul class=\"nav navbar-nav\">\r");t.b("\n" + i);t.b("                    <li class=\"click-active\"><a id=\"menu\"><span class=\"font-h4 wmicon icon-menu\"></span></a></li>\r");t.b("\n" + i);t.b("                    <li class=\"click-active\"><a id=\"back\"><span class=\"font-h4 wmicon icon-left-open-big\"></span></a></li>\r");t.b("\n" + i);t.b("                    <li><p class=\"navbar-text navbar-text-color font-h4\" id=\"viewName\"></p></li>\r");t.b("\n" + i);t.b("              </ul>\r");t.b("\n" + i);t.b("              <ul class=\"nav navbar-nav navbar-right refresh\">\r");t.b("\n" + i);t.b("                    <li class=\"click-active\"><a id=\"refresh\"><span class=\"font-h4 glyphicon glyphicon-refresh animate-spin\"></span></a></li>\r");t.b("\n" + i);t.b("              </ul>\r");t.b("\n" + i);t.b("		</nav>\r");t.b("\n" + i);t.b("		<nav class=\"cbp-spmenu cbp-spmenu-vertical cbp-spmenu-left navbar-inverted \" id=\"cbp-spmenu-s1\">\r");t.b("\n" + i);t.b("            <ul class=\"menu-list\">\r");t.b("\n" + i);t.b("                <li class=\"private-menu-item\"><a href=\"#home/overview\" class=\"nav-home\">");t.b(t.v(t.d("Strings.COMMON.HOME",c,p,0)));t.b("</a></li>\r");t.b("\n" + i);t.b("                <li class=\"private-menu-item\"><a href=\"#profile\" class=\"nav-profile\">");t.b(t.v(t.d("Strings.PLAYER_DETAILS.MY_PROFILE",c,p,0)));t.b("</a></li>\r");t.b("\n" + i);t.b("                <li class=\"private-menu-item\"><a href=\"#matches/second\" class=\"nav-matches\">");t.b(t.v(t.d("Strings.MATCH.TITLE",c,p,0)));t.b("</a></li>\r");t.b("\n" + i);t.b("                <li class=\"private-menu-item\"><a href=\"#groups\" class=\"nav-groups\">");t.b(t.v(t.d("Strings.GROUPS.TITLE",c,p,0)));t.b("</a></li>\r");t.b("\n" + i);t.b("                <li class=\"private-menu-item\"><a href=\"#playersearch\" class=\"nav-players\">");t.b(t.v(t.d("Strings.COMMON.PLAYERS",c,p,0)));t.b("</a></li>\r");t.b("\n" + i);t.b("                <li class=\"private-menu-item\"><a href=\"#home/ranking\" class=\"nav-rankings\">");t.b(t.v(t.d("Strings.COMMON.RANKING",c,p,0)));t.b("</a></li>\r");t.b("\n" + i);t.b("                <li class=\"private-menu-item\"><a href=\"#bonustips\" class=\"nav-bonus\">");t.b(t.v(t.d("Strings.COMMON.BONUS_TIPS",c,p,0)));t.b("</a></li>\r");t.b("\n" + i);t.b("                <li class=\"private-menu-item\"><a href=\"#messages\" class=\"nav-messages\">");t.b(t.v(t.d("Strings.MESSAGES.TITLE",c,p,0)));t.b("<span id='new-messages'></span></a></li>\r");t.b("\n" + i);t.b("                <br/><br/>\r");t.b("\n" + i);t.b("                <li class=\"private-menu-item\"><a href=\"#inviteext\" id=\"invite\" class=\"nav-invite\">");t.b(t.v(t.d("Strings.INVITE_TO_MY_GROUP.INVITE_EXT",c,p,0)));t.b("</a></li>\r");t.b("\n" + i);t.b("                <li class=\"private-menu-item\"><a href=\"#feedback\" class=\"nav-feedback\">");t.b(t.v(t.d("Strings.HOMEOVERVIEW.FEEDBACK",c,p,0)));t.b("</a></li>\r");t.b("\n" + i);t.b("                <li class=\"\"><a href=\"#help\" class=\"nav-help\">");t.b(t.v(t.d("Strings.COMMON.HELP",c,p,0)));t.b("</a></li>\r");t.b("\n" + i);t.b("                <li class=\"\"><a href=\"#about\" id=\"about\" class=\"nav-about\">");t.b(t.v(t.d("Strings.ABOUT.ABOUT",c,p,0)));t.b("</a></li>\r");t.b("\n" + i);t.b("                <li class=\"\"><a href=\"#login\" id=\"login\" class=\"nav-login\">");t.b(t.v(t.d("Strings.LOGIN.LOGIN",c,p,0)));t.b("</a></li>\r");t.b("\n" + i);t.b("                <li class=\"private-menu-item\"><a href=\"#login\" id=\"logoff\" class=\"nav-logout\">");t.b(t.v(t.d("Strings.COMMON.LOGOUT",c,p,0)));t.b("</a></li>\r");t.b("\n" + i);t.b("                <li class=\"\"><a href=\"#\" id=\"exit\" class=\"nav-logout\">");t.b(t.v(t.d("Strings.COMMON.EXIT",c,p,0)));t.b("</a></li>\r");t.b("\n" + i);t.b("            </ul>\r");t.b("\n" + i);t.b("		</nav>\r");t.b("\n" + i);t.b("		\r");t.b("\n" + i);t.b("		<section class=\"container-liquid\" id=\"wmapp\"></section>\r");t.b("\n" + i);t.b("		<section id=\"messages\" class=\"hidden alert\"></section>\r");t.b("\n" + i);t.b("		<section id=\"popup-messages-backdrop\" style=\"display:none\"></section>\r");t.b("\n" + i);t.b("		<section id=\"popup-messages\" style=\"display:none\"></section>");return t.fl(); },partials: {}, subs: {  }}, "", hogan);  function render(){ return tmpl.render.apply(tmpl, arguments); } render.template = tmpl; return render;});

// moment.js language configuration
// language : german (de)
// author : lluchs : https://github.com/lluchs
// author: Menelion Elensúle: https://github.com/Oire

(function (factory) {
    if (typeof define === 'function' && define.amd) {
        define('moment/lang/de',['moment'], factory); // AMD
    } else if (typeof exports === 'object') {
        module.exports = factory(require('../moment')); // Node
    } else {
        factory(window.moment); // Browser global
    }
}(function (moment) {
    function processRelativeTime(number, withoutSuffix, key, isFuture) {
        var format = {
            'm': ['eine Minute', 'einer Minute'],
            'h': ['eine Stunde', 'einer Stunde'],
            'd': ['ein Tag', 'einem Tag'],
            'dd': [number + ' Tage', number + ' Tagen'],
            'M': ['ein Monat', 'einem Monat'],
            'MM': [number + ' Monate', number + ' Monaten'],
            'y': ['ein Jahr', 'einem Jahr'],
            'yy': [number + ' Jahre', number + ' Jahren']
        };
        return withoutSuffix ? format[key][0] : format[key][1];
    }

    return moment.lang('de', {
        months : "Januar_Februar_März_April_Mai_Juni_Juli_August_September_Oktober_November_Dezember".split("_"),
        monthsShort : "Jan._Febr._Mrz._Apr._Mai_Jun._Jul._Aug._Sept._Okt._Nov._Dez.".split("_"),
        weekdays : "Sonntag_Montag_Dienstag_Mittwoch_Donnerstag_Freitag_Samstag".split("_"),
        weekdaysShort : "So._Mo._Di._Mi._Do._Fr._Sa.".split("_"),
        weekdaysMin : "So_Mo_Di_Mi_Do_Fr_Sa".split("_"),
        longDateFormat : {
            LT: "H:mm [Uhr]",
            L : "DD.MM.YYYY",
            LL : "D. MMMM YYYY",
            LLL : "D. MMMM YYYY LT",
            LLLL : "dddd, D. MMMM YYYY LT"
        },
        calendar : {
            sameDay: "[Heute um] LT",
            sameElse: "L",
            nextDay: '[Morgen um] LT',
            nextWeek: 'dddd [um] LT',
            lastDay: '[Gestern um] LT',
            lastWeek: '[letzten] dddd [um] LT'
        },
        relativeTime : {
            future : "in %s",
            past : "vor %s",
            s : "ein paar Sekunden",
            m : processRelativeTime,
            mm : "%d Minuten",
            h : processRelativeTime,
            hh : "%d Stunden",
            d : processRelativeTime,
            dd : processRelativeTime,
            M : processRelativeTime,
            MM : processRelativeTime,
            y : processRelativeTime,
            yy : processRelativeTime
        },
        ordinal : '%d.',
        week : {
            dow : 1, // Monday is the first day of the week.
            doy : 4  // The week that contains Jan 4th is the first week of the year.
        }
    });
}));

// moment.js language configuration
// language : italian (it)
// author : Lorenzo : https://github.com/aliem
// author: Mattia Larentis: https://github.com/nostalgiaz

(function (factory) {
    if (typeof define === 'function' && define.amd) {
        define('moment/lang/it',['moment'], factory); // AMD
    } else if (typeof exports === 'object') {
        module.exports = factory(require('../moment')); // Node
    } else {
        factory(window.moment); // Browser global
    }
}(function (moment) {
    return moment.lang('it', {
        months : "Gennaio_Febbraio_Marzo_Aprile_Maggio_Giugno_Luglio_Agosto_Settembre_Ottobre_Novembre_Dicembre".split("_"),
        monthsShort : "Gen_Feb_Mar_Apr_Mag_Giu_Lug_Ago_Set_Ott_Nov_Dic".split("_"),
        weekdays : "Domenica_Lunedì_Martedì_Mercoledì_Giovedì_Venerdì_Sabato".split("_"),
        weekdaysShort : "Dom_Lun_Mar_Mer_Gio_Ven_Sab".split("_"),
        weekdaysMin : "D_L_Ma_Me_G_V_S".split("_"),
        longDateFormat : {
            LT : "HH:mm",
            L : "DD/MM/YYYY",
            LL : "D MMMM YYYY",
            LLL : "D MMMM YYYY LT",
            LLLL : "dddd, D MMMM YYYY LT"
        },
        calendar : {
            sameDay: '[Oggi alle] LT',
            nextDay: '[Domani alle] LT',
            nextWeek: 'dddd [alle] LT',
            lastDay: '[Ieri alle] LT',
            lastWeek: '[lo scorso] dddd [alle] LT',
            sameElse: 'L'
        },
        relativeTime : {
            future : function (s) {
                return ((/^[0-9].+$/).test(s) ? "tra" : "in") + " " + s;
            },
            past : "%s fa",
            s : "secondi",
            m : "un minuto",
            mm : "%d minuti",
            h : "un'ora",
            hh : "%d ore",
            d : "un giorno",
            dd : "%d giorni",
            M : "un mese",
            MM : "%d mesi",
            y : "un anno",
            yy : "%d anni"
        },
        ordinal: '%dº',
        week : {
            dow : 1, // Monday is the first day of the week.
            doy : 4  // The week that contains Jan 4th is the first week of the year.
        }
    });
}));

// moment.js language configuration
// language : spanish (es)
// author : Julio Napurí : https://github.com/julionc

(function (factory) {
    if (typeof define === 'function' && define.amd) {
        define('moment/lang/es',['moment'], factory); // AMD
    } else if (typeof exports === 'object') {
        module.exports = factory(require('../moment')); // Node
    } else {
        factory(window.moment); // Browser global
    }
}(function (moment) {
    return moment.lang('es', {
        months : "enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre".split("_"),
        monthsShort : "ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.".split("_"),
        weekdays : "domingo_lunes_martes_miércoles_jueves_viernes_sábado".split("_"),
        weekdaysShort : "dom._lun._mar._mié._jue._vie._sáb.".split("_"),
        weekdaysMin : "Do_Lu_Ma_Mi_Ju_Vi_Sá".split("_"),
        longDateFormat : {
            LT : "H:mm",
            L : "DD/MM/YYYY",
            LL : "D [de] MMMM [de] YYYY",
            LLL : "D [de] MMMM [de] YYYY LT",
            LLLL : "dddd, D [de] MMMM [de] YYYY LT"
        },
        calendar : {
            sameDay : function () {
                return '[hoy a la' + ((this.hours() !== 1) ? 's' : '') + '] LT';
            },
            nextDay : function () {
                return '[mañana a la' + ((this.hours() !== 1) ? 's' : '') + '] LT';
            },
            nextWeek : function () {
                return 'dddd [a la' + ((this.hours() !== 1) ? 's' : '') + '] LT';
            },
            lastDay : function () {
                return '[ayer a la' + ((this.hours() !== 1) ? 's' : '') + '] LT';
            },
            lastWeek : function () {
                return '[el] dddd [pasado a la' + ((this.hours() !== 1) ? 's' : '') + '] LT';
            },
            sameElse : 'L'
        },
        relativeTime : {
            future : "en %s",
            past : "hace %s",
            s : "unos segundos",
            m : "un minuto",
            mm : "%d minutos",
            h : "una hora",
            hh : "%d horas",
            d : "un día",
            dd : "%d días",
            M : "un mes",
            MM : "%d meses",
            y : "un año",
            yy : "%d años"
        },
        ordinal : '%dº',
        week : {
            dow : 1, // Monday is the first day of the week.
            doy : 4  // The week that contains Jan 4th is the first week of the year.
        }
    });
}));

// moment.js language configuration
// language : french (fr)
// author : John Fischer : https://github.com/jfroffice

(function (factory) {
    if (typeof define === 'function' && define.amd) {
        define('moment/lang/fr',['moment'], factory); // AMD
    } else if (typeof exports === 'object') {
        module.exports = factory(require('../moment')); // Node
    } else {
        factory(window.moment); // Browser global
    }
}(function (moment) {
    return moment.lang('fr', {
        months : "janvier_février_mars_avril_mai_juin_juillet_août_septembre_octobre_novembre_décembre".split("_"),
        monthsShort : "janv._févr._mars_avr._mai_juin_juil._août_sept._oct._nov._déc.".split("_"),
        weekdays : "dimanche_lundi_mardi_mercredi_jeudi_vendredi_samedi".split("_"),
        weekdaysShort : "dim._lun._mar._mer._jeu._ven._sam.".split("_"),
        weekdaysMin : "Di_Lu_Ma_Me_Je_Ve_Sa".split("_"),
        longDateFormat : {
            LT : "HH:mm",
            L : "DD/MM/YYYY",
            LL : "D MMMM YYYY",
            LLL : "D MMMM YYYY LT",
            LLLL : "dddd D MMMM YYYY LT"
        },
        calendar : {
            sameDay: "[Aujourd'hui à] LT",
            nextDay: '[Demain à] LT',
            nextWeek: 'dddd [à] LT',
            lastDay: '[Hier à] LT',
            lastWeek: 'dddd [dernier à] LT',
            sameElse: 'L'
        },
        relativeTime : {
            future : "dans %s",
            past : "il y a %s",
            s : "quelques secondes",
            m : "une minute",
            mm : "%d minutes",
            h : "une heure",
            hh : "%d heures",
            d : "un jour",
            dd : "%d jours",
            M : "un mois",
            MM : "%d mois",
            y : "un an",
            yy : "%d ans"
        },
        ordinal : function (number) {
            return number + (number === 1 ? 'er' : '');
        },
        week : {
            dow : 1, // Monday is the first day of the week.
            doy : 4  // The week that contains Jan 4th is the first week of the year.
        }
    });
}));

// moment.js language configuration
// language : great britain english (en-gb)
// author : Chris Gedrim : https://github.com/chrisgedrim

(function (factory) {
    if (typeof define === 'function' && define.amd) {
        define('moment/lang/en-gb',['moment'], factory); // AMD
    } else if (typeof exports === 'object') {
        module.exports = factory(require('../moment')); // Node
    } else {
        factory(window.moment); // Browser global
    }
}(function (moment) {
    return moment.lang('en-gb', {
        months : "January_February_March_April_May_June_July_August_September_October_November_December".split("_"),
        monthsShort : "Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),
        weekdays : "Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),
        weekdaysShort : "Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),
        weekdaysMin : "Su_Mo_Tu_We_Th_Fr_Sa".split("_"),
        longDateFormat : {
            LT : "HH:mm",
            L : "DD/MM/YYYY",
            LL : "D MMMM YYYY",
            LLL : "D MMMM YYYY LT",
            LLLL : "dddd, D MMMM YYYY LT"
        },
        calendar : {
            sameDay : '[Today at] LT',
            nextDay : '[Tomorrow at] LT',
            nextWeek : 'dddd [at] LT',
            lastDay : '[Yesterday at] LT',
            lastWeek : '[Last] dddd [at] LT',
            sameElse : 'L'
        },
        relativeTime : {
            future : "in %s",
            past : "%s ago",
            s : "a few seconds",
            m : "a minute",
            mm : "%d minutes",
            h : "an hour",
            hh : "%d hours",
            d : "a day",
            dd : "%d days",
            M : "a month",
            MM : "%d months",
            y : "a year",
            yy : "%d years"
        },
        ordinal : function (number) {
            var b = number % 10,
                output = (~~ (number % 100 / 10) === 1) ? 'th' :
                (b === 1) ? 'st' :
                (b === 2) ? 'nd' :
                (b === 3) ? 'rd' : 'th';
            return number + output;
        },
        week : {
            dow : 1, // Monday is the first day of the week.
            doy : 4  // The week that contains Jan 4th is the first week of the year.
        }
    });
}));

/*global require*/

// Require.js allows us to configure shortcut alias


define('main',[
	'jquery',
	'backbone',
	'routers/router',
	'util/context',
	'models/settings',
	'util/wait',
	'util/message',
	'models/player',
	'collections/results',
	'collections/tips',
	'jqhammer',
	'fastclick',
	'util/feedback',
	'hgn!templates/index',
	"i18n!nls/strings",
	'moment',
	'moment/lang/de',
	'moment/lang/it',
	'moment/lang/es',
	'moment/lang/fr',
	'moment/lang/de',
	'moment/lang/en-gb'
], function ($,Backbone,Workspace,Context,Settings,Wait,Message,PlayerModel,Results,Tips, Hammer,
	FastClick,Feedback,IndexTemplate,Strings, moment) {
	
return {start:function(){

var oldGetCacheKey=Backbone.fetchCache.getCacheKey;
	Backbone.fetchCache.getCacheKey = function(instance, options) {
  		var s=oldGetCacheKey(instance,options);
		var x=s.replace(/_it=[0-9]*&?/,"");
		console.log(s+"--->"+x)
		return x;
	};

	$("body").append(IndexTemplate({Strings:Strings}));

	if (!window.device || device.platform !='Android'){
		$("#exit").hide();
	}
	
	if(( document.URL.indexOf( 'http://' ) > -1 || document.URL.indexOf( 'https://' ) > -1)&&
	 navigator.userAgent.toLowerCase().indexOf('chrome') === -1 /*&& navigator.userAgent.toLowerCase().indexOf('firefox') === -1*/){
	 	//Web and not chrome
		$("#unsupported").show();
	}


	FastClick.attach(document.body);
	moment.lang(navigator.language);


	$(document).hammer().on("swipeleft", function(direction) {
		if ($(".cbp-spmenu-open").length === 0) {
			//this causes back during text selection with the mouse when mouse moved too far avay from the text control
		//	window.history.back();
		} else {
			$("#cbp-spmenu-s1").removeClass("cbp-spmenu-open");
		}
	});

	$(document).on("menubutton",function(){
		$("#cbp-spmenu-s1").toggleClass("cbp-spmenu-open");
	});
	var exitWaiting=false;
	$(document).on("backbutton",function(e){
        e.preventDefault();
		if (window.location.hash.indexOf("#home")===0){
			
            if (exitWaiting){
	    	  /*navigator.notification.confirm("Are you sure you want to exit ?",
    	    	 function(button){
    	    	 	if(button==2){
    			        return;
    			    }else{
    			        navigator.app.exitApp();
    			    }
    	    	 }, 
	    		"Confirmation", "Yes,No"); */
				exitWaiting=false;
				navigator.app.exitApp();
            }else{
                window.plugins.toast.showShortTop(Strings.COMMON.ONCE_MORE_TO_EXIT);
                exitWaiting=true;
                setTimeout(function(){exitWaiting=false;},3000);
            }
    	}else{
    		console.log("going back");
    		window.history.back();
    		//navigator.app.backHistory()
    	}
	});
	$(document).hammer().on("swiperight", function(direction) {
		$("#cbp-spmenu-s1").addClass("cbp-spmenu-open");
		Context.updateMessagesCount();
	});

	$("#back").on("click",function(){
		window.history.back();
	});
	
	$("#back2").on("click",function(){
		window.history.back();
	});
	$("#messages").on("click",function(){
		Message.hide();
	});
	$("#menu").on("click",function(event){
		event.stopPropagation();
		$("#cbp-spmenu-s1").toggleClass("cbp-spmenu-open");
		Context.updateMessagesCount();
	});
	$("#refresh").on("click",function(event){
		Wait.forceStop();
		if (Context.currentView && Context.currentView.refresh){
			Context.currentView.refresh();
		}

	});
	$("#cbp-spmenu-s1 a").on("click",function(){
		$("#cbp-spmenu-s1").removeClass("cbp-spmenu-open");
	});

	$("body").on("click",function(event){

		$("#cbp-spmenu-s1").removeClass("cbp-spmenu-open");
		
	});
	$("#logoff").on("click",function(){
		Context.logoff();
	});
	$("#exit").on("click",function(){
		navigator.app.exitApp();
	});
	var lastFocused=null,
		focusSet=null;
	$(document).on("showkeyboard",function(){
		console.log("show keyboard")
		setTimeout(function(){
				console.log("focused scrolling")
				document.activeElement.scrollIntoView(false)
			},500)
		lastFocused=document.activeElement;
		focusSet=new Date().getTime();
	})

	$(document).on("blur",'select',function(event){
			console.log("focusLost")
			if (new Date().getTime()-focusSet<1000){
				console.log("resetfocus")
				$(lastFocused).focus();
			}
	})
		
	$(document).on("focus",'select',function(event){
			console.log("focusGot")
			lastFocused=event.target;
			focusSet=new Date().getTime();
		}
	)

	$(window).on('hashchange',function( event){
		console.log("Hash change from :"+event.originalEvent.oldURL+ " to "+ event.originalEvent.newURL);
	}
	)
	window.onerror = function (errorMsg, url, lineNumber, column, errorObj) {
		var errorObject={
			errorMsg:errorMsg,
			url:url,
			lineNumber:lineNumber,
			column:column
		};

       if (errorObj&& errorObj.stack){
       		errorObject.stack=""+errorObj.stack
       }
       if (Context.currentView && Context.currentView.el){
       	errorObject.currentView=Context.currentView.el.id;
       }
       var message="";
        try{
				message=JSON.stringify(errorObject)
		}catch(e){
			message="Failed to serrialize;"+errorMsg+";"+url+";"+lineNumber+";"+column;
		}
  		console.log(message);
		if (Settings.get('extendedError')){
			
			Message.showError(message);
		}else{
			Message.showError(Strings.COMMON.UNEXPECTED);
		}
		
		if (Context.signedin){
			Feedback.sendLog("Unexpected Exception",message);
		}
		Wait.forceStop();
    }

	$(document).on('click',function(event){

		$('*[data-popover="true"]').each(function(index,element){
			if (element!= event.target){
				$(element).popover('hide');
			}
		})
	})
	Workspace.on('route',function(){
		if (Context.doNotRemoveNextMessages){
			delete Context.doNotRemoveNextMessages;
		}else{
			Message.hide();
		}
	});

	$(document).ajaxComplete( function(ev, jqXHR, ajaxOptions){
		if (jqXHR.status===401){
			if (!ajaxOptions.url.match('/log')){
				Context.cleanLoginData();
			}
		}
	});

	$.ajaxSetup({
		contentType : 'application/json;charset=utf-8',
		accepts : {
			json : 'application/json'
		},
		processData : false,
		dataType : "json",
		beforeSend: Context.sendAuthentication 

	});
 
	$.ajaxPrefilter(function(options, originalOptions, jqXHR) {

		//options.headers = options.headers || {};
		//var user = CP.context("UserPreferences");
		//if (user) {
	    //	options.headers["Accept-Language"] = user.DataLocal + "-" + user.CountryCode;
		//}

		if (options.headers && options.headers['Cache-Control']){
			options.cache =false;
		}
		if (options.dataType !== "text" && options.contentType.match('application/json')) {
			options.contentType = 'application/json;charset=utf-8';
			if (!options.type || options.type.toLowerCase() === "get") {
				var extended=$.extend({},options.data||{},{
					_user:Settings.get('playerId')
				})
				options.data = jQuery.param(extended,true);
			} else {
				if (options.data ){
					if(options.data.serialize){
						options.data = options.data.serialize();
					}
					if (typeof options.data!== "string"){
						options.data = JSON.stringify(options.data);
					}
				}

			}
		}

	});
	console.log("Starting Application");
	Settings.fetch().always(function(){
		if (!Context.haveLocalToken()){
			Backbone.history.start();
			if (!window.location.hash){
				Context.Router.navigate("#login", {trigger: true, replace: true});
			}
			Wait.stop();
		}else{
			Context.loginUsingToken()
			.done(function(){
					Backbone.history.start();
					if (!window.location.hash){
						Context.Router.navigate('#home/overview', {trigger: true, replace: true});
					}
					Wait.stop();
			}).fail(function(data){
					Backbone.history.start();
					if (!window.location.hash){
						Context.Router.navigate("#login", {trigger: true, replace: true});
					}
					Wait.stop();
			});
		}
	})
	}};
});


//# sourceMappingURL=app.js.map